//
//    Implementation of XSIL time object.
//
#include "PConfig.h"
#ifdef __GNU_STDC_OLD
#include <gnusstream.h>
#else
#include <sstream>
#endif
#include "xsil/time.hh"
#include "xsil/Xwriter.hh"
#ifdef P__WIN32
// trick: win does not distinguish Time.hh and time.hh
#include "xsil/../Time.hh"
#else
#include "Time.hh"
#endif
#include <cstdlib>

using namespace std;

namespace xsil {

//--------------------------------------  Null Constructor
time::time(void) {
}

//--------------------------------------  Data Constructor
time::time(const char* Name, const char* Type, const char* Value) 
  : xobj(Name, Type) {
    setValue(Value);
}

//--------------------------------------  Data Constructor
time::time(const char* Name, const Time& t) 
  : xobj(Name) {
    setValue(t);
}

//--------------------------------------  destructor
time::~time(void) {
}

//--------------------------------------  Write the object to an XML document
void
time::Spew(Xwriter& xout) const {
    const int   nattr=2;
    const char* attnames[]={"Name","Type"};
    const char* attstrings[]={getName(), getType()};
    xout.Tag(getObjType(), nattr, attnames, attstrings);
    xout.text(mValue);
    xout.endTag(getObjType());
}

//--------------------------------------  Return object type
time* 
time::Clone(void) const {
    return new time(getName(), getType(), getValue());
}

//--------------------------------------  Return object type
const char* 
time::getObjType(void) const {
    return "Time";
}

//--------------------------------------  Get the time value
Time
time::getTime(void) const {
    if (refType() == "GPS") return Time(strtol(mValue.c_str(), 0, 0));
    return Time(strtol(mValue.c_str(), 0, 0));
}

//--------------------------------------  Set the time value
void 
time::setValue(const char* Value) {
    if (Value) {
        mValue = Value;
	setType("GPS");
    }
}

//--------------------------------------  Set the time value
void 
time::setValue(unsigned int Value) {
    ostringstream out;
    out << Value;
    mValue = out.str();
    setType("GPS-sec");
}

//--------------------------------------  Set the time value
void 
time::setValue(const string& Value) {
    mValue = Value;
    setType("GPS");
}

//--------------------------------------  Set the time value
void 
time::setValue(const Time& Value) {
    // long long tval = Value.getS();
    // tval *= 1000000000;
    // tval += Value.getN();
    long tval = 0;
    tval = Value.getS();
    ostringstream out;
    out << tval;
    mValue = out.str();
    setType("GPS");
}

}
