(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.2'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    624422,      17688]*)
(*NotebookOutlinePosition[    653850,      18756]*)
(*  CellTagsIndexPosition[    653806,      18752]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[StyleBox["NDS Interface for Mathematica",
  FontSlant->"Italic"]], "Subtitle"],

Cell["\<\
Laser Interferometer Gravitational-wave Observatory
LIGO T990000-D
December 1999, Version 1.0\
\>", "Copyright"],

Cell["\<\
Daniel Sigg
LIGO Hanford Observatory
P.O. Box 1970 S9-02
Richland, WA\
\>", "Author"],

Cell[CellGroupData[{

Cell["Getting started", "Section"],

Cell[CellGroupData[{

Cell["Features", "Subsection"],

Cell[TextData[{
  "The ",
  StyleBox["Network Data Server",
    FontSlant->"Italic"],
  " (NDS) interface for ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " supports remote and local access to the data stream of the Laser \
Interferometer Gravitational-wave Observatory (LIGO). The user can retrieve \
data from right within a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " notebook by importing the ",
  StyleBox["NDS package",
    FontSlant->"Italic"],
  ". The ",
  StyleBox["NDS package",
    FontSlant->"Italic"],
  " is build around the ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " interface which allows data exchange through standard network protocols. \
At the obseravtories a ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " server program is constantly listening to user requests which are then \
translated into requests for the ",
  StyleBox["Network Data Server",
    FontSlant->"Italic"],
  ". When the ",
  StyleBox["Network Data Server",
    FontSlant->"Italic"],
  " completes a request the data is sent back through the ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " interface to the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " notebook where it is then accessible to the user for further processing \
and viewing as a native ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " data array."
}], "Text",
  TextJustification->1]
}, Closed]],

Cell[CellGroupData[{

Cell["Installation", "Subsection"],

Cell[TextData[{
  "At the user side the ",
  StyleBox["NDS package",
    FontSlant->"Italic"],
  " \"nds.m\" must be installed at a placed where it can be seen by the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " notebook. Most conveniently a new directory with name \"LIGO\" is created \
in the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " subdirectory  \"Mathematica/AddOns/Applications\". After copying the ",
  StyleBox["NDS package",
    FontSlant->"Italic"],
  " into this new folder the user can load it into the notebook with"
}], "Text",
  TextJustification->1],

Cell[BoxData[
    \(Needs["\<LIGO`nds`\>"]\)], "Input"],

Cell["\<\
or if you installed the package under the current path load it with\
\>", "Text"],

Cell[BoxData[
    \(<< "\<./nds.m\>"\)], "Input"],

Cell["Get the package from here ", "Text"],

Cell["http://www.ligo-wa.caltech.edu/gds/math/nds.nb", "Definition"]
}, Closed]],

Cell[CellGroupData[{

Cell["First steps", "Subsection"],

Cell["\<\
To test if the new package the user can request a list of channel names and \
computes the length of the returned list\
\>", "Text"],

Cell[BoxData[
    \(channels\  = \ NDSGetChannel[Site \[Rule] LHO]; \nLength[channels]\)], 
  "Input"],

Cell["\<\
In the above example the site from which to obtain the channel names has been \
explicitely specified as an option. When obmitted the default site is \
determined by the variable\
\>", "Text"],

Cell[BoxData[
    \(Site \[Rule] $DefaultSite\)], "Input"],

Cell["Currently supported site values are", "Text"],

Cell[BoxData[
    FormBox[GridBox[{
          {
            StyleBox["LHO",
              FontFamily->"Courier"], 
            \(for\ the\ LIGO\ Hanford\ Observatory\)},
          {
            StyleBox["LLO",
              FontFamily->"Courier"], 
            \(for\ the\ LIGO\ Livingston\ Observatory\)}
          }], TextForm]], "Definition"],

Cell["Now get some data from the server and plot it", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\($DefaultSite = LHO;\)\), "\n", 
    \(\(t0 = {1999, 12, 12, 16, 57, 0};\)\), "\n", 
    \(\(trend = 
        NDSGetTrend[{"\<H0:PEM-LVEA_SEISX\>"}, t0, 1000*60, 
          DataRate \[Rule] 1. /60. ];\)\), "\n", 
    \(\(ListPlot[trend\[LeftDoubleBracket]1\[RightDoubleBracket], 
        PlotJoined \[Rule] True];\)\)}], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 1.58889e-005 1072.01 1.00533 [
[.1827 .31769 -19.9688 -16.375 ]
[.1827 .31769 19.9688 0 ]
[.34159 .31769 -19.9688 -16.375 ]
[.34159 .31769 19.9688 0 ]
[.50048 .31769 -19.9688 -16.375 ]
[.50048 .31769 19.9688 0 ]
[.65937 .31769 -19.9688 -16.375 ]
[.65937 .31769 19.9688 0 ]
[.81825 .31769 -19.9688 -16.375 ]
[.81825 .31769 19.9688 0 ]
[.97714 .31769 -19.9688 -16.375 ]
[.97714 .31769 19.9688 0 ]
[.01131 .02859 -52.5 -8.1875 ]
[.01131 .02859 0 8.1875 ]
[.01131 .12912 -52.5 -8.1875 ]
[.01131 .12912 0 8.1875 ]
[.01131 .22966 -52.5 -8.1875 ]
[.01131 .22966 0 8.1875 ]
[.01131 .43072 -52.5 -8.1875 ]
[.01131 .43072 0 8.1875 ]
[.01131 .53126 -52.5 -8.1875 ]
[.01131 .53126 0 8.1875 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
.1827 .33019 m
.1827 .33644 L
s
gsave
.1827 .31769 -80.9688 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(10000) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.34159 .33019 m
.34159 .33644 L
s
gsave
.34159 .31769 -80.9688 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(20000) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.50048 .33019 m
.50048 .33644 L
s
gsave
.50048 .31769 -80.9688 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(30000) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.65937 .33019 m
.65937 .33644 L
s
gsave
.65937 .31769 -80.9688 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(40000) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.81825 .33019 m
.81825 .33644 L
s
gsave
.81825 .31769 -80.9688 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(50000) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.97714 .33019 m
.97714 .33644 L
s
gsave
.97714 .31769 -80.9688 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(60000) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.125 Mabswid
.05559 .33019 m
.05559 .33394 L
s
.08737 .33019 m
.08737 .33394 L
s
.11914 .33019 m
.11914 .33394 L
s
.15092 .33019 m
.15092 .33394 L
s
.21448 .33019 m
.21448 .33394 L
s
.24625 .33019 m
.24625 .33394 L
s
.27803 .33019 m
.27803 .33394 L
s
.30981 .33019 m
.30981 .33394 L
s
.37337 .33019 m
.37337 .33394 L
s
.40514 .33019 m
.40514 .33394 L
s
.43692 .33019 m
.43692 .33394 L
s
.4687 .33019 m
.4687 .33394 L
s
.53225 .33019 m
.53225 .33394 L
s
.56403 .33019 m
.56403 .33394 L
s
.59581 .33019 m
.59581 .33394 L
s
.62759 .33019 m
.62759 .33394 L
s
.69114 .33019 m
.69114 .33394 L
s
.72292 .33019 m
.72292 .33394 L
s
.7547 .33019 m
.7547 .33394 L
s
.78648 .33019 m
.78648 .33394 L
s
.85003 .33019 m
.85003 .33394 L
s
.88181 .33019 m
.88181 .33394 L
s
.91359 .33019 m
.91359 .33394 L
s
.94537 .33019 m
.94537 .33394 L
s
.25 Mabswid
0 .33019 m
1 .33019 L
s
.02381 .02859 m
.03006 .02859 L
s
gsave
.01131 .02859 -113.5 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
72.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(1066.3) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1
%%+ font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1
%%+ Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Math1
%%+ font Helvetica
grestore
.02381 .12912 m
.03006 .12912 L
s
gsave
.01131 .12912 -113.5 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
72.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(1066.2) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1
%%+ font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1
%%+ Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Math1
%%+ font Helvetica
grestore
.02381 .22966 m
.03006 .22966 L
s
gsave
.01131 .22966 -113.5 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
72.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(1066.1) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1
%%+ font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1
%%+ Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Math1
%%+ font Helvetica
grestore
.02381 .43072 m
.03006 .43072 L
s
gsave
.01131 .43072 -113.5 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
72.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(1065.9) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1
%%+ font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1
%%+ Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Math1
%%+ font Helvetica
grestore
.02381 .53126 m
.03006 .53126 L
s
gsave
.01131 .53126 -113.5 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
72.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(1065.8) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1
%%+ font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1
%%+ Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Math1
%%+ font Helvetica
grestore
.125 Mabswid
.02381 .0487 m
.02756 .0487 L
s
.02381 .0688 m
.02756 .0688 L
s
.02381 .08891 m
.02756 .08891 L
s
.02381 .10902 m
.02756 .10902 L
s
.02381 .14923 m
.02756 .14923 L
s
.02381 .16934 m
.02756 .16934 L
s
.02381 .18944 m
.02756 .18944 L
s
.02381 .20955 m
.02756 .20955 L
s
.02381 .24976 m
.02756 .24976 L
s
.02381 .26987 m
.02756 .26987 L
s
.02381 .28998 m
.02756 .28998 L
s
.02381 .31008 m
.02756 .31008 L
s
.02381 .3503 m
.02756 .3503 L
s
.02381 .3704 m
.02756 .3704 L
s
.02381 .39051 m
.02756 .39051 L
s
.02381 .41062 m
.02756 .41062 L
s
.02381 .45083 m
.02756 .45083 L
s
.02381 .47094 m
.02756 .47094 L
s
.02381 .49104 m
.02756 .49104 L
s
.02381 .51115 m
.02756 .51115 L
s
.02381 .00848 m
.02756 .00848 L
s
.02381 .55136 m
.02756 .55136 L
s
.02381 .57147 m
.02756 .57147 L
s
.02381 .59158 m
.02756 .59158 L
s
.02381 .61168 m
.02756 .61168 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .28555 m
.02476 .25983 L
.02572 .23646 L
.02667 .24694 L
.02762 .19379 L
.02858 .24078 L
.02953 .24065 L
.03048 .27724 L
.03144 .24026 L
.03239 .16826 L
.03334 .2146 L
.0343 .28568 L
.03525 .21775 L
.0362 .22593 L
.03716 .23326 L
.03811 .19811 L
.03906 .20158 L
.04002 .06773 L
.04097 .2148 L
.04192 .21041 L
.04288 .18679 L
.04383 .21343 L
.04478 .26618 L
.04574 .27521 L
.04669 .19032 L
.04764 .26513 L
.0486 .21441 L
.04955 .27364 L
.0505 .22743 L
.05146 .27861 L
.05241 .29805 L
.05336 .34407 L
.05432 .32613 L
.05527 .39191 L
.05622 .37417 L
.05718 .32882 L
.05813 .33176 L
.05908 .32122 L
.06004 .33765 L
.06099 .30545 L
.06194 .26507 L
.0629 .23221 L
.06385 .30676 L
.0648 .37745 L
.06576 .32437 L
.06671 .30054 L
.06766 .27397 L
.06862 .2633 L
.06957 .26284 L
.07052 .26009 L
Mistroke
.07148 .24942 L
.07243 .28274 L
.07338 .44074 L
.07434 .38314 L
.07529 .33196 L
.07624 .31573 L
.0772 .27744 L
.07815 .26526 L
.0791 .3099 L
.08006 .26795 L
.08101 .27272 L
.08196 .28326 L
.08292 .24563 L
.08387 .25289 L
.08482 .24805 L
.08578 .39322 L
.08673 .47098 L
.08768 .35303 L
.08864 .34649 L
.08959 .33202 L
.09054 .26893 L
.0915 .24563 L
.09245 .27272 L
.0934 .24033 L
.09436 .24229 L
.09531 .3027 L
.09626 .33759 L
.09722 .32672 L
.09817 .29229 L
.09912 .30637 L
.10008 .3173 L
.10103 .34976 L
.10198 .31553 L
.10294 .3099 L
.10389 .28909 L
.10484 .25643 L
.1058 .36004 L
.10675 .42241 L
.1077 .27645 L
.10866 .29962 L
.10961 .30859 L
.11056 .31285 L
.11152 .22841 L
.11247 .26592 L
.11342 .30198 L
.11438 .24465 L
.11533 .27619 L
.11628 .30702 L
.11724 .27109 L
.11819 .29511 L
Mistroke
.11914 .32947 L
.1201 .36004 L
.12105 .2885 L
.122 .27436 L
.12296 .3008 L
.12391 .3082 L
.12486 .24936 L
.12582 .23398 L
.12677 .31841 L
.12772 .3097 L
.12868 .28051 L
.12963 .18162 L
.13058 .19752 L
.13154 .15053 L
.13249 .25427 L
.13344 .26592 L
.1344 .26644 L
.13535 .19484 L
.1363 .16218 L
.13726 .22782 L
.13821 .23771 L
.13916 .25296 L
.14012 .23843 L
.14107 .17874 L
.14202 .25564 L
.14298 .23332 L
.14393 .2275 L
.14488 .19523 L
.14584 .20393 L
.14679 .24019 L
.14774 .26055 L
.1487 .22658 L
.14965 .24537 L
.1506 .27469 L
.15156 .18705 L
.15251 .25885 L
.15346 .2688 L
.15442 .26395 L
.15537 .24969 L
.15632 .32391 L
.15728 .27351 L
.15823 .23745 L
.15918 .26723 L
.16014 .22638 L
.16109 .30303 L
.16204 .31311 L
.163 .2273 L
.16395 .22324 L
.1649 .22004 L
.16586 .19575 L
Mistroke
.16681 .26952 L
.16776 .23267 L
.16872 .16885 L
.16967 .18371 L
.17062 .18443 L
.17158 .20701 L
.17253 .25839 L
.17348 .24726 L
.17444 .24091 L
.17539 .24746 L
.17634 .22809 L
.1773 .24923 L
.17825 .29151 L
.1792 .19713 L
.18016 .24694 L
.18111 .30224 L
.18206 .34623 L
.18302 .33085 L
.18397 .3245 L
.18492 .33209 L
.18588 .26854 L
.18683 .2148 L
.18778 .21205 L
.18874 .21997 L
.18969 .28667 L
.19064 .27083 L
.1916 .30964 L
.19255 .30702 L
.1935 .3012 L
.19446 .29858 L
.19541 .24438 L
.19636 .2616 L
.19732 .27534 L
.19827 .31677 L
.19922 .30846 L
.20018 .27305 L
.20113 .25224 L
.20208 .20335 L
.20304 .23038 L
.20399 .30466 L
.20494 .26003 L
.2059 .19484 L
.20685 .20695 L
.2078 .23031 L
.20876 .19045 L
.20971 .20708 L
.21066 .2631 L
.21162 .28058 L
.21257 .2631 L
.21352 .21395 L
Mistroke
.21448 .24026 L
.21543 .28928 L
.21638 .30984 L
.21734 .24537 L
.21829 .29485 L
.21924 .31121 L
.2202 .27502 L
.22115 .31769 L
.2221 .27933 L
.22306 .32941 L
.22401 .2993 L
.22496 .26029 L
.22592 .24137 L
.22687 .25001 L
.22782 .23097 L
.22878 .20832 L
.22973 .21493 L
.23068 .25767 L
.23164 .22724 L
.23259 .22174 L
.23354 .32862 L
.2345 .31128 L
.23545 .2523 L
.2364 .24726 L
.23736 .23424 L
.23831 .27063 L
.23926 .22665 L
.24022 .29275 L
.24117 .26886 L
.24212 .22684 L
.24308 .27698 L
.24403 .30624 L
.24498 .29223 L
.24594 .28143 L
.24689 .23156 L
.24784 .20897 L
.2488 .24255 L
.24975 .20708 L
.2507 .19111 L
.25166 .20564 L
.25261 .21388 L
.25356 .22049 L
.25452 .24288 L
.25547 .25747 L
.25642 .10576 L
.25738 .25564 L
.25833 .21722 L
.25928 .21689 L
.26024 .17167 L
.26119 .1468 L
Mistroke
.26214 .19058 L
.2631 .17501 L
.26405 .21185 L
.265 .21343 L
.26596 .20681 L
.26691 .21644 L
.26786 .24327 L
.26882 .23424 L
.26977 .27894 L
.27072 .2415 L
.27168 .23522 L
.27263 .22206 L
.27358 .25106 L
.27454 .24203 L
.27549 .25885 L
.27644 .25721 L
.2774 .17546 L
.27835 .19804 L
.2793 .19811 L
.28026 .10576 L
.28121 .17887 L
.28216 .17154 L
.28312 .16676 L
.28407 .2271 L
.28502 .22704 L
.28598 .22652 L
.28693 .1394 L
.28788 .11898 L
.28884 .20007 L
.28979 .27953 L
.29074 .32149 L
.2917 .28477 L
.29265 .28765 L
.2936 .26598 L
.29456 .24052 L
.29551 .19412 L
.29646 .19497 L
.29742 .23398 L
.29837 .2074 L
.29932 .16722 L
.30028 .18934 L
.30123 .14699 L
.30218 .23188 L
.30314 .20727 L
.30409 .27024 L
.30504 .22586 L
.306 .24419 L
.30695 .22278 L
.3079 .23339 L
.30886 .23064 L
Mistroke
.30981 .20433 L
.31076 .24759 L
.31172 .20969 L
.31267 .20119 L
.31362 .20119 L
.31458 .16565 L
.31553 .17926 L
.31648 .229 L
.31744 .204 L
.31839 .24026 L
.31934 .19117 L
.3203 .15426 L
.32125 .17101 L
.3222 .1396 L
.32316 .19785 L
.32411 .13102 L
.32506 .14797 L
.32602 .12304 L
.32697 .14712 L
.32792 .1091 L
.32888 .1485 L
.32983 .14261 L
.33078 .20001 L
.33174 .0998 L
.33269 .13776 L
.33364 .18639 L
.3346 .22828 L
.33555 .17775 L
.3365 .19719 L
.33746 .17271 L
.33841 .23941 L
.33936 .26945 L
.34032 .14523 L
.34127 .17245 L
.34222 .16846 L
.34318 .23437 L
.34413 .20858 L
.34508 .11302 L
.34604 .17946 L
.34699 .15328 L
.34794 .13803 L
.3489 .14483 L
.34985 .13272 L
.3508 .08894 L
.35176 .16283 L
.35271 .14091 L
.35366 .15648 L
.35462 .24321 L
.35557 .16722 L
.35652 .13979 L
Mistroke
.35748 .17743 L
.35843 .1555 L
.35938 .16447 L
.36034 .20649 L
.36129 .21853 L
.36224 .21545 L
.3632 .17625 L
.36415 .13246 L
.3651 .16015 L
.36606 .08841 L
.36701 .12376 L
.36796 .10059 L
.36892 .08913 L
.36987 .11394 L
.37082 .15419 L
.37178 .13443 L
.37273 .11957 L
.37368 .09928 L
.37464 .16911 L
.37559 .11924 L
.37654 .16722 L
.3775 .15923 L
.37845 .19863 L
.3794 .15491 L
.38036 .15269 L
.38131 .14339 L
.38226 .13279 L
.38322 .10294 L
.38417 .1091 L
.38512 .12814 L
.38608 .13979 L
.38703 .06217 L
.38798 .04122 L
.38894 .13659 L
.38989 .20014 L
.39084 .15112 L
.3918 .08429 L
.39275 .1593 L
.3937 .12775 L
.39466 .1324 L
.39561 .11368 L
.39656 .13083 L
.39752 .08606 L
.39847 .05418 L
.39942 .13312 L
.40038 .20341 L
.40133 .16434 L
.40228 .17978 L
.40324 .15328 L
.40419 .09175 L
Mistroke
.40514 .12009 L
.4061 .10222 L
.40705 .13325 L
.408 .14117 L
.40896 .14509 L
.40991 .12219 L
.41086 .01472 L
.41182 .05772 L
.41277 .10641 L
.41372 .07984 L
.41468 .10694 L
.41563 .15059 L
.41658 .19772 L
.41754 .14621 L
.41849 .10883 L
.41944 .15786 L
.4204 .19883 L
.42135 .11721 L
.4223 .11328 L
.42326 .14084 L
.42421 .18639 L
.42516 .1805 L
.42612 .21997 L
.42707 .15956 L
.42802 .17913 L
.42898 .20106 L
.42993 .23961 L
.43088 .1485 L
.43184 .17016 L
.43279 .15157 L
.43374 .16525 L
.4347 .14149 L
.43565 .22344 L
.4366 .19065 L
.43756 .18456 L
.43851 .1447 L
.43946 .19431 L
.44042 .17271 L
.44137 .17625 L
.44232 .1841 L
.44328 .19503 L
.44423 .20767 L
.44518 .14968 L
.44614 .22678 L
.44709 .23411 L
.44804 .1231 L
.449 .18541 L
.44995 .15786 L
.4509 .22815 L
.45186 .24275 L
Mistroke
.45281 .19294 L
.45376 .14202 L
.45472 .15472 L
.45567 .14549 L
.45662 .13763 L
.45758 .12997 L
.45853 .15079 L
.45948 .17114 L
.46044 .16911 L
.46139 .25963 L
.46234 .47301 L
.4633 .4516 L
.46425 .46109 L
.4652 .22717 L
.46616 .27351 L
.46711 .27331 L
.46806 .30787 L
.46902 .23875 L
.46997 .22822 L
.47092 .23306 L
.47188 .18967 L
.47283 .19608 L
.47378 .17926 L
.47474 .24661 L
.47569 .2167 L
.47664 .18862 L
.4776 .18914 L
.47855 .17173 L
.4795 .1553 L
.48046 .21271 L
.48141 .19392 L
.48236 .12507 L
.48332 .14038 L
.48427 .19621 L
.48522 .12984 L
.48618 .18751 L
.48713 .20472 L
.48808 .19307 L
.48904 .14542 L
.48999 .11649 L
.49094 .17474 L
.4919 .13881 L
.49285 .21598 L
.4938 .37902 L
.49476 .19844 L
.49571 .21375 L
.49666 .20596 L
.49762 .24857 L
.49857 .18829 L
.49952 .21971 L
Mistroke
.50048 .18868 L
.50143 .12585 L
.50238 .15458 L
.50334 .16381 L
.50429 .1826 L
.50524 .2455 L
.5062 .21153 L
.50715 .20439 L
.5081 .16388 L
.50906 .2131 L
.51001 .15328 L
.51096 .12834 L
.51192 .20583 L
.51287 .15812 L
.51382 .22226 L
.51478 .23208 L
.51573 .17651 L
.51668 .16015 L
.51764 .12441 L
.51859 .13652 L
.51954 .13966 L
.5205 .0854 L
.52145 .16623 L
.5224 .07709 L
.52336 .09018 L
.52431 .16781 L
.52526 .15746 L
.52622 .14261 L
.52717 .11859 L
.52812 .10949 L
.52908 .05143 L
.53003 .06734 L
.53098 .11381 L
.53194 .10019 L
.53289 .06721 L
.53384 .0856 L
.5348 .14208 L
.53575 .11505 L
.5367 .13861 L
.53766 .18646 L
.53861 .18973 L
.53956 .15871 L
.54052 .09659 L
.54147 .12022 L
.54242 .13069 L
.54338 .02879 L
.54433 .08933 L
.54528 .11008 L
.54624 .13403 L
.54719 .19019 L
Mistroke
.54814 .14542 L
.5491 .1665 L
.55005 .17802 L
.551 .07028 L
.55196 .17363 L
.55291 .14176 L
.55386 .11924 L
.55482 .0911 L
.55577 .14208 L
.55672 .11885 L
.55768 .19117 L
.55863 .16002 L
.55958 .17409 L
.56054 .17533 L
.56149 .168 L
.56244 .16506 L
.5634 .16905 L
.56435 .12422 L
.5653 .17684 L
.56626 .2635 L
.56721 .22442 L
.56816 .21729 L
.56912 .21951 L
.57007 .20943 L
.57102 .19863 L
.57198 .21035 L
.57293 .17082 L
.57388 .19844 L
.57484 .29511 L
.57579 .22652 L
.57674 .16493 L
.5777 .26147 L
.57865 .26094 L
.5796 .27102 L
.58056 .2347 L
.58151 .18652 L
.58246 .14961 L
.58342 .14287 L
.58437 .19981 L
.58532 .20387 L
.58628 .21127 L
.58723 .12781 L
.58818 .19156 L
.58914 .15871 L
.59009 .22206 L
.59104 .27056 L
.592 .12945 L
.59295 .12736 L
.5939 .17114 L
.59486 .2328 L
Mistroke
.59581 .12742 L
.59676 .1862 L
.59772 .28588 L
.59867 .32207 L
.59962 .32149 L
.60058 .28935 L
.60153 .28483 L
.60248 .29413 L
.60344 .21637 L
.60439 .19209 L
.60534 .25185 L
.6063 .26173 L
.60725 .34302 L
.6082 .26566 L
.60916 .24314 L
.61011 .28005 L
.61106 .28261 L
.61202 .26395 L
.61297 .21428 L
.61392 .22527 L
.61488 .34708 L
.61583 .29943 L
.61678 .24537 L
.61774 .23993 L
.61869 .27443 L
.61964 .34943 L
.6206 .30329 L
.62155 .29544 L
.6225 .23103 L
.62346 .28163 L
.62441 .24203 L
.62536 .21329 L
.62632 .22213 L
.62727 .22082 L
.62822 .2972 L
.62918 .30211 L
.63013 .18227 L
.63108 .25793 L
.63204 .24059 L
.63299 .30159 L
.63394 .28398 L
.6349 .301 L
.63585 .26376 L
.6368 .26912 L
.63776 .31487 L
.63871 .32142 L
.63966 .30316 L
.64062 .24707 L
.64157 .31887 L
.64252 .24229 L
Mistroke
.64348 .31723 L
.64443 .30303 L
.64538 .29053 L
.64634 .30283 L
.64729 .26585 L
.64824 .30388 L
.6492 .29524 L
.65015 .30133 L
.6511 .37411 L
.65206 .30296 L
.65301 .31939 L
.65396 .31095 L
.65492 .31978 L
.65587 .28503 L
.65682 .30964 L
.65778 .27475 L
.65873 .30552 L
.65968 .38596 L
.66064 .32273 L
.66159 .28424 L
.66254 .28994 L
.6635 .30558 L
.66445 .27475 L
.6654 .38046 L
.66636 .35048 L
.66731 .32044 L
.66826 .4232 L
.66922 .46482 L
.67017 .47713 L
.67112 .43511 L
.67208 .36946 L
.67303 .3118 L
.67398 .27639 L
.67494 .3027 L
.67589 .28424 L
.67684 .36763 L
.6778 .31638 L
.67875 .27587 L
.6797 .33104 L
.68066 .32142 L
.68161 .28752 L
.68256 .32947 L
.68352 .36337 L
.68447 .27371 L
.68542 .3133 L
.68638 .22841 L
.68733 .32037 L
.68828 .33425 L
.68924 .36416 L
.69019 .31605 L
Mistroke
.69114 .33091 L
.6921 .27364 L
.69305 .31422 L
.694 .33202 L
.69496 .26147 L
.69591 .27142 L
.69686 .35971 L
.69782 .30093 L
.69877 .25963 L
.69972 .26912 L
.70068 .22141 L
.70163 .26376 L
.70258 .18273 L
.70354 .25008 L
.70449 .2671 L
.70544 .23974 L
.7064 .20426 L
.70735 .23463 L
.7083 .24851 L
.70926 .25394 L
.71021 .33026 L
.71116 .26854 L
.71212 .32869 L
.71307 .26474 L
.71402 .31697 L
.71498 .27999 L
.71593 .28536 L
.71688 .26317 L
.71784 .36639 L
.71879 .2523 L
.71974 .27541 L
.7207 .27822 L
.72165 .31101 L
.7226 .28928 L
.72356 .36711 L
.72451 .38602 L
.72546 .3851 L
.72642 .36599 L
.72737 .32973 L
.72832 .2864 L
.72928 .32803 L
.73023 .28091 L
.73118 .25963 L
.73214 .2508 L
.73309 .2527 L
.73404 .2974 L
.735 .29354 L
.73595 .27744 L
.7369 .30002 L
.73786 .32495 L
Mistroke
.73881 .28922 L
.73976 .21041 L
.74072 .18135 L
.74167 .19026 L
.74262 .20119 L
.74358 .37084 L
.74453 .25021 L
.74548 .29229 L
.74644 .51444 L
.74739 .20243 L
.74834 .24667 L
.7493 .31239 L
.75025 .34799 L
.7512 .30637 L
.75216 .23267 L
.75311 .32057 L
.75406 .32103 L
.75502 .29818 L
.75597 .30558 L
.75692 .30133 L
.75788 .21925 L
.75883 .2612 L
.75978 .30879 L
.76074 .30041 L
.76169 .34269 L
.76264 .35369 L
.7636 .27456 L
.76455 .31631 L
.7655 .28051 L
.76646 .30794 L
.76741 .24327 L
.76836 .2415 L
.76932 .2131 L
.77027 .31154 L
.77122 .24229 L
.77218 .22678 L
.77313 .25512 L
.77408 .33818 L
.77504 .29072 L
.77599 .59344 L
.77694 .42811 L
.7779 .30322 L
.77885 .30689 L
.7798 .37319 L
.78076 .36141 L
.78171 .34047 L
.78266 .33693 L
.78362 .34989 L
.78457 .30506 L
.78552 .26448 L
Mistroke
.78648 .26245 L
.78743 .29256 L
.78838 .31854 L
.78934 .27881 L
.79029 .32024 L
.79124 .32083 L
.7922 .32338 L
.79315 .3567 L
.7941 .2991 L
.79506 .31605 L
.79601 .31154 L
.79696 .30394 L
.79792 .36252 L
.79887 .36462 L
.79982 .3241 L
.80078 .27142 L
.80173 .29138 L
.80268 .23136 L
.80364 .26199 L
.80459 .28601 L
.80554 .28208 L
.8065 .26965 L
.80745 .29144 L
.8084 .29904 L
.80936 .2868 L
.81031 .34302 L
.81126 .31874 L
.81222 .25178 L
.81317 .22357 L
.81412 .28464 L
.81508 .37084 L
.81603 .32927 L
.81698 .37934 L
.81794 .35545 L
.81889 .38308 L
.81984 .39617 L
.8208 .30571 L
.82175 .32063 L
.8227 .3366 L
.82366 .30643 L
.82461 .33588 L
.82556 .37084 L
.82652 .34027 L
.82747 .32011 L
.82842 .32587 L
.82938 .34799 L
.83033 .39577 L
.83128 .4014 L
.83224 .32495 L
.83319 .33687 L
Mistroke
.83414 .37084 L
.8351 .34184 L
.83605 .28405 L
.837 .32901 L
.83796 .30938 L
.83891 .38294 L
.83986 .41763 L
.84082 .37967 L
.84177 .33602 L
.84272 .38039 L
.84368 .40382 L
.84463 .37437 L
.84558 .42758 L
.84654 .46692 L
.84749 .39466 L
.84844 .40022 L
.8494 .33582 L
.85035 .38118 L
.8513 .36481 L
.85226 .40474 L
.85321 .35676 L
.85416 .39865 L
.85512 .32954 L
.85607 .35578 L
.85702 .29851 L
.85798 .35761 L
.85893 .36429 L
.85988 .34328 L
.86084 .36036 L
.86179 .3891 L
.86274 .4304 L
.8637 .3476 L
.86465 .37358 L
.8656 .33189 L
.86656 .39001 L
.86751 .42928 L
.86846 .35801 L
.86942 .45848 L
.87037 .42869 L
.87132 .3639 L
.87228 .36946 L
.87323 .43334 L
.87418 .41148 L
.87514 .45638 L
.87609 .37653 L
.87704 .41515 L
.878 .39839 L
.87895 .36796 L
.8799 .32829 L
.88086 .32738 L
Mistroke
.88181 .3476 L
.88276 .36508 L
.88372 .40919 L
.88467 .37679 L
.88562 .33922 L
.88658 .3296 L
.88753 .37967 L
.88848 .36089 L
.88944 .39021 L
.89039 .40337 L
.89134 .26559 L
.8923 .2832 L
.89325 .32836 L
.8942 .36036 L
.89516 .33628 L
.89611 .351 L
.89706 .38838 L
.89802 .36678 L
.89897 .37679 L
.89992 .37948 L
.90088 .39558 L
.90183 .39204 L
.90278 .39165 L
.90374 .44781 L
.90469 .38811 L
.90564 .37804 L
.9066 .33228 L
.90755 .3298 L
.9085 .38923 L
.90946 .41286 L
.91041 .40696 L
.91136 .35257 L
.91232 .40474 L
.91327 .43321 L
.91422 .37398 L
.91518 .40258 L
.91613 .33909 L
.91708 .42064 L
.91804 .60332 L
.91899 .37221 L
.91994 .39021 L
.9209 .4069 L
.92185 .36992 L
.9228 .3349 L
.92376 .41102 L
.92471 .41901 L
.92566 .39322 L
.92662 .43727 L
.92757 .42248 L
.92852 .41685 L
Mistroke
.92948 .40075 L
.93043 .39708 L
.93138 .39944 L
.93234 .42797 L
.93329 .45474 L
.93424 .41809 L
.9352 .46005 L
.93615 .41875 L
.9371 .4499 L
.93806 .38471 L
.93901 .33962 L
.93996 .33615 L
.94092 .33713 L
.94187 .4033 L
.94282 .44309 L
.94378 .34976 L
.94473 .37457 L
.94568 .44303 L
.94664 .48367 L
.94759 .48773 L
.94854 .46914 L
.9495 .41587 L
.95045 .41757 L
.9514 .42215 L
.95236 .41181 L
.95331 .39885 L
.95426 .34767 L
.95522 .333 L
.95617 .39767 L
.95712 .33863 L
.95808 .42647 L
.95903 .47301 L
.95998 .45749 L
.96094 .5452 L
.96189 .54121 L
.96284 .57766 L
.9638 .56097 L
.96475 .49775 L
.9657 .45965 L
.96666 .47274 L
.96761 .48374 L
.96856 .50089 L
.96952 .46253 L
.97047 .45422 L
.97142 .47111 L
.97238 .47719 L
.97333 .43851 L
.97428 .38936 L
.97524 .42804 L
.97619 .43164 L
Mistroke
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{345.25, 213.25},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000850001?a000`40O003h00Oogooogoo1goo003o
OoooOol7Ool00?mooomoo`Moo`00ogooogoo1goo003oOoooOol7Ool00?mooomoo`Moo`00ogooogoo
1goo003oOoooOol7Ool00?mooomoo`Moo`00ogooogoo1goo003oOoooOol7Ool00?mooomoo`Moo`00
ogooogoo1goo003oOoooOol7Ool00?mooomoo`Moo`00ogooogoo1goo003oOoooOol7Ool00?mooomo
o`Moo`00ogooogoo1goo003oOoooOol7Ool00?mooomoo`Moo`00ogooogoo1goo003oOoooOol7Ool0
0?mooomoo`Moo`00ogooogoo1goo003oOoooOol7Ool00?mooomoo`Moo`00ogooogoo1goo003oOooo
Ool7Ool005Aoo`03001oogoo0?moojmoo`00E7oo00<007ooOol0ogoo[goo001DOol00`00Oomoo`3o
Oon_Ool005Aoo`800?mook1oo`00E7oo00<007ooOol0ogoo[goo001DOol00`00Oomoo`3oOon_Ool0
01Eoo`8000Moo`D000Moo`@000Moo`@000Eoo`8000Eoo`D000Yoo`03001oogoo0:1oo`03001oogoo
0?moo`aoo`005Goo0P001Woo1`0017oo1`0017oo1`0017oo0P0017oo1`002Goo00<007ooOol0X7oo
00<007ooOol0ogoo37oo000EOol20006Ool20003Ool20004Ool20003Ool20004Ool20003Ool20009
Ool30003Ool30008Ool00`00Oomoo`2POol00`00Oomoo`3oOol<Ool001Eoo`8000Eoo`8000Eoo`80
009oo`8000Eoo`80009oo`8000Eoo`8000Qoo`8000Eoo`8000Qoo`03001oogoo0:1oo`03001oogoo
0?moo`aoo`0017oo2P001goo0P001Goo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P003goo
0P0027oo00<007ooOol0X7oo00<007ooOol0ogoo37oo000EOol20005Ool20005Ool20002Ool20005
Ool20002Ool20005Ool2000?Ool20008Ool00`00Oomoo`2POol00`00Oomoo`3oOol<Ool001Eoo`80
00Eoo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8000ioo`<000Qoo`<00:1oo`03001oogoo
03Eoo`03001oogoo0==oo`005Goo0P001Goo0P001Goo0P000Woo0`000goo0`000Woo0`000goo0`00
37oo10002Goo00<007ooOol0X7oo00<007ooOol0=Goo00<007ooOol0dgoo000EOol20005Ool20005
Ool20002Ool80003Ool8000=Ool2000;Ool00`00Oomoo`2POol00`00Oomoo`0eOol00`00Oomoo`3C
Ool001Eoo`8000Eoo`8000Eoo`80009oo`800003Ool00000008000Aoo`800003Ool00000008000mo
o`<000Uoo`03001oogoo0:1oo`8003Ioo`03001oogoo0==oo`004Woo0P0000=oo`0000001Goo0P00
1Goo0P000Woo0P002Goo0P005Goo0P002Goo00<007ooOol0X7oo0P00=Woo00<007ooOol0dgoo000B
Ool50005Ool20005Ool20003Ool20004Ool20003Ool20004Ool20008Ool20004Ool20009Ool00`00
Oomoo`2FOol00`00Oomoo`07Ool2000fOol00`00Oomoo`3COol001Aoo`<000Ioo`8000=oo`8000Ao
o`8000=oo`<000=oo`8000=oo`<000Qoo`<0009oo`<000Uoo`03001oogoo09Ioo`03001oogoo00Mo
o`8003Ioo`03001oogoo0==oo`005Goo0P001Woo1`001Goo1P001Goo1P002Woo1P002Woo00<007oo
Ool0UWoo00<007ooOol01goo0P00=Woo00<007ooOol0dgoo000EOol20007Ool50007Ool40007Ool4
000<Ool4000;Ool2002GOol00`00Oomoo`07Ool2000fOol00`00Oomoo`3COol005Aoo`03001oogoo
09Ioo`03001oogoo00Moo`80031oo`03001oogoo00=oo`03001oogoo0==oo`00E7oo00<007ooOol0
UWoo00<007ooOol00Woo00<007ooOol00Woo0P00<7oo00<007ooOol00goo00<007ooOol0dgoo001D
Ool00`00Oomoo`2FOol00`00Oomoo`02Ool00`00Oomoo`02Ool2000`Ool00`00Oomoo`03Ool00`00
Oomoo`3COol005Aoo`03001oogoo09Ioo`03001oogoo009oo`03001oogoo009oo`80031oo`03001o
ogoo00=oo`03001oogoo0==oo`00E7oo00<007ooOol0UWoo00<007ooOol00Woo00<007ooOol00Woo
0P00<7oo00<007ooOol00goo0P00e7oo001DOol00`00Oomoo`2FOol00`00Oomoo`02Ool00`00Oomo
o`02Ool2000`Ool00`00Oomoo`03Ool2003DOol005Aoo`03001oogoo09Ioo`03001oogoo009oo`03
001oogoo009oo`80031oo`03001oogoo00=oo`800=Aoo`00E7oo00<007ooOol00goo00<007ooOol0
T7oo00<007ooOol00Woo00<007ooOol00Woo0P00<7oo00<007oo00000goo0P00e7oo001DOol20004
Ool00`00Oomoo`2@Ool01000Oomoogoo0P0017oo0P00<7oo00<007oo00000goo0P0000=oo`00Ool0
dGoo001DOol00`00Oomoo`03Ool00`00Oomoo`2@Ool01000Oomoogoo0P0017oo0P00<7oo00<007oo
00000goo0P0000=oo`00Ool0dGoo001DOol00`00Oomoo`03Ool00`00Oomoo`2@Ool01000Oomoogoo
0P0017oo0P00<7oo00<007oo00000goo0P0000=oo`00Ool0dGoo001DOol00`00Oomoo`03Ool00`00
Oomoo`2@Ool01000Oomoogoo0P0017oo0P00;Goo00H007ooOol007oo0003Ool200000goo001oo`3A
Ool005Aoo`03001oogoo00=oo`03001oogoo091oo`04001oogooOol20004Ool2000]Ool01P00Oomo
o`00Ool000=oo`800003Ool007oo0=5oo`00E7oo00<007ooOol00goo00<007ooOol0T7oo00@007oo
Oomoo`8000Aoo`8002eoo`06001oogoo001oo`000goo0P0000=oo`00Ool0dGoo001DOol00`00Oomo
o`03Ool00`00Oomoo`2@Ool01000Ool007oo0P0017oo0P00;Goo0P0000Aoo`00000000=oo`800003
Ool007oo0=5oo`00E7oo00<007ooOol00goo00<007ooOol0T7oo00@007oo001oo`8000Aoo`8002ao
o`<00004Ool000000003Ool200000goo001oo`3AOol005Aoo`8000Aoo`03001oogoo085oo`03001o
ogoo00=oo`8000Moo`<00003Ool0000000Aoo`8002aoo`<00004Ool000000003Ool200000goo001o
o`3AOol005Aoo`03001oogoo00=oo`03001oogoo085oo`03001oogoo00=oo`8000Moo`<00005Ool0
0000Ool00002Ool2000/Ool3000017oo000000000goo0P0000Aoo`00Ool00=1oo`00E7oo00<007oo
Ool00goo00<007ooOol0PGoo00<007ooOol00goo0P001goo0`0000Eoo`00001oo`00009oo`<002]o
o`<00004Ool000000003Ool2000017oo001oo`00d7oo001DOol00`00Oomoo`03Ool00`00Oomoo`21
Ool00`00Oomoo`03Ool20007Ool300001Goo000007oo00000Woo0`00:goo0`0000Aoo`000000009o
o`<00004Ool007oo003@Ool005Aoo`03001oogoo00=oo`03001oogoo085oo`03001oogoo00=oo`80
00Moo`<00005Ool00000Ool00002Ool3000[Ool3000017oo000000000Woo0`0000Aoo`00Ool00=1o
o`00E7oo00<007ooOol00goo00<007ooOol0NGoo00<007ooOol01Goo00<007ooOol00goo0P0000=o
o`00Ool017oo0`0000Eoo`00001oo`00009oo`<002]oo`<00004Ool000000002Ool3000017oo001o
o`00d7oo001DOol00`00Oomoo`03Ool00`00Oomoo`1iOol00`00Oomoo`05Ool00`00Oomoo`03Ool4
0003Ool00`00Ool00002000017oo000007oo0P0000Aoo`00000002]oo`<00004Ool000000002Ool3
000017oo001oo`00d7oo001DOol00`00Oomoo`03Ool00`00Oomoo`1IOol00`00Oomoo`07Ool00`00
Oomoo`0COol00`00Oomoo`05Ool00`00Oomoo`03Ool40003Ool00`00Ool00002000017oo000007oo
0P0000Aoo`00000002]oo`<00004Ool000000002Ool3000017oo001oo`00d7oo001DOol00`00Oomo
o`03Ool00`00Oomoo`1IOol00`00Oomoo`07Ool00`00Oomoo`0COol00`00Oomoo`05Ool00`00Oomo
o`03Ool40003Ool5000017oo000007oo0P0000Aoo`00000002]oo`<00004Ool000000002Ool30000
17oo001oo`00d7oo001DOol20004Ool00`00Oomoo`1IOol00`00Oomoo`07Ool00`00Oomoo`0AOol0
0`00Ool00007Ool00`00Oomoo`03Ool40003Ool5000017oo000007oo0P0000=oo`00Ool00P00:Woo
0`0000Aoo`000000009oo`<00004Ool007oo003@Ool005Aoo`03001oogoo00=oo`03001oogoo05Uo
o`03001oogoo00Moo`03001oogoo015oo`03001oo`0000Aoo`04001oo`000005Ool40003Ool50000
17oo000007oo0P0000=oo`00Ool00P0000=oo`00Ool09goo0`0000Aoo`000000009oo`<00004Ool0
07oo003@Ool001Eoo`8000Moo`D000Moo`@000Moo`@000Eoo`8000=oo`T000Qoo`03001oogoo00=o
o`03001oogoo05Uoo`03001oogoo00Moo`03001oogoo015oo`03001oo`0000Aoo`04001oo`000005
Ool40003Ool5000017oo000007oo0P0000=oo`00Ool00P0000=oo`00Ool09goo100000Eoo`000000
07oo00<00004Ool007oo003@Ool001Eoo`8000Ioo`L000Aoo`L000Aoo`L000Aoo`8000=oo`T000Qo
o`03001oogoo00=oo`03001oogoo05Uoo`03001oogoo00Moo`03001oogoo015oo`03001oo`0000Ao
o`04001oo`000005Ool40003Ool5000017oo000007oo0P0000=oo`00Ool00P0000=oo`00Ool06goo
00<007ooOol02Goo10000Woo0P0000Qoo`00000007oo001oo`00d7oo000EOol20006Ool20003Ool2
0004Ool20003Ool20004Ool20003Ool2000:Ool2000>Ool00`00Oomoo`03Ool00`00Oomoo`1IOol0
0`00Oomoo`07Ool01000Oomoo`0047oo00<007oo000017oo00@007oo000000Eoo`D0009oo`D00004
Ool00000Ool200000goo001oo`0200000goo001oo`0KOol00`00Oomoo`09Ool40002Ool200001goo
00000000Ool007oo00800<moo`005Goo0P001Goo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo
0P002Woo0P003Goo00<007ooOol00goo00<007ooOol0FGoo00<007ooOol01goo00@007ooOol0011o
o`03001oo`0000Aoo`04001oo`000005Ool50002Ool5000017oo000007oo0P0000=oo`00Ool00P00
00=oo`00Ool06goo00<007ooOol02Goo10000Woo0P0000Eoo`00000007oo00@00<moo`0017oo2P00
1goo0P001Goo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P002goo0P0037oo00<007ooOol0
0goo00<007ooOol0FGoo00<007ooOol01goo00@007ooOol000moo`800003Ool007oo00=oo`04001o
o`000005Ool50002Ool800001Woo000007oo001oo`800003Ool007oo00Uoo`03001oogoo00]oo`05
001oogooOol0000:Ool50002Ool200001Goo00000000Ool010000Woo00<007ooOol0bWoo000EOol2
0005Ool20005Ool20002Ool20005Ool20002Ool20005Ool2000<Ool2000;Ool00`00Oomoo`03Ool0
0`00Oomoo`1IOol00`00Oomoo`07Ool01000Oomoo`003goo0P0000=oo`00Ool00goo00@007oo0000
00Eoo`03001oo`000080009oo`P00006Ool00000Ool007oo0P0000=oo`00Ool02Goo00<007ooOol0
2goo00D007ooOomoo`0000=oo`03001oogoo00Aoo`D0009oo`800005Ool00000001oo`040002Ool0
0`00Oomoo`09Ool2002oOol001Eoo`8000Eoo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`80
00eoo`8000Yoo`<000=oo`03001oogoo05Uoo`03001oogoo00Moo`04001oogoo000?Ool200000goo
001oo`03Ool01000Ool000001Goo00<007oo00000P000Woo200000Ioo`00001oo`00Ool200000goo
001oo`09Ool01@00Oomoogoo00002Goo00D007oo001oo`0000=oo`05001oogooOol00002Ool50002
Ool3000017oo000007oo10000Woo00<007ooOol01goo00@007oo00000;moo`005Goo0P001Goo0P00
1Goo0P000Woo0`000goo0`000Woo0`000goo0`003Woo0P002Goo00<007ooOol00goo00<007ooOol0
FGoo00<007ooOol01goo00@007ooOol000ioo`<00003Ool007oo00=oo`04001oo`000005Ool00`00
Ool000020002Ool800001Woo000007oo001oo`800003Ool007oo00Uoo`05001oogooOol00009Ool0
0`00Ool000020003Ool01@00Oomoogoo00000Woo1@000Woo0`0000Aoo`00001oo`@0009oo`03001o
ogoo00Moo`04001oo`00002oOol001Eoo`8000Eoo`8000Eoo`80009oo`P000=oo`P0011oo`03001o
ogoo00Moo`03001oogoo00=oo`03001oogoo05Uoo`03001oogoo00Moo`04001oogoo000>Ool30000
0goo001oo`03Ool01000Ool0000017oo0P0000Aoo`000000009oo`H00003Ool007oo00<0009oo`@0
00Yoo`03001oogoo008000Uoo`D000=oo`05001oogooOol00002Ool50002Ool30002Ool00`00Ool0
00030002Ool00`00Oomoo`07Ool01000Ool00000_goo000EOol20005Ool20005Ool20002Ool20000
0goo000000020004Ool200000goo00000002000AOol20008Ool00`00Oomoo`03Ool00`00Oomoo`1I
Ool00`00Oomoo`07Ool01000Oomoo`003Woo0`0000=oo`00Ool00goo00@007oo000000Aoo`800004
Ool000000002Ool00`00Ool0000300000goo001oo`030002Ool4000:Ool00`00Oomoo`020009Ool5
0003Ool01@00Oomoogoo00000Woo1@000Woo0`000Woo00<007oo00000`000Woo00<007ooOol01goo
00@007oo00000;moo`004Woo0P0000=oo`0000001Goo0P001Goo0P000Woo0P002Goo0P005Woo0P00
27oo00<007ooOol00goo00<007ooOol0FGoo00<007ooOol01goo00@007ooOol000ioo`<00003Ool0
07oo00=oo`04001oo`000004Ool2000017oo000000000Woo00<007oo00000`0000Ioo`00Ool007oo
0002Ool4000:Ool00`00Oomoo`020009Ool50003Ool01@00Oomoogoo00000Woo1@000Woo0`000Woo
00<007oo00000`000Woo00<007ooOol01goo00@007oo00000;moo`004Woo1@001Goo0P001Goo0P00
0goo0P0017oo0P000goo0P0017oo0P0027oo0P001Goo0P0027oo00<007ooOol00goo0P00FWoo00<0
07ooOol01goo00@007ooOol000ioo`<00003Ool007oo00=oo`03001oo`0000<0009oo`800004Ool0
00000002Ool00`00Ool0000300001Woo001oo`00Ool0009oo`@000Aoo`03001oogoo00=oo`03001o
ogoo008000Uoo`D000=oo`80009oo`03001oogoo00D0009oo`<0009oo`03001oo`0000<0009oo`03
001oogoo00Moo`04001oo`00002oOol001Aoo`<000Ioo`8000=oo`8000Aoo`8000=oo`<000=oo`80
00=oo`<000Uoo`8000Aoo`8000Qoo`03001oogoo00=oo`8005Yoo`03001oogoo00Moo`04001oogoo
000>Ool50004Ool00`00Ool000030002Ool200000goo00000002000017oo001oogoo0`0000Ioo`00
Ool007oo0002Ool40004Ool00`00Oomoo`03Ool00`00Ool000020009Ool50003Ool20002Ool01@00
Oomoo`00Ool00`000Woo0`000Woo00<007oo00000`000Woo00<007ooOol01Goo00@007oo001oo`80
0;moo`005Goo0P001Woo1`001Goo1P001Goo1P002Woo1`002Goo00<007ooOol00goo0P00FWoo00<0
07ooOol01goo00@007ooOol000ioo`D0009oo`03001oo`0000D0009oo`800003Ool007oo00800004
Ool007ooOol20002Ool01@00Ool007oo00000Woo100017oo00<007oo00000goo00<007oo00000P00
2Goo1@000goo0P000Woo00D007ooOol007oo00<000=oo`80009oo`@00005Ool007ooOol00007Ool0
1000Ool007oo0P00_goo000EOol20007Ool50007Ool40007Ool4000<Ool5000:Ool00`00Oomoo`03
Ool2001JOol00`00Ool00007Ool01000Oomoo`001Goo00<007ooOol01Woo1@000Woo00D007oo0000
07oo00<0009oo`800003Ool007oo00<000=oo`80009oo`03001oo`0000Aoo`@000=oo`800003Ool0
07oo009oo`03001oo`00008000Uoo`D000=oo`80009oo`05001oogoo001oo`030003Ool20002Ool4
00001Goo001oogoo00001goo00@007oo001oo`800;moo`00E7oo0P0017oo0P009Goo00<007ooOol0
<Woo00<007oo00001goo00@007ooOol000Eoo`03001oogoo00Ioo`03001oogoo0080009oo`05001o
o`00001oo`030002Ool200000goo001oo`030003Ool20002Ool00`00Ool00004Ool40003Ool20000
17oo001oogoo0P0000Aoo`00000000Uoo`D000=oo`80009oo`05001oogoo001oo`020004Ool20002
Ool400001Goo001oogoo00001goo00@007oo001oo`800;moo`00E7oo00<007ooOol00goo0P009Goo
00<007ooOol0<Woo00<007oo00001goo00@007ooOol000Eoo`03001oogoo00Ioo`03001oogoo0080
009oo`05001oo`00001oo`030002Ool200000goo001oo`030003Ool20002Ool00`00Ool00004Ool4
0003Ool2000017oo001oogoo0P0000Aoo`00000000Uoo`D000=oo`80009oo`05001oogoo001oo`02
0004Ool20002Ool400001Goo001oogoo00001goo00@007oo001oo`800;moo`00E7oo00<007ooOol0
0goo0P009Goo00<007ooOol0<Woo00<007oo00001goo00@007ooOol000Eoo`03001oogoo00Eoo`80
009oo`80009oo`05001oo`00001oo`0400001Woo000007oo001oo`<000=oo`80009oo`03001oo`00
00Aoo`@000=oo`800004Ool007ooOol2000017oo0000000027oo1P000goo0P000Woo00D007ooOol0
07oo008000Aoo`80009oo`@00005Ool007ooOol00007Ool01000Ool007oo0P00_goo001DOol00`00
Oomoo`03Ool2000UOol00`00Oomoo`0bOol00`00Ool00007Ool01000Oomoo`001Goo00<007ooOol0
1Goo0P000Woo0P000Woo00D007oo000007oo00@00003Ool0000000=oo`<000=oo`80009oo`03001o
o`0000Aoo`@000=oo`800004Ool007ooOol400000goo001oo`07Ool60003Ool20002Ool200000goo
001oo`020004Ool20002Ool400001Goo001oogoo00001goo00@007oo001oo`<00;ioo`00E7oo00<0
07ooOol00goo0P009Goo00<007ooOol0<Woo0`001goo00<007ooOol00P0017oo00<007ooOol01Goo
0P000Woo0P000Woo100000=oo`0000000P0000=oo`0000000goo0`000goo0P000Woo00<007oo0000
17oo100000=oo`00Ool00P0000Aoo`00Oomoo`@00003Ool007oo00Moo`H000=oo`80009oo`800003
Ool007oo008000Aoo`80009oo`@00005Ool00000Ool00007Ool01000Ool007oo0`00_Woo001DOol0
0`00Oomoo`03Ool2000UOol00`00Ool0000bOol30007Ool00`00Oomoo`020004Ool00`00Oomoo`05
Ool20002Ool20002Ool400000goo0000000200000goo001oo`03Ool30003Ool20002Ool00`00Ool0
0004Ool400000goo001oo`02000017oo001oogoo10000Woo00<007ooOol01Goo1P000goo0P000Woo
0P0000=oo`00Ool00P001Goo00@007ooOomoo`<00005Ool00000Ool00007Ool01000Ool007oo0`00
_Woo001DOol00`00Oomoo`03Ool2000UOol00`00Ool0000bOol30007Ool00`00Oomoo`020004Ool0
0`00Oomoo`05Ool20002Ool20002Ool40002Ool300000goo001oo`03Ool30003Ool20002Ool30004
Ool3000017oo000007oo0P0000Aoo`00Oomoo`@0009oo`03001oogoo00Eoo`H000=oo`800007Ool0
0000001oo`00Ool00P001Goo00@007ooOomoo`<00003Ool00000008000Aoo`07001oogoo001oo`00
Ool00`00_Woo001DOol00`00Oomoo`03Ool2000UOol00`00Ool0000bOol30007Ool200000goo0000
0003Ool20004Ool00`00Oomoo`020002Ool20002Ool40002Ool300000goo001oo`03Ool30003Ool2
0002Ool20005Ool3000017oo000007oo10000Woo10000Woo00<007ooOol01Goo1P000goo0P0000Mo
o`00000007oo001oo`020005Ool01000Oomoogoo0`0000=oo`0000000P0017oo00L007ooOol007oo
001oo`03002nOol005Aoo`800004Ool007ooOol2000UOol00`00Ool0000=Ool00`00Oomoo`0ROol3
0007Ool200000goo00000003Ool20004Ool00`00Oomoo`020002Ool20002Ool40002Ool300000goo
001oo`03Ool30003Ool20003Ool00`00Oomoo`03Ool3000017oo000007oo10000Woo10000Woo00<0
07ooOol01Goo1P000goo0P0000=oo`0000000P002Goo00@007ooOomoo`<00003Ool00000008000Ao
o`07001oo`00001oo`00Ool00`00_Woo001DOol01000Oomoo`000Woo0P009Goo00<007oo00003Goo
00<007ooOol08Woo0`001goo0P0000=oo`0000000goo0P0017oo00<007ooOol00P000Woo0P000Woo
10000Woo0`0000=oo`00Ool017oo0P000goo0P000goo00<007ooOol00goo0`0000Aoo`00001oo`@0
009oo`@0009oo`03001oogoo00Eoo`H000=oo`800003Ool00000008000Uoo`04001oogooOol30000
0goo000000020003Ool200000goo00000002000017oo00000000_Woo001DOol01000Oomoo`000Woo
0P009Goo00<007oo00003Goo00<007ooOol08Woo0`001goo0P0000=oo`0000000goo0P0017oo0`00
00Aoo`00Oomoo`800003Ool0000000<0009oo`<00003Ool007oo00Aoo`8000=oo`8000=oo`03001o
ogoo00Aoo`800004Ool00000Ool500000goo000000020002Ool00`00Oomoo`05Ool60003Ool00`00
Ool000040009Ool01000Oomoogoo0`0000Eoo`00001oo`0000=oo`800003Ool0000000800004Ool0
0000002nOol005Aoo`04001oogoo0002Ool2000UOol00`00Ool0000=Ool00`00Oomoo`0ROol40005
Ool300000goo00000003Ool20004Ool3000017oo001oogoo0P0000=oo`0000000`000Woo0`0000=o
o`00Ool017oo0P000goo00<007ooOol00Woo00<007ooOol017oo0P0000Aoo`00001oo`D00003Ool0
00000080009oo`03001oogoo00Eoo`H000=oo`03001oo`0000@000Uoo`04001oogooOol20002Ool2
00000goo001oo`02Ool200000goo00000002000017oo00000000_Woo001DOol01000Oomoo`000Woo
0P009Goo00D007oo001oo`0000]oo`03001oogoo029oo`@000Eoo`<00003Ool0000000=oo`8000Ao
o`<00004Ool007ooOol80002Ool300000goo001oo`04Ool00`00Oomoo`02Ool00`00Oomoo`02Ool0
0`00Oomoo`04Ool2000017oo000007oo2P000Woo00<007ooOol00goo00<007oo0000100000Ioo`00
Oomoo`00Ool50009Ool01000Oomoogoo0P000Woo00@007ooOol000=oo`800003Ool0000000800004
Ool00000002nOol005Aoo`04001oogoo0002Ool2000TOol2000017oo001oo`002goo00<007ooOol0
8Goo1@001Goo0`0000=oo`0000000goo0P0017oo0`0000Aoo`00Oomoo`L000=oo`<00003Ool007oo
00Aoo`03001oogoo009oo`03001oogoo009oo`03001oogoo00Aoo`800004Ool00000Ool:0002Ool0
0`00Oomoo`03Ool700001Woo001oogoo001oo`D000Uoo`05001oogooOol00003Ool01000Oomoo`00
0goo0P0000=oo`0000000P0000Aoo`00000003aoo`03001oogoo07moo`00E7oo00@007ooOol0009o
o`8002Aoo`800004Ool007oo000;Ool00`00Oomoo`0QOol50005Ool400000goo001oo`02Ool20004
Ool3000017oo001oogoo1`000goo0`0000=oo`00Ool017oo00<007ooOol00Woo00<007ooOol00Woo
00<007ooOol017oo0P0000Ioo`00001oo`00Ool80002Ool00`00Oomoo`03Ool200000goo00000002
00001Woo001oogoo001oo`D000Uoo`05001oogooOol00003Ool01000Oomoo`000goo0P0000=oo`00
00000P0000Aoo`00000000eoo`03001oogoo01]oo`03001oogoo00ioo`03001oogoo07moo`00E7oo
00@007ooOol0009oo`8002Aoo`800004Ool007oo000;Ool2000ROol50005Ool400000goo001oo`02
Ool20004Ool3000017oo001oogoo1`000goo0P0000=oo`0000001Goo00<007ooOol00Woo00<007oo
Ool00Woo00<007ooOol017oo0P0000Ioo`00001oo`00Ool80002Ool00`00Oomoo`03Ool200000goo
0000000200001Woo001oogoo001oo`D000Uoo`05001oogooOol00003Ool01000Oomoo`000goo0P00
00=oo`0000000P0000Aoo`00000000eoo`03001oogoo01]oo`03001oogoo00ioo`03001oogoo07mo
o`00E7oo00@007ooOol0009oo`<002=oo`800004Ool007oo0004Ool00`00Oomoo`04Ool2000ROol5
0005Ool400000goo001oo`02Ool20004Ool3000017oo001oogoo1`000goo0P0000=oo`0000001Goo
00<007ooOol00Woo00<007ooOol00Woo00<007ooOol017oo0P0000Ioo`00001oo`00Ool500000goo
00000002Ool00`00Oomoo`02Ool300000goo0000000200000goo001oo`0200000goo000000030009
Ool01@00Oomoogoo00000goo00@007ooOol000=oo`800003Ool0000000800004Ool00000000=Ool0
0`00Oomoo`0KOol00`00Oomoo`0>Ool00`00Oomoo`1oOol005Aoo`800004Ool007ooOol4000ROol2
000017oo001oo`0017oo00<007ooOol017oo0P003goo00<007ooOol03Woo00<007oo000010001Goo
100000=oo`00Ool00Woo0P0017oo0`0000Aoo`00Oomoo`L000=oo`800003Ool0000000Eoo`03001o
ogoo009oo`03001oogoo009oo`03001oogoo00Aoo`800003Ool0000000800004Ool00000Ool20000
0goo00000002Ool00`00Oomoo`02Ool300000goo0000000200000goo001oo`0200000goo00000003
0009Ool01@00Oomoogoo00000goo00@007ooOol0009oo`<00003Ool0000000H000eoo`03001oogoo
01]oo`03001oogoo00ioo`03001oogoo07moo`00E7oo0P0000Aoo`00001oo`@0029oo`800004Ool0
07oo0004Ool00`00Oomoo`04Ool2000?Ool00`00Oomoo`0>Ool00`00Ool0000200000goo001oo`04
Ool400000goo001oo`02Ool20004Ool3000017oo001oogoo1`000goo0P0000=oo`0000001Goo00<0
07ooOol00Woo00<007ooOol00Woo00<007ooOol017oo0P0000=oo`0000000P0000Aoo`00001oo`80
0003Ool00000009oo`03001oogoo009oo`<00003Ool0000000800003Ool007oo00800003Ool00000
00<001Aoo`03001oogoo00<00003Ool0000000H0009oo`03001oogoo00Qoo`03001oogoo01]oo`03
001oogoo00ioo`03001oogoo07moo`00E7oo0P0000Aoo`00001oo`@0029oo`800006Ool007oo001o
o`000Woo00<007ooOol00goo0`003Goo00<007oo000047oo00<007oo00000P0000=oo`00Ool017oo
100000=oo`00Ool0100017oo0`0000Aoo`00Oomoo`L000=oo`800003Ool0000000Eoo`03001oogoo
009oo`03001oogoo009oo`03001oogoo00Aoo`800004Ool000000002Ool2000017oo000007oo0P00
0Woo00<007ooOol00Woo0`0000=oo`0000000P0000=oo`00Ool00P0000=oo`0000000`0057oo00<0
07ooOol00`0000Eoo`00000007oo00@0009oo`03001oogoo00Qoo`03001oogoo01]oo`03001oogoo
00eoo`80085oo`00E7oo0P0000Aoo`00001oo`@0029oo`800006Ool007oo001oo`000Woo00<007oo
Ool00goo0`000Woo00<007ooOol027oo00<007oo000047oo1@0000=oo`00Ool017oo0P0000Eoo`00
Ool007oo00@000Aoo`<00004Ool007ooOol70003Ool200000goo00000005Ool00`00Oomoo`02Ool0
0`00Oomoo`02Ool00`00Oomoo`04Ool2000017oo000000000Woo00<007ooOol00P0000=oo`000000
0Woo00<007ooOol00Woo0`0000=oo`00Ool00P0000=oo`0000000P0000=oo`0000000`0057oo00<0
07ooOol00`0000Eoo`00000007oo00@0009oo`03001oogoo00Qoo`03001oogoo01]oo`03001oogoo
00eoo`80085oo`00E7oo0P0000Aoo`00001oo`@0029oo`800006Ool007oo001oo`000Woo00<007oo
Ool00goo0`000Woo00<007ooOol027oo00<007oo000047oo1@0000=oo`00Ool017oo00L007ooOol0
07oo001oo`040004Ool30004Ool70003Ool200000goo00000005Ool00`00Oomoo`02Ool00`00Oomo
o`02Ool00`00Oomoo`05Ool00`00Ool000020002Ool00`00Oomoo`0200000goo00000002Ool00`00
Oomoo`02Ool3000017oo001oo`000Woo100000=oo`0000000`0057oo00<007ooOol00`0000Eoo`00
000007oo00@0009oo`03001oogoo00Qoo`03001oogoo01]oo`03001oogoo00eoo`80085oo`00E7oo
0P0000Aoo`00001oo`@0029oo`800006Ool007oo001oo`000Woo00<007ooOol00goo00<007oo0000
0Woo00<007ooOol027oo0`0047oo1@0000=oo`00Ool017oo00L007ooOol007oo001oo`040004Ool3
0005Ool400000goo001oo`02Ool200000goo0000000?Ool00`00Oomoo`08Ool20002Ool00`00Oomo
o`0200000goo00000002Ool00`00Oomoo`02Ool3000017oo001oo`000Woo100000=oo`0000000`00
57oo00<007ooOol00`0000Moo`00Ool007oo001oo`020002Ool00`00Oomoo`08Ool00`00Oomoo`0K
Ool00`00Oomoo`0=Ool200000goo001oo`1nOol005Aoo`800004Ool00000Ool4000ROol200001Woo
001oo`00Ool0009oo`03001oogoo00=oo`03001oo`00009oo`03001oogoo00Moo`@0011oo`D00003
Ool007oo00Aoo`07001oogoo001oo`00Ool010000Woo00<007oo00000P001Goo100000=oo`00Ool0
0Woo00<007ooOol00P003goo00<007ooOol027oo0P000Woo00<007ooOol00P0000=oo`0000000Woo
00<007ooOol00Woo0`0000Aoo`00Ool0009oo`@00003Ool0000000<001Aoo`03001oo`0000<00007
Ool007oo001oo`00Ool00P0000=oo`0000002Woo00<007ooOol06goo00<007oo00003Goo0P0000=o
o`00Ool0OWoo001DOol2000017oo000007oo10008Woo0P0000Ioo`00Ool007oo0002Ool00`00Oomo
o`03Ool00`00Ool00002Ool00`00Oomoo`07Ool4000@Ool500000goo00000004Ool01`00Oomoo`00
Ool007oo00D00004Ool00000Ool20006Ool300000goo001oo`02Ool00`00Oomoo`02000JOol20002
Ool00`00Oomoo`0200000goo00000002Ool00`00Oomoo`02Ool300000goo001oo`03Ool400000goo
00000003000DOol00`00Ool0000300001goo001oo`00Ool007oo00800003Ool0000000Yoo`03001o
ogoo01]oo`03001oo`0000eoo`800003Ool007oo07ioo`00E7oo0P0000Aoo`00001oo`@0029oo`80
0006Ool007oo001oo`000Woo00<007ooOol00Woo0P0000Eoo`00Oomoo`0000Uoo`@000Moo`03001o
ogoo00Eoo`H00003Ool0000000Aoo`07001oogoo001oo`00Ool01@0000Aoo`00001oo`8000Ioo`<0
0003Ool007oo009oo`03001oogoo008001Yoo`80009oo`06001oogooOol007oo0P000Woo00<007oo
Ool00Woo0`0000=oo`00Ool00goo100000Ioo`00Ool007oo000DOol00`00Ool0000300001goo001o
o`00Ool007oo00800003Ool0000000Yoo`03001oogoo01]oo`03001oo`0000eoo`800003Ool007oo
07ioo`00E7oo0P0000Aoo`00001oo`@0029oo`800003Ool007oo00<0009oo`03001oogoo009oo`80
0005Ool007ooOol00003Ool00`00Oomoo`03Ool40007Ool20006Ool600000goo00000004Ool01`00
Oomoo`00Ool007oo00D00004Ool00000Ool20006Ool300000goo001oo`02Ool00`00Oomoo`02000J
Ool20002Ool01P00Oomoogoo001oo`80009oo`06001oogooOol007oo0P0000=oo`00Ool00goo1000
00Ioo`00Ool007oo000DOol01000Ool007oo0P0000Moo`00Ool007oo001oo`0200000goo0000000:
Ool00`00Oomoo`0KOol00`00Ool0000=Ool200000goo001oo`1nOol005Aoo`800004Ool00000Ool5
000QOol200000goo001oo`030002Ool00`00Oomoo`02Ool200001Goo001oogoo00000Woo0P001Goo
100000=oo`00Ool017oo0P001Woo1P0000=oo`00000017oo00L007ooOol007oo001oo`05000017oo
000007oo0P001Woo0`0000=oo`00Ool00Woo00<007ooOol00P006Woo0P000Woo00H007ooOomoo`00
Ool20002Ool01P00Oomoogoo001oo`8000Ioo`@00006Ool007oo001oo`0057oo00@007oo001oo`@0
00=oo`05001oo`00001oo`020003Ool00`00Oomoo`02Ool00`00Ool0000MOol00`00Ool0000=Ool2
00000goo001oo`08Ool00`00Oomoo`1cOol001Eoo`8000Moo`D000Moo`@000Moo`@000Eoo`8000Mo
o`8000]oo`H0009oo`<0025oo`800003Ool007oo00<0009oo`03001oogoo009oo`800005Ool007oo
Ool00002Ool20005Ool400000goo001oo`04Ool20006Ool60002Ool01@00Oomoogoo00000goo00@0
07oo001oo`D00004Ool00000Ool20006Ool300000goo001oo`02Ool00`00Oomoo`02000JOol20002
Ool01P00Oomoogoo001oo`80009oo`06001oogooOol007oo0P001Woo100000=oo`00Ool00Woo00<0
07ooOol04Woo00@007oo001oo`@000=oo`05001oo`00001oo`020003Ool00`00Oomoo`02Ool00`00
Ool0000MOol3000=Ool200000goo001oo`08Ool00`00Oomoo`1cOol001Eoo`8000Ioo`L000Aoo`L0
00Aoo`L000Aoo`8000Moo`8000]oo`H0009oo`<0025oo`800003Ool007oo00<0009oo`03001oogoo
009oo`800005Ool007ooOol00002Ool20005Ool400000goo001oo`04Ool20006Ool2000017oo0000
00000Woo00D007ooOomoo`0000=oo`04001oo`00Ool5000017oo000007oo0P001Woo0`0000=oo`00
Ool00Woo00@007ooOol001]oo`80009oo`07001oogooOol007oo00000goo00H007ooOomoo`00Ool2
0006Ool400000goo001oo`02Ool00`00Oomoo`0BOol01000Ool007oo10000goo00D007oo000007oo
008000=oo`03001oogoo009oo`03001oo`0001eoo`<000eoo`800003Ool007oo00=oo`03001oogoo
009oo`03001oogoo07=oo`005Goo0P001Woo0P000goo0P0017oo0P000goo0P0017oo0P000goo0P00
3Goo0P002goo1P000Woo0`008Goo0P0000=oo`00Ool00`000Woo00<007ooOol00Woo0P0000Eoo`00
Oomoo`00009oo`8000Eoo`@00003Ool007oo00Aoo`<000Eoo`800004Ool000000002Ool00`00Oomo
o`020003Ool01000Ool007oo0P0000Aoo`00001oo`800003Ool0000000Ioo`<00003Ool007oo009o
o`03001oogoo01aoo`80009oo`07001oogooOol007oo00000goo00H007ooOomoo`00Ool20006Ool3
0002Ool01@00Oomoogoo00005Goo00<007ooOol010000goo00D007oo000007oo008000=oo`03001o
ogoo009oo`<001eoo`<000eoo`800003Ool007oo00=oo`03001oogoo009oo`03001oogoo07=oo`00
5Goo0P001Goo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P0037oo0P002goo1P000Woo0`00
8Goo0P0000=oo`00Ool00P0000Aoo`00Ool000Aoo`800004Ool007ooOol200000goo00000005Ool4
00000goo001oo`04Ool30005Ool2000017oo000000000Woo00<007ooOol00P000goo00@007oo001o
o`800003Ool0000000<00003Ool0000000Ioo`<00003Ool007oo009oo`03001oogoo01eoo`04001o
ogoo0003Ool00`00Ool00003Ool01P00Oomoogoo001oo`8000Ioo`<0009oo`05001oogooOol0000E
Ool00`00Oomoo`040003Ool01@00Ool00000Ool00P000goo00<007ooOol00Woo0`007Goo0`003Goo
0P0000=oo`00Ool00goo00<007ooOol00Woo00<007ooOol03goo00<007ooOol0HGoo0004Ool:0007
Ool20005Ool20005Ool20002Ool20005Ool20002Ool20005Ool2000<Ool2000;Ool60002Ool3000Q
Ool200000goo001oo`02000017oo001oo`000Woo0P0000Aoo`00Ool0009oo`800003Ool0000000Eo
o`@00003Ool007oo00Aoo`<0009oo`800004Ool00000Ool30002Ool00`00Oomoo`020003Ool01000
Ool007oo0P0000=oo`0000000`0000=oo`0000001Woo0`0000=oo`00Ool00Woo00<007ooOol07Goo
00<007ooOol017oo00<007oo00000goo00H007ooOomoo`00Ool20006Ool30002Ool01@00Oomoogoo
00005Goo00<007ooOol010000goo00D007oo000007oo008000=oo`03001oogoo009oo`<001eoo`<0
00eoo`800003Ool0000000=oo`03001oogoo009oo`80011oo`03001oogoo065oo`005Goo0P001Goo
0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P0037oo0P002goo1P000Woo10006Goo00<007oo
Ool017oo0P0000=oo`00Ool00P0000Aoo`00Ool0009oo`800005Ool007oo001oo`0300000goo0000
0004Ool500000goo001oo`04Ool30002Ool2000017oo000007oo0`000Woo00<007oo00000P000goo
00@007oo001oo`800003Ool0000000<00003Ool0000000Ioo`<00003Ool007oo009oo`03001oogoo
01eoo`03001oogoo00Aoo`03001oo`0000=oo`06001oo`00Ool007oo0P001Woo0`000Woo00D007oo
Oomoo`0001Eoo`03001oogoo00@000=oo`05001oo`00001oo`020003Ool01P00Oomoogoo001oo`80
01Moo`03001oogoo00=oo`<000eoo`800003Ool0000000=oo`04001oogooOol3000@Ool00`00Oomo
o`1QOol001Eoo`8000Eoo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8000aoo`8000]oo`H0
009oo`@001Uoo`03001oogoo00Aoo`@00007Ool00000Ool007oo00000Woo0P0000Eoo`00Ool007oo
00<00003Ool0000000Aoo`D00003Ool007oo00Aoo`<0009oo`800004Ool00000Ool30003Ool40005
Ool01@00Ool00000Ool01@000Woo00<007ooOol01Goo0P0000=oo`00Ool00Woo00<007ooOol07Goo
00<007ooOol017oo00<007oo00000goo00H007oo001oo`00Ool20006Ool30002Ool01@00Oomoogoo
00005Goo00<007ooOol010000goo00D007oo000007oo008000=oo`06001oogooOol007oo0P005goo
00<007ooOol00goo0`003Goo0P0000=oo`0000000goo00@007ooOomoo`<000aoo`05001oogooOol0
001SOol001Eoo`8000Eoo`8000Eoo`80009oo`<000=oo`<0009oo`<000=oo`<000aoo`8000]oo`H0
009oo`@000Aoo`03001oogoo019oo`03001oogoo00=oo`@00005Ool00000001oo`030002Ool20000
1Goo001oo`00Ool00`0000=oo`00000017oo0`0000Aoo`00Ool000Eoo`<0009oo`800004Ool00000
Ool30003Ool40005Ool01@00Ool00000Ool01@000Woo00<007ooOol01Goo0P0000=oo`00Ool00Woo
00<007ooOol07Goo00<007ooOol017oo00<007oo00000goo00L007oo001oo`00Ool00007Ool30002
Ool01@00Oomoogoo00005Goo00<007ooOol010000goo00D007oo000007oo008000=oo`06001oogoo
Ool007oo0P005goo00<007ooOol00goo0`003Goo0P0000Aoo`000000009oo`04001oogooOol3000<
Ool01@00Oomoogoo0000Hgoo000EOol20005Ool20005Ool20002Ool80003Ool8000=Ool2000;Ool5
0003Ool40004Ool00`00Oomoo`0BOol00`00Oomoo`03Ool400001Goo00000000Ool00`000Woo0P00
00Eoo`00Ool007oo00<00003Ool0000000Aoo`<0009oo`8000Eoo`<00005Ool00000001oo`020000
17oo000000000goo10001Goo00<007oo00000goo0P0000Eoo`00Oomoo`0000Moo`800003Ool007oo
009oo`03001oogoo01eoo`03001oogoo00Aoo`03001oo`0000=oo`07001oo`00Ool007oo00001goo
0`000Woo00<007ooOol05goo00<007ooOol010000goo00D007ooOol007oo008000=oo`06001oogoo
Ool007oo0P005goo00<007ooOol00goo0`003Goo0P0000Aoo`000000009oo`04001oogooOol3000<
Ool01@00Oomoogoo0000Hgoo000EOol20005Ool20005Ool20002Ool200000goo000000020004Ool2
00000goo00000002000>Ool2000;Ool50003Ool40004Ool00`00Oomoo`0BOol00`00Oomoo`03Ool4
00001Goo00000000Ool00`0000Uoo`00000007oo001oo`00Ool00`0000=oo`00000017oo0`000Woo
0P001Goo0`0000Eoo`00000007oo00800003Ool0000000Aoo`@000Eoo`03001oo`0000=oo`800005
Ool007ooOol00007Ool20005Ool00`00Oomoo`0MOol00`00Oomoo`06Ool02`00Oomoogoo001oo`00
Ool007oo00001goo0`000Woo00<007ooOol05goo00<007ooOol010000goo00D007ooOol007oo0080
00=oo`06001oogooOol007oo0P005goo00<007ooOol00goo0`003Goo0P0000Aoo`000000009oo`04
001oogooOol3000<Ool01@00Oomoogoo0000Hgoo000BOol200000goo00000005Ool20005Ool20002
Ool20009Ool2000@Ool200000goo0000000;Ool50003Ool40004Ool00`00Oomoo`0:Ool00`00Oomo
o`05Ool00`00Oomoo`03Ool400001Goo00000000Ool00`0000Moo`00000007oo001oo`0500000goo
00000004Ool30002Ool20004Ool01@00Ool00000Ool00`0000Aoo`00001oo`8000Aoo`@000Eoo`03
001oo`0000=oo`800005Ool007ooOol00007Ool20005Ool00`00Oomoo`0MOol00`00Oomoo`06Ool0
2`00Oomoogoo001oo`00Ool007oo00001goo0`000Woo00<007ooOol05goo00<007ooOol010000goo
00D007ooOol007oo0080009oo`800005Ool007oo001oo`03000FOol00`00Oomoo`03Ool3000=Ool2
000017oo000000000Woo00@007ooOomoo`<000aoo`05001oogooOol0001SOol0019oo`D000Eoo`80
00Eoo`8000=oo`8000Aoo`8000=oo`8000Aoo`8000Uoo`D000]oo`D000=oo`@000Aoo`03001oogoo
00Yoo`03001oogoo00Eoo`03001oogoo009oo`D00005Ool00000001oo`0300001goo00000000Ool0
07oo00D00003Ool0000000Aoo`<0009oo`8000Aoo`03001oo`0000D00004Ool00000Ool20004Ool4
0005Ool00`00Ool00003Ool200000goo001oo`09Ool20005Ool00`00Oomoo`0VOol02`00Oomoogoo
001oo`00Ool007oo00001goo0`000Woo00<007ooOol05goo00<007ooOol010000goo00D007ooOol0
07oo00800003Ool0000000<00003Ool007oo00<000=oo`03001oogoo011oo`03001oogoo00=oo`@0
00aoo`800004Ool000000002Ool01000Oomoogoo0`0037oo00D007ooOomoo`0006=oo`0057oo0`00
1Woo0P000goo0P0017oo0P000goo0`000goo0P000goo0`002goo0`002goo1@000goo100017oo00<0
07ooOol01Woo00<007ooOol00P001goo0P000goo1@0000=oo`0000000Woo0`0000Moo`00000007oo
001oo`0500000goo00000002Ool00`00Ool000020002Ool30003Ool00`00Ool00005000017oo001o
ogoo0P0017oo10001Goo00<007oo00000goo0P0000=oo`00Ool02Goo0P00<Woo00L007oo001oo`00
Ool00007Ool30002Ool00`00Oomoo`0GOol00`00Oomoo`040003Ool01@00Oomoo`00Ool00P0000=o
o`0000000`0000=oo`00Ool00`000goo00<007ooOol047oo00<007ooOol00Woo1@0037oo0P0000Ao
o`000000009oo`04001oogooOol3000<Ool01@00Oomoogoo0000Hgoo000EOol20006Ool70005Ool6
0005Ool6000=Ool2000;Ool50003Ool40004Ool00`00Oomoo`06Ool00`00Oomoo`020007Ool20003
Ool500000goo00000002Ool300001goo00000000Ool007oo00D00003Ool00000009oo`03001oo`00
0080009oo`<000=oo`03001oo`0000D00004Ool007ooOol20004Ool40005Ool00`00Ool00003Ool0
1000Oomoo`002Woo0P00<Woo00L007oo001oo`00Ool00007Ool3000LOol00`00Oomoo`040003Ool0
1@00Oomoo`00Ool00P0000=oo`0000000`0000=oo`00Ool00`000Woo0P004Woo00<007ooOol00Woo
1@0037oo0P0000Aoo`000000009oo`04001oogooOol3000<Ool01@00Oomoogoo0000Hgoo000EOol2
0007Ool50007Ool40007Ool4000>Ool2000;Ool50003Ool40004Ool01@00Oomoogoo000017oo00<0
07ooOol00P001goo0P000goo1@0000=oo`0000000Woo1`0000=oo`00Ool01@0000=oo`0000000Woo
00<007oo00000P000Woo0`000goo00<007oo00001@0000Aoo`00Oomoo`8000Aoo`@000Eoo`03001o
o`0000=oo`03001oogoo00]oo`80039oo`05001oo`00Ool00009Ool3000LOol00`00Oomoo`040003
Ool01@00Oomoo`00Ool00P0000=oo`0000000`0000=oo`00Ool00`000Woo0P004Woo00<007ooOol0
0Woo1@0037oo1P000Woo00@007ooOomoo`<000aoo`05001oogooOol0001SOol005Aoo`03001oo`00
008000=oo`@000Aoo`05001oogooOol00004Ool00`00Oomoo`020007Ool20003Ool500000goo0000
0002Ool700000goo001oo`0500000goo00000002Ool00`00Ool000020002Ool30003Ool00`00Ool0
0005000017oo001oogoo0P0017oo10001Goo00<007oo00000goo00<007ooOol02goo0P00<Woo00D0
07oo001oo`0000Uoo`8001eoo`03001oogoo00@000=oo`05001oogoo001oo`0200000goo00000003
00000goo001oo`030002Ool2000BOol00`00Oomoo`02Ool50003Ool00`00Oomoo`03Ool00`00Oomo
o`060002Ool01000Oomoogoo0`0037oo00D007ooOomoo`0006=oo`00E7oo00<007oo00000P000goo
100017oo00<007ooOol00P0017oo00<007ooOol00P001goo0P000goo1@000Woo00<007ooOol01`00
00Eoo`00Ool007oo00<00007Ool007oo001oo`00Ool00`000Woo0`000goo00<007oo00001@0000Ao
o`00Oomoo`8000Aoo`800003Ool007oo00Aoo`03001oo`0000=oo`03001oogoo00aoo`03001oogoo
031oo`05001oo`00Ool00009Ool2000MOol00`00Oomoo`030004Ool01@00Oomoo`00Ool00P0000=o
o`0000000`0000=oo`00Ool00`000Woo0P004Woo00<007ooOol00Woo0`0000=oo`00Ool00Woo00<0
07ooOol00goo00<007ooOol01P000Woo00@007ooOomoo`<000aoo`04001oogooOol2001ROol005Ao
o`03001oo`00008000=oo`@000Aoo`03001oogoo008000Aoo`03001oogoo008000Aoo`03001oogoo
008000=oo`D0009oo`03001oogoo00L00005Ool007oo001oo`0300001goo001oo`00Ool007oo00<0
009oo`<000=oo`03001oo`0000H000=oo`8000Aoo`800003Ool007oo00Aoo`03001oo`0000=oo`03
001oogoo00aoo`03001oogoo031oo`05001oo`00Ool00009Ool2000MOol00`00Oomoo`030004Ool0
1`00Oomoo`00Ool007oo00H00003Ool007oo00<0009oo`80019oo`03001oogoo009oo`<00003Ool0
07oo009oo`03001oogoo009oo`80009oo`H0009oo`04001oogooOol3000<Ool01000Oomoogoo0P00
HWoo001DOol01@00Ool007oo00000goo100017oo00<007ooOol00P0017oo00<007ooOol00P0017oo
00<007ooOol00P000goo1@001Goo1`0000Eoo`00Ool007oo00<00007Ool007oo001oo`00Ool00`00
0Woo0`000goo00<007ooOol01P002Goo0P0000=oo`00Ool017oo00<007oo00000goo00<007ooOol0
37oo00<007ooOol0<7oo00D007oo001oo`0000Uoo`8001eoo`03001oogoo00<000Aoo`07001oogoo
001oo`00Ool01P0000=oo`00Ool00`000Woo0P004Woo00<007ooOol00Woo0`0000=oo`00Ool00Woo
00<007ooOol00Woo0P000Woo1P000Woo00@007ooOomoo`<000aoo`04001oogooOol2001ROol005Ao
o`05001oo`00Ool00003Ool40004Ool00`00Oomoo`020004Ool00`00Oomoo`020004Ool00`00Oomo
o`020003Ool300000goo001oo`04Ool700000goo001oo`02Ool300001goo001oo`00Ool007oo00<0
009oo`<000=oo`03001oogoo00H000Uoo`03001oogoo00Ioo`03001oo`0000=oo`03001oogoo00ao
o`03001oogoo031oo`05001oo`00Ool00009Ool2000MOol01P00Oomoo`00Ool000Aoo`07001oogoo
001oo`00Ool01P0000Ioo`00Ool007oo0002Ool2000BOol01@00Oomoo`00Ool00`0000=oo`00Ool0
0Woo00<007ooOol00Woo0P000Woo1P0000=oo`0000000goo0`001Goo00<007ooOol00goo0P000goo
0P00HWoo001DOol01@00Ool007oo00000goo100017oo00<007oo00000P000goo0P000Woo0P0017oo
00<007ooOol00P000goo0`0000=oo`00Ool01Goo1P0000=oo`00Ool00Woo0`0000Moo`00Ool007oo
001oo`030002Ool30003Ool00`00Oomoo`060009Ool00`00Oomoo`06Ool00`00Ool00003Ool00`00
Oomoo`0<Ool00`00Oomoo`0`Ool01@00Ool007oo00002Goo0P007Goo00H007ooOol007oo0004Ool0
1`00Oomoo`00Ool007oo00P00003Ool007oo00800003Ool00000019oo`05001oogoo001oo`030000
0goo001oo`040003Ool20002Ool600000goo00000003Ool30004Ool20005Ool20003Ool2001ROol0
05Aoo`05001oo`00Ool00003Ool40004Ool01@00Ool007oo00000Woo0`000Woo0P0017oo00<007oo
Ool00P000goo0`0000=oo`00Ool01Goo1P0000=oo`00Ool00Woo0`0000Moo`00Ool007oo001oo`03
0002Ool00`00Ool00003Ool00`00Oomoo`060009Ool00`00Oomoo`06Ool00`00Ool00003Ool00`00
Oomoo`0<Ool00`00Oomoo`0`Ool01@00Ool007oo00002Goo0P0087oo00<007oo000017oo00L007oo
Ool007oo001oo`0800000goo001oo`0500000goo001oo`0?Ool00`00Oomoo`0300001Goo001oo`00
Ool010000goo0P000Woo1P0000=oo`0000000goo0`0017oo0P001Goo0P000goo0P007goo00<007oo
Ool0@7oo001DOol01@00Ool007oo00000goo100017oo00D007oo001oo`00009oo`<0009oo`8000Ao
o`03001oogoo008000=oo`<000Qoo`H00003Ool007oo009oo`<00005Ool007oo001oo`0300001goo
001oogoo001oo`0000=oo`03001oogoo00H000Uoo`03001oogoo00Ioo`03001oo`0000=oo`03001o
ogoo00aoo`03001oogoo031oo`05001oo`00Ool00009Ool2000POol00`00Ool00004Ool01`00Oomo
o`00Ool007oo00P00003Ool007oo00D00003Ool007oo00moo`03001oogoo00<000=oo`H000=oo`80
009oo`H00003Ool0000000=oo`<000Aoo`8000Eoo`8000=oo`8001moo`03001oogoo041oo`00E7oo
0`0000=oo`00Ool00goo0`0017oo00D007oo001oo`00009oo`<0009oo`8000Aoo`03001oo`0000<0
009oo`<000Qoo`06001oo`00001oo`001Woo0P0000Eoo`00Ool007oo00<00006Ool007ooOol007oo
0P000Woo00<007ooOol01P002Goo00<007ooOol01Woo00<007oo00000goo00<007ooOol0?goo00D0
07oo001oo`0000Uoo`80021oo`03001oo`0000Aoo`07001oogoo001oo`00Ool0200000=oo`00Ool0
1@0000=oo`00Ool03Woo0P000Woo0`000goo1P000goo0P000Woo1P0000=oo`0000000goo0`0017oo
0P001Goo0`000Woo0P007goo00<007ooOol0@7oo001DOol01@00Ool007oo000017oo0`0017oo00H0
07oo001oo`00Ool40002Ool30002Ool200000goo000000020002Ool30008Ool01P00Ool00000Ool0
00Ioo`800005Ool007oo001oo`0300001Woo001oogoo001oo`80009oo`03001oogoo00800004Ool0
00000009Ool00`00Oomoo`06Ool00`00Ool00015Ool01000Ool000002Woo0P0087oo00<007ooOol0
1goo00@007oo001oo`P00003Ool007oo00D00003Ool007oo00ioo`800003Ool00000008000=oo`H0
00=oo`80009oo`H00003Ool0000000=oo`<000Aoo`8000Eoo`03001oo`00009oo`8001moo`03001o
ogoo041oo`00E7oo00D007oo001oo`0000Aoo`03001oo`0000Aoo`06001oo`00Ool007oo10000Woo
00<007oo00000Woo1`0000=oo`00000010001Woo1P001Woo1P0000Eoo`00000007oo00P00003Ool0
07oo00L000Uoo`03001oogoo009oo`T000Aoo`D000Ioo`D000Ioo`D000Ioo`D0019oo`D0009oo`03
001oo`0000H000Eoo`H000Ioo`D000Ioo`D000Ioo`03001oogoo00Moo`04001oo`00Ool800000goo
00000005000017oo001oogoo1@000goo00<007oo00002@000Woo1`000goo00@007oo001oo`H00003
Ool0000000H000Aoo`H00004Ool007oo0002Ool50006Ool50006Ool50006Ool2000;Ool40006Ool5
0006Ool50006Ool50006Ool50006Ool005Aoo`05001oo`00Ool00006Ool00`00Oomoo`02Ool01P00
Ool007oo001oo`@0009oo`03001oo`00009oo`L00003Ool0000000@000Ioo`L000Aoo`L00003Ool0
000000X00003Ool0000000L000Uoo`03001oogoo009oo`T000=oo`L000Aoo`L000Aoo`L000Aoo`L0
011oo`L00003Ool007oo00P000Aoo`L000Aoo`L000Aoo`L000Eoo`03001oogoo00Moo`04001oo`00
Ool@00000goo001oo`070002Ool00`00Ool0000900000goo000000060003Ool01000Ool007oo3`00
17oo200000=oo`00Ool01`0017oo1`0017oo1`001Goo0P002Goo1`0017oo1`0017oo1`0017oo1`00
17oo1`001Goo001DOol01@00Oomoogoo00001Woo00<007ooOol00Woo00H007oo001oo`00Ool40002
Ool00`00Ool00002Ool700000goo000000040006Ool400000goo00000004Ool300001Goo00000000
Ool01@000Woo1P0000=oo`0000000P0000Aoo`00000000moo`800003Ool0000000Ioo`8000=oo`80
00Aoo`8000=oo`8000Aoo`8000=oo`8000Aoo`8000=oo`8000moo`<000=oo`@00004Ool000000003
Ool20004Ool20003Ool20004Ool20003Ool20004Ool20003Ool20005Ool00`00Oomoo`07Ool01000
Ool007oo2P0000=oo`0000000`0000=oo`00Ool00`000Woo0P000Woo00D007oo000007oo00H0009o
o`P000=oo`04001oo`00Ool<00000goo00000004Ool30002Ool300000goo001oo`030002Ool20004
Ool20003Ool20004Ool20003Ool20005Ool20009Ool20003Ool20004Ool20003Ool20004Ool20003
Ool20004Ool20003Ool20004Ool20003Ool20005Ool005Aoo`05001oogooOol00006Ool00`00Oomo
o`02Ool01P00Ool007oo001oo`@0009oo`03001oo`00009oo`L00003Ool0000000@000Eoo`800004
Ool00000Ool30002Ool400000goo001oo`070004Ool6000017oo000007oo0`0047oo10001Goo0P00
1Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P003Woo0P001Goo0`0000=oo`00
00001Goo0P000Woo0`0017oo0P000Woo0P001Goo0P000Woo0P001Goo0P0017oo00<007ooOol01Woo
3`0000=oo`0000000`0000Ioo`00000007oo0002Ool3000017oo000000000Woo0P0000Eoo`000000
07oo00800003Ool0000000@000=oo`04001oo`00Ool9000017oo000007oo0`000Woo10000goo0P00
00=oo`0000000`000goo0P000Woo0P001Goo0P000Woo0P001Goo0P0017oo0P0027oo0P001Goo0P00
0Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P0017oo001DOol01@00
Oomoogoo00001Woo00<007ooOol00Woo00H007oo001oo`00Ool200001goo001oogoo001oo`00009o
o`L00003Ool0000000@000Eoo`800004Ool00000Ool30002Ool400000goo001oo`070004Ool60000
17oo000007oo0`004Goo0`001Goo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P00
1Goo0P005Goo0`0000=oo`0000001Goo0P000Woo0`0017oo0P000Woo0P001Goo0P000Woo0P001Goo
0P0017oo00<007ooOol01Woo3`0000=oo`0000000`0000Moo`00000007oo001oo`04000017oo0000
00000Woo0P0000Eoo`00000007oo00800003Ool0000000@000=oo`04001oo`00Ool800001Goo0000
0000Ool00`000Woo10000goo0P0000Ioo`00000007oo0003Ool20002Ool20005Ool20002Ool20005
Ool20004Ool20008Ool20005Ool20002Ool20005Ool20002Ool20005Ool20002Ool20005Ool20002
Ool20005Ool20004Ool005Aoo`03001oogoo00Qoo`04001oogooOol200001Goo001oo`00Ool00P00
00Moo`00Oomoo`00Ool00002Ool700000goo000000040005Ool20002Ool00`00Ool000020002Ool4
00000goo001oo`070004Ool6000017oo000007oo0`004Woo0P001Goo0P001Goo0P000Woo0P001Goo
0P000Woo0P001Goo0P000Woo0P001Goo0P005Goo0`0000=oo`0000001Goo0P000Woo0`0017oo0P00
0Woo0P001Goo0P000Woo0P001Goo0P0017oo00<007ooOol01Woo100000=oo`000000200000=oo`00
00000`0000Moo`00000007oo001oo`0400001Goo00000000Ool00`0000Eoo`00000007oo00800003
Ool0000000@0009oo`800005Ool007oo001oo`0600001Goo00000000Ool00`000Woo10000goo0P00
0Woo0P0000=oo`00Ool00Woo0P000Woo0P001Goo0P000Woo0P001Goo0P0017oo0P0027oo0P001Goo
0P000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P0017oo001DOol2
000:Ool00`00Oomoo`0200001Goo001oo`00Ool00P0000Moo`00Oomoo`00Ool00002Ool700000goo
000000040005Ool20002Ool00`00Ool000020002Ool400000goo001oo`070004Ool6000017oo0000
07oo0`004Woo0`0017oo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P00
57oo100000=oo`0000001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P0017oo
00<007ooOol01goo0`0000=oo`000000200000=oo`0000001`0000=oo`00Ool0100000Eoo`000000
07oo00<00005Ool00000001oo`0200000goo00000005000017oo000007oo0`0000=oo`0000001000
00Eoo`00000007oo00<0009oo`@000=oo`80009oo`800003Ool007oo009oo`80009oo`8000Eoo`80
009oo`8000Eoo`8000Aoo`8000Qoo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`80009oo`80
00Eoo`80009oo`8000Eoo`8000Aoo`00E7oo00<007ooOol02Goo00<007ooOol00P0000Eoo`00Ool0
07oo00800006Ool007ooOol007oo0P0000=oo`0000000`0000=oo`00Ool01P001Goo0P000Woo00<0
07oo00000P000Woo100000=oo`00Ool01`0017oo1P0000Aoo`00001oo`<0019oo`@000=oo`8000Eo
o`80009oo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`80019oo`@00003Ool007oo008000Eo
o`80009oo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8000Aoo`03001oogoo00Moo`<00003
Ool0000000P00003Ool0000000L00003Ool0000000@00005Ool00000001oo`0300001Goo00000000
Ool00P000Woo1P0000Aoo`00001oo`<00003Ool0000000P00004Ool000000002Ool40003Ool20002
Ool200000goo001oo`02Ool20002Ool20005Ool20002Ool20005Ool20004Ool20008Ool30003Ool3
0002Ool20005Ool20002Ool20005Ool20002Ool20005Ool20002Ool20005Ool20004Ool005Aoo`03
001oogoo00Uoo`03001oogoo00800005Ool007oo001oo`0200001Woo001oogoo001oo`800003Ool0
000000<00003Ool007oo00<00003Ool0000000Eoo`80009oo`03001oo`000080009oo`800005Ool0
07oo001oo`070004Ool600000goo00000002Ool2000BOol200000goo001oo`02Ool20005Ool20002
Ool20005Ool20002Ool20005Ool20002Ool20005Ool2000BOol20003Ool01000Ool000001Goo0P00
0Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P0017oo00<007ooOol01goo0`0000=oo`00
0000200000=oo`0000001`0000=oo`000000100000Eoo`00000007oo00<00005Ool00000001oo`02
0002Ool6000017oo000007oo0`0000=oo`00000030000Woo10000goo0P000Woo0P0000=oo`00Ool0
0Woo0P000Woo0P001Goo0P000Woo0P001Goo0P0017oo0P0027oo20000goo0P001Goo0P000Woo0P00
1Goo0P000Woo0P001Goo0P000Woo0P001Goo0P0017oo001DOol00`00Oomoo`09Ool00`00Oomoo`02
00001Goo001oo`00Ool00P0000Ioo`00Oomoo`00Ool200000goo0000000300000goo001oo`030000
0goo00000005Ool20002Ool00`00Ool000020002Ool200001Goo001oo`00Ool01`0017oo1P0000=o
o`0000000Woo0P004Woo0P0000=oo`0000000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo
0P000Woo0P001Goo0P004goo0`0000=oo`00Ool00P001Goo0P000Woo0P001Goo0P000Woo0P001Goo
0P000Woo0P001Goo0P003Woo3P0000=oo`0000001`0000=oo`000000100000Eoo`00000007oo00<0
0005Ool00000001oo`020002Ool6000017oo000007oo0`0000=oo`0000002P0000Aoo`00Oomoo`@0
00=oo`80009oo`800003Ool007oo009oo`80009oo`8000Eoo`80009oo`8000Eoo`8000Aoo`8000Qo
o`800003Ool00000008000Aoo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eo
o`8000Aoo`00E7oo00<007ooOol02Goo00<007ooOol00P0000Eoo`00Ool007oo00800006Ool007oo
Ool007oo0P0000=oo`0000000`0000=oo`00Ool00`0000=oo`0000001Goo0P000Woo00<007oo0000
0P000Woo0P0000Eoo`00Ool007oo00L000Aoo`H00003Ool00000009oo`80019oo`800003Ool00000
009oo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8001Aoo`800003Ool0
07oo008000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8000ioo`800003Ool0
000000T00003Ool0000000L00003Ool0000000@00005Ool00000001oo`0300001Goo00000000Ool0
0P000Woo1P0000=oo`0000000Woo0P0000=oo`000000200000Aoo`00Ool0009oo`@00003Ool007oo
0080009oo`800003Ool007oo009oo`80009oo`8000Eoo`800004Ool000000005Ool20004Ool20008
Ool20009Ool20005Ool20002Ool20005Ool20002Ool20005Ool20002Ool20005Ool20004Ool005Ao
o`03001oogoo00Uoo`03001oogoo00800005Ool007oo001oo`02000017oo001oo`000Woo0P0000=o
o`0000000`0000=oo`00Ool01P001Goo0P000Woo00<007oo00000P000Woo0P0000Eoo`00Ool007oo
00800003Ool00000008000Aoo`H00003Ool00000009oo`8000ioo`80009oo`800003Ool00000009o
o`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8000ioo`8000Aoo`800003
Ool007oo008000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8000ioo`80009o
o`X00003Ool0000000L00003Ool0000000@00005Ool00000001oo`0300001Goo00000000Ool00P00
0Woo1P0000=oo`0000000Woo0P0000=oo`000000200000Aoo`00Ool0009oo`@00003Ool007oo0080
009oo`800003Ool007oo009oo`80009oo`8000Eoo`800004Ool000000005Ool20004Ool20009Ool2
0004Ool20002Ool20005Ool20002Ool20005Ool20002Ool20005Ool20002Ool20005Ool20004Ool0
05Aoo`03001oogoo00Uoo`03001oogoo00800005Ool007oo001oo`02000017oo001oo`000Woo0P00
00=oo`0000000P000Woo00<007oo00001@001Woo0P0000=oo`00Ool00P0017oo0`0000=oo`000000
0P0000=oo`0000000P000goo1@0000=oo`0000000P0000Aoo`00000000moo`800004Ool007ooOol2
0003Ool20003Ool20004Ool20003Ool20004Ool20003Ool20004Ool20003Ool2000?Ool30002Ool3
00000goo001oo`030003Ool20004Ool20003Ool20004Ool20003Ool20004Ool20003Ool2000?Ool2
0002Ool700000goo00000003Ool=0002Ool;0002Ool200000goo0000000400000goo00000002Ool2
00000goo0000000800000goo001oo`020002Ool3000017oo0000000017oo0`000Woo0P0017oo0P00
0goo0P000Woo00@007oo000000=oo`8000Eoo`8000Uoo`8000=oo`<000=oo`8000=oo`8000Aoo`80
00=oo`8000Aoo`8000=oo`8000Aoo`8000=oo`8000Eoo`00E7oo00<007ooOol02Goo00<007ooOol0
0P0000Eoo`00Ool007oo00800004Ool007oo0002Ool200000goo000000020002Ool00`00Ool00002
00000goo00000006Ool70004Ool800000goo0000000:00000goo00000006000?Ool70004Ool70004
Ool70004Ool70004Ool7000@Ool60002Ool00`00Ool000070004Ool70004Ool70004Ool7000?Ool2
0002Ool700000goo0000000700000goo000000060002Ool;0002Ool900000goo00000002Ool20000
0goo0000000>00000goo000000050004Ool70004Ool70002Ool00`00Ool000060005Ool2000:Ool6
0004Ool70004Ool70004Ool70004Ool70005Ool005Aoo`03001oogoo00Uoo`03001oogoo00800005
Ool007oo001oo`02000017oo001oo`000Woo0P0000=oo`0000000P000Woo00<007oo00000P0000=o
o`0000001goo1@001Woo1P000Woo0P0000=oo`0000001`000Woo1@004Woo1@001Woo1@001Woo1@00
1Woo1@001Woo1@004Woo10000goo00<007oo00001P001Goo1P001Woo1@001Woo1@0047oo0P000Woo
00<007oo000010000Woo200000=oo`0000001@000goo2`000goo200000=oo`0000000Woo00<007oo
Ool0400000=oo`0000001@0017oo1`0017oo1P000goo00<007ooOol01@001Woo0P002goo10001Woo
1@001Woo1@001Woo1@001Woo1@001Woo001DOol2000:Ool00`00Oomoo`0200001Goo001oo`00Ool0
0P0000Aoo`00Ool0009oo`800003Ool000000080009oo`05001oo`00Ool0000<Ool00`00Ool00007
Ool00`00Ool00004Ool20006Ool40003Ool2000FOol00`00Oomoo`15Ool00`00Ool0000;Ool00`00
Oomoo`0YOol20002Ool00`00Oomoo`040006Ool30002Ool3000017oo000000000goo1`0000Aoo`00
000000Eoo`H00003Ool00000009oo`03001oogoo00D0009oo`<00003Ool007oo00<00005Ool007oo
001oo`030004Ool01`00Ool007oo001oo`0000Aoo`8000Moo`03001oogoo00]oo`80045oo`00E7oo
00<007ooOol02Goo00<007ooOol00`000Woo00<007oo00000Woo00<007oo00000Woo1`000goo0P00
00=oo`00Ool02goo00<007oo00001goo00<007oo000017oo00<007ooOol01goo0P000goo0P005Woo
00<007ooOol0AGoo00<007oo00002goo00<007ooOol0:Goo0P000Woo00<007ooOol010001Woo00<0
07oo00000Woo0`0000Aoo`00000000=oo`L00004Ool000000005Ool400000goo001oo`020002Ool0
0`00Oomoo`050002Ool30003Ool300001Goo001oo`00Ool00`0017oo00@007oo001oo`<000=oo`<0
00Moo`03001oogoo00]oo`80045oo`00E7oo00<007ooOol02Goo00<007ooOol00`000Woo00<007oo
00000Woo00<007oo00000Woo1P0017oo0P0000=oo`00Ool02goo00<007ooOol01goo00<007oo0000
17oo00<007ooOol01goo0P000goo0P005Woo00<007ooOol0AGoo00<007oo00002goo00<007ooOol0
:Goo0P000Woo00<007ooOol010001Woo00<007oo00000Woo0`0000Aoo`00000000=oo`/000Eoo`80
0005Ool007oo001oo`020002Ool00`00Oomoo`050002Ool30003Ool300001Goo001oo`00Ool00`00
17oo00@007oo001oo`<000=oo`<000Moo`03001oogoo00]oo`80045oo`00E7oo00<007ooOol02Goo
00<007ooOol00`000Woo00<007oo00000Woo00<007oo00000Woo1P0017oo0P0000=oo`00Ool02goo
00<007ooOol01goo00<007oo00003Woo0P000goo0P005Woo00<007ooOol0AGoo00<007oo00002goo
00<007ooOol0:Goo0P000Woo00<007ooOol010001Woo00<007oo00000Woo0`0000Aoo`00000000=o
o`D00003Ool0000000<000Eoo`800005Ool007oo001oo`020002Ool00`00Oomoo`050002Ool20004
Ool300001Goo001oo`00Ool00`0017oo00@007oo001oo`<000=oo`<000Moo`03001oogoo00]oo`80
045oo`00E7oo00<007ooOol02Goo00<007oo00000`000Woo00<007oo00000goo0P000Woo1P0017oo
0P003Woo00<007ooOol01goo00<007oo00003goo00D007ooOomoo`0001Moo`03001oogoo04Eoo`03
001oo`0000]oo`03001oogoo02Uoo`80009oo`03001oogoo00@000Ioo`03001oogoo009oo`<00003
Ool000000080009oo`D00003Ool0000000<000Eoo`07001oogoo001oo`00Ool00P000Woo00<007oo
Ool01@000Woo0P0017oo0`0000=oo`00Ool01@0017oo00@007oo001oo`@0009oo`<000Moo`03001o
ogoo00]oo`80045oo`00E7oo00<007ooOol02Goo00<007oo00000`000Woo00<007oo00000goo0P00
0Woo1P0017oo0P003Woo00<007ooOol01goo00<007oo00003goo00D007ooOomoo`0005moo`03001o
o`0000]oo`03001oogoo02Uoo`05001oogooOol00002Ool01000Ool000002goo0P000Woo10000Woo
1@0000=oo`0000000`001Goo00L007ooOol007oo001oo`020002Ool00`00Oomoo`040003Ool20004
Ool3000017oo001oogoo100017oo00@007oo001oo`D00004Ool000000007Ool00`00Oomoo`0;Ool2
0011Ool005Aoo`03001oogoo00Uoo`03001oo`0000<0009oo`8000Aoo`80009oo`H000Aoo`03001o
ogoo01Moo`<000moo`05001oogooOol0001OOol00`00Ool0000;Ool00`00Oomoo`0]Ool01@00Oomo
o`00Ool00P002goo0P000Woo10000Woo1@0000=oo`0000000`001Goo00L007ooOol007oo001oo`02
0002Ool00`00Oomoo`040003Ool20004Ool3000017oo001oogoo100017oo00@007oo001oo`T000Mo
o`03001oogoo00]oo`80045oo`00E7oo00<007ooOol02Goo00<007oo00000`000Woo0P0017oo0P00
0Woo1P0017oo00<007ooOol05goo0`003goo00D007ooOomoo`0005moo`03001oo`0000]oo`03001o
ogoo02eoo`06001oogoo001oo`0037oo0P000Woo0P0000Ioo`00Oomoo`00Ool300000goo00000003
0005Ool01`00Oomoo`00Ool007oo008000Eoo`@000=oo`8000Aoo`<00004Ool007ooOol40004Ool0
1000Ool007oo2@001Woo0P0027oo0P000goo0P00@Goo0019Oooo002Y000DOol005Aoo`03001oogoo
00Uoo`H0009oo`8000Aoo`80009oo`04001oo`00Ool20004Ool00`00Oomoo`0:Ool00`00Oomoo`0:
Ool3000;Ool00`00Oomoo`0;Ool00`00Oomoo`0:Ool00`00Oomoo`0:Ool00`00Oomoo`0;Ool00`00
Oomoo`0:Ool00`00Oomoo`0;Ool00`00Oomoo`0:Ool00`00Oomoo`07Ool01@00Ool007oo00002Goo
00@007ooOol000aoo`03001oogoo00]oo`03001oogoo00Yoo`03001oogoo009oo`06001oogoo001o
o`000goo00<007ooOol01Woo0P000Woo0P0000Yoo`00Oomoo`00Ool007oo001oo`D000Qoo`05001o
o`00Ool00006Ool200000goo00000002Ool20004Ool3000017oo000007oo100017oo00<007oo0000
200000=oo`00Ool00goo00@007oo0000009oo`03001oogoo00=oo`800003Ool007oo008000Eoo`04
001oogoo000<Ool01@00Oomoogoo000027oo00<007ooOol077oo001DOol00`00Oomoo`09Ool60002
Ool20004Ool20002Ool01000Ool007oo0P0017oo00<007ooOol02Woo00<007ooOol02Woo0P0037oo
00<007ooOol02goo00<007ooOol02Woo00<007ooOol02Woo00<007ooOol02goo00<007ooOol02Woo
00<007ooOol02goo00<007ooOol02Woo00<007ooOol01goo00D007oo001oo`0000Uoo`04001oogoo
000<Ool00`00Oomoo`0;Ool00`00Oomoo`0:Ool00`00Oomoo`02Ool01P00Oomoo`00Ool000=oo`03
001oogoo00Ioo`80009oo`80000:Ool007ooOol007oo001oo`00Ool300000goo001oo`07Ool01@00
Ool007oo00001Woo0P0000=oo`0000000Woo0P0017oo0`0000Aoo`00001oo`800003Ool007oo00=o
o`03001oo`0000P00003Ool007oo009oo`800003Ool00000009oo`03001oogoo00=oo`800003Ool0
07oo008000Eoo`04001oogoo0002Ool00`00Oomoo`07Ool01@00Oomoogoo000027oo00<007ooOol0
77oo001DOol00`00Oomoo`09Ool00`00Ool000030002Ool20004Ool20002Ool01000Ool007oo0P00
17oo00<007ooOol05goo0P00@Woo00<007ooOol0;goo00<007oo00002goo00@007ooOol002aoo`06
001oogoo001oo`0037oo0P000Woo0P0000Yoo`00Oomoo`00Ool007oo001oo`80009oo`03001oogoo
00Ioo`05001oo`00Ool00006Ool200000goo001oo`02Ool20004Ool50002Ool200000goo001oo`03
Ool00`00Ool0000800000goo00000002Ool00`00Oomoo`020002Ool00`00Oomoo`03Ool200000goo
001oo`030004Ool00`00Oomoo`03Ool00`00Oomoo`05Ool00`00Oomoo`03Ool00`00Oomoo`06Ool0
0`00Oomoo`0LOol005Aoo`03001oogoo00Uoo`04001oo`00Ool20002Ool20004Ool20004Ool01000
Ool0000017oo00<007ooOol05goo00<007ooOol0Lgoo00<007oo00002goo00<007ooOol0;Goo00H0
07ooOol007oo000=Ool00`00Oomoo`0200002Woo001oogoo001oo`00Ool007oo0P000Woo00<007oo
Ool01Woo00D007oo001oo`0000Ioo`800003Ool007oo009oo`8000Aoo`D000=oo`03001oo`0000Ao
o`06001oo`00Ool007oo1@0000=oo`0000000Woo00<007ooOol00P000Woo00<007ooOol00goo0P00
00=oo`00Ool00`0017oo00L007ooOomoo`00Ool00006Ool20005Ool2000VOol005Aoo`03001oogoo
00Uoo`03001oo`00009oo`03001oogoo008000Aoo`8000Aoo`04001oo`000004Ool00`00Oomoo`0G
Ool00`00Oomoo`1cOol00`00Ool0000;Ool00`00Oomoo`0]Ool01P00Oomoo`00Ool000eoo`03001o
ogoo0080000:Ool007ooOol007oo001oo`00Ool20002Ool00`00Oomoo`06Ool01@00Ool007oo0000
1Woo0P0000=oo`00Ool00Woo0P0017oo1@000goo00<007oo000017oo00H007oo001oo`00Ool50000
0goo00000002Ool00`00Oomoo`020002Ool00`00Oomoo`03Ool200000goo001oo`030004Ool01`00
Oomoogoo001oo`0000Ioo`8000Eoo`8002Ioo`00E7oo00<007ooOol02Goo00<007oo00000Woo00<0
07ooOol00P0017oo0P0017oo00@007oo000000Aoo`03001oogoo01Moo`03001oogoo07=oo`03001o
o`0000]oo`03001oogoo02eoo`06001oogoo001oo`003Goo00<007ooOol00P0000Yoo`00Oomoo`00
Ool007oo001oo`80009oo`03001oogoo00Ioo`05001oo`00Ool00006Ool200000goo001oo`02Ool2
0004Ool50003Ool00`00Ool00004Ool01P00Ool007oo001oo`D00003Ool00000009oo`03001oogoo
00<00003Ool007oo00Aoo`800003Ool007oo00<000Aoo`07001oogooOol007oo00001Woo0P001Goo
0P009Woo001DOol00`00Oomoo`09Ool00`00Ool00002Ool00`00Oomoo`020004Ool20004Ool01000
Ool0000017oo00<007ooOol05goo00<007ooOol0Lgoo00<007oo00002goo00<007ooOol0<7oo00<0
07oo00003Goo00<007ooOol00P0000Yoo`00Oomoo`00Ool007oo001oo`80009oo`03001oogoo00Io
o`05001oo`00Ool00006Ool200000goo001oo`02Ool20004Ool50003Ool00`00Ool00006Ool01000
Ool007oo1@0000=oo`0000000Woo00<007ooOol01@001Goo0P0000=oo`00Ool00`0017oo00L007oo
Oomoo`00Ool00006Ool20005Ool2000VOol005Aoo`03001oogoo00Uoo`03001oo`00009oo`03001o
ogoo008000Aoo`8000Aoo`04001oo`000004Ool00`00Oomoo`2=Ool00`00Ool0000;Ool00`00Oomo
o`0bOol00`00Oomoo`0;Ool00`00Oomoo`0200002Goo001oogoo001oo`00Ool00002Ool01000Oomo
o`0027oo00@007oo000000Moo`800003Ool007oo00=oo`03001oogoo009oo`D000=oo`03001oo`00
00Ioo`04001oo`00Ool500000goo00000002Ool00`00Oomoo`050005Ool200000goo001oo`030004
Ool01`00Oomoogoo001oo`0000Ioo`<000Aoo`8002Ioo`00E7oo0P002Woo00<007oo00000Woo00<0
07ooOol00P0017oo0P001Woo0P0017oo00<007ooOol0SGoo00<007oo00002goo00<007ooOol0@7oo
00<007ooOol00P0000Uoo`00Oomoo`00Ool007oo00000Woo00@007ooOol000Qoo`04001oo`000007
Ool20006Ool00`00Oomoo`02Ool30005Ool00`00Ool00006Ool01000Ool007oo1@0000=oo`000000
0Woo00<007ooOol01@001Goo0P0000=oo`00Ool00`0017oo00L007ooOomoo`00Ool00006Ool30004
Ool2000VOol005Aoo`03001oogoo00Uoo`8000=oo`03001oogoo008000Aoo`8000Ioo`8000Aoo`03
001oogoo08eoo`03001oo`0000]oo`03001oogoo041oo`03001oogoo00800009Ool007ooOol007oo
001oo`00009oo`04001oogoo0008Ool01000Ool000001goo0P001Woo00<007ooOol00Woo0`001Goo
00<007oo00001Woo00@007oo001oo`D00003Ool00000009oo`03001oogoo00D000Eoo`800003Ool0
07oo00<000Aoo`07001oo`00Ool007oo00001Woo0`0017oo0P009Woo001DOol00`00Oomoo`09Ool2
0003Ool00`00Oomoo`020004Ool20006Ool20004Ool00`00Oomoo`2=Ool00`00Ool0000;Ool00`00
Oomoo`10Ool01000Oomoo`000Woo00P007ooOol007oo001oo`000Woo00@007ooOol000Qoo`04001o
o`000007Ool2000;Ool30005Ool00`00Ool00006Ool300000goo001oo`0300000goo00000002Ool0
0`00Ool000050005Ool00`00Ool000050003Ool200001Woo001oo`00Ool000Ioo`<000Aoo`8002Io
o`00E7oo00<007ooOol02Woo00D007ooOomoo`00009oo`8000Aoo`8000Ioo`8000Aoo`03001oogoo
08eoo`03001oo`0000]oo`03001oogoo041oo`04001oogoo0002Ool02000Oomoo`00Ool007oo0002
Ool01000Oomoo`0027oo00@007oo000000Moo`8000]oo`<000Moo`03001oogoo00Aoo`<00003Ool0
07oo00<00005Ool00000001oo`090004Ool01000Ool007oo10000goo0P0000Ioo`00Ool007oo0006
Ool30004Ool2000VOol005Aoo`03001oogoo00Yoo`05001oogooOol00002Ool20004Ool20006Ool2
0004Ool00`00Oomoo`2=Ool00`00Ool0000;Ool00`00Oomoo`10Ool01000Oomoo`000Woo00P007oo
Ool007oo001oo`000Woo00@007ooOol000Qoo`04001oo`000007Ool2000;Ool30007Ool00`00Oomo
o`04Ool300000goo001oo`0300001Goo00000000Ool02@0017oo00@007oo001oo`@000=oo`800006
Ool007oo001oo`001Woo0`0017oo0P009Woo001DOol00`00Oomoo`0:Ool01@00Oomoogoo00000Woo
0P0017oo0P001Woo0P00U7oo00<007oo00002goo00<007ooOol0@7oo00@007ooOol0009oo`08001o
ogoo001oo`00Ool0009oo`03001oogoo00Uoo`04001oo`000007Ool2000;Ool30007Ool00`00Oomo
o`04Ool300000goo001oo`0300001Goo00000000Ool02@0000Qoo`00Oomoo`00Ool007oo0P0000=o
o`00Ool00Woo0P0000Ioo`00Ool007oo0006Ool40003Ool2000VOol005Aoo`03001oogoo00Yoo`05
001oogooOol00002Ool20004Ool20006Ool2002EOol2000;Ool00`00Oomoo`10Ool01000Oomoo`00
0Woo00H007ooOol007oo000@Ool01000Ool000001goo0P002goo0`001goo00<007ooOol017oo0`00
00=oo`00Ool00`0000Eoo`00000007oo00<00003Ool0000000<00008Ool007ooOol007oo001oo`80
0003Ool007oo009oo`800006Ool007oo001oo`001Woo10000goo0P009Woo001DOol00`00Oomoo`0:
Ool01@00Oomoogoo00000Woo0P0017oo0P001Woo0P00UGoo0P002goo00<007ooOol0@7oo00@007oo
Ool0009oo`06001oogoo001oo`004Woo0P001goo0P002goo0`003Woo0`0000=oo`00Ool00`0000Eo
o`00000007oo00<0009oo`@00008Ool007ooOol007oo001oo`800003Ool00000009oo`800006Ool0
07oo001oo`001Woo10000goo0P009Woo001DOol2000;Ool01@00Oomoogoo00000Woo0P0017oo0P00
1Woo0P00UGoo0P002goo00<007ooOol0@7oo00@007ooOol0009oo`04001oogoo000DOol20007Ool2
000;Ool3000>Ool300000goo001oo`0300001Goo00000000Ool00`000Woo100000Qoo`00Oomoo`00
Ool007oo0P000Woo00<007ooOol00P0000Ioo`00Ool007oo0006Ool01000Ool000000goo0P009Woo
001DOol00`00Oomoo`0:Ool01@00Oomoogoo00000Woo0P0017oo0P001Woo0P00UGoo0P002goo00<0
07ooOol0@7oo00@007ooOol0009oo`800003Ool007oo01=oo`8000Qoo`03001oogoo00Uoo`<000mo
o`8000=oo`80009oo`<00004Ool000000002Ool4000027oo001oogoo001oo`00Ool20002Ool00`00
Oomoo`0200000goo000000040005Ool01000Ool000000Woo00<007oo00009Woo001DOol00`00Oomo
o`0:Ool01@00Oomoogoo00000Woo0P0017oo0P001Woo0P00UGoo0P002goo00<007ooOol0@7oo00@0
07ooOol0009oo`800003Ool007oo01=oo`8000Qoo`03001oogoo00Uoo`80011oo`8000=oo`80009o
o`<00004Ool000000003Ool01@00Ool007oo00000Woo00@007oo001oo`80009oo`03001oogoo0080
0003Ool0000000@000Eoo`04001oo`000002Ool00`00Ool0000VOol005Aoo`03001oogoo00Yoo`05
001oogooOol00002Ool20004Ool20006Ool2002EOol2000;Ool00`00Oomoo`13Ool00`00Oomoo`02
00000goo001oo`0DOol00`00Oomoo`06Ool00`00Oomoo`09Ool2000@Ool20003Ool20002Ool30000
17oo000000000goo00H007oo001oo`00Ool200000goo001oo`020002Ool00`00Oomoo`0200000goo
000000040005Ool01000Ool000000Woo00<007oo00009Woo001DOol00`00Oomoo`0:Ool00`00Oomo
o`04Ool20004Ool20006Ool2002EOol2000;Ool00`00Oomoo`13Ool00`00Oomoo`0200000goo001o
o`0DOol00`00Oomoo`06Ool00`00Oomoo`09Ool2000@Ool20003Ool20002Ool3000017oo00000000
0goo00H007oo001oo`00Ool200000goo001oo`020002Ool2000017oo000007oo1P001Goo00@007oo
0000009oo`03001oo`0002Ioo`00E7oo00<007ooOol02Woo00<007ooOol017oo0P0017oo0P001Woo
0P00UGoo0P00DGoo00L007ooOomoo`00Ool0000EOol00`00Oomoo`06Ool00`00Oomoo`09Ool2000A
Ool01000Oomoogoo0P000Woo0`0000Aoo`00000000=oo`06001oo`00Ool007oo0P0000=oo`00Ool0
0P000Woo0P0000=oo`0000001`001Goo00@007oo0000009oo`03001oo`0002Ioo`00E7oo00<007oo
Ool02Woo00<007ooOol017oo0P0017oo0P001Woo0P00UGoo0P00DGoo00L007ooOomoo`00Ool0000E
Ool00`00Oomoo`06Ool00`00Oomoo`09Ool2000AOol01000Oomoogoo0P000Woo0`0000Aoo`000000
00=oo`06001oo`00Ool007oo0P0000=oo`00Ool00P000Woo0P0000=oo`0000001`001Goo00@007oo
0000009oo`03001oo`0002Ioo`00E7oo00<007ooOol02Woo00<007ooOol017oo0P0017oo0P001Woo
0P00UGoo0P00DGoo00L007ooOomoo`00Ool0000EOol00`00Oomoo`06Ool00`00Oomoo`09Ool2000A
Ool01000Oomoogoo0P000Woo0`0000Aoo`00Ool000=oo`06001oo`00Ool007oo0P0000=oo`00Ool0
0P000Woo0P0000=oo`0000001`001Goo00@007oo0000009oo`03001oo`0002Ioo`00E7oo00<007oo
Ool02Woo00<007ooOol017oo0P0017oo0P001Woo0P00UGoo0P00EGoo00<007oo00007Woo00<007oo
Ool02Goo0P004Goo00@007ooOomoo`80009oo`<00004Ool007oo0003Ool01P00Ool007oo001oo`80
0003Ool007oo0080009oo`800003Ool0000000L000Eoo`04001oo`000002Ool00`00Ool00006Ool0
0`00Oomoo`0MOol005Aoo`8000]oo`03001oogoo00Aoo`8000Aoo`8000Moo`03001oogoo09=oo`80
05Eoo`03001oo`0001ioo`03001oogoo00Uoo`80015oo`04001oogooOol20002Ool3000017oo001o
o`000goo00H007oo001oo`00Ool200000goo001oo`020003Ool;0005Ool01000Ool000000Woo00<0
07oo00001Woo00<007ooOol07Goo001DOol00`00Oomoo`0AOol20004Ool20007Ool00`00Oomoo`2C
Ool2001EOol00`00Ool0000NOol00`00Oomoo`09Ool2000AOol01000Oomoogoo0P000Woo0`0000Ao
o`00Ool000=oo`06001oo`00Ool007oo0P0000=oo`00Ool00P000goo300017oo00@007oo0000009o
o`03001oo`0000Ioo`03001oogoo01eoo`00E7oo00<007ooOol04Goo0P0017oo0P001goo00<007oo
Ool0Tgoo0P00EGoo00<007oo00007Woo00<007ooOol02Goo0P004Goo00@007ooOomoo`80009oo`<0
0004Ool007oo0003Ool00`00Ool0000200001Goo000007oo00000Woo00@007ooOomoo``00006Ool0
07ooOol007oo0P000Woo00<007oo00001Woo00<007ooOol07Goo001DOol00`00Oomoo`0AOol20004
Ool20007Ool00`00Oomoo`2COol2001EOol00`00Ool0000NOol00`00Oomoo`09Ool2000FOol02P00
Oomoo`00Ool007oo001oo`000goo00<007oo00000P0000Eoo`00001oo`00009oo`03001oogoo009o
o`d0009oo`04001oo`000002Ool00`00Ool00006Ool00`00Oomoo`0MOol005Aoo`03001oogoo015o
o`8000Aoo`8000Moo`03001oogoo09=oo`8005Eoo`03001oo`0001ioo`03001oogoo00Uoo`8001Io
o`05001oogoo001oo`0300000goo001oo`02Ool00`00Ool0000200001Goo000007oo00000Woo00<0
07ooOol00Woo0P0000=oo`00000020000Woo00@007oo0000009oo`03001oo`0000Ioo`03001oogoo
01eoo`00E7oo00<007ooOol04Goo0P0017oo0P001goo00<007ooOol0Tgoo0P00EGoo0P007goo00<0
07ooOol02Goo0P006Goo00<007oo00000P0000=oo`00Ool00Woo00D007oo000007oo00<00005Ool0
07ooOol00004Ool200000goo000000080002Ool01000Ool000000Woo00<007oo00001Woo00<007oo
Ool07Goo001DOol00`00Oomoo`0AOol20004Ool20007Ool00`00Oomoo`2COol2001EOol2000OOol0
0`00Oomoo`09Ool2000IOol00`00Oomoo`0200000goo001oo`02Ool01@00Ool00000Ool00`0000=o
o`00Ool01Woo0P0000Aoo`00001oo`L0009oo`04001oo`000002Ool00`00Ool00006Ool00`00Oomo
o`0MOol005Aoo`03001oogoo015oo`8000Aoo`8000Moo`03001oogoo09=oo`8005Eoo`8001moo`03
001oogoo00Uoo`8001Uoo`03001oogoo008000Eoo`05001oo`00001oo`020002Ool00`00Oomoo`05
Ool2000017oo000007oo1`000Woo00@007oo0000009oo`03001oo`0000Ioo`8001ioo`00E7oo0P00
4Woo0P0017oo0P001goo00<007ooOol0Tgoo0P00EGoo0P007goo00<007ooOol02Goo0P006Goo00<0
07ooOol00P001Goo00D007oo000007oo008000Yoo`800004Ool00000Ool200000goo000000020002
Ool01000Ool000000Woo00<007oo00001Woo0P007Woo001DOol00`00Oomoo`0AOol00`00Oomoo`03
Ool00`00Oomoo`06Ool00`00Oomoo`2COol2001EOol2000OOol00`00Oomoo`09Ool2000IOol00`00
Oomoo`020005Ool01@00Ool00000Ool00P002Woo0P0000Aoo`00001oo`800003Ool000000080009o
o`04001oo`000002Ool00`00Ool00006Ool2000NOol005Aoo`03001oogoo015oo`03001oogoo00=o
o`03001oogoo00Ioo`03001oogoo09=oo`8005Eoo`8001moo`03001oogoo00Uoo`8001Uoo`03001o
ogoo008000Eoo`05001oo`00001oo`02000:Ool20002Ool01@00Ool00000Ool010000Woo00D007oo
000007oo00800003Ool007oo00Eoo`8001ioo`005Goo0P001goo1@001goo10001Woo10001Woo0P00
1Goo10002goo00<007ooOol04Goo00<007ooOol00goo00<007ooOol01Woo00<007ooOol0Tgoo0P00
EGoo0P007goo00<007ooOol02Goo0P006Goo00<007ooOol00P001Goo00H007oo000007oo000;Ool2
0002Ool01@00Ool00000Ool01@0000=oo`00Ool00P0000Eoo`00001oo`0000Ioo`8001ioo`005Goo
0P001Woo1`0017oo1`0017oo1`0017oo0P0017oo1`002Goo00<007ooOol04Goo00<007ooOol00goo
00<007ooOol01Woo00<007ooOol0Tgoo0P00EGoo0P007goo00<007ooOol02Goo0P0077oo0P001Goo
00H007oo000007oo000;Ool01P00Oomoogoo001oo`800004Ool00000Ool4000017oo000007oo0P00
00=oo`00Ool01Goo0P007Woo000EOol20006Ool20003Ool20004Ool20003Ool20003Ool30003Ool2
0009Ool30003Ool20009Ool00`00Oomoo`0AOol00`00Oomoo`03Ool00`00Oomoo`06Ool00`00Oomo
o`2COol2001EOol2000OOol00`00Oomoo`09Ool2000LOol20005Ool01P00Ool00000Ool000]oo`06
001oogooOol007oo0P000Woo00<007oo00000`0000Aoo`00001oo`800003Ool007oo00Eoo`8001io
o`005Goo0P001Goo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P0027oo0P001Goo0P0027oo
00<007ooOol04Goo00<007ooOol00goo00<007ooOol0W7oo0P00EGoo0P007goo00<007ooOol02Goo
0P0077oo0P001Goo00H007oo000007oo000;Ool01P00Oomoogoo001oo`80009oo`03001oo`0000<0
0005Ool00000Ool00002Ool00`00Oomoo`04Ool2000NOol000Aoo`X000Moo`8000Eoo`8000Eoo`80
009oo`8000Eoo`8000Uoo`8000moo`8000Qoo`03001oogoo015oo`03001oogoo00=oo`03001oogoo
09aoo`8005Eoo`8001moo`03001oogoo00Uoo`8001aoo`8000Eoo`06001oo`00001oo`002goo00H0
07ooOomoo`00Ool20002Ool00`00Ool0000300001Goo000007oo00000Woo00<007ooOol017oo0P00
7Woo000EOol20005Ool20005Ool20002Ool20005Ool20009Ool2000:Ool400000goo00000008Ool0
0`00Oomoo`0AOol00`00Oomoo`03Ool00`00Oomoo`2LOol2001EOol2000OOol00`00Oomoo`09Ool2
000LOol20005Ool01P00Ool00000Ool000]oo`06001oogooOol007oo0P000Woo00<007oo00000`00
00Eoo`00001oo`00009oo`03001oogoo00Aoo`8001ioo`005Goo0P001Goo0P001Goo0P000Woo0P00
1Goo0P002Goo0P002Goo200027oo0`004Goo00<007ooOol00goo00<007ooOol0W7oo0P00EGoo0P00
7goo00<007ooOol02Goo0P0077oo00<007ooOol01goo00<007oo00002goo00H007ooOomoo`00Ool2
0002Ool00`00Ool0000300001Goo000007oo00000Woo00<007ooOol017oo0P007Woo000EOol20005
Ool20005Ool20002Ool30003Ool30002Ool20004Ool30008Ool30003Ool30008Ool00`00Oomoo`0A
Ool00`00Oomoo`03Ool00`00Oomoo`2LOol2001EOol2000OOol00`00Oomoo`09Ool2000LOol00`00
Oomoo`07Ool00`00Ool0000;Ool01P00Oomoogoo001oo`80009oo`03001oo`0000<00005Ool00000
Ool00002Ool00`00Oomoo`04Ool00`00Oomoo`0MOol001Eoo`8000Eoo`8000Eoo`80009oo`P000=o
o`P000Uoo`8000Eoo`8000Qoo`03001oogoo015oo`03001oogoo00=oo`03001oogoo09aoo`8005Eo
o`8001moo`03001oogoo00Uoo`8001aoo`03001oogoo00Moo`03001oo`0000]oo`03001oogoo00=o
o`80009oo`03001oo`0000<00005Ool00000Ool00002Ool00`00Oomoo`04Ool00`00Oomoo`0MOol0
01Eoo`8000Eoo`8000Eoo`80009oo`800003Ool00000008000Aoo`800003Ool00000008000Yoo`80
00Eoo`8000Qoo`03001oogoo015oo`03001oogoo00=oo`03001oogoo09aoo`8005Eoo`8001moo`03
001oogoo00Uoo`8001aoo`03001oogoo00Moo`03001oo`0000]oo`03001oogoo00=oo`8000Aoo`@0
0005Ool00000Ool00002Ool00`00Oomoo`04Ool00`00Oomoo`0MOol0019oo`800003Ool0000000Eo
o`8000Eoo`80009oo`8000Uoo`8000moo`8000Eoo`8000Qoo`03001oogoo015oo`03001oogoo00=o
o`03001oogoo09aoo`8005Eoo`8001moo`03001oogoo00Uoo`8001aoo`03001oogoo00Moo`03001o
o`0000]oo`03001oogoo00=oo`8000Eoo`<00005Ool00000Ool00002Ool00`00Oomoo`04Ool00`00
Oomoo`0MOol0019oo`D000Eoo`8000Eoo`8000=oo`8000Aoo`8000=oo`8000ioo`8000Eoo`8000Qo
o`03001oogoo01Moo`03001oogoo09aoo`8005Eoo`8001moo`03001oogoo00Uoo`8001aoo`03001o
ogoo00Moo`03001oo`0000]oo`03001oogoo00=oo`8000Eoo`<00004Ool000000003Ool00`00Oomo
o`04Ool00`00Oomoo`0MOol001Aoo`<000Ioo`8000=oo`8000Aoo`8000=oo`<000=oo`8000moo`80
00=oo`8000Uoo`03001oogoo01Moo`03001oogoo09aoo`8005Eoo`8001moo`03001oogoo00Uoo`80
01aoo`03001oogoo00Moo`03001oo`0000]oo`03001oogoo00=oo`8000Eoo`<0009oo`8000=oo`03
001oogoo00Aoo`03001oogoo01eoo`005Goo0P001Woo1`001Goo1P0017oo1`002Woo1`002Goo00<0
07ooOol05goo00<007ooOol0W7oo0P00EGoo0P007goo00<007ooOol02Goo0P0077oo00<007ooOol0
1goo00<007oo00002goo00<007ooOol00goo0P001Goo0`000Woo0P000goo00<007ooOol017oo00<0
07ooOol07Goo000EOol20007Ool50007Ool40005Ool7000;Ool4000;Ool2000HOol00`00Oomoo`2L
Ool2001EOol2000OOol00`00Oomoo`09Ool2000LOol00`00Oomoo`07Ool00`00Ool0000AOol20005
Ool30002Ool20004Ool00`00Oomoo`03Ool00`00Oomoo`0MOol005Aoo`03001oogoo01Moo`03001o
ogoo09aoo`8005Eoo`8001moo`03001oogoo00Uoo`8001aoo`03001oogoo00Moo`03001oo`00015o
o`8000Eoo`8000=oo`8000Aoo`03001oogoo00=oo`03001oogoo01eoo`00E7oo00<007ooOol05goo
00<007ooOol0W7oo0P00EGoo0P007goo00<007ooOol02Goo0P0077oo00<007ooOol01goo00<007oo
00004Goo0P001Goo0P000goo0P0017oo00<007ooOol00Woo0P007goo001DOol00`00Oomoo`0GOol0
0`00Oomoo`2LOol2001EOol2000OOol00`00Oomoo`09Ool2000LOol00`00Oomoo`07Ool00`00Oomo
o`0AOol20006Ool01000Oomoogoo0P0017oo00<007ooOol00Woo00<007ooOol07Woo001DOol00`00
Oomoo`0GOol00`00Oomoo`2LOol2001EOol2000OOol00`00Oomoo`09Ool2000LOol00`00Oomoo`0K
Ool20006Ool01000Oomoogoo0P0017oo00<007ooOol00`0087oo001DOol00`00Oomoo`0GOol00`00
Oomoo`2LOol00`00Oomoo`1DOol2000OOol00`00Oomoo`09Ool2000LOol00`00Oomoo`0KOol2000:
Ool20004Ool00`00Oomoo`03000POol005Aoo`03001oogoo01Moo`03001oogoo09aoo`03001oogoo
05Aoo`8001moo`03001oogoo00Uoo`8001aoo`03001oogoo01]oo`8000]oo`03001oogoo009oo`03
001oogoo00<0021oo`00E7oo00<007ooOol05goo00<007ooOol0W7oo00<007ooOol0E7oo0P007goo
00<007ooOol02Goo0P0077oo00<007ooOol06goo0P002goo00<007ooOol00Woo00<007ooOol00`00
87oo001DOol00`00Oomoo`0GOol00`00Oomoo`2LOol00`00Oomoo`1DOol2000OOol00`00Oomoo`09
Ool2000jOol2000;Ool00`00Oomoo`02Ool00`00Oomoo`03000POol005Aoo`8001Qoo`03001oogoo
09aoo`03001oogoo05Eoo`03001oogoo01eoo`03001oogoo00Uoo`8003Yoo`8000]oo`03001oogoo
009oo`03001oogoo00<0021oo`00E7oo00<007ooOol0ogoo3goo00<007ooOol07Goo00<007ooOol0
2Goo0P00>Woo0P002goo00<007ooOol00Woo00<007ooOol00`0087oo001DOol00`00Oomoo`3oOol?
Ool00`00Oomoo`0MOol00`00Oomoo`09Ool2000jOol2000;Ool00`00Oomoo`02Ool00`00Oomoo`03
000POol005Aoo`03001oogoo0?moobmoo`03001oogoo00Uoo`8003Yoo`8000]oo`03001oogoo009o
o`03001oo`000080025oo`00E7oo00<007ooOol0ogoo;goo00<007ooOol02Goo0P00>Woo0P002goo
00<007ooOol00Woo00<007oo00000P008Goo001DOol00`00Oomoo`3oOol_Ool00`00Oomoo`09Ool2
000jOol2000;Ool00`00Oomoo`02Ool00`00Ool00002000QOol005Aoo`03001oogoo0?moobmoo`03
001oogoo00Uoo`8003Yoo`8000]oo`03001oogoo009oo`03001oo`000080025oo`00E7oo00<007oo
Ool0ogoo;goo00<007ooOol02Goo0P00>Woo00<007ooOol02Woo00<007ooOol00Woo00<007oo0000
0P008Goo001DOol2003oOol`Ool00`00Oomoo`09Ool2000jOol00`00Oomoo`0?Ool00`00Ool00002
000QOol005Aoo`03001oogoo0?moobmoo`03001oogoo00Uoo`8003Yoo`03001oogoo00moo`05001o
o`00Ool0000QOol005Aoo`03001oogoo0?moobmoo`03001oogoo00Uoo`8003Yoo`03001oogoo00mo
o`05001oo`00Ool0000QOol005Aoo`03001oogoo0?moobmoo`03001oogoo00Uoo`8003Yoo`03001o
ogoo00moo`05001oo`00Ool0000QOol005Aoo`03001oogoo0?moobmoo`03001oogoo00Uoo`8003Yo
o`03001oogoo00moo`05001oo`00Ool0000QOol005Aoo`03001oogoo0?moobmoo`03001oogoo00Uo
o`8003Yoo`03001oogoo011oo`8002=oo`00E7oo00<007ooOol0ogoo;goo00<007ooOol02Goo0P00
>Woo00<007ooOol047oo0P008goo001DOol00`00Oomoo`3oOol_Ool00`00Oomoo`09Ool2000jOol0
0`00Oomoo`0@Ool2000SOol005Aoo`03001oogoo0?moobmoo`03001oogoo00Uoo`03001oogoo03Uo
o`03001oogoo011oo`8002=oo`00E7oo0P00ogoo<7oo00<007ooOol02Goo00<007ooOol0>Goo00<0
07ooOol047oo0P008goo001DOol00`00Oomoo`3oOol_Ool00`00Oomoo`09Ool00`00Oomoo`0iOol0
0`00Oomoo`0@Ool2000SOol001Eoo`8000Moo`D000Moo`@000Ioo`@000Ioo`8000Eoo`D000Yoo`03
001oogoo0?mooc]oo`03001oogoo03Uoo`03001oogoo011oo`8002=oo`005Goo0P001Woo1`0017oo
1`0017oo1`0017oo0P0017oo1`002Goo00<007ooOol0ogoo>goo00<007ooOol0>Goo00<007ooOol0
47oo0P008goo000EOol20006Ool20003Ool20004Ool20003Ool20003Ool30003Ool20009Ool30003
Ool30008Ool00`00Oomoo`3oOolkOol00`00Oomoo`0iOol00`00Oomoo`0@Ool2000SOol001Eoo`80
00Eoo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8000Qoo`8000Eoo`8000Qoo`03001oogoo
0?mooc]oo`03001oogoo03Uoo`03001oogoo011oo`8002=oo`0017oo2P001goo0P001Goo0P001Goo
0P000Woo0P001Goo0P002Goo0P0027oo0P001Goo0P0027oo00<007ooOol0ogoo>goo00<007ooOol0
>Goo00<007ooOol047oo0P008goo000EOol20005Ool20005Ool20002Ool20005Ool20009Ool20008
Ool20005Ool20008Ool00`00Oomoo`3oOolkOol00`00Oomoo`0iOol00`00Oomoo`0@Ool2000SOol0
01Eoo`8000Eoo`8000Eoo`80009oo`8000Eoo`8000Uoo`8000Qoo`<000=oo`<000Qoo`<00?mooc]o
o`03001oogoo03Uoo`03001oogoo011oo`8002=oo`005Goo0P001Goo0P001Goo0P000Woo0`000goo
0`000Woo0P0017oo0`002Goo1`002Goo00<007ooOol0ogoo>goo00<007ooOol0>Goo00<007ooOol0
47oo0P008goo000EOol20005Ool20005Ool20002Ool80003Ool8000:Ool70009Ool00`00Oomoo`3o
OolkOol00`00Oomoo`0iOol00`00Oomoo`0@Ool2000SOol001Eoo`8000Eoo`8000Eoo`80009oo`80
0003Ool00000008000Aoo`800003Ool00000008000Yoo`<000=oo`<000Qoo`03001oogoo0?mooc]o
o`03001oogoo03Uoo`03001oogoo011oo`8002=oo`004Woo0P0000=oo`0000001Goo0P001Goo0P00
0Woo0P002Goo0P003goo0P001Goo0P0027oo00<007ooOol0ogoo>goo00<007ooOol0>Goo00<007oo
Ool047oo0P008goo000BOol50005Ool20005Ool20003Ool20004Ool20003Ool2000>Ool20005Ool2
0008Ool00`00Oomoo`3oOolkOol00`00Oomoo`0iOol00`00Oomoo`0@Ool2000SOol001Aoo`<000Io
o`8000=oo`8000Aoo`8000=oo`<000=oo`8000ioo`<000=oo`<000Qoo`03001oogoo0?mooc]oo`03
001oogoo03Uoo`03001oogoo011oo`8002=oo`005Goo0P001Woo1`001Goo1P0017oo1`002Woo1`00
2Goo00<007ooOol0ogoo>goo00<007ooOol0>Goo00<007ooOol047oo0P008goo000EOol20007Ool5
0007Ool40005Ool7000;Ool5000:Ool00`00Oomoo`3oOolkOol00`00Oomoo`0iOol00`00Oomoo`0@
Ool2000SOol005Aoo`800?moocaoo`03001oogoo03Uoo`03001oogoo011oo`8002=oo`00E7oo00<0
07ooOol0ogoo>goo00<007ooOol0>Goo00<007ooOol047oo0P008goo001DOol00`00Oomoo`3oOolk
Ool00`00Oomoo`0iOol00`00Oomoo`0@Ool2000SOol005Aoo`03001oogoo0?mooc]oo`03001oogoo
03Uoo`03001oogoo011oo`8002=oo`00E7oo00<007ooOol0ogoo>goo00<007ooOol0>Goo00<007oo
Ool047oo0P008goo001DOol00`00Oomoo`3oOolkOol00`00Oomoo`0iOol00`00Oomoo`0@Ool2000S
Ool005Aoo`03001oogoo0?mooc]oo`03001oogoo03Uoo`03001oogoo011oo`8002=oo`00E7oo00<0
07ooOol0ogoo>goo00<007ooOol0>Goo00<007ooOol047oo0P008goo001DOol2003oOollOol00`00
Oomoo`0iOol00`00Oomoo`0@Ool00`00Oomoo`0ROol005Aoo`03001oogoo0?mooc]oo`03001oogoo
03Uoo`03001oogoo011oo`03001oogoo029oo`00E7oo00<007ooOol0ogoo>goo00<007ooOol0>Goo
00<007ooOol047oo00<007ooOol08Woo001DOol00`00Oomoo`3oOolkOol00`00Oomoo`0iOol00`00
Oomoo`0@Ool00`00Oomoo`0ROol005Aoo`03001oogoo0?mooc]oo`03001oogoo03Uoo`03001oogoo
03Eoo`00E7oo00<007ooOol0ogoo>goo00<007ooOol0>Goo00<007ooOol0=Goo001DOol00`00Oomo
o`3oOolkOol00`00Oomoo`0iOol00`00Oomoo`0eOol005Aoo`03001oogoo0?mooc]oo`03001oogoo
03Uoo`03001oogoo03Eoo`00E7oo00<007ooOol0ogoo>goo00<007ooOol0>Goo00<007ooOol0=Goo
001DOol2003oOollOol00`00Oomoo`0iOol00`00Oomoo`0eOol005Aoo`03001oogoo0?moogMoo`03
001oogoo03Eoo`00E7oo00<007ooOol0ogooMgoo00<007ooOol0=Goo001DOol00`00Oomoo`3oOomg
Ool00`00Oomoo`0eOol005Aoo`03001oogoo0?moogMoo`03001oogoo03Eoo`00E7oo00<007ooOol0
ogooMgoo00<007ooOol0=Goo001DOol00`00Oomoo`3oOon_Ool005Aoo`03001oogoo0?moojmoo`00
E7oo0P00ogoo/7oo001DOol00`00Oomoo`3oOon_Ool005Aoo`03001oogoo0?moojmoo`00E7oo00<0
07ooOol0ogoo[goo003oOoooOol7Ool00?mooomoo`Moo`00ogooogoo1goo003oOoooOol7Ool00?mo
oomoo`Moo`00ogooogoo1goo003oOoooOol7Ool00?mooomoo`Moo`00ogooogoo1goo003oOoooOol7
Ool00?mooomoo`Moo`00ogooogoo1goo003oOoooOol7Ool00?mooomoo`Moo`00ogooogoo1goo003o
OoooOol7Ool00?mooomoo`Moo`00ogooogoo1goo003oOoooOol7Ool00?mooomoo`Moo`00ogooogoo
1goo003oOoooOol7Ool00?mooomoo`Moo`00ogooogoo1goo003oOoooOol7Ool00?mooomoo`Moo`00
ogooogoo1goo003oOoooOol7Ool00001\
\>"],
  ImageRangeCache->{{{0, 516.875}, {318.875, 0}} -> {-12514.8, -1066.39, 
  223.388, 0.00353057}}]
}, Open  ]],

Cell["\<\
If we are done, we can close the connection to the network data server. \
Otherwise, the connection stays open and will be reused if the same site is \
selected for the next request.\
\>", "Text"],

Cell[BoxData[
    \(NDSClose[]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Channel Information", "Subsection"],

Cell[BoxData[
    FormBox[GridBox[{
          {
            StyleBox[\(NDSGetChannel[]\),
              FontFamily->"Courier"], 
            \(obtain\ a\ list\ of\ channel\ names\)},
          {
            StyleBox[\(NDSGetChannelInfo[]\),
              FontFamily->"Courier"], 
            \(obtain\ a\ list\ of\ channel\ information\ \n
            records\)}
          }], TextForm]], "Definition"],

Cell[TextData[{
  StyleBox["NDSGetChannel",
    FontFamily->"Courier"],
  " returns a list of string values describing the channel names, whereas ",
  StyleBox["NDSGetChannelInfo",
    FontFamily->"Courier"],
  " returns a list of channel information records with the following \
elements: "
}], "Text"],

Cell[BoxData[
    FormBox[GridBox[{
          {
            StyleBox[\(ChannelName \[Rule] name\),
              FontFamily->"Courier"], \(Name\ of\ the\ channel\)},
          {
            StyleBox[\(DataRate \[Rule] rate\),
              FontFamily->"Courier"], \(Data\ rate\ of\ the\ channel\)},
          {
            StyleBox[\(InterferometerId \[Rule] ifo\),
              FontFamily->"Courier"], \(Interferometer\ identification\)},
          {
            StyleBox[\(DCU \[Rule] id\),
              FontFamily->"Courier"], 
            \(Data\ collection\ unit\ identification\)},
          {
            StyleBox[\(NDSDataType \[Rule] dtype\),
              FontFamily->"Courier"], \(Data\ type\)}
          }], TextForm]], "Definition"]
}, Closed]],

Cell[CellGroupData[{

Cell["Requesting Fast Data", "Subsection"],

Cell[BoxData[
    FormBox[GridBox[{
          {
            RowBox[{
              StyleBox["NDSGetData",
                FontFamily->"Courier"], 
              StyleBox["[",
                FontFamily->"Courier"], 
              StyleBox[
                RowBox[{
                  StyleBox["c",
                    FontFamily->"Courier",
                    FontSize->13], 
                  StyleBox[",",
                    FontFamily->"Courier",
                    FontSize->13], 
                  SubscriptBox[
                    StyleBox["t",
                      FontFamily->"Courier",
                      FontSize->13], "0"], 
                  StyleBox[",",
                    FontFamily->"Courier",
                    FontSize->13], 
                  StyleBox["\[CapitalDelta]t",
                    FontFamily->"Courier",
                    FontSize->13]}],
                FontSize->11], 
              StyleBox["]",
                FontFamily->"Courier"]}], 
            \(get\ raw\ data\ of\ a\ single\ channel\)},
          {
            RowBox[{
              StyleBox["NDSGetData",
                FontFamily->"Courier"], 
              StyleBox["[",
                FontFamily->"Courier"], 
              RowBox[{
                RowBox[{
                  StyleBox["{",
                    FontFamily->"Courier"], 
                  RowBox[{
                    SubscriptBox[
                      StyleBox["c",
                        FontFamily->"Courier"], "1"], 
                    StyleBox[",",
                      FontFamily->"Courier"], 
                    SubscriptBox[
                      StyleBox["c",
                        FontFamily->"Courier"], "2"], 
                    StyleBox[",",
                      FontFamily->"Courier"], "\[Ellipsis]"}], "}"}], ",", 
                StyleBox[\(t\_0\),
                  FontFamily->"Courier",
                  FontSize->11], 
                StyleBox[",",
                  FontFamily->"Courier",
                  FontSize->13], 
                StyleBox["\[CapitalDelta]t",
                  FontFamily->"Courier",
                  FontSize->13]}], 
              StyleBox["]",
                FontFamily->"Courier"]}], 
            \(get\ raw\ data\ of\ multiple\ channels\)}
          }], TextForm]], "Definition"],

Cell[TextData[{
  StyleBox["NDSGetData",
    FontFamily->"Courier"],
  " requests data from the network data server. Either a single channel name \
or a list of channel names can be specified as the first argument. The start \
time ",
  Cell[BoxData[
      FormBox[
        StyleBox[
          SubscriptBox[
            StyleBox["t",
              FontSlant->"Plain"], "0"],
          FontFamily->"Courier"], TraditionalForm]]],
  " can be specified either as a UTC date or in GPS time. The duration ",
  Cell[BoxData[
      FormBox[
        StyleBox["\[CapitalDelta]t",
          FontFamily->"Courier"], TraditionalForm]]],
  " is always specified in seconds."
}], "Text"],

Cell[BoxData[
    FormBox[GridBox[{
          {
            RowBox[{
              StyleBox["{",
                FontFamily->"Courier"], 
              StyleBox[\(year, month, day, hour, min, sec\),
                FontFamily->"Courier"], 
              StyleBox["}",
                FontFamily->"Courier"]}], \(UTC\ date\)},
          {
            StyleBox[\({GPS\ sec, \ nano\ sec}\),
              FontFamily->"Courier"], \(GPS\ time\)}
          }], TextForm]], "Definition"],

Cell[TextData[{
  "The following options are supprted by ",
  StyleBox["NDSGetData",
    FontFamily->"Courier"]
}], "Text"],

Cell[BoxData[
    FormBox[GridBox[{
          {
            StyleBox["option",
              CellFrame->{{0, 0}, {2, 0}},
              FontFamily->"Helvetica",
              FontSize->10,
              FontSlant->"Italic"], 
            StyleBox[\(default\ value\),
              CellFrame->{{0, 0}, {2, 0}},
              FontFamily->"Helvetica",
              FontSize->10,
              FontSlant->"Italic"], 
            StyleBox["description",
              CellFrame->{{0, 0}, {2, 0}},
              FontFamily->"Helvetica",
              FontSize->10,
              FontSlant->"Italic"]},
          {
            StyleBox["Site",
              FontFamily->"Courier"], 
            StyleBox["LHO",
              FontFamily->"Courier"], \(location\ of\ the\ data\ server\)},
          {
            StyleBox["DataRate",
              FontFamily->"Courier"], 
            StyleBox["256",
              FontFamily->"Courier"], \(specifies\ the\ data\ rate\)},
          {
            StyleBox["Heterodyne",
              FontFamily->"Courier"], 
            StyleBox["0",
              FontFamily->"Courier"], \(down\[Dash]conversion\ frequency\)},
          {
            StyleBox["DataFormat",
              FontFamily->"Courier"], 
            StyleBox["ColumnFormat",
              FontFamily->"Courier"], \(format\ of\ returned\ array\)}
          }], TextForm]], "Definition"],

Cell["\<\
The rate of retunred data channels has to identical for all requested \
channels. If the native data rate of a channel is larger than the requested \
rate, the data is automatically filtered and decimated. If the native data \
rate of a channel is smaller than the requested rate, the missing data points \
are filled in using linear interpolation. If the heterodyne frequency is \
non-zero, the data is down-converted first and the returned data array \
consists of complex numbers representing both the in-phase and quad-phase \
signal. The returned array always contains an extra channel describing the \
time of the data points. The format of the returned data is a two dimensional \
array either in column or row layout. In column format the data is organized \
as a list of channel data with the first column representing the time, the \
second column representing the first channel, and so on. The row format is \
the transposed of the column format with the data organized as a list of \
measurement points.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Requesting Trend Data", "Subsection"],

Cell[BoxData[
    FormBox[GridBox[{
          {
            RowBox[{
              StyleBox[
                RowBox[{
                  StyleBox["NDSGet",
                    FontFamily->"Courier"], 
                  StyleBox["Trend",
                    FontFamily->"Courier"]}]], 
              StyleBox["[",
                FontFamily->"Courier"], 
              RowBox[{
                StyleBox["c",
                  FontFamily->"Courier",
                  FontSize->13], 
                StyleBox[",",
                  FontFamily->"Courier",
                  FontSize->13], 
                StyleBox[
                  SubscriptBox[
                    StyleBox["t",
                      FontFamily->"Courier",
                      FontSize->13], "0"],
                  FontSize->11], 
                StyleBox[",",
                  FontFamily->"Courier",
                  FontSize->13], 
                StyleBox["\[CapitalDelta]t",
                  FontFamily->"Courier",
                  FontSize->13]}], 
              StyleBox["]",
                FontFamily->"Courier"]}], 
            \(get\ trend\ data\ of\ a\ single\ channel\)},
          {
            RowBox[{
              StyleBox["NDSGetTrend",
                FontFamily->"Courier"], 
              StyleBox["[",
                FontFamily->"Courier"], 
              RowBox[{
                RowBox[{
                  StyleBox["{",
                    FontFamily->"Courier"], 
                  RowBox[{
                    SubscriptBox[
                      StyleBox["c",
                        FontFamily->"Courier"], "1"], 
                    StyleBox[",",
                      FontFamily->"Courier"], 
                    SubscriptBox[
                      StyleBox["c",
                        FontFamily->"Courier"], "2"], 
                    StyleBox[",",
                      FontFamily->"Courier"], "\[Ellipsis]"}], "}"}], ",", 
                StyleBox[\(t\_0\),
                  FontFamily->"Courier",
                  FontSize->11], 
                StyleBox[",",
                  FontFamily->"Courier",
                  FontSize->13], 
                StyleBox["\[CapitalDelta]t",
                  FontFamily->"Courier",
                  FontSize->13]}], 
              StyleBox["]",
                FontFamily->"Courier"]}], 
            \(get\ trend\ data\ of\ multiple\ channels\)}
          }], TextForm]], "Definition"],

Cell[TextData[{
  StyleBox["NDSGetTrend",
    FontFamily->"Courier"],
  " requests trend data from the network data server. Either a single channel \
name or a list of channel names can be specified as the first argument. The \
start time ",
  Cell[BoxData[
      FormBox[
        StyleBox[
          SubscriptBox[
            StyleBox["t",
              FontSlant->"Plain"], "0"],
          FontFamily->"Courier"], TraditionalForm]]],
  " can be specified either as a UTC date or in GPS seconds. The duration ",
  Cell[BoxData[
      FormBox[
        StyleBox["\[CapitalDelta]t",
          FontFamily->"Courier"], TraditionalForm]]],
  " is always specified in seconds. The data is returned as a three \
dimensional array describing a list of channels. Each channel then descibes a \
list of data points. Each data point is either represented by"
}], "Text"],

Cell[BoxData[
    FormBox[GridBox[{
          {
            RowBox[{
              StyleBox["{",
                FontFamily->"Courier"], 
              RowBox[{
                StyleBox["t",
                  FontFamily->"Courier"], 
                StyleBox[",",
                  FontFamily->"Courier"], 
                StyleBox["n",
                  FontFamily->"Courier"], 
                StyleBox[",",
                  FontFamily->"Courier"], 
                RowBox[{
                  StyleBox[\(1\/n\),
                    FontFamily->"Courier"], 
                  RowBox[{
                    StyleBox[\(\[Sum]\+\(i = 1\)\%n\),
                      FontFamily->"Courier"], 
                    SubscriptBox[
                      StyleBox["x",
                        FontFamily->"Courier"], "i"]}]}], 
                StyleBox[",",
                  FontFamily->"Courier"], 
                StyleBox[\(\[Sigma]\_x\),
                  FontFamily->"Courier"], 
                StyleBox[",",
                  FontFamily->"Courier"], 
                StyleBox[
                  RowBox[{
                    StyleBox["min",
                      FontFamily->"Courier"], 
                    RowBox[{
                      StyleBox["{",
                        FontFamily->"Courier"], 
                      SubscriptBox[
                        StyleBox["x",
                          FontFamily->"Courier"], "i"], 
                      StyleBox["}",
                        FontFamily->"Courier"]}]}],
                  FontFamily->"Courier"], 
                StyleBox[",",
                  FontFamily->"Courier"], 
                StyleBox[
                  RowBox[{
                    StyleBox["max",
                      FontFamily->"Courier"], 
                    RowBox[{
                      StyleBox["{",
                        FontFamily->"Courier"], 
                      SubscriptBox[
                        StyleBox["x",
                          FontFamily->"Courier"], "i"], 
                      StyleBox["}",
                        FontFamily->"Courier"]}]}],
                  FontFamily->"Courier"]}], 
              StyleBox["}",
                FontFamily->"Courier"]}], \(Trend\ data\ point\)}
          }], TextForm]], "Definition"],

Cell[TextData[StyleBox["or by",
  FontFamily->"Courier"]], "Text"],

Cell[BoxData[
    FormBox[GridBox[{
          {
            StyleBox[
              RowBox[{"{", 
                RowBox[{"t", ",", 
                  RowBox[{\(1\/n\), 
                    RowBox[{\(\[Sum]\+\(i = 1\)\%n\), 
                      SubscriptBox[
                        StyleBox["x",
                          FontFamily->"Courier"], "i"]}]}]}], "}"}],
              FontFamily->"Courier"], \(Simple\ trend\ data\ point\)}
          }], TextForm]], "Definition"],

Cell[TextData[{
  "if the simple formatias chosen. The following options are supprted by ",
  StyleBox["NDSGetTrend",
    FontFamily->"Courier"]
}], "Text"],

Cell[BoxData[
    FormBox[GridBox[{
          {
            StyleBox["option",
              CellFrame->{{0, 0}, {2, 0}},
              FontFamily->"Helvetica",
              FontSize->10,
              FontSlant->"Italic"], 
            StyleBox[\(default\ value\),
              CellFrame->{{0, 0}, {2, 0}},
              FontFamily->"Helvetica",
              FontSize->10,
              FontSlant->"Italic"], 
            StyleBox["description",
              CellFrame->{{0, 0}, {2, 0}},
              FontFamily->"Helvetica",
              FontSize->10,
              FontSlant->"Italic"]},
          {
            StyleBox["Site",
              FontFamily->"Courier"], 
            StyleBox["LHO",
              FontFamily->"Courier"], \(location\ of\ the\ data\ server\)},
          {
            StyleBox["DataRate",
              FontFamily->"Courier"], 
            StyleBox["1",
              FontFamily->"Courier"], \(specifies\ the\ data\ rate\)},
          {
            StyleBox["SimpleFormat",
              FontFamily->"Courier New"], 
            StyleBox["True",
              
              FontFamily->
                "Courier New"], \(specifies\ the\ returned\ data\ format\)}
          }], TextForm]], "Definition"],

Cell[TextData[{
  "Trend data is stored by the network data server at both the one second and \
the one minute rate. Therefore the requested data rate must be smaller than \
one. For rates between, ",
  Cell[BoxData[
      \(TraditionalForm\`1\/60 < f \[LessEqual] 1\)]],
  ", the one seconnd trend information is used, whereas for rate smaller \
than, ",
  Cell[BoxData[
      \(TraditionalForm\`\(f < 1\/60, \)\)]],
  " the rate is rounded to the closest minute interval and the one minute \
trend information is used."
}], "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Section"],

Cell[CellGroupData[{

Cell["Example 1: Looking at seismic activities", "Subsection"],

Cell["\<\
We first load some graphics packages and setup a few common plot options.\
\>", "Text"],

Cell[BoxData[
    \(Needs["\<Graphics`Graphics`\>"]; \n
    Needs["\<Graphics`MultipleListPlot`\>"]; \n
    Needs["\<Statistics`DescriptiveStatistics`\>"]; 
    \n$TextStyle = {FontFamily \[Rule] "\<Helvetica\>", FontSize \[Rule] 13}; 
    \nplotopt = 
      Sequence[Frame \[Rule] True, GridLines \[Rule] Automatic, 
        PlotJoined \[Rule] True]; \)], "Input"],

Cell["\<\
To take a look at the microseismic peak we retrieve a stretch of data from a \
seismometer in the mid station covering an interval of 128 seconds at a \
sampling rate of 16 Hz (after decimation). This request will take a \
while...\
\>", "Text"],

Cell[BoxData[
    \(t0\  = \ ToDate[FromDate[Date[0]]\  - \ 3600]; \n
    data = NDSGetData[{"\<H0:PEM-MX_SEISZ\>"}, \ t0, 128, 
        DataRate \[Rule] 16, Heterodyne \[Rule] 0, 
        DataFormat \[Rule] RowFormat]; \)], "Input"],

Cell["First we take a look at the time trace", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ListPlot[data, plotopt]; \)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.00744411 2.70466 0.0142287 [
[.02381 -0.0125 -5.59375 -16.375 ]
[.02381 -0.0125 5.59375 0 ]
[.17269 -0.0125 -9.1875 -16.375 ]
[.17269 -0.0125 9.1875 0 ]
[.32157 -0.0125 -9.1875 -16.375 ]
[.32157 -0.0125 9.1875 0 ]
[.47046 -0.0125 -9.1875 -16.375 ]
[.47046 -0.0125 9.1875 0 ]
[.61934 -0.0125 -9.1875 -16.375 ]
[.61934 -0.0125 9.1875 0 ]
[.76822 -0.0125 -12.7813 -16.375 ]
[.76822 -0.0125 12.7813 0 ]
[.9171 -0.0125 -12.7813 -16.375 ]
[.9171 -0.0125 12.7813 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .0012 -34.5625 -8.1875 ]
[-0.0125 .0012 0 8.1875 ]
[-0.0125 .14349 -34.5625 -8.1875 ]
[-0.0125 .14349 0 8.1875 ]
[-0.0125 .28577 -34.5625 -8.1875 ]
[-0.0125 .28577 0 8.1875 ]
[-0.0125 .42806 -34.5625 -8.1875 ]
[-0.0125 .42806 0 8.1875 ]
[-0.0125 .57035 -34.5625 -8.1875 ]
[-0.0125 .57035 0 8.1875 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
.17269 0 m
.17269 .61803 L
s
.32157 0 m
.32157 .61803 L
s
.47046 0 m
.47046 .61803 L
s
.61934 0 m
.61934 .61803 L
s
.76822 0 m
.76822 .61803 L
s
.9171 0 m
.9171 .61803 L
s
0 .14349 m
1 .14349 L
s
0 .28577 m
1 .28577 L
s
0 .42806 m
1 .42806 L
s
0 .57035 m
1 .57035 L
s
0 g
.02381 0 m
.02381 .00625 L
s
gsave
.02381 -0.0125 -66.5938 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.17269 0 m
.17269 .00625 L
s
gsave
.17269 -0.0125 -70.1875 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(20) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.32157 0 m
.32157 .00625 L
s
gsave
.32157 -0.0125 -70.1875 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(40) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.47046 0 m
.47046 .00625 L
s
gsave
.47046 -0.0125 -70.1875 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(60) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.61934 0 m
.61934 .00625 L
s
gsave
.61934 -0.0125 -70.1875 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(80) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.76822 0 m
.76822 .00625 L
s
gsave
.76822 -0.0125 -73.7813 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(100) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.9171 0 m
.9171 .00625 L
s
gsave
.9171 -0.0125 -73.7813 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(120) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.125 Mabswid
.06103 0 m
.06103 .00375 L
s
.09825 0 m
.09825 .00375 L
s
.13547 0 m
.13547 .00375 L
s
.20991 0 m
.20991 .00375 L
s
.24713 0 m
.24713 .00375 L
s
.28435 0 m
.28435 .00375 L
s
.35879 0 m
.35879 .00375 L
s
.39602 0 m
.39602 .00375 L
s
.43324 0 m
.43324 .00375 L
s
.50768 0 m
.50768 .00375 L
s
.5449 0 m
.5449 .00375 L
s
.58212 0 m
.58212 .00375 L
s
.65656 0 m
.65656 .00375 L
s
.69378 0 m
.69378 .00375 L
s
.731 0 m
.731 .00375 L
s
.80544 0 m
.80544 .00375 L
s
.84266 0 m
.84266 .00375 L
s
.87988 0 m
.87988 .00375 L
s
.95432 0 m
.95432 .00375 L
s
.99154 0 m
.99154 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .0012 m
.00625 .0012 L
s
gsave
-0.0125 .0012 -95.5625 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
72.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(190) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1
%%+ font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1
%%+ Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Math1
%%+ font Helvetica
grestore
0 .14349 m
.00625 .14349 L
s
gsave
-0.0125 .14349 -95.5625 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
72.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(180) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1
%%+ font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1
%%+ Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Math1
%%+ font Helvetica
grestore
0 .28577 m
.00625 .28577 L
s
gsave
-0.0125 .28577 -95.5625 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
72.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(170) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1
%%+ font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1
%%+ Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Math1
%%+ font Helvetica
grestore
0 .42806 m
.00625 .42806 L
s
gsave
-0.0125 .42806 -95.5625 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
72.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(160) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1
%%+ font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1
%%+ Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Math1
%%+ font Helvetica
grestore
0 .57035 m
.00625 .57035 L
s
gsave
-0.0125 .57035 -95.5625 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
72.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(150) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1
%%+ font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1
%%+ Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Math1
%%+ font Helvetica
grestore
.125 Mabswid
0 .02966 m
.00375 .02966 L
s
0 .05811 m
.00375 .05811 L
s
0 .08657 m
.00375 .08657 L
s
0 .11503 m
.00375 .11503 L
s
0 .17194 m
.00375 .17194 L
s
0 .2004 m
.00375 .2004 L
s
0 .22886 m
.00375 .22886 L
s
0 .25732 m
.00375 .25732 L
s
0 .31423 m
.00375 .31423 L
s
0 .34269 m
.00375 .34269 L
s
0 .37115 m
.00375 .37115 L
s
0 .3996 m
.00375 .3996 L
s
0 .45652 m
.00375 .45652 L
s
0 .48498 m
.00375 .48498 L
s
0 .51343 m
.00375 .51343 L
s
0 .54189 m
.00375 .54189 L
s
0 .59881 m
.00375 .59881 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.02381 .61178 m
.02381 .61803 L
s
.17269 .61178 m
.17269 .61803 L
s
.32157 .61178 m
.32157 .61803 L
s
.47046 .61178 m
.47046 .61803 L
s
.61934 .61178 m
.61934 .61803 L
s
.76822 .61178 m
.76822 .61803 L
s
.9171 .61178 m
.9171 .61803 L
s
.125 Mabswid
.06103 .61428 m
.06103 .61803 L
s
.09825 .61428 m
.09825 .61803 L
s
.13547 .61428 m
.13547 .61803 L
s
.20991 .61428 m
.20991 .61803 L
s
.24713 .61428 m
.24713 .61803 L
s
.28435 .61428 m
.28435 .61803 L
s
.35879 .61428 m
.35879 .61803 L
s
.39602 .61428 m
.39602 .61803 L
s
.43324 .61428 m
.43324 .61803 L
s
.50768 .61428 m
.50768 .61803 L
s
.5449 .61428 m
.5449 .61803 L
s
.58212 .61428 m
.58212 .61803 L
s
.65656 .61428 m
.65656 .61803 L
s
.69378 .61428 m
.69378 .61803 L
s
.731 .61428 m
.731 .61803 L
s
.80544 .61428 m
.80544 .61803 L
s
.84266 .61428 m
.84266 .61803 L
s
.87988 .61428 m
.87988 .61803 L
s
.95432 .61428 m
.95432 .61803 L
s
.99154 .61428 m
.99154 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .0012 m
1 .0012 L
s
.99375 .14349 m
1 .14349 L
s
.99375 .28577 m
1 .28577 L
s
.99375 .42806 m
1 .42806 L
s
.99375 .57035 m
1 .57035 L
s
.125 Mabswid
.99625 .02966 m
1 .02966 L
s
.99625 .05811 m
1 .05811 L
s
.99625 .08657 m
1 .08657 L
s
.99625 .11503 m
1 .11503 L
s
.99625 .17194 m
1 .17194 L
s
.99625 .2004 m
1 .2004 L
s
.99625 .22886 m
1 .22886 L
s
.99625 .25732 m
1 .25732 L
s
.99625 .31423 m
1 .31423 L
s
.99625 .34269 m
1 .34269 L
s
.99625 .37115 m
1 .37115 L
s
.99625 .3996 m
1 .3996 L
s
.99625 .45652 m
1 .45652 L
s
.99625 .48498 m
1 .48498 L
s
.99625 .51343 m
1 .51343 L
s
.99625 .54189 m
1 .54189 L
s
.99625 .59881 m
1 .59881 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .18074 m
.02427 .22119 L
.02474 .19913 L
.02521 .2179 L
.02567 .20159 L
.02614 .20859 L
.0266 .1796 L
.02707 .23251 L
.02753 .21451 L
.028 .19379 L
.02846 .21358 L
.02893 .21287 L
.02939 .20245 L
.02986 .22497 L
.03032 .19537 L
.03079 .17953 L
.03125 .24671 L
.03172 .17467 L
.03218 .20574 L
.03265 .22119 L
.03311 .19406 L
.03358 .20227 L
.03405 .19727 L
.03451 .21326 L
.03498 .20055 L
.03544 .19423 L
.03591 .20703 L
.03637 .21034 L
.03684 .20927 L
.0373 .22945 L
.03777 .23795 L
.03823 .20504 L
.0387 .22694 L
.03916 .19853 L
.03963 .24512 L
.04009 .21324 L
.04056 .22695 L
.04102 .2407 L
.04149 .26751 L
.04195 .20069 L
.04242 .32801 L
.04289 .18997 L
.04335 .30426 L
.04382 .24868 L
.04428 .25916 L
.04475 .25588 L
.04521 .28827 L
.04568 .26504 L
.04614 .2862 L
.04661 .30577 L
Mistroke
.04707 .27377 L
.04754 .31122 L
.048 .2714 L
.04847 .31665 L
.04893 .24478 L
.0494 .30371 L
.04986 .31208 L
.05033 .30821 L
.05079 .30304 L
.05126 .32574 L
.05172 .31853 L
.05219 .31714 L
.05266 .32692 L
.05312 .31114 L
.05359 .32995 L
.05405 .33829 L
.05452 .30267 L
.05498 .36835 L
.05545 .3237 L
.05591 .34218 L
.05638 .31761 L
.05684 .3837 L
.05731 .29166 L
.05777 .39464 L
.05824 .27056 L
.0587 .36463 L
.05917 .31516 L
.05963 .33362 L
.0601 .36033 L
.06056 .30092 L
.06103 .35815 L
.0615 .32303 L
.06196 .31125 L
.06243 .34947 L
.06289 .29517 L
.06336 .36094 L
.06382 .27816 L
.06429 .33682 L
.06475 .31819 L
.06522 .2949 L
.06568 .32763 L
.06615 .32257 L
.06661 .27964 L
.06708 .33002 L
.06754 .29173 L
.06801 .31729 L
.06847 .29882 L
.06894 .29105 L
.0694 .30007 L
.06987 .28781 L
Mistroke
.07034 .29416 L
.0708 .30079 L
.07127 .28738 L
.07173 .28566 L
.0722 .302 L
.07266 .27969 L
.07313 .24391 L
.07359 .31544 L
.07406 .24875 L
.07452 .29149 L
.07499 .27834 L
.07545 .26993 L
.07592 .26914 L
.07638 .2953 L
.07685 .24347 L
.07731 .28443 L
.07778 .26892 L
.07824 .26222 L
.07871 .27724 L
.07918 .27385 L
.07964 .27827 L
.08011 .26413 L
.08057 .3059 L
.08104 .24091 L
.0815 .27246 L
.08197 .24158 L
.08243 .26588 L
.0829 .26561 L
.08336 .26813 L
.08383 .26966 L
.08429 .25929 L
.08476 .25745 L
.08522 .26705 L
.08569 .26898 L
.08615 .23015 L
.08662 .26932 L
.08708 .25128 L
.08755 .25893 L
.08801 .24821 L
.08848 .24064 L
.08895 .28787 L
.08941 .21332 L
.08988 .26052 L
.09034 .21435 L
.09081 .27591 L
.09127 .20005 L
.09174 .25433 L
.0922 .20849 L
.09267 .23739 L
.09313 .24339 L
Mistroke
.0936 .18152 L
.09406 .24313 L
.09453 .23148 L
.09499 .18568 L
.09546 .21255 L
.09592 .24346 L
.09639 .20737 L
.09685 .20257 L
.09732 .21222 L
.09779 .22914 L
.09825 .23002 L
.09872 .20359 L
.09918 .25006 L
.09965 .1835 L
.10011 .22439 L
.10058 .26024 L
.10104 .16768 L
.10151 .30378 L
.10197 .19758 L
.10244 .24605 L
.1029 .23113 L
.10337 .25845 L
.10383 .23113 L
.1043 .26742 L
.10476 .25376 L
.10523 .24748 L
.10569 .24568 L
.10616 .27244 L
.10663 .24674 L
.10709 .30247 L
.10756 .2512 L
.10802 .2645 L
.10849 .30342 L
.10895 .25404 L
.10942 .28519 L
.10988 .2881 L
.11035 .25646 L
.11081 .31425 L
.11128 .2673 L
.11174 .28414 L
.11221 .35796 L
.11267 .26027 L
.11314 .32804 L
.1136 .32205 L
.11407 .2946 L
.11453 .34359 L
.115 .30672 L
.11547 .33971 L
.11593 .35607 L
.1164 .31047 L
Mistroke
.11686 .35498 L
.11733 .35097 L
.11779 .31056 L
.11826 .36929 L
.11872 .338 L
.11919 .34451 L
.11965 .36421 L
.12012 .33672 L
.12058 .34316 L
.12105 .33987 L
.12151 .32808 L
.12198 .3572 L
.12244 .31304 L
.12291 .33109 L
.12337 .37046 L
.12384 .30443 L
.12431 .30759 L
.12477 .34014 L
.12524 .27959 L
.1257 .31111 L
.12617 .32034 L
.12663 .28564 L
.1271 .31186 L
.12756 .28026 L
.12803 .27769 L
.12849 .30073 L
.12896 .27573 L
.12942 .25368 L
.12989 .3058 L
.13035 .23579 L
.13082 .25652 L
.13128 .26243 L
.13175 .22492 L
.13221 .25994 L
.13268 .21083 L
.13314 .21963 L
.13361 .24448 L
.13408 .18943 L
.13454 .22111 L
.13501 .18878 L
.13547 .20837 L
.13594 .18452 L
.1364 .18256 L
.13687 .17587 L
.13733 .17256 L
.1378 .17896 L
.13826 .16604 L
.13873 .1551 L
.13919 .17988 L
.13966 .12147 L
Mistroke
.14012 .15086 L
.14059 .16624 L
.14105 .13847 L
.14152 .14542 L
.14198 .13901 L
.14245 .17271 L
.14292 .12554 L
.14338 .12061 L
.14385 .17646 L
.14431 .12453 L
.14478 .17986 L
.14524 .15831 L
.14571 .17334 L
.14617 .19062 L
.14664 .14958 L
.1471 .20784 L
.14757 .14713 L
.14803 .22349 L
.1485 .21054 L
.14896 .19262 L
.14943 .23969 L
.14989 .22607 L
.15036 .23947 L
.15082 .23633 L
.15129 .25197 L
.15176 .25864 L
.15222 .25964 L
.15269 .2425 L
.15315 .30133 L
.15362 .26023 L
.15408 .30716 L
.15455 .27674 L
.15501 .30651 L
.15548 .27343 L
.15594 .29297 L
.15641 .28232 L
.15687 .33134 L
.15734 .28632 L
.1578 .3133 L
.15827 .31756 L
.15873 .30924 L
.1592 .30293 L
.15966 .33212 L
.16013 .29523 L
.1606 .29748 L
.16106 .3472 L
.16153 .28346 L
.16199 .3392 L
.16246 .32972 L
.16292 .31464 L
Mistroke
.16339 .3112 L
.16385 .33829 L
.16432 .33835 L
.16478 .28904 L
.16525 .3319 L
.16571 .32387 L
.16618 .2898 L
.16664 .33672 L
.16711 .32472 L
.16757 .30696 L
.16804 .33158 L
.1685 .31326 L
.16897 .31946 L
.16943 .32972 L
.1699 .30358 L
.17037 .31623 L
.17083 .35022 L
.1713 .32605 L
.17176 .31364 L
.17223 .32853 L
.17269 .3354 L
.17316 .33671 L
.17362 .30866 L
.17409 .31451 L
.17455 .36066 L
.17502 .30906 L
.17548 .31828 L
.17595 .34115 L
.17641 .3428 L
.17688 .30992 L
.17734 .35955 L
.17781 .3084 L
.17827 .30883 L
.17874 .34905 L
.17921 .28848 L
.17967 .35622 L
.18014 .33309 L
.1806 .32467 L
.18107 .30738 L
.18153 .32799 L
.182 .30782 L
.18246 .31059 L
.18293 .30879 L
.18339 .34471 L
.18386 .3104 L
.18432 .32324 L
.18479 .33374 L
.18525 .31681 L
.18572 .3125 L
.18618 .30971 L
Mistroke
.18665 .33095 L
.18711 .29639 L
.18758 .32657 L
.18805 .31724 L
.18851 .30822 L
.18898 .29886 L
.18944 .33573 L
.18991 .29145 L
.19037 .32374 L
.19084 .27936 L
.1913 .30013 L
.19177 .29345 L
.19223 .28396 L
.1927 .30361 L
.19316 .24473 L
.19363 .29154 L
.19409 .29251 L
.19456 .24805 L
.19502 .27189 L
.19549 .27804 L
.19595 .2485 L
.19642 .26222 L
.19689 .23388 L
.19735 .26936 L
.19782 .25664 L
.19828 .23462 L
.19875 .24285 L
.19921 .25773 L
.19968 .23398 L
.20014 .22785 L
.20061 .23881 L
.20107 .24599 L
.20154 .22228 L
.202 .22269 L
.20247 .24372 L
.20293 .20965 L
.2034 .26488 L
.20386 .19033 L
.20433 .23923 L
.20479 .23184 L
.20526 .19679 L
.20572 .24692 L
.20619 .1896 L
.20666 .25469 L
.20712 .1912 L
.20759 .22558 L
.20805 .2212 L
.20852 .2079 L
.20898 .2308 L
.20945 .20958 L
Mistroke
.20991 .22547 L
.21038 .22737 L
.21084 .20113 L
.21131 .20987 L
.21177 .24767 L
.21224 .20952 L
.2127 .21189 L
.21317 .2489 L
.21363 .21927 L
.2141 .22985 L
.21456 .22846 L
.21503 .24869 L
.2155 .23048 L
.21596 .23766 L
.21643 .26008 L
.21689 .25874 L
.21736 .22348 L
.21782 .28222 L
.21829 .27022 L
.21875 .24267 L
.21922 .28748 L
.21968 .25797 L
.22015 .26246 L
.22061 .29063 L
.22108 .29478 L
.22154 .26528 L
.22201 .31105 L
.22247 .26101 L
.22294 .28874 L
.2234 .32148 L
.22387 .27434 L
.22434 .31718 L
.2248 .34277 L
.22527 .28092 L
.22573 .36472 L
.2262 .28901 L
.22666 .31383 L
.22713 .37451 L
.22759 .31518 L
.22806 .3242 L
.22852 .37475 L
.22899 .29842 L
.22945 .34408 L
.22992 .35473 L
.23038 .31368 L
.23085 .36011 L
.23131 .33061 L
.23178 .33762 L
.23224 .3368 L
.23271 .35411 L
Mistroke
.23318 .31563 L
.23364 .35725 L
.23411 .28938 L
.23457 .36279 L
.23504 .3238 L
.2355 .31922 L
.23597 .308 L
.23643 .35548 L
.2369 .27082 L
.23736 .3336 L
.23783 .32068 L
.23829 .27928 L
.23876 .3231 L
.23922 .28303 L
.23969 .29749 L
.24015 .29096 L
.24062 .29338 L
.24108 .24467 L
.24155 .32419 L
.24202 .27656 L
.24248 .29082 L
.24295 .23659 L
.24341 .29713 L
.24388 .25049 L
.24434 .24956 L
.24481 .2828 L
.24527 .25627 L
.24574 .2681 L
.2462 .2475 L
.24667 .27308 L
.24713 .2165 L
.2476 .26386 L
.24806 .24715 L
.24853 .22301 L
.24899 .2554 L
.24946 .24162 L
.24992 .19968 L
.25039 .24571 L
.25085 .23807 L
.25132 .21242 L
.25179 .2483 L
.25225 .18853 L
.25272 .24252 L
.25318 .20306 L
.25365 .23883 L
.25411 .20268 L
.25458 .23864 L
.25504 .2073 L
.25551 .20151 L
.25597 .20734 L
Mistroke
.25644 .21923 L
.2569 .22202 L
.25737 .21823 L
.25783 .20591 L
.2583 .21267 L
.25876 .24493 L
.25923 .17981 L
.25969 .23474 L
.26016 .21499 L
.26063 .23333 L
.26109 .17502 L
.26156 .22905 L
.26202 .21289 L
.26249 .21524 L
.26295 .23246 L
.26342 .22143 L
.26388 .21113 L
.26435 .21556 L
.26481 .23303 L
.26528 .22575 L
.26574 .23088 L
.26621 .24255 L
.26667 .22797 L
.26714 .24442 L
.2676 .2411 L
.26807 .27398 L
.26853 .25244 L
.269 .24607 L
.26947 .29588 L
.26993 .26487 L
.2704 .26455 L
.27086 .31079 L
.27133 .28306 L
.27179 .32089 L
.27226 .31816 L
.27272 .30947 L
.27319 .33228 L
.27365 .35041 L
.27412 .31823 L
.27458 .3617 L
.27505 .3726 L
.27551 .3227 L
.27598 .38819 L
.27644 .35997 L
.27691 .3695 L
.27737 .39681 L
.27784 .41327 L
.27831 .37577 L
.27877 .39696 L
.27924 .42272 L
Mistroke
.2797 .35729 L
.28017 .40852 L
.28063 .38525 L
.2811 .43179 L
.28156 .40563 L
.28203 .37921 L
.28249 .4231 L
.28296 .42946 L
.28342 .38281 L
.28389 .40222 L
.28435 .3819 L
.28482 .3869 L
.28528 .39787 L
.28575 .37921 L
.28621 .3816 L
.28668 .39421 L
.28714 .37238 L
.28761 .34079 L
.28808 .37238 L
.28854 .33907 L
.28901 .35237 L
.28947 .30292 L
.28994 .3662 L
.2904 .3063 L
.29087 .29646 L
.29133 .29997 L
.2918 .32499 L
.29226 .27325 L
.29273 .28621 L
.29319 .26794 L
.29366 .25743 L
.29412 .28404 L
.29459 .23168 L
.29505 .23417 L
.29552 .23937 L
.29598 .20186 L
.29645 .23175 L
.29692 .17411 L
.29738 .22143 L
.29785 .19736 L
.29831 .20679 L
.29878 .16009 L
.29924 .17575 L
.29971 .17368 L
.30017 .13562 L
.30064 .155 L
.3011 .163 L
.30157 .15596 L
.30203 .15194 L
.3025 .17072 L
Mistroke
.30296 .12168 L
.30343 .16143 L
.30389 .14314 L
.30436 .12714 L
.30482 .15834 L
.30529 .13856 L
.30576 .14602 L
.30622 .1312 L
.30669 .1636 L
.30715 .14163 L
.30762 .17578 L
.30808 .15111 L
.30855 .14181 L
.30901 .15176 L
.30948 .16451 L
.30994 .1447 L
.31041 .16511 L
.31087 .19372 L
.31134 .15945 L
.3118 .17298 L
.31227 .20737 L
.31273 .18049 L
.3132 .19085 L
.31366 .20089 L
.31413 .20201 L
.3146 .22644 L
.31506 .21407 L
.31553 .2457 L
.31599 .24813 L
.31646 .25839 L
.31692 .24848 L
.31739 .24441 L
.31785 .28406 L
.31832 .27512 L
.31878 .28122 L
.31925 .29516 L
.31971 .29565 L
.32018 .33974 L
.32064 .31104 L
.32111 .31495 L
.32157 .36409 L
.32204 .29223 L
.3225 .36343 L
.32297 .32376 L
.32343 .35354 L
.3239 .38751 L
.32437 .35976 L
.32483 .37519 L
.3253 .39179 L
.32576 .35292 L
Mistroke
.32623 .39049 L
.32669 .37693 L
.32716 .38166 L
.32762 .39589 L
.32809 .39169 L
.32855 .40534 L
.32902 .39803 L
.32948 .39442 L
.32995 .39186 L
.33041 .36254 L
.33088 .41216 L
.33134 .34386 L
.33181 .38323 L
.33227 .38842 L
.33274 .33064 L
.33321 .41364 L
.33367 .32616 L
.33414 .36627 L
.3346 .33763 L
.33507 .35188 L
.33553 .32475 L
.336 .29832 L
.33646 .33384 L
.33693 .32846 L
.33739 .28832 L
.33786 .30505 L
.33832 .29839 L
.33879 .25888 L
.33925 .30178 L
.33972 .25873 L
.34018 .25879 L
.34065 .25909 L
.34111 .25339 L
.34158 .23505 L
.34205 .25562 L
.34251 .24239 L
.34298 .22938 L
.34344 .221 L
.34391 .21527 L
.34437 .21035 L
.34484 .23053 L
.3453 .20289 L
.34577 .22223 L
.34623 .18293 L
.3467 .19712 L
.34716 .18065 L
.34763 .18349 L
.34809 .17426 L
.34856 .17945 L
.34902 .15699 L
Mistroke
.34949 .16385 L
.34995 .15517 L
.35042 .17674 L
.35089 .12176 L
.35135 .16049 L
.35182 .13948 L
.35228 .13812 L
.35275 .13406 L
.35321 .11452 L
.35368 .13332 L
.35414 .12602 L
.35461 .10633 L
.35507 .11639 L
.35554 .13505 L
.356 .08594 L
.35647 .11646 L
.35693 .11718 L
.3574 .11505 L
.35786 .12518 L
.35833 .10212 L
.35879 .14358 L
.35926 .11519 L
.35973 .11038 L
.36019 .11473 L
.36066 .14383 L
.36112 .12703 L
.36159 .13799 L
.36205 .1537 L
.36252 .12192 L
.36298 .1521 L
.36345 .15745 L
.36391 .12642 L
.36438 .18142 L
.36484 .14477 L
.36531 .16138 L
.36577 .18179 L
.36624 .17113 L
.3667 .17933 L
.36717 .19009 L
.36763 .19876 L
.3681 .18866 L
.36856 .20445 L
.36903 .19417 L
.3695 .2141 L
.36996 .20916 L
.37043 .2165 L
.37089 .197 L
.37136 .23791 L
.37182 .20809 L
.37229 .23993 L
Mistroke
.37275 .23131 L
.37322 .2344 L
.37368 .21366 L
.37415 .24352 L
.37461 .25001 L
.37508 .23674 L
.37554 .23131 L
.37601 .2633 L
.37647 .25468 L
.37694 .22894 L
.3774 .25552 L
.37787 .23651 L
.37834 .25672 L
.3788 .24011 L
.37927 .23108 L
.37973 .25496 L
.3802 .26063 L
.38066 .24024 L
.38113 .21308 L
.38159 .28072 L
.38206 .22907 L
.38252 .24052 L
.38299 .26861 L
.38345 .24826 L
.38392 .2526 L
.38438 .24766 L
.38485 .26443 L
.38531 .2476 L
.38578 .26769 L
.38624 .27369 L
.38671 .25566 L
.38718 .26932 L
.38764 .28127 L
.38811 .27606 L
.38857 .27493 L
.38904 .2813 L
.3895 .30661 L
.38997 .28556 L
.39043 .30525 L
.3909 .287 L
.39136 .30974 L
.39183 .30268 L
.39229 .32045 L
.39276 .28347 L
.39322 .33793 L
.39369 .31111 L
.39415 .29969 L
.39462 .34226 L
.39508 .28799 L
.39555 .33721 L
Mistroke
.39602 .30051 L
.39648 .3039 L
.39695 .33869 L
.39741 .28572 L
.39788 .31167 L
.39834 .2799 L
.39881 .2975 L
.39927 .28048 L
.39974 .26302 L
.4002 .29633 L
.40067 .27077 L
.40113 .24911 L
.4016 .25693 L
.40206 .26027 L
.40253 .22517 L
.40299 .24365 L
.40346 .1962 L
.40392 .21633 L
.40439 .21203 L
.40485 .19017 L
.40532 .19807 L
.40579 .18312 L
.40625 .16144 L
.40672 .15798 L
.40718 .15274 L
.40765 .13476 L
.40811 .12222 L
.40858 .14093 L
.40904 .12407 L
.40951 .11288 L
.40997 .08681 L
.41044 .12334 L
.4109 .10907 L
.41137 .0598 L
.41183 .11244 L
.4123 .06427 L
.41276 .08908 L
.41323 .08853 L
.41369 .06193 L
.41416 .08415 L
.41463 .07106 L
.41509 .07368 L
.41556 .05334 L
.41602 .06428 L
.41649 .09777 L
.41695 .01472 L
.41742 .1049 L
.41788 .03648 L
.41835 .07385 L
.41881 .06094 L
Mistroke
.41928 .07243 L
.41974 .07206 L
.42021 .05617 L
.42067 .06536 L
.42114 .08457 L
.4216 .04099 L
.42207 .10976 L
.42253 .10393 L
.423 .05565 L
.42347 .13555 L
.42393 .09288 L
.4244 .1053 L
.42486 .16437 L
.42533 .1235 L
.42579 .1582 L
.42626 .15988 L
.42672 .19558 L
.42719 .1851 L
.42765 .21959 L
.42812 .22939 L
.42858 .22562 L
.42905 .27834 L
.42951 .25891 L
.42998 .28071 L
.43044 .27707 L
.43091 .3369 L
.43137 .32367 L
.43184 .34815 L
.43231 .37217 L
.43277 .37136 L
.43324 .3888 L
.4337 .39891 L
.43417 .39878 L
.43463 .39573 L
.4351 .44989 L
.43556 .44209 L
.43603 .44647 L
.43649 .48118 L
.43696 .43652 L
.43742 .47568 L
.43789 .46535 L
.43835 .46254 L
.43882 .48105 L
.43928 .47813 L
.43975 .48355 L
.44021 .49856 L
.44068 .48318 L
.44114 .45588 L
.44161 .50884 L
.44208 .48025 L
Mistroke
.44254 .47698 L
.44301 .46063 L
.44347 .47008 L
.44394 .46652 L
.4444 .45577 L
.44487 .45849 L
.44533 .41165 L
.4458 .44541 L
.44626 .4144 L
.44673 .39707 L
.44719 .40841 L
.44766 .37992 L
.44812 .40943 L
.44859 .34692 L
.44905 .35759 L
.44952 .34515 L
.44998 .33876 L
.45045 .31306 L
.45092 .34507 L
.45138 .27211 L
.45185 .31122 L
.45231 .31347 L
.45278 .23562 L
.45324 .26191 L
.45371 .25072 L
.45417 .22096 L
.45464 .22662 L
.4551 .19762 L
.45557 .2385 L
.45603 .18646 L
.4565 .16436 L
.45696 .18496 L
.45743 .15011 L
.45789 .16551 L
.45836 .12871 L
.45882 .11996 L
.45929 .14383 L
.45976 .11242 L
.46022 .08067 L
.46069 .13819 L
.46115 .09704 L
.46162 .09057 L
.46208 .09148 L
.46255 .06648 L
.46301 .09078 L
.46348 .05762 L
.46394 .06292 L
.46441 .06901 L
.46487 .08109 L
.46534 .07534 L
Mistroke
.4658 .07334 L
.46627 .08145 L
.46673 .04252 L
.4672 .06722 L
.46766 .04236 L
.46813 .06651 L
.4686 .045 L
.46906 .0748 L
.46953 .08434 L
.46999 .083 L
.47046 .07894 L
.47092 .10745 L
.47139 .07162 L
.47185 .09643 L
.47232 .10977 L
.47278 .0995 L
.47325 .1373 L
.47371 .14619 L
.47418 .14355 L
.47464 .16153 L
.47511 .18653 L
.47557 .14758 L
.47604 .21002 L
.4765 .23468 L
.47697 .21721 L
.47744 .24838 L
.4779 .25878 L
.47837 .29119 L
.47883 .28148 L
.4793 .32616 L
.47976 .28241 L
.48023 .35397 L
.48069 .34712 L
.48116 .33851 L
.48162 .39299 L
.48209 .36932 L
.48255 .40192 L
.48302 .396 L
.48348 .38862 L
.48395 .40952 L
.48441 .4212 L
.48488 .40114 L
.48534 .43507 L
.48581 .44564 L
.48627 .44135 L
.48674 .42008 L
.48721 .4482 L
.48767 .44714 L
.48814 .41313 L
.4886 .45094 L
Mistroke
.48907 .43823 L
.48953 .41197 L
.49 .49083 L
.49046 .41031 L
.49093 .4296 L
.49139 .43891 L
.49186 .38206 L
.49232 .42973 L
.49279 .41107 L
.49325 .41868 L
.49372 .39706 L
.49418 .38166 L
.49465 .44774 L
.49511 .33973 L
.49558 .41815 L
.49605 .37225 L
.49651 .34737 L
.49698 .38863 L
.49744 .34518 L
.49791 .36157 L
.49837 .33722 L
.49884 .33937 L
.4993 .36993 L
.49977 .28572 L
.50023 .32891 L
.5007 .30575 L
.50116 .29605 L
.50163 .30647 L
.50209 .27818 L
.50256 .27797 L
.50302 .27378 L
.50349 .24891 L
.50395 .27784 L
.50442 .23602 L
.50489 .24715 L
.50535 .25133 L
.50582 .18914 L
.50628 .25621 L
.50675 .18321 L
.50721 .21076 L
.50768 .23992 L
.50814 .21177 L
.50861 .21067 L
.50907 .18576 L
.50954 .19991 L
.51 .16385 L
.51047 .2047 L
.51093 .17773 L
.5114 .19205 L
.51186 .19835 L
Mistroke
.51233 .1837 L
.51279 .16902 L
.51326 .1919 L
.51373 .14262 L
.51419 .17646 L
.51466 .16803 L
.51512 .16378 L
.51559 .17539 L
.51605 .17022 L
.51652 .16417 L
.51698 .17319 L
.51745 .16163 L
.51791 .1637 L
.51838 .13383 L
.51884 .18844 L
.51931 .1375 L
.51977 .17126 L
.52024 .19284 L
.5207 .13353 L
.52117 .1858 L
.52163 .1638 L
.5221 .14437 L
.52256 .17986 L
.52303 .1681 L
.5235 .15111 L
.52396 .21469 L
.52443 .1655 L
.52489 .17652 L
.52536 .16143 L
.52582 .19643 L
.52629 .16554 L
.52675 .21094 L
.52722 .18661 L
.52768 .21584 L
.52815 .1892 L
.52861 .27046 L
.52908 .18971 L
.52954 .24871 L
.53001 .23973 L
.53047 .24365 L
.53094 .2752 L
.5314 .24184 L
.53187 .28979 L
.53234 .26126 L
.5328 .29085 L
.53327 .30009 L
.53373 .28964 L
.5342 .30141 L
.53466 .30247 L
.53513 .32499 L
Mistroke
.53559 .28338 L
.53606 .32303 L
.53652 .33174 L
.53699 .29789 L
.53745 .33364 L
.53792 .30383 L
.53838 .31419 L
.53885 .34982 L
.53931 .30508 L
.53978 .31761 L
.54024 .36209 L
.54071 .30198 L
.54118 .32875 L
.54164 .3154 L
.54211 .33718 L
.54257 .34827 L
.54304 .30208 L
.5435 .3588 L
.54397 .32639 L
.54443 .29902 L
.5449 .36951 L
.54536 .30376 L
.54583 .34636 L
.54629 .31333 L
.54676 .30594 L
.54722 .36773 L
.54769 .29071 L
.54815 .31979 L
.54862 .33333 L
.54908 .29684 L
.54955 .32007 L
.55002 .30201 L
.55048 .33077 L
.55095 .30869 L
.55141 .32377 L
.55188 .29261 L
.55234 .28957 L
.55281 .31463 L
.55327 .30541 L
.55374 .27995 L
.5542 .30934 L
.55467 .30277 L
.55513 .28218 L
.5556 .28745 L
.55606 .32718 L
.55653 .27492 L
.55699 .27203 L
.55746 .29538 L
.55792 .26798 L
.55839 .32353 L
Mistroke
.55886 .26813 L
.55932 .28288 L
.55979 .3037 L
.56025 .27324 L
.56072 .29683 L
.56118 .27773 L
.56165 .26278 L
.56211 .294 L
.56258 .27519 L
.56304 .28423 L
.56351 .29693 L
.56397 .23152 L
.56444 .28599 L
.5649 .2703 L
.56537 .2293 L
.56583 .28742 L
.5663 .23561 L
.56676 .26719 L
.56723 .25261 L
.56769 .25418 L
.56816 .23204 L
.56863 .22814 L
.56909 .23463 L
.56956 .2392 L
.57002 .22427 L
.57049 .23136 L
.57095 .24594 L
.57142 .1933 L
.57188 .21096 L
.57235 .24523 L
.57281 .1841 L
.57328 .19271 L
.57374 .21148 L
.57421 .20495 L
.57467 .17602 L
.57514 .20267 L
.5756 .16672 L
.57607 .21194 L
.57653 .18006 L
.577 .16581 L
.57747 .21114 L
.57793 .17487 L
.5784 .20753 L
.57886 .18229 L
.57933 .19328 L
.57979 .19358 L
.58026 .18096 L
.58072 .20912 L
.58119 .25068 L
.58165 .18301 L
Mistroke
.58212 .24026 L
.58258 .21176 L
.58305 .253 L
.58351 .20995 L
.58398 .24177 L
.58444 .25366 L
.58491 .23687 L
.58537 .24308 L
.58584 .25834 L
.58631 .27198 L
.58677 .23885 L
.58724 .26105 L
.5877 .28194 L
.58817 .24603 L
.58863 .26598 L
.5891 .27593 L
.58956 .25413 L
.59003 .30414 L
.59049 .26931 L
.59096 .26622 L
.59142 .29477 L
.59189 .24237 L
.59235 .30273 L
.59282 .25016 L
.59328 .2626 L
.59375 .30868 L
.59421 .26934 L
.59468 .29287 L
.59515 .29294 L
.59561 .26898 L
.59608 .3023 L
.59654 .25399 L
.59701 .28196 L
.59747 .27039 L
.59794 .28681 L
.5984 .29264 L
.59887 .2644 L
.59933 .30392 L
.5998 .27025 L
.60026 .27802 L
.60073 .27923 L
.60119 .26982 L
.60166 .27214 L
.60212 .2593 L
.60259 .30151 L
.60305 .27475 L
.60352 .27394 L
.60398 .31182 L
.60445 .27308 L
.60492 .27033 L
Mistroke
.60538 .2657 L
.60585 .28886 L
.60631 .2725 L
.60678 .26235 L
.60724 .31772 L
.60771 .25178 L
.60817 .32647 L
.60864 .27132 L
.6091 .28034 L
.60957 .31273 L
.61003 .26052 L
.6105 .29957 L
.61096 .30215 L
.61143 .2794 L
.61189 .31247 L
.61236 .33483 L
.61282 .29107 L
.61329 .32392 L
.61376 .30396 L
.61422 .28417 L
.61469 .3497 L
.61515 .28364 L
.61562 .34031 L
.61608 .31496 L
.61655 .33668 L
.61701 .30226 L
.61748 .34013 L
.61794 .28555 L
.61841 .30793 L
.61887 .33008 L
.61934 .29803 L
.6198 .33635 L
.62027 .28364 L
.62073 .33029 L
.6212 .26957 L
.62166 .28082 L
.62213 .32339 L
.6226 .24728 L
.62306 .29226 L
.62353 .2688 L
.62399 .25578 L
.62446 .29592 L
.62492 .23332 L
.62539 .24989 L
.62585 .26298 L
.62632 .22444 L
.62678 .23131 L
.62725 .20954 L
.62771 .22748 L
.62818 .19296 L
Mistroke
.62864 .22583 L
.62911 .23076 L
.62957 .16348 L
.63004 .24375 L
.6305 .15189 L
.63097 .20044 L
.63144 .20771 L
.6319 .15948 L
.63237 .19394 L
.63283 .19341 L
.6333 .1665 L
.63376 .16703 L
.63423 .20428 L
.63469 .14589 L
.63516 .16625 L
.63562 .2043 L
.63609 .14861 L
.63655 .19678 L
.63702 .16165 L
.63748 .17844 L
.63795 .19441 L
.63841 .17814 L
.63888 .18364 L
.63934 .17257 L
.63981 .21518 L
.64027 .2373 L
.64074 .16336 L
.64121 .22251 L
.64167 .23279 L
.64214 .17796 L
.6426 .2556 L
.64307 .21639 L
.64353 .24317 L
.644 .25985 L
.64446 .24045 L
.64493 .24859 L
.64539 .27655 L
.64586 .25267 L
.64632 .27679 L
.64679 .30253 L
.64725 .28315 L
.64772 .30977 L
.64818 .31233 L
.64865 .31822 L
.64911 .31759 L
.64958 .32495 L
.65005 .33021 L
.65051 .33563 L
.65098 .35124 L
.65144 .31088 L
Mistroke
.65191 .37288 L
.65237 .35361 L
.65284 .35619 L
.6533 .34382 L
.65377 .388 L
.65423 .3505 L
.6547 .36281 L
.65516 .38188 L
.65563 .35589 L
.65609 .36302 L
.65656 .37634 L
.65702 .36323 L
.65749 .36988 L
.65795 .38151 L
.65842 .36133 L
.65889 .3549 L
.65935 .37144 L
.65982 .36416 L
.66028 .34006 L
.66075 .3271 L
.66121 .35182 L
.66168 .35373 L
.66214 .3078 L
.66261 .35906 L
.66307 .32497 L
.66354 .32969 L
.664 .29215 L
.66447 .32982 L
.66493 .30252 L
.6654 .29938 L
.66586 .31631 L
.66633 .27847 L
.66679 .31155 L
.66726 .2818 L
.66773 .2896 L
.66819 .26572 L
.66866 .30903 L
.66912 .25648 L
.66959 .27621 L
.67005 .27309 L
.67052 .26728 L
.67098 .26406 L
.67145 .26063 L
.67191 .26414 L
.67238 .22976 L
.67284 .27514 L
.67331 .23356 L
.67377 .24065 L
.67424 .26886 L
.6747 .22331 L
Mistroke
.67517 .28318 L
.67563 .20141 L
.6761 .28515 L
.67657 .24141 L
.67703 .24745 L
.6775 .24584 L
.67796 .26622 L
.67843 .25078 L
.67889 .28875 L
.67936 .23557 L
.67982 .29434 L
.68029 .26016 L
.68075 .27018 L
.68122 .29877 L
.68168 .24733 L
.68215 .2816 L
.68261 .28875 L
.68308 .25959 L
.68354 .29953 L
.68401 .30093 L
.68447 .25995 L
.68494 .30435 L
.6854 .28724 L
.68587 .29905 L
.68634 .27097 L
.6868 .31647 L
.68727 .26016 L
.68773 .29023 L
.6882 .30022 L
.68866 .2695 L
.68913 .29373 L
.68959 .25896 L
.69006 .3107 L
.69052 .24542 L
.69099 .30362 L
.69145 .27774 L
.69192 .25279 L
.69238 .29205 L
.69285 .27418 L
.69331 .27474 L
.69378 .28434 L
.69424 .26706 L
.69471 .29707 L
.69518 .27448 L
.69564 .2623 L
.69611 .28558 L
.69657 .25547 L
.69704 .28887 L
.6975 .28954 L
.69797 .26637 L
Mistroke
.69843 .28016 L
.6989 .25383 L
.69936 .29262 L
.69983 .25731 L
.70029 .25003 L
.70076 .29872 L
.70122 .22995 L
.70169 .26934 L
.70215 .25961 L
.70262 .21956 L
.70308 .28124 L
.70355 .20423 L
.70402 .25304 L
.70448 .21923 L
.70495 .23511 L
.70541 .22236 L
.70588 .19682 L
.70634 .24505 L
.70681 .19375 L
.70727 .22686 L
.70774 .19899 L
.7082 .17951 L
.70867 .23358 L
.70913 .19355 L
.7096 .20291 L
.71006 .21477 L
.71053 .17964 L
.71099 .21598 L
.71146 .22018 L
.71192 .19568 L
.71239 .22472 L
.71286 .20281 L
.71332 .22396 L
.71379 .21298 L
.71425 .24673 L
.71472 .22205 L
.71518 .25767 L
.71565 .2599 L
.71611 .23888 L
.71658 .274 L
.71704 .26736 L
.71751 .25961 L
.71797 .2997 L
.71844 .28542 L
.7189 .31296 L
.71937 .29801 L
.71983 .29564 L
.7203 .35146 L
.72076 .27072 L
.72123 .35838 L
Mistroke
.72169 .32983 L
.72216 .32553 L
.72263 .36914 L
.72309 .34955 L
.72356 .33383 L
.72402 .36777 L
.72449 .32825 L
.72495 .37878 L
.72542 .33995 L
.72588 .36263 L
.72635 .35251 L
.72681 .37776 L
.72728 .3539 L
.72774 .37382 L
.72821 .33558 L
.72867 .38412 L
.72914 .36442 L
.7296 .36672 L
.73007 .36898 L
.73053 .39736 L
.731 .3569 L
.73147 .37011 L
.73193 .39704 L
.7324 .36229 L
.73286 .38665 L
.73333 .37438 L
.73379 .39007 L
.73426 .36144 L
.73472 .38553 L
.73519 .3996 L
.73565 .36497 L
.73612 .39465 L
.73658 .3879 L
.73705 .35596 L
.73751 .40022 L
.73798 .41001 L
.73844 .36376 L
.73891 .4121 L
.73937 .35961 L
.73984 .39406 L
.74031 .36019 L
.74077 .39223 L
.74124 .36949 L
.7417 .36773 L
.74217 .36858 L
.74263 .36568 L
.7431 .37163 L
.74356 .33417 L
.74403 .35202 L
.74449 .35852 L
Mistroke
.74496 .32068 L
.74542 .34324 L
.74589 .31868 L
.74635 .328 L
.74682 .34093 L
.74728 .30369 L
.74775 .29885 L
.74821 .3063 L
.74868 .29885 L
.74915 .2722 L
.74961 .28296 L
.75008 .30557 L
.75054 .2563 L
.75101 .26678 L
.75147 .27721 L
.75194 .25658 L
.7524 .26097 L
.75287 .2843 L
.75333 .24498 L
.7538 .26824 L
.75426 .2624 L
.75473 .23867 L
.75519 .25133 L
.75566 .26268 L
.75612 .26335 L
.75659 .23769 L
.75705 .29484 L
.75752 .22361 L
.75798 .28036 L
.75845 .24622 L
.75892 .26404 L
.75938 .27673 L
.75985 .26416 L
.76031 .27492 L
.76078 .28675 L
.76124 .28774 L
.76171 .27591 L
.76217 .29918 L
.76264 .29656 L
.7631 .30515 L
.76357 .30885 L
.76403 .33065 L
.7645 .3101 L
.76496 .35324 L
.76543 .34064 L
.76589 .329 L
.76636 .35218 L
.76682 .3753 L
.76729 .34444 L
.76776 .38217 L
Mistroke
.76822 .38145 L
.76869 .42243 L
.76915 .37639 L
.76962 .43031 L
.77008 .39045 L
.77055 .41351 L
.77101 .47494 L
.77148 .39021 L
.77194 .46267 L
.77241 .47017 L
.77287 .46223 L
.77334 .47078 L
.7738 .51711 L
.77427 .46999 L
.77473 .52104 L
.7752 .49729 L
.77566 .52708 L
.77613 .523 L
.7766 .56121 L
.77706 .51708 L
.77753 .58325 L
.77799 .54372 L
.77846 .5687 L
.77892 .55574 L
.77939 .57344 L
.77985 .5695 L
.78032 .59067 L
.78078 .56424 L
.78125 .58846 L
.78171 .55879 L
.78218 .58634 L
.78264 .55947 L
.78311 .60332 L
.78357 .53755 L
.78404 .57997 L
.7845 .54705 L
.78497 .57773 L
.78544 .51546 L
.7859 .58127 L
.78637 .52169 L
.78683 .52361 L
.7873 .51619 L
.78776 .51451 L
.78823 .486 L
.78869 .49232 L
.78916 .4749 L
.78962 .44732 L
.79009 .47503 L
.79055 .45381 L
.79102 .4185 L
Mistroke
.79148 .44248 L
.79195 .43213 L
.79241 .39127 L
.79288 .3852 L
.79334 .38234 L
.79381 .3795 L
.79428 .38975 L
.79474 .30854 L
.79521 .38349 L
.79567 .34241 L
.79614 .3091 L
.7966 .32889 L
.79707 .33491 L
.79753 .29007 L
.798 .30842 L
.79846 .30899 L
.79893 .27834 L
.79939 .32225 L
.79986 .29429 L
.80032 .28077 L
.80079 .31857 L
.80125 .29299 L
.80172 .26751 L
.80218 .29612 L
.80265 .29892 L
.80311 .26855 L
.80358 .29622 L
.80405 .27044 L
.80451 .29896 L
.80498 .28897 L
.80544 .27319 L
.80591 .29258 L
.80637 .29132 L
.80684 .28355 L
.8073 .28927 L
.80777 .27636 L
.80823 .31407 L
.8087 .26985 L
.80916 .28651 L
.80963 .30385 L
.81009 .29162 L
.81056 .26753 L
.81102 .31021 L
.81149 .28107 L
.81195 .29664 L
.81242 .29301 L
.81289 .2581 L
.81335 .31588 L
.81382 .29847 L
.81428 .29313 L
Mistroke
.81475 .33321 L
.81521 .26872 L
.81568 .29858 L
.81614 .3204 L
.81661 .29799 L
.81707 .29884 L
.81754 .33017 L
.818 .30678 L
.81847 .33183 L
.81893 .30918 L
.8194 .30904 L
.81986 .31435 L
.82033 .33014 L
.82079 .34836 L
.82126 .3111 L
.82173 .34361 L
.82219 .31864 L
.82266 .32244 L
.82312 .33572 L
.82359 .33039 L
.82405 .34439 L
.82452 .31861 L
.82498 .33726 L
.82545 .33338 L
.82591 .30168 L
.82638 .32536 L
.82684 .3391 L
.82731 .31342 L
.82777 .31531 L
.82824 .33238 L
.8287 .34332 L
.82917 .33532 L
.82963 .2912 L
.8301 .36537 L
.83057 .30082 L
.83103 .33232 L
.8315 .34549 L
.83196 .31735 L
.83243 .35745 L
.83289 .344 L
.83336 .31872 L
.83382 .33546 L
.83429 .35921 L
.83475 .31595 L
.83522 .3458 L
.83568 .33606 L
.83615 .29507 L
.83661 .33835 L
.83708 .32348 L
.83754 .29482 L
Mistroke
.83801 .30669 L
.83847 .32993 L
.83894 .27749 L
.8394 .31361 L
.83987 .29475 L
.84034 .29876 L
.8408 .26305 L
.84127 .29459 L
.84173 .29193 L
.8422 .25777 L
.84266 .29416 L
.84313 .26265 L
.84359 .27166 L
.84406 .2711 L
.84452 .26439 L
.84499 .25656 L
.84545 .26093 L
.84592 .25896 L
.84638 .26904 L
.84685 .25446 L
.84731 .26335 L
.84778 .26879 L
.84824 .26921 L
.84871 .25531 L
.84918 .29199 L
.84964 .25714 L
.85011 .2705 L
.85057 .29468 L
.85104 .2562 L
.8515 .27771 L
.85197 .30056 L
.85243 .26811 L
.8529 .30781 L
.85336 .27062 L
.85383 .29706 L
.85429 .27937 L
.85476 .28624 L
.85522 .30787 L
.85569 .2871 L
.85615 .32736 L
.85662 .26681 L
.85708 .30827 L
.85755 .29364 L
.85802 .29699 L
.85848 .27252 L
.85895 .30956 L
.85941 .26652 L
.85988 .28126 L
.86034 .27747 L
.86081 .23479 L
Mistroke
.86127 .28098 L
.86174 .22871 L
.8622 .21595 L
.86267 .23955 L
.86313 .20897 L
.8636 .20669 L
.86406 .2019 L
.86453 .20137 L
.86499 .19373 L
.86546 .14758 L
.86592 .21174 L
.86639 .1611 L
.86686 .14657 L
.86732 .19124 L
.86779 .13076 L
.86825 .15521 L
.86872 .1766 L
.86918 .11927 L
.86965 .14724 L
.87011 .17079 L
.87058 .10992 L
.87104 .16709 L
.87151 .14582 L
.87197 .13402 L
.87244 .18028 L
.8729 .13928 L
.87337 .13906 L
.87383 .1825 L
.8743 .12499 L
.87476 .13274 L
.87523 .17908 L
.87569 .13506 L
.87616 .17642 L
.87663 .17535 L
.87709 .15351 L
.87756 .19051 L
.87802 .15345 L
.87849 .17376 L
.87895 .17764 L
.87942 .19053 L
.87988 .17504 L
.88035 .17485 L
.88081 .2104 L
.88128 .18426 L
.88174 .17059 L
.88221 .21108 L
.88267 .1713 L
.88314 .19873 L
.8836 .17174 L
.88407 .18154 L
Mistroke
.88453 .19512 L
.885 .19417 L
.88547 .17622 L
.88593 .18455 L
.8864 .18572 L
.88686 .19676 L
.88733 .15939 L
.88779 .19178 L
.88826 .16727 L
.88872 .1656 L
.88919 .19365 L
.88965 .16429 L
.89012 .19996 L
.89058 .16751 L
.89105 .188 L
.89151 .2001 L
.89198 .17598 L
.89244 .1823 L
.89291 .21196 L
.89337 .18131 L
.89384 .1969 L
.89431 .21896 L
.89477 .18513 L
.89524 .21743 L
.8957 .19663 L
.89617 .22522 L
.89663 .21347 L
.8971 .21818 L
.89756 .22271 L
.89803 .22588 L
.89849 .22683 L
.89896 .2477 L
.89942 .23921 L
.89989 .24651 L
.90035 .26251 L
.90082 .27221 L
.90128 .25387 L
.90175 .27879 L
.90221 .29455 L
.90268 .26085 L
.90315 .31882 L
.90361 .28297 L
.90408 .31697 L
.90454 .33595 L
.90501 .29493 L
.90547 .36407 L
.90594 .31222 L
.9064 .35718 L
.90687 .33822 L
.90733 .35042 L
Mistroke
.9078 .34035 L
.90826 .35703 L
.90873 .34869 L
.90919 .3354 L
.90966 .38941 L
.91012 .33328 L
.91059 .35572 L
.91105 .37068 L
.91152 .3571 L
.91199 .34304 L
.91245 .37613 L
.91292 .2974 L
.91338 .37741 L
.91385 .3432 L
.91431 .3405 L
.91478 .32646 L
.91524 .33655 L
.91571 .34416 L
.91617 .31429 L
.91664 .30497 L
.9171 .34705 L
.91757 .317 L
.91803 .31651 L
.9185 .29415 L
.91896 .32413 L
.91943 .29642 L
.91989 .30067 L
.92036 .27322 L
.92082 .29469 L
.92129 .30187 L
.92176 .28775 L
.92222 .23802 L
.92269 .28467 L
.92315 .23168 L
.92362 .23285 L
.92408 .25734 L
.92455 .19503 L
.92501 .243 L
.92548 .22651 L
.92594 .17067 L
.92641 .24579 L
.92687 .18263 L
.92734 .20378 L
.9278 .19124 L
.92827 .192 L
.92873 .16754 L
.9292 .15944 L
.92966 .18618 L
.93013 .19965 L
.9306 .14981 L
Mistroke
.93106 .19112 L
.93153 .15956 L
.93199 .17265 L
.93246 .14661 L
.93292 .15169 L
.93339 .18814 L
.93385 .15302 L
.93432 .14042 L
.93478 .18572 L
.93525 .17437 L
.93571 .15989 L
.93618 .1772 L
.93664 .18918 L
.93711 .12941 L
.93757 .21832 L
.93804 .16031 L
.9385 .17972 L
.93897 .19204 L
.93944 .17494 L
.9399 .1973 L
.94037 .16156 L
.94083 .17097 L
.9413 .21644 L
.94176 .17383 L
.94223 .20089 L
.94269 .18047 L
.94316 .20086 L
.94362 .18432 L
.94409 .20044 L
.94455 .18894 L
.94502 .22295 L
.94548 .2124 L
.94595 .22083 L
.94641 .20176 L
.94688 .2342 L
.94734 .22239 L
.94781 .23973 L
.94828 .25148 L
.94874 .23507 L
.94921 .26407 L
.94967 .25687 L
.95014 .26976 L
.9506 .26279 L
.95107 .29081 L
.95153 .28786 L
.952 .26525 L
.95246 .30346 L
.95293 .29753 L
.95339 .30051 L
.95386 .31577 L
Mistroke
.95432 .31069 L
.95479 .30983 L
.95525 .33284 L
.95572 .3231 L
.95618 .31685 L
.95665 .31713 L
.95711 .36074 L
.95758 .3069 L
.95805 .35861 L
.95851 .33161 L
.95898 .34781 L
.95944 .33762 L
.95991 .35968 L
.96037 .31962 L
.96084 .34561 L
.9613 .34432 L
.96177 .35246 L
.96223 .37043 L
.9627 .34585 L
.96316 .37371 L
.96363 .32875 L
.96409 .36985 L
.96456 .32776 L
.96502 .34934 L
.96549 .34602 L
.96595 .3678 L
.96642 .36576 L
.96689 .35445 L
.96735 .36342 L
.96782 .37173 L
.96828 .34974 L
.96875 .34437 L
.96921 .3527 L
.96968 .35016 L
.97014 .35935 L
.97061 .34905 L
.97107 .34698 L
.97154 .35553 L
.972 .31664 L
.97247 .35123 L
.97293 .33271 L
.9734 .33459 L
.97386 .33444 L
.97433 .30848 L
.97479 .33081 L
.97526 .31306 L
.97573 .3148 L
.97619 .2983 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{349.875, 216.125},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40008<0001A1000`40O003h00Oogooogoo3Woo003o
OoooOol>Ool00?mooomoo`ioo`00ogooogoo3Woo003oOoooOol>Ool00?mooomoo`ioo`00ogooogoo
3Woo003oOoooOol>Ool00?mooomoo`ioo`00@Woo1@00>Goo2@0017oo1@00>7oo0P001Woo1@00=goo
10001Woo1@00=Goo1@001Woo1@00<Woo0P001goo1@001Woo1@00;7oo0P001Goo2@0017oo1@006Woo
0011Ool7000hOol90003Ool7000gOol20005Ool7000dOol70004Ool7000cOol70004Ool7000aOol2
0006Ool70004Ool7000[Ool20005Ool90003Ool7000IOol0045oo`8000=oo`8003Uoo`8000Uoo`80
00=oo`8003Moo`8000Eoo`8000=oo`8003Aoo`8000=oo`8000Aoo`8000=oo`80039oo`<000=oo`<0
00=oo`8000=oo`80035oo`8000Ioo`8000=oo`8000Aoo`8000=oo`8002]oo`8000Ioo`8000Uoo`80
00=oo`8001Uoo`00@7oo0P001Goo0P00>Goo0P001goo0P001Goo0P00<7oo2P000Woo0P001Goo0P00
<Woo0P001Goo0P000Woo0P001Goo0P00<Goo0P001Goo0P000Woo0P001Goo0P00<7oo0P001Goo0P00
1Goo0P000Woo0P001Goo0P00:Woo0P001goo0P001goo0P001Goo0P0067oo0010Ool20005Ool2000j
Ool20006Ool20005Ool2000`Ool:0002Ool20005Ool2000bOol20005Ool20002Ool20005Ool2000a
Ool20005Ool20002Ool20005Ool2000`Ool20005Ool20005Ool20002Ool20005Ool2000ZOol20008
Ool20006Ool20005Ool2000HOol0041oo`8000Eoo`8003]oo`8000Eoo`8000Eoo`80031oo`8000Ao
o`8000Aoo`8000Eoo`80039oo`8000Eoo`80009oo`8000Eoo`80035oo`8000Eoo`80009oo`8000Eo
o`80031oo`8000Eoo`8000Eoo`80009oo`8000Eoo`8002Yoo`8000Uoo`8000Eoo`8000Eoo`8001Qo
o`00@7oo0P001Goo0P00?7oo0P0017oo0P001Goo0P00<Goo0P000goo0P0017oo0P001Goo0P00<Woo
0P001Goo0P000Woo0P001Goo0P00<Goo0`000goo0`000Woo0P001Goo0P00<7oo0P001Goo0P001Goo
0P000Woo0P001Goo0P00:Woo0P002Woo0P0017oo0P001Goo0P0067oo0010Ool20005Ool2000mOol2
0003Ool20005Ool2000bOol20002Ool20004Ool20005Ool2000bOol30003Ool30002Ool20005Ool2
000bOol70003Ool20005Ool2000`Ool20005Ool20005Ool20002Ool20005Ool2000ZOol2000;Ool2
0003Ool20005Ool2000HOol0041oo`8000Eoo`8003ioo`04001oogooOol20005Ool2000bOol20002
Ool20004Ool20005Ool2000bOol80003Ool20005Ool2000bOol70003Ool20005Ool2000`Ool20005
Ool20005Ool20002Ool20005Ool2000ZOol2000<Ool01000Oomoogoo0P001Goo0P0067oo0010Ool2
0005Ool2000nOol20002Ool20005Ool2000cOol200000goo00000004Ool20005Ool2000bOol20000
0goo000000020004Ool20005Ool2000aOol30003Ool30002Ool20005Ool2000`Ool20005Ool20005
Ool20002Ool20005Ool2000ZOol2000<Ool20002Ool20005Ool2000HOol0041oo`8000Eoo`8003io
o`80009oo`8000Eoo`8003Aoo`@000Aoo`8000Eoo`80039oo`8000Uoo`8000Eoo`80035oo`8000Eo
o`80009oo`8000Eoo`8002eoo`800003Ool0000000Eoo`8000Eoo`80009oo`8000Eoo`8002Moo`80
0003Ool0000000aoo`80009oo`8000Eoo`8001Qoo`00@7oo0P001Goo0P00=goo0P001Goo0P000Woo
0P001Goo0P00=Goo0`0017oo0P001Goo0P00<goo0P0017oo0P000Woo0P001Goo0P00<Goo0P001Goo
0P000Woo0P001Goo0P00;Goo1@001Goo0P001Goo0P000Woo0P001Goo0P009goo1@001Goo0P001Goo
0P000Woo0P001Goo0P0067oo0011Ool20003Ool2000iOol20004Ool20003Ool20003Ool2000fOol3
0005Ool20003Ool2000dOol20003Ool30003Ool20003Ool2000bOol30003Ool30003Ool20003Ool2
000`Ool30006Ool20003Ool20004Ool20003Ool2000ZOol30006Ool20004Ool20003Ool20003Ool2
000IOol0045oo`L003Uoo`L000Aoo`L003Moo`8000Eoo`L003Eoo`H000Aoo`L003=oo`L000Aoo`L0
035oo`8000Ioo`L000Aoo`L002]oo`8000Ioo`L000Aoo`L001Uoo`00@Woo1@00>goo1@001Woo1@00
>Goo00<007ooOol017oo1@00=goo10001Woo1@00=Goo1@001Woo1@00<Woo0P001goo1@001Woo1@00
;7oo0P001goo1@001Woo1@006Woo003oOoooOol>Ool00?mooomoo`ioo`00ogooogoo3Woo003oOooo
Ool>Ool001Eoo`8000Moo`@000Moo`D00?moommoo`005Goo0P001Woo1`0017oo1`00ogoogWoo000E
Ool20005Ool30003Ool20004Ool20003Ool2003oOooNOol001Eoo`8000Eoo`8000Eoo`80009oo`80
00Eoo`800?moomeoo`0017oo2P001goo0P0037oo0P000Woo0P001Goo0P00ogoogGoo000EOol20007
Ool400000goo00000002Ool20005Ool20009Oooo003D000001Eoo`8000Ioo`P0009oo`8000Eoo`80
00Uoo`@000Moo`03001oogoo00moo`03001oogoo00ioo`03001oogoo00ioo`03001oogoo00moo`03
001oogoo00ioo`03001oogoo00ioo`03001oogoo00moo`03001oogoo00ioo`03001oogoo00ioo`03
001oogoo00moo`03001oogoo00ioo`03001oogoo00ioo`03001oogoo00moo`03001oogoo00ioo`03
001oogoo00ioo`03001oogoo00moo`03001oogoo00ioo`03001oogoo00ioo`03001oogoo00moo`03
001oogoo00ioo`03001oogoo00ioo`03001oogoo00moo`03001oogoo00ioo`03001oogoo00ioo`03
001oogoo00moo`03001oogoo00ioo`D000005Goo0P001Goo0`000goo0`000Woo0P001Goo0P002Goo
00<007ooOol027oo00<007ooOol03goo00<007ooOol03Woo00<007ooOol03Woo00<007ooOol03goo
00<007ooOol03Woo00<007ooOol03Woo00<007ooOol03goo00<007ooOol03Woo00<007ooOol03Woo
00<007ooOol03goo00<007ooOol03Woo00<007ooOol03Woo00<007ooOol03goo00<007ooOol03Woo
00<007ooOol03Woo00<007ooOol03goo00<007ooOol03Woo00<007ooOol03Woo00<007ooOol03goo
00<007ooOol03Woo00<007ooOol03Woo00<007ooOol03goo00<007ooOol03Woo00<007ooOol03Woo
00<007ooOol03goo00<007ooOol03Woo00@007ooOomoo`4000005Goo0P001Goo0P001Goo0P000Woo
0P001Goo0P002Goo00<007ooOol027oo00<007ooOol0@goo00<007ooOol0@Woo00<007ooOol0@Woo
00<007ooOol0@goo00<007ooOol0@Woo00<007ooOol0@Woo00<007ooOol097oo0@00000EOol20005
Ool20005Ool20002Ool20005Ool20009Ool00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`12
Ool00`0?Oomoo`13Ool00`0?Oomoo`12Ool00`0?Oomoo`12Ool00`0?Oomoo`0TOol10000019oo`80
0003Ool0000000Eoo`8000Eoo`80009oo`8000Eoo`8000Uoo`03001oogoo04ioo`0300moogoo049o
o`0300moogoo049oo`0300moogoo04=oo`0300moogoo049oo`0300moogoo049oo`0300moogoo02Ao
o`4000004Woo1@001Goo0P001Goo0P000Woo0P001Goo0P002Goo00<007ooOol0CWoo00<03gooOol0
@Woo00<03gooOol0@Woo00<03gooOol0@goo00<03gooOol0@Woo00<03gooOol0@Woo00<03gooOol0
97oo0@00000DOol30006Ool20003Ool20004Ool20003Ool2000:Ool00`00Oomoo`1>Ool00`0?Oomo
o`12Ool00`0?Oomoo`0YOol00`00Oomoo`0FOol00`0?Oomoo`13Ool00`0?Oomoo`12Ool00`0?Oomo
o`12Ool00`0?Oomoo`0TOol1000001Eoo`8000Ioo`L000Aoo`L000Yoo`03001oogoo04ioo`0300mo
ogoo049oo`0300moogoo02Uoo`03001oogoo01Ioo`0300moogoo04=oo`0300moogoo049oo`0300mo
ogoo049oo`0300moogoo02Aoo`4000005Goo0P001goo10001goo1@002goo00<007ooOol0CWoo00<0
3gooOol0@Woo00<03gooOol0:Goo00<007ooOol05Woo00<03gooOol0@goo00<03gooOol0@Woo00<0
3gooOol0@Woo00<03gooOol097oo0@00000iOol00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomo
o`0YOol00`00Oomoo`0FOol00`0?Oomoo`13Ool00`0?Oomoo`12Ool00`0?Oomoo`12Ool00`0?Oomo
o`0TOol1000003Uoo`03001oogoo04ioo`0300moogoo049oo`0300moogoo02Uoo`03001oogoo01Io
o`0300moogoo04=oo`0300moogoo049oo`0300moogoo049oo`0300moogoo02Aoo`400000>Goo00<0
07ooOol0CWoo00<03gooOol0@Woo00<03gooOol0:Goo00<007ooOol05Woo00<03gooOol0@goo00<0
3gooOol0@Woo00<03gooOol0@Woo00<03gooOol097oo0@00000iOol00`00Oomoo`1>Ool00`0?Oomo
o`12Ool00`0?Oomoo`0YOol00`00Oomoo`0FOol00`0?Oomoo`13Ool00`0?Oomoo`12Ool00`0?Oomo
o`12Ool00`0?Oomoo`0TOol1000003Uoo`<004ioo`0300moogoo049oo`0300moogoo02Uoo`03001o
ogoo01Ioo`0300moogoo04=oo`0300moogoo049oo`0300moogoo049oo`0300moogoo029oo`<00000
>Goo00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol0:Goo00<007ooOol05Woo00<03gooOol0
@goo00<03gooOol0@Woo00<03gooOol0@Woo00<03gooOol097oo0@00000iOol00`00Oomoo`1>Ool0
0`0?Oomoo`12Ool00`0?Oomoo`0YOol00`00Oomoo`0FOol00`0?Oomoo`13Ool00`0?Oomoo`12Ool0
0`0?Oomoo`12Ool00`0?Oomoo`0TOol1000003Uoo`03001oogoo04ioo`0300moogoo049oo`0300mo
ogoo02Uoo`8001Moo`0300moogoo04=oo`0300moogoo049oo`0300moogoo049oo`0300moogoo02Ao
o`400000>Goo00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol0:Goo0P005goo00<03gooOol0
@goo00<03gooOol0@Woo00<03gooOol0@Woo00<03gooOol097oo0@00000iOol00`00Oomoo`1>Ool0
0`0?Oomoo`12Ool00`0?Oomoo`0YOol200000goo001oo`0DOol00`0?Oomoo`13Ool00`0?Oomoo`12
Ool00`0?Oomoo`12Ool00`0?Oomoo`0TOol1000003Uoo`03001oogoo04ioo`0300moogoo049oo`03
00moogoo02Uoo`800003Ool007oo01=oo`0300003goo04Aoo`0300moogoo049oo`0300moogoo049o
o`0300moogoo02Aoo`400000>Goo00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol0:Goo0P00
00=oo`00Ool04goo0P00AGoo00<03gooOol0@Woo00<03gooOol0@Woo00<03gooOol097oo0@00000i
Ool00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`0YOol200000goo001oo`0COol20015Ool0
0`0?Oomoo`12Ool00`0?Oomoo`12Ool00`0?Oomoo`0TOol1000003Uoo`03001oogoo04ioo`0300mo
ogoo049oo`0300moogoo02Uoo`800003Ool007oo01=oo`8004Eoo`0300moogoo049oo`0300moogoo
049oo`0300moogoo02Aoo`400000>Goo00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol0:Goo
0P0000=oo`00Ool04goo0P00AGoo00<03gooOol0@Woo00<03gooOol0@Woo00<03gooOol097oo0@00
000iOol00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`0YOol200000goo001oo`0COol20015
Ool00`0?Oomoo`12Ool00`0?Oomoo`12Ool00`0?Oomoo`0TOol1000003Uoo`03001oogoo04ioo`03
00moogoo049oo`0300moogoo02Uoo`@001Aoo`8004Eoo`0300moogoo049oo`0300moogoo049oo`03
00moogoo02Aoo`400000>Goo0`00CWoo00<03gooOol0@Woo00<03gooOol0:Goo10004Woo00@007oo
000004Eoo`0300moogoo049oo`0300moogoo049oo`0300moogoo029oo`<00000>Goo00<007ooOol0
CWoo00<03gooOol0@Woo00<03gooOol09goo00<007oo00000`004Woo00@007oo000004Eoo`0300mo
ogoo049oo`0300moogoo049oo`0300moogoo02Aoo`400000>Goo00<007ooOol0CWoo00<03gooOol0
@Woo00<03gooOol09goo1P004Woo1000AGoo00<03gooOol0@Woo00<03gooOol0@Woo00<03gooOol0
97oo0@00000iOol00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`0WOol6000BOol40015Ool0
0`0?Oomoo`12Ool00`0?Oomoo`12Ool00`0?Oomoo`0TOol1000003Uoo`03001oogoo04ioo`0300mo
ogoo049oo`0300moogoo02Moo`H0019oo`@004Eoo`0300moogoo049oo`0300moogoo049oo`0300mo
ogoo02Aoo`400000>Goo00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol09goo1P004Woo0P00
00=oo`00Ool0A7oo00<03gooOol0@Woo00<03gooOol0@Woo00<03gooOol097oo0@00000iOol00`00
Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`0WOol6000BOol200000goo00000014Ool00`0?Oomo
o`12Ool00`0?Oomoo`12Ool00`0?Oomoo`0TOol1000003Uoo`03001oogoo04ioo`0300moogoo049o
o`0300moogoo02Moo`H0019oo`800003Ool0000004Aoo`0300moogoo049oo`0300moogoo049oo`03
00moogoo02Aoo`400000>Goo00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol09goo1P004Woo
0P0000=oo`000000A7oo00<03gooOol0@Woo00<03gooOol0@Woo00<03gooOol097oo0@00000iOol0
0`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`0WOol6000BOol200000goo00000014Ool00`0?
Oomoo`12Ool00`0?Oomoo`12Ool00`0?Oomoo`0TOol1000003Uoo`03001oogoo04ioo`0300moogoo
049oo`0300moogoo02Moo`H0015oo`<00003Ool0000004Aoo`0300moogoo049oo`0300moogoo049o
o`0300moogoo02Aoo`400000>Goo00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol09goo1P00
4Goo0`0000=oo`000000A7oo00<03gooOol0@Woo00<03gooOol0@Woo00<03gooOol097oo0@00000i
Ool00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`0WOol6000AOol20002Ool20014Ool00`0?
Oomoo`12Ool00`0?Oomoo`12Ool00`0?Oomoo`0TOol1000003Uoo`<004ioo`0300moogoo049oo`03
00moogoo00eoo`03001oogoo01Moo`H0015oo`80009oo`8004Aoo`0300moogoo049oo`0300moogoo
049oo`0300moogoo029oo`<00000>Goo00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol03Goo
00<007ooOol05Woo1@0000=oo`00Ool047oo0P000Woo0P00A7oo00<03gooOol0@Woo00<03gooOol0
@Woo00<03gooOol097oo0@00000iOol00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`0=Ool0
0`00Oomoo`0FOol200001Goo000007oo00004Goo0P000Woo0P00A7oo00<03gooOol0@Woo00<03goo
Ool0@Woo00<03gooOol097oo0@00000iOol00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`0=
Ool00`00Oomoo`0FOol2000017oo000007oo0P0047oo00@007ooOomoo`8004Aoo`0300moogoo049o
o`0300moogoo049oo`0300moogoo02Aoo`400000>Goo00<007ooOol0CWoo00<03gooOol0@Woo00<0
3gooOol03Goo00<007ooOol05Woo0P0000Aoo`00001oo`80011oo`06001oogooOol03`00A7oo00<0
3gooOol0@Woo00<03gooOol0@Woo00<03gooOol097oo0@00000iOol00`00Oomoo`1>Ool00`0?Oomo
o`12Ool00`0?Oomoo`0=Ool00`00Oomoo`0FOol2000017oo000007oo0P003goo0P000goo00<03`00
Ool0@goo00<03gooOol0@Woo00<03gooOol0@Woo00<03gooOol097oo0@00000iOol00`00Oomoo`1>
Ool00`0?Oomoo`12Ool00`0?Oomoo`0=Ool00`00Oomoo`0FOol2000017oo000007oo0P003goo0P00
0goo00<03`00Ool0@goo00<03gooOol0@Woo00<03gooOol0@Woo00<03gooOol097oo0@00000iOol0
0`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`0=Ool00`00Oomoo`0FOol20002Ool01000Ool0
00003goo0P000goo00<03`00Ool0@goo00<03gooOol0@Woo00<03gooOol0@Woo00<03gooOol097oo
0@00000iOol00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`0=Ool2000GOol20002Ool01000
Ool000003goo0P000goo00<03`00Ool0@goo00<03gooOol0@Woo00<03gooOol0@Woo00<03gooOol0
97oo0@00000iOol00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`0=Ool2000GOol20002Ool0
1000Ool000003goo0P000goo00<03`00Ool0@goo00<03gooOol0@Woo00<03gooOol0@Woo00<03goo
Ool097oo0@00000iOol00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`0<Ool3000GOol20004
Ool2000?Ool20003Ool00`0?001oo`13Ool00`0?Oomoo`12Ool00`0?Oomoo`12Ool00`0?Oomoo`0T
Ool1000003Uoo`03001oogoo04ioo`0300moogoo049oo`0300moogoo00aoo`<001Moo`8000Aoo`80
00moo`8000=oo`0300l007oo04=oo`0300moogoo049oo`0300moogoo02eoo`03001oogoo019oo`03
00moogoo02Aoo`400000>Goo00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol037oo10005Woo
0P0017oo0P003goo0P000goo00<03`00Ool0@goo00<03gooOol0@Woo00<03gooOol0;Goo00<007oo
Ool04Woo00<03gooOol097oo0@00000iOol00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`0<
Ool4000FOol20004Ool2000?Ool20003Ool00`0?001oo`13Ool00`0?Oomoo`12Ool00`0?Oomoo`0]
Ool00`00Oomoo`0BOol00`0?Oomoo`0TOol1000003Uoo`<004ioo`0300moogoo049oo`0300moogoo
00aoo`@001Ioo`8000Aoo`8000moo`8000=oo`0300l007oo04=oo`0300moogoo049oo`0300moogoo
02eoo`03001oogoo019oo`0300moogoo029oo`<00000>Goo00<007ooOol0CWoo00<03gooOol0@Woo
00<03gooOol037oo10005Woo00<007ooOol00goo0P003goo0P000goo00<03goo0000@goo00<03goo
Ool0@Woo00<03gooOol0;Goo00<007ooOol04Woo00<03gooOol097oo0@00000iOol00`00Oomoo`10
Ool00`00Oomoo`0;Ool00`0?Oomoo`12Ool00`0?Oomoo`0<Ool4000FOol00`00Oomoo`03Ool2000?
Ool20003Ool00`0?Ool00013Ool00`0?Oomoo`12Ool00`0?Oomoo`0/Ool2000DOol00`0?Oomoo`0T
Ool1000003Uoo`03001oogoo03ioo`03001oo`0000eoo`0300moogoo03Uoo`03001oogoo00Ioo`03
00moogoo00]oo`H001Aoo`8000Eoo`8000moo`8000=oo`0300moo`0004=oo`0300moogoo049oo`03
00moogoo02aoo`8001Aoo`0300moogoo02Aoo`400000>Goo00<007ooOol0?Woo00<007oo00003Goo
00<03gooOol0>Goo00<007ooOol01Woo00<03gooOol02goo1P0057oo0P001Goo0P003goo0P000goo
00<03goo0000@goo00<03gooOol0@Woo00<03gooOol0;7oo0P0000=oo`00Ool04Goo00<03gooOol0
97oo0@00000iOol00`00Oomoo`0nOol00`00Ool0000=Ool00`0?Oomoo`0iOol20007Ool00`0?Oomo
o`0;Ool7000COol20005Ool2000?Ool20003Ool00`0?Ool00013Ool00`0?Oomoo`12Ool00`0?Oomo
o`0/Ool200000goo001oo`0AOol00`0?Oomoo`0TOol1000003Uoo`03001oogoo03ioo`03001oo`00
00eoo`0300moogoo03Uoo`8000Moo`0300moogoo00]oo`L001=oo`8000Eoo`8000moo`8000=oo`03
00moo`0004=oo`0300moogoo049oo`0300moogoo02aoo`800003Ool007oo015oo`0300moogoo00Mo
o`03001oogoo01Yoo`4000005Goo0P001goo1@001Woo1@002goo00<007ooOol0?Woo00<007oo0000
3Goo00<03gooOol0>Goo0`001Woo00<03gooOol02goo1`004goo0P001Goo0P003goo0P000goo00<0
3goo0000@goo00<03gooOol0@Woo00<03gooOol0;7oo0P0000=oo`00Ool04Goo00<03gooOol01goo
00<007ooOol06Woo0@00000EOol20006Ool70004Ool7000:Ool00`00Oomoo`0nOol00`00Ool0000=
Ool00`0?Oomoo`0iOol30006Ool00`0?Oomoo`0;Ool7000COol20005Ool2000?Ool20003Ool00`0?
Ool0000DOol2000]Ool00`0?Oomoo`12Ool00`0?Oomoo`0/Ool200000goo001oo`0AOol00`0?Oomo
o`07Ool00`00Oomoo`0JOol1000001Eoo`8000Eoo`<000=oo`<000=oo`8000=oo`8000Yoo`03001o
ogoo03ioo`03001oo`0000eoo`0300moogoo03Qoo`@000Ioo`0300moogoo00]oo`L001=oo`8000Eo
o`8000moo`8000=oo`0300moo`0001Aoo`8002eoo`0300moogoo049oo`0300moogoo02aoo`800003
Ool007oo015oo`0300moogoo00Moo`03001oogoo01Yoo`4000005Goo0P001Goo0P001Goo0P000Woo
0P001Goo0P002Goo00<007ooOol0?Woo00<007oo00003Goo00<03gooOol0>7oo10001Woo00<03goo
Ool02goo00<007ooOol010004goo0P001Woo00<007ooOol03Goo0P000goo00<03goo000057oo0P00
;Goo00<03gooOol0@Woo00<03gooOol0;7oo0P0000=oo`00Ool04Goo00<03gooOol01goo00<007oo
Ool06Woo0@000004Ool:0007Ool20005Ool20005Ool20002Ool20005Ool20009Ool00`00Oomoo`0n
Ool3000=Ool00`0?Oomoo`0hOol40006Ool00`0?Oomoo`0;Ool00`00Oomoo`04000COol20006Ool0
0`00Oomoo`0=Ool20003Ool00`0?Ool0000DOol2000]Ool00`0?Oomoo`12Ool00`0?Oomoo`0/Ool4
000BOol00`0?Oomoo`07Ool00`00Oomoo`0JOol1000001Eoo`8000Eoo`8000Eoo`80009oo`8000Eo
o`8000Uoo`03001oogoo03ioo`<000eoo`0300moogoo03Qoo`D000Eoo`0300moogoo00]oo`03001o
ogoo00@001=oo`8000Ioo`8000ioo`03001oogoo009oo`0300moo`0001Aoo`8002eoo`0300moogoo
049oo`0300moogoo02aoo`@0019oo`0300moogoo00Ioo`8001aoo`4000005Goo0P001Goo0`000goo
0`000Woo0P001Goo0P002Goo1000?@0?0`00B00?1@004`0?00<0000?00l010004`0?00<0000?00l0
1@0?0P003P0?00<0000?00l0100?00<0000?00l0400?00@0000?00000:403`@001/03`8001T03`@0
00005Goo0P001Woo1`000goo0P001Goo0P002Goo00<007ooOol0?Woo0`003Goo00<03gooOol0>7oo
1P0017oo00<03gooOol02goo00<007ooOol00Woo0P004goo00<007ooOol01Goo0P003Woo00<007oo
Ool00Woo00<03goo00004Woo00@007oo000002eoo`0300moogoo00Aoo`03001oogoo03]oo`0300mo
ogoo02]oo`D0019oo`0300moogoo00Eoo`<001aoo`4000005Goo0P001Woo1`000goo0P001Goo0P00
2Goo00<007ooOol0?Woo0`0000=oo`00Ool02Woo00<03gooOol0>7oo1P0017oo00<03gooOol02goo
00<007ooOol00Woo0P004goo00<007ooOol01Goo0P003Woo00<007ooOol00Woo00@03goo0000015o
o`@002eoo`0300moogoo00Aoo`8003aoo`0300moogoo02Yoo`H0019oo`0300moogoo00Eoo`<001ao
o`4000005Goo0P001Goo0`000goo0`000Woo0P001Goo0P002Goo00<007ooOol0?Woo1@002goo00<0
3gooOol0>7oo1P0017oo00<03gooOol02Woo0P0017oo0P004goo00<007ooOol01Goo0P003Goo0P00
17oo00@03goo0000015oo`@002eoo`0300moogoo00Aoo`8003aoo`0300moogoo02Yoo`H0019oo`03
00moogoo00Aoo`@001aoo`4000004Woo0P0000=oo`0000001Goo0P001Goo0P000Woo0P001Goo0P00
2Goo00<007ooOol0?Woo1@002goo00<03gooOol0>7oo1P0017oo00<03gooOol02Woo0P0017oo0P00
4goo00<007ooOol01Goo0P003Goo0P0017oo00@03goo0000015oo`D002aoo`0300moogoo009oo`04
001oo`00000lOol00`0?Oomoo`0ZOol6000BOol00`0?Oomoo`04Ool4000LOol10000019oo`D000Eo
o`8000Eoo`80009oo`8000Eoo`8000Uoo`03001oogoo03ioo`D000]oo`0300moogoo03Qoo`H000Ao
o`0300moogoo00Yoo`8000Aoo`8001=oo`03001oogoo00Eoo`8000eoo`8000Aoo`0400moo`00000A
Ool5000/Ool00`0?Oomoo`02Ool01000Ool00000?7oo00<03gooOol0:Woo1`004Goo00<03gooOol0
17oo100077oo0@00000DOol30005Ool30003Ool30003Ool20003Ool2000:Ool00`00Oomoo`0nOol5
000;Ool00`0?Oomoo`0hOol60004Ool00`0?Oomoo`0:Ool20005Ool00`00Oomoo`0AOol00`00Oomo
o`05Ool2000=Ool20004Ool0100?Ool000004Goo1@00;7oo00<03gooOol00Woo00@007oo000003ao
o`0300moogoo02Yoo`L0015oo`0300moogoo00Aoo`@001aoo`4000005Goo0P001Woo1`0017oo1`00
2Woo00<007ooOol0?Woo1@002goo00<03gooOol0>7oo1P0017oo00<03gooOol02Woo0P001Goo00<0
07ooOol04Goo00<007ooOol01Goo0P003Goo0P0017oo00@03goo0000015oo`D002aoo`0300moogoo
009oo`@003aoo`0300moogoo02Yoo`L000Aoo`03001oogoo00Yoo`0300moogoo00=oo`D001aoo`40
00005Goo0P001goo1@001Woo1@002goo00<007ooOol0?Woo1@002goo00<03gooOol0=goo1`0017oo
00<03gooOol02Woo0P001Goo00<007ooOol047oo00<007ooOol01Woo0P003Goo0P0017oo00@03goo
0000015oo`H002]oo`0300moogoo009oo`@003aoo`0300moogoo02Yoo`L000Aoo`03001oogoo00Yo
o`0300moogoo00=oo`H001]oo`400000>Goo00<007ooOol0?Woo1@002goo00<03gooOol0=goo0`00
00Aoo`00000000Aoo`0300moogoo00Yoo`03001oogoo00Aoo`03001oogoo011oo`03001oogoo00Io
o`8000eoo`8000Aoo`0400moo`00000?Ool00`00Ool00005000[Ool00`0?Oomoo`02Ool400000goo
001oo`0iOol00`0?Oomoo`0ZOol70004Ool00`00Oomoo`0:Ool00`0?Oomoo`03Ool6000KOol10000
03Uoo`03001oogoo03ioo`D000]oo`0300moogoo03Moo`<0009oo`8000Aoo`0300moogoo00Yoo`03
001oogoo00Aoo`03001oogoo011oo`03001oogoo00Ioo`8000eoo`8000Aoo`0400moo`00000?Ool0
0`00Ool00005000GOol00`00Oomoo`0AOol00`0?Oomoo`02Ool400000goo001oo`0iOol00`0?Oomo
o`0ZOol80003Ool2000;Ool00`0?Oomoo`03Ool6000KOol1000003Uoo`03001oogoo02aoo`03001o
ogoo00moo`D000]oo`0300moogoo03Moo`<0009oo`8000Aoo`0300moogoo00Yoo`03001oogoo00Ao
o`03001oogoo011oo`03001oogoo00Moo`03001oogoo00]oo`03001oogoo00=oo`0400moo`00000?
Ool00`00Ool00005000FOol2000COol00`0?Oomoo`02Ool400000goo001oo`0iOol00`0?Oomoo`0Z
Ool80003Ool2000;Ool00`0?Oomoo`03Ool7000JOol1000003Uoo`03001oogoo02aoo`03001oogoo
00moo`03001oo`00008000]oo`0300moogoo03Moo`<0009oo`8000Aoo`0300moogoo00Yoo`03001o
ogoo00Aoo`03001oogoo011oo`03001oogoo00Moo`03001oogoo00]oo`03001oogoo00=oo`0400mo
o`00000?Ool8000FOol2000COol00`0?Oomoo`02Ool400000goo001oo`0iOol00`0?Oomoo`0ZOol8
0003Ool2000;Ool00`0?Oomoo`03Ool7000JOol1000003Uoo`<002aoo`03001oogoo00ioo`800004
Ool00000000;Ool00`0?Oomoo`0gOol30002Ool20004Ool00`0?Oomoo`0:Ool00`00Oomoo`04Ool2
000AOol00`00Oomoo`07Ool00`00Oomoo`0;Ool00`00Oomoo`03Ool0100?Ool000003goo20005Woo
0P004goo00<03gooOol00Woo100000=oo`00Ool0>Goo00<03gooOol0:Woo2P0000=oo`0000002goo
00<03gooOol00Woo200067oo0`00000iOol00`00Oomoo`0<Ool00`00Oomoo`0MOol00`00Oomoo`0>
Ool2000017oo000000002goo00<03gooOol0=goo0P000goo0P0017oo00<03gooOol02Goo0P001Woo
0P004Goo00<007ooOol01goo00<007ooOol02goo00<007ooOol00goo00@03gooOol000moo`P001Io
o`8001=oo`0300moogoo009oo`H003Yoo`0300moogoo02Yoo`<00003Ool0000000@00003Ool00000
00]oo`0300moogoo009oo`P001Yoo`400000>Goo00<007ooOol037oo00<007ooOol07Goo00<007oo
Ool03Woo0P0000Aoo`00000000]oo`0300moogoo02Ioo`03001oogoo00ioo`8000=oo`8000Aoo`03
00moogoo00Uoo`8000Ioo`80015oo`03001oogoo00Moo`03001oogoo00]oo`03001oogoo00=oo`04
00moogoo000?Ool300000goo00000002000FOol2000COol00`0?Oomoo`02Ool6000jOol00`0?Oomo
o`0ZOol300000goo00000008000:Ool00`0?Oomoo`02Ool8000JOol1000003Uoo`03001oogoo00ao
o`03001oogoo01eoo`03001oogoo00ioo`80009oo`8000]oo`0300moogoo02Ioo`03001oogoo00io
o`03001oogoo00=oo`03001oogoo009oo`0300moogoo00Uoo`8000Ioo`80015oo`03001oogoo00Mo
o`03001oogoo00]oo`03001oogoo00=oo`0400moogoo000?Ool20002Ool4000FOol2000COol00`0?
Oomoo`02Ool6000jOol00`0?Oomoo`0ZOol20002Ool:000:Ool00`0?Oomoo`02Ool8000JOol10000
03Uoo`03001oogoo00Qoo`05001oo`00Ool0000OOol00`00Oomoo`0>Ool20002Ool2000;Ool00`0?
Oomoo`0UOol2000@Ool00`00Oomoo`03Ool20003Ool00`0?Oomoo`09Ool20006Ool2000AOol00`00
Oomoo`07Ool00`00Oomoo`0;Ool00`00Oomoo`03Ool0100?Oomoo`003goo0P000Woo10005Woo0P00
00=oo`00Ool047oo00<03gooOol00Woo1P007Woo0P006Woo00<03gooOol0:Woo0P000Woo2P002Woo
00<03gooOol00Woo20006Woo0@00000iOol00`00Oomoo`08Ool01@00Ool007oo000077oo00@007oo
Ool0011oo`04001oogooOol2000;Ool00`0?Oomoo`0UOol2000@Ool00`00Oomoo`03Ool20003Ool0
0`0?Oomoo`08Ool20007Ool2000AOol00`00Oomoo`07Ool00`00Oomoo`0;Ool00`00Oomoo`03Ool0
100?Oomoo`003Goo00@007oo0000009oo`@001Ioo`@0015oo`0300moogoo009oo`H001ioo`8001Yo
o`0300moogoo02Yoo`80009oo`03001oo`0000L000Yoo`0300moogoo009oo`P001Yoo`400000>Goo
00<007ooOol027oo00D007oo001oo`0001aoo`04001oogoo000@Ool01000Oomoogoo0P002goo00<0
3gooOol09Goo0P0047oo00<007ooOol00goo0P000goo00<03gooOol027oo0P0027oo00<007ooOol0
3goo00<007ooOol01goo00<007ooOol02goo00<007ooOol00goo00@03gooOol000eoo`04001oo`00
0002Ool4000EOol5000AOol00`0?Oomoo`02Ool6000NOol2000JOol00`0?Oomoo`0ZOol20004Ool9
0009Ool00`0?Oomoo`02Ool9000IOol1000003Uoo`03001oogoo00Qoo`05001oo`00Ool0000LOol0
1000Oomoo`0047oo00@007ooOomoo`8000]oo`0300moogoo02Eoo`80011oo`03001oogoo00=oo`80
00=oo`0300moogoo00Qoo`8000Qoo`03001oogoo00moo`03001oogoo00Moo`03001oogoo00]oo`03
001oogoo00=oo`0400moogoo000=Ool40002Ool5000DOol5000AOol00`0?Oomoo`02Ool6000NOol2
000JOol00`0?Oomoo`0ZOol20004Ool90009Ool00`0?Oomoo`02Ool9000IOol1000003Uoo`03001o
ogoo00Qoo`03001oo`00008001aoo`04001oogoo000?Ool20003Ool2000;Ool00`0?Oomoo`0<Ool0
0`00Oomoo`0COol00`00Oomoo`02000@Ool00`00Oomoo`03Ool20003Ool00`0?Oomoo`08Ool20008
Ool2000@Ool00`00Oomoo`07Ool00`00Oomoo`0;Ool00`00Oomoo`03Ool0100?Oomoo`003Goo1000
0Woo1@0057oo1@004Goo00<03gooOol00Woo1P007Woo0P006Woo00<03gooOol0:Woo0P0017oo2@00
2Goo00<03gooOol00Woo100000=oo`0000000P006Goo0@00000iOol00`00Oomoo`08Ool00`00Ool0
00020004Ool00`00Oomoo`0EOol01000Oomoo`003goo0P000goo0P002goo00<03gooOol02goo0`00
57oo00<007ooOol00P0047oo00<007ooOol00goo0P000goo00<03gooOol027oo0P0027oo0P0047oo
00<007ooOol01goo00<007ooOol02goo00<007ooOol00goo00@03gooOol000eoo`@000=oo`D001=o
o`D0015oo`0300moogoo009oo`H001ioo`8001Yoo`0300moogoo02Yoo`8000Aoo`T000Uoo`0300mo
ogoo009oo`<0009oo`@001Uoo`400000>Goo00<007ooOol027oo00<007oo00000P0017oo00<007oo
Ool05Goo00@007ooOol000moo`8000Aoo`8000Yoo`0300moogoo00]oo`<001Aoo`03001oogoo0080
011oo`03001oogoo00=oo`8000=oo`0300moogoo00Qoo`8000Qoo`80011oo`03001oogoo00Moo`03
001oogoo00]oo`03001oogoo00=oo`0400moogoo000=Ool40003Ool5000BOol6000AOol0100?Oomo
ogoo1`007Woo0P006Woo00<03gooOol0:Woo0P001Woo1`002Goo00<03gooOol00Woo0`000Woo1000
6Goo0@00000iOol00`00Oomoo`08Ool70002Ool00`00Oomoo`0EOol01000Oomoo`003goo0P0017oo
0P002Woo00<03gooOol02goo0`0057oo00<007ooOol00P0047oo00<007ooOol017oo00D007ooOomo
o`0?00Yoo`8000Qoo`80011oo`03001oogoo00Moo`03001oogoo00]oo`03001oogoo00=oo`0400mo
ogoo000=Ool40004Ool4000BOol6000AOol0100?Oomoogoo0P0000=oo`0000000P007Woo0`006Goo
00<03gooOol0:Woo0P001Woo0`0000Aoo`00000000Uoo`0400moogooOol2000017oo001oogoo1000
6Goo0@00000iOol00`00Oomoo`08Ool70002Ool00`00Oomoo`0EOol00`00Oomoo`02000>Ool00`00
Oomoo`03Ool2000:Ool00`0?Oomoo`0;Ool3000DOol00`00Oomoo`02000@Ool00`00Oomoo`04Ool0
1@00Oomoogoo00l02Woo0P0027oo0`003Woo0P002Goo00<007ooOol02Woo0P001Goo00@03gooOol0
00eoo`@000Aoo`@0019oo`H0015oo`0400moogooOol200000goo00000002000MOol4000IOol00`0?
Oomoo`0ZOol20006Ool300000goo000000020008Ool0100?Oomoogoo0P0000Aoo`00Oomoo`@001Uo
o`400000>Goo0`0027oo200000=oo`00Ool05Goo0`0000=oo`0000003Woo00<007ooOol00goo0P00
2Woo00<03gooOol02goo0`004goo0P000Woo0P0047oo00<007ooOol017oo00D007ooOomoo`0?00Yo
o`03001oogoo00Moo`<000ioo`8000Uoo`03001oogoo00Yoo`8000Eoo`0400moogoo000=Ool40004
Ool01000Ool000004Woo1P004Goo00@03gooOomoo`800003Ool007oo008001eoo`@001Uoo`0300mo
ogoo02Yoo`8000Ioo`80009oo`@000Qoo`0400moogooOol200001Woo001oogoo001oo`8001Moo`<0
0000>Goo00<007ooOol027oo200000=oo`00Ool05Goo0`0000=oo`0000003Woo00<007ooOol00goo
0P002Woo00<03gooOol02goo10004Woo0`0000=oo`0000003goo0P001Woo0P000Woo00<03gooOol0
27oo00<007ooOol027oo0P003Woo0P002Goo00<007ooOol02Woo0P001Goo00@03gooOol000eoo`<0
00Eoo`04001oo`00000BOol6000AOol0100?Oomoogoo0P0000=oo`00Ool00P003goo00<007ooOol0
2goo10006Goo00<03gooOol0:Woo0P001Woo00<007ooOol00Woo0`0027oo00@03gooOomoo`8000Ao
o`03001oo`00008001Qoo`400000>Goo00<007ooOol027oo200000=oo`00Ool05Goo1P003Woo00<0
07ooOol00goo0P002Woo00<03gooOol02goo10004Woo0`0000=oo`0000003goo0P001Woo0P000Woo
00<03gooOol027oo00<007ooOol027oo0P003Woo0P002Goo00<007ooOol02Woo0P001Goo00@03goo
Ool000eoo`<000Eoo`04001oo`00000BOol6000AOol0100?Oomoogoo0P0000=oo`00Ool00P003goo
00<007ooOol02Woo1@006Goo00<03gooOol0:Woo0P001Woo00<007ooOol00Woo0`0027oo00@03goo
Oomoo`8000Aoo`03001oo`00008001Qoo`400000>Goo00<007ooOol027oo200000=oo`00Ool05Goo
1P003Woo00<007ooOol00goo0P002Woo00<03gooOol02goo1@004Goo1P003goo0P001Woo0P000Woo
00<03gooOol027oo00<007ooOol027oo0P003Woo0P002Goo00<007ooOol02Woo0P001Goo00@03goo
Ool000eoo`8000Ioo`04001oo`00000BOol6000AOol0100?Oomoogoo0P000goo0P003goo00<007oo
Ool02Woo1@006Goo00<03gooOol0:Woo0P001Woo00<007ooOol00Woo0`0027oo00@03gooOomoo`80
00Aoo`05001oo`00Ool0000HOol1000003Uoo`03001oogoo00Qoo`P00003Ool007oo01Eoo`H000io
o`03001oogoo00=oo`8000Yoo`0300moogoo00]oo`D0015oo`H000moo`8000Ioo`80009oo`0300mo
ogoo00Qoo`03001oogoo00Qoo`8000ioo`8000Uoo`03001oogoo00Yoo`8000Eoo`0400moogoo000=
Ool20006Ool01000Ool000004Woo1P004Goo00@03gooOomoo`8000=oo`8000moo`03001oogoo00Yo
o`D001Uoo`0300moogoo02Uoo`03001oo`0000Ioo`03001oogoo009oo`<000Qoo`0400moogooOol2
0004Ool01@00Ool007oo000067oo0@00000iOol00`00Oomoo`08Ool800000goo001oo`0EOol6000=
Ool20005Ool2000:Ool00`0?Oomoo`0;Ool5000AOol7000>Ool20007Ool01000Oomoo`0?2Woo00<0
07ooOol027oo0P003Woo0P002Goo00<007ooOol02Woo0P001Goo00@03gooOol000eoo`8000Ioo`04
001oo`00000BOol400000goo0000000@Ool00`0?Oomoo`030003Ool2000?Ool00`00Oomoo`0:Ool5
000IOol00`0?Oomoo`0YOol00`00Ool00006Ool00`00Oomoo`02Ool30008Ool0100?Oomoogoo0P00
17oo00D007oo001oo`0001Qoo`400000>Goo00<007ooOol027oo2P005Woo1P003Goo0P001Goo0P00
2Woo00<03gooOol02goo1@004Goo1`003Woo0P001goo00@007ooOol03`Uoo`8000Yoo`8000Aoo`03
001oogoo00Moo`8000Uoo`03001oogoo00Yoo`03001oogoo00Aoo`0400moogoo000=Ool20006Ool0
1000Ool000004Woo100000=oo`00000047oo00<03gooOol00`000goo0P003goo00<007ooOol02Woo
1P0067oo00<03gooOol0:Goo00<007oo00002goo0`0027oo00@03gooOomoo`8000Aoo`05001oo`00
Ool0000HOol1000003Uoo`03001oogoo00Qoo`D00003Ool00000008001Eoo`L000eoo`8000Eoo`80
00Yoo`0300moogoo00]oo`D0015oo`L000ioo`8000Moo`04001oogoo00l9Ool2000:Ool30003Ool0
0`00Oomoo`07Ool00`00Oomoo`08Ool00`00Oomoo`0:Ool00`00Oomoo`04Ool0100?Oomoo`003Goo
0P001Woo00@007oo0000019oo`8000=oo`80011oo`0300moogoo00<000=oo`8000moo`03001oogoo
00Yoo`H001Qoo`0300moogoo02Uoo`03001oogoo00aoo`8000Qoo`0400moogooOol20004Ool01@00
Ool007oo000067oo0@00000iOol00`00Oomoo`08Ool500000goo00000002000EOol7000=Ool20005
Ool2000:Ool00`0?Oomoo`0;Ool5000?Ool00`00Ool00006000>Ool20007Ool01000Oomoo`0?2Goo
0P002goo0P000goo00<007ooOol01goo00<007ooOol027oo00<007ooOol02Woo00<007ooOol017oo
00@03gooOol000eoo`8000Qoo`80019oo`8000=oo`80011oo`0300moogoo00<000=oo`<000ioo`03
001oogoo00Yoo`H001Qoo`0300moogoo02Uoo`03001oogoo00aoo`8000Qoo`0400moogooOol20004
Ool01@00Ool007oo000067oo0@00000iOol00`00Oomoo`08Ool500000goo00000002000EOol7000=
Ool20005Ool2000:Ool00`0?Oomoo`0;Ool6000>Ool00`00Ool00006000>Ool20007Ool01000Oomo
o`0?2Goo0P002goo0P000goo00<007ooOol01goo00<007ooOol027oo00<007ooOol02Woo00<007oo
Ool017oo00@03gooOol000eoo`8000Qoo`80019oo`8000=oo`80011oo`0300moogoo00<000=oo`<0
00ioo`03001oogoo00Yoo`<00003Ool0000001Qoo`0300moogoo02Uoo`03001oogoo00aoo`8000Qo
o`0400moogooOol20004Ool01@00Oomoogoo000067oo0@00000iOol00`00Oomoo`08Ool00`00Ool0
00020002Ool3000EOol7000=Ool20005Ool2000:Ool00`0?Oomoo`0;Ool6000>Ool9000>Ool20007
Ool01000Oomoo`0?2Goo0P002goo0P000goo00<007ooOol01goo00<007ooOol027oo00<007ooOol0
2Woo00<007ooOol017oo00@03gooOol000eoo`8000Qoo`80019oo`8000=oo`80011oo`0300moogoo
00<000=oo`<000ioo`03001oogoo00Yoo`<00003Ool0000001Qoo`0300moogoo02Uoo`03001oogoo
00eoo`03001oogoo00Ioo`0400moogooOol20008Ool00`00Oomoo`0FOol1000003Uoo`03001oogoo
00Yoo`<0009oo`<001Eoo`L000eoo`03001oogoo00Aoo`8000Yoo`0300moogoo00Yoo`P000eoo`T0
00ioo`03001oogoo00Ioo`04001oogoo00l9Ool2000;Ool20003Ool00`00Oomoo`07Ool00`00Oomo
o`08Ool00`00Oomoo`0:Ool00`00Oomoo`04Ool0100?Oomoo`003Goo0P0027oo0P004Woo0P000goo
0P0047oo00<03gooOol00`000goo0`003Woo00<007ooOol02Woo0`0000=oo`0000004goo00<007oo
Ool00Woo00<03gooOol0:Goo00<007ooOol03Goo00<007ooOol01Woo00@03gooOomoo`8000Qoo`80
01Moo`400000>Goo00<007ooOol02Woo0`000Woo0`005Goo1`003Goo00<007ooOol01Goo00<007oo
Ool027oo00<03gooOol02Woo20003Goo1@0000=oo`0000000P003Goo00<007ooOol01Woo00@007oo
Ool03`Uoo`8000]oo`8000=oo`03001oogoo00Moo`03001oogoo00Uoo`03001oogoo00Uoo`03001o
ogoo00Aoo`0400moogoo000=Ool20008Ool2000BOol20003Ool2000@Ool00`0?Oomoo`030003Ool3
000>Ool00`00Oomoo`0:Ool300000goo0000000COol00`00Oomoo`02Ool00`0?Oomoo`0YOol00`00
Oomoo`0=Ool20007Ool0100?Oomoogoo0P0027oo0P005goo0@00000iOol00`00Oomoo`0:Ool00`00
Ool00002Ool3000EOol7000=Ool00`00Oomoo`05Ool00`00Oomoo`08Ool00`0?Oomoo`09Ool40000
0goo00000002000=Ool500000goo00000002000=Ool00`00Oomoo`06Ool01000Oomoo`0?2Goo0P00
2goo0P000goo00<007ooOol01goo00<007ooOol02Goo00<007ooOol02Goo00<007ooOol017oo00@0
3gooOol000eoo`8000Qoo`80015oo`<000=oo`80011oo`0300moogoo00<000Aoo`8000ioo`03001o
ogoo00Yoo`<0009oo`03001oogoo015oo`03001oogoo009oo`0300moogoo02Uoo`03001oogoo00io
o`03001oogoo00Eoo`0400moogooOol20008Ool2000GOol1000003Uoo`<000Yoo`03001oo`00009o
o`<001=oo`03001oo`0000H000eoo`03001oogoo00Eoo`03001oogoo00Qoo`0300moogoo00Uoo`@0
0003Ool00000008000eoo`D00003Ool00000008000eoo`03001oogoo00Ioo`04001oogoo00l9Ool2
000;Ool400000goo001oo`08Ool00`00Oomoo`09Ool00`00Oomoo`09Ool00`00Oomoo`04Ool0100?
Oomoo`003Goo0P0027oo0P0047oo10000goo0P0047oo00<03gooOol00`0017oo0P0037oo00<007oo
00002goo10000Woo00<007ooOol04Goo00<007ooOol00Woo00<03gooOol0:Goo00<007ooOol03Woo
00<007ooOol01Goo00@03gooOomoo`8000Qoo`8001Eoo`<00000>Goo00<007ooOol02Woo00<007oo
00000Woo0`004goo00<007oo00001`0037oo00<007ooOol01Goo00<007ooOol027oo00<03gooOol0
2Goo100000=oo`0000000P003Goo1@0000=oo`0000000P0037oo0P0027oo00@007ooOol03`Uoo`80
00]oo`H000Uoo`03001oogoo00Uoo`03001oogoo00Uoo`03001oogoo00Aoo`0400moogoo000=Ool2
0008Ool2000?Ool50003Ool2000@Ool00`0?Oomoo`030004Ool2000<Ool00`00Ool0000;Ool40002
Ool00`00Oomoo`0AOol00`00Oomoo`02Ool00`0?Oomoo`0YOol00`00Oomoo`0>Ool00`00Oomoo`05
Ool00`0?Oomoo`030008Ool2000GOol1000003Uoo`03001oogoo00aoo`03001oogoo00<001=oo`03
001oo`0000L000aoo`03001oogoo00Eoo`03001oogoo00Qoo`0300moogoo00Qoo`D0009oo`<000eo
o`D00005Ool00000Ool0000<Ool20008Ool01000Oomoo`0?2Goo00<007ooOol02Woo1P002Goo00<0
07ooOol02Goo00<007ooOol02Goo00<007ooOol017oo00<03gooOol00P0037oo0P0027oo0P003goo
1@000goo0P0047oo00H03goo000007oo0004Ool2000<Ool3000;Ool40002Ool00`00Oomoo`0AOol0
0`00Oomoo`02Ool00`0?Oomoo`0YOol00`00Oomoo`0>Ool00`00Oomoo`05Ool00`0?Oomoo`030008
Ool2000GOol1000003Uoo`03001oogoo00aoo`03001oogoo00<001=oo`03001oo`0000L000]oo`80
00Moo`03001oogoo00Qoo`0300moogoo00Qoo`D0009oo`<000]oo`03001oo`0000@00004Ool007oo
Ool2000;Ool20008Ool01000Oomoo`0?27oo0P0037oo1P002Goo00<007ooOol02Goo00<007ooOol0
27oo0P001Woo00D03gooOomoo`0000]oo`<000Qoo`8000moo`D000=oo`<000moo`0600moo`00001o
o`0017oo0P0037oo0`0000=oo`00Ool027oo0`000goo00<007ooOol04Goo00<007ooOol00Woo00<0
3gooOol0:7oo0P0047oo00<007ooOol01Goo00<03gooOol00`002Goo00<007ooOol05Goo0@00000i
Ool00`00Oomoo`0<Ool00`00Oomoo`03000COol00`00Ool00007000;Ool20007Ool00`00Oomoo`08
Ool00`0?Oomoo`08Ool50002Ool3000;Ool00`00Ool00004000017oo001oogoo0P002goo0P0027oo
00@007ooOol03`Qoo`8000aoo`H000Uoo`03001oogoo00Uoo`03001oogoo00Qoo`8000Ioo`0500mo
ogooOol0000;Ool30008Ool2000?Ool50003Ool4000>Ool01P0?Ool00000Ool000Aoo`8000aoo`<0
0003Ool007oo00Qoo`<000=oo`80015oo`8000Aoo`0300moogoo02Qoo`80011oo`03001oogoo00Eo
o`0300moogoo00<000Uoo`03001oogoo01Eoo`400000>Goo00<007ooOol037oo00@007ooOomoo`80
015oo`03001oo`0000T000]oo`8000Moo`03001oogoo00Qoo`0300moogoo00Qoo`D0009oo`<000]o
o`03001oo`0000<0009oo`03001oogoo008000]oo`8000Qoo`04001oogoo00l8Ool2000=Ool50009
Ool00`00Oomoo`09Ool00`00Oomoo`08Ool20006Ool01@0?Oomoogoo00002goo0`0027oo0P003goo
1@000goo10003Woo00H03goo000007oo0005Ool00`00Oomoo`0:Ool300000goo001oo`08Ool30003
Ool2000AOol20004Ool00`0?Oomoo`0XOol2000@Ool00`00Oomoo`05Ool00`0?Oomoo`030009Ool0
0`00Oomoo`0EOol1000003Uoo`03001oogoo00aoo`04001oogooOol2000AOol00`00Ool00009000;
Ool20007Ool20009Ool00`0?Oomoo`08Ool50002Ool4000:Ool00`00Ool000030002Ool00`00Oomo
o`02000;Ool00`00Oomoo`07Ool01000Oomoo`0?27oo0P003Woo10002Goo00<007ooOol02Goo00<0
07ooOol027oo0P001Woo00D03gooOomoo`0000]oo`8000Uoo`<000ioo`D000=oo`@00003Ool007oo
00]oo`0600moo`00001oo`001Goo00<007ooOol02Woo0`0000=oo`00Ool027oo0`000goo0P004Goo
0P0017oo00<03gooOol0:7oo0P0047oo00<007ooOol01Goo00<03gooOol00`002Goo00<007ooOol0
5Goo0@00000iOol00`00Oomoo`0<Ool01000Oomoogoo0P000Woo00<007ooOol027oo00H007oo001o
o`00Ool:000;Ool20008Ool00`00Oomoo`07Ool00`0?Oomoo`06Ool00`00Ool000040002Ool4000:
Ool00`00Ool000020003Ool01@00Oomoogoo00002goo00<007ooOol01goo0P0000=oo`0?Ool01goo
0P003Woo10002Goo00<007ooOol02Goo00<007ooOol027oo0P001Woo00D03gooOomoo`0000]oo`80
00Uoo`<000ioo`D000=oo`@00003Ool007oo00]oo`0600moo`00001oo`001Goo0P002goo1@0017oo
00<007ooOol00Woo0`000goo0P0047oo0`0017oo00<03gooOol0:7oo0P0047oo00<007ooOol01Goo
00<03gooOol00`002Goo00<007ooOol05Goo0@00000iOol00`00Oomoo`0<Ool01000Oomoogoo0P00
0Woo00<007ooOol027oo00H007oo001oo`00Ool40002Ool5000:Ool20008Ool00`00Oomoo`07Ool0
0`0?Oomoo`06Ool00`00Ool000040002Ool4000:Ool00`00Ool000020007Ool2000:Ool00`00Oomo
o`07Ool200000goo00moo`07Ool2000>Ool400000goo001oo`06Ool00`00Oomoo`09Ool00`00Oomo
o`08Ool20006Ool01@0?Oomoogoo00002goo0P002Goo0`003Woo100017oo100000=oo`00Ool02goo
00@03`00000000Moo`8000]oo`H000=oo`03001oogoo009oo`8000Aoo`80011oo`@000=oo`0300mo
ogoo02Qoo`80011oo`03001oogoo00Eoo`0300moogoo00<000Uoo`03001oogoo01Eoo`400000>Goo
00<007ooOol047oo0`0000=oo`00Ool02Goo0`0000=oo`00Ool010000Woo1@002Woo0P0027oo00<0
07ooOol01goo00<03gooOol01Woo100000=oo`0000000Woo10002Woo1@001goo0P002Woo00<007oo
Ool01goo0P0000=oo`0?Ool01goo0P003Woo1P001Woo0P002goo00<007ooOol027oo0P001Woo00D0
3gooOomoo`0000]oo`8000Uoo`<000ioo`<000Eoo`@00003Ool007oo00]oo`0400l000000007Ool2
000;Ool60003Ool00`00Oomoo`02Ool20004Ool2000@Ool40003Ool00`0?Oomoo`0XOol2000@Ool0
0`00Oomoo`05Ool00`0?Oomoo`02000:Ool00`00Oomoo`0EOol1000003Uoo`03001oogoo011oo`<0
0003Ool007oo00Uoo`<00003Ool007oo00@0009oo`D000Yoo`8000Qoo`03001oogoo00Moo`0300mo
ogoo00Ioo`@00003Ool0000000Aoo`8000Yoo`@000Qoo`8000Yoo`03001oogoo00Moo`800003Ool0
3goo00Moo`8000ioo`H000Ioo`8000]oo`03001oogoo00Qoo`8000Ioo`0500moogooOol0000;Ool2
0009Ool3000>Ool30005Ool400000goo001oo`05Ool00`00Oomoo`03Ool0100?000000001goo0P00
2goo1P000goo00@007ooOomoo`<000Aoo`80011oo`@000=oo`0300moogoo02Qoo`80011oo`03001o
ogoo00Eoo`0300moogoo008000Yoo`03001oogoo01Eoo`400000>Goo00<007ooOol047oo0`0000=o
o`00Ool02Goo0`0000=oo`00Ool010000goo1@0027oo0`0027oo00<007ooOol01goo00<03gooOol0
1Woo100000=oo`00000017oo0P002Woo100027oo0P002Woo00<007ooOol01goo0P0000=oo`0?Ool0
1goo0P003Woo1P001Woo0P002goo00<007ooOol027oo0P001Woo00D03gooOomoo`0000]oo`8000Uo
o`<000ioo`<000Ioo`<00004Ool007oo0004Ool00`00Oomoo`03Ool0100?000000001goo0P002goo
1P000goo0P000Woo0`0017oo0P0047oo10000goo00<03gooOol0:7oo0P0047oo0P001Woo00<03goo
Ool00P002Woo00<007ooOol05Goo0@00000iOol00`00Oomoo`0AOol200000goo001oo`09Ool30000
0goo000000040003Ool50008Ool30008Ool00`00Oomoo`07Ool00`0?Oomoo`06Ool400000goo0000
0004Ool2000:Ool40008Ool2000:Ool00`00Oomoo`08Ool00`00Ool03`08Ool2000>Ool60006Ool2
000;Ool00`00Oomoo`08Ool20006Ool01@0?Oomoogoo00002goo0P002Goo0`003Woo0`001goo1000
00=oo`00Ool00goo00<007ooOol00goo00@03`00000000Moo`8000]oo`H000=oo`800003Ool00000
008000Aoo`80011oo`@000=oo`0300moogoo029oo`<000=oo`03001oogoo011oo`03001oogoo00Ao
o`0300moogoo008000Yoo`03001oogoo01Eoo`400000>Goo0`004Goo0P0000=oo`00Ool02Goo0`00
00=oo`0000000`0017oo1P001goo0`0027oo00<007ooOol01goo00<03gooOol01Woo100000=oo`00
Ool017oo0`002Goo100027oo0P002Woo00<007ooOol027oo00<007oo00l027oo00<007ooOol03Goo
1P001Woo0P002goo00<007ooOol027oo00<007ooOol01Goo00D03gooOomoo`0000]oo`03001oogoo
00Qoo`<000ioo`8000Qoo`H000Aoo`03001oogoo00=oo`0400l000000007Ool2000:Ool70003Ool2
00000goo000000020004Ool2000?Ool50003Ool00`0?Oomoo`0BOol00`00Oomoo`0;Ool50003Ool0
0`00Oomoo`0@Ool00`00Oomoo`04Ool00`0?Oomoo`02000:Ool2000DOol3000003Uoo`03001oogoo
015oo`800003Ool007oo00Uoo`<00003Ool0000000<000Aoo`L000Ioo`<000Qoo`8000Qoo`0300mo
ogoo00Ioo`<0009oo`03001oogoo00=oo`<000Uoo`@000Qoo`8000Yoo`03001oogoo00Qoo`03001o
o`0?00Moo`8000moo`03001oo`0000<000Ioo`8000]oo`03001oogoo00Qoo`03001oogoo00Eoo`05
00moogooOol0000;Ool00`00Oomoo`08Ool00`00Ool0000>Ool20008Ool60002Ool00`00Ool00005
Ool0100?000000001goo0P002Woo300000=oo`0000000P001Goo00<007ooOol03Goo1@000goo00<0
3gooOol04Woo00<007ooOol02goo1@000goo00<007ooOol047oo00<007ooOol017oo00@03gooOol0
00]oo`8001Ioo`400000>Goo00<007ooOol04Goo0P0000=oo`00Ool02Goo2@0017oo00<007oo0000
10001Woo0P002Goo0P0027oo00<03gooOol01Woo0`000Woo00<007ooOol017oo0`0027oo100027oo
0P002Woo00<007ooOol027oo00<007oo00l01goo00<007ooOol03Woo00<007oo00000`001Woo00<0
07ooOol02Woo00<007ooOol027oo00<007ooOol01Goo00D03gooOomoo`0000]oo`03001oogoo00Qo
o`03001oo`0000ioo`8000Qoo`H0009oo`04001oo`000004Ool0100?0000000027oo00<007ooOol0
27oo4@001Goo00<007ooOol03Goo1@000goo00<03gooOol04Woo00<007ooOol02goo1@000goo00<0
07ooOol047oo0P001Goo00@03gooOol000]oo`8001Ioo`400000>Goo00<007ooOol04Goo0P0000=o
o`00Ool02Goo2@0017oo00<007oo000010001Woo0P002Goo0P0027oo00<03gooOol01Woo0`000Woo
00<007ooOol017oo0`0027oo100027oo0P002Woo00<007ooOol027oo00<007oo00l01goo00<007oo
Ool03Woo00<007ooOol0100017oo0P0037oo00<007ooOol027oo00<007ooOol01Goo00D03gooOomo
o`0000]oo`03001oogoo00Qoo`03001oo`0000eoo`<000Qoo`L00003Ool007oo008000Aoo`0400l0
00000008Ool00`00Oomoo`08OolA0005Ool00`00Oomoo`0=Ool50003Ool00`0?Oomoo`0BOol00`00
Oomoo`0:Ool60003Ool00`00Oomoo`0@Ool20005Ool0100?Oomoo`002goo0P005Woo0@00000iOol0
0`00Oomoo`0AOol200000goo001oo`09Ool90004Ool00`00Ool000040006Ool20009Ool20008Ool0
0`0?Oomoo`06Ool30002Ool00`00Oomoo`04Ool30008Ool30009Ool2000:Ool00`00Oomoo`08Ool0
0`00Ool03`07Ool00`00Oomoo`0AOol40004Ool2000<Ool00`00Oomoo`08Ool00`00Oomoo`05Ool0
1@0?Oomoogoo00002goo00<007ooOol027oo00<007oo00003Goo0`0027oo1`0000=oo`0000000P00
17oo00<03`0000002Goo00<007ooOol027oo4@001Goo00<007ooOol03Goo0P0000=oo`0000000goo
00<03gooOol04Woo00<007ooOol02Woo1P0000Aoo`00Ool0019oo`8000Eoo`0400moogoo000<Ool2
000EOol1000003Uoo`03001oogoo015oo`800003Ool007oo00Uoo`T000Aoo`03001oo`0000@000Io
o`8000Uoo`8000Qoo`0300moogoo00Ioo`<000Uoo`<000Qoo`<000Uoo`8000Yoo`03001oogoo00Qo
o`03001oo`0?00Moo`03001oogoo015oo`@000Aoo`8000aoo`03001oogoo00Qoo`03001oogoo00Eo
o`0500moogooOol0000;Ool00`00Oomoo`08Ool00`00Ool0000;Ool50008Ool700000goo00000002
0004Ool00`0?00000009Ool00`00Oomoo`08OolA0005Ool00`00Oomoo`0=Ool200000goo00000003
Ool00`0?Oomoo`0=Ool01000Oomoogoo0`002goo0P0000Qoo`00000007oo001oo`004Woo0P001Goo
00@03gooOol000aoo`8001Eoo`400000>Goo00<007ooOol04Goo0P0000=oo`00Ool00Woo00<007oo
Ool017oo1P0000=oo`00000017oo00<007ooOol010001Woo0P002Goo0P0027oo00<03gooOol01Woo
0`002Goo0`001Woo1@002Goo0P002Woo00<007ooOol027oo00<007oo00l01goo00<007ooOol04Goo
00@007oo000000Aoo`8000aoo`8000Uoo`03001oogoo00Eoo`0500moogooOol0000;Ool00`00Oomo
o`08Ool00`00Ool0000;Ool50008Ool<0004Ool00`0?00000009Ool00`00Oomoo`08Ool01@00Ool0
0000Ool030001Goo00<007oo00003Goo0P0000=oo`0000000goo00<03gooOol03Goo0P0000=oo`00
00000P002goo0P0000=oo`0000000`0000=oo`00Ool04Goo0P001Goo00@03gooOol000aoo`8001Eo
o`400000>Goo00<007ooOol04Goo0P0000=oo`00Ool00Woo00<007ooOol017oo1@000goo00<007oo
Ool00Woo00<007ooOol010001Woo0P002Woo00<007ooOol01Woo00<03gooOol01Woo0P002Woo0`00
1Woo1@002Goo0P002Goo0P002Woo00<007oo00l01goo00<007ooOol04Goo00@007oo000000Aoo`80
00aoo`8000Qoo`8000Moo`0500moogooOol0000;Ool00`00Oomoo`0:Ool00`00Oomoo`09Ool50008
Ool<0004Ool00`0?00000009Ool00`00Oomoo`08Ool01@00Ool00000Ool030001Goo00<007oo0000
37oo0`0000=oo`0000000goo00<03gooOol03Goo0P0000=oo`0000000P002goo0P000Woo1P004Woo
0P001Goo00@03goo000000aoo`8001Eoo`4000005Goo0P001goo0P002Goo1@002goo00<007ooOol0
4Goo10000goo00<007ooOol017oo1@000goo00<007ooOol00Woo00<007ooOol010001Woo0P002Woo
0P001goo00<03gooOol01Woo0P002Woo0`001Woo10002Woo0P002Goo0P002Woo00<007oo00l01goo
00<007ooOol04Goo00@007oo000000Aoo`8000aoo`8000Qoo`8000Moo`0500moogooOol0000;Ool0
0`00Oomoo`0:Ool00`00Oomoo`08Ool60009Ool;0004Ool00`0?00000009Ool00`00Oomoo`08Ool0
1@00Ool00000Ool030001Goo00<007oo000037oo0`0000=oo`0000000goo00<03gooOol03Goo0P00
00=oo`0000000P002goo0P000Woo1P004Woo0P001Goo00@03goo000000aoo`8001Eoo`4000005Goo
0P001goo0P0027oo1`002Woo00<007ooOol04Goo10000goo00<007ooOol017oo1@000goo00<007oo
Ool00Woo00<007ooOol010001Woo0P002Woo0P001goo00<03gooOol01Woo0P002Woo0`001Woo1000
2goo00<007ooOol01goo0P002Woo00<007oo00l01goo00<007ooOol04Goo00@007ooOol000Aoo`80
00aoo`8000Qoo`8000Moo`0500moogooOol0000:Ool2000<Ool00`00Oomoo`08Ool60009Ool;0004
Ool00`0?00000009Ool00`00Oomoo`08Ool01@00Oomoo`00Ool030001Goo00<007oo000037oo0`00
00Aoo`000000009oo`0300moogoo00eoo`800003Ool00000008000Yoo`<0009oo`H0019oo`8000Eo
o`0400moo`00000<Ool2000EOol1000001Eoo`8000Moo`8000Qoo`8000=oo`8000Yoo`03001oogoo
015oo`@000=oo`03001oogoo0080009oo`D000=oo`03001oogoo009oo`03001oogoo00@000Eoo`<0
00Yoo`8000Moo`0300moogoo00Eoo`<000Yoo`<000Ioo`@000]oo`03001oogoo00Moo`8000Yoo`80
000300moogoo00Eoo`03001oogoo015oo`04001oogoo0004Ool2000<Ool20008Ool20007Ool01@0?
Oomoogoo00002Woo0P0037oo00<007ooOol027oo1P002Goo30000goo00<03`0000002Goo00<007oo
Ool01goo0P000Woo00<007oo00002`001Goo00<007oo000037oo0`0000Aoo`000000009oo`0300mo
ogoo00]oo`05001oo`00001oo`04000:Ool30002Ool6000BOol20005Ool0100?Ool0000037oo0P00
5Goo0@00000EOol20007Ool20007Ool20005Ool20009Ool00`00Oomoo`0AOol40003Ool00`00Oomo
o`020002Ool50003Ool00`00Oomoo`02Ool00`00Oomoo`040005Ool3000:Ool20007Ool00`0?Oomo
o`05Ool3000:Ool40005Ool4000;Ool00`00Oomoo`07Ool2000:Ool200000`0?Oomoo`05Ool00`00
Oomoo`0AOol01000Oomoo`0017oo0P003Goo00<007ooOol01Woo0P001goo00D03gooOomoo`0000Yo
o`03001oogoo00]oo`03001oogoo00Moo`L000Uoo``000=oo`0300l0000000Uoo`03001oogoo00Mo
o`80009oo`03001oo`0000/000Ioo`8000aoo`<00004Ool007oo0002Ool00`0?Oomoo`0;Ool40000
0goo00000002000:Ool30002Ool6000BOol20005Ool0100?Ool0000037oo0P005Goo0@000004Ool:
0007Ool20008Ool20006Ool20005Ool20009Ool00`00Oomoo`0AOol40003Ool00`00Oomoo`020002
Ool50003Ool00`00Oomoo`02Ool00`00Oomoo`040005Ool3000:Ool200000goo001oo`04Ool00`0?
Oomoo`05Ool3000:Ool40005Ool4000;Ool20008Ool2000:Ool200000`0?Oomoo`05Ool00`00Oomo
o`0DOol00`00Ool00002Ool2000=Ool00`00Oomoo`06Ool20007Ool0100?Oomoogoo0P002Goo00<0
07ooOol02goo00<007oo00001goo1`002Woo2`0000=oo`00Ool00`002Goo0P0027oo0P000Woo00<0
07oo00002P001goo0P0037oo0`0000Aoo`00Ool0009oo`0300moogoo00]oo`T000Yoo`<0009oo`H0
019oo`8000Eoo`0400moo`00000<Ool2000EOol1000001Eoo`8000Qoo`8000Ioo`8000Eoo`8000Uo
o`@001003`@000<03`0300003`0?008000803`<0000300l0000?00803`0300003`0?00803`030000
3`0?00@000D03`<000X03`80000300l0000?00`03`<000X03`@000D03`@000/03`8000P03`8000/0
3`0300003`0?00H03`0300003`0?01@03`<0000400l00000000=00l00`0000l03`0600l2000;00l2
000800l2000=00l00`0000l0000700l7000:00lA000900l2000800l2000200l00`0000l0000:0007
00l2000<00l30000100?00003`00400?2@002P0?0`000P0?1P004P0?0P001`0?0P00300?0P004P0?
1000000EOol20008Ool20006Ool20005Ool20009Ool00`00Oomoo`0AOol40003Ool00`00Oomoo`02
00000goo0000000200000goo001oo`02Ool00`00Oomoo`02Ool00`00Oomoo`040005Ool3000:Ool2
00000goo001oo`04Ool00`0?Oomoo`05Ool3000:Ool40005Ool4000;Ool20008Ool00`00Oomoo`0:
Ool00`0000moo`05Ool2000FOol7000=Ool00`00Oomoo`06Ool20007Ool0100?Oomoogoo0P0027oo
0P003Goo00<007oo00001goo1`002Woo1@0000=oo`0000002@002Goo0P0027oo0P0017oo2`001goo
0P0037oo0P000Woo00<007oo00000Woo00<03gooOol02goo2@002Woo0`000Woo1@0057oo00<007oo
Ool00goo00@03goo000000aoo`8001Eoo`4000005Goo0P002Goo0P001Goo0P001Goo0P002Goo00<0
07ooOol04Goo00@007oo000000=oo`03001oogoo00800003Ool0000000800003Ool007oo00Moo`03
001oogoo00@000Eoo`<000Yoo`800003Ool007oo0080009oo`0400moogoo0004Ool3000;Ool40002
Ool00`00Ool00003000;Ool20008Ool00`00Oomoo`0:Ool00`0000moo`05Ool2000FOol7000=Ool0
0`00Oomoo`06Ool20007Ool0100?Oomoogoo0P0027oo0P003Goo0`001goo1P002goo1@0000=oo`00
00002@002Woo00<007ooOol01Woo0P0017oo2`001goo0P0037oo0P000Woo00<007oo00000Woo00<0
3gooOol02goo2@002Woo0`000Woo1@0057oo00<007ooOol00goo00@03goo000000aoo`8001Eoo`40
00005Goo0P002Goo0P001Goo0P001Goo0P002Goo00<007ooOol04Goo00@007oo000000=oo`03001o
ogoo00L00003Ool007oo00Moo`03001oogoo00@000Eoo`<000Yoo`800003Ool007oo0080009oo`04
00moogoo0004Ool3000;Ool40002Ool00`00Ool00003000;Ool20008Ool00`00Oomoo`0:Ool20006
Ool2000FOol7000=Ool00`00Oomoo`06Ool20007Ool0100?Oomoogoo0P0027oo0P003Goo0`0017oo
00<007oo00001P002goo1@0000=oo`0000002@002Woo00<007ooOol01Goo0`0017oo200000=oo`00
00001goo0P0037oo0P000Woo00<007oo00000Woo00<03gooOol02goo2@001Woo00@007ooOomoo`<0
009oo`D001Aoo`03001oogoo00=oo`0400moo`00000<Ool2000EOol1000001Eoo`8000Yoo`8000Ao
o`8000Eoo`8000Uoo`03001oogoo015oo`04001oo`000003Ool00`00Oomoo`05000017oo001oo`00
27oo00<007ooOol010001Goo0`002Woo0P0000=oo`00Ool00P000Woo00@03gooOol000Aoo`8000ao
o`@0009oo`03001oo`0000<000]oo`8000Qoo`03001oogoo00Yoo`8000Ioo`8001Moo`H000eoo`03
001oogoo00Ioo`8000Moo`0400moogooOol20008Ool2000=Ool30004Ool00`00Ool00006000;Ool5
00000goo00000009000:Ool00`00Oomoo`05Ool30004Ool600001Goo001oogoo00001goo0P0037oo
0P000Woo00<007oo00000Woo00<03gooOol02goo100000=oo`0000000P001Woo00<007ooOol01000
0Woo1@0057oo00<007ooOol00goo00@03`00000000eoo`03001oogoo01=oo`4000004Woo0P0000=o
o`0000002Woo0P0017oo0P001Goo0P002Goo00<007ooOol04Goo00@007oo000000=oo`03001oo`00
00D00004Ool007oo0008Ool00`00Oomoo`040005Ool3000:Ool200000goo001oo`020002Ool0100?
Oomoo`000Woo00@007oo000000eoo`<0009oo`03001oo`0000<000]oo`8000Qoo`03001oogoo00Yo
o`8000Ioo`8001Moo`H000eoo`03001oogoo00Ioo`8000Moo`0400moogooOol20008Ool2000>Ool2
0004Ool700000goo001oo`0:Ool500000goo00000009000:Ool00`00Oomoo`05Ool30005Ool50000
1Goo001oogoo00001goo0P0037oo0P0017oo0P0000=oo`0?Ool037oo100000=oo`0000000P001Woo
00<007ooOol00`0017oo100057oo0P0017oo00<03`0000003Woo00<007ooOol04goo0@00000BOol5
000;Ool20003Ool20005Ool20009Ool00`00Oomoo`0AOol01000Ool000000goo00<007oo00001@00
0goo00<007ooOol01Woo00<007ooOol010001Goo0`002Woo0P0000=oo`00Ool00P000Woo00@03goo
Ool0009oo`@000eoo`@00003Ool007oo00@000]oo`8000Qoo`03001oogoo00Yoo`8000Ioo`8001Mo
o`D000ioo`03001oogoo00Ioo`8000Moo`0400moogooOol20008Ool2000>Ool20003Ool8000=Ool5
00000goo00000008000;Ool00`00Oomoo`05Ool30005Ool500001Goo001oogoo00001goo0P0037oo
0P0017oo0P0000=oo`0?Ool037oo100000=oo`0000000`001Goo00<007ooOol00`0017oo100057oo
0P000goo00@0000?000000ioo`03001oogoo00Qoo`03001oogoo00Qoo`40000057oo0`0037oo00<0
07ooOol00Woo0P000goo0P002Woo00<007ooOol04Goo00@007oo000000=oo`P000=oo`03001oogoo
00Ioo`03001oogoo00@000Eoo`<000Yoo`@00003Ool00000009oo`0400moogoo0002Ool4000=Ool4
00000goo001oo`03000<Ool20008Ool00`00Oomoo`0:Ool20006Ool2000GOol5000>Ool00`00Oomo
o`06Ool20007Ool0100?Oomoogoo0P0027oo0P003Woo0P000goo20003Goo1@0000=oo`0000002000
2goo00<007ooOol01Goo0`001Woo100037oo0P0037oo0P0017oo0P0000=oo`0?Ool037oo0`000Woo
1@001Goo00<007ooOol00`0017oo100057oo0P000goo00@0000?000000ioo`03001oogoo00Qoo`03
001oogoo00Qoo`4000005Goo0P001Goo2@000goo1`002Woo00<007ooOol04Goo00<007oo00000P00
00=oo`0000001@0000=oo`00Ool00Woo00<007ooOol01Woo00<007ooOol010001Goo00<007oo0000
2Woo100000=oo`0000000Woo00@03gooOol0009oo`@000eoo`@00003Ool007oo00<000aoo`8000Mo
o`8000aoo`8000Ioo`8001Moo`D000ioo`03001oogoo00Ioo`8000Moo`0400moogooOol20008Ool2
000>Ool200000goo00000008000=Ool500000goo00000008000;Ool00`00Oomoo`05Ool30006Ool2
00000goo001oo`0;Ool2000;Ool30005Ool00`00Ool03`0=Ool30002Ool50003Ool00`00Ool00002
Ool30005Ool3000DOol20003Ool0100000l000003Woo00<007ooOol027oo00<007ooOol027oo0@00
000EOol20005Ool90004Ool5000;Ool00`00Oomoo`0AOol00`00Ool0000200000goo000000050000
0goo001oo`02Ool00`00Oomoo`06Ool00`00Oomoo`02Ool20004Ool200000goo001oo`09Ool40000
1Woo00000000Ool03`9oo`03001oogoo00@000eoo`@00006Ool007oo001oo`0037oo0P001goo0P00
37oo0P001Woo0P005goo1@003Woo00<007ooOol01Woo0P001goo00<03gooOol00Woo00<007ooOol0
1Woo0P003Woo3@003Goo00D007oo001oo`00009oo`T000]oo`03001oogoo00Eoo`<000Ioo`800003
Ool007oo00]oo`8000]oo`<000Eoo`03001oo`0?00eoo`<0009oo`D000=oo`03001oo`00009oo`80
00Ioo`<001Aoo`8000=oo`@000ioo`03001oogoo00Qoo`03001oogoo00Qoo`400000>Goo00<007oo
Ool04Goo00<007oo00000P0000=oo`0000001@0000=oo`00Ool00Woo00<007ooOol02goo0`000goo
0P0000=oo`00Ool02Goo100000Ioo`00000007oo00l2Ool2000017oo000000003Woo100000Ioo`00
Ool007oo000<Ool20007Ool2000<Ool20006Ool00`00Oomoo`0FOol5000>Ool00`00Oomoo`06Ool2
0007Ool00`0?Oomoo`02Ool00`00Oomoo`06Ool2000>Ool<000@Ool00`00Oomoo`02Ool9000;Ool0
0`00Oomoo`05Ool30007Ool00`00Ool0000<Ool2000;Ool20006Ool00`00Ool03`0=Ool20003Ool5
0003Ool00`00Ool00002Ool20006Ool3000DOol20003Ool4000>Ool00`00Ool00008Ool00`00Oomo
o`08Ool1000003Uoo`03001oogoo015oo`03001oo`0000800003Ool0000000D00003Ool007oo011o
o`<000=oo`8000eoo`<00005Ool00000001oo`05000017oo000000003Woo100000Ioo`00000007oo
000<Ool20007Ool2000<Ool20006Ool00`00Oomoo`0GOol4000>Ool00`00Oomoo`06Ool20007Ool0
0`0?Oomoo`02Ool00`00Oomoo`06Ool00`00Oomoo`0=Ool<000@Ool00`00Oomoo`02Ool9000;Ool0
0`00Oomoo`04Ool40007Ool00`00Ool0000<Ool2000;Ool00`00Oomoo`05Ool00`00Ool03`0;Ool4
0003Ool60002Ool00`00Ool00002Ool20006Ool3000DOol20003Ool4000?Ool20007Ool2000:Ool1
000003Uoo`03001oogoo015oo`03001oo`0000X00003Ool007oo011oo`@0009oo`8000eooa4000io
o`@00006Ool00000001oo`0037oo0P001goo0P0037oo0P001Woo00<007ooOol05goo10003Woo00<0
07ooOol01Woo0P001goo00<03gooOol00Woo00<007ooOol01Woo00<007ooOol03Goo30005Goo2@00
2goo00<007oo000017oo0`0027oo00<007oo000037oo0P002goo00<007ooOol01Goo00<007oo00l0
2goo10000goo1`0000Aoo`00Ool0009oo`8000Moo`03001oogoo01=oo`8000=oo`<0011oo`8000Mo
o`8000Yoo`400000>Goo00<007ooOol04Goo00<007ooOol02P0000=oo`00Ool047oo100000Aoo`00
000000eooa4000ioo`P00003Ool007oo00aoo`03001oogoo00Eoo`8000aoo`8000Ioo`03001oogoo
01Moo`<000moo`03001oogoo00Ioo`8000Moo`0300moogoo009oo`03001oogoo00Ioo`03001oogoo
00eoo``001Ioo`P000]oo`03001oo`0000Aoo`8000Uoo`03001oogoo00aoo`8000]oo`03001oogoo
00Eoo`03001oo`0?00]oo`@000=oo`03001oo`0000@00004Ool000000002Ool20007Ool00`00Oomo
o`0COol20003Ool3000@Ool20007Ool2000:Ool1000003Uoo`<0015oo`03001oogoo00X00003Ool0
07oo011oo`@00004Ool00000000=OolA000>Ool800000goo001oo`0<Ool00`00Oomoo`05Ool2000<
Ool20006Ool00`00Oomoo`0GOol3000?Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`0?Oomoo`02
Ool00`00Oomoo`06Ool00`00Oomoo`0>Ool;000FOol8000;Ool00`00Ool00004Ool20009Ool00`00
Oomoo`0=Ool00`00Oomoo`09Ool00`00Oomoo`05Ool00`00Ool03`0;Ool40005Ool500001Goo0000
0000Ool00`001goo00<007ooOol04goo0P000goo0`0047oo0P001goo0P0027oo0`00000iOol00`00
Oomoo`0AOol00`00Oomoo`0:000COol4000017oo000000003Goo4@003Woo200000=oo`00Ool037oo
0P001Woo0P0037oo0P001Woo00<007ooOol05goo0`003goo00<007ooOol01Woo00<007ooOol01Woo
00<03gooOol00Woo00<007ooOol01Woo00<007ooOol03Woo200000=oo`0000005Woo20002goo0`00
17oo0P002Goo00<007ooOol03Goo00<007ooOol02Goo00<007ooOol01Goo00<007oo00l02goo1000
1Woo100000=oo`0000001@001goo00<007ooOol04goo0P000goo0`0047oo0P001Woo0`002Woo0@00
000iOol00`00Oomoo`0AOol01000Oomoogoo200057oo100000Aoo`00000000eoo`03001oo`0000h0
00ioo`P00003Ool007oo00aoo`8000Ioo`8000aoo`8000Ioo`03001oogoo01Moo`<000moo`03001o
ogoo00Ioo`03001oogoo00Ioo`0300moogoo009oo`03001oogoo00Ioo`03001oogoo00ioo`P00003
Ool0000001Moo`L000aoo`8000Aoo`8001Uoo`03001oogoo00Uoo`03001oogoo00Eoo`03001oo`0?
00]oo`@000Ioo``000Moo`03001oogoo01=oo`8000=oo`<0011oo`800003Ool007oo00=oo`<000Yo
o`400000>Goo00<007ooOol04Goo00@007ooOomoo`P001Aoo`@00003Ool0000000ioo`03001oo`00
00h000ioo`P00003Ool007oo00aoo`<000Eoo`8000aoo`8000Ioo`03001oogoo01Moo`<000moo`03
001oogoo00Ioo`03001oogoo00Ioo`0300moogoo009oo`03001oogoo00Ioo`03001oogoo00ioo`P0
0003Ool0000001Moo`L000aoo`8000Aoo`8001Uoo`03001oogoo00Qoo`8000Moo`03001oo`0?00]o
o`@000Ioo``000Moo`03001oogoo01=oo`8000=oo`<0011oo`800003Ool007oo00=oo`8000]oo`40
0000>Goo00<007ooOol04Goo00@007ooOomoo`P001Aoo`@00003Ool0000000ioo`03001oo`0000h0
00moo`L00003Ool007oo00aoo`<000Eoo`8000aoo`<000Eoo`03001oogoo01Moo`<000moo`03001o
ogoo00Ioo`03001oogoo00Ioo`0300moogoo009oo`03001oogoo00Ioo`03001oogoo00ioo`P00003
Ool0000001Moo`03001oo`0000@000aoo`8000Aoo`8001Uoo`03001oogoo00Qoo`8000Moo`03001o
o`0?00]oo`<000Moo``000Moo`03001oogoo01=oo`8000=oo`<0011oo`800003Ool007oo00=oo`80
00]oo`400000>Goo00<007ooOol04Goo00@007ooOomoo`P001Aoo`@00003Ool0000000ioo`03001o
o`0000d0011oo`L000moo`<000Eoo`8000aoo`<000Aoo`8001Uoo`<000moo`03001oogoo00Ioo`03
001oogoo00Ioo`0300moogoo009oo`03001oogoo00Ioo`03001oogoo00ioo`P00003Ool007oo01Mo
o`03001oo`0000<000eoo`8000Aoo`8001Uoo`8000Uoo`8000Moo`03001oo`0?00]oo`<000Moo``0
00Moo`03001oogoo01=oo`8000=oo`<0011oo`800003Ool007oo00=oo`8000]oo`400000>Goo00<0
07ooOol04Goo00<007ooOol00Woo1`0057oo100000=oo`0000003Woo00<007oo00003@0047oo1`00
3goo0`001Goo00<007ooOol02goo0`0017oo0P006Goo0`003goo00<007ooOol01Woo00<007ooOol0
1Woo00<03gooOol00Woo00<007ooOol01Woo00<007ooOol03Woo200000=oo`00Ool06Goo10003Goo
0P000goo0`006Goo0`0027oo0P001goo0P0000<03gooOol02Goo0`001goo30001goo00<007ooOol0
57oo00@007ooOomoo`<0011oo`800004Ool000000002Ool2000;Ool1000003Uoo`03001oogoo01Io
o`L001Aoo`05001oo`00001oo`02000>Ool00`00Ool00009000017oo0000000047oo1`003goo0`00
1Goo00<007ooOol02goo0`000goo0`006Goo0`003goo00<007ooOol01Woo00<007ooOol01Woo00<0
3gooOol00Woo00<007ooOol01Woo00<007ooOol03Woo200077oo10003Goo0P000goo0`006Goo0`00
27oo0P001goo0P0000<03gooOol02Goo0`001goo30007Woo00@007ooOomoo`<0011oo`800004Ool0
00000002Ool2000;Ool1000003Uoo`03001oogoo01Ioo`H001Eoo`05001oo`00001oo`02000>Ool0
0`00Ool0000300000goo00000003000017oo001oo`0047oo1`003goo0`001Goo00<007ooOol02goo
0`000goo0`006Goo0`003goo00<007ooOol01Woo00<007ooOol01Woo00<03gooOol00Woo00<007oo
Ool01Woo00<007ooOol03Woo1`007Goo10003Goo0P000goo0P006Woo0`0027oo0P0027oo00<0000?
Ool02Woo0`001goo00<007oo000020007goo00@007oo001oo`80015oo`H0009oo`8000]oo`400000
>Goo00<007ooOol05Woo1P005Goo00D007oo000007oo0080011oo`@00003Ool0000000<00004Ool0
07oo000@Ool7000?Ool30005Ool00`00Oomoo`0;Ool30003Ool3000IOol3000?Ool00`00Oomoo`06
Ool00`00Oomoo`06Ool00`0?Oomoo`02Ool00`00Oomoo`06Ool00`00Oomoo`0?Ool5000NOol4000=
Ool20003Ool2000JOol40007Ool20008Ool00`0000moo`0:Ool30009Ool9000OOol01000Ool007oo
0P004Woo1@000Woo0P002goo0@00000iOol00`00Oomoo`0FOol6000EOol00`00Ool00004000@Ool4
00000goo00000003000017oo001oo`0047oo1P0047oo0`001Goo00<007ooOol02goo0`000goo0`00
6Goo0`003goo00<007ooOol01Woo00<007ooOol01Woo00<03gooOol00Woo00<007ooOol01Goo0P00
4Goo1@007goo0`003Goo0P000goo0P006Woo10001goo0P0027oo00<0000?Ool02Woo0P002Woo2@00
7goo1P004Woo1@000Woo0P002goo0@00000iOol00`00Oomoo`0FOol6000EOol00`00Ool00004000@
Ool30002Ool00`0000l0000200000goo001oo`0AOol6000@Ool30005Ool00`00Oomoo`0;Ool30003
Ool2000JOol3000?Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`0?Oomoo`02Ool00`00Oomoo`04
Ool3000AOol5000OOol200000`0?Oomoo`0;Ool20003Ool2000JOol40007Ool20008Ool00`0000mo
o`0:Ool2000;Ool8000OOol6000BOol50002Ool2000;Ool1000003Uoo`03001oogoo01Moo`D001Eo
o`03001oo`0000@0011oo`03001oogoo009oo`0300003`0000800003Ool007oo015oo`H0015oo`80
00Aoo`8000ioo`8000=oo`8001]oo`03001oogoo00ioo`03001oogoo00Ioo`03001oogoo00Ioo`03
00moogoo009oo`03001oogoo00Aoo`<0015oo`D001moo`80000300moogoo00]oo`8000=oo`8001Yo
o`@000Moo`8000Qoo`0300003goo00Yoo`8000]oo`800003Ool000000080021oo`H0019oo`D0009o
o`8000]oo`400000>Goo0`005goo1@005Goo00<007oo0000100047oo00<007ooOol00Woo00<0000?
00000P0000=oo`00Ool04Goo1P004Goo0P0017oo0P003Woo0P000goo0P006goo00<007ooOol03Woo
00<007ooOol01Woo00<007ooOol01Woo00<03gooOol00Woo00<007ooOol017oo0`004Goo1@007goo
00<007oo00l03Goo0`000Woo0P006Woo10001goo0P0027oo00<0000?Ool02Woo0P002goo0P0000=o
o`0000000P0087oo1P004Woo1@000Woo00<007ooOol027oo0`00000iOol00`00Oomoo`0GOol5000E
Ool00`00Ool00004000@Ool00`00Oomoo`02Ool00`0000l0000200000goo001oo`0AOol6000AOol2
0004Ool2000>Ool20003Ool2000/Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`0?Oomoo`02Ool0
0`00Oomoo`04Ool3000AOol5000OOol00`00Ool03`0=Ool30002Ool2000JOol40007Ool00`00Oomo
o`07Ool2000;Ool2000;Ool20002Ool3000POol6000BOol500000goo0000000<Ool1000003Uoo`03
001oogoo01Moo`D001Eoo`03001oo`0000@0011oo`03001oogoo009oo`0300003`00008001Aoo`H0
015oo`8000Aoo`8000ioo`8000=oo`8002aoo`03001oogoo00Eoo`8000Qoo`0300moogoo009oo`03
001oogoo00Aoo`<0015oo`D001moo`03001oo`0?00eoo`<0009oo`8001Yoo`@000Moo`03001oogoo
00Moo`8000]oo`8000]oo`04001oogooOol3000POol6000BOol8000<Ool1000003Uoo`03001oogoo
01Moo`D001Eoo`03001oo`0000@001Eoo`0300003`00008001Aoo`H0015oo`8000Aoo`8000ioo`80
00=oo`8002aoo`03001oogoo00Eoo`8000Qoo`0300moogoo009oo`03001oogoo00Aoo`<0015oo`D0
01moo`03001oo`0?00eoo`<0009oo`8001Yoo`@000Moo`03001oogoo00Moo`8000]oo`8000moo`<0
021oo`D0000300moogoo011oo`P000aoo`400000>Goo00<007ooOol05goo1@005Goo00<007oo0000
10005Woo00@03`00000001Aoo`H0015oo`8000Aoo`8000ioo`8000=oo`8002aoo`03001oogoo00Eo
o`03001oogoo00Moo`0300moogoo009oo`03001oogoo00Aoo`<0019oo`@0025oo`0300moogoo00]o
o`<0009oo`8001Yoo`@000Moo`03001oogoo00Moo`8000]oo`8000moo`<0021oo`D0000300moogoo
011oo`P000aoo`400000>Goo00<007ooOol05goo1@005Goo00<007oo000010005Woo00@03`000000
01Aoo`H0015oo`8000Aoo`8000ioo`<0009oo`8002aoo`03001oogoo00Eoo`03001oogoo00Moo`03
00moogoo009oo`03001oogoo00Aoo`<0019oo`@0025oo`0300moogoo00aoo`800004Ool00000000J
Ool40006Ool20009Ool2000;Ool2000?Ool3000POol500000`0?Oomoo`0@Ool8000<Ool1000003Uo
o`03001oogoo01Moo`D001Eoo`03001oo`0000@001Ioo`0400l00000000DOol6000AOol20004Ool2
000>Ool30002Ool2000/Ool00`00Oomoo`05Ool00`00Oomoo`07Ool00`0?Oomoo`02Ool00`00Oomo
o`04Ool3000BOol4000QOol00`0?Oomoo`0<Ool6000JOol400001Woo001oogoo001oo`8000Uoo`80
00]oo`8000moo`<0021oo`D0000300moogoo011oo`P000aoo`400000>Goo00<007ooOol05goo1@00
5Goo00<007ooOol010005Woo00<03`0000005Goo1@004Woo0`000goo0P003Woo0`000Woo0P00;7oo
00<007ooOol01Goo00<007ooOol01goo00<03gooOol00Woo00<007ooOol017oo0`004Woo10008Goo
00<03gooOol037oo1P006Woo100000Ioo`00Oomoo`00Ool20009Ool2000;Ool2000?Ool3000POol5
00000`0?Oomoo`0@Ool7000=Ool1000003Uoo`03001oogoo01Moo`<00003Ool007oo01Moo`800003
Ool007oo01Eoo`0300l0000001Eoo`D0019oo`<000=oo`8000ioo`<0009oo`8002eoo`03001oogoo
00Aoo`03001oogoo00Moo`0300moogoo009oo`03001oogoo00Aoo`<0019oo`04001oo`00000QOol0
0`0?Oomoo`0<Ool5000LOol50002Ool3000:Ool2000;Ool2000?Ool00`00Oomoo`0POol00`00Ool0
000200000`0?Oomoo`0@Ool7000=Ool1000003Uoo`03001oogoo01Moo`<001Yoo`800003Ool007oo
01Eoo`0300moogoo01Eoo`<00003Ool007oo015oo`<000=oo`8000ioo`<00004Ool00000000]Ool0
0`00Oomoo`04Ool00`00Oomoo`07Ool00`0?Oomoo`02Ool00`00Oomoo`04Ool3000BOol01000Ool0
00008Goo00<03gooOol037oo1@0077oo1P0000Aoo`00000000Yoo`8000]oo`8000moo`03001oogoo
021oo`03001oo`000080000300moogoo01=oo`<000ioo`400000>Goo00<007ooOol05goo0`006Woo
0P0000=oo`00Ool05Goo00<03gooOol05Goo0`0057oo0`000goo0P003Woo0`0000Aoo`00000002eo
o`03001oogoo00Aoo`03001oogoo00Moo`0300moogoo009oo`03001oogoo00Aoo`<001Aoo`80025o
o`0300moogoo00aoo`D001aoo`X000Yoo`8000]oo`8000moo`03001oogoo021oo`03001oo`000080
000300moogoo01=oo`<000ioo`400000>Goo00<007ooOol05goo0P006goo00@007ooOol001Ioo`03
00moogoo01Ioo`8001Aoo`<000=oo`8000ioo`0400l00000Ool3000]Ool00`00Oomoo`04Ool00`00
Oomoo`07Ool00`0?Oomoo`03Ool00`00Oomoo`03Ool3000DOol2000QOol00`0?Oomoo`0<Ool5000L
Ool:000:Ool2000;Ool2000dOol300000`0?Oomoo`0COol3000>Ool1000003Uoo`03001oogoo01Mo
o`8001]oo`04001oogoo000FOol00`0?Oomoo`0FOol2000DOol30003Ool00`00Oomoo`0=Ool0100?
000007oo0`00;Goo00<007ooOol017oo00<007ooOol01goo00<03gooOol00goo00<007ooOol00goo
0`0057oo00<007ooOol087oo00<03gooOol037oo10007Goo2P002Woo0P002goo0P00=7oo0`0000<0
3gooOol04goo00<007oo00003Woo0@00000iOol3000HOol00`00Oomoo`0LOol00`00Oomoo`0DOol0
0`0?Oomoo`0FOol2000DOol30003Ool00`00Oomoo`0=Ool0100?000007oo0P00;Woo00<007ooOol0
17oo00<007ooOol01goo00<03gooOol00goo00<007ooOol00goo0P00>7oo00<03gooOol037oo1000
7Woo2@002Woo0P002goo0P00=7oo0`0000<03gooOol04goo00<007oo000037oo0`00000iOol00`00
Oomoo`0HOol00`00Oomoo`0cOol00`0?Oomoo`0FOol2000DOol30003Ool00`00Oomoo`0=Ool0100?
000007oo0P00;Woo00<007ooOol017oo00<007ooOol01goo00<03gooOol00goo00<007ooOol00goo
0P00>7oo00<03gooOol037oo10007Woo2@002Woo0P002goo0P00=7oo0`0000<03gooOol04goo00<0
07ooOol03Woo0@00000iOol00`00Oomoo`0HOol00`00Oomoo`0cOol00`0?Oomoo`0FOol2000EOol2
0003Ool00`00Oomoo`0=Ool0100?000007oo0P00;Woo00<007ooOol017oo00<007ooOol01goo00<0
3gooOol00goo00<007ooOol00goo0P00>7oo00<03gooOol03Goo0`007Woo2@002Woo0`002Woo0P00
=7oo0`0000<03gooOol097oo0@00000iOol00`00Oomoo`0HOol00`00Oomoo`0cOol00`0?Oomoo`0]
Ool20003Ool00`00Oomoo`0=Ool0100?000007oo0P00;Woo00<007ooOol017oo00<007ooOol01goo
00<03gooOol00goo00<007ooOol00goo0P00>7oo00<03gooOol03Goo00<007oo00007Woo2@002goo
0P002Woo0P00=7oo00@007oo00003bIoo`400000>Goo00<007ooOol067oo00<007ooOol0<goo00<0
3gooOol0;Goo0P0000Aoo`00000000moo`0300l0000000<002ioo`03001oogoo00Aoo`03001oogoo
00Moo`0300moogoo00=oo`03001oogoo00=oo`8003Qoo`0300moogoo00eoo`03001oo`0001ioo`03
001oo`0000H000]oo`8000Yoo`8003Aoo`04001oogoo00lVOol1000003Uoo`03001oogoo01Qoo`03
001oogoo03=oo`0300moogoo02eoo`800004Ool00000000?Ool00`0?00000003000^Ool00`00Oomo
o`04Ool00`00Oomoo`07Ool00`0?Oomoo`03Ool01000Oomoogoo1000>7oo00<03gooOol03Goo00<0
07oo000087oo1P0037oo0P002Woo0P00=7oo00@007ooOol03bIoo`400000>Goo00<007ooOol067oo
00<007ooOol0<goo00<03gooOol0;Goo1P003goo00<03`0000000`00;Woo00<007ooOol017oo00<0
07ooOol01goo00<03gooOol00goo00@007ooOomoo`@003Qoo`0300moogoo00eoo`03001oogoo021o
o`H000aoo`8000Yoo`8003Aoo`04001oogoo00lVOol1000003Uoo`03001oogoo01Qoo`03001oogoo
03=oo`0300moogoo02eoo`H000moo`0300l0000000<002ioo`03001oogoo00Aoo`03001oogoo00Mo
o`0300moogoo00=oo`04001oogooOol4000hOol00`0?Oomoo`0=Ool00`00Oomoo`0QOol5000<Ool2
000:Ool00`00Oomoo`0cOol01000Oomoo`0?9Woo0@00000iOol00`00Oomoo`0HOol00`00Oomoo`0c
Ool00`0?Oomoo`0]Ool6000?Ool00`0?00000003000^Ool00`00Oomoo`04Ool00`00Oomoo`07Ool0
0`0?Oomoo`03Ool01000Oomoogoo1000>7oo00<03gooOol03Goo00<007ooOol08Goo1@0037oo0P00
2Goo0P00=Goo00@007ooOol03bIoo`400000>Goo00<007ooOol067oo00<007ooOol0<goo00<03goo
Ool0;Goo1P003goo00<03goo00000`00;Woo00<007ooOol017oo00<007ooOol01goo00<03gooOol0
0goo00@007ooOomoo`<003Uoo`0300moogoo035oo`D000aoo`<000Qoo`8003Eoo`04001oogoo00lV
Ool1000003Uoo`03001oogoo01Qoo`03001oogoo03=oo`0300moogoo02eoo`H000moo`0300moo`00
00<002ioo`03001oogoo00Aoo`03001oogoo00Moo`0300moogoo00=oo`04001oogooOol3000iOol0
0`0?Oomoo`0aOol00`00Ool00002000<Ool30008Ool00`00Oomoo`0gOol00`0?Oomoo`0TOol10000
03Uoo`03001oogoo01Qoo`03001oogoo03=oo`0300moogoo02eoo`H000moo`0300moogoo00<002io
o`03001oogoo00Aoo`03001oogoo00Moo`0300moogoo00=oo`04001oogooOol3000iOol00`0?Oomo
o`0aOol00`00Ool00002000<Ool30008Ool00`00Oomoo`0gOol00`0?Oomoo`0TOol1000003Uoo`03
001oogoo04ioo`0300moogoo02eoo`D0011oo`0300moogoo00<002ioo`8000Aoo`8000Uoo`0300mo
ogoo00=oo`04001oogooOol3000iOol00`0?Oomoo`0aOol01000Ool000003Goo0`0027oo00<007oo
Ool0=goo00<03gooOol097oo0@00000iOol3001>Ool00`0?Oomoo`0]Ool4000AOol00`0?Oomoo`03
000^Ool20004Ool20009Ool00`0?Oomoo`03Ool20002Ool3000iOol00`0?Oomoo`0cOol2000=Ool3
0008Ool00`00Oomoo`0gOol00`0?Oomoo`0ROol3000003Uoo`03001oogoo04ioo`0300moogoo02eo
o`@0015oo`0300moogoo00<002moo`03001oogoo009oo`8000Uoo`0300moogoo00=oo`80009oo`<0
03Uoo`0300moogoo03Aoo`03001oogoo00]oo`<000Qoo`03001oogoo03Moo`0300moogoo02Aoo`40
0000>Goo00<007ooOol0CWoo00<03gooOol0;Goo10004Goo00<03gooOol00`00;goo00<007ooOol0
0Woo0P002Goo00<03gooOol017oo00<007ooOol00`00>Goo00<03gooOol0=7oo00<007ooOol02goo
0`0027oo00<007ooOol0=goo00<03gooOol097oo0@00000iOol00`00Oomoo`1>Ool00`0?Oomoo`0^
Ool3000AOol00`0?Oomoo`03000_Ool00`00Oomoo`02Ool20009Ool00`0?Oomoo`04Ool00`00Oomo
o`03000iOol00`0?Oomoo`0dOol00`00Oomoo`0;Ool30008Ool00`00Oomoo`0gOol00`0?Oomoo`0T
Ool1000003Uoo`03001oogoo04ioo`0300moogoo02ioo`<0015oo`0400moogooOol2000_Ool00`00
Oomoo`02Ool20009Ool00`0?Oomoo`04Ool00`00Oomoo`03000iOol00`0?Oomoo`0dOol00`00Oomo
o`0;Ool30008Ool00`00Oomoo`0gOol00`0?Oomoo`0TOol1000003Uoo`03001oogoo04ioo`0300mo
ogoo02ioo`<0015oo`0400moogooOol2000_Ool00`00Oomoo`02Ool20009Ool00`0?Oomoo`04Ool0
0`00Oomoo`03000iOol00`0?Oomoo`0dOol00`00Oomoo`0;Ool30008Ool00`00Oomoo`0gOol00`0?
Oomoo`0TOol1000003Uoo`03001oogoo04ioo`0300moogoo02ioo`<0015oo`0400moogooOol2000_
Ool00`00Oomoo`02Ool00`00Oomoo`08Ool00`0?Oomoo`04Ool00`00Ool00003000iOol00`0?Oomo
o`0dOol00`00Oomoo`0;Ool30008Ool00`00Oomoo`0gOol00`0?Oomoo`0TOol1000003Uoo`03001o
ogoo04ioo`0300moogoo02ioo`<0015oo`0300moogoo009oo`03001oogoo02eoo`03001oogoo009o
o`03001oogoo00Qoo`0300moogoo00Aoo`03001oo`0000<003Uoo`0300moogoo049oo`<000Qoo`03
001oogoo03Moo`0300moogoo02Aoo`4000005Goo0P0027oo10001Woo1@002goo00<007ooOol0CWoo
00<03gooOol0;Woo0`004Goo00<03gooOol0<Woo00<007ooOol00Woo00<007ooOol027oo00<03goo
Ool017oo00<007oo00000`00>Goo00<03gooOol0@Woo0`0027oo00<007ooOol0=goo00<03gooOol0
97oo0@00000EOol20006Ool70004Ool7000:Ool00`00Oomoo`1>Ool00`0?Oomoo`0^Ool3000AOol0
0`0?Oomoo`0bOol00`00Oomoo`02Ool00`00Oomoo`08Ool00`0?Oomoo`04Ool00`00Ool00003000i
Ool00`0?Oomoo`12Ool30008Ool00`00Oomoo`0gOol00`0?Oomoo`0TOol1000001Eoo`8000Ioo`80
00=oo`8000Aoo`8000=oo`8000Yoo`03001oogoo04ioo`0300moogoo02ioo`<0015oo`0300moogoo
039oo`03001oogoo009oo`03001oogoo00Qoo`0300moogoo00Aoo`H003Uoo`0300moogoo049oo`<0
00Qoo`03001oogoo03Moo`0300moogoo02Aoo`4000005Goo0P001Goo0P001Goo0P000Woo0P001Goo
0P002Goo00<007ooOol0CWoo00<03gooOol0;Woo0`004Goo00<03gooOol0<Woo00<007ooOol00Woo
00<007ooOol027oo00<03gooOol017oo1P00>Goo00<03gooOol0@Woo0`0027oo00<007ooOol0=goo
00<03gooOol097oo0@000004Ool:0007Ool20005Ool20005Ool20002Ool20005Ool20009Ool00`00
Oomoo`1>Ool00`0?Oomoo`0_Ool2000AOol00`0?Oomoo`0bOol00`00Oomoo`02Ool00`00Oomoo`08
Ool00`0?Oomoo`04Ool6000iOol00`0?Oomoo`12Ool00`0?00000008Ool00`00Oomoo`0gOol00`0?
Oomoo`0TOol1000001Eoo`8000Eoo`8000Eoo`80009oo`8000Eoo`8000Uoo`03001oogoo04ioo`03
00moogoo02moo`80015oo`0300moogoo039oo`03001oogoo009oo`03001oogoo00Qoo`0300moogoo
00Aoo`H003Uoo`0300moogoo049oo`0300l0000000Qoo`03001oogoo03Moo`0300moogoo02Aoo`40
00005Goo0P001Goo0P001Goo0P000Woo0P001Goo0P002Goo1000O`0?0P00AP0?00<0000?00l00P0?
00<0000?00l03`0?1P00O`0?0P00200?00<0000?00l0F`0?1000000EOol20005Ool30003Ool30002
Ool20005Ool20009Ool00`00Oomoo`1>Ool00`0?Oomoo`0_Ool00`00Oomoo`0@Ool00`0?Oomoo`0b
Ool00`00Oomoo`02Ool00`00Oomoo`08Ool00`0?Oomoo`04Ool400000goo001oo`0hOol00`0?Oomo
o`12Ool00`0?00000008Ool00`00Oomoo`0gOol00`0?Oomoo`0TOol1000001Eoo`8000Eoo`P000=o
o`8000Eoo`8000Uoo`03001oogoo04ioo`0300moogoo049oo`0300moogoo039oo`03001oogoo009o
o`03001oogoo00Qoo`0300moogoo00Aoo`@00003Ool007oo03Qoo`0300moogoo049oo`0300l00000
00Qoo`03001oogoo03Moo`0300moogoo02Aoo`4000005Goo0P001Goo0P0000=oo`0000000P0017oo
0P001Goo0P002Goo00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol0<Woo00@007ooOomoo`80
00Yoo`0300moogoo00Aoo`@00003Ool007oo03Qoo`0300moogoo049oo`0300l0000000Qoo`03001o
ogoo03Moo`0300moogoo02Aoo`4000004Woo0P0000=oo`0000001Goo0P002Goo0P001Goo0P002Goo
00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol0<Woo0P000Woo0P002Woo00<03gooOol017oo
100000=oo`00Ool0>7oo00<03gooOol0@Woo00<03`00000027oo00<007ooOol0=goo00<03gooOol0
97oo0@00000BOol50006Ool20004Ool20002Ool20005Ool20009Ool00`00Oomoo`1>Ool00`0?Oomo
o`12Ool00`0?Oomoo`0bOol20002Ool2000:Ool00`0?Oomoo`04Ool400000goo001oo`0hOol00`0?
Oomoo`12Ool00`0?00000008Ool00`00Oomoo`0gOol00`0?Oomoo`0TOol1000001Aoo`<000Ioo`80
00=oo`<000=oo`8000=oo`8000Yoo`03001oogoo04ioo`0300moogoo049oo`0300moogoo039oo`80
009oo`8000Yoo`0300moogoo00Eoo`<00003Ool007oo03Qoo`0300moogoo049oo`0300l0000000Qo
o`03001oogoo03Moo`0300moogoo02Aoo`4000005Goo0P001goo1P0017oo1`002Woo00<007ooOol0
CWoo00<03gooOol0@Woo00<03gooOol0<Woo0P000Woo0P002Woo00<03gooOol01Goo0`0000=oo`00
Ool0>7oo00<03gooOol0@Woo00<03`00000027oo00<007ooOol0=goo00<03gooOol097oo0@00000E
Ool20008Ool40006Ool5000;Ool00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`0bOol20002
Ool2000:Ool00`0?Oomoo`05Ool300000goo001oo`0hOol00`0?Oomoo`12Ool00`0?00000008Ool0
0`00Oomoo`0gOol00`0?Oomoo`0TOol1000003Uoo`03001oogoo04ioo`0300moogoo049oo`0300mo
ogoo039oo`80009oo`03001oogoo00Uoo`0300moogoo00Eoo`<00003Ool007oo03Qoo`0300moogoo
049oo`0300l0000000Moo`8003Uoo`0300moogoo02Aoo`400000>Goo00<007ooOol0CWoo00<03goo
Ool0@Woo00<03gooOol0<Woo0P000Woo00<007ooOol02Goo00<03gooOol01Woo00<007ooOol0>Woo
00<03gooOol0@Woo00<03`0000001goo0P00>Goo00<03gooOol097oo0@00000iOol00`00Oomoo`1>
Ool00`0?Oomoo`12Ool00`0?Oomoo`0bOol20002Ool00`00Oomoo`09Ool00`0?Oomoo`06Ool00`00
Oomoo`0jOol00`0?Oomoo`12Ool00`0?00000007Ool2000iOol00`0?Oomoo`0TOol1000003Uoo`03
001oogoo04ioo`0300moogoo049oo`0300moogoo039oo`80009oo`03001oogoo00Uoo`0300moogoo
00Ioo`03001oogoo03Yoo`0300moogoo049oo`0300l0000000Moo`8003Uoo`0300moogoo02Aoo`40
0000>Goo0`00CWoo00<03gooOol0@Woo00<03gooOol0<Woo0P0000=oo`0000002goo00<03gooOol0
1Woo00<007ooOol0>Woo00<03gooOol0@Woo00<03`0000001goo0P00>Goo00<03gooOol08Woo0`00
000iOol00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`0bOol200000goo0000000;Ool00`0?
Oomoo`06Ool00`00Oomoo`0jOol00`0?Oomoo`12Ool00`0?00000007Ool2000iOol00`0?Oomoo`0T
Ool1000003Uoo`03001oogoo04ioo`0300moogoo049oo`0300moogoo039oo`800003Ool0000000]o
o`0300moogoo00Ioo`03001oogoo03Yoo`0300moogoo049oo`0300l0000000Moo`8003Uoo`0300mo
ogoo02Aoo`400000>Goo00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol0<goo00@007oo0000
00]oo`0300moogoo00Ioo`03001oogoo03Yoo`0300moogoo049oo`0300l0000000Moo`8003Uoo`03
00moogoo02Aoo`400000>Goo00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol0<goo00@007oo
000000]oo`0300moogoo00Ioo`03001oogoo03Yoo`0300moogoo049oo`0400l000000006Ool00`00
Oomoo`0hOol00`0?Oomoo`0TOol1000003Uoo`03001oogoo04ioo`0300moogoo049oo`0300moogoo
03=oo`04001oo`00000;Ool00`0?Oomoo`06Ool00`00Oomoo`0jOol00`0?Oomoo`12Ool0100?0000
00001Woo00<007ooOol0>7oo00<03gooOol097oo0@00000iOol00`00Oomoo`1>Ool00`0?Oomoo`12
Ool00`0?Oomoo`0cOol01000Ool000002goo00<03gooOol01Woo00<007ooOol0>Woo00<03gooOol0
@Woo00@03`00000000Ioo`03001oogoo03Qoo`0300moogoo02Aoo`400000>Goo00<007ooOol0CWoo
00<03gooOol0@Woo00<03gooOol0<goo0`0037oo00<03gooOol01Woo00<007ooOol0>Woo00<03goo
Ool0@Woo00@03`00Ool000Ioo`03001oogoo03Qoo`0300moogoo02Aoo`400000>Goo00<007ooOol0
CWoo00<03gooOol0@Woo00<03gooOol0<goo0`0037oo00<03gooOol01Woo00<007ooOol0>Woo00<0
3gooOol0@Woo00@03`00Ool000Ioo`03001oogoo03Qoo`0300moogoo02Aoo`400000>Goo00<007oo
Ool0CWoo00<03gooOol0@Woo00<03gooOol0<goo0`0037oo00<03gooOol01Woo00<007ooOol0>Woo
00<03gooOol0@Woo00@03gooOol000Ioo`03001oogoo03Qoo`0300moogoo02Aoo`400000>Goo00<0
07ooOol0CWoo00<03gooOol0@Woo00<03gooOol0<goo0`0037oo00<03gooOol01Woo00<007ooOol0
>Woo00<03gooOol0@Woo00@03gooOol000Ioo`03001oogoo03Qoo`0300moogoo02Aoo`400000>Goo
00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol0<goo0`0037oo00<03gooOol01Woo00<007oo
Ool0>Woo00<03gooOol0@Woo00@03gooOol000Ioo`03001oogoo03Qoo`0300moogoo02Aoo`400000
>Goo00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol0=7oo0P0037oo00<03gooOol01Woo00<0
07ooOol0>Woo00<03gooOol0@Woo00@03gooOol000Ioo`03001oogoo03Qoo`0300moogoo02Aoo`40
0000>Goo0`00CWoo00<03gooOol0@Woo00<03gooOol0=7oo0P0037oo00<03gooOol01Woo00<007oo
Ool0>Woo00<03gooOol0@Woo00@03gooOol000Ioo`03001oogoo03Qoo`0300moogoo029oo`<00000
>Goo00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol0=7oo0P0037oo00<03gooOol01Woo00<0
07ooOol0>Woo00<03gooOol0@Woo00@03gooOol000Eoo`8003Yoo`0300moogoo02Aoo`400000>Goo
00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol0=7oo0P0037oo00<03gooOol01Woo00<007oo
Ool0>Woo00<03gooOol0@Woo00@03gooOol000Eoo`8003Yoo`0300moogoo02Aoo`400000>Goo00<0
07ooOol0CWoo00<03gooOol0@Woo00<03gooOol0=7oo0P0037oo00<03gooOol01Woo00<007ooOol0
>Woo00<03gooOol0@Woo00@03gooOol000Eoo`8003Yoo`0300moogoo02Aoo`400000>Goo00<007oo
Ool0CWoo00<03gooOol0@Woo00<03gooOol0=7oo0P0037oo00<03gooOol0@goo00<03gooOol0@Woo
00@03gooOol000Eoo`8003Yoo`0300moogoo02Aoo`400000>Goo00<007ooOol0CWoo00<03gooOol0
@Woo00<03gooOol0=7oo0P0037oo00<03gooOol0@goo00<03gooOol0@Woo00@03gooOol000Eoo`03
001oogoo03Uoo`0300moogoo02Aoo`400000>Goo00<007ooOol0CWoo00<03gooOol0@Woo00<03goo
Ool0=7oo0P0037oo00<03gooOol0@goo00<03gooOol0@Woo00@03gooOol000Eoo`03001oogoo03Uo
o`0300moogoo02Aoo`400000>Goo00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol0=Goo00<0
07ooOol02Woo00<03gooOol0@goo00<03gooOol0@Woo00@03gooOol000Eoo`03001oogoo03Uoo`03
00moogoo02Aoo`400000>Goo00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol0=Goo00<007oo
Ool02Woo00<03gooOol0@goo00<03gooOol0@Woo00@03gooOol000Eoo`03001oogoo03Uoo`0300mo
ogoo02Aoo`400000>Goo00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol0=Goo00<007ooOol0
2Woo00<03gooOol0@goo00<03gooOol0@Woo00@03gooOol000Eoo`03001oogoo03Uoo`0300moogoo
02Aoo`400000>Goo00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol0=Goo00<007ooOol02Woo
00<03gooOol0@goo00<03gooOol0@Woo00@03gooOol000Eoo`03001oogoo03Uoo`0300moogoo02Ao
o`400000>Goo00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol0=Goo00<007ooOol02Woo00<0
3gooOol0@goo00<03gooOol0@Woo00@03gooOol000Eoo`03001oogoo03Uoo`0300moogoo02Aoo`40
0000>Goo00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol0@Woo00<03gooOol0@goo00<03goo
Ool0@Woo00@03gooOol000Eoo`03001oogoo03Uoo`0300moogoo02Aoo`400000>Goo0`00CWoo00<0
3gooOol0@Woo00<03gooOol0@Woo00<03gooOol0@goo00<03gooOol0@Woo00<03gooOol00P0017oo
00<007ooOol0>Goo00<03gooOol08Woo0`00000iOol00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?
Oomoo`12Ool00`0?Oomoo`13Ool00`0?Oomoo`12Ool00`0?Oomoo`020003Ool2000kOol00`0?Oomo
o`0TOol1000003Uoo`03001oogoo04ioo`0300moogoo049oo`0300moogoo049oo`0300moogoo04=o
o`0300moogoo049oo`0300moogoo008000=oo`8003]oo`0300moogoo02Aoo`400000>Goo00<007oo
Ool0CWoo00<03gooOol0@Woo00<03gooOol0@Woo00<03gooOol0@goo00<03gooOol0@Woo00<03goo
Ool00P000goo0P00>goo00<03gooOol097oo0@00000iOol00`00Oomoo`1>Ool00`0?Oomoo`12Ool0
0`0?Oomoo`12Ool00`0?Oomoo`13Ool00`0?Oomoo`12Ool00`0?Oomoo`020003Ool2000kOol00`0?
Oomoo`0TOol1000003Uoo`03001oogoo04ioo`0300moogoo049oo`0300moogoo049oo`0300moogoo
04=oo`0300moogoo049oo`0500moogooOol00003Ool2000kOol00`0?Oomoo`0TOol1000003Uoo`03
001oogoo04ioo`0300moogoo049oo`0300moogoo049oo`0300moogoo04=oo`0300moogoo049oo`05
00moogooOol00003Ool2000kOol00`0?Oomoo`0TOol1000003Uoo`03001oogoo04ioo`0300moogoo
049oo`0300moogoo049oo`0300moogoo04=oo`0300moogoo049oo`0500moogooOol00003Ool2000k
Ool00`0?Oomoo`0TOol1000003Uoo`03001oogoo04ioo`0300moogoo049oo`0300moogoo049oo`03
00moogoo04=oo`0300moogoo049oo`0500moogooOol00003Ool2000kOol00`0?Oomoo`0TOol10000
03Uoo`03001oogoo04ioo`0300moogoo049oo`0300moogoo049oo`0300moogoo04=oo`0300moogoo
049oo`0500moogooOol00003Ool2000kOol00`0?Oomoo`0TOol1000003Uoo`03001oogoo04ioo`03
00moogoo049oo`0300moogoo049oo`0300moogoo04=oo`0300moogoo049oo`0500moogooOol00003
Ool2000kOol00`0?Oomoo`0TOol1000003Uoo`03001oogoo04ioo`0300moogoo049oo`0300moogoo
049oo`0300moogoo04=oo`0300moogoo049oo`0500moogooOol00003Ool2000kOol00`0?Oomoo`0T
Ool1000003Uoo`03001oogoo04ioo`0300moogoo049oo`0300moogoo049oo`0300moogoo04=oo`03
00moogoo049oo`0500moogooOol00002Ool3000kOol00`0?Oomoo`0TOol1000003Uoo`03001oogoo
04ioo`0300moogoo049oo`0300moogoo049oo`0300moogoo04=oo`0300moogoo049oo`0500moogoo
Ool00002Ool3000kOol00`0?Oomoo`0TOol1000003Uoo`<004ioo`0300moogoo049oo`0300moogoo
049oo`0300moogoo04=oo`0300moogoo049oo`0400moogooOol2000017oo00000000>goo00<03goo
Ool08Woo0`00000iOol00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`12Ool00`0?Oomoo`13
Ool00`0?Oomoo`12Ool0100?Oomoogoo0P0000Aoo`00000003]oo`0300moogoo02Aoo`400000>Goo
00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol0@Woo00<03gooOol0@goo00<03gooOol0@Woo
00@03gooOomoo`800004Ool00000000kOol00`0?Oomoo`0TOol1000003Uoo`03001oogoo04ioo`03
00moogoo049oo`0300moogoo049oo`0300moogoo04=oo`0300moogoo049oo`0400moogooOol20000
17oo00000000>goo00<03gooOol097oo0@00000iOol00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?
Oomoo`12Ool00`0?Oomoo`13Ool00`0?Oomoo`12Ool0100?Oomoogoo0P0000=oo`000000?7oo00<0
3gooOol097oo0@00000iOol00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`12Ool00`0?Oomo
o`13Ool00`0?Oomoo`12Ool0100?Oomoogoo0P0000=oo`000000?7oo00<03gooOol097oo0@00000i
Ool00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`12Ool00`0?Oomoo`13Ool00`0?Oomoo`12
Ool0100?Oomoogoo0P0000=oo`000000?7oo00<03gooOol097oo0@00000EOol20007Ool40007Ool5
000;Ool00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`12Ool00`0?Oomoo`13Ool00`0?Oomo
o`12Ool0100?Oomoogoo1@00?7oo00<03gooOol097oo0@00000EOol20006Ool70004Ool7000:Ool0
0`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`12Ool00`0?Oomoo`13Ool00`0?Oomoo`12Ool0
100?Oomoogoo1@00?7oo00<03gooOol097oo0@00000EOol20005Ool30003Ool20004Ool20003Ool2
000:Ool00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`12Ool00`0?Oomoo`13Ool00`0?Oomo
o`12Ool0100?Oomoogoo1@00?7oo00<03gooOol097oo0@00000EOol20005Ool20005Ool20002Ool2
0005Ool20009Ool00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`12Ool00`0?Oomoo`13Ool0
0`0?Oomoo`12Ool0100?Oomoogoo1@00?7oo00<03gooOol097oo0@000004Ool:0007Ool2000<Ool2
0002Ool20005Ool20009Ool00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`12Ool00`0?Oomo
o`13Ool00`0?Oomoo`12Ool0100?Oomoogoo1@00?7oo00<03gooOol097oo0@00000EOol2000<Ool2
0002Ool20005Ool20009Ool00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`12Ool00`0?Oomo
o`13Ool00`0?Oomoo`12Ool0100?Oomoogoo1@00?7oo00<03gooOol097oo0@00000EOol2000<Ool2
0002Ool20005Ool20009Ool4003o00mW00l5001P00l4000001Eoo`8000Eoo`8000Aoo`<0009oo`80
00Eoo`8000Uoo`03001oogoo04ioo`0300moogoo049oo`0300moogoo049oo`0300moogoo04=oo`03
00moogoo049oo`0400moogooOol5000lOol00`0?Oomoo`0TOol1000001Eoo`8000Eoo`P000=oo`80
00Eoo`8000Uoo`03001oogoo04ioo`0300moogoo049oo`0300moogoo049oo`0300moogoo04=oo`03
00moogoo049oo`0600moogooOol007oo0`00?7oo00<03gooOol097oo0@00000EOol20005Ool20000
0goo000000020004Ool20005Ool20009Ool00`00Oomoo`1>Ool00`0?Oomoo`12Ool00`0?Oomoo`12
Ool00`0?Oomoo`13Ool00`0?Oomoo`12Ool01P0?Oomoogoo001oo`<003aoo`0300moogoo02Aoo`40
00004Woo0P0000=oo`0000001Goo0P002Goo0P001Goo0P002Goo00<007ooOol0CWoo00<03gooOol0
@Woo00<03gooOol0@Woo00<03gooOol0@goo00<03gooOol0@Woo00H03gooOomoo`00Ool3000lOol0
0`0?Oomoo`0TOol10000019oo`D000Ioo`8000Qoo`8000Eoo`8000Uoo`03001oogoo04ioo`0300mo
ogoo049oo`0300moogoo049oo`0300moogoo04=oo`0300moogoo049oo`0600moogooOol007oo0`00
?7oo00<03gooOol097oo0@00000DOol30006Ool20009Ool20003Ool2000:Ool00`00Oomoo`1>Ool0
0`0?Oomoo`12Ool00`0?Oomoo`12Ool00`0?Oomoo`13Ool00`0?Oomoo`12Ool01P0?Oomoogoo001o
o`8003eoo`0300moogoo02Aoo`4000005Goo0P001Woo1`0017oo1`002Woo00<007ooOol0CWoo00<0
3gooOol0@Woo00<03gooOol0@Woo00<03gooOol0@goo00<03gooOol0@Woo00<03gooOol00goo0P00
?Goo00<03gooOol097oo0@00000EOol20006Ool70005Ool5000;Ool00`00Oomoo`1>Ool00`0?Oomo
o`12Ool00`0?Oomoo`12Ool00`0?Oomoo`13Ool00`0?Oomoo`12Ool00`0?Oomoo`03Ool2000mOol0
0`0?Oomoo`0TOol1000003Uoo`03001oogoo04ioo`0300moogoo049oo`0300moogoo049oo`0300mo
ogoo04=oo`0300moogoo049oo`0300moogoo00=oo`8003eoo`0300moogoo02Aoo`400000>Goo00<0
07ooOol0CWoo00<03gooOol0@Woo00<03gooOol0@Woo00<03gooOol0@goo00<03gooOol0@Woo00<0
3gooOol017oo00<007ooOol0>goo00<03gooOol097oo0@00000iOol00`00Oomoo`1>Ool00`0?Oomo
o`12Ool00`0?Oomoo`12Ool00`0?Oomoo`13Ool00`0?Oomoo`12Ool00`0?Oomoo`04Ool00`00Oomo
o`0kOol00`0?Oomoo`0TOol1000003Uoo`03001oogoo04ioo`0300moogoo049oo`0300moogoo049o
o`0300moogoo04=oo`0300moogoo049oo`0300moogoo00Aoo`03001oogoo03]oo`0300moogoo02Ao
o`400000>Goo0`00CWoo00<03gooOol0@Woo00<03gooOol0@Woo00<03gooOol0@goo00<03gooOol0
@Woo00<03gooOol017oo00<007ooOol0>goo00<03gooOol08Woo0`00000iOol00`00Oomoo`1>Ool0
0`0?Oomoo`12Ool00`0?Oomoo`12Ool00`0?Oomoo`13Ool00`0?Oomoo`12Ool00`0?Oomoo`04Ool0
0`00Oomoo`0kOol00`0?Oomoo`0TOol1000003Uoo`03001oogoo04ioo`0300moogoo049oo`0300mo
ogoo049oo`0300moogoo04=oo`0300moogoo049oo`0300moogoo00Aoo`03001oogoo03]oo`0300mo
ogoo02Aoo`400000>Goo00<007ooOol0CWoo00<03gooOol0@Woo00<03gooOol0@Woo00<03gooOol0
@goo00<03gooOol0@Woo00<03gooOol0@Woo00<03gooOol097oo0@00000iOol00`00Oomoo`1>Ool0
0`0?Oomoo`12Ool00`0?Oomoo`12Ool00`0?Oomoo`13Ool00`0?Oomoo`12Ool00`0?Oomoo`12Ool0
0`0?Oomoo`0TOol1000003Uoo`03001oogoo04ioo`0300moogoo049oo`0300moogoo049oo`0300mo
ogoo04=oo`0300moogoo049oo`0300moogoo049oo`0300moogoo02Aoo`400000>Goo00<007ooOol0
27oo00<007ooOol0@goo00<007ooOol0@Woo00<007ooOol0@Woo00<007ooOol0@goo00<007ooOol0
@Woo00<007ooOol0@Woo00<007ooOol097oo0@00000iOol00`00Oomoo`08Ool00`00Oomoo`0?Ool0
0`00Oomoo`0>Ool00`00Oomoo`0>Ool00`00Oomoo`0?Ool00`00Oomoo`0>Ool00`00Oomoo`0>Ool0
0`00Oomoo`0?Ool00`00Oomoo`0>Ool00`00Oomoo`0>Ool00`00Oomoo`0?Ool00`00Oomoo`0>Ool0
0`00Oomoo`0>Ool00`00Oomoo`0?Ool00`00Oomoo`0>Ool00`00Oomoo`0>Ool00`00Oomoo`0?Ool0
0`00Oomoo`0>Ool00`00Oomoo`0>Ool00`00Oomoo`0?Ool00`00Oomoo`0>Ool00`00Oomoo`0>Ool0
0`00Oomoo`0?Ool00`00Oomoo`0>Ool00`00Oomoo`0>Ool00`00Oomoo`0?Ool00`00Oomoo`0>Ool0
1000Oomoogoo0@00000iOol00`00Oomoo`08Ool00`00Oomoo`0?Ool00`00Oomoo`0>Ool00`00Oomo
o`0>Ool00`00Oomoo`0?Ool00`00Oomoo`0>Ool00`00Oomoo`0>Ool00`00Oomoo`0?Ool00`00Oomo
o`0>Ool00`00Oomoo`0>Ool00`00Oomoo`0?Ool00`00Oomoo`0>Ool00`00Oomoo`0>Ool00`00Oomo
o`0?Ool00`00Oomoo`0>Ool00`00Oomoo`0>Ool00`00Oomoo`0?Ool00`00Oomoo`0>Ool00`00Oomo
o`0>Ool00`00Oomoo`0?Ool00`00Oomoo`0>Ool00`00Oomoo`0>Ool00`00Oomoo`0?Ool00`00Oomo
o`0>Ool00`00Oomoo`0>Ool00`00Oomoo`0?Ool00`00Oomoo`0>Ool01000Oomoogoo0@00000iOooo
003D00000?mooomoo`ioo`00ogooogoo3Woo0000\
\>"],
  ImageRangeCache->{{{0, 523.813}, {323.188, 0}} -> {-19.8249, -195.032, 
  0.432451, 0.226248}}]
}, Open  ]],

Cell["\<\
After subtracting the mean value from the time series we compute the Fourier \
series and estimate the power spectral density (no windowing function)\
\>", "Text"],

Cell[BoxData[
    \(ts = \(Transpose[data]\)\[LeftDoubleBracket]2\[RightDoubleBracket]; \n
    ts\  = \ ts\  - \ Mean[ts]; \)], "Input"],

Cell[BoxData[
    \(PowerSpectrum[l_List]\  := \ 
      With[{len\  = \ Round[Length\ [l]\/2]}, 
        Abs[Take\ [l, len]]\^2\  + \ 
          Abs[Prepend\ [Reverse\ [Take\ [l, \(-len\) + 1]], 0]]\^2]\)], 
  "Input"],

Cell[BoxData[
    \(ps = PowerSpectrum[Fourier[ts]]; \n
    ps = Transpose[{Table[i/128. , {i, 0, Length[ps] - 1}], ps}]; \)], "Input"],

Cell[TextData[
"Plotting the result we clearly see the peak at the microseismic frequency \
around 0.1\[Dash]0.2 Hz."], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(LogLogListPlot[Drop[ps, 1], plotopt, 
      PlotRange \[Rule] {{0.01, 8}, {0.3, 100000}}]; \)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.688921 0.344461 0.0585124 0.111904 [
[.24077 -0.0125 -14.5625 -16.375 ]
[.24077 -0.0125 14.5625 0 ]
[.34446 -0.0125 -10.9688 -16.375 ]
[.34446 -0.0125 10.9688 0 ]
[.58523 -0.0125 -10.9688 -16.375 ]
[.58523 -0.0125 10.9688 0 ]
[.68892 -0.0125 -5.59375 -16.375 ]
[.68892 -0.0125 5.59375 0 ]
[.92969 -0.0125 -5.59375 -16.375 ]
[.92969 -0.0125 5.59375 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .05851 -11.1875 -8.1875 ]
[-0.0125 .05851 0 8.1875 ]
[-0.0125 .17042 -18.375 -8.1875 ]
[-0.0125 .17042 0 8.1875 ]
[-0.0125 .28232 -25.5625 -8.1875 ]
[-0.0125 .28232 0 8.1875 ]
[-0.0125 .39423 -32.75 -8.1875 ]
[-0.0125 .39423 0 8.1875 ]
[-0.0125 .50613 -39.9375 -8.1875 ]
[-0.0125 .50613 0 8.1875 ]
[-0.0125 .61803 -50.6875 -8.1875 ]
[-0.0125 .61803 0 8.1875 ]
[ 0 0 -0.125 0 ]
[0 .63053 -14.5625 0 ]
[0 .63053 14.5625 16.375 ]
[.24077 .63053 -14.5625 0 ]
[.24077 .63053 14.5625 16.375 ]
[.34446 .63053 -10.9688 0 ]
[.34446 .63053 10.9688 16.375 ]
[.58523 .63053 -10.9688 0 ]
[.58523 .63053 10.9688 16.375 ]
[.68892 .63053 -5.59375 0 ]
[.68892 .63053 5.59375 16.375 ]
[.92969 .63053 -5.59375 0 ]
[.92969 .63053 5.59375 16.375 ]
[ 0 .61803 .125 0 ]
[1.0125 .05851 0 -8.1875 ]
[1.0125 .05851 11.1875 8.1875 ]
[1.0125 .17042 0 -8.1875 ]
[1.0125 .17042 18.375 8.1875 ]
[1.0125 .28232 0 -8.1875 ]
[1.0125 .28232 25.5625 8.1875 ]
[1.0125 .39423 0 -8.1875 ]
[1.0125 .39423 32.75 8.1875 ]
[1.0125 .50613 0 -8.1875 ]
[1.0125 .50613 39.9375 8.1875 ]
[ 1 0 .125 0 ]
[ -0.0005 -0.0005 0 0 ]
[ 1.0005 .61853 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
0 0 m
0 .61803 L
s
.24077 0 m
.24077 .61803 L
s
.34446 0 m
.34446 .61803 L
s
.58523 0 m
.58523 .61803 L
s
.68892 0 m
.68892 .61803 L
s
.92969 0 m
.92969 .61803 L
s
.10369 0 m
.10369 .61803 L
s
.16435 0 m
.16435 .61803 L
s
.20739 0 m
.20739 .61803 L
s
.26804 0 m
.26804 .61803 L
s
.2911 0 m
.2911 .61803 L
s
.31108 0 m
.31108 .61803 L
s
.3287 0 m
.3287 .61803 L
s
.44815 0 m
.44815 .61803 L
s
.50881 0 m
.50881 .61803 L
s
.55185 0 m
.55185 .61803 L
s
.6125 0 m
.6125 .61803 L
s
.63556 0 m
.63556 .61803 L
s
.65554 0 m
.65554 .61803 L
s
.67316 0 m
.67316 .61803 L
s
.79261 0 m
.79261 .61803 L
s
.85327 0 m
.85327 .61803 L
s
.89631 0 m
.89631 .61803 L
s
.95696 0 m
.95696 .61803 L
s
.98002 0 m
.98002 .61803 L
s
0 .05851 m
1 .05851 L
s
0 .17042 m
1 .17042 L
s
0 .28232 m
1 .28232 L
s
0 .39423 m
1 .39423 L
s
0 .50613 m
1 .50613 L
s
0 .61803 m
1 .61803 L
s
0 0 m
1 0 L
s
0 .01398 m
1 .01398 L
s
0 .02483 m
1 .02483 L
s
0 .03369 m
1 .03369 L
s
0 .04118 m
1 .04118 L
s
0 .04767 m
1 .04767 L
s
0 .05339 m
1 .05339 L
s
0 .0922 m
1 .0922 L
s
0 .1119 m
1 .1119 L
s
0 .12589 m
1 .12589 L
s
0 .13673 m
1 .13673 L
s
0 .14559 m
1 .14559 L
s
0 .15308 m
1 .15308 L
s
0 .15957 m
1 .15957 L
s
0 .1653 m
1 .1653 L
s
0 .2041 m
1 .2041 L
s
0 .22381 m
1 .22381 L
s
0 .23779 m
1 .23779 L
s
0 .24863 m
1 .24863 L
s
0 .2575 m
1 .2575 L
s
0 .26499 m
1 .26499 L
s
0 .27148 m
1 .27148 L
s
0 .2772 m
1 .2772 L
s
0 .31601 m
1 .31601 L
s
0 .33571 m
1 .33571 L
s
0 .34969 m
1 .34969 L
s
0 .36054 m
1 .36054 L
s
0 .3694 m
1 .3694 L
s
0 .37689 m
1 .37689 L
s
0 .38338 m
1 .38338 L
s
0 .3891 m
1 .3891 L
s
0 .42791 m
1 .42791 L
s
0 .44762 m
1 .44762 L
s
0 .4616 m
1 .4616 L
s
0 .47244 m
1 .47244 L
s
0 .4813 m
1 .4813 L
s
0 .4888 m
1 .4888 L
s
0 .49529 m
1 .49529 L
s
0 .50101 m
1 .50101 L
s
0 .53982 m
1 .53982 L
s
0 .55952 m
1 .55952 L
s
0 .5735 m
1 .5735 L
s
0 .58435 m
1 .58435 L
s
0 .59321 m
1 .59321 L
s
0 .6007 m
1 .6007 L
s
0 .60719 m
1 .60719 L
s
0 .61291 m
1 .61291 L
s
0 g
.24077 0 m
.24077 .00625 L
s
gsave
.24077 -0.0125 -75.5625 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0.05) show
88.125000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.34446 0 m
.34446 .00625 L
s
gsave
.34446 -0.0125 -71.9688 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0.1) show
80.937500 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.58523 0 m
.58523 .00625 L
s
gsave
.58523 -0.0125 -71.9688 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0.5) show
80.937500 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.68892 0 m
.68892 .00625 L
s
gsave
.68892 -0.0125 -66.5938 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(1) show
70.187500 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.92969 0 m
.92969 .00625 L
s
gsave
.92969 -0.0125 -66.5938 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(5) show
70.187500 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.001 w
.10369 0 m
.10369 .00375 L
s
.16435 0 m
.16435 .00375 L
s
.20739 0 m
.20739 .00375 L
s
.26804 0 m
.26804 .00375 L
s
.2911 0 m
.2911 .00375 L
s
.31108 0 m
.31108 .00375 L
s
.3287 0 m
.3287 .00375 L
s
.44815 0 m
.44815 .00375 L
s
.50881 0 m
.50881 .00375 L
s
.55185 0 m
.55185 .00375 L
s
.6125 0 m
.6125 .00375 L
s
.63556 0 m
.63556 .00375 L
s
.65554 0 m
.65554 .00375 L
s
.67316 0 m
.67316 .00375 L
s
.79261 0 m
.79261 .00375 L
s
.85327 0 m
.85327 .00375 L
s
.89631 0 m
.89631 .00375 L
s
.95696 0 m
.95696 .00375 L
s
.98002 0 m
.98002 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .05851 m
.00625 .05851 L
s
gsave
-0.0125 .05851 -72.1875 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(1) show
70.187500 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
0 .17042 m
.00625 .17042 L
s
gsave
-0.0125 .17042 -79.375 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(10) show
77.375000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
0 .28232 m
.00625 .28232 L
s
gsave
-0.0125 .28232 -86.5625 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(100) show
84.562500 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
0 .39423 m
.00625 .39423 L
s
gsave
-0.0125 .39423 -93.75 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(1000) show
91.750000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
0 .50613 m
.00625 .50613 L
s
gsave
-0.0125 .50613 -100.938 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(10000) show
98.937500 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
0 .61803 m
.00625 .61803 L
s
gsave
-0.0125 .61803 -111.688 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(100000.) show
109.687500 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.001 w
0 0 m
.00375 0 L
s
0 .01398 m
.00375 .01398 L
s
0 .02483 m
.00375 .02483 L
s
0 .03369 m
.00375 .03369 L
s
0 .04118 m
.00375 .04118 L
s
0 .04767 m
.00375 .04767 L
s
0 .05339 m
.00375 .05339 L
s
0 .0922 m
.00375 .0922 L
s
0 .1119 m
.00375 .1119 L
s
0 .12589 m
.00375 .12589 L
s
0 .13673 m
.00375 .13673 L
s
0 .14559 m
.00375 .14559 L
s
0 .15308 m
.00375 .15308 L
s
0 .15957 m
.00375 .15957 L
s
0 .1653 m
.00375 .1653 L
s
0 .2041 m
.00375 .2041 L
s
0 .22381 m
.00375 .22381 L
s
0 .23779 m
.00375 .23779 L
s
0 .24863 m
.00375 .24863 L
s
0 .2575 m
.00375 .2575 L
s
0 .26499 m
.00375 .26499 L
s
0 .27148 m
.00375 .27148 L
s
0 .2772 m
.00375 .2772 L
s
0 .31601 m
.00375 .31601 L
s
0 .33571 m
.00375 .33571 L
s
0 .34969 m
.00375 .34969 L
s
0 .36054 m
.00375 .36054 L
s
0 .3694 m
.00375 .3694 L
s
0 .37689 m
.00375 .37689 L
s
0 .38338 m
.00375 .38338 L
s
0 .3891 m
.00375 .3891 L
s
0 .42791 m
.00375 .42791 L
s
0 .44762 m
.00375 .44762 L
s
0 .4616 m
.00375 .4616 L
s
0 .47244 m
.00375 .47244 L
s
0 .4813 m
.00375 .4813 L
s
0 .4888 m
.00375 .4888 L
s
0 .49529 m
.00375 .49529 L
s
0 .50101 m
.00375 .50101 L
s
0 .53982 m
.00375 .53982 L
s
0 .55952 m
.00375 .55952 L
s
0 .5735 m
.00375 .5735 L
s
0 .58435 m
.00375 .58435 L
s
0 .59321 m
.00375 .59321 L
s
0 .6007 m
.00375 .6007 L
s
0 .60719 m
.00375 .60719 L
s
0 .61291 m
.00375 .61291 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 .61178 m
0 .61803 L
s
gsave
0 .63053 -75.5625 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0.01) show
88.125000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.24077 .61178 m
.24077 .61803 L
s
gsave
.24077 .63053 -75.5625 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0.05) show
88.125000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.34446 .61178 m
.34446 .61803 L
s
gsave
.34446 .63053 -71.9688 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0.1) show
80.937500 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.58523 .61178 m
.58523 .61803 L
s
gsave
.58523 .63053 -71.9688 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0.5) show
80.937500 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.68892 .61178 m
.68892 .61803 L
s
gsave
.68892 .63053 -66.5938 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(1) show
70.187500 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.92969 .61178 m
.92969 .61803 L
s
gsave
.92969 .63053 -66.5938 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(5) show
70.187500 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.001 w
.10369 .61428 m
.10369 .61803 L
s
.16435 .61428 m
.16435 .61803 L
s
.20739 .61428 m
.20739 .61803 L
s
.26804 .61428 m
.26804 .61803 L
s
.2911 .61428 m
.2911 .61803 L
s
.31108 .61428 m
.31108 .61803 L
s
.3287 .61428 m
.3287 .61803 L
s
.44815 .61428 m
.44815 .61803 L
s
.50881 .61428 m
.50881 .61803 L
s
.55185 .61428 m
.55185 .61803 L
s
.6125 .61428 m
.6125 .61803 L
s
.63556 .61428 m
.63556 .61803 L
s
.65554 .61428 m
.65554 .61803 L
s
.67316 .61428 m
.67316 .61803 L
s
.79261 .61428 m
.79261 .61803 L
s
.85327 .61428 m
.85327 .61803 L
s
.89631 .61428 m
.89631 .61803 L
s
.95696 .61428 m
.95696 .61803 L
s
.98002 .61428 m
.98002 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .05851 m
1 .05851 L
s
gsave
1.0125 .05851 -61 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(1) show
70.187500 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.99375 .17042 m
1 .17042 L
s
gsave
1.0125 .17042 -61 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(10) show
77.375000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.99375 .28232 m
1 .28232 L
s
gsave
1.0125 .28232 -61 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(100) show
84.562500 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.99375 .39423 m
1 .39423 L
s
gsave
1.0125 .39423 -61 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(1000) show
91.750000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.99375 .50613 m
1 .50613 L
s
gsave
1.0125 .50613 -61 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(10000) show
98.937500 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.001 w
.99625 0 m
1 0 L
s
.99625 .01398 m
1 .01398 L
s
.99625 .02483 m
1 .02483 L
s
.99625 .03369 m
1 .03369 L
s
.99625 .04118 m
1 .04118 L
s
.99625 .04767 m
1 .04767 L
s
.99625 .05339 m
1 .05339 L
s
.99625 .0922 m
1 .0922 L
s
.99625 .1119 m
1 .1119 L
s
.99625 .12589 m
1 .12589 L
s
.99625 .13673 m
1 .13673 L
s
.99625 .14559 m
1 .14559 L
s
.99625 .15308 m
1 .15308 L
s
.99625 .15957 m
1 .15957 L
s
.99625 .1653 m
1 .1653 L
s
.99625 .2041 m
1 .2041 L
s
.99625 .22381 m
1 .22381 L
s
.99625 .23779 m
1 .23779 L
s
.99625 .24863 m
1 .24863 L
s
.99625 .2575 m
1 .2575 L
s
.99625 .26499 m
1 .26499 L
s
.99625 .27148 m
1 .27148 L
s
.99625 .2772 m
1 .2772 L
s
.99625 .31601 m
1 .31601 L
s
.99625 .33571 m
1 .33571 L
s
.99625 .34969 m
1 .34969 L
s
.99625 .36054 m
1 .36054 L
s
.99625 .3694 m
1 .3694 L
s
.99625 .37689 m
1 .37689 L
s
.99625 .38338 m
1 .38338 L
s
.99625 .3891 m
1 .3891 L
s
.99625 .42791 m
1 .42791 L
s
.99625 .44762 m
1 .44762 L
s
.99625 .4616 m
1 .4616 L
s
.99625 .47244 m
1 .47244 L
s
.99625 .4813 m
1 .4813 L
s
.99625 .4888 m
1 .4888 L
s
.99625 .49529 m
1 .49529 L
s
.99625 .50101 m
1 .50101 L
s
.99625 .53982 m
1 .53982 L
s
.99625 .55952 m
1 .55952 L
s
.99625 .5735 m
1 .5735 L
s
.99625 .58435 m
1 .58435 L
s
.99625 .59321 m
1 .59321 L
s
.99625 .6007 m
1 .6007 L
s
.99625 .60719 m
1 .60719 L
s
.99625 .61291 m
1 .61291 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
0 .40937 m
.06676 .50537 L
.12742 .53825 L
.17046 .40991 L
.20384 .43744 L
.23111 .1711 L
.25417 .3922 L
.27415 .35763 L
.29177 .3007 L
.30753 .31187 L
.32179 .14287 L
.33481 .35579 L
.34678 .305 L
.35787 .36567 L
.36819 .39944 L
.37784 .49478 L
.38691 .54953 L
.39546 .49442 L
.40355 .50515 L
.41122 .50719 L
.41852 .46676 L
.42548 .48313 L
.43213 .47554 L
.4385 .45405 L
.44461 .38544 L
.45047 .39725 L
.45612 .33215 L
.46156 .34607 L
.46681 .25877 L
.47188 .31982 L
.47679 .32478 L
.48154 .20487 L
.48614 .27513 L
.4906 .317 L
.49494 .27362 L
.49916 .22958 L
.50325 .2836 L
.50724 .29471 L
.51113 .24048 L
.51492 .25852 L
.51861 .28927 L
.52222 .26415 L
.52574 .17542 L
.52918 .22198 L
.53254 .01445 L
.53582 .26103 L
.53904 .20508 L
.54219 .24552 L
.54528 .18027 L
.5483 .2546 L
Mistroke
.55126 .1538 L
.55417 .23687 L
.55702 .04773 L
.55981 .15683 L
.56256 .13456 L
.56525 .21244 L
.5679 .17138 L
.5705 .1615 L
.57306 .1636 L
.57557 .20277 L
.57805 .20193 L
.58048 .16595 L
.58287 .16912 L
.58523 .11737 L
.58755 .18817 L
.58983 .1032 L
.59208 .11569 L
.5943 .18599 L
.59648 .16612 L
.59863 .12782 L
.60076 .09203 L
.60285 .10973 L
.60491 .0358 L
.60695 .14404 L
.60896 .10938 L
.61094 .13961 L
.61289 .13517 L
.61482 .11554 L
.61673 .14577 L
.61861 .20409 L
Mfstroke
.61861 .20409 m
.62041 0 L
s
.62054 0 m
.6223 .15951 L
.62412 .14526 L
.62591 .07871 L
.62768 .16951 L
.62943 .10976 L
.63116 .11744 L
.63287 .06734 L
.63456 .13495 L
.63623 .15103 L
.63788 .15543 L
.63952 .10667 L
.64114 .09573 L
.64274 .13998 L
.64432 .12075 L
.64588 .0734 L
.64743 .05352 L
.64897 .12069 L
.65049 .0965 L
.65199 .14111 L
.65348 .07804 L
.65495 .09906 L
.65641 .12748 L
.65786 .11303 L
.65929 .10504 L
.66071 .07147 L
.66211 .1092 L
.6635 .10724 L
.66488 .11219 L
.66625 .06649 L
.6676 .09192 L
.66895 .0631 L
.67027 .13056 L
.67159 .11006 L
.6729 .12689 L
.67419 .09621 L
.67548 .08743 L
.67675 .07782 L
.67801 .11186 L
.67927 .11477 L
.68051 .08461 L
.68174 .0762 L
.68296 .08123 L
.68417 .09118 L
.68537 .0742 L
.68657 .09291 L
.68775 .09004 L
.68892 .08396 L
.69009 .07895 L
.69124 .10817 L
Mistroke
.69239 .0889 L
.69352 .05763 L
.69465 .0825 L
.69577 .05658 L
.69689 .08642 L
.69799 .08146 L
.69909 .083 L
.70017 .09047 L
.70125 .08019 L
Mfstroke
.70125 .08019 m
.70205 0 L
s
.7026 0 m
.70339 .08062 L
.70445 .06158 L
.7055 .00742 L
.70654 .06858 L
.70758 .04959 L
.7086 .08308 L
.70963 .06591 L
.71064 .08171 L
.71165 .05223 L
.71265 .05809 L
.71364 .09876 L
.71463 .07527 L
.71561 .06751 L
.71659 .10573 L
.71755 .06446 L
.71852 .07585 L
.71947 .05744 L
.72042 .08245 L
s
.72042 .08245 m
.72124 0 L
s
.7215 0 m
.7223 .07638 L
.72323 .04729 L
.72416 .07357 L
.72508 .06419 L
s
.72508 .06419 m
.72598 0 L
s
.72601 0 m
.72691 .04796 L
.72781 .086 L
.72871 .0894 L
.7296 .04807 L
.73049 .07428 L
.73137 .07232 L
.73225 .10996 L
.73312 .07595 L
.73399 .02524 L
.73485 .05801 L
.73571 .09188 L
.73656 .07089 L
.73741 .06947 L
.73825 .0316 L
.73909 .07439 L
.73992 .06466 L
.74075 .05059 L
.74158 .06203 L
.7424 .0967 L
.74321 .07267 L
.74402 .07529 L
.74483 .05138 L
.74563 .07376 L
.74643 .10064 L
.74722 .08354 L
.74801 .05402 L
.7488 .06504 L
.74958 .03511 L
.75035 .02181 L
.75113 .03016 L
s
.75113 .03016 m
.75155 0 L
s
.75213 0 m
.75266 .05757 L
.75342 .03812 L
.75418 .06999 L
.75493 .07779 L
.75568 .10277 L
.75643 .02985 L
.75717 .05232 L
.75791 .0879 L
.75865 .02568 L
.75938 .02193 L
.76011 .0719 L
.76083 .04499 L
.76155 .02186 L
.76227 .05862 L
.76298 .03643 L
.76369 .01872 L
.7644 .0298 L
.76511 .09359 L
.76581 .04782 L
.7665 .10742 L
s
.7665 .10742 m
.76717 0 L
s
.7684 0 m
.76858 .0249 L
.76926 .11828 L
.76994 .10496 L
.77062 .06129 L
.7713 .0487 L
s
.7713 .0487 m
.77171 0 L
s
.77239 0 m
.77264 .01753 L
.7733 .08255 L
.77397 .01731 L
.77463 .08489 L
.77529 .1537 L
.77594 .06672 L
.77659 .10126 L
.77724 .10429 L
.77789 .04106 L
.77853 .02094 L
.77917 .094 L
s
.77917 .094 m
.77977 0 L
s
.78077 0 m
.78108 .10951 L
.78171 .03581 L
.78233 .0907 L
.78296 .02294 L
s
.78296 .02294 m
.78307 0 L
s
.78392 0 m
.7842 .09195 L
s
.7842 .09195 m
.78468 0 L
s
.78501 0 m
.78543 .05996 L
.78604 .05247 L
.78665 .06695 L
s
.78665 .06695 m
.78707 0 L
s
.78739 0 m
.78786 .10313 L
.78847 .03051 L
.78907 .07177 L
.78966 .07794 L
.79026 .0786 L
.79085 .00896 L
.79144 .08574 L
.79203 .0588 L
.79261 .11192 L
.7932 .06688 L
.79378 .06997 L
.79436 .04785 L
s
.79436 .04785 m
.79483 0 L
s
.79586 0 m
.79608 .04919 L
s
.79608 .04919 m
.79641 0 L
s
.79685 0 m
.79722 .06873 L
.79778 .08849 L
.79835 .00865 L
.79891 .08347 L
.79947 .08125 L
.80002 .07063 L
.80058 .06539 L
.80113 .0544 L
.80168 .00752 L
s
.80168 .00752 m
.8017 0 L
s
.80283 0 m
.80332 .08426 L
.80387 .06408 L
.80441 .07908 L
s
.80441 .07908 m
.80489 0 L
s
.80615 0 m
.80655 .09125 L
s
.80655 .09125 m
.80707 0 L
s
.8071 0 m
.80761 .05774 L
.80814 .05111 L
s
.80814 .05111 m
.80856 0 L
s
.80879 0 m
.80919 .04798 L
.80971 .04593 L
.81023 .02969 L
.81075 .02809 L
.81127 .0104 L
.81178 .07819 L
.8123 .07937 L
.81281 .13116 L
.81332 .02469 L
.81383 .06468 L
.81433 .02965 L
s
.81433 .02965 m
.81447 0 L
s
.81534 0 m
.81584 .06678 L
.81634 .08479 L
.81684 .04944 L
.81733 .06178 L
.81783 .05019 L
.81832 .02638 L
s
.81832 .02638 m
.81862 0 L
s
.81914 0 m
.8193 .00829 L
.81979 .03207 L
.82028 .05129 L
s
.82028 .05129 m
.82063 0 L
s
.82091 0 m
.82125 .04583 L
.82173 .01589 L
.82221 .10149 L
s
.82221 .10149 m
.82254 0 L
s
.82284 0 m
.82316 .09444 L
.82364 .0702 L
s
.82364 .0702 m
.82389 0 L
s
.82448 0 m
.82459 .01709 L
.82506 .05458 L
.82553 .04577 L
.826 .08733 L
s
.826 .08733 m
.82618 0 L
s
.82717 0 m
.82739 .03691 L
.82785 .08779 L
s
.82785 .08779 m
.8281 0 L
s
.82879 0 m
.82923 .07556 L
.82969 .04174 L
.83015 .07897 L
.8306 .02648 L
s
.8306 .02648 m
.83084 0 L
s
.83126 0 m
.8315 .02749 L
s
.8315 .02749 m
.83157 0 L
s
.83319 0 m
.83329 .0275 L
.83374 .05414 L
s
.83374 .05414 m
.83415 0 L
s
.83421 0 m
.83462 .05206 L
s
.83462 .05206 m
.83493 0 L
s
.83518 0 m
.8355 .06423 L
.83594 .08242 L
.83638 .04093 L
.83681 .05233 L
s
.83681 .05233 m
.83718 0 L
s
.8379 0 m
.83811 .01623 L
.83854 .00705 L
.83897 .06654 L
.8394 .04277 L
.83983 .07341 L
s
.83983 .07341 m
.84025 0 L
s
.84026 0 m
.84068 .02726 L
.8411 .00718 L
.84152 .02801 L
.84194 .08949 L
s
.84194 .08949 m
.84218 0 L
s
.8426 0 m
.84278 .05161 L
s
.84278 .05161 m
.84301 0 L
s
.84347 0 m
.84362 .02321 L
.84403 .02821 L
s
.84403 .02821 m
.8442 0 L
s
.84464 0 m
.84486 .04333 L
.84527 .00974 L
s
.84527 .00974 m
.8456 0 L
s
.84571 0 m
.84609 .02694 L
.8465 .04303 L
.8469 .01531 L
s
.8469 .01531 m
.847 0 L
s
.84786 0 m
.84812 .03671 L
s
.84812 .03671 m
.84843 0 L
s
.84926 0 m
.84932 .01843 L
.84972 .00996 L
.85012 .00611 L
s
.85012 .00611 m
.85032 0 L
s
.85069 0 m
.85091 .00793 L
s
.85091 .00793 m
.85095 0 L
s
.85269 0 m
.85288 .02603 L
.85327 .04787 L
s
.85327 .04787 m
.85351 0 L
s
.85393 0 m
.85405 .01203 L
.85443 .04833 L
s
.85443 .04833 m
.85479 0 L
s
.85545 0 m
.85559 .03911 L
s
.85559 .03911 m
.85575 0 L
s
.85655 0 m
.85674 .03288 L
s
.85674 .03288 m
.85709 0 L
s
.85714 0 m
.8575 .03504 L
s
.8575 .03504 m
.85776 0 L
s
.85884 0 m
.859 .06125 L
.85938 .07158 L
.85975 .04216 L
s
.85975 .04216 m
.8599 0 L
s
.86039 0 m
.8605 .02713 L
.86087 .08388 L
.86124 .03972 L
s
.86124 .03972 m
.86145 0 L
s
.8637 0 m
.8638 .0269 L
s
.8638 .0269 m
.86393 0 L
s
.86514 0 m
.86524 .02644 L
.8656 .0892 L
.86596 .07583 L
.86632 .11105 L
s
.86632 .11105 m
.86657 0 L
s
.86687 0 m
.86703 .03725 L
.86739 .02471 L
.86774 .0753 L
s
.86774 .0753 m
.86799 0 L
s
.86873 0 m
.8688 .00922 L
.86915 .02357 L
.8695 .02745 L
s
.8695 .02745 m
.86983 0 L
s
.87028 0 m
.87054 .03722 L
s
.87054 .03722 m
.8708 0 L
s
.87105 0 m
.87124 .01701 L
.87158 .05223 L
s
.87158 .05223 m
.87168 0 L
s
.87242 0 m
.87261 .03032 L
.87295 .11871 L
.8733 .10862 L
.87364 .07812 L
s
.87364 .07812 m
.87396 0 L
s
.87412 0 m
.87431 .00453 L
.87465 4e-005 L
s
.87465 4e-005 m
.87465 0 L
s
.87561 0 m
.87566 .01483 L
.876 .01634 L
.87633 .08198 L
s
.87633 .08198 m
.87659 0 L
s
.87684 0 m
.877 .02214 L
.87733 .05994 L
.87766 .09256 L
s
.87766 .09256 m
.87787 0 L
s
.87838 0 m
.87865 .06101 L
.87898 .02477 L
s
.87898 .02477 m
.87907 0 L
s
.87945 0 m
.87963 .07324 L
.87996 .09126 L
.88029 .08558 L
s
.88029 .08558 m
.88043 0 L
s
.8808 0 m
.88093 .06894 L
.88126 .06401 L
s
.88126 .06401 m
.88134 0 L
s
.88179 0 m
.8819 .09221 L
.88222 .1046 L
.88254 .08135 L
.88286 .02298 L
s
.88286 .02298 m
.88311 0 L
s
.88322 0 m
.8835 .0489 L
.88382 .01889 L
s
.88382 .01889 m
.88388 0 L
s
.88431 0 m
.88445 .06615 L
s
.88445 .06615 m
.88475 0 L
s
.88479 0 m
.88509 .07426 L
s
.88509 .07426 m
.88538 0 L
s
.88546 0 m
.88571 .02221 L
.88603 .07098 L
.88634 .10277 L
.88665 .10316 L
.88696 .00774 L
s
.88696 .00774 m
.88705 0 L
s
.88732 0 m
.88758 .10596 L
.88789 .03093 L
.8882 .09124 L
.88851 .10083 L
.88882 .03954 L
s
.88882 .03954 m
.88899 0 L
s
.88981 0 m
.89004 .05503 L
.89035 .13884 L
.89065 .09327 L
.89095 .13614 L
s
.89095 .13614 m
.89108 0 L
s
.89148 0 m
.89156 .06601 L
.89186 .01419 L
.89216 .0384 L
.89246 .06522 L
.89276 .05756 L
.89306 .07388 L
s
.89306 .07388 m
.89331 0 L
s
.89384 0 m
.89395 .07227 L
.89425 .06191 L
.89454 .10061 L
.89484 .01712 L
.89513 .13667 L
.89543 .22243 L
.89572 .06505 L
.89601 .13782 L
s
.89601 .13782 m
.8963 0 L
s
.89632 0 m
.8966 .12073 L
s
.8966 .12073 m
.89674 0 L
s
.89713 0 m
.89718 .02461 L
.89747 .04971 L
.89776 .04532 L
.89805 .07101 L
.89834 .15174 L
.89863 .1219 L
.89891 .10663 L
.8992 .11244 L
.89949 .09865 L
s
.89949 .09865 m
.89962 0 L
s
.89993 0 m
.90006 .1004 L
.90034 .11857 L
.90063 .03215 L
.90091 .10727 L
.90119 .11635 L
.90148 .09124 L
.90176 .1184 L
.90204 .09902 L
.90232 .08316 L
.9026 .05189 L
.90288 .03895 L
.90316 .12111 L
.90344 .0346 L
.90372 .02501 L
.90399 .03613 L
.90427 .17519 L
.90455 .02794 L
.90483 .04783 L
.9051 .04632 L
.90538 .04424 L
.90565 .02236 L
.90593 .11497 L
.9062 .02302 L
s
.9062 .02302 m
.90633 0 L
s
.90654 0 m
.90675 .08162 L
.90702 .12632 L
.90729 .06161 L
.90756 .1002 L
.90783 .01755 L
.9081 .00015 L
.90837 .05046 L
s
.90837 .05046 m
.90846 0 L
s
.90876 0 m
.90891 .12642 L
.90918 .07932 L
.90945 .0747 L
.90971 .07203 L
s
.90971 .07203 m
.90995 0 L
s
.91001 0 m
.91025 .07723 L
s
.91025 .07723 m
.9105 0 L
s
.91054 0 m
.91078 .04024 L
.91104 .16103 L
.91131 .15949 L
.91157 .00371 L
.91183 .10957 L
.9121 .01668 L
.91236 .02901 L
.91262 .09888 L
.91288 .01808 L
.91315 .0797 L
.91341 .05181 L
.91367 .10785 L
.91393 .12968 L
s
.91393 .12968 m
.91418 0 L
s
.91421 0 m
.91445 .05273 L
.9147 .0865 L
.91496 .08059 L
.91522 .14267 L
.91548 .01298 L
s
.91548 .01298 m
.9155 0 L
s
.91591 0 m
.91599 .05752 L
s
.91599 .05752 m
.91613 0 L
s
.91648 0 m
.9165 .00575 L
.91676 .07293 L
.91701 .09085 L
.91727 .07002 L
.91752 .09759 L
.91777 .13337 L
.91803 .11069 L
.91828 .10505 L
.91853 .04629 L
.91878 .02785 L
.91903 .05277 L
.91928 .02401 L
.91953 .04065 L
s
.91953 .04065 m
.91964 0 L
s
.91988 0 m
.92003 .08659 L
.92028 .05597 L
.92053 .02679 L
s
.92053 .02679 m
.92066 0 L
s
.92082 0 m
.92103 .11816 L
.92128 .0763 L
.92152 .0162 L
.92177 .09355 L
.92202 .07463 L
.92226 .10739 L
.92251 .12659 L
.92275 .11586 L
.923 .11293 L
.92324 .11987 L
.92348 .07406 L
.92373 .1125 L
s
.92373 .1125 m
.92391 0 L
s
.92408 0 m
.92421 .0426 L
.92446 .10345 L
.9247 .14198 L
.92494 .15811 L
.92518 .09811 L
.92542 .13058 L
.92566 .05446 L
.9259 .04874 L
.92614 .10278 L
.92638 .03765 L
.92662 .03216 L
.92686 .16651 L
.9271 .13684 L
.92733 .17567 L
.92757 .13968 L
s
.92757 .13968 m
.92777 0 L
s
.92805 0 m
.92828 .25423 L
.92852 .14212 L
.92875 .07602 L
.92899 .15223 L
s
.92899 .15223 m
.92916 0 L
s
.92943 0 m
.92945 .00605 L
.92969 .13112 L
s
.92969 .13112 m
.92984 0 L
s
.93 0 m
.93016 .12468 L
.93039 .10461 L
.93062 .07752 L
.93085 .10954 L
.93108 .11127 L
.93132 .02554 L
s
.93132 .02554 m
.93138 0 L
s
.93181 0 m
.93201 .11779 L
.93224 .07592 L
.93247 .15776 L
.9327 .17857 L
.93293 .09137 L
.93315 .11317 L
.93338 .06557 L
s
.93338 .06557 m
.93349 0 L
s
.9337 0 m
.93384 .09846 L
.93407 .08895 L
.93429 .11223 L
.93452 .22465 L
.93474 .08388 L
s
.93474 .08388 m
.93489 0 L
s
.93507 0 m
.9352 .06362 L
.93542 .11423 L
.93565 .14642 L
.93587 .03477 L
s
.93587 .03477 m
.93598 0 L
s
.93672 0 m
.93676 .03379 L
.93699 .12791 L
.93721 .14913 L
.93743 .1398 L
.93765 .13797 L
.93788 .12874 L
s
.93788 .12874 m
.93808 0 L
s
.93811 0 m
.93832 .10865 L
s
.93832 .10865 m
.93849 0 L
s
.93858 0 m
.93876 .1149 L
.93898 .08687 L
s
.93898 .08687 m
.93905 0 L
s
.93934 0 m
.93942 .11002 L
.93964 .03019 L
.93985 .00361 L
.94007 .09622 L
.94029 .03095 L
.94051 .11043 L
.94072 .10386 L
.94094 .12494 L
.94116 .25477 L
.94137 .23718 L
.94159 .0611 L
.94181 .13562 L
.94202 .08529 L
.94224 .00772 L
.94245 .06294 L
.94267 .02237 L
.94288 .07967 L
.94309 .12967 L
.94331 .07402 L
s
.94331 .07402 m
.94341 0 L
s
.94361 0 m
.94373 .1104 L
s
.94373 .1104 m
.94391 0 L
s
.94401 0 m
.94416 .04901 L
.94437 .09399 L
.94458 .18758 L
.94479 .10099 L
.94501 .04168 L
.94522 .09522 L
.94543 .09793 L
.94564 .09185 L
.94585 .06401 L
.94606 .06939 L
.94627 .13667 L
s
.94627 .13667 m
.94638 0 L
s
.94659 0 m
.94668 .08674 L
.94689 .12231 L
.9471 .15918 L
.94731 .10579 L
.94752 .12094 L
.94772 .06459 L
s
.94772 .06459 m
.94793 0 L
s
.94794 0 m
.94814 .02661 L
.94834 .15528 L
.94855 .07947 L
s
.94855 .07947 m
.94868 0 L
s
.94881 0 m
.94896 .12481 L
.94917 .18045 L
.94937 .18953 L
.94958 .0982 L
.94978 .11793 L
.94999 .07719 L
.95019 .17141 L
.95039 .16595 L
s
.95039 .16595 m
.95057 0 L
s
.95062 0 m
.9508 .21028 L
.951 .20508 L
.95121 .10942 L
.95141 .14985 L
.95161 .10843 L
.95181 .07167 L
.95201 .10688 L
.95221 .07334 L
.95241 .04093 L
.95262 .03177 L
.95282 .15453 L
.95302 .12653 L
.95322 .12943 L
.95342 .09074 L
.95361 .11887 L
.95381 .14176 L
.95401 .13817 L
.95421 .14209 L
.95441 .1784 L
.95461 .09196 L
.95481 .11364 L
.955 .14597 L
.9552 .02152 L
.9554 .26979 L
.95559 .17981 L
.95579 .14831 L
.95599 .10277 L
.95618 .05375 L
.95638 .16481 L
.95657 .11383 L
.95677 .14624 L
.95696 .17097 L
.95716 .11912 L
.95735 .1013 L
.95755 .12433 L
.95774 .02513 L
.95793 .12242 L
.95813 .03836 L
.95832 .09393 L
.95851 .16455 L
.95871 .14388 L
.9589 .05809 L
.95909 .13168 L
.95928 .15859 L
.95947 .17641 L
.95967 .07359 L
.95986 .06124 L
s
.95986 .06124 m
.96 0 L
s
.96009 0 m
.96024 .08298 L
.96043 .13187 L
.96062 .07427 L
.96081 .17048 L
.961 .15614 L
.96119 .14331 L
.96138 .06276 L
.96157 .14113 L
.96176 .11201 L
.96194 .09173 L
.96213 .03037 L
.96232 .08552 L
.96251 .04285 L
.9627 .05699 L
.96288 .11199 L
.96307 .09426 L
.96326 .18707 L
.96344 .20157 L
.96363 .1189 L
.96382 .07535 L
.964 .18656 L
.96419 .09678 L
.96437 .15659 L
.96456 .10852 L
.96474 .12554 L
.96493 .17918 L
.96511 .16246 L
.9653 .11334 L
.96548 .20259 L
.96567 .19028 L
.96585 .20564 L
.96603 .12672 L
.96622 .17985 L
.9664 .14399 L
.96658 .25496 L
.96676 .26489 L
.96695 .20545 L
.96713 .23098 L
.96731 .19524 L
.96749 .23152 L
.96767 .12435 L
.96785 .23476 L
.96804 .19374 L
.96822 .22904 L
.9684 .12054 L
.96858 .14075 L
.96876 .17594 L
.96894 .09128 L
.96912 .14448 L
Mistroke
Mfstroke
.96912 .14448 m
.96929 0 L
s
.9693 0 m
.96948 .17904 L
.96966 .20298 L
.96983 .23273 L
.97001 .13806 L
.97019 .19163 L
.97037 .07967 L
.97055 .07504 L
.97073 .12336 L
.9709 .04865 L
.97108 .05226 L
.97126 .05273 L
.97143 .10809 L
.97161 .07847 L
.97179 .11639 L
.97196 .15971 L
.97214 .23313 L
.97232 .15872 L
.97249 .06256 L
.97267 .04967 L
.97284 .07953 L
.97302 .13028 L
.97319 .11128 L
.97337 .08181 L
.97354 .13502 L
.97372 .03052 L
.97389 .16827 L
.97406 .12265 L
.97424 .16848 L
.97441 .16851 L
.97458 .11755 L
.97476 .18908 L
.97493 .04744 L
.9751 .06556 L
.97527 .15025 L
.97545 .15941 L
.97562 .12957 L
.97579 .06688 L
.97596 .12001 L
.97613 .01758 L
.97631 .19549 L
.97648 .08169 L
.97665 .1263 L
.97682 .16557 L
.97699 .15766 L
.97716 .04722 L
.97733 .08722 L
.9775 .14103 L
.97767 .10217 L
.97784 .12343 L
Mistroke
.97801 .14581 L
.97818 .02171 L
.97835 .20635 L
.97851 .1185 L
.97868 .21456 L
.97885 .16772 L
.97902 .05654 L
.97919 .15697 L
.97935 .08854 L
.97952 .05961 L
.97969 .01137 L
.97986 .09608 L
.98002 .05884 L
.98019 .17621 L
Mfstroke
.98019 .17621 m
.98033 0 L
s
.98038 0 m
.98052 .16223 L
s
.98052 .16223 m
.98063 0 L
s
.98075 0 m
.98086 .16624 L
.98102 .18695 L
.98119 .07381 L
.98135 .0879 L
.98152 .15889 L
s
.98152 .15889 m
.98166 0 L
s
.98173 0 m
.98185 .07853 L
.98201 .09478 L
.98218 .12129 L
.98234 .20346 L
.98251 .1977 L
.98267 .18596 L
.98284 .11472 L
.983 .11988 L
.98316 .12091 L
.98333 .156 L
.98349 .11413 L
.98365 .09426 L
.98382 .09359 L
.98398 .06021 L
.98414 .10129 L
s
.98414 .10129 m
.98425 0 L
s
.98437 0 m
.98447 .0727 L
.98463 .01457 L
.98479 .08587 L
.98495 .12378 L
.98511 .04687 L
.98527 .06996 L
.98543 .09221 L
.9856 .08224 L
.98576 .08171 L
.98592 .09515 L
.98608 .11628 L
.98624 .09893 L
.9864 .0663 L
.98656 .10362 L
.98672 .12185 L
.98688 .06098 L
.98704 .08022 L
.9872 .03898 L
.98735 .10246 L
s
.98735 .10246 m
.9875 0 L
s
.98771 0 m
.98783 .13633 L
.98799 .20802 L
.98815 .02775 L
.98831 .14421 L
.98846 .13294 L
.98862 .01429 L
.98878 .07401 L
s
.98878 .07401 m
.98882 0 L
s
.98906 0 m
.98909 .07069 L
.98925 .08368 L
.98941 .05173 L
.98956 .05157 L
.98972 .10797 L
.98988 .14199 L
.99003 .09277 L
.99019 .05054 L
.99035 .10973 L
s
.99035 .10973 m
.99046 0 L
s
.99053 0 m
.99066 .19045 L
.99081 .09745 L
.99097 .19187 L
.99112 .07462 L
.99128 .09842 L
.99143 .06115 L
.99159 .14333 L
.99174 .04394 L
.9919 .14486 L
.99205 .1405 L
.9922 .09029 L
.99236 .06651 L
.99251 .16038 L
s
.99251 .16038 m
.99264 0 L
s
.99269 0 m
.99282 .12136 L
.99297 .07718 L
.99312 .04991 L
.99328 .15765 L
.99343 .13127 L
.99358 .04876 L
.99373 .19067 L
.99389 .13479 L
.99404 .14781 L
.99419 .0942 L
.99434 .07678 L
.99449 .11612 L
.99465 .14346 L
.9948 .12208 L
.99495 .04862 L
.9951 .04842 L
.99525 .10932 L
.9954 .05352 L
.99555 .07514 L
.9957 .11882 L
.99585 .13938 L
s
.99585 .13938 m
.99595 0 L
s
.99617 0 m
.9963 .13436 L
s
.9963 .13436 m
.99641 0 L
s
.99652 0 m
.9966 .07261 L
.99675 .01694 L
.9969 .12694 L
.99705 .04104 L
.9972 .13766 L
.99735 .13943 L
.9975 .03331 L
s
.9975 .03331 m
.9976 0 L
s
.99766 0 m
.99779 .11255 L
.99794 .11574 L
s
.99794 .11574 m
.99799 0 L
s
.9982 0 m
.99824 .07093 L
.99838 .04642 L
.99853 .13894 L
.99868 .08887 L
.99883 .15774 L
.99897 .09296 L
.99912 .01564 L
.99927 .01393 L
.99941 .12368 L
.99956 .0015 L
.99971 .09237 L
.99985 .09106 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{349.313, 215.813},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40008;0001@a000`40O003h00Oogooogoo3Goo003o
OoooOol=Ool00?mooomoo`eoo`00ogooogoo3Goo003oOoooOol=Ool00?mooomoo`eoo`00ogooogoo
3Goo003oOoooOol=Ool00?mooomoo`eoo`00ogooogoo3Goo003oOoooOol=Ool00?mooomoo`eoo`00
ogooogoo3Goo003oOoooOol=Ool00?mooomoo`eoo`00ogooogoo3Goo003oOoooOol=Ool00?mooomo
o`eoo`00ogooogoo3Goo003oOoooOol=Ool00?mooomoo`eoo`00ogooogoo3Goo002KOol50005Ool2
0005Ool50006Ool4000=Ool50005Ool20007Ool20016Ool50005Ool20005Ool4000LOol2001GOol4
001JOol009Yoo`L000Aoo`8000Aoo`L000Aoo`L000Yoo`L000Aoo`8000Moo`8004Eoo`L000Aoo`80
00Aoo`L001Yoo`8005Ioo`L005Qoo`00VWoo0P000goo0P002Woo0P000goo0P000goo0`000goo0P00
2Woo0P000goo0P003Goo0P00AGoo0P000goo0P002Goo0`000goo0P006Woo0P00EGoo0`000goo0P00
F7oo002IOol20005Ool20008Ool20005Ool20002Ool20005Ool20008Ool20005Ool2000<Ool20014
Ool20005Ool20008Ool20005Ool2000IOol2001EOol20005Ool2001GOol009Uoo`8000Eoo`8000Qo
o`8000Eoo`8000Uoo`8000Qoo`8000Eoo`8000aoo`8004Aoo`8000Eoo`8000moo`8001Uoo`8005ao
o`8005Moo`00VGoo0P001Goo0P0027oo0P001Goo0P002Goo0P0027oo0P001Goo0P0037oo0P00A7oo
0P001Goo0P003goo0P006Goo0P00G7oo0P00Egoo002IOol20005Ool20008Ool20005Ool20009Ool2
0008Ool20005Ool2000<Ool20014Ool20005Ool2000?Ool2000IOol2001LOol2001GOol009Uoo`80
00Eoo`8000Qoo`8000Eoo`80009oo`8000Aoo`<000Qoo`8000Eoo`8000aoo`8004Aoo`8000Eoo`80
00Qoo`8000Aoo`<001Uoo`8005Eoo`8000Aoo`<005Moo`00VGoo0P001Goo0P0027oo0P001Goo0P00
0Woo20002Goo0P001Goo0P0037oo0P00A7oo0P001Goo0P0027oo20006Woo0P00EGoo2000F7oo002I
Ool20005Ool20008Ool20005Ool20002Ool200000goo00000002000:Ool20005Ool2000<Ool20014
Ool20005Ool20008Ool200000goo00000002000KOol2001EOol200000goo00000002001IOol009Uo
o`8000Eoo`8000Qoo`8000Eoo`80009oo`8000moo`8000Eoo`8000Uoo`800003Ool0000004Aoo`80
00Eoo`8000Qoo`8001eoo`800003Ool0000005Eoo`8005ioo`00VGoo0P001Goo0P0027oo0P001Goo
0P000goo0P003Woo0P001Goo0P002Goo1@00A7oo0P001Goo0P002Goo0P0077oo1@00EWoo0P00GGoo
002JOol20003Ool2000:Ool20003Ool20004Ool2000?Ool20003Ool2000<Ool30015Ool20003Ool2
000:Ool2000NOol3001FOol2001MOol009Yoo`L000Yoo`L000Aoo`L000Yoo`L000eoo`8004Eoo`L0
00Yoo`L001Yoo`8005Ioo`L005Qoo`00Vgoo1@0037oo1@001Goo1`002goo1@003Woo0P00AWoo1@00
2goo1`006Woo0P00EWoo1`00F7oo003oOoooOol=Ool00?mooomoo`eoo`00ogooogoo3Goo003oOooo
Ool=Ool00?mooomoo`eoo`00ogooogoo3Goo003oOoooOol=Ool0051oool007`0041oo`00D7oo00<0
07ooOol097oo00<007ooOol057oo00<007ooOol03Woo00<007ooOol02Goo00<007ooOol01goo00<0
07ooOol01Woo00<007ooOol01Goo00<007ooOol00goo00<007ooOol00goo00<007ooOol097oo00<0
07ooOol057oo00<007ooOol03Woo00<007ooOol02Goo00<007ooOol027oo00@007oo000000Aoo`03
001oogoo00Eoo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo009oo`8000Ioo`8000Uoo`03
001oogoo00=oo`03001oo`00009oo`@00004Ool00000Ool400000goo0000000O00000goo00000006
00000goo000000070002Ool20002Ool01000Oomoogoo2@00@7oo001@Ool00`00Oomoo`0TOol00`00
Oomoo`0DOol00`00Oomoo`0>Ool00`00Oomoo`09Ool00`00Oomoo`07Ool00`00Oomoo`06Ool00`00
Oomoo`05Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`0TOol00`00Oomoo`0DOol00`00
Oomoo`0>Ool00`00Oomoo`09Ool00`00Oomoo`08Ool01000Ool0000017oo00<007ooOol01Goo00<0
07ooOol00goo00<007ooOol00goo00<007ooOol00Woo0P001Woo0P002Goo00<007ooOol00goo00<0
07oo00000Woo100000Aoo`00001oo`@00003Ool0000001l00003Ool0000000H00003Ool0000000L0
009oo`80009oo`04001oogooOol90010Ool0051oo`03001oogoo02Aoo`0300moogoo01Aoo`0300mo
ogoo00ioo`0300moogoo00Uoo`03001oogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0300mo
ogoo00=oo`0300moogoo00=oo`03001oogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0300mo
ogoo00Uoo`03001oogoo00Qoo`0400moo`000004Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?
Oomoo`03Ool00`00Oomoo`02Ool30005Ool20009Ool00`00Oomoo`03Ool00`00Ool00002Ool40000
17oo000007oo100000=oo`0000007`0000=oo`0000001P0000=oo`0000001`000Woo00D03`00Oomo
o`0000=oo`T0041oo`00D7oo00<007ooOol097oo00<03gooOol057oo00<03gooOol03Woo00<03goo
Ool02Goo00<03gooOol01goo00<03gooOol01Woo00<03gooOol01Goo00<03gooOol00goo00<03goo
Ool00goo00<03gooOol097oo00<03gooOol057oo00<03gooOol03Woo00<03gooOol02Goo00<03goo
Ool027oo00@03goo000000Aoo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300mo
ogoo009oo`<000Eoo`8000Uoo`03001oogoo00=oo`03001oo`00009oobh00003Ool0000000H00003
Ool0000000L0009oo`0500l007ooOol00003Ool90010Ool0051oo`03001oogoo02Aoo`0300moogoo
01Aoo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo
00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Aoo`0300moogoo01Aoo`0300moogoo
00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0400moo`000004Ool00`0?Oomoo`05Ool00`0?Oomo
o`03Ool00`0?Oomoo`03Ool00`0?Oomoo`02Ool30005Ool20009Ool00`00Oomoo`03Ool00`00Ool0
0002Ool^00000goo0000000600000goo000000070002Ool01@0?001oogoo00000goo2@00@7oo001@
Ool3003600l00`0000l03`0N00l2000M00l3000500l2000900l00`0000l03`0300l00`0000l00002
00l^00000`0?0000000600000`0?00000007000300l0100000l03`000`0?2@00@7oo001@Ool00`00
Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`07Ool00`0?
Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0TOol00`0?
Oomoo`0DOol00`0?Oomoo`06Ool00`00Oomoo`05Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool0100?
Ool0000017oo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol00Woo0`00
1Goo0P002Goo00<007ooOol00goo00<007oo00000Woo;P0000=oo`0000001P0000=oo`0000001`00
0Woo00D03`00Oomoo`00009oo`X0041oo`00D7oo00<007ooOol097oo00<03gooOol057oo00<03goo
Ool03Woo00<03gooOol02Goo00<03gooOol01goo00<03gooOol01Woo00<03gooOol01Goo00<03goo
Ool00goo00<03gooOol00goo00<03gooOol097oo00<03gooOol057oo00<03gooOol01Woo00<007oo
Ool01Goo00<03gooOol02Goo00<03gooOol027oo00@03goo000000Aoo`0300moogoo00Eoo`0300mo
ogoo00=oo`0300moogoo00=oo`0300moogoo009oo`<000Eoo`8000Uoo`03001oogoo009oo`800003
Ool007oo01/00003Ool0000001400003Ool0000000H00003Ool0000000L0009oo`0500l007ooOol0
0002Ool:0010Ool0051oo`03001oogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0300moogoo
00Uoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo
00=oo`0300moogoo02Aoo`0300moogoo01Aoo`0300moogoo00Ioo`03001oogoo00Eoo`0300moogoo
00Uoo`0300moogoo00Qoo`0400moo`000004Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomo
o`03Ool00`0?Oomoo`02Ool30005Ool20009Ool00`00Oomoo`0400000goo001oo`0K00000goo0000
000A00000goo0000000600000goo00000007000017oo00003`000Woo00<007ooOol02P00@7oo001@
Ool3003600l00`0000l03`0N00l2000M00l3000500l300000`0?00003`0500l00`0000l03`060000
0`0?0000000I00000`0?0000000J00000`0?000000070000100?000000000P0?00<0000?00l02P00
@7oo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?
Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?
Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`06Ool00`00Oomoo`05Ool00`0?Oomoo`09Ool00`0?
Oomoo`08Ool0100?Ool0000017oo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<0
3gooOol00Woo0`001Goo0`0000=oo`00Ool01Goo00<007ooOol01P0000=oo`0000006@0000=oo`00
00006P0000=oo`0000001`0000Aoo`000000009oo`03001oogoo00X0041oo`00D7oo00<007ooOol0
97oo00<03gooOol057oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol01goo00<03gooOol0
1Woo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol097oo00<03gooOol0
57oo00<03gooOol01Woo00<007ooOol01Goo00<03gooOol02Goo00<03gooOol027oo00@03goo0000
00Aoo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo009oo`<000Eoo`<0
0004Ool007oo0003Ool200000goo0000000500000goo0000000I00000goo0000000200000goo0000
000E00000goo0000000<00000goo001oo`0;0010Ool0051oo`<00<H03`0300003`0?01h03`8001d0
3`<000D03`<0000400l0000?000300l200000`0?0000000500000`0?0000000I00000`0?00000002
00000`0?0000000T00000`0?00003`0;0010Ool0051oo`03001oogoo02Aoo`0300moogoo01Aoo`03
00moogoo00ioo`0300moogoo00Uoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`03
00moogoo00=oo`0300moogoo00=oo`0300moogoo02Aoo`0300moogoo01Aoo`0300moogoo00Ioo`03
001oogoo00Eoo`0300moogoo00Uoo`0300moogoo00Eoo`05001oogoo00moo`020004Ool00`0?Oomo
o`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`02Ool30005Ool3000017oo001oo`00
0goo0P0000=oo`0000001@0000=oo`0000006@0000=oo`0000000P0000=oo`000000900000=oo`00
Ool02`00@7oo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`09
Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03
Ool00`0?Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`06Ool00`00Oomoo`05Ool00`0?Oomoo`09
Ool00`0?Oomoo`05Ool01@00Oomoo`0?Ool00P0017oo00<03gooOol01Goo00<03gooOol00goo00<0
3gooOol00goo00<03gooOol00Woo0`001Goo0`0000Aoo`00Ool000=oo`X00003Ool0000001T00006
Ool00000Ool007oo9P0000=oo`00Ool02`00@7oo001@Ool3003600l00`0000l03`0I00l00`0000l0
3`0200l2000M00l3000400l40000100?000000000`0?2P0000<03`0000006@0000H03`0000003`00
00lV00000`0?00003`0;0010Ool004=oo`8000]oo`03001oogoo02Aoo`0300moogoo01Aoo`0300mo
ogoo00ioo`0300moogoo00Uoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0300mo
ogoo00=oo`0300moogoo00=oo`0300moogoo02Aoo`0300moogoo01Aoo`0300moogoo00Ioo`03001o
ogoo00Eoo`0300moogoo00Uoo`0300moogoo00Eoo`05001oogoo00moo`020004Ool00`0?Oomoo`05
Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`02Ool30004Ool4000017oo000000000goo
2P0000=oo`0000006@0000Ioo`00001oo`00OolV00000goo001oo`0;000<Ool2000bOol004=oo`80
00]oo`<00<H03`0300003`0?01T03`0300003`0?00803`8001d03`<000@03`@0000400l000000003
00l:00000`0?0000000G000300l200000`0?00003`0V00000`0?00003`0;000<Ool2000bOol004=o
o`8000]oo`03001oogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0300moogoo00Uoo`0300mo
ogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300mo
ogoo02Aoo`0300moogoo01Aoo`0300moogoo00Ioo`03001oogoo00Eoo`0300moo`0000Uoo`0300mo
ogoo00Eoo`05001oogoo00moo`020004Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03
Ool00`0?Oomoo`02Ool30004Ool80003Ool:00000goo0000000G0003Ool200000goo001oo`0V0000
0goo0000000;000<Ool2000bOol004=oo`8000]oo`03001oogoo02Aoo`0300moogoo01Aoo`0300mo
ogoo00ioo`0300moogoo00Uoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0300mo
ogoo00=oo`0300moogoo00=oo`0300moogoo02Aoo`0300moogoo01Aoo`0300moogoo00Ioo`03001o
ogoo00Eoo`0300moo`0000Uoo`0300moogoo00Eoo`05001oogoo00moo`020004Ool00`0?Oomoo`05
Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`02Ool3000017oo001oogoo2P0000=oo`00
00005@0000=oo`0000002P000goo00D03gooOol007oo02H00003Ool0000000/000aoo`80039oo`00
@goo0P002goo0`00aP0?00<0000?00l01`0?00<0000?00l03`0?00<0000?00l00P0?0P002@0?00<0
000?00l04@0?0`0000@03`0000l03`X0000300l0000001D0000500l0000000003`08000600l01@00
00l0000000l08`0000<03`0000002`0037oo0P00<Woo0013Ool2000;Ool00`00Oomoo`0TOol00`0?
Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?
Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0TOol00`0?Oomoo`0DOol00`0?
Oomoo`06Ool00`00Oomoo`05Ool00`0?Ool00009Ool00`0?Oomoo`05Ool01@00Oomoo`0?Ool00P00
17oo00<03gooOol00Woo00@007ooOol03`Eoo`0300moogoo00=oo`0500moo`00001oo`03000017oo
001oogoo2P0000=oo`0000005@0000Eoo`00000007oo00@00004Ool000000003Ool01@0?Oomoo`00
Ool00P0000=oo`0000008@0000=oo`0000002`0037oo0P00<Woo0013Ool2000;Ool4003500l00`00
00l03`0700l00`0000l03`0?00l00`0000l03`0200l2000900l00`0000l03`0>00l200001P0?0000
000000l000803`X0000300l0000?01D0000500l0000000003`040000100?000000001P0?00D0000?
0000000?02<0000300l0000000/000aoo`80039oo`00@goo0P002goo00<007ooOol097oo00<03goo
Ool057oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol01goo00<03gooOol01Woo00<03goo
Ool01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol097oo00<03gooOol057oo00<03goo
Ool01Woo00<007ooOol01Goo00<03goo00002Goo00<03gooOol01Goo00D007ooOol03goo008000Ao
o`0300moogoo009oo`04001oogoo00l4Ool00`0000moo`04Ool01@0?Ool00000Ool00`0000Aoo`00
Oomoo``00005Ool00000001oo`0A00001Goo00000000Ool0100000Aoo`00000000=oo`0500moogoo
001oo`0200000goo0000000Q00000goo0000000;000<Ool2000bOol004=oo`8000]oo`03001oogoo
02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Moo`0300moogoo
00Ioo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Aoo`0300moogoo
01Aoo`0300moogoo00Ioo`03001oogoo00Eoo`0300moo`0000Uoo`0300moogoo00Eoo`05001oogoo
00moo`020004Ool0100?Oomoogoo0P000Woo00<03gooOol00Woo00<0000?Ool017oo00D03goo0000
07oo00D00003Ool0000000/00005Ool00000001oo`0A00001Goo00000000Ool0100000Aoo`000000
00=oo`0500moogoo001oo`0200000goo0000000Q00000goo0000000;000<Ool2000bOol004=oo`80
00]oo`03001oogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo
00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo
02Aoo`0300moogoo01Aoo`0300moogoo00Ioo`03001oogoo00Eoo`0300moo`0000Uoo`0300moogoo
00Eoo`05001oogoo00moo`020003Ool00`0000moo`02Ool20002Ool0100?Oomoogoo0P0000<03goo
Ool00goo00D03goo000007oo01<00005Ool00000001oo`0>000017oo000007oo0P000Woo100000Ao
o`00000000=oo`0500moogoo001oo`0200000goo0000000Q00000goo0000000;000<Ool2000bOol0
041oo`800003Ool0000000]oo`03001oogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0300mo
ogoo00Uoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0300moogoo00=oo`0300mo
ogoo00=oo`0300moogoo02Aoo`0300moogoo01Aoo`0300moogoo00Ioo`03001oogoo00Eoo`0300mo
o`0000Uoo`0300moogoo00Eoo`05001oogoo00moo`020003Ool00`0000moo`02Ool20002Ool0100?
Ool007oo0P0000<03gooOol00goo00D03goo000007oo00800003Ool0000000d0009oo`<00003Ool0
000000H00003Ool0000000<00004Ool00000Ool20002Ool4000017oo000000000goo00D03gooOol0
07oo00800003Ool0000002400003Ool0000000/000Uoo`800003Ool00000039oo`00@7oo1@002goo
00<007ooOol097oo00<03gooOol057oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol01goo
00<03gooOol01Woo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol097oo
00<03gooOol057oo00<03gooOol01Woo00<007ooOol01Goo00<03goo00002Goo00<03gooOol017oo
0P000Woo00@03goo000000=oo`0300003goo009oo`80009oo`0400moo`00Ool200000`0?Oomoo`02
Ool00`0000moo`02000017oo000007oo3`000Woo0`0000=oo`0000001P0000=oo`0000000`0000Ao
o`00001oo`80009oo`@00004Ool000000003Ool01@0?Oomoo`00Ool00P0000=oo`0000008@0000=o
o`0000002`002Goo1@00<Woo0012Ool3000;Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomo
o`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomo
o`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`06Ool00`00Oomo
o`05Ool00`0?Ool00009Ool00`0?Oomoo`04Ool20002Ool0100?Ool000000goo00<0000?Ool00Woo
0P000Woo00@03goo001oo`80000400moogooOol200001@0?Ool00000Ool00P0000=oo`0000001000
00=oo`0000001P000Woo0P000Woo200000=oo`0000000`0000Aoo`00001oo`80009oo`@00004Ool0
07oo0003Ool01@0?Oomoo`00Ool00P0000=oo`0000008@0000=oo`0000002`002goo0`00<Woo0013
Ool2000;Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?
Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?
Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`06Ool20006Ool00`0?Ool00009Ool00`0?Oomoo`04
Ool20002Ool02@0?Ool00000Ool007oo00003`03Ool200001Woo00003goo001oo`80000400moo`00
Ool200001@0?00000000Ool00P0000=oo`0000000`000Woo100000Aoo`000000009oo`80009oo`P0
0003Ool0000000<00004Ool00000Ool20002Ool4000017oo001oo`000goo00H03gooOol007oo0002
Ool400000goo0000000L00000goo0000000;000<Ool2000bOol004=oo`8000]oo`03001oogoo02Ao
o`0300moogoo01Aoo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Moo`0300moogoo00Io
o`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Aoo`0300moogoo01Ao
o`0300moogoo00Ioo`8000Ioo`0300moo`0000Uoo`0300moogoo00Aoo`80009oo`0900moo`00001o
o`00Ool0000?00=oo`800006Ool0000?Ool007oo0P0000<03goo00002P0000=oo`0000000`000Woo
10000Woo0P000Woo0P000Woo200000=oo`0000000`0000Aoo`00001oo`80009oo`<0009oo`03001o
o`0000=oo`0600moogoo001oo`000Woo100000=oo`000000700000=oo`0000002`0037oo0P00<Woo
001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomo
o`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomo
o`0TOol00`0?Oomoo`0DOol00`0?Oomoo`06Ool20006Ool00`0?Ool00009Ool00`0?Oomoo`04Ool2
0002Ool02@0?Ool00000Ool007oo00003`03Ool200001Woo00003goo001oo`80000300l0000000T0
009oo`03001oo`000080009oo`@0009oo`80009oo`80009oo`P00003Ool0000000<00004Ool00000
Ool20002Ool30002Ool00`00Ool00003Ool01P0?Oomoo`00Ool0009oo`05001oo`00001oo`0N0000
0goo0000000;0010Ool0051oo`03001oogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0300mo
ogoo00Uoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0300moogoo00=oo`0300mo
ogoo00=oo`0300moogoo02Aoo`0300moogoo01Aoo`0300moogoo00Ioo`8000Ioo`0300moo`0000Uo
o`0300moogoo00Aoo`80009oo`0900moo`00001oo`00Ool0000?00=oo`800006Ool0000?Ool007oo
0P0000D03`00000007oo00L000Aoo`8000=oo`@0009oo`80009oo`80009oo`<00003Ool000000080
0003Ool00000008000=oo`03001oo`0000=oo`<000Aoo`05001oogooOol03`04Ool01@00Oomoo`00
Ool00P0000=oo`000000700000=oo`0000002`00@7oo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0D
Ool00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05
Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`06
Ool20006Ool00`0?Ool00009Ool00`0?Oomoo`04Ool20002Ool02@0?Ool00000Ool007oo00003`03
Ool200000goo00000002000017oo0000000?0`0000=oo`0000000P000Woo00<007ooOol00Woo0P00
0goo10000Woo0P000Woo0P000Woo0`0000=oo`0000000P0000=oo`00Ool00P000goo00<007oo0000
0goo00<007oo000017oo00D007ooOomoo`0?00Aoo`05001oogoo001oo`0200000goo0000000L0000
0goo0000000;0010Ool0051oo`<00<H03`8000P03`0300003`0?00d03`<000@03`80000400l0000?
000400l200000`0?0000000500001`0?0000000000l0000?008000803`0300003`0?00803`8000D0
3`8000803`8000<03`0300003`0?00<0000300l000000080000400l0000?000400l00`0000l00003
00l00`0000l03`0=00l01@0000l03`0000l00P0000<03`000000700000<03`0000002`00@7oo001@
Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`07
Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0T
Ool00`0?Oomoo`0DOol00`0?Oomoo`06Ool20006Ool00`0?Ool00009Ool00`0?Oomoo`03Ool30002
Ool02@0?Ool00000Ool007oo00003`03Ool200000goo0000000300000goo00003`0300000goo0000
3`020007Ool20005Ool01000Oomoogoo0P000goo00<007ooOol00`0000Aoo`000000009oo`03001o
o`0000Ioo`05001oogooOol0000:Ool00`0?Oomoo`02Ool01000Oomoo`000Woo00<007oo00007@00
00=oo`0000002`00@7oo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?
Oomoo`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?
Oomoo`03Ool00`0?Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`06Ool20006Ool00`0?00000009
Ool00`0?Oomoo`03Ool30002Ool0300?Ool00000Ool007oo00003goo001oo`P00003Ool0000?00<0
009oo`0300l0000000Moo`8000Eoo`04001oogooOol20003Ool01000Oomoogoo0P0000Aoo`000000
009oo`03001oo`0000Ioo`05001oogooOol0000:Ool00`0?Oomoo`02Ool01000Oomoo`000Woo00<0
07oo00007@0000=oo`0000002`00@7oo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomo
o`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomo
o`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`06Ool20006Ool0
0`0?00000009Ool00`0?Oomoo`03Ool30002Ool0300?Ool00000Ool007oo00003goo001oo`P00003
Ool0000?00<0009oo`0300l007oo00Moo`8000Eoo`03001oogoo009oo`05001oogooOol00003Ool2
000017oo000000000Woo00<007oo00001Woo00D007ooOomoo`0000Yoo`0300moogoo009oo`04001o
ogoo0002Ool00`00Ool0000M00000goo0000000;0010Ool0051oo`03001oogoo02Aoo`0300moogoo
01Aoo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo
00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Aoo`0300moogoo01Aoo`0300moogoo
00Ioo`8000Ioo`0300l0000000Uoo`0300moo`0000=oo`<0009oo`0700moo`00001oo`00Ool00P00
00=oo`00Ool0200000=oo`0000l00`000Woo00<03`00Ool027oo00<007ooOol00goo00<007ooOol0
1Woo00@007ooOomoo`800004Ool000000002Ool00`00Ool00006Ool01@00Oomoogoo00002Woo00<0
3gooOol00Woo00@007ooOol0009oo`03001oo`0001d00003Ool0000000/0041oo`00D7oo00<007oo
Ool097oo00<03gooOol057oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol01goo00<03goo
Ool01Woo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol097oo00<03goo
Ool057oo00<03gooOol01Woo0P001Woo00@03`00Ool000Qoo`0300moo`0000=oo`<0009oo`0700mo
o`00001oo`00Ool00P0000=oo`00Ool0200000=oo`0000l00`000Woo00<03`00Ool027oo00<007oo
Ool00goo00<007ooOol02Woo0P0000Aoo`00000000Aoo`03001oogoo00Aoo`03001oogoo00aoo`03
00moogoo009oo`04001oogoo0002Ool00`00Ool0000M00000goo0000000;0010Ool0051oo`03001o
ogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Moo`0300mo
ogoo00Ioo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Aoo`0300mo
ogoo01Aoo`0300moogoo00Ioo`8000Ioo`0400l007oo0008Ool00`0?Ool00003Ool30002Ool01`0?
Ool00000Ool007oo00800003Ool007oo00P00003Ool0000?00<0009oo`0300l007oo00ioo`03001o
ogoo00Yoo`800004Ool007oo0004Ool00`00Oomoo`04Ool00`00Oomoo`0<Ool00`0?Oomoo`02Ool0
1000Oomoo`001Goo7@0000=oo`0000002`00@7oo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol0
0`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool0
0`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`06Ool2
0006Ool0100?001oo`0027oo00L03goo000007oo001oo`0300000goo00moo`0200000goo00000002
00000goo001oo`0300000goo0000000200000goo000000030002Ool00`0?Oomoo`0>Ool00`00Oomo
o`0;Ool01@00Ool007oo000017oo00<007ooOol017oo00<007ooOol037oo00<03gooOol00Woo00@0
07ooOol000Eooad00003Ool0000000/0041oo`00D7oo0`00aP0?0P001`0?00<0000?00002P0?0P00
00@03`0000l03`8000<03`L0000300l0000?00<0000600l0000000l0000?0P0000<03`0000008@0?
00<0000?00001P0?00<0000?00l0100?00<0000?00l0500?00<0000?00l00`0?:`00@7oo001@Ool0
0`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`07Ool0
0`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0TOol0
0`0?Oomoo`0DOol00`0?Oomoo`06Ool20006Ool0100?001oo`0027oo00H03goo000007oo0002Ool2
00000goo001oo`0700000goo001oo`0300000goo00000003Ool20002Ool01000Oomoo`0?7Woo00<0
07oo00001Woo00<03gooOol017oo00<007ooOol037oo00<03gooOol01Goo00<007ooOol00goo:`00
@7oo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?
Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?
Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`05Ool00`00Ool00006Ool0100?001oo`0027oo00H0
07oo000007oo0002Ool200000goo001oo`0700000goo0000000300000goo00000003Ool20005Ool0
0`0?Oomoo`0LOol00`00Ool00006Ool00`0?Oomoo`04Ool00`00Oomoo`0<Ool00`0?Oomoo`05Ool0
0`00Oomoo`03OolC00000goo0000000E0010Ool0051oo`03001oogoo02Aoo`0300moogoo01Aoo`03
00moogoo00ioo`0300moogoo00Uoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`03
00moogoo00=oo`0300moogoo00=oo`0300moogoo02Aoo`0300moogoo01Aoo`0300moogoo00Eoo`03
001oo`0000Ioo`0400l007oo0008Ool01P00Ool00000Ool0009oo`800003Ool007oo00D00003Ool0
07oo00D00003Ool007oo00=oo`8000Eoo`0300moogoo01ioo`03001oogoo00Aoo`0300moogoo00Ao
o`03001oogoo00aoo`0300moogoo00]oo``00003Ool0000000@00003Ool0000001D0041oo`00D7oo
00<007ooOol097oo00<03gooOol057oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol01goo
00<03gooOol01Woo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol097oo
00<03gooOol057oo00<03gooOol01Goo00<007oo00001Woo00@03`00Ool000Qoo`06001oo`00001o
o`000Woo0P0000=oo`00Ool01@0000=oo`00Ool00`0000Aoo`00Ool000Aoo`8000Eoo`0300moogoo
01ioo`03001oogoo00Aoo`0300moogoo00Aoo`03001oogoo00aoo`0300moogoo00]oo`D00003Ool0
000000@00003Ool0000000@00003Ool0000001D0041oo`00D7oo0`00a@0?00<0000?00001`0?00<0
000?0000200?00H0000?0000000?000200l400000`0?0000000300000`0?00003`030000100?0000
3`00100?0P009P0?00<0000?00l02`0?00<0000?00l06P0?1@0000<03`000000100000<03`000000
100000<03`0000005@00@7oo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool0
0`0?Oomoo`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool0
0`0?Oomoo`03Ool00`0?Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`05Ool00`00Ool00006Ool0
100?001oo`0027oo00H007oo000007oo0002Ool01@00Ool00000Ool01@0000=oo`00Ool00`0000Ao
o`00Ool000Aoo`0300003goo00Aoo`0300moogoo01ioo`03001oogoo00Aoo`0300moogoo00Aoo`03
001oogoo00aoo`0300moogoo00]oo`D00003Ool007oo00@00003Ool0000000@00003Ool0000001D0
041oo`00D7oo00<007ooOol097oo00<03gooOol057oo00<03gooOol03Woo00<03gooOol02Goo00<0
3gooOol01goo00<03gooOol01Woo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<0
3gooOol097oo00<03gooOol057oo00<03gooOol01Goo00<007oo00001Woo00@03`00Ool000Qoo`06
001oo`00001oo`000Woo00<007oo00001`0000Eoo`00001oo`00009oo`03001oo`0?00Aoo`030000
3goo00Aoo`0300moogoo01ioo`03001oogoo00Aoo`0300moogoo00Aoo`03001oogoo00aoo`0300mo
ogoo00]oo`D00005Ool007oo001oo`0200000goo0000000400000goo0000000E0010Ool0051oo`03
001oogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Moo`03
00moogoo00Ioo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Aoo`03
00moogoo01Aoo`0300moogoo00Eoo`03001oo`0000Ioo`0300l007oo008000Moo`06001oo`00001o
o`000Woo00<007oo00001`0000Eoo`00001oo`00009oo`03001oo`0?00Eoo`0300moogoo00=oo`03
00moogoo01ioo`03001oogoo00Aoo`0300moogoo01=oo`0300moogoo00]oo`D00005Ool007oo001o
o`0200000goo0000000400000goo0000000E0010Ool0051oo`<00<D03`0300003`0000L03`040000
3`00000700l01P0000l0000000l000803`0300003`0000L0000500l0000000l0000200l00`0000l0
3`0/00l00`0000l03`0X00l500001P0?00003`0000l000803`H0000300l0000001D0041oo`00D7oo
00<007ooOol097oo00<03gooOol057oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol01goo
00<03gooOol01Woo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol097oo
00<03gooOol057oo00<03gooOol01Goo00<007oo00001Woo00<03`00Ool00P001goo00H007oo0000
07oo0002Ool01000Ool0000?1@000Woo0P0000Moo`00Oomoo`00Ool03`05Ool00`0?Oomoo`03Ool0
0`0?Oomoo`0NOol00`00Oomoo`04Ool00`0?Oomoo`0COol00`0?Oomoo`0;Ool01000Ool000000Woo
00D007oo001oo`00009oo`H00003Ool0000001D0041oo`00D7oo00<007ooOol097oo00<03gooOol0
57oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol01goo00<03gooOol01Woo00<03gooOol0
1Goo00L03gooOomoo`00Ool03`05Ool00`0?Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`05Ool0
0`00Ool00006Ool00`0?001oo`020007Ool01P00Ool00000Ool0009oo`04001oogoo00l50002Ool2
0004Ool00`00Ool03`05Ool00`0?Oomoo`03Ool00`0?Oomoo`0NOol00`00Oomoo`04Ool00`0?Oomo
o`0COol00`0?Oomoo`0=Ool20002Ool01@00Ool007oo00000Woo1P0000=oo`0000005@00@7oo001@
Ool3001g00l00`0000l03`1;00l00`0000l0000700l0100000l000001P0?0P0000D03`0000003`00
00803`0400003`0?00l5000200l2000c00l00`0000l03`0Z00l2000200l00`0000l0000400l60000
0`0?0000000E0010Ool0051oo`03001oogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0300mo
ogoo00Uoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0700moogooOol007oo00l0
1Goo00<03gooOol097oo00<03gooOol057oo00<03gooOol01Goo00<007oo00001Woo00<03`00Ool0
0P001Woo0`0000=oo`0000001Woo00<03`0000000`000Woo0P001Woo00<03gooOol00goo00<03goo
Ool00goo00<03gooOol07Woo00<007ooOol017oo00<03gooOol04goo00<03gooOol03Goo0P000Woo
00<007oo000017oo1P0000=oo`0000005@00@7oo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol0
0`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool0
1`0?Oomoogoo001oo`0?00Eoo`0300moogoo02Aoo`0300moogoo01Aoo`0300moogoo00Eoo`03001o
o`0000Ioo`0300l007oo008000Ioo`<00003Ool0000000Ioo`0300l0000000<0009oo`8000Ioo`03
00moogoo00=oo`0300moogoo00=oo`0300moogoo01ioo`03001oogoo00Aoo`0300moogoo01=oo`03
00moogoo00eoo`80009oo`03001oo`0000Aoo`H00003Ool0000001D0041oo`00D7oo0`00M`0?00<0
000?00l0B`0?00<0000?00001`0?00@0000?000000H03`<000803`0300003`0?00D03`<0000400l0
000?00l2000c00l00`0000l03`0Z00l2000200l00`0000l0000400l5000200lG0010Ool003Qoo`80
00Moo`D000Yoo`03001oogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0300moogoo00Uoo`03
00moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0700moogooOol007oo00l01Goo00<03goo
Ool097oo00<03gooOol057oo00<03gooOol01Goo00<007oo00001Goo00@0000?001oo`8000Ioo`03
00003`00009oo`03001oogoo00Aoo`0600l00000001oo`000Woo00<03`00Ool01Goo00<03gooOol0
0goo00<03gooOol00goo00<03gooOol07Woo00<007ooOol017oo00<03gooOol04goo00<03gooOol0
3Goo00L0000?Oomoo`00Ool00004Ool50002OolG000<Ool20007Ool5000VOol003Qoo`8000Ioo`L0
00Uoo`03001oogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo
00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0300moogoo00800003Ool03goo00Aoo`0300moogoo
02Aoo`0300moogoo01Aoo`0300moogoo00Eoo`03001oo`0000Eoo`0400003`00Ool20006Ool00`00
00l00002Ool00`00Oomoo`04Ool01P0?00000000Ool0009oo`0300moogoo00Eoo`0300moogoo00=o
o`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00eoo`0700003gooOol0
07oo000017oo1@000Woo100000=oo`0000002@0000=oo`000000100037oo0P001Woo1`009Goo000h
Ool20006Ool20003Ool20009Ool3001f00l2001=00l00`0000l0000500l00`0000l0000200l01000
00l03`000`0?00<0000?00000P0?00<0000?00l01@0?0`0000<03`0000l0H`0?00L0000?00l03`00
00l0000500l4000200l400000`0?0000000900000`0?00000004000<Ool20006Ool20003Ool2000U
Ool003Qoo`8000Eoo`8000Eoo`8000Qoo`03001oogoo02Aoo`0300moogoo01Aoo`0300moogoo00io
o`0300moogoo00Uoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0300moogoo0080
0003Ool03goo00Aoo`0300moogoo02Aoo`0300moogoo01Aoo`0300moogoo00Eoo`03001oo`0000Eo
o`0300003`00009oo`03001oogoo0080009oo`0300003`00009oo`03001oogoo00Aoo`0300l00000
009oo`04001oogoo00l7Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomo
o`0COol00`0?Oomoo`0=Ool01@0000moogoo00001goo10000Woo0P0000=oo`00Ool02`0000Aoo`00
0000009oo`03001oogoo00Yoo`8000Eoo`8000Eoo`8002Aoo`00>7oo0P001Goo0P001Goo0P0027oo
0`00MP0?00<0000?0000C00?00<0000?00001@0?00<0000?00000P0?00H0000?00003`0000l20000
1@0?00003`0?00001`0?0P000P0?00<0000?00l0HP0?00D0000?00l03`0000L03`@000803`800003
00l0000?00/0000300l0000000@000aoo`8000Eoo`8000Eoo`8002Aoo`00>7oo0P001Goo0P001Goo
0P0027oo00<007ooOol097oo00<03gooOol057oo00<03gooOol03Woo00<03gooOol02Goo00<03goo
Ool01goo00<03gooOol01Woo00<03gooOol01Goo00L03gooOol007oo00003`05Ool00`0?Oomoo`0T
Ool00`0?Oomoo`0DOol00`0?Oomoo`05Ool00`00Ool00005Ool00`0000l00002Ool01P00Ool007oo
001oo`80000500l007ooOol00006Ool00`0?00000002Ool01000Oomoo`0?1goo00<03gooOol00goo
00<03gooOol00goo00<03gooOol09Goo00<03gooOol04goo00<03gooOol03Goo00D0000?Oomoo`00
00Moo`@0009oo`800003Ool007oo00/00004Ool000000002Ool00`00Oomoo`0:Ool20005Ool20005
Ool2000TOol003Qoo`8000Eoo`8000Eoo`8000Qoo`@005@03`0300003`0?01h03`0300003`0004`0
3`0300003`0000D03`0300003`0000803`0700003`0000l0000?00000P0?00@0000?00l000L03`80
06L03`0500003`0?00l0000700l4000200l200000`0?00003`0;00000`0?00000004000<Ool20005
Ool20005Ool2000TOol003Qoo`8000Eoo`8000Eoo`8000Qoo`03001oogoo02Aoo`0300moogoo01Ao
o`0300moogoo00ioo`0300moogoo00Ioo`04001oogoo00l9Ool00`0?Oomoo`06Ool00`0?Oomoo`05
Ool01`0?Oomoo`00Ool0000?00Eoo`0300moogoo02Aoo`0300moogoo01Aoo`0300moogoo00Eoo`03
001oo`0000Eoo`0300003`0000=oo`800007Ool007oo001oo`0?00000Woo00<007ooOol017oo00<0
3`0000001Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol09Goo00<0
3gooOol04goo00<03gooOol03Goo00D0000?Oomoo`0000Moo`@0009oo`800005Ool007oo00003`09
000017oo000000000Woo00<007ooOol02Woo0P001Goo0P001Goo0P0097oo000hOol20005Ool20005
Ool20008Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`06Ool01000
Oomoo`0?2Goo00<03gooOol01Woo00<03gooOol01Goo00L03gooOol007oo00003`05Ool00`0?Oomo
o`0TOol00`0?Oomoo`0DOol00`0?Oomoo`04Ool200000goo001oo`04Ool00`0000l00003Ool20000
1goo001oo`00Ool03`00009oo`03001oogoo00Aoo`0300moo`0000Eoo`0300moogoo00Eoo`0300mo
ogoo00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00eoo`050000
3gooOol00007Ool40002Ool200001Goo001oo`0000l02@0000Aoo`000000009oo`03001oogoo00Yo
o`8000Eoo`8000Eoo`8002Aoo`00>7oo0P001Goo0P001Goo0P0027oo00<007ooOol097oo00<03goo
Ool057oo00<03gooOol03Woo00<03gooOol01Woo00@007ooOol03`Uoo`0300moogoo00Ioo`0300mo
ogoo00Eoo`0700moogoo001oo`0000l01Goo00<03gooOol097oo00<03gooOol057oo00<03gooOol0
17oo0P0000=oo`00Ool017oo00<0000?00000goo0P0000Moo`00Ool007oo00l00002Ool00`00Oomo
o`04Ool00`0?Ool00005Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomo
o`0UOol00`0?Oomoo`0COol00`0?Oomoo`0=Ool00`0000moo`0:Ool30002Ool200001Woo001oo`00
00moo`P00004Ool000000002Ool00`00Oomoo`0:Ool20005Ool20005Ool2000TOol003Eoo`800003
Ool0000000Eoo`8000Eoo`8000Qoo`03001oogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`03
00moogoo00Ioo`04001oogoo00l9Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool01`0?Oomoo`00Ool0
000?00Eoo`0300moogoo02Aoo`0300moogoo01Aoo`0300moogoo00Aoo`800003Ool007oo009oo`05
001oo`0000l00003Ool200001goo001oo`00Ool03`00009oo`03001oogoo00Aoo`0300moo`0000Eo
o`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=o
o`0300moogoo00eoo`0300003goo00Yoo`03001oo`00009oo`800006Ool007oo00003goo200000Ao
o`000000009oo`03001oogoo00Moo`800003Ool0000000Eoo`8000Eoo`8002Aoo`00=Goo1@001Goo
0P001Goo0P0027oo00<007ooOol097oo00<03gooOol057oo00<03gooOol03Woo00<03gooOol01Goo
0P000Woo00<03gooOol01goo00<03gooOol01Woo00<03gooOol01Goo00<03goo00000Woo00<0000?
Ool017oo00<03gooOol097oo00<03gooOol057oo00<03gooOol017oo0P0000=oo`00Ool00Woo00D0
07oo00003`0000=oo`80009oo`800003Ool03`00009oo`03001oogoo00Aoo`0300moo`0000Eoo`03
00moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`03
00moogoo00eoo`0300003goo00Yoo`03001oo`00009oo`800006Ool007oo00003goo200000Aoo`00
0000009oo`03001oogoo00Moo`D000Eoo`8000Eoo`8002Aoo`00=goo0`001Woo0P000goo0P002Goo
00<007ooOol097oo00<03gooOol057oo00<03gooOol03Woo00<03gooOol01Goo00D007oo001oo`0?
00Uoo`0300moogoo00Ioo`0300moogoo00Eoo`0300moo`00009oo`0300003goo00Aoo`0300moogoo
02Aoo`0300moogoo01Aoo`0300moogoo00Aoo`800003Ool007oo009oo`05001oo`0000l00003Ool2
0002Ool200000goo00l00002Ool00`00Oomoo`04Ool00`0?Ool00005Ool00`0?Oomoo`05Ool00`0?
Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0=Ool00`00
00moo`0:Ool00`00Ool00002Ool200001Woo001oo`0000moo`P00004Ool000000002Ool00`00Oomo
o`09Ool30006Ool20003Ool2000UOol003Qoo`8000Ioo`L000Uoo`03001oogoo02Aoo`0300moogoo
01Aoo`0300moogoo00ioo`0300moogoo00Eoo`05001oo`00Ool03`09Ool00`0?Oomoo`06Ool00`0?
Oomoo`05Ool00`0?Ool00002Ool00`0000moo`04Ool00`0?Oomoo`0TOol00`0?Oomoo`0DOol00`0?
Oomoo`04Ool200000goo001oo`02Ool01@00Ool0000?00000goo0P000Woo0P0000=oo`0?00002Goo
00<03goo00001Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol09Goo
00<03gooOol04goo00<03gooOol03Goo00<0000?Ool02Woo00<007oo00000Woo00P007ooOol007oo
00003goo200000Aoo`000000009oo`03001oogoo00Yoo`8000Ioo`L002Eoo`00>7oo0P001goo1@00
2Woo00<007ooOol097oo00<03gooOol057oo00<03gooOol03Woo00<03gooOol01Goo00D007oo001o
o`0?00Uoo`0300moogoo00Ioo`0300moogoo00Eoo`0300moo`00009oo`0300003goo00Aoo`0300mo
ogoo02Aoo`0300moogoo01Aoo`0300moogoo00Aoo`800003Ool007oo009oo`04001oo`000004Ool0
1000Oomoogoo0P0000=oo`0?Ool02Goo00<03goo00001Goo00<03gooOol01Goo00<03gooOol00goo
00<03gooOol00goo00<03gooOol09Goo00<03gooOol04goo00<03gooOol03Goo00<0000?Ool02Woo
00<007oo00000Woo00P007ooOol007oo00003goo100000Eoo`00000007oo008000=oo`03001oogoo
00Yoo`8000Moo`D002Ioo`00D7oo00<007ooOol097oo00<03gooOol057oo00<03gooOol03Woo00<0
3gooOol01Goo00D007oo001oo`0?00Uoo`0300moogoo00Ioo`0300moogoo00Eoo`0300moo`00009o
o`0300003goo00Aoo`0300moogoo02Aoo`0300moogoo01Aoo`0300moogoo00Aoo`800003Ool007oo
009oo`04001oo`000004Ool01000Oomoogoo0P0000=oo`0?Ool02Goo00<03goo00001Goo00<03goo
Ool01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol09Goo00<03gooOol04goo00<03goo
Ool03Goo00<0000?Ool02Woo00<007oo00000Woo00P007ooOol007oo00003goo100000Ioo`000000
07oo0004Ool00`00Oomoo`0nOol0051oo`03001oogoo02Aoo`0300moogoo01Aoo`0300moogoo00io
o`0300moogoo00Eoo`05001oo`00Ool03`09Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Ool0
0002Ool00`0000moo`04Ool00`0?Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`04Ool200000goo
001oo`02Ool01000Ool0000017oo00@007ooOomoo`800003Ool03goo00Uoo`0300moo`0000Eoo`03
00moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`03
00moogoo00eoo`0300003goo00Yoo`03001oo`00009oo`08001oogoo001oo`0000moo`@0009oo`80
0003Ool007oo00=oo`03001oogoo03ioo`00D7oo00<007ooOol097oo00<03gooOol057oo00<03goo
Ool03Woo00<03gooOol01Goo00D007oo001oo`0?00Uoo`0300moogoo00Ioo`0300moogoo00Eoo`03
00moo`00009oo`0300003goo00Aoo`0300moogoo02Aoo`0300moogoo01Aoo`0300moogoo00=oo`03
001oo`00009oo`05001oogoo001oo`020004Ool01000Oomoogoo0P0000=oo`0?Ool02Goo00<03goo
00001Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol09Goo00<03goo
Ool04goo00<03gooOol03Goo00<0000?Ool02Woo00<007oo00000Woo00P007ooOol007oo00003goo
10000Woo0P0000=oo`00Ool00goo00<007ooOol0?Woo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0D
Ool00`0?Oomoo`0>Ool00`0?Oomoo`05Ool01@00Ool007oo00l02Goo00<03gooOol01Woo00<03goo
Ool01Goo00<03goo00000Woo00<0000?Ool017oo00<03gooOol097oo00<03gooOol057oo00<03goo
Ool00goo00<007oo00000Woo00D007ooOol007oo008000Aoo`04001oogooOol200000goo00moo`09
Ool00`0?Ool00005Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0U
Ool00`0?Oomoo`0COol00`0?Oomoo`0=Ool00`0000moo`0:Ool00`00Ool00002Ool02000Oomoo`00
Ool0000?Ool40002Ool200000goo001oo`03Ool00`00Oomoo`0nOol0051oo`<005@03`0300003`00
01h03`0400003`0?000j00l00`0000l03`0=00l00`0000l0000200l200001@0?0000000?0000100?
00<0000?00l04@0?00<0000?00l0I@0?00<0000?00l02P0?00<0000?00000P0?00H0000?00l0000?
000300l3000200l0100000l03`000P0?0`00@7oo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol0
0`0?Oomoo`0>Ool00`0?Oomoo`04Ool01P00Oomoo`00Ool03`Uoo`0300moogoo00Ioo`0300moogoo
00Eoo`0300moo`0000=oo`03001oogoo00=oo`0300moogoo02Aoo`0300moogoo00Yoo`03001oogoo
00Moo`0300moogoo00=oo`03001oo`00009oo`800005Ool00000Ool00004Ool00`00Oomoo`04Ool0
0`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool0
0`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0=Ool00`0000moo`0:Ool00`00Ool00002Ool0
1`00Oomoo`00Ool0000?009oo`<0009oo`04001oogoo0004Ool00`00Oomoo`0nOol0051oo`03001o
ogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0300moogoo00Aoo`06001oogoo001oo`0?2Goo
00<03gooOol01Woo00<03gooOol01Goo00<03goo00000goo00<007ooOol00goo00<03gooOol097oo
00<03gooOol02Woo00<007ooOol01goo00<03gooOol00goo00<007oo00000Woo0P0000Eoo`00001o
o`0000Aoo`03001oogoo00Aoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`0300mo
ogoo00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00eoo`030000
3goo00Yoo`03001oo`00009oo`07001oogoo001oo`0000l00Woo0`000Woo00<007ooOol01Goo00<0
07ooOol0?Woo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`04
Ool01P00Oomoogoo00003`Uoo`0300moogoo00Ioo`0300moogoo00Eoo`0300l007oo00=oo`03001o
ogoo00=oo`0300moogoo02Aoo`0300moogoo00Yoo`03001oogoo00Moo`0300moogoo00=oo`03001o
o`00009oo`D00003Ool007oo00=oo`03001oogoo00Aoo`0300moogoo00Qoo`0300moogoo00Eoo`03
00moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`03
00moogoo00eoo`0300003goo00Yoo`03001oo`00009oo`03001oogoo009oo`0400003gooOol30002
Ool00`00Oomoo`05Ool00`00Oomoo`0nOol0051oo`03001oogoo02Aoo`0300moogoo01Aoo`0300mo
ogoo00ioo`0300moogoo00Aoo`06001oogooOol0000?2Goo00<03gooOol01Woo00<03gooOol01Goo
00<03`00Ool00goo00<007ooOol00goo00<03gooOol097oo00<03gooOol02Woo00<007ooOol01goo
00<03gooOol00goo00<007oo00000Woo1@0000=oo`00Ool02Woo00<03gooOol027oo00<03gooOol0
1Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol09Goo00<03gooOol0
4goo00<03gooOol03Goo00<0000?Ool02Woo00<007oo00000Woo00<007ooOol00Woo00@0000?Oomo
o`<0009oo`0300moogoo00Eoo`03001oogoo03ioo`00D7oo00<007ooOol097oo00<03gooOol057oo
00<03gooOol03Woo00<03gooOol017oo00H007ooOomoo`0000l9Ool00`0?Oomoo`06Ool00`0?Oomo
o`05Ool00`0?001oo`03Ool00`00Oomoo`03Ool00`0?Oomoo`0TOol00`0?Oomoo`0:Ool00`00Oomo
o`07Ool00`0?Oomoo`03Ool00`00Ool00002Ool3000017oo001oo`002goo00<03gooOol027oo00<0
3gooOol01Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol09Goo00<0
3gooOol04goo00<03gooOol03Goo00<0000?Ool02Woo00<007oo00000Woo00<007ooOol00Woo00@0
000?Oomoo`<0009oo`0300moogoo00Eoo`03001oogoo03ioo`00D7oo00<007ooOol097oo00<03goo
Ool057oo00<03gooOol03Woo00<03gooOol017oo00H007ooOomoo`0000l9Ool00`0?Oomoo`06Ool0
0`0?Oomoo`05Ool00`0?001oo`03Ool00`00Oomoo`03Ool00`0?Oomoo`0TOol00`0?Oomoo`0:Ool0
0`00Oomoo`07Ool00`0?Oomoo`03Ool00`00Ool00002Ool3000017oo001oo`002goo00<03gooOol0
27oo00<03gooOol01Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol0
9Goo00<03gooOol04goo00<03gooOol03Goo00<0000?Ool02Woo00<007oo00000Woo00<007ooOol0
0Woo00@0000?Oomoo`<0009oo`0300moogoo00Eoo`03001oogoo03ioo`00D7oo0`00D`0?00D0000?
00l03`0001`03`0300003`0?00803`0300003`0?03L03`8000h03`0300003`0?00803`<0000400l0
000?002=00l00`0000l0000200l00`0000l03`0200l0100000l03`0?0`00200?0`00@7oo001@Ool0
0`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`04Ool01P00Oomoogoo0000
3`Uoo`0300moogoo00Ioo`0300moogoo00Eoo`0300l007oo00=oo`03001oogoo00=oo`0300moogoo
02Aoo`0300moogoo00Yoo`8000Qoo`0300moogoo00=oo`03001oogoo009oo`<00004Ool007oo000;
Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03
Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`00Oomoo`02
Ool00`00Oomoo`02Ool0100000moogoo0`000Woo00<03gooOol01Goo00<007ooOol0?Woo001@Ool0
0`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`04Ool01P00Oomoogoo0000
3`Uoo`0300moogoo00Ioo`0300moogoo00Eoo`0300l007oo00=oo`03001oogoo00=oo`0300moogoo
02Aoo`0300moogoo00Yoo`8000Eoo`04001oogoo00l5Ool00`00Oomoo`02Ool3000017oo001oo`00
2goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol0
0goo00<03gooOol09Goo00<03gooOol04goo00<03gooOol03Woo00<03gooOol02Goo00<007ooOol0
0Woo00<007ooOol00Woo00@0000?Oomoo`<0009oo`0300moogoo00Eoo`03001oogoo03ioo`00D7oo
00<007ooOol097oo00<03gooOol057oo00<03gooOol03Woo00<03gooOol00goo00<007ooOol00Woo
00<0000?Ool027oo00<03gooOol01Woo00<03gooOol01Goo00<03`00Ool00goo00<007ooOol00goo
00<03gooOol097oo00<03gooOol02Woo0P001Goo00@007ooOol03`Eoo`03001oogoo009oo`<00004
Ool007oo000;Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool0
0`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool0
0`00Oomoo`02Ool00`00Oomoo`02Ool0100000moogoo0`000Woo00<03gooOol01Goo00<007ooOol0
?Woo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`03Ool00`00
Oomoo`02Ool00`0000moo`08Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?001oo`03Ool00`00
Oomoo`03Ool00`0?Oomoo`0TOol00`0?Oomoo`0:Ool20005Ool01000Oomoo`0?1Goo00<007ooOol0
0Woo00L007oo001oo`00Ool0000;Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool0
0`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool0
0`0?Oomoo`09Ool00`00Oomoo`02Ool00`00Oomoo`02Ool0100000moogoo0`000Woo00<03gooOol0
1Goo00<007ooOol0?Woo001@Ool3001B00l00`0000l03`0200l00`0000l03`0J00l00`0000l03`02
00l00`0000l03`0f00l00`0000l0000500l00`0000l03`0600l00`0000l03`0200l01`0000l0000?
00003`0008d03`0300003`0?00803`0300003`0?00803`0500003`0?00l0000:00l30010Ool0051o
o`03001oogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0300moogoo00=oo`03001oogoo009o
o`0300003goo00Qoo`0300moogoo00Ioo`0300moogoo00Eoo`0300l007oo00=oo`03001oogoo00=o
o`0300moogoo02Aoo`0300moogoo00Uoo`03001oo`0000Eoo`05001oogoo00l00004Ool00`00Oomo
o`02Ool01`00Ool007oo001oo`0?00]oo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eo
o`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00io
o`0300moogoo00Uoo`03001oogoo009oo`03001oogoo009oo`0500003gooOol00004Ool00`0?Oomo
o`05Ool00`00Oomoo`0nOol0051oo`03001oogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`03
00moogoo00=oo`03001oogoo00=oo`03001oogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`03
001oogoo00=oo`03001oogoo00=oo`0300moogoo02Aoo`0300moogoo00Uoo`03001oo`0000Aoo`80
009oo`0300l007oo009oo`03001oogoo00=oo`07001oo`00Ool007oo00l02goo00<03gooOol027oo
00<03gooOol01Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol09Goo
00<03gooOol04goo00<03gooOol03Woo00<03gooOol02Goo00<007ooOol00Woo00<007ooOol00Woo
00D0000?Oomoo`0000Aoo`0300moogoo00Eoo`03001oogoo03ioo`00D7oo00<007ooOol097oo00<0
3gooOol057oo00<03gooOol03Woo00<03gooOol00goo00<007ooOol00goo00<007ooOol01goo00<0
3gooOol01Woo00<03gooOol01Goo00<007ooOol00goo00<007ooOol00goo00<03gooOol097oo00<0
3gooOol02Goo00<007oo000017oo0P000Woo00<03`0000000Woo00<007ooOol00goo00L007oo001o
o`00Ool03`0;Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool0
0`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool0
0`00Oomoo`02Ool00`00Oomoo`02Ool01@0000moogoo000017oo00<03gooOol01Goo00<007ooOol0
?Woo001@Ool3001B00l00`0000l03`0300l00`0000l03`0H00l00`0000l03`0300l00`0000l03`0f
00l00`0000l0000400l2000300l2000200l00`0000l03`0300l01@0000l03`0?0000S`0?00<0000?
00l00P0?00<0000?00l00P0?00D0000?00l03`0000X03`<0041oo`00D7oo00<007ooOol097oo00<0
3gooOol057oo00<03gooOol03Woo00<03gooOol00goo00<007ooOol00goo00<007ooOol01goo00<0
3gooOol01Woo00<03gooOol01Goo00<007ooOol00goo00<03`00Ool00goo00<03gooOol097oo00<0
3gooOol02Goo00<007oo000017oo0P000Woo00<03`0000000Woo00<007ooOol00goo00L007ooOomo
o`00Ool03`0;Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool0
0`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool0
0`00Oomoo`02Ool00`00Oomoo`02Ool01@0000moogoo000017oo00<03gooOol01Goo00<007ooOol0
?Woo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`03Ool00`00
Oomoo`03Ool00`00Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`00Oomoo`03Ool00`0?
001oo`03Ool00`0?Oomoo`0TOol00`0?Oomoo`09Ool00`00Ool00004Ool20002Ool00`0?00000002
Ool00`00Oomoo`03Ool01`00Oomoogoo001oo`0?00]oo`0300moogoo00Qoo`0300moogoo00Eoo`03
00moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`03
00moogoo00ioo`0300moogoo00Uoo`03001oogoo009oo`03001oogoo009oo`0500003gooOol00004
Ool00`0?Oomoo`05Ool00`00Oomoo`0nOol0051oo`03001oogoo02Aoo`0300moogoo01Aoo`0300mo
ogoo00ioo`0300moogoo00=oo`03001oogoo00=oo`03001oogoo00Moo`0300moogoo00Ioo`0300mo
ogoo00Eoo`03001oogoo00=oo`0300l007oo00=oo`0300moogoo02Aoo`0300moogoo00Uoo`03001o
o`0000Aoo`80009oo`0600l007oo001oo`001Goo00L007ooOomoo`00Ool03`0;Ool00`0?Oomoo`08
Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0U
Ool00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`02Ool00`00Oomoo`02
Ool01@0000moogoo000017oo00<03gooOol01Goo00<007ooOol0?Woo001@Ool3001A00l00`0000l0
3`0400l00`0000l03`0H00l00`0000l03`0400l00`0000l03`0a00l01@0000l03`0?00000P0?00L0
000?00l03`0000l0000200l01@0000l0000?00001@0?00<0000?00l0V`0?00D0000?00l03`0000X0
3`<0041oo`00D7oo00<007ooOol097oo00<03gooOol057oo00<03gooOol03Woo00<03gooOol00Woo
00<007ooOol017oo00<007ooOol01goo00<03gooOol01Woo00<03gooOol01Goo00<007ooOol00goo
00<03`00Ool00goo00<03gooOol097oo00<03gooOol01Goo00D007ooOomoo`00009oo`0>001oogoo
Ool007oo001oo`0?001oo`00Ool000Eoo`03001oogoo00=oo`0300moogoo00Uoo`0300moogoo00Qo
o`0300moogoo00Eoo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Eo
o`0300moogoo01=oo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Moo`0500003gooOol0
0004Ool00`0?Oomoo`05Ool00`00Oomoo`0nOol0051oo`03001oogoo02Aoo`0300moogoo01Aoo`03
00moogoo00ioo`0300moogoo009oo`03001oogoo00Aoo`03001oogoo00Moo`0300moogoo00Ioo`03
00moogoo00Eoo`03001oogoo00=oo`0300l007oo00=oo`0300moogoo02Aoo`0300moogoo00Eoo`05
001oogooOol00002Ool01000Oomoo`000Woo00P007oo00l007oo001oo`002goo00<03gooOol02Goo
00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo
00<03gooOol09Goo00<03gooOol04goo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol01goo
00D0000?Oomoo`0000Aoo`0300moogoo00Eoo`03001oogoo03ioo`00;Goo0P001goo1@001Woo1@00
2Woo0`00D@0?00<0000?00l0100?00<0000?00l0600?00<0000?00l0100?00<0000?00l0<@0?00D0
000?00l03`0000803`0400003`0?000200l0200000l03`0000l0000?002S00l00`0000l03`0<00l3
000<Ool20007Ool50006Ool5000KOol002eoo`8000Ioo`L000Aoo`L000Uoo`03001oogoo02Aoo`03
00moogoo01Aoo`0300moogoo00ioo`0300moogoo009oo`03001oogoo00Aoo`0300l007oo00Moo`03
00moogoo00Ioo`0300moogoo00Aoo`0300003goo00Aoo`0300l007oo00=oo`0300moogoo02Aoo`03
00moogoo00Eoo`05001oogooOol00002Ool01000Oomoo`000Woo00<007oo00000Woo00<007oo0000
2goo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<03gooOol0
0goo00<03gooOol00goo00<03gooOol09Goo00<03gooOol04goo00<03gooOol03Woo00<03gooOol0
2Goo00<03gooOol01goo00<0000?Ool01Woo00<03gooOol01Goo00<007ooOol02Woo0P001Woo1`00
17oo1`006Woo000]Ool20006Ool20003Ool20004Ool20003Ool20009Ool3001A00l00`0000l03`05
00l00`0000l03`0F00l00`0000l03`0500l00`0000l03`0a00l01@0000l03`0?00000P0?00@0000?
00l000803`0300003`0000803`0300003`000;803`<000aoo`8000Ioo`8000=oo`8000Aoo`8000=o
o`8001Yoo`00;Goo0P001Goo0P001Goo0P000Woo0P001Goo0P0027oo00<007ooOol097oo00<03goo
Ool057oo00<03gooOol03Woo00<03gooOol00Woo00<007ooOol017oo00<03`00Ool01goo00<03goo
Ool01Woo00<03gooOol017oo00<0000?Ool017oo00<03`00Ool00goo00<03gooOol097oo00<03goo
Ool01Goo00D007ooOomoo`00009oo`04001oogoo0002Ool00`00Ool00003Ool2000;Ool00`0?Oomo
o`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomo
o`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomo
o`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`00Oomoo`0:Ool20005Ool20005Ool20002Ool2
0005Ool2000IOol002eoo`8000Eoo`8000Eoo`80009oo`8000Eoo`8000Qoo`<005403`0300003`0?
00D03`0300003`0?01H03`0300003`0?00D03`0300003`0?03403`0500003`0?00l0000200l01000
00l03`000P0?00<0000?00000`0?00<0000?00l0/@0?0`0037oo0P001Goo0P001Goo0P000Woo0P00
1Goo0P006Goo000]Ool20005Ool20005Ool20002Ool20005Ool20008Ool00`00Oomoo`0TOol00`0?
Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`02Ool00`00Oomoo`04Ool00`0?001oo`07Ool00`0?
Oomoo`06Ool00`0?Oomoo`04Ool00`0000moo`04Ool00`0?001oo`03Ool00`0?Oomoo`0TOol00`0?
Oomoo`05Ool01@00Oomoogoo00000Woo00@007ooOol0009oo`03001oo`0000=oo`03001oogoo00Yo
o`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`0300moogoo00=o
o`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00ioo`0300moogoo00Uo
o`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03001oogoo00Yoo`8000Eoo`8000Eo
o`80009oo`8000Eoo`8001Uoo`00;Goo0P001Goo0P001Goo0P000Woo0P001Goo0P0027oo1000C`0?
00<0000?00l01P0?00<0000?00l05P0?00<0000?00l01@0?00<0000?00l0<00?0P000`0?00@0000?
00l000803`0600003`0?00003`000`0?00<0000?00l0/00?100037oo0P001Goo0P001Goo0P000Woo
0P001Goo0P006Goo000]Ool20005Ool20005Ool20002Ool20005Ool20008Ool00`00Oomoo`0TOol0
0`0?Oomoo`0DOol00`0?Oomoo`0>Ool01@0?Oomoogoo00001goo00<03`00Ool01goo00<03gooOol0
1Woo00<03gooOol017oo00<0000?Ool017oo00<03`00Ool00goo00<03gooOol097oo00<03gooOol0
17oo0P000goo00H007ooOol007oo0003Ool00`00Ool00003Ool00`00Oomoo`0:Ool00`0?Oomoo`09
Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03
Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08
Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`00Oomoo`0:Ool20005Ool20005Ool20002Ool20005
Ool2000IOol002eoo`8000Eoo`8000Eoo`80009oo`8000Eoo`8000Qoo`03001oogoo02Aoo`0300mo
ogoo01Aoo`0300moogoo00ioo`0500moogooOol00007Ool00`0?001oo`07Ool00`0?Oomoo`06Ool0
0`0?Oomoo`04Ool00`0000moo`04Ool00`0?001oo`03Ool00`0?Oomoo`0TOol00`0?Oomoo`04Ool2
0003Ool01P00Oomoo`00Ool000Aoo`8000=oo`03001oogoo00Yoo`0300moogoo00Uoo`0300moogoo
00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo
02Eoo`0300moogoo01=oo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo
00Eoo`0300moogoo00Eoo`03001oogoo00Yoo`8000Eoo`8000Eoo`80009oo`8000Eoo`8001Uoo`00
;Goo0P001Goo0P001Goo0P000Woo0P001Goo0P0027oo00<007ooOol097oo00<03gooOol057oo00<0
3gooOol03Woo00D03gooOomoo`0000Moo`0300l007oo00Moo`0300moogoo00Ioo`0300moogoo00Ao
o`0300003goo00Aoo`0300l007oo00=oo`0300moogoo02Aoo`0300moogoo00Aoo`03001oo`00009o
o`06001oogoo001oo`0017oo0P000goo00<007ooOol02Woo00<03gooOol02Goo00<03gooOol027oo
00<03gooOol01Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol09Goo
00<03gooOol04goo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo
00<03gooOol01Goo00<007ooOol02Woo0P001Goo0P001Goo0P000Woo0P001Goo0P006Goo000ZOol2
00000goo00000005Ool20005Ool20002Ool20005Ool20008Ool00`00Oomoo`0TOol00`0?Oomoo`0D
Ool00`0?Oomoo`0>Ool01@0?Oomoogoo00001goo00<03`00Ool01goo00<03gooOol01Woo00<03goo
Ool017oo00<0000?Ool017oo00<03goo00000goo00<03gooOol097oo00<03gooOol017oo00<007oo
00000Woo00H007ooOol007oo0005Ool00`00Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool0
0`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0UOol0
0`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool0
0`0?Oomoo`05Ool00`00Oomoo`07Ool200000goo00000005Ool20005Ool20002Ool20005Ool2000I
Ool002Yoo`D000Eoo`8000Eoo`80009oo`8000Eoo`8000Qoo`03001oogoo02Aoo`0300moogoo01Ao
o`0300moogoo00ioo`0500moogooOol00007Ool00`0?001oo`07Ool00`0?Oomoo`06Ool00`0?Oomo
o`04Ool00`0000moo`04Ool00`0?Ool00003Ool00`0?Oomoo`0TOol00`0?Oomoo`04Ool01@00Ool0
07oo000017oo0P001Goo00<007ooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol0
1Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol09Goo00<03gooOol0
4goo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol0
1Goo00<007ooOol01goo1@001Goo0P001Goo0P000Woo0P001Goo0P006Goo000/Ool30006Ool20003
Ool20004Ool20003Ool20009Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool01@0?
Oomoogoo00001goo00<03goo00001goo00<03gooOol01Woo00<03gooOol00goo00<007oo00l01Goo
00<03goo00000goo00<03gooOol097oo00<03gooOol017oo00D007oo001oo`0000Aoo`8000Eoo`03
00moogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03
00moogoo00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00ioo`03
00moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03001oogoo00Uoo`<0
00Ioo`8000=oo`8000Aoo`8000=oo`8001Yoo`00;Goo0P001Woo1`0017oo1`002Goo00<007ooOol0
97oo00<03gooOol057oo00<03gooOol03Woo00D03gooOomoo`0000Moo`0300moo`0000Moo`0300mo
ogoo00Ioo`03001oogoo00=oo`03001oo`0?00Eoo`0300moo`0000=oo`0300moogoo02Aoo`0300mo
ogoo00Aoo`05001oo`00Ool00004Ool20005Ool00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomo
o`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomo
o`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomo
o`05Ool00`0?Oomoo`05Ool00`00Oomoo`0:Ool20006Ool70004Ool7000JOol002eoo`8000Moo`D0
00Ioo`D000Yoo`03001oogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0500moogooOol00007
Ool00`0?Ool00007Ool00`0?Oomoo`06Ool30003Ool00`00Ool03`05Ool00`0?Ool00003Ool00`0?
Oomoo`0TOol00`0?Oomoo`04Ool01@00Ool007oo000017oo0P001Goo00<03gooOol03Woo00<03goo
Ool02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<03gooOol00goo00<03goo
Ool00goo00<03gooOol09Goo00<03gooOol04goo00<03gooOol03Woo00<03gooOol02Goo00<03goo
Ool027oo00<03gooOol01Goo00<03gooOol01Goo00<007ooOol02Woo0P001goo1@001Woo1@006goo
001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool0100?Oomoo`0027oo00<03goo
00001goo00<03gooOol01Goo00D0000?Oomoo`00009oo`03001oo`0?00Eoo`0300moo`0000=oo`03
00l007oo02Aoo`0300moogoo00Aoo`05001oo`00Ool00004Ool00`00Oomoo`04Ool00`0?Oomoo`0>
Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03
Ool00`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09
Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`00Oomoo`0nOol0051oo`03001o
ogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0400moogoo0008Ool00`0?Ool00007Ool00`0?
Oomoo`05Ool00`0000moo`02Ool300000goo00moo`04Ool00`0?Ool00003Ool00`0?001oo`0TOol0
0`0?Oomoo`04Ool01@00Ool007oo000017oo00<007ooOol017oo00<03gooOol03Woo00<03gooOol0
2Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol0
0goo00<03gooOol09Goo00<03gooOol04goo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol0
27oo00<03gooOol01Goo00<03gooOol01Goo00<007ooOol0?Woo001@Ool00`00Oomoo`0TOol00`0?
Oomoo`0DOol00`0?Oomoo`0>Ool0100?Oomoo`0027oo00<03goo00001goo00<03gooOol01Goo00<0
000?Ool017oo00<007oo00l01Goo00<03goo00000goo00<03`00Ool097oo00<03gooOol017oo00D0
07oo001oo`0000Aoo`03001oogoo00Aoo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Qo
o`0300moogoo00Eoo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Eo
o`0300moogoo01=oo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eo
o`0300moogoo00Eoo`03001oogoo03ioo`00D7oo00<007ooOol097oo00<03gooOol057oo00<03goo
Ool03Woo00@03gooOol000Qoo`0300moo`0000Moo`0300moogoo00Eoo`0300003goo00Ioo`0300mo
ogoo00=oo`0300moo`0000=oo`03001oo`0002Aoo`0300moogoo00Aoo`05001oo`00Ool00004Ool0
0`00Oomoo`04Ool00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool0
0`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol0
0`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool0
0`00Oomoo`0nOol0051oo`<004l03`0300003`0?00P03`0300003`0?00`03`0300003`0?00l03`07
00003`0?00l0000?0000:`0?00D0000?00003`0000@03`0300003`0?0;`03`<0041oo`00D7oo00<0
07ooOol097oo00<03gooOol057oo00<03gooOol03Woo00@03gooOol000Qoo`0300moo`0000Moo`03
00moogoo00Aoo`03001oo`0?00Moo`0300moogoo00=oo`0300moo`0000=oo`03001oo`0002Aoo`03
00moogoo00Aoo`05001oo`00Ool0000;Ool00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08
Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0U
Ool00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05
Ool00`0?Oomoo`05Ool00`00Oomoo`0nOol0051oo`03001oogoo02Aoo`0300moogoo01Aoo`0300mo
ogoo00ioo`0400moogoo0008Ool00`0?Ool00007Ool00`0?Oomoo`04Ool00`00Ool03`07Ool00`0?
Oomoo`03Ool00`0?Ool00003Ool00`00Ool0000TOol00`0?Oomoo`04Ool00`00Oomoo`02000;Ool0
0`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool0
0`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool0
0`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`00Oomoo`0nOol0
051oo`03001oogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0400moogoo0008Ool0100?Oomo
o`001Woo00<03gooOol017oo00<007oo00l01goo00<03gooOol00goo00<03goo00000goo00<007oo
000097oo00<03gooOol00goo00<007ooOol00Woo00<007ooOol02Goo00<03gooOol03Woo00<03goo
Ool02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<03gooOol00goo00<03goo
Ool00goo00<03gooOol09Goo00<03gooOol04goo00<03gooOol03Woo00<03gooOol02Goo00<03goo
Ool027oo00<03gooOol01Goo00<03gooOol01Goo00<007ooOol0?Woo001@Ool00`00Oomoo`0TOol0
0`0?Oomoo`0DOol00`0?Oomoo`0>Ool0100?Oomoo`0027oo00@03gooOol000Ioo`0300moogoo00=o
o`04001oogoo00l7Ool00`0?Oomoo`03Ool00`0?Ool00002Ool0100000moo`0097oo00<03gooOol0
0goo00<007ooOol03Woo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol0
1Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol09Goo00<03gooOol0
4goo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol0
1Goo00<007ooOol0?Woo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?
Ool00009Ool0100?Oomoo`001Woo00<03gooOol00goo00@007ooOol03`Moo`0300moogoo00=oo`03
00moo`00009oo`0500003gooOol0000SOol00`0?Oomoo`03Ool00`00Oomoo`0>Ool00`0?Oomoo`0>
Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03
Ool00`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09
Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`00Oomoo`0nOol0051oo`03001o
ogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0300moo`0000Uoo`0400moogoo0006Ool00`0?
Oomoo`03Ool01000Oomoo`0?1goo00<03gooOol00goo00<03goo00000Woo00D0000?Oomoo`0002=o
o`0400moogoo0002Ool00`00Oomoo`0>Ool00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08
Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0U
Ool00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05
Ool00`0?Oomoo`05Ool00`00Oomoo`0nOol0051oo`<004h03`0300003`0?00X03`0300003`0?00T0
3`0300003`0?01803`0300003`0000<03`0300003`0?02@03`80000300l0000?0<L03`<0041oo`00
D7oo00<007ooOol097oo00<03gooOol057oo00<03gooOol03Woo00<03goo00002Goo00@03gooOol0
00Ioo`0300moogoo009oo`05001oogooOol03`07Ool00`0?Oomoo`03Ool00`0?Oomoo`0200001Goo
00moogoo00008goo00<03gooOol00P0000=oo`00Ool03goo00<03gooOol03Woo00<03gooOol02Goo
00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo
00<03gooOol09Goo00<03gooOol04goo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo
00<03gooOol01Goo00<03gooOol01Goo00<007ooOol0?Woo001@Ool00`00Oomoo`0TOol00`0?Oomo
o`0DOol00`0?Oomoo`0>Ool00`0?Ool00009Ool0100?Oomoo`001Woo00<03gooOol00Woo00D007oo
Oomoo`0?00Moo`0300moogoo00=oo`0300moogoo00800005Ool03gooOol0000SOol01@0?Oomoo`00
Ool00P0047oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<0
3gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol09Goo00<03gooOol04goo00<0
3gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<0
07ooOol0?Woo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Ool00009
Ool0100?Oomoo`001Woo00<03gooOol00Woo00D007ooOomoo`0?00Moo`0300moogoo00=oo`0300mo
ogoo00800005Ool03gooOol0000SOol01@0?Oomoo`00Ool00P0047oo00<03gooOol03Woo00<03goo
Ool02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<03gooOol00goo00<03goo
Ool00goo00<03gooOol09Goo00<03gooOol04goo00<03gooOol03Woo00<03gooOol02Goo00<03goo
Ool027oo00<03gooOol01Goo00<03gooOol01Goo00<007ooOol0?Woo001@Ool00`00Oomoo`0TOol0
0`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Ool00009Ool0100?Oomoo`001Woo00D03gooOomoo`00
00Aoo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00800003Ool03goo009oo`03001oogoo
021oo`0400moogoo0002Ool00`00Oomoo`0>Ool00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomo
o`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomo
o`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomo
o`05Ool00`0?Oomoo`05Ool00`00Oomoo`0nOol0051oo`03001oogoo02Aoo`0300moogoo01Aoo`03
00moogoo00ioo`0300moo`0000Uoo`0400moogoo0006Ool01@0?Oomoogoo000017oo00<03gooOol0
1Goo00<03gooOol00goo00<03gooOol00P0000=oo`0?Ool00Woo00<007ooOol087oo00<03goo0000
57oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol0
1Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol09Goo00<03gooOol04goo00<03gooOol0
3Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<007ooOol0
?Woo001@Ool3001>00l00`0000l03`0:00l00`0000l03`0800l00`0000l03`0C00l00`0000l03`04
00l00`0000l03`0R00l00`0000l03`3:00l30010Ool0051oo`03001oogoo02Aoo`0300moogoo01Ao
o`0300moogoo00ioo`0300l007oo00Uoo`0500moogooOol00005Ool01@0?Oomoogoo000017oo00<0
3gooOol01Goo00<03gooOol00goo00@03gooOol0009oo`0500moogooOol0000ROol00`0?Ool0000D
Ool00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05
Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>
Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`00Oomoo`0n
Ool0051oo`03001oogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0300l007oo00Uoo`0500mo
ogooOol00005Ool0100?Oomoo`001Goo00<03gooOol01Goo00<03gooOol00goo00@03gooOol0009o
o`0500moogooOol0000ROol00`0?Ool0000DOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomo
o`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomo
o`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomo
o`05Ool00`0?Oomoo`05Ool00`00Oomoo`0nOol0051oo`03001oogoo02Aoo`0300moogoo01Aoo`03
00moogoo00ioo`0300l007oo00Uoo`0500moogooOol00005Ool0100?Oomoo`001Goo00<03gooOol0
1Goo00<03gooOol00goo00<03gooOol00goo00D03gooOomoo`00029oo`0300moo`0001Aoo`0300mo
ogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`0300mo
ogoo00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00ioo`0300mo
ogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03001oogoo03ioo`00D7oo
0`00C@0?00<0000?00l0300?00<0000?00l01@0?00<0000?00l07@0?00<0000?00l08@0?00<0000?
00l0bP0?0`00@7oo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?001o
o`09Ool01@0?Oomoogoo00001Goo00<03goo00001Woo00<03gooOol01Goo00<03gooOol00goo00<0
3gooOol00goo00<03gooOol00Woo00<007ooOol07goo00<03goo000057oo00<03gooOol03Woo00<0
3gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<03gooOol00goo00<0
3gooOol00goo00<03gooOol09Goo00<03gooOol04goo00<03gooOol03Woo00<03gooOol02Goo00<0
3gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<007ooOol0?Woo001@Ool00`00Oomoo`0T
Ool00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?001oo`09Ool01@0?Oomoogoo00001Goo00<03`00
Ool01Woo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol00Woo00<007oo
Ool07goo00<03goo000057oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03goo
Ool01Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol09Goo00<03goo
Ool04goo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03goo
Ool01Goo00<007ooOol0?Woo001@Ool3001=00l00`0000l03`0<00l00`0000l03`0400l00`0000l0
3`0N00l00`0000l03`0Q00l00`0000l03`3:00l30010Ool0051oo`03001oogoo02Aoo`0300moogoo
01Aoo`0300moogoo00ioo`0300l007oo00Uoo`0500moogooOol00005Ool00`00Oomoo`06Ool00`0?
Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`00Oomoo`0NOol00`0?
Ool0000DOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?
Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?
Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`00
Oomoo`0nOol0051oo`03001oogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0300l007oo00Uo
o`0500moogooOol00004Ool00`0000moo`07Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomo
o`03Ool00`0?Oomoo`03Ool00`00Oomoo`0NOol00`0?Ool0000DOol00`0?Oomoo`0>Ool00`0?Oomo
o`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomo
o`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomo
o`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`00Oomoo`0nOol0051oo`<004d03`0300003`0?
00`03`0300003`0?00803`0300003`0?02403`0300003`0?02003`0300003`0?0<X03`<0041oo`00
8Woo0P001goo1@001Woo1@001Woo1@002Woo00<007ooOol097oo00<03gooOol057oo00<03gooOol0
3Woo00<007ooOol02Goo00<03gooOol00Woo00H007ooOol007oo00l8Ool00`0?Oomoo`05Ool00`0?
Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`04Ool00`00Oomoo`0MOol00`0?001oo`0DOol00`0?
Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?
Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?
Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`00Oomoo`0:Ool20007
Ool50006Ool50006Ool5000@Ool0029oo`8000Ioo`L000Aoo`L000Aoo`L000Uoo`<004`03`030000
3`0?00h03`0300003`0002H03`0300003`0?01h03`0300003`0?0</03`<000aoo`8000Ioo`L000Ao
o`L000Aoo`L000moo`008Woo0P001Woo0P000goo0P0017oo0P000goo0P0017oo0P000goo0P002Goo
00<007ooOol097oo00<03gooOol057oo00<03gooOol03Woo00<007ooOol02Goo00<03gooOol00Woo
00<007oo00000Woo00<03gooOol01Woo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo
00<03gooOol017oo00<007ooOol077oo00<0000?00005Goo00<03gooOol03Woo00<03gooOol02Goo
00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo
00<03gooOol09Goo00<03gooOol04goo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo
00<03gooOol01Goo00<03gooOol01Goo00<007ooOol02Woo0P001Woo0P000goo0P0017oo0P000goo
0P0017oo0P000goo0P003goo000ROol20005Ool20005Ool20002Ool20005Ool20002Ool20005Ool2
0008Ool3001<00l00`0000l03`0>00l2000X00l00`0000l03`0K00l3003=00l3000<Ool20005Ool2
0005Ool20002Ool20005Ool20002Ool20005Ool2000>Ool0029oo`8000Eoo`8000Eoo`80009oo`80
00Eoo`80009oo`8000Eoo`8000Qoo`03001oogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`03
001oogoo00Uoo`0300moogoo009oo`8000=oo`0300moogoo00Ioo`0300moogoo00Eoo`0300moogoo
00=oo`0300moogoo00=oo`0300moogoo00Eoo`03001oogoo01]oo`<001Eoo`0300moogoo00ioo`03
00moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`0300moogoo00=oo`03
00moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00ioo`0300moogoo00Uoo`03
00moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03001oogoo00Yoo`8000Eoo`8000Eoo`80
009oo`8000Eoo`80009oo`8000Eoo`8000ioo`008Woo0P001Goo0P001Goo0P000Woo0P001Goo0P00
0Woo0P001Goo0P0027oo1000B`0?00<0000?00l03P0?00<0000?00l09`0?00<0000?00l06`0?00<0
000?0000c00?100037oo0P001Goo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P003Woo000R
Ool20005Ool20005Ool20002Ool20005Ool20002Ool20005Ool20008Ool00`00Oomoo`0TOol00`0?
Oomoo`0DOol00`0?Oomoo`0>Ool00`00Oomoo`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?
Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`06Ool00`00Oomoo`0JOol00`00
00l0000EOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?
Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?
Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`00
Oomoo`0:Ool20005Ool20005Ool20002Ool20005Ool20002Ool20005Ool2000>Ool0029oo`8000Eo
o`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8000Qoo`03001oogoo02Aoo`0300moogoo01Ao
o`0300moogoo00ioo`03001oogoo00Uoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eo
o`0300moogoo00=oo`0300moogoo00=oo`0300moogoo00Ioo`03001oogoo01Yoo`0300003goo01Eo
o`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eo
o`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00io
o`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03001oogoo00Yo
o`8000Eoo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8000ioo`008Woo0P001Goo0P001Goo
0P000Woo0P001Goo0P000Woo0P001Goo0P0027oo00<007ooOol097oo00<03gooOol057oo00<03goo
Ool03Woo00<007ooOol02Goo00<03gooOol01goo00<03gooOol01Woo00<03gooOol01Goo00<03goo
Ool00goo00<03gooOol00goo00<03gooOol01Woo00<007ooOol06Woo00<0000?Ool05Goo00<03goo
Ool03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<03goo
Ool00goo00<03gooOol00goo00<03gooOol09Goo00<03gooOol04goo00<03gooOol03Woo00<03goo
Ool02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<007ooOol02Woo0P001Goo
0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P003Woo000ROol20005Ool20005Ool20002Ool2
0005Ool20002Ool20005Ool20008Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0=Ool0
0`0000moo`0:Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool0
0`0?Oomoo`03Ool00`0?Oomoo`06Ool00`00Oomoo`0IOol00`00Ool03`0FOol00`0?Oomoo`0>Ool0
0`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool0
0`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool0
0`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`00Oomoo`0:Ool20005Ool20005Ool2
0002Ool20005Ool20002Ool20005Ool2000>Ool001moo`800003Ool0000000Eoo`8000Eoo`80009o
o`8000Eoo`80009oo`8000Eoo`8000Qoo`03001oogoo02Aoo`0300moogoo01Aoo`0300moogoo00eo
o`0300003goo00Yoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0300moogoo00=o
o`0300moogoo00=oo`0300moogoo00Ioo`03001oogoo01Uoo`03001oo`0?01Ioo`0300moogoo00io
o`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`0300moogoo00=o
o`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00ioo`0300moogoo00Uo
o`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03001oogoo00Moo`800003Ool00000
00Eoo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8000ioo`007goo1@001Goo0P001Goo0P00
0Woo0P001Goo0P000Woo0P001Goo0P0027oo00<007ooOol097oo00<03gooOol057oo00@03gooOol0
00aoo`0300003goo00Yoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0300moogoo
00=oo`0300moogoo00=oo`0300moogoo00Ioo`03001oogoo01Uoo`03001oo`0?01Ioo`0300moogoo
00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`0300moogoo
00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00ioo`0300moogoo
00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03001oogoo00Moo`D000Eoo`80
00Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8000ioo`008Goo0`001Woo0P000goo0P0017oo0P00
0goo0P0017oo0P000goo0P002Goo0P009Goo00<03gooOol057oo00<03gooOol00P002goo00<0000?
Ool02Woo00<03gooOol01goo00<03gooOol01Woo00<03gooOol01Goo00<03gooOol00goo00<03goo
Ool00goo00<03gooOol01goo00<007ooOol067oo00<007oo00l05Woo00<03gooOol03Woo00<03goo
Ool02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<03gooOol00goo00<03goo
Ool00goo00<03gooOol09Goo00<03gooOol04goo00<03gooOol03Woo00<03gooOol02Goo00<03goo
Ool027oo00<03gooOol01Goo00<03gooOol01Goo00<007ooOol02Goo0`001Woo0P000goo0P0017oo
0P000goo0P0017oo0P000goo0P003goo000ROol20006Ool70004Ool70004Ool70009Ool2000UOol0
0`0?Oomoo`0DOol00`0?Ool00002Ool00`00Oomoo`08Ool00`0000moo`0:Ool00`0?Oomoo`07Ool0
0`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`07Ool0
0`00Oomoo`0HOol00`00Ool03`0FOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool0
0`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0UOol0
0`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool0
0`0?Oomoo`05Ool00`00Oomoo`0:Ool20006Ool70004Ool70004Ool7000?Ool0029oo`8000Moo`D0
00Ioo`D000Ioo`D000Yoo`03001oo`0002Aoo`0300moogoo01Aoo`0300moo`0000=oo`03001oogoo
00Moo`0300003goo00Yoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0300moogoo
00=oo`0300moogoo00=oo`0300moogoo00Moo`03001oogoo01Qoo`03001oo`0?01Ioo`0300moogoo
00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`0300moogoo
00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00ioo`0300moogoo
00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03001oogoo00Yoo`8000Moo`D0
00Ioo`D000Ioo`D0011oo`00D7oo00@007ooOol002=oo`0300moogoo01Aoo`0300moo`0000Aoo`03
001oogoo00Ioo`0300003goo00Yoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`03
00moogoo00=oo`0300moogoo00=oo`0300moogoo00Moo`03001oogoo01Qoo`03001oo`0?01Ioo`03
00moogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03
00moogoo00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00ioo`03
00moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03001oogoo03ioo`00
D7oo00@007ooOol002=oo`0300moogoo01Aoo`0300l007oo00Eoo`03001oogoo00Eoo`0300003goo
00Yoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo
00=oo`0300moogoo00Moo`03001oogoo01Qoo`03001oo`0?01Ioo`0300moogoo00ioo`0300moogoo
00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo
00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo
00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03001oogoo03ioo`00D7oo00D007ooOomoo`00029o
o`0300moogoo01Aoo`0300l007oo00Ioo`8000Eoo`0300003goo00Yoo`0300moogoo00Moo`0300mo
ogoo00Ioo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo00Moo`03001o
ogoo01Qoo`03001oo`0?01Ioo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300mo
ogoo00Eoo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300mo
ogoo01=oo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300mo
ogoo00Eoo`03001oogoo03ioo`00D7oo0`000P0?00<0000?00l0=`0?00<0000?00l01`0?00D0000?
00l03`0003h03`0300003`0?01P03`0300003`0?0<h03`<0041oo`00D7oo00<007ooOol00goo00<0
07ooOol07Woo00<03gooOol057oo00<007ooOol02Goo00H007ooOol007oo00l;Ool00`0?Oomoo`07
Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`07
Ool00`00Oomoo`0HOol00`00Ool03`0FOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08
Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0U
Ool00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05
Ool00`0?Oomoo`05Ool00`00Oomoo`0nOol0051oo`03001oogoo00=oo`03001oogoo01ioo`0300mo
ogoo01Aoo`03001oogoo00Yoo`05001oo`00Ool03`0;Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool0
0`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`07Ool00`00Oomoo`0HOol0
0`00Ool03`0FOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool0
0`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol0
0`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool0
0`00Oomoo`0nOol0051oo`03001oogoo00Aoo`03001oogoo01eoo`0300moogoo01Aoo`03001oogoo
00]oo`800003Ool03goo00Yoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0300mo
ogoo00=oo`0300moogoo00=oo`0300moogoo00Qoo`03001oogoo01Moo`03001oo`0?01Ioo`0300mo
ogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`0300mo
ogoo00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00ioo`0300mo
ogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03001oogoo03ioo`00D7oo
00<007ooOol01Goo00<007ooOol077oo00<03gooOol04goo00<0000?Ool03Goo00<007oo00l02goo
00<03gooOol01goo00<03gooOol01Woo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo
00<03gooOol027oo00<007ooOol05Woo00@007ooOol03aIoo`0300moogoo00ioo`0300moogoo00Uo
o`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=o
o`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Qo
o`0300moogoo00Eoo`0300moogoo00Eoo`03001oogoo03ioo`00D7oo00<007ooOol01Goo00<007oo
Ool077oo00<03gooOol04goo00<0000?Ool03goo00<03gooOol02Goo00<03gooOol01goo00<03goo
Ool01Woo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol027oo00<007oo
Ool05Woo00@007ooOol03aIoo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300mo
ogoo00Eoo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300mo
ogoo01=oo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300mo
ogoo00Eoo`03001oogoo03ioo`00D7oo00<007ooOol01Woo00<007ooOol06goo00<03gooOol04Woo
00<007oo00l047oo00<03gooOol02Goo00<03gooOol01goo00<03gooOol01Woo00<03gooOol01Goo
00<03gooOol00goo00<03gooOol00goo00<03gooOol027oo00<007ooOol05Woo00@007ooOol03aIo
o`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eo
o`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00io
o`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03001oogoo03io
o`00D7oo0`001`0?00<0000?00l0;`0?00<0000?00l0CP0?00<0000?00l05P0?00<0000?00l0c`0?
0`00@7oo001@Ool00`00Oomoo`07Ool00`00Oomoo`0JOol00`0?Oomoo`0BOol00`00Ool03`0@Ool0
0`0?Oomoo`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool0
0`0?Oomoo`03Ool00`0?Oomoo`08Ool00`00Oomoo`0FOol01000Oomoo`0?5Woo00<03gooOol03Woo
00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<03gooOol00goo
00<03gooOol00goo00<03gooOol09Goo00<03gooOol04goo00<03gooOol03Woo00<03gooOol02Goo
00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<007ooOol0?Woo001@Ool00`00Oomo
o`08Ool00`00Oomoo`0IOol00`0?Oomoo`0AOol01000Oomoo`0?47oo00<03gooOol02Goo00<03goo
Ool01goo00<03gooOol01Woo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03goo
Ool027oo00<007ooOol05Woo00@007ooOol03aIoo`0300moogoo00ioo`0300moogoo00Uoo`0300mo
ogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300mo
ogoo02Eoo`0300moogoo01=oo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300mo
ogoo00Eoo`0300moogoo00Eoo`03001oogoo03ioo`00D7oo00<007ooOol02Goo00<007ooOol067oo
00<03gooOol04Goo00@007ooOol03a1oo`0300moogoo00Uoo`0300moogoo00Moo`0300moogoo00Io
o`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo00Qoo`03001oogoo01Io
o`04001oogoo00lFOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05
Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0C
Ool00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05
Ool00`00Oomoo`0nOol0051oo`03001oogoo00Yoo`03001oogoo01Moo`0300moogoo015oo`04001o
ogoo00l@Ool00`0?Oomoo`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?
Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`08Ool00`00Oomoo`0FOol01000Oomoo`0?5Woo00<0
3gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<0
3gooOol00goo00<03gooOol00goo00<03gooOol09Goo00<03gooOol04goo00<03gooOol03Woo00<0
3gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<007ooOol0?Woo001@
Ool00`00Oomoo`0:Ool00`00Oomoo`0GOol00`0?Oomoo`0@Ool01@00Oomoogoo00l047oo00<03goo
Ool02Goo00<03gooOol01goo00<03gooOol01Woo00<03gooOol01Goo00<03gooOol00goo00<03goo
Ool00goo00<03gooOol02Goo00<007ooOol057oo00D007ooOomoo`0?01Ioo`0300moogoo00ioo`03
00moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`0300moogoo00=oo`03
00moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00ioo`0300moogoo00Uoo`03
00moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03001oogoo03ioo`00D7oo0`002`0?00<0
000?00l0:@0?00<0000?00l0D@0?00<0000?00l0500?00<0000?00l0d00?0`00@7oo001@Ool00`00
Oomoo`0<Ool00`00Oomoo`0EOol00`0?Oomoo`0@Ool01@00Oomoogoo00l047oo00<03gooOol02Goo
00<03gooOol01goo00<03gooOol01Woo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo
00<03gooOol02Goo00<007ooOol04goo00<007ooOol00Woo00<03gooOol057oo00<03gooOol03Woo
00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<03gooOol00goo
00<03gooOol00goo00<03gooOol09Goo00<03gooOol04goo00<03gooOol03Woo00<03gooOol02Goo
00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<007ooOol0?Woo001@Ool00`00Oomo
o`0<Ool00`00Oomoo`0EOol00`0?Oomoo`0?Ool00`00Oomoo`02Ool00`0?Oomoo`0>Ool00`0?Oomo
o`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomo
o`03Ool00`0?Oomoo`09Ool00`00Oomoo`0=Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`0?Oomo
o`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomo
o`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomo
o`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`00Oomo
o`0nOol0051oo`03001oogoo00eoo`03001oogoo01Aoo`0300moogoo00moo`03001oogoo009oo`03
00moogoo00ioo`0300moogoo00Uoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`03
00moogoo00=oo`0300moogoo00=oo`0300moogoo00Uoo`03001oogoo00eoo`8000Aoo`03001oogoo
009oo`0300moogoo01Aoo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo
00Eoo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo
01=oo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo
00Eoo`03001oogoo03ioo`00D7oo0`003P0?00<0000?00l09@0?00<0000?00l0DP0?00<0000?00l0
3@0?0P000`0?00<0000?00l0dP0?0`00@7oo001@Ool00`00Oomoo`0>Ool00`00Oomoo`0COol00`0?
Oomoo`0>Ool00`00Oomoo`03Ool00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`07Ool00`0?
Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`09Ool00`00
Oomoo`0=Ool00`00Ool00002Ool00`00Oomoo`03Ool00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?
Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?
Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?
Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`00Oomoo`0nOol0051oo`03001oogoo00mo
o`03001oogoo019oo`0300moogoo00ioo`03001oogoo00=oo`0300moogoo00ioo`0300moogoo00Uo
o`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=o
o`0300moogoo00Uoo`03001oogoo00aoo`06001oogoo001oo`001Woo00<03gooOol057oo00<03goo
Ool03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<03goo
Ool00goo00<03gooOol00goo00<03gooOol09Goo00<03gooOol04goo00<03gooOol03Woo00<03goo
Ool02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<007ooOol0?Woo001@Ool3
000@00l00`0000l03`0R00l00`0000l03`1C00l00`0000l03`0<00l0100000l03`0?0P00e@0?0`00
@7oo001@Ool00`00Oomoo`0@Ool00`00Oomoo`0AOol00`0?Oomoo`0=Ool00`00Oomoo`04Ool00`0?
Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?
Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0:Ool00`00Oomoo`0;Ool01@00Oomoogoo00001goo
00<03gooOol057oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo
00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol09Goo00<03gooOol04goo
00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo
00<007ooOol0?Woo001@Ool00`00Oomoo`0AOol00`00Oomoo`0@Ool00`0?Oomoo`0=Ool00`00Oomo
o`04Ool00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomo
o`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0:Ool00`00Oomoo`0;Ool00`00Oomo
o`09Ool00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomo
o`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomo
o`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomo
o`05Ool00`00Oomoo`0nOol0051oo`<001803`0300003`0?01l03`0300003`0?05D03`0300003`0?
00/03`0300003`0?0=P03`<0041oo`0067oo0P001goo1@001Woo1@001Woo1@001Woo1@002Goo00<0
07ooOol04goo00<007ooOol03Woo00<03gooOol037oo00<007ooOol01Goo00<03gooOol03Woo00<0
3gooOol02Goo00<03gooOol01goo00<03gooOol01Woo00<03gooOol01Goo00<03gooOol00goo00<0
3gooOol00goo00<03gooOol02Woo00<007ooOol02goo00<007ooOol02Goo00<03gooOol057oo00<0
3gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<0
3gooOol00goo00<03gooOol00goo00<03gooOol09Goo00<03gooOol04goo00<03gooOol03Woo00<0
3gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<007ooOol02Woo0P00
1goo1@001Woo1@001Woo1@001Woo1@001Goo000HOol20006Ool70004Ool70004Ool70004Ool70008
Ool3000C00l00`0000l03`0M00l00`0000l03`1F00l00`0000l03`0400l00`0000l03`0400l00`00
00l03`3H00l3000<Ool20006Ool70004Ool70004Ool70004Ool70004Ool001Qoo`8000Ioo`8000=o
o`8000Aoo`8000=oo`8000Aoo`8000=oo`8000Aoo`8000=oo`8000Qoo`03001oogoo01Aoo`03001o
ogoo00eoo`0300moogoo00aoo`03001oogoo00Eoo`0300moogoo00ioo`0300moogoo00Uoo`0300mo
ogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300mo
ogoo00Yoo`03001oogoo00Aoo`8000Eoo`03001oogoo00Uoo`0300moogoo01Aoo`0300moogoo00io
o`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`0300moogoo00=o
o`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00ioo`0300moogoo00Uo
o`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03001oogoo00Yoo`8000Ioo`8000=o
o`8000Aoo`8000=oo`8000Aoo`8000=oo`8000Aoo`8000=oo`8000Aoo`0067oo0P001Goo0P001Goo
0P000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P001goo00<007ooOol05Goo00<007oo
Ool037oo00<03gooOol02goo00<007ooOol01Woo00<03gooOol03Woo00<03gooOol02Goo00<03goo
Ool01goo00<03gooOol01Woo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03goo
Ool02Woo00<007ooOol017oo00<007oo00000goo00<007ooOol02Woo00<03gooOol057oo00<03goo
Ool03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<03goo
Ool00goo00<03gooOol00goo00<03gooOol09Goo00<03gooOol04goo00<03gooOol03Woo00<03goo
Ool02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<007ooOol02Woo0P001Goo
0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000goo000HOol20005Ool2
0005Ool20002Ool20005Ool20002Ool20005Ool20002Ool20005Ool20007Ool3000E00l00`0000l0
3`0J00l00`0000l03`1G00l00`0000l03`0300l0100000l03`000`0?00<0000?00l0f@0?0`0037oo
0P001Goo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000goo000HOol2
0005Ool20005Ool20002Ool20005Ool20002Ool20005Ool20002Ool20005Ool20007Ool00`00Oomo
o`0FOol00`00Oomoo`0;Ool00`0?Oomoo`0;Ool00`00Oomoo`06Ool00`0?Oomoo`0>Ool00`0?Oomo
o`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomo
o`03Ool00`0?Oomoo`0;Ool00`00Oomoo`02Ool01000Oomoogoo0P0000=oo`00Ool02goo00<03goo
Ool057oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03goo
Ool01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol09Goo00<03gooOol04goo00<03goo
Ool03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<007oo
Ool02Woo0P001Goo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000goo
000HOol20005Ool20005Ool20002Ool20005Ool20002Ool20005Ool20002Ool20005Ool20007Ool4
000F00l2000H00l00`0000l03`1I00l00`0000l03`0200l00`0000l03`0300l2003J00l4000<Ool2
0005Ool20005Ool20002Ool20005Ool20002Ool20005Ool20002Ool20005Ool20003Ool001Qoo`80
00Eoo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8000Moo`03001oogoo
01Uoo`8000Uoo`0300moogoo00Yoo`03001oogoo00Moo`0300moogoo00ioo`0300moogoo00Uoo`03
00moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`03
00moogoo00]oo`03001oogoo009oo`03001oogoo015oo`0300moogoo01Aoo`0300moogoo00ioo`03
00moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`0300moogoo00=oo`03
00moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00ioo`0300moogoo00Uoo`03
00moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03001oogoo00Yoo`8000Eoo`8000Eoo`80
009oo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8000=oo`0067oo0P001Goo0P001Goo0P00
0Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P001goo00<007ooOol06goo0P001goo00<0
3gooOol02Goo00<007ooOol027oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol01goo00<0
3gooOol01Woo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol02goo00<0
07ooOol00Woo00<007ooOol04Goo00<03gooOol057oo00<03gooOol03Woo00<03gooOol02Goo00<0
3gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<0
3gooOol09Goo00<03gooOol04goo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<0
3gooOol01Goo00<03gooOol01Goo00<007ooOol02Woo0P001Goo0P001Goo0P000Woo0P001Goo0P00
0Woo0P001Goo0P000Woo0P001Goo0P000goo000HOol20005Ool20005Ool20002Ool20005Ool20002
Ool20005Ool20002Ool20005Ool20007Ool00`00Oomoo`0MOol00`00Oomoo`04Ool00`0?Oomoo`09
Ool00`00Oomoo`08Ool00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06
Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0;Ool01@00Oomoogoo
000057oo00<03gooOol057oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03goo
Ool01Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol09Goo00<03goo
Ool04goo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03goo
Ool01Goo00<007ooOol02Woo0P001Goo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000Woo
0P001Goo0P000goo000EOol200000goo00000005Ool20005Ool20002Ool20005Ool20002Ool20005
Ool20002Ool20005Ool20007Ool00`00Oomoo`0NOol20004Ool00`0?Oomoo`09Ool00`00Oomoo`08
Ool00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05
Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0;Ool01@00Oomoogoo000057oo00<03goo
Ool057oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03goo
Ool01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol09Goo00<03gooOol04goo00<03goo
Ool03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<007oo
Ool01goo0P0000=oo`0000001Goo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P00
1Goo0P000goo000EOol50005Ool20005Ool20002Ool20005Ool20002Ool20005Ool20002Ool20005
Ool20007Ool00`00Oomoo`0POol20002Ool00`0?Oomoo`08Ool00`00Oomoo`09Ool00`0?Oomoo`0>
Ool00`0?Oomoo`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03
Ool00`0?Oomoo`03Ool00`0?Oomoo`0;Ool01@00Oomoogoo000057oo00<03gooOol057oo00<03goo
Ool03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<03goo
Ool00goo00<03gooOol00goo00<03gooOol09Goo00<03gooOol04goo00<03gooOol03Woo00<03goo
Ool02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<007ooOol01goo1@001Goo
0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000goo000GOol30006Ool2
0003Ool20004Ool20003Ool20004Ool20003Ool20004Ool20003Ool20008Ool00`00Oomoo`0ROol2
00000`0?Oomoo`08Ool00`00Oomoo`09Ool00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`07
Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0<
Ool01000Oomoo`0057oo00<03gooOol057oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol0
27oo00<03gooOol01Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol0
9Goo00<03gooOol04goo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol0
1Goo00<03gooOol01Goo00<007ooOol02Goo0`001Woo0P000goo0P0017oo0P000goo0P0017oo0P00
0goo0P0017oo0P000goo0P0017oo000HOol20006Ool70004Ool70004Ool70004Ool70008Ool00`00
Oomoo`0TOol20009Ool00`00Oomoo`09Ool00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`07
Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0<
Ool01000Oomoo`0057oo00<03gooOol057oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol0
27oo00<03gooOol01Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol0
9Goo00<03gooOol04goo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol0
1Goo00<03gooOol01Goo00<007ooOol02Woo0P001Woo1`0017oo1`0017oo1`0017oo1`0017oo000H
Ool20007Ool50006Ool50006Ool50006Ool50009Ool00`00Oomoo`0TOol00`0?Ool00007Ool00`00
Oomoo`0:Ool00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?
Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0<Ool01000Oomoo`0057oo00<0
3gooOol057oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<0
3gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol09Goo00<03gooOol04goo00<0
3gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<0
07ooOol02Woo0P001goo1@001Woo1@001Woo1@001Woo1@001Goo001@Ool00`00Oomoo`0TOol00`0?
Oomoo`020005Ool00`00Oomoo`0:Ool00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`07Ool0
0`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0<Ool0
0`00Ool0000EOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool0
0`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0UOol0
0`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool0
0`0?Oomoo`05Ool00`00Oomoo`0nOol0051oo`03001oogoo02Aoo`0300moogoo009oo`8000=oo`03
001oogoo00Yoo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Moo`0300moogoo00Ioo`03
00moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo00aoo`03001oo`0001Eoo`03
00moogoo01Aoo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`03
00moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`03
00moogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03
001oogoo03ioo`00D7oo00<007ooOol097oo00<03gooOol017oo0`003Goo00<03gooOol03Woo00<0
3gooOol02Goo00<03gooOol01goo00<03gooOol01Woo00<03gooOol01Goo00<03gooOol00goo00<0
3gooOol00goo00<03gooOol037oo00<007oo00005Goo00<03gooOol057oo00<03gooOol03Woo00<0
3gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<03gooOol00goo00<0
3gooOol00goo00<03gooOol09Goo00<03gooOol04goo00<03gooOol03Woo00<03gooOol02Goo00<0
3gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<007ooOol0?Woo001@Ool3000]00l00`00
00l03`1N00l00`0000l0003T00l30010Ool0051oo`03001oogoo02Aoo`0300moogoo01Aoo`0300mo
ogoo00ioo`0300moogoo00Uoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0300mo
ogoo00=oo`0300moogoo00=oo`0300moogoo00eoo`8001Eoo`0300moogoo01Aoo`0300moogoo00io
o`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`0300moogoo00=o
o`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00ioo`0300moogoo00Uo
o`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03001oogoo03ioo`00D7oo00<007oo
Ool097oo00<03gooOol057oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol01goo00<03goo
Ool01Woo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol03Goo00<007oo
Ool057oo00<03gooOol057oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03goo
Ool01Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol09Goo00<03goo
Ool04goo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03goo
Ool01Goo00<007ooOol0?Woo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool0
0`0?Oomoo`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool0
0`0?Oomoo`03Ool00`0?Oomoo`0=Ool00`00Oomoo`0DOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool0
0`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool0
0`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool0
0`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`00Oomoo`0nOol0051oo`03001oogoo
02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Moo`0300moogoo
00Ioo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo00eoo`03001oogoo
01Aoo`0300moogoo01Aoo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo
00Eoo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo
01=oo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo
00Eoo`03001oogoo03ioo`00D7oo00<007ooOol097oo00<03gooOol057oo00<03gooOol03Woo00<0
3gooOol02Goo00<03gooOol01goo00<03gooOol01Woo00<03gooOol01Goo00<03gooOol00goo00<0
3gooOol00goo00<03gooOol097oo00<03gooOol057oo00<03gooOol03Woo00<03gooOol02Goo00<0
3gooOol027oo00<03gooOol01Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<0
3gooOol09Goo00<03gooOol04goo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<0
3gooOol01Goo00<03gooOol01Goo00<007ooOol0?Woo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0D
Ool00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05
Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>
Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03
Ool00`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09
Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`00Oomoo`0nOol0051oo`03001o
ogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Moo`0300mo
ogoo00Ioo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Aoo`0300mo
ogoo01Aoo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300mo
ogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300mo
ogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03001o
ogoo03ioo`00D7oo0`00o`0?MP0?0`00@7oo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?
Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?
Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?
Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?
Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?
Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`00Oomoo`0nOol0051oo`03001oogoo02Ao
o`0300moogoo01Aoo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Moo`0300moogoo00Io
o`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Aoo`0300moogoo01Ao
o`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eo
o`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00io
o`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03001oogoo03io
o`00D7oo00<007ooOol097oo00<03gooOol057oo00<03gooOol03Woo00<03gooOol02Goo00<03goo
Ool01goo00<03gooOol01Woo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03goo
Ool097oo00<03gooOol057oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03goo
Ool01Goo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol09Goo00<03goo
Ool04goo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03goo
Ool01Goo00<007ooOol0?Woo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool0
0`0?Oomoo`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool0
0`0?Oomoo`03Ool00`0?Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool0
0`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool0
0`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool0
0`0?Oomoo`05Ool00`0?Oomoo`05Ool00`00Oomoo`0nOol0051oo`<00?l03gH03`<0041oo`00D7oo
00<007ooOol097oo00<03gooOol057oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol01goo
00<03gooOol01Woo00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol097oo
00<03gooOol057oo00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo
00<03gooOol01Goo00<03gooOol00goo00<03gooOol00goo00<03gooOol09Goo00<03gooOol04goo
00<03gooOol03Woo00<03gooOol02Goo00<03gooOol027oo00<03gooOol01Goo00<03gooOol01Goo
00<007ooOol0?Woo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomo
o`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomo
o`03Ool00`0?Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomo
o`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomo
o`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomo
o`05Ool00`0?Oomoo`05Ool00`00Oomoo`0nOol0051oo`03001oogoo02Aoo`0300moogoo01Aoo`03
00moogoo00ioo`0300moogoo00Uoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`03
00moogoo00=oo`0300moogoo00=oo`0300moogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`03
00moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`0300moogoo00=oo`03
00moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00ioo`0300moogoo00Uoo`03
00moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03001oogoo03ioo`00D7oo0`00o`0?MP0?
0`00@7oo001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool0
0`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool0
0`0?Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool0
0`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0UOol0
0`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool0
0`0?Oomoo`05Ool00`00Oomoo`0nOol0051oo`03001oogoo02Aoo`0300moogoo01Aoo`0300moogoo
00ioo`0300moogoo00Uoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0300moogoo
00=oo`0300moogoo00=oo`0300moogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0300moogoo
00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo
00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo
00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03001oogoo03ioo`00D7oo0`00o`0?MP0?0`00@7oo
001@Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomo
o`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomo
o`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomo
o`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomo
o`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomo
o`05Ool00`00Oomoo`0nOol0051oo`03001oogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`03
00moogoo00Uoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo00Eoo`0300moogoo00=oo`03
00moogoo00=oo`0300moogoo02Aoo`0300moogoo01Aoo`0300moogoo00ioo`0300moogoo00Uoo`03
00moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`0300moogoo00=oo`0300moogoo00=oo`03
00moogoo02Eoo`0300moogoo01=oo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Qoo`03
00moogoo00Eoo`0300moogoo00Eoo`03001oogoo03ioo`00D7oo0`00o`0?MP0?0`00@7oo0008Ool2
0007Ool50006Ool50006Ool50006Ool50006Ool50005Ool20007Ool00`00Oomoo`0TOol00`0?Oomo
o`0DOol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomo
o`05Ool00`0?Oomoo`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomo
o`0>Ool00`0?Oomoo`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomo
o`03Ool00`0?Oomoo`03Ool00`0?Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomo
o`09Ool00`0?Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`00Oomoo`0nOol000Qoo`80
00Ioo`L000Aoo`L000Aoo`L000Aoo`L000Aoo`L000Aoo`8000Moo`03001oogoo02Aoo`0300moogoo
01Aoo`0300moogoo00ioo`0300moogoo00Uoo`0300moogoo00Moo`0300moogoo00Ioo`0300moogoo
00Eoo`0300moogoo00=oo`0300moogoo00=oo`0300moogoo02Aoo`0300moogoo01Aoo`0300moogoo
00ioo`0300moogoo00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`0300moogoo
00=oo`0300moogoo00=oo`0300moogoo02Eoo`0300moogoo01=oo`0300moogoo00ioo`0300moogoo
00Uoo`0300moogoo00Qoo`0300moogoo00Eoo`0300moogoo00Eoo`03001oogoo03ioo`0027oo0P00
1Woo0P000goo0P0017oo0P000goo0P0017oo0P000goo0P0017oo0P000goo0P0017oo0P000goo0P00
3Goo0`00o`0?MP0?0`00@7oo0008Ool20005Ool20005Ool20002Ool20005Ool20002Ool20005Ool2
0002Ool20005Ool20002Ool20005Ool2000<Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomo
o`0>Ool00`0?Oomoo`09Ool00`00Oomoo`07Ool00`0?Oomoo`06Ool00`0?Oomoo`05Ool00`0?Oomo
o`03Ool00`0?Oomoo`03Ool00`00Oomoo`0TOol00`0?Oomoo`0DOol00`0?Oomoo`0>Ool00`0?Oomo
o`09Ool00`00Oomoo`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`0?Oomoo`03Ool00`0?Oomo
o`03Ool00`00Oomoo`0UOol00`0?Oomoo`0COol00`0?Oomoo`0>Ool00`0?Oomoo`09Ool00`00Oomo
o`08Ool00`0?Oomoo`05Ool00`0?Oomoo`05Ool00`00Oomoo`0nOol000Qoo`8000Eoo`8000Eoo`80
009oo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8000aoo`<002@03`03
00003`0?01@03`0300003`0?00h03`0300003`0?00T03`0300003`0?00L03`0300003`0?00H03`03
00003`0?00D03`0300003`0?00<03`0300003`0?00<03`0300003`0?02@03`0300003`0?01@03`03
00003`0?00h03`0300003`0?00T03`0300003`0?00P03`0300003`0?00D03`0300003`0?00D03`03
00003`0?00<03`0300003`0?00<03`0300003`0?02D03`0300003`0?01<03`0300003`0?00h03`03
00003`0?00T03`0300003`0?00P03`0300003`0?00D03`0300003`0?00<03`<0041oo`0027oo0P00
1Goo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P00
37oo00<007ooOol097oo00<007ooOol057oo00<007ooOol03Woo00<007ooOol02Goo00<007ooOol0
1goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol00goo00<007ooOol00goo00<007ooOol0
97oo00<007ooOol057oo00<007ooOol03Woo00<007ooOol02Goo00<007ooOol027oo00<007ooOol0
1Goo00<007ooOol01Goo00<007ooOol00goo00<007ooOol00goo00<007ooOol09Goo00<007ooOol0
4goo00<007ooOol03Woo00<007ooOol02Goo00<007ooOol027oo00<007ooOol01Goo00<007ooOol0
1Goo00<007ooOol0?Woo0008Ool20005Ool20005Ool20002Ool20005Ool20002Ool20005Ool20002
Ool20005Ool20002Ool20005Ool2000<Oooo001l0010Ool000Qoo`8000Eoo`8000Eoo`80009oo`80
00Eoo`80009oo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`800?moolQoo`0027oo0P001Goo
0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P00ogoo
b7oo0008Ool20005Ool20005Ool20002Ool20005Ool20002Ool20005Ool20002Ool20005Ool20002
Ool20005Ool2003oOoo8Ool000Eoo`800003Ool0000000Eoo`8000Eoo`80009oo`8000Eoo`80009o
o`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`800?moolQoo`001Goo1@001Goo0P001Goo0P00
0Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P00ogoob7oo0007Ool3
0006Ool20003Ool20004Ool20003Ool20004Ool20003Ool20004Ool20003Ool20004Ool20003Ool2
003oOoo9Ool000Qoo`8000Ioo`L000Aoo`L000Aoo`L000Aoo`L000Aoo`L00?moolUoo`0027oo0P00
1goo1@001Woo1@001Woo1@001Woo1@001Woo1@00ogoobWoo003oOoooOol=Ool00?mooomoo`eoo`00
@7oo1@001Goo0P001Goo1@0027oo0P00>goo1@001Goo0P001Goo1@001Woo10003Goo1@001Goo0P00
1goo0P00AWoo1@001Goo0P001Goo100077oo0P00Egoo1000FWoo000oOol70004Ool20004Ool70007
Ool2000jOol70004Ool20004Ool70004Ool7000:Ool70004Ool20007Ool20015Ool70004Ool20004
Ool7000JOol2001FOol7001HOol003moo`8000=oo`8000Yoo`8000=oo`8000Moo`8003Yoo`8000=o
o`8000Yoo`8000=oo`8000=oo`<000=oo`8000Yoo`8000=oo`8000eoo`8004Eoo`8000=oo`8000Uo
o`<000=oo`8001Yoo`8005Eoo`<000=oo`8005Qoo`00?Woo0P001Goo0P0027oo0P001Goo0P001Woo
0P00>Goo0P001Goo0P0027oo0P001Goo0P000Woo0P001Goo0P0027oo0P001Goo0P0037oo0P00A7oo
0P001Goo0P0027oo0P001Goo0P006Goo0P00EGoo0P001Goo0P00Egoo000nOol20005Ool20008Ool2
0005Ool20006Ool2000iOol20005Ool20008Ool20005Ool20009Ool20008Ool20005Ool2000<Ool2
0014Ool20005Ool2000?Ool2000IOol2001LOol2001GOol003ioo`8000Eoo`8000Qoo`8000Eoo`80
00Ioo`8003Uoo`8000Eoo`8000Qoo`8000Eoo`8000Uoo`8000Qoo`8000Eoo`8000aoo`8004Aoo`80
00Eoo`8000moo`8001Uoo`8005aoo`8005Moo`00?Woo0P001Goo0P0027oo0P001Goo0P001Woo0P00
>Goo0P001Goo0P0027oo0P001Goo0P002Goo0P0027oo0P001Goo0P0037oo0P00A7oo0P001Goo0P00
3goo0P006Goo0P00G7oo0P00Egoo000nOol20005Ool20008Ool20005Ool20006Ool2000iOol20005
Ool20008Ool20005Ool20002Ool20004Ool30008Ool20005Ool2000<Ool20014Ool20005Ool20008
Ool20004Ool3000IOol2001EOol20004Ool3001GOol003ioo`8000Eoo`8000Qoo`8000Eoo`8000Io
o`8003Uoo`8000Eoo`8000Qoo`8000Eoo`80009oo`P000Uoo`8000Eoo`8000aoo`8004Aoo`8000Eo
o`8000Qoo`P001Yoo`8005Eoo`P005Qoo`00?Woo0P001Goo0P0027oo0P001Goo0P001Woo0P00>Goo
0P001Goo0P0027oo0P001Goo0P000Woo0P0000=oo`0000000P002Woo0P001Goo0P0037oo0P00A7oo
0P001Goo0P0027oo0P0000=oo`0000000P006goo0P00EGoo0P0000=oo`0000000P00FGoo000nOol2
0005Ool20008Ool20005Ool20003Ool200000goo0000000iOol20005Ool20008Ool20005Ool20002
Ool2000?Ool20005Ool20009Ool200000goo00000014Ool20005Ool20008Ool2000MOol200000goo
0000001EOol2001NOol003ioo`8000Eoo`8000Qoo`8000Eoo`8000=oo`D003Uoo`8000Eoo`8000Qo
o`8000Eoo`8000=oo`8000ioo`8000Eoo`8000Uoo`D004Aoo`8000Eoo`8000Uoo`8001aoo`D005Io
o`8005eoo`00?goo0P000goo0P002Woo0P000goo0P001Woo0`00>Woo0P000goo0P002Woo0P000goo
0P0017oo0P003goo0P000goo0P0037oo0`00AGoo0P000goo0P002Woo0P007Woo0`00EWoo0P00GGoo
000oOol7000:Ool70007Ool2000jOol7000:Ool70004Ool7000:Ool7000=Ool20015Ool7000:Ool7
000JOol2001FOol7001HOol0041oo`D000aoo`D000Qoo`8003]oo`D000aoo`D000Eoo`L000]oo`D0
00ioo`8004Ioo`D000]oo`L001Yoo`8005Ioo`L005Qoo`00ogooogoo3Goo003oOoooOol=Ool00?mo
oomoo`eoo`00ogooogoo3Goo003oOoooOol=Ool00?mooomoo`eoo`00ogooogoo3Goo003oOoooOol=
Ool00?mooomoo`eoo`00ogooogoo3Goo003oOoooOol=Ool00?mooomoo`eoo`00ogooogoo3Goo003o
OoooOol=Ool00?mooomoo`eoo`00ogooogoo3Goo003oOoooOol=Ool00?mooomoo`eoo`00ogooogoo
3Goo0000\
\>"],
  ImageRangeCache->{{{0, 522.938}, {322.688, 0}} -> {-2.62091, -1.58022, 
  0.0115337, 0.0355028}}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Example 2: Investigating an earthquake", "Subsection"],

Cell["\<\
There was an magnitude 7 earthquake in Turkey on November 11, 1999, 16:57:20 \
UTC. The one-minute trend data of the seismometers in the LVEA and the mid \
stations can be retrived with\
\>", "Text"],

Cell[BoxData[{
    \(\(t0 = {1999, 11, 12, 16, 57, 0};\)\), "\n", 
    \(\(trend = 
        NDSGetTrend[{"\<H0:PEM-LVEA_SEISX\>", "\<H0:PEM-LVEA_SEISY\>", \
"\<H0:PEM-LVEA_SEISZ\>", "\<H0:PEM-MX_SEISX\>", "\<H0:PEM-MX_SEISY\>", \
"\<H0:PEM-MX_SEISZ\>", "\<H0:PEM-MY_SEISX\>", "\<H0:PEM-MY_SEISY\>", \
"\<H0:PEM-MY_SEISZ\>"}, \ t0, 10000, DataRate \[Rule] 1. /60. , 
          SimpleFormat \[Rule] False];\)\), "\n", 
    \(\(trend = NDSTrendEliminateInvalid[trend];\)\)}], "Input"],

Cell["\<\
where invalid points have been eliminated. Setting up some plot options first\
\
\>", "Text"],

Cell[BoxData[
    \(\(Needs["\<Graphics`MultipleListPlot`\>"]; \)\)], "Input"],

Cell[BoxData[
    \($TextStyle = {FontFamily \[Rule] "\<Helvetica\>", FontSize \[Rule] 13}; 
    plotopt = 
      Sequence[Frame \[Rule] True, GridLines \[Rule] Automatic, 
        PlotJoined \[Rule] True, 
        PlotStyle 
          \[Rule] {{Thickness[0.005], RGBColor[1, 0, 0]}, {Thickness[0.005], 
              RGBColor[0, 0, 1]}, {Thickness[0.005], RGBColor[0, 0, 1]}}, 
        SymbolStyle \[Rule] {RGBColor[1, 0, 0]}, \n
        SymbolShape \[Rule] {PlotSymbol[Star], None, None}]; \)], "Input"],

Cell["\<\
one can plot the time series around the time of the earthquake. The red line \
represents the mean readout of the seismomter, the blue lines indicate the \
minimum and maximum values at each minute, and the error bars represent the \
standard deviation of the signal.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(trace = 1; \n
    MultipleListPlot[
      \({{#\[LeftDoubleBracket]1\[RightDoubleBracket], #\[LeftDoubleBracket]3
                \[RightDoubleBracket]}, 
            ErrorBar[#\[LeftDoubleBracket]4\[RightDoubleBracket]]}&\)/@
        \((trend\[LeftDoubleBracket]trace\[RightDoubleBracket])\), 
      \({#\[LeftDoubleBracket]1\[RightDoubleBracket], #\[LeftDoubleBracket]5
              \[RightDoubleBracket]}&\)/@
        \((trend\[LeftDoubleBracket]trace\[RightDoubleBracket])\), 
      \({#\[LeftDoubleBracket]1\[RightDoubleBracket], #\[LeftDoubleBracket]6
              \[RightDoubleBracket]}&\)/@
        \((trend\[LeftDoubleBracket]trace\[RightDoubleBracket])\), 
      PlotRange \[Rule] {\(-1500\), \(-100\)}, plotopt]; \)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 9.62001e-005 0.662179 0.000441453 [
[.02381 -0.0125 -5.59375 -16.375 ]
[.02381 -0.0125 5.59375 0 ]
[.21621 -0.0125 -16.375 -16.375 ]
[.21621 -0.0125 16.375 0 ]
[.40861 -0.0125 -16.375 -16.375 ]
[.40861 -0.0125 16.375 0 ]
[.60101 -0.0125 -16.375 -16.375 ]
[.60101 -0.0125 16.375 0 ]
[.79341 -0.0125 -16.375 -16.375 ]
[.79341 -0.0125 16.375 0 ]
[.98581 -0.0125 -19.9688 -16.375 ]
[.98581 -0.0125 19.9688 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .04415 -41.75 -8.1875 ]
[-0.0125 .04415 0 8.1875 ]
[-0.0125 .13244 -41.75 -8.1875 ]
[-0.0125 .13244 0 8.1875 ]
[-0.0125 .22073 -41.75 -8.1875 ]
[-0.0125 .22073 0 8.1875 ]
[-0.0125 .30902 -34.5625 -8.1875 ]
[-0.0125 .30902 0 8.1875 ]
[-0.0125 .39731 -34.5625 -8.1875 ]
[-0.0125 .39731 0 8.1875 ]
[-0.0125 .4856 -34.5625 -8.1875 ]
[-0.0125 .4856 0 8.1875 ]
[-0.0125 .57389 -34.5625 -8.1875 ]
[-0.0125 .57389 0 8.1875 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
.21621 0 m
.21621 .61803 L
s
.40861 0 m
.40861 .61803 L
s
.60101 0 m
.60101 .61803 L
s
.79341 0 m
.79341 .61803 L
s
0 .04415 m
1 .04415 L
s
0 .13244 m
1 .13244 L
s
0 .22073 m
1 .22073 L
s
0 .30902 m
1 .30902 L
s
0 .39731 m
1 .39731 L
s
0 .4856 m
1 .4856 L
s
0 .57389 m
1 .57389 L
s
0 g
.02381 0 m
.02381 .00625 L
s
gsave
.02381 -0.0125 -66.5938 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.21621 0 m
.21621 .00625 L
s
gsave
.21621 -0.0125 -77.375 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(2000) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.40861 0 m
.40861 .00625 L
s
gsave
.40861 -0.0125 -77.375 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(4000) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.60101 0 m
.60101 .00625 L
s
gsave
.60101 -0.0125 -77.375 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(6000) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.79341 0 m
.79341 .00625 L
s
gsave
.79341 -0.0125 -77.375 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(8000) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.98581 0 m
.98581 .00625 L
s
gsave
.98581 -0.0125 -80.9688 -20.375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(10000) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Helvetica
grestore
.125 Mabswid
.07191 0 m
.07191 .00375 L
s
.12001 0 m
.12001 .00375 L
s
.16811 0 m
.16811 .00375 L
s
.26431 0 m
.26431 .00375 L
s
.31241 0 m
.31241 .00375 L
s
.36051 0 m
.36051 .00375 L
s
.45671 0 m
.45671 .00375 L
s
.50481 0 m
.50481 .00375 L
s
.55291 0 m
.55291 .00375 L
s
.64911 0 m
.64911 .00375 L
s
.69721 0 m
.69721 .00375 L
s
.74531 0 m
.74531 .00375 L
s
.84151 0 m
.84151 .00375 L
s
.88961 0 m
.88961 .00375 L
s
.93771 0 m
.93771 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .04415 m
.00625 .04415 L
s
gsave
-0.0125 .04415 -102.75 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
72.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(1400) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1
%%+ font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1
%%+ Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Math1
%%+ font Helvetica
grestore
0 .13244 m
.00625 .13244 L
s
gsave
-0.0125 .13244 -102.75 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
72.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(1200) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1
%%+ font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1
%%+ Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Math1
%%+ font Helvetica
grestore
0 .22073 m
.00625 .22073 L
s
gsave
-0.0125 .22073 -102.75 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
72.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(1000) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1
%%+ font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1
%%+ Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Math1
%%+ font Helvetica
grestore
0 .30902 m
.00625 .30902 L
s
gsave
-0.0125 .30902 -95.5625 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
72.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(800) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1
%%+ font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1
%%+ Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Math1
%%+ font Helvetica
grestore
0 .39731 m
.00625 .39731 L
s
gsave
-0.0125 .39731 -95.5625 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
72.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(600) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1
%%+ font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1
%%+ Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Math1
%%+ font Helvetica
grestore
0 .4856 m
.00625 .4856 L
s
gsave
-0.0125 .4856 -95.5625 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
72.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(400) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1
%%+ font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1
%%+ Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Math1
%%+ font Helvetica
grestore
0 .57389 m
.00625 .57389 L
s
gsave
-0.0125 .57389 -95.5625 -12.1875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 24.375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 20.375000 moveto
 464.000000 20.375000 lineto
 464.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 20.375000 lineto
closepath
clip newpath
63.000000 16.625000 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(-) show
72.000000 16.625000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 13.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(200) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1
%%+ font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1
%%+ Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: font Math1
%%+ font Helvetica
grestore
.125 Mabswid
0 .06622 m
.00375 .06622 L
s
0 .08829 m
.00375 .08829 L
s
0 .11036 m
.00375 .11036 L
s
0 .15451 m
.00375 .15451 L
s
0 .17658 m
.00375 .17658 L
s
0 .19865 m
.00375 .19865 L
s
0 .2428 m
.00375 .2428 L
s
0 .26487 m
.00375 .26487 L
s
0 .28694 m
.00375 .28694 L
s
0 .33109 m
.00375 .33109 L
s
0 .35316 m
.00375 .35316 L
s
0 .37523 m
.00375 .37523 L
s
0 .41938 m
.00375 .41938 L
s
0 .44145 m
.00375 .44145 L
s
0 .46353 m
.00375 .46353 L
s
0 .50767 m
.00375 .50767 L
s
0 .52974 m
.00375 .52974 L
s
0 .55182 m
.00375 .55182 L
s
0 .02207 m
.00375 .02207 L
s
0 .59596 m
.00375 .59596 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.02381 .61178 m
.02381 .61803 L
s
.21621 .61178 m
.21621 .61803 L
s
.40861 .61178 m
.40861 .61803 L
s
.60101 .61178 m
.60101 .61803 L
s
.79341 .61178 m
.79341 .61803 L
s
.98581 .61178 m
.98581 .61803 L
s
.125 Mabswid
.07191 .61428 m
.07191 .61803 L
s
.12001 .61428 m
.12001 .61803 L
s
.16811 .61428 m
.16811 .61803 L
s
.26431 .61428 m
.26431 .61803 L
s
.31241 .61428 m
.31241 .61803 L
s
.36051 .61428 m
.36051 .61803 L
s
.45671 .61428 m
.45671 .61803 L
s
.50481 .61428 m
.50481 .61803 L
s
.55291 .61428 m
.55291 .61803 L
s
.64911 .61428 m
.64911 .61803 L
s
.69721 .61428 m
.69721 .61803 L
s
.74531 .61428 m
.74531 .61803 L
s
.84151 .61428 m
.84151 .61803 L
s
.88961 .61428 m
.88961 .61803 L
s
.93771 .61428 m
.93771 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .04415 m
1 .04415 L
s
.99375 .13244 m
1 .13244 L
s
.99375 .22073 m
1 .22073 L
s
.99375 .30902 m
1 .30902 L
s
.99375 .39731 m
1 .39731 L
s
.99375 .4856 m
1 .4856 L
s
.99375 .57389 m
1 .57389 L
s
.125 Mabswid
.99625 .06622 m
1 .06622 L
s
.99625 .08829 m
1 .08829 L
s
.99625 .11036 m
1 .11036 L
s
.99625 .15451 m
1 .15451 L
s
.99625 .17658 m
1 .17658 L
s
.99625 .19865 m
1 .19865 L
s
.99625 .2428 m
1 .2428 L
s
.99625 .26487 m
1 .26487 L
s
.99625 .28694 m
1 .28694 L
s
.99625 .33109 m
1 .33109 L
s
.99625 .35316 m
1 .35316 L
s
.99625 .37523 m
1 .37523 L
s
.99625 .41938 m
1 .41938 L
s
.99625 .44145 m
1 .44145 L
s
.99625 .46353 m
1 .46353 L
s
.99625 .50767 m
1 .50767 L
s
.99625 .52974 m
1 .52974 L
s
.99625 .55182 m
1 .55182 L
s
.99625 .02207 m
1 .02207 L
s
.99625 .59596 m
1 .59596 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.005 w
.02381 .28275 m
.02958 .28245 L
.03535 .28236 L
.04113 .28233 L
.0469 .28228 L
.05267 .28248 L
.05844 .28171 L
.06421 .28232 L
.06999 .28224 L
.07576 .28228 L
.08153 .28137 L
.0873 .28188 L
.09307 .28209 L
.09885 .28155 L
.10462 .28238 L
.11039 .28188 L
.11616 .28231 L
.12193 .28133 L
.12771 .28156 L
.13348 .28161 L
.13925 .28216 L
.14502 .28198 L
.15079 .28194 L
.15657 .27977 L
.16234 .284 L
.16811 .28169 L
.17388 .27939 L
.17965 .28316 L
.18543 .28358 L
.1912 .28421 L
.19697 .27734 L
.20274 .283 L
.20851 .28117 L
.21429 .28357 L
.22006 .28111 L
.22583 .27354 L
.2316 .29272 L
.23737 .28216 L
.24315 .27683 L
.24892 .28281 L
.25469 .28788 L
.26046 .2706 L
.26623 .30086 L
.27201 .26098 L
.27778 .29756 L
.28355 .27708 L
.28932 .28128 L
.29509 .27469 L
.30087 .28613 L
.30664 .27932 L
Mistroke
.31241 .28137 L
.31818 .28616 L
.32395 .28087 L
.32973 .27756 L
.3355 .27678 L
.34127 .28562 L
.34704 .28189 L
.35281 .27658 L
.35859 .28756 L
.36436 .27631 L
.37013 .28936 L
.3759 .27996 L
.38167 .27972 L
.38745 .27819 L
.39322 .27991 L
.39899 .28094 L
.40476 .28166 L
.41053 .28335 L
.41631 .28058 L
.42208 .27737 L
.42785 .28514 L
.43362 .2808 L
.43939 .28206 L
.44517 .28208 L
.45094 .27958 L
.45671 .28262 L
.46248 .28016 L
.46825 .27808 L
.47403 .28339 L
.4798 .28106 L
.48557 .27952 L
.49134 .28201 L
.49711 .28153 L
.50289 .28156 L
.50866 .28001 L
.51443 .28105 L
.5202 .2813 L
.52597 .28105 L
.53175 .2826 L
.53752 .2792 L
.54329 .28046 L
.54906 .28221 L
.55483 .28004 L
.56061 .28131 L
.56638 .28047 L
.57215 .28162 L
.57792 .28082 L
.58369 .2814 L
.58947 .28084 L
.59524 .2804 L
Mistroke
.60101 .28026 L
.60678 .28226 L
.61255 .28095 L
.61833 .28137 L
.6241 .27973 L
.62987 .28029 L
.63564 .28106 L
.64141 .28028 L
.64719 .28055 L
.65296 .28059 L
.65873 .28089 L
.6645 .28038 L
.67027 .27995 L
.67605 .28261 L
.68182 .27881 L
.68759 .28185 L
.69336 .28042 L
.69913 .28133 L
.70491 .28021 L
.71068 .28075 L
.71645 .28099 L
.72222 .28064 L
.72799 .28058 L
.74531 .28055 L
.75108 .28083 L
.75685 .28032 L
.76263 .28059 L
.7684 .28011 L
.77417 .27661 L
.77994 .28394 L
.78571 .28094 L
.79149 .28104 L
.79726 .28057 L
.80303 .28171 L
.8088 .27984 L
.81457 .28134 L
.82035 .27982 L
.82612 .28079 L
.83189 .28082 L
.83766 .28085 L
.84343 .27985 L
.84921 .28088 L
.85498 .28081 L
.86075 .28026 L
.86652 .2808 L
.87229 .28013 L
.87807 .28081 L
.88384 .2808 L
.88961 .28044 L
.89538 .28027 L
Mistroke
.90115 .2809 L
.90693 .28007 L
.9127 .28092 L
.91847 .28006 L
.92424 .28026 L
.93001 .28098 L
.93579 .28087 L
.94156 .27996 L
.94733 .28099 L
.9531 .28095 L
.95887 .27933 L
.96465 .27985 L
.97042 .28171 L
.97619 .28077 L
Mfstroke
0 g
.001 w
.02381 .29117 m
.02381 .27433 L
s
.02381 .29117 1.5 0 Mabsadd m
.02381 .29117 -1.5 0 Mabsadd L s
.02381 .27433 1.5 0 Mabsadd m
.02381 .27433 -1.5 0 Mabsadd L s
.02958 .29071 m
.02958 .2742 L
s
.02958 .29071 1.5 0 Mabsadd m
.02958 .29071 -1.5 0 Mabsadd L s
.02958 .2742 1.5 0 Mabsadd m
.02958 .2742 -1.5 0 Mabsadd L s
.03535 .29053 m
.03535 .27419 L
s
.03535 .29053 1.5 0 Mabsadd m
.03535 .29053 -1.5 0 Mabsadd L s
.03535 .27419 1.5 0 Mabsadd m
.03535 .27419 -1.5 0 Mabsadd L s
.04113 .29462 m
.04113 .27004 L
s
.04113 .29462 1.5 0 Mabsadd m
.04113 .29462 -1.5 0 Mabsadd L s
.04113 .27004 1.5 0 Mabsadd m
.04113 .27004 -1.5 0 Mabsadd L s
.0469 .2904 m
.0469 .27416 L
s
.0469 .2904 1.5 0 Mabsadd m
.0469 .2904 -1.5 0 Mabsadd L s
.0469 .27416 1.5 0 Mabsadd m
.0469 .27416 -1.5 0 Mabsadd L s
.05267 .29052 m
.05267 .27445 L
s
.05267 .29052 1.5 0 Mabsadd m
.05267 .29052 -1.5 0 Mabsadd L s
.05267 .27445 1.5 0 Mabsadd m
.05267 .27445 -1.5 0 Mabsadd L s
.05844 .28961 m
.05844 .27381 L
s
.05844 .28961 1.5 0 Mabsadd m
.05844 .28961 -1.5 0 Mabsadd L s
.05844 .27381 1.5 0 Mabsadd m
.05844 .27381 -1.5 0 Mabsadd L s
.06421 .28988 m
.06421 .27477 L
s
.06421 .28988 1.5 0 Mabsadd m
.06421 .28988 -1.5 0 Mabsadd L s
.06421 .27477 1.5 0 Mabsadd m
.06421 .27477 -1.5 0 Mabsadd L s
.06999 .29009 m
.06999 .27438 L
s
.06999 .29009 1.5 0 Mabsadd m
.06999 .29009 -1.5 0 Mabsadd L s
.06999 .27438 1.5 0 Mabsadd m
.06999 .27438 -1.5 0 Mabsadd L s
.07576 .29058 m
.07576 .27398 L
s
.07576 .29058 1.5 0 Mabsadd m
.07576 .29058 -1.5 0 Mabsadd L s
.07576 .27398 1.5 0 Mabsadd m
.07576 .27398 -1.5 0 Mabsadd L s
.08153 .28942 m
.08153 .27332 L
s
.08153 .28942 1.5 0 Mabsadd m
.08153 .28942 -1.5 0 Mabsadd L s
.08153 .27332 1.5 0 Mabsadd m
.08153 .27332 -1.5 0 Mabsadd L s
.0873 .28994 m
.0873 .27383 L
s
.0873 .28994 1.5 0 Mabsadd m
.0873 .28994 -1.5 0 Mabsadd L s
.0873 .27383 1.5 0 Mabsadd m
.0873 .27383 -1.5 0 Mabsadd L s
.09307 .2907 m
.09307 .27347 L
s
.09307 .2907 1.5 0 Mabsadd m
.09307 .2907 -1.5 0 Mabsadd L s
.09307 .27347 1.5 0 Mabsadd m
.09307 .27347 -1.5 0 Mabsadd L s
.09885 .28983 m
.09885 .27327 L
s
.09885 .28983 1.5 0 Mabsadd m
.09885 .28983 -1.5 0 Mabsadd L s
.09885 .27327 1.5 0 Mabsadd m
.09885 .27327 -1.5 0 Mabsadd L s
.10462 .29097 m
.10462 .2738 L
s
.10462 .29097 1.5 0 Mabsadd m
.10462 .29097 -1.5 0 Mabsadd L s
.10462 .2738 1.5 0 Mabsadd m
.10462 .2738 -1.5 0 Mabsadd L s
.11039 .2904 m
.11039 .27337 L
s
.11039 .2904 1.5 0 Mabsadd m
.11039 .2904 -1.5 0 Mabsadd L s
.11039 .27337 1.5 0 Mabsadd m
.11039 .27337 -1.5 0 Mabsadd L s
.11616 .29082 m
.11616 .2738 L
s
.11616 .29082 1.5 0 Mabsadd m
.11616 .29082 -1.5 0 Mabsadd L s
.11616 .2738 1.5 0 Mabsadd m
.11616 .2738 -1.5 0 Mabsadd L s
.12193 .28953 m
.12193 .27313 L
s
.12193 .28953 1.5 0 Mabsadd m
.12193 .28953 -1.5 0 Mabsadd L s
.12193 .27313 1.5 0 Mabsadd m
.12193 .27313 -1.5 0 Mabsadd L s
.12771 .28995 m
.12771 .27317 L
s
.12771 .28995 1.5 0 Mabsadd m
.12771 .28995 -1.5 0 Mabsadd L s
.12771 .27317 1.5 0 Mabsadd m
.12771 .27317 -1.5 0 Mabsadd L s
.13348 .28972 m
.13348 .2735 L
s
.13348 .28972 1.5 0 Mabsadd m
.13348 .28972 -1.5 0 Mabsadd L s
.13348 .2735 1.5 0 Mabsadd m
.13348 .2735 -1.5 0 Mabsadd L s
.13925 .29011 m
.13925 .27421 L
s
.13925 .29011 1.5 0 Mabsadd m
.13925 .29011 -1.5 0 Mabsadd L s
.13925 .27421 1.5 0 Mabsadd m
.13925 .27421 -1.5 0 Mabsadd L s
.14502 .29017 m
.14502 .27379 L
s
.14502 .29017 1.5 0 Mabsadd m
.14502 .29017 -1.5 0 Mabsadd L s
.14502 .27379 1.5 0 Mabsadd m
.14502 .27379 -1.5 0 Mabsadd L s
.15079 .29022 m
.15079 .27366 L
s
.15079 .29022 1.5 0 Mabsadd m
.15079 .29022 -1.5 0 Mabsadd L s
.15079 .27366 1.5 0 Mabsadd m
.15079 .27366 -1.5 0 Mabsadd L s
.15657 .3592 m
.15657 .20034 L
s
.15657 .3592 1.5 0 Mabsadd m
.15657 .3592 -1.5 0 Mabsadd L s
.15657 .20034 1.5 0 Mabsadd m
.15657 .20034 -1.5 0 Mabsadd L s
.16234 .32061 m
.16234 .24739 L
s
.16234 .32061 1.5 0 Mabsadd m
.16234 .32061 -1.5 0 Mabsadd L s
.16234 .24739 1.5 0 Mabsadd m
.16234 .24739 -1.5 0 Mabsadd L s
.16811 .30226 m
.16811 .26112 L
s
.16811 .30226 1.5 0 Mabsadd m
.16811 .30226 -1.5 0 Mabsadd L s
.16811 .26112 1.5 0 Mabsadd m
.16811 .26112 -1.5 0 Mabsadd L s
.17388 .30243 m
.17388 .25634 L
s
.17388 .30243 1.5 0 Mabsadd m
.17388 .30243 -1.5 0 Mabsadd L s
.17388 .25634 1.5 0 Mabsadd m
.17388 .25634 -1.5 0 Mabsadd L s
.17965 .30204 m
.17965 .26428 L
s
.17965 .30204 1.5 0 Mabsadd m
.17965 .30204 -1.5 0 Mabsadd L s
.17965 .26428 1.5 0 Mabsadd m
.17965 .26428 -1.5 0 Mabsadd L s
.18543 .30468 m
.18543 .26248 L
s
.18543 .30468 1.5 0 Mabsadd m
.18543 .30468 -1.5 0 Mabsadd L s
.18543 .26248 1.5 0 Mabsadd m
.18543 .26248 -1.5 0 Mabsadd L s
.1912 .31309 m
.1912 .25533 L
s
.1912 .31309 1.5 0 Mabsadd m
.1912 .31309 -1.5 0 Mabsadd L s
.1912 .25533 1.5 0 Mabsadd m
.1912 .25533 -1.5 0 Mabsadd L s
.19697 .29922 m
.19697 .25547 L
s
.19697 .29922 1.5 0 Mabsadd m
.19697 .29922 -1.5 0 Mabsadd L s
.19697 .25547 1.5 0 Mabsadd m
.19697 .25547 -1.5 0 Mabsadd L s
.20274 .29987 m
.20274 .26613 L
s
.20274 .29987 1.5 0 Mabsadd m
.20274 .29987 -1.5 0 Mabsadd L s
.20274 .26613 1.5 0 Mabsadd m
.20274 .26613 -1.5 0 Mabsadd L s
.20851 .29805 m
.20851 .26429 L
s
.20851 .29805 1.5 0 Mabsadd m
.20851 .29805 -1.5 0 Mabsadd L s
.20851 .26429 1.5 0 Mabsadd m
.20851 .26429 -1.5 0 Mabsadd L s
.21429 .30803 m
.21429 .25912 L
s
.21429 .30803 1.5 0 Mabsadd m
.21429 .30803 -1.5 0 Mabsadd L s
.21429 .25912 1.5 0 Mabsadd m
.21429 .25912 -1.5 0 Mabsadd L s
.22006 .29765 m
.22006 .26458 L
s
.22006 .29765 1.5 0 Mabsadd m
.22006 .29765 -1.5 0 Mabsadd L s
.22006 .26458 1.5 0 Mabsadd m
.22006 .26458 -1.5 0 Mabsadd L s
.22583 .30656 m
.22583 .24052 L
s
.22583 .30656 1.5 0 Mabsadd m
.22583 .30656 -1.5 0 Mabsadd L s
.22583 .24052 1.5 0 Mabsadd m
.22583 .24052 -1.5 0 Mabsadd L s
.2316 .33261 m
.2316 .25282 L
s
.2316 .33261 1.5 0 Mabsadd m
.2316 .33261 -1.5 0 Mabsadd L s
.2316 .25282 1.5 0 Mabsadd m
.2316 .25282 -1.5 0 Mabsadd L s
.23737 .31984 m
.23737 .24448 L
s
.23737 .31984 1.5 0 Mabsadd m
.23737 .31984 -1.5 0 Mabsadd L s
.23737 .24448 1.5 0 Mabsadd m
.23737 .24448 -1.5 0 Mabsadd L s
.24315 .31592 m
.24315 .23774 L
s
.24315 .31592 1.5 0 Mabsadd m
.24315 .31592 -1.5 0 Mabsadd L s
.24315 .23774 1.5 0 Mabsadd m
.24315 .23774 -1.5 0 Mabsadd L s
.24892 .30936 m
.24892 .25627 L
s
.24892 .30936 1.5 0 Mabsadd m
.24892 .30936 -1.5 0 Mabsadd L s
.24892 .25627 1.5 0 Mabsadd m
.24892 .25627 -1.5 0 Mabsadd L s
.25469 .35615 m
.25469 .21962 L
s
.25469 .35615 1.5 0 Mabsadd m
.25469 .35615 -1.5 0 Mabsadd L s
.25469 .21962 1.5 0 Mabsadd m
.25469 .21962 -1.5 0 Mabsadd L s
.26046 .34009 m
.26046 .20111 L
s
.26046 .34009 1.5 0 Mabsadd m
.26046 .34009 -1.5 0 Mabsadd L s
.26046 .20111 1.5 0 Mabsadd m
.26046 .20111 -1.5 0 Mabsadd L s
.26623 .37464 m
.26623 .22709 L
s
.26623 .37464 1.5 0 Mabsadd m
.26623 .37464 -1.5 0 Mabsadd L s
.26623 .22709 1.5 0 Mabsadd m
.26623 .22709 -1.5 0 Mabsadd L s
.27201 .33599 m
.27201 .18598 L
s
.27201 .33599 1.5 0 Mabsadd m
.27201 .33599 -1.5 0 Mabsadd L s
.27201 .18598 1.5 0 Mabsadd m
.27201 .18598 -1.5 0 Mabsadd L s
.27778 .37884 m
.27778 .21629 L
s
.27778 .37884 1.5 0 Mabsadd m
.27778 .37884 -1.5 0 Mabsadd L s
.27778 .21629 1.5 0 Mabsadd m
.27778 .21629 -1.5 0 Mabsadd L s
.28355 .424 m
.28355 .13015 L
s
.28355 .424 1.5 0 Mabsadd m
.28355 .424 -1.5 0 Mabsadd L s
.28355 .13015 1.5 0 Mabsadd m
.28355 .13015 -1.5 0 Mabsadd L s
.28932 .37147 m
.28932 .19109 L
s
.28932 .37147 1.5 0 Mabsadd m
.28932 .37147 -1.5 0 Mabsadd L s
.28932 .19109 1.5 0 Mabsadd m
.28932 .19109 -1.5 0 Mabsadd L s
.29509 .34456 m
.29509 .20483 L
s
.29509 .34456 1.5 0 Mabsadd m
.29509 .34456 -1.5 0 Mabsadd L s
.29509 .20483 1.5 0 Mabsadd m
.29509 .20483 -1.5 0 Mabsadd L s
.30087 .34505 m
.30087 .22721 L
s
.30087 .34505 1.5 0 Mabsadd m
.30087 .34505 -1.5 0 Mabsadd L s
.30087 .22721 1.5 0 Mabsadd m
.30087 .22721 -1.5 0 Mabsadd L s
.30664 .32645 m
.30664 .2322 L
s
.30664 .32645 1.5 0 Mabsadd m
.30664 .32645 -1.5 0 Mabsadd L s
.30664 .2322 1.5 0 Mabsadd m
.30664 .2322 -1.5 0 Mabsadd L s
.31241 .32838 m
.31241 .23436 L
s
.31241 .32838 1.5 0 Mabsadd m
.31241 .32838 -1.5 0 Mabsadd L s
.31241 .23436 1.5 0 Mabsadd m
.31241 .23436 -1.5 0 Mabsadd L s
.31818 .33392 m
.31818 .23841 L
s
.31818 .33392 1.5 0 Mabsadd m
.31818 .33392 -1.5 0 Mabsadd L s
.31818 .23841 1.5 0 Mabsadd m
.31818 .23841 -1.5 0 Mabsadd L s
.32395 .36454 m
.32395 .1972 L
s
.32395 .36454 1.5 0 Mabsadd m
.32395 .36454 -1.5 0 Mabsadd L s
.32395 .1972 1.5 0 Mabsadd m
.32395 .1972 -1.5 0 Mabsadd L s
.32973 .34781 m
.32973 .20732 L
s
.32973 .34781 1.5 0 Mabsadd m
.32973 .34781 -1.5 0 Mabsadd L s
.32973 .20732 1.5 0 Mabsadd m
.32973 .20732 -1.5 0 Mabsadd L s
.3355 .37718 m
.3355 .17639 L
s
.3355 .37718 1.5 0 Mabsadd m
.3355 .37718 -1.5 0 Mabsadd L s
.3355 .17639 1.5 0 Mabsadd m
.3355 .17639 -1.5 0 Mabsadd L s
.34127 .4083 m
.34127 .16295 L
s
.34127 .4083 1.5 0 Mabsadd m
.34127 .4083 -1.5 0 Mabsadd L s
.34127 .16295 1.5 0 Mabsadd m
.34127 .16295 -1.5 0 Mabsadd L s
.34704 .34249 m
.34704 .22129 L
s
.34704 .34249 1.5 0 Mabsadd m
.34704 .34249 -1.5 0 Mabsadd L s
.34704 .22129 1.5 0 Mabsadd m
.34704 .22129 -1.5 0 Mabsadd L s
.35281 .3555 m
.35281 .19766 L
s
.35281 .3555 1.5 0 Mabsadd m
.35281 .3555 -1.5 0 Mabsadd L s
.35281 .19766 1.5 0 Mabsadd m
.35281 .19766 -1.5 0 Mabsadd L s
.35859 .34872 m
.35859 .22641 L
s
.35859 .34872 1.5 0 Mabsadd m
.35859 .34872 -1.5 0 Mabsadd L s
.35859 .22641 1.5 0 Mabsadd m
.35859 .22641 -1.5 0 Mabsadd L s
.36436 .34365 m
.36436 .20896 L
s
.36436 .34365 1.5 0 Mabsadd m
.36436 .34365 -1.5 0 Mabsadd L s
.36436 .20896 1.5 0 Mabsadd m
.36436 .20896 -1.5 0 Mabsadd L s
.37013 .36072 m
.37013 .218 L
s
.37013 .36072 1.5 0 Mabsadd m
.37013 .36072 -1.5 0 Mabsadd L s
.37013 .218 1.5 0 Mabsadd m
.37013 .218 -1.5 0 Mabsadd L s
.3759 .3179 m
.3759 .24203 L
s
.3759 .3179 1.5 0 Mabsadd m
.3759 .3179 -1.5 0 Mabsadd L s
.3759 .24203 1.5 0 Mabsadd m
.3759 .24203 -1.5 0 Mabsadd L s
.38167 .32088 m
.38167 .23855 L
s
.38167 .32088 1.5 0 Mabsadd m
.38167 .32088 -1.5 0 Mabsadd L s
.38167 .23855 1.5 0 Mabsadd m
.38167 .23855 -1.5 0 Mabsadd L s
.38745 .34095 m
.38745 .21543 L
s
.38745 .34095 1.5 0 Mabsadd m
.38745 .34095 -1.5 0 Mabsadd L s
.38745 .21543 1.5 0 Mabsadd m
.38745 .21543 -1.5 0 Mabsadd L s
.39322 .30642 m
.39322 .2534 L
s
.39322 .30642 1.5 0 Mabsadd m
.39322 .30642 -1.5 0 Mabsadd L s
.39322 .2534 1.5 0 Mabsadd m
.39322 .2534 -1.5 0 Mabsadd L s
.39899 .33059 m
.39899 .23129 L
s
.39899 .33059 1.5 0 Mabsadd m
.39899 .33059 -1.5 0 Mabsadd L s
.39899 .23129 1.5 0 Mabsadd m
.39899 .23129 -1.5 0 Mabsadd L s
.40476 .31098 m
.40476 .25234 L
s
.40476 .31098 1.5 0 Mabsadd m
.40476 .31098 -1.5 0 Mabsadd L s
.40476 .25234 1.5 0 Mabsadd m
.40476 .25234 -1.5 0 Mabsadd L s
.41053 .31917 m
.41053 .24752 L
s
.41053 .31917 1.5 0 Mabsadd m
.41053 .31917 -1.5 0 Mabsadd L s
.41053 .24752 1.5 0 Mabsadd m
.41053 .24752 -1.5 0 Mabsadd L s
.41631 .30777 m
.41631 .25339 L
s
.41631 .30777 1.5 0 Mabsadd m
.41631 .30777 -1.5 0 Mabsadd L s
.41631 .25339 1.5 0 Mabsadd m
.41631 .25339 -1.5 0 Mabsadd L s
.42208 .32133 m
.42208 .23342 L
s
.42208 .32133 1.5 0 Mabsadd m
.42208 .32133 -1.5 0 Mabsadd L s
.42208 .23342 1.5 0 Mabsadd m
.42208 .23342 -1.5 0 Mabsadd L s
.42785 .32826 m
.42785 .24202 L
s
.42785 .32826 1.5 0 Mabsadd m
.42785 .32826 -1.5 0 Mabsadd L s
.42785 .24202 1.5 0 Mabsadd m
.42785 .24202 -1.5 0 Mabsadd L s
.43362 .31873 m
.43362 .24287 L
s
.43362 .31873 1.5 0 Mabsadd m
.43362 .31873 -1.5 0 Mabsadd L s
.43362 .24287 1.5 0 Mabsadd m
.43362 .24287 -1.5 0 Mabsadd L s
.43939 .3025 m
.43939 .26162 L
s
.43939 .3025 1.5 0 Mabsadd m
.43939 .3025 -1.5 0 Mabsadd L s
.43939 .26162 1.5 0 Mabsadd m
.43939 .26162 -1.5 0 Mabsadd L s
.44517 .31887 m
.44517 .24529 L
s
.44517 .31887 1.5 0 Mabsadd m
.44517 .31887 -1.5 0 Mabsadd L s
.44517 .24529 1.5 0 Mabsadd m
.44517 .24529 -1.5 0 Mabsadd L s
.45094 .30703 m
.45094 .25212 L
s
.45094 .30703 1.5 0 Mabsadd m
.45094 .30703 -1.5 0 Mabsadd L s
.45094 .25212 1.5 0 Mabsadd m
.45094 .25212 -1.5 0 Mabsadd L s
.45671 .32205 m
.45671 .24319 L
s
.45671 .32205 1.5 0 Mabsadd m
.45671 .32205 -1.5 0 Mabsadd L s
.45671 .24319 1.5 0 Mabsadd m
.45671 .24319 -1.5 0 Mabsadd L s
.46248 .30981 m
.46248 .2505 L
s
.46248 .30981 1.5 0 Mabsadd m
.46248 .30981 -1.5 0 Mabsadd L s
.46248 .2505 1.5 0 Mabsadd m
.46248 .2505 -1.5 0 Mabsadd L s
.46825 .30188 m
.46825 .25427 L
s
.46825 .30188 1.5 0 Mabsadd m
.46825 .30188 -1.5 0 Mabsadd L s
.46825 .25427 1.5 0 Mabsadd m
.46825 .25427 -1.5 0 Mabsadd L s
.47403 .31 m
.47403 .25678 L
s
.47403 .31 1.5 0 Mabsadd m
.47403 .31 -1.5 0 Mabsadd L s
.47403 .25678 1.5 0 Mabsadd m
.47403 .25678 -1.5 0 Mabsadd L s
.4798 .30036 m
.4798 .26175 L
s
.4798 .30036 1.5 0 Mabsadd m
.4798 .30036 -1.5 0 Mabsadd L s
.4798 .26175 1.5 0 Mabsadd m
.4798 .26175 -1.5 0 Mabsadd L s
.48557 .29897 m
.48557 .26006 L
s
.48557 .29897 1.5 0 Mabsadd m
.48557 .29897 -1.5 0 Mabsadd L s
.48557 .26006 1.5 0 Mabsadd m
.48557 .26006 -1.5 0 Mabsadd L s
.49134 .30181 m
.49134 .26222 L
s
.49134 .30181 1.5 0 Mabsadd m
.49134 .30181 -1.5 0 Mabsadd L s
.49134 .26222 1.5 0 Mabsadd m
.49134 .26222 -1.5 0 Mabsadd L s
.49711 .29919 m
.49711 .26387 L
s
.49711 .29919 1.5 0 Mabsadd m
.49711 .29919 -1.5 0 Mabsadd L s
.49711 .26387 1.5 0 Mabsadd m
.49711 .26387 -1.5 0 Mabsadd L s
.50289 .29928 m
.50289 .26384 L
s
.50289 .29928 1.5 0 Mabsadd m
.50289 .29928 -1.5 0 Mabsadd L s
.50289 .26384 1.5 0 Mabsadd m
.50289 .26384 -1.5 0 Mabsadd L s
.50866 .29716 m
.50866 .26287 L
s
.50866 .29716 1.5 0 Mabsadd m
.50866 .29716 -1.5 0 Mabsadd L s
.50866 .26287 1.5 0 Mabsadd m
.50866 .26287 -1.5 0 Mabsadd L s
.51443 .29726 m
.51443 .26484 L
s
.51443 .29726 1.5 0 Mabsadd m
.51443 .29726 -1.5 0 Mabsadd L s
.51443 .26484 1.5 0 Mabsadd m
.51443 .26484 -1.5 0 Mabsadd L s
.5202 .30137 m
.5202 .26123 L
s
.5202 .30137 1.5 0 Mabsadd m
.5202 .30137 -1.5 0 Mabsadd L s
.5202 .26123 1.5 0 Mabsadd m
.5202 .26123 -1.5 0 Mabsadd L s
.52597 .29384 m
.52597 .26825 L
s
.52597 .29384 1.5 0 Mabsadd m
.52597 .29384 -1.5 0 Mabsadd L s
.52597 .26825 1.5 0 Mabsadd m
.52597 .26825 -1.5 0 Mabsadd L s
.53175 .30854 m
.53175 .25666 L
s
.53175 .30854 1.5 0 Mabsadd m
.53175 .30854 -1.5 0 Mabsadd L s
.53175 .25666 1.5 0 Mabsadd m
.53175 .25666 -1.5 0 Mabsadd L s
.53752 .29247 m
.53752 .26593 L
s
.53752 .29247 1.5 0 Mabsadd m
.53752 .29247 -1.5 0 Mabsadd L s
.53752 .26593 1.5 0 Mabsadd m
.53752 .26593 -1.5 0 Mabsadd L s
.54329 .29854 m
.54329 .26239 L
s
.54329 .29854 1.5 0 Mabsadd m
.54329 .29854 -1.5 0 Mabsadd L s
.54329 .26239 1.5 0 Mabsadd m
.54329 .26239 -1.5 0 Mabsadd L s
.54906 .29481 m
.54906 .26962 L
s
.54906 .29481 1.5 0 Mabsadd m
.54906 .29481 -1.5 0 Mabsadd L s
.54906 .26962 1.5 0 Mabsadd m
.54906 .26962 -1.5 0 Mabsadd L s
.55483 .29178 m
.55483 .26831 L
s
.55483 .29178 1.5 0 Mabsadd m
.55483 .29178 -1.5 0 Mabsadd L s
.55483 .26831 1.5 0 Mabsadd m
.55483 .26831 -1.5 0 Mabsadd L s
.56061 .2968 m
.56061 .26581 L
s
.56061 .2968 1.5 0 Mabsadd m
.56061 .2968 -1.5 0 Mabsadd L s
.56061 .26581 1.5 0 Mabsadd m
.56061 .26581 -1.5 0 Mabsadd L s
.56638 .29651 m
.56638 .26444 L
s
.56638 .29651 1.5 0 Mabsadd m
.56638 .29651 -1.5 0 Mabsadd L s
.56638 .26444 1.5 0 Mabsadd m
.56638 .26444 -1.5 0 Mabsadd L s
.57215 .29285 m
.57215 .2704 L
s
.57215 .29285 1.5 0 Mabsadd m
.57215 .29285 -1.5 0 Mabsadd L s
.57215 .2704 1.5 0 Mabsadd m
.57215 .2704 -1.5 0 Mabsadd L s
.57792 .29588 m
.57792 .26576 L
s
.57792 .29588 1.5 0 Mabsadd m
.57792 .29588 -1.5 0 Mabsadd L s
.57792 .26576 1.5 0 Mabsadd m
.57792 .26576 -1.5 0 Mabsadd L s
.58369 .29718 m
.58369 .26562 L
s
.58369 .29718 1.5 0 Mabsadd m
.58369 .29718 -1.5 0 Mabsadd L s
.58369 .26562 1.5 0 Mabsadd m
.58369 .26562 -1.5 0 Mabsadd L s
.58947 .2916 m
.58947 .27007 L
s
.58947 .2916 1.5 0 Mabsadd m
.58947 .2916 -1.5 0 Mabsadd L s
.58947 .27007 1.5 0 Mabsadd m
.58947 .27007 -1.5 0 Mabsadd L s
.59524 .29217 m
.59524 .26863 L
s
.59524 .29217 1.5 0 Mabsadd m
.59524 .29217 -1.5 0 Mabsadd L s
.59524 .26863 1.5 0 Mabsadd m
.59524 .26863 -1.5 0 Mabsadd L s
.60101 .29209 m
.60101 .26842 L
s
.60101 .29209 1.5 0 Mabsadd m
.60101 .29209 -1.5 0 Mabsadd L s
.60101 .26842 1.5 0 Mabsadd m
.60101 .26842 -1.5 0 Mabsadd L s
.60678 .29661 m
.60678 .2679 L
s
.60678 .29661 1.5 0 Mabsadd m
.60678 .29661 -1.5 0 Mabsadd L s
.60678 .2679 1.5 0 Mabsadd m
.60678 .2679 -1.5 0 Mabsadd L s
.61255 .29262 m
.61255 .26928 L
s
.61255 .29262 1.5 0 Mabsadd m
.61255 .29262 -1.5 0 Mabsadd L s
.61255 .26928 1.5 0 Mabsadd m
.61255 .26928 -1.5 0 Mabsadd L s
.61833 .29417 m
.61833 .26857 L
s
.61833 .29417 1.5 0 Mabsadd m
.61833 .29417 -1.5 0 Mabsadd L s
.61833 .26857 1.5 0 Mabsadd m
.61833 .26857 -1.5 0 Mabsadd L s
.6241 .29038 m
.6241 .26908 L
s
.6241 .29038 1.5 0 Mabsadd m
.6241 .29038 -1.5 0 Mabsadd L s
.6241 .26908 1.5 0 Mabsadd m
.6241 .26908 -1.5 0 Mabsadd L s
.62987 .29179 m
.62987 .2688 L
s
.62987 .29179 1.5 0 Mabsadd m
.62987 .29179 -1.5 0 Mabsadd L s
.62987 .2688 1.5 0 Mabsadd m
.62987 .2688 -1.5 0 Mabsadd L s
.63564 .29259 m
.63564 .26954 L
s
.63564 .29259 1.5 0 Mabsadd m
.63564 .29259 -1.5 0 Mabsadd L s
.63564 .26954 1.5 0 Mabsadd m
.63564 .26954 -1.5 0 Mabsadd L s
.64141 .29135 m
.64141 .26922 L
s
.64141 .29135 1.5 0 Mabsadd m
.64141 .29135 -1.5 0 Mabsadd L s
.64141 .26922 1.5 0 Mabsadd m
.64141 .26922 -1.5 0 Mabsadd L s
.64719 .29034 m
.64719 .27076 L
s
.64719 .29034 1.5 0 Mabsadd m
.64719 .29034 -1.5 0 Mabsadd L s
.64719 .27076 1.5 0 Mabsadd m
.64719 .27076 -1.5 0 Mabsadd L s
.65296 .29059 m
.65296 .27058 L
s
.65296 .29059 1.5 0 Mabsadd m
.65296 .29059 -1.5 0 Mabsadd L s
.65296 .27058 1.5 0 Mabsadd m
.65296 .27058 -1.5 0 Mabsadd L s
.65873 .29087 m
.65873 .27092 L
s
.65873 .29087 1.5 0 Mabsadd m
.65873 .29087 -1.5 0 Mabsadd L s
.65873 .27092 1.5 0 Mabsadd m
.65873 .27092 -1.5 0 Mabsadd L s
.6645 .28939 m
.6645 .27138 L
s
.6645 .28939 1.5 0 Mabsadd m
.6645 .28939 -1.5 0 Mabsadd L s
.6645 .27138 1.5 0 Mabsadd m
.6645 .27138 -1.5 0 Mabsadd L s
.67027 .28994 m
.67027 .26996 L
s
.67027 .28994 1.5 0 Mabsadd m
.67027 .28994 -1.5 0 Mabsadd L s
.67027 .26996 1.5 0 Mabsadd m
.67027 .26996 -1.5 0 Mabsadd L s
.67605 .29217 m
.67605 .27305 L
s
.67605 .29217 1.5 0 Mabsadd m
.67605 .29217 -1.5 0 Mabsadd L s
.67605 .27305 1.5 0 Mabsadd m
.67605 .27305 -1.5 0 Mabsadd L s
.68182 .28879 m
.68182 .26882 L
s
.68182 .28879 1.5 0 Mabsadd m
.68182 .28879 -1.5 0 Mabsadd L s
.68182 .26882 1.5 0 Mabsadd m
.68182 .26882 -1.5 0 Mabsadd L s
.68759 .29285 m
.68759 .27085 L
s
.68759 .29285 1.5 0 Mabsadd m
.68759 .29285 -1.5 0 Mabsadd L s
.68759 .27085 1.5 0 Mabsadd m
.68759 .27085 -1.5 0 Mabsadd L s
.69336 .28956 m
.69336 .27128 L
s
.69336 .28956 1.5 0 Mabsadd m
.69336 .28956 -1.5 0 Mabsadd L s
.69336 .27128 1.5 0 Mabsadd m
.69336 .27128 -1.5 0 Mabsadd L s
.69913 .29053 m
.69913 .27214 L
s
.69913 .29053 1.5 0 Mabsadd m
.69913 .29053 -1.5 0 Mabsadd L s
.69913 .27214 1.5 0 Mabsadd m
.69913 .27214 -1.5 0 Mabsadd L s
.70491 .28857 m
.70491 .27185 L
s
.70491 .28857 1.5 0 Mabsadd m
.70491 .28857 -1.5 0 Mabsadd L s
.70491 .27185 1.5 0 Mabsadd m
.70491 .27185 -1.5 0 Mabsadd L s
.71068 .28959 m
.71068 .27191 L
s
.71068 .28959 1.5 0 Mabsadd m
.71068 .28959 -1.5 0 Mabsadd L s
.71068 .27191 1.5 0 Mabsadd m
.71068 .27191 -1.5 0 Mabsadd L s
.71645 .28959 m
.71645 .27239 L
s
.71645 .28959 1.5 0 Mabsadd m
.71645 .28959 -1.5 0 Mabsadd L s
.71645 .27239 1.5 0 Mabsadd m
.71645 .27239 -1.5 0 Mabsadd L s
.72222 .28995 m
.72222 .27133 L
s
.72222 .28995 1.5 0 Mabsadd m
.72222 .28995 -1.5 0 Mabsadd L s
.72222 .27133 1.5 0 Mabsadd m
.72222 .27133 -1.5 0 Mabsadd L s
.72799 .28933 m
.72799 .27183 L
s
.72799 .28933 1.5 0 Mabsadd m
.72799 .28933 -1.5 0 Mabsadd L s
.72799 .27183 1.5 0 Mabsadd m
.72799 .27183 -1.5 0 Mabsadd L s
.74531 .28886 m
.74531 .27224 L
s
.74531 .28886 1.5 0 Mabsadd m
.74531 .28886 -1.5 0 Mabsadd L s
.74531 .27224 1.5 0 Mabsadd m
.74531 .27224 -1.5 0 Mabsadd L s
.75108 .28936 m
.75108 .27231 L
s
.75108 .28936 1.5 0 Mabsadd m
.75108 .28936 -1.5 0 Mabsadd L s
.75108 .27231 1.5 0 Mabsadd m
.75108 .27231 -1.5 0 Mabsadd L s
.75685 .29013 m
.75685 .27051 L
s
.75685 .29013 1.5 0 Mabsadd m
.75685 .29013 -1.5 0 Mabsadd L s
.75685 .27051 1.5 0 Mabsadd m
.75685 .27051 -1.5 0 Mabsadd L s
.76263 .28927 m
.76263 .27191 L
s
.76263 .28927 1.5 0 Mabsadd m
.76263 .28927 -1.5 0 Mabsadd L s
.76263 .27191 1.5 0 Mabsadd m
.76263 .27191 -1.5 0 Mabsadd L s
.7684 .28951 m
.7684 .27072 L
s
.7684 .28951 1.5 0 Mabsadd m
.7684 .28951 -1.5 0 Mabsadd L s
.7684 .27072 1.5 0 Mabsadd m
.7684 .27072 -1.5 0 Mabsadd L s
.77417 .28871 m
.77417 .26452 L
s
.77417 .28871 1.5 0 Mabsadd m
.77417 .28871 -1.5 0 Mabsadd L s
.77417 .26452 1.5 0 Mabsadd m
.77417 .26452 -1.5 0 Mabsadd L s
.77994 .29313 m
.77994 .27475 L
s
.77994 .29313 1.5 0 Mabsadd m
.77994 .29313 -1.5 0 Mabsadd L s
.77994 .27475 1.5 0 Mabsadd m
.77994 .27475 -1.5 0 Mabsadd L s
.78571 .28935 m
.78571 .27254 L
s
.78571 .28935 1.5 0 Mabsadd m
.78571 .28935 -1.5 0 Mabsadd L s
.78571 .27254 1.5 0 Mabsadd m
.78571 .27254 -1.5 0 Mabsadd L s
.79149 .28953 m
.79149 .27255 L
s
.79149 .28953 1.5 0 Mabsadd m
.79149 .28953 -1.5 0 Mabsadd L s
.79149 .27255 1.5 0 Mabsadd m
.79149 .27255 -1.5 0 Mabsadd L s
.79726 .28908 m
.79726 .27207 L
s
.79726 .28908 1.5 0 Mabsadd m
.79726 .28908 -1.5 0 Mabsadd L s
.79726 .27207 1.5 0 Mabsadd m
.79726 .27207 -1.5 0 Mabsadd L s
.80303 .29054 m
.80303 .27289 L
s
.80303 .29054 1.5 0 Mabsadd m
.80303 .29054 -1.5 0 Mabsadd L s
.80303 .27289 1.5 0 Mabsadd m
.80303 .27289 -1.5 0 Mabsadd L s
.8088 .28947 m
.8088 .27021 L
s
.8088 .28947 1.5 0 Mabsadd m
.8088 .28947 -1.5 0 Mabsadd L s
.8088 .27021 1.5 0 Mabsadd m
.8088 .27021 -1.5 0 Mabsadd L s
.81457 .29028 m
.81457 .27241 L
s
.81457 .29028 1.5 0 Mabsadd m
.81457 .29028 -1.5 0 Mabsadd L s
.81457 .27241 1.5 0 Mabsadd m
.81457 .27241 -1.5 0 Mabsadd L s
.82035 .28925 m
.82035 .27039 L
s
.82035 .28925 1.5 0 Mabsadd m
.82035 .28925 -1.5 0 Mabsadd L s
.82035 .27039 1.5 0 Mabsadd m
.82035 .27039 -1.5 0 Mabsadd L s
.82612 .29134 m
.82612 .27023 L
s
.82612 .29134 1.5 0 Mabsadd m
.82612 .29134 -1.5 0 Mabsadd L s
.82612 .27023 1.5 0 Mabsadd m
.82612 .27023 -1.5 0 Mabsadd L s
.83189 .28866 m
.83189 .27297 L
s
.83189 .28866 1.5 0 Mabsadd m
.83189 .28866 -1.5 0 Mabsadd L s
.83189 .27297 1.5 0 Mabsadd m
.83189 .27297 -1.5 0 Mabsadd L s
.83766 .29105 m
.83766 .27065 L
s
.83766 .29105 1.5 0 Mabsadd m
.83766 .29105 -1.5 0 Mabsadd L s
.83766 .27065 1.5 0 Mabsadd m
.83766 .27065 -1.5 0 Mabsadd L s
.84343 .29005 m
.84343 .26965 L
s
.84343 .29005 1.5 0 Mabsadd m
.84343 .29005 -1.5 0 Mabsadd L s
.84343 .26965 1.5 0 Mabsadd m
.84343 .26965 -1.5 0 Mabsadd L s
.84921 .28885 m
.84921 .2729 L
s
.84921 .28885 1.5 0 Mabsadd m
.84921 .28885 -1.5 0 Mabsadd L s
.84921 .2729 1.5 0 Mabsadd m
.84921 .2729 -1.5 0 Mabsadd L s
.85498 .28864 m
.85498 .27299 L
s
.85498 .28864 1.5 0 Mabsadd m
.85498 .28864 -1.5 0 Mabsadd L s
.85498 .27299 1.5 0 Mabsadd m
.85498 .27299 -1.5 0 Mabsadd L s
.86075 .28774 m
.86075 .27278 L
s
.86075 .28774 1.5 0 Mabsadd m
.86075 .28774 -1.5 0 Mabsadd L s
.86075 .27278 1.5 0 Mabsadd m
.86075 .27278 -1.5 0 Mabsadd L s
.86652 .28862 m
.86652 .27297 L
s
.86652 .28862 1.5 0 Mabsadd m
.86652 .28862 -1.5 0 Mabsadd L s
.86652 .27297 1.5 0 Mabsadd m
.86652 .27297 -1.5 0 Mabsadd L s
.87229 .28832 m
.87229 .27195 L
s
.87229 .28832 1.5 0 Mabsadd m
.87229 .28832 -1.5 0 Mabsadd L s
.87229 .27195 1.5 0 Mabsadd m
.87229 .27195 -1.5 0 Mabsadd L s
.87807 .28848 m
.87807 .27315 L
s
.87807 .28848 1.5 0 Mabsadd m
.87807 .28848 -1.5 0 Mabsadd L s
.87807 .27315 1.5 0 Mabsadd m
.87807 .27315 -1.5 0 Mabsadd L s
.88384 .28859 m
.88384 .27301 L
s
.88384 .28859 1.5 0 Mabsadd m
.88384 .28859 -1.5 0 Mabsadd L s
.88384 .27301 1.5 0 Mabsadd m
.88384 .27301 -1.5 0 Mabsadd L s
.88961 .29124 m
.88961 .26963 L
s
.88961 .29124 1.5 0 Mabsadd m
.88961 .29124 -1.5 0 Mabsadd L s
.88961 .26963 1.5 0 Mabsadd m
.88961 .26963 -1.5 0 Mabsadd L s
.89538 .28833 m
.89538 .2722 L
s
.89538 .28833 1.5 0 Mabsadd m
.89538 .28833 -1.5 0 Mabsadd L s
.89538 .2722 1.5 0 Mabsadd m
.89538 .2722 -1.5 0 Mabsadd L s
.90115 .28873 m
.90115 .27307 L
s
.90115 .28873 1.5 0 Mabsadd m
.90115 .28873 -1.5 0 Mabsadd L s
.90115 .27307 1.5 0 Mabsadd m
.90115 .27307 -1.5 0 Mabsadd L s
.90693 .29107 m
.90693 .26908 L
s
.90693 .29107 1.5 0 Mabsadd m
.90693 .29107 -1.5 0 Mabsadd L s
.90693 .26908 1.5 0 Mabsadd m
.90693 .26908 -1.5 0 Mabsadd L s
.9127 .28971 m
.9127 .27213 L
s
.9127 .28971 1.5 0 Mabsadd m
.9127 .28971 -1.5 0 Mabsadd L s
.9127 .27213 1.5 0 Mabsadd m
.9127 .27213 -1.5 0 Mabsadd L s
.91847 .28898 m
.91847 .27115 L
s
.91847 .28898 1.5 0 Mabsadd m
.91847 .28898 -1.5 0 Mabsadd L s
.91847 .27115 1.5 0 Mabsadd m
.91847 .27115 -1.5 0 Mabsadd L s
.92424 .28915 m
.92424 .27136 L
s
.92424 .28915 1.5 0 Mabsadd m
.92424 .28915 -1.5 0 Mabsadd L s
.92424 .27136 1.5 0 Mabsadd m
.92424 .27136 -1.5 0 Mabsadd L s
.93001 .29004 m
.93001 .27191 L
s
.93001 .29004 1.5 0 Mabsadd m
.93001 .29004 -1.5 0 Mabsadd L s
.93001 .27191 1.5 0 Mabsadd m
.93001 .27191 -1.5 0 Mabsadd L s
.93579 .28958 m
.93579 .27216 L
s
.93579 .28958 1.5 0 Mabsadd m
.93579 .28958 -1.5 0 Mabsadd L s
.93579 .27216 1.5 0 Mabsadd m
.93579 .27216 -1.5 0 Mabsadd L s
.94156 .28968 m
.94156 .27024 L
s
.94156 .28968 1.5 0 Mabsadd m
.94156 .28968 -1.5 0 Mabsadd L s
.94156 .27024 1.5 0 Mabsadd m
.94156 .27024 -1.5 0 Mabsadd L s
.94733 .2894 m
.94733 .27259 L
s
.94733 .2894 1.5 0 Mabsadd m
.94733 .2894 -1.5 0 Mabsadd L s
.94733 .27259 1.5 0 Mabsadd m
.94733 .27259 -1.5 0 Mabsadd L s
.9531 .28952 m
.9531 .27239 L
s
.9531 .28952 1.5 0 Mabsadd m
.9531 .28952 -1.5 0 Mabsadd L s
.9531 .27239 1.5 0 Mabsadd m
.9531 .27239 -1.5 0 Mabsadd L s
.95887 .28735 m
.95887 .27132 L
s
.95887 .28735 1.5 0 Mabsadd m
.95887 .28735 -1.5 0 Mabsadd L s
.95887 .27132 1.5 0 Mabsadd m
.95887 .27132 -1.5 0 Mabsadd L s
.96465 .28751 m
.96465 .2722 L
s
.96465 .28751 1.5 0 Mabsadd m
.96465 .28751 -1.5 0 Mabsadd L s
.96465 .2722 1.5 0 Mabsadd m
.96465 .2722 -1.5 0 Mabsadd L s
.97042 .28922 m
.97042 .2742 L
s
.97042 .28922 1.5 0 Mabsadd m
.97042 .28922 -1.5 0 Mabsadd L s
.97042 .2742 1.5 0 Mabsadd m
.97042 .2742 -1.5 0 Mabsadd L s
.97619 .28822 m
.97619 .27333 L
s
.97619 .28822 1.5 0 Mabsadd m
.97619 .28822 -1.5 0 Mabsadd L s
.97619 .27333 1.5 0 Mabsadd m
.97619 .27333 -1.5 0 Mabsadd L s
1 0 0 r
.0001 w
.02381 .28275 0 2.5 Mabsadd m
.02381 .28275 -0.56518 .7779 Mabsadd L
.02381 .28275 -2.37764 .77254 Mabsadd L
.02381 .28275 -0.91448 -0.29713 Mabsadd L
.02381 .28275 -1.46946 -2.02254 Mabsadd L
.02381 .28275 0 -0.96154 Mabsadd L
.02381 .28275 1.46946 -2.02254 Mabsadd L
.02381 .28275 .91448 -0.29713 Mabsadd L
.02381 .28275 2.37764 .77254 Mabsadd L
.02381 .28275 .56518 .7779 Mabsadd L
.02381 .28275 0 2.5 Mabsadd L closepath F
.02958 .28245 0 2.5 Mabsadd m
.02958 .28245 -0.56518 .7779 Mabsadd L
.02958 .28245 -2.37764 .77254 Mabsadd L
.02958 .28245 -0.91448 -0.29713 Mabsadd L
.02958 .28245 -1.46946 -2.02254 Mabsadd L
.02958 .28245 0 -0.96154 Mabsadd L
.02958 .28245 1.46946 -2.02254 Mabsadd L
.02958 .28245 .91448 -0.29713 Mabsadd L
.02958 .28245 2.37764 .77254 Mabsadd L
.02958 .28245 .56518 .7779 Mabsadd L
.02958 .28245 0 2.5 Mabsadd L closepath F
.03535 .28236 0 2.5 Mabsadd m
.03535 .28236 -0.56518 .7779 Mabsadd L
.03535 .28236 -2.37764 .77254 Mabsadd L
.03535 .28236 -0.91448 -0.29713 Mabsadd L
.03535 .28236 -1.46946 -2.02254 Mabsadd L
.03535 .28236 0 -0.96154 Mabsadd L
.03535 .28236 1.46946 -2.02254 Mabsadd L
.03535 .28236 .91448 -0.29713 Mabsadd L
.03535 .28236 2.37764 .77254 Mabsadd L
.03535 .28236 .56518 .7779 Mabsadd L
.03535 .28236 0 2.5 Mabsadd L closepath F
.04113 .28233 0 2.5 Mabsadd m
.04113 .28233 -0.56518 .7779 Mabsadd L
.04113 .28233 -2.37764 .77254 Mabsadd L
.04113 .28233 -0.91448 -0.29713 Mabsadd L
.04113 .28233 -1.46946 -2.02254 Mabsadd L
.04113 .28233 0 -0.96154 Mabsadd L
.04113 .28233 1.46946 -2.02254 Mabsadd L
.04113 .28233 .91448 -0.29713 Mabsadd L
.04113 .28233 2.37764 .77254 Mabsadd L
.04113 .28233 .56518 .7779 Mabsadd L
.04113 .28233 0 2.5 Mabsadd L closepath F
.0469 .28228 0 2.5 Mabsadd m
.0469 .28228 -0.56518 .7779 Mabsadd L
.0469 .28228 -2.37764 .77254 Mabsadd L
.0469 .28228 -0.91448 -0.29713 Mabsadd L
.0469 .28228 -1.46946 -2.02254 Mabsadd L
.0469 .28228 0 -0.96154 Mabsadd L
.0469 .28228 1.46946 -2.02254 Mabsadd L
.0469 .28228 .91448 -0.29713 Mabsadd L
.0469 .28228 2.37764 .77254 Mabsadd L
.0469 .28228 .56518 .7779 Mabsadd L
.0469 .28228 0 2.5 Mabsadd L closepath F
.05267 .28248 0 2.5 Mabsadd m
.05267 .28248 -0.56518 .7779 Mabsadd L
.05267 .28248 -2.37764 .77254 Mabsadd L
.05267 .28248 -0.91448 -0.29713 Mabsadd L
.05267 .28248 -1.46946 -2.02254 Mabsadd L
.05267 .28248 0 -0.96154 Mabsadd L
.05267 .28248 1.46946 -2.02254 Mabsadd L
.05267 .28248 .91448 -0.29713 Mabsadd L
.05267 .28248 2.37764 .77254 Mabsadd L
.05267 .28248 .56518 .7779 Mabsadd L
.05267 .28248 0 2.5 Mabsadd L closepath F
.05844 .28171 0 2.5 Mabsadd m
.05844 .28171 -0.56518 .7779 Mabsadd L
.05844 .28171 -2.37764 .77254 Mabsadd L
.05844 .28171 -0.91448 -0.29713 Mabsadd L
.05844 .28171 -1.46946 -2.02254 Mabsadd L
.05844 .28171 0 -0.96154 Mabsadd L
.05844 .28171 1.46946 -2.02254 Mabsadd L
.05844 .28171 .91448 -0.29713 Mabsadd L
.05844 .28171 2.37764 .77254 Mabsadd L
.05844 .28171 .56518 .7779 Mabsadd L
.05844 .28171 0 2.5 Mabsadd L closepath F
.06421 .28232 0 2.5 Mabsadd m
.06421 .28232 -0.56518 .7779 Mabsadd L
.06421 .28232 -2.37764 .77254 Mabsadd L
.06421 .28232 -0.91448 -0.29713 Mabsadd L
.06421 .28232 -1.46946 -2.02254 Mabsadd L
.06421 .28232 0 -0.96154 Mabsadd L
.06421 .28232 1.46946 -2.02254 Mabsadd L
.06421 .28232 .91448 -0.29713 Mabsadd L
.06421 .28232 2.37764 .77254 Mabsadd L
.06421 .28232 .56518 .7779 Mabsadd L
.06421 .28232 0 2.5 Mabsadd L closepath F
.06999 .28224 0 2.5 Mabsadd m
.06999 .28224 -0.56518 .7779 Mabsadd L
.06999 .28224 -2.37764 .77254 Mabsadd L
.06999 .28224 -0.91448 -0.29713 Mabsadd L
.06999 .28224 -1.46946 -2.02254 Mabsadd L
.06999 .28224 0 -0.96154 Mabsadd L
.06999 .28224 1.46946 -2.02254 Mabsadd L
.06999 .28224 .91448 -0.29713 Mabsadd L
.06999 .28224 2.37764 .77254 Mabsadd L
.06999 .28224 .56518 .7779 Mabsadd L
.06999 .28224 0 2.5 Mabsadd L closepath F
.07576 .28228 0 2.5 Mabsadd m
.07576 .28228 -0.56518 .7779 Mabsadd L
.07576 .28228 -2.37764 .77254 Mabsadd L
.07576 .28228 -0.91448 -0.29713 Mabsadd L
.07576 .28228 -1.46946 -2.02254 Mabsadd L
.07576 .28228 0 -0.96154 Mabsadd L
.07576 .28228 1.46946 -2.02254 Mabsadd L
.07576 .28228 .91448 -0.29713 Mabsadd L
.07576 .28228 2.37764 .77254 Mabsadd L
.07576 .28228 .56518 .7779 Mabsadd L
.07576 .28228 0 2.5 Mabsadd L closepath F
.08153 .28137 0 2.5 Mabsadd m
.08153 .28137 -0.56518 .7779 Mabsadd L
.08153 .28137 -2.37764 .77254 Mabsadd L
.08153 .28137 -0.91448 -0.29713 Mabsadd L
.08153 .28137 -1.46946 -2.02254 Mabsadd L
.08153 .28137 0 -0.96154 Mabsadd L
.08153 .28137 1.46946 -2.02254 Mabsadd L
.08153 .28137 .91448 -0.29713 Mabsadd L
.08153 .28137 2.37764 .77254 Mabsadd L
.08153 .28137 .56518 .7779 Mabsadd L
.08153 .28137 0 2.5 Mabsadd L closepath F
.0873 .28188 0 2.5 Mabsadd m
.0873 .28188 -0.56518 .7779 Mabsadd L
.0873 .28188 -2.37764 .77254 Mabsadd L
.0873 .28188 -0.91448 -0.29713 Mabsadd L
.0873 .28188 -1.46946 -2.02254 Mabsadd L
.0873 .28188 0 -0.96154 Mabsadd L
.0873 .28188 1.46946 -2.02254 Mabsadd L
.0873 .28188 .91448 -0.29713 Mabsadd L
.0873 .28188 2.37764 .77254 Mabsadd L
.0873 .28188 .56518 .7779 Mabsadd L
.0873 .28188 0 2.5 Mabsadd L closepath F
.09307 .28209 0 2.5 Mabsadd m
.09307 .28209 -0.56518 .7779 Mabsadd L
.09307 .28209 -2.37764 .77254 Mabsadd L
.09307 .28209 -0.91448 -0.29713 Mabsadd L
.09307 .28209 -1.46946 -2.02254 Mabsadd L
.09307 .28209 0 -0.96154 Mabsadd L
.09307 .28209 1.46946 -2.02254 Mabsadd L
.09307 .28209 .91448 -0.29713 Mabsadd L
.09307 .28209 2.37764 .77254 Mabsadd L
.09307 .28209 .56518 .7779 Mabsadd L
.09307 .28209 0 2.5 Mabsadd L closepath F
.09885 .28155 0 2.5 Mabsadd m
.09885 .28155 -0.56518 .7779 Mabsadd L
.09885 .28155 -2.37764 .77254 Mabsadd L
.09885 .28155 -0.91448 -0.29713 Mabsadd L
.09885 .28155 -1.46946 -2.02254 Mabsadd L
.09885 .28155 0 -0.96154 Mabsadd L
.09885 .28155 1.46946 -2.02254 Mabsadd L
.09885 .28155 .91448 -0.29713 Mabsadd L
.09885 .28155 2.37764 .77254 Mabsadd L
.09885 .28155 .56518 .7779 Mabsadd L
.09885 .28155 0 2.5 Mabsadd L closepath F
.10462 .28238 0 2.5 Mabsadd m
.10462 .28238 -0.56518 .7779 Mabsadd L
.10462 .28238 -2.37764 .77254 Mabsadd L
.10462 .28238 -0.91448 -0.29713 Mabsadd L
.10462 .28238 -1.46946 -2.02254 Mabsadd L
.10462 .28238 0 -0.96154 Mabsadd L
.10462 .28238 1.46946 -2.02254 Mabsadd L
.10462 .28238 .91448 -0.29713 Mabsadd L
.10462 .28238 2.37764 .77254 Mabsadd L
.10462 .28238 .56518 .7779 Mabsadd L
.10462 .28238 0 2.5 Mabsadd L closepath F
.11039 .28188 0 2.5 Mabsadd m
.11039 .28188 -0.56518 .7779 Mabsadd L
.11039 .28188 -2.37764 .77254 Mabsadd L
.11039 .28188 -0.91448 -0.29713 Mabsadd L
.11039 .28188 -1.46946 -2.02254 Mabsadd L
.11039 .28188 0 -0.96154 Mabsadd L
.11039 .28188 1.46946 -2.02254 Mabsadd L
.11039 .28188 .91448 -0.29713 Mabsadd L
.11039 .28188 2.37764 .77254 Mabsadd L
.11039 .28188 .56518 .7779 Mabsadd L
.11039 .28188 0 2.5 Mabsadd L closepath F
.11616 .28231 0 2.5 Mabsadd m
.11616 .28231 -0.56518 .7779 Mabsadd L
.11616 .28231 -2.37764 .77254 Mabsadd L
.11616 .28231 -0.91448 -0.29713 Mabsadd L
.11616 .28231 -1.46946 -2.02254 Mabsadd L
.11616 .28231 0 -0.96154 Mabsadd L
.11616 .28231 1.46946 -2.02254 Mabsadd L
.11616 .28231 .91448 -0.29713 Mabsadd L
.11616 .28231 2.37764 .77254 Mabsadd L
.11616 .28231 .56518 .7779 Mabsadd L
.11616 .28231 0 2.5 Mabsadd L closepath F
.12193 .28133 0 2.5 Mabsadd m
.12193 .28133 -0.56518 .7779 Mabsadd L
.12193 .28133 -2.37764 .77254 Mabsadd L
.12193 .28133 -0.91448 -0.29713 Mabsadd L
.12193 .28133 -1.46946 -2.02254 Mabsadd L
.12193 .28133 0 -0.96154 Mabsadd L
.12193 .28133 1.46946 -2.02254 Mabsadd L
.12193 .28133 .91448 -0.29713 Mabsadd L
.12193 .28133 2.37764 .77254 Mabsadd L
.12193 .28133 .56518 .7779 Mabsadd L
.12193 .28133 0 2.5 Mabsadd L closepath F
.12771 .28156 0 2.5 Mabsadd m
.12771 .28156 -0.56518 .7779 Mabsadd L
.12771 .28156 -2.37764 .77254 Mabsadd L
.12771 .28156 -0.91448 -0.29713 Mabsadd L
.12771 .28156 -1.46946 -2.02254 Mabsadd L
.12771 .28156 0 -0.96154 Mabsadd L
.12771 .28156 1.46946 -2.02254 Mabsadd L
.12771 .28156 .91448 -0.29713 Mabsadd L
.12771 .28156 2.37764 .77254 Mabsadd L
.12771 .28156 .56518 .7779 Mabsadd L
.12771 .28156 0 2.5 Mabsadd L closepath F
.13348 .28161 0 2.5 Mabsadd m
.13348 .28161 -0.56518 .7779 Mabsadd L
.13348 .28161 -2.37764 .77254 Mabsadd L
.13348 .28161 -0.91448 -0.29713 Mabsadd L
.13348 .28161 -1.46946 -2.02254 Mabsadd L
.13348 .28161 0 -0.96154 Mabsadd L
.13348 .28161 1.46946 -2.02254 Mabsadd L
.13348 .28161 .91448 -0.29713 Mabsadd L
.13348 .28161 2.37764 .77254 Mabsadd L
.13348 .28161 .56518 .7779 Mabsadd L
.13348 .28161 0 2.5 Mabsadd L closepath F
.13925 .28216 0 2.5 Mabsadd m
.13925 .28216 -0.56518 .7779 Mabsadd L
.13925 .28216 -2.37764 .77254 Mabsadd L
.13925 .28216 -0.91448 -0.29713 Mabsadd L
.13925 .28216 -1.46946 -2.02254 Mabsadd L
.13925 .28216 0 -0.96154 Mabsadd L
.13925 .28216 1.46946 -2.02254 Mabsadd L
.13925 .28216 .91448 -0.29713 Mabsadd L
.13925 .28216 2.37764 .77254 Mabsadd L
.13925 .28216 .56518 .7779 Mabsadd L
.13925 .28216 0 2.5 Mabsadd L closepath F
.14502 .28198 0 2.5 Mabsadd m
.14502 .28198 -0.56518 .7779 Mabsadd L
.14502 .28198 -2.37764 .77254 Mabsadd L
.14502 .28198 -0.91448 -0.29713 Mabsadd L
.14502 .28198 -1.46946 -2.02254 Mabsadd L
.14502 .28198 0 -0.96154 Mabsadd L
.14502 .28198 1.46946 -2.02254 Mabsadd L
.14502 .28198 .91448 -0.29713 Mabsadd L
.14502 .28198 2.37764 .77254 Mabsadd L
.14502 .28198 .56518 .7779 Mabsadd L
.14502 .28198 0 2.5 Mabsadd L closepath F
.15079 .28194 0 2.5 Mabsadd m
.15079 .28194 -0.56518 .7779 Mabsadd L
.15079 .28194 -2.37764 .77254 Mabsadd L
.15079 .28194 -0.91448 -0.29713 Mabsadd L
.15079 .28194 -1.46946 -2.02254 Mabsadd L
.15079 .28194 0 -0.96154 Mabsadd L
.15079 .28194 1.46946 -2.02254 Mabsadd L
.15079 .28194 .91448 -0.29713 Mabsadd L
.15079 .28194 2.37764 .77254 Mabsadd L
.15079 .28194 .56518 .7779 Mabsadd L
.15079 .28194 0 2.5 Mabsadd L closepath F
.15657 .27977 0 2.5 Mabsadd m
.15657 .27977 -0.56518 .7779 Mabsadd L
.15657 .27977 -2.37764 .77254 Mabsadd L
.15657 .27977 -0.91448 -0.29713 Mabsadd L
.15657 .27977 -1.46946 -2.02254 Mabsadd L
.15657 .27977 0 -0.96154 Mabsadd L
.15657 .27977 1.46946 -2.02254 Mabsadd L
.15657 .27977 .91448 -0.29713 Mabsadd L
.15657 .27977 2.37764 .77254 Mabsadd L
.15657 .27977 .56518 .7779 Mabsadd L
.15657 .27977 0 2.5 Mabsadd L closepath F
.16234 .284 0 2.5 Mabsadd m
.16234 .284 -0.56518 .7779 Mabsadd L
.16234 .284 -2.37764 .77254 Mabsadd L
.16234 .284 -0.91448 -0.29713 Mabsadd L
.16234 .284 -1.46946 -2.02254 Mabsadd L
.16234 .284 0 -0.96154 Mabsadd L
.16234 .284 1.46946 -2.02254 Mabsadd L
.16234 .284 .91448 -0.29713 Mabsadd L
.16234 .284 2.37764 .77254 Mabsadd L
.16234 .284 .56518 .7779 Mabsadd L
.16234 .284 0 2.5 Mabsadd L closepath F
.16811 .28169 0 2.5 Mabsadd m
.16811 .28169 -0.56518 .7779 Mabsadd L
.16811 .28169 -2.37764 .77254 Mabsadd L
.16811 .28169 -0.91448 -0.29713 Mabsadd L
.16811 .28169 -1.46946 -2.02254 Mabsadd L
.16811 .28169 0 -0.96154 Mabsadd L
.16811 .28169 1.46946 -2.02254 Mabsadd L
.16811 .28169 .91448 -0.29713 Mabsadd L
.16811 .28169 2.37764 .77254 Mabsadd L
.16811 .28169 .56518 .7779 Mabsadd L
.16811 .28169 0 2.5 Mabsadd L closepath F
.17388 .27939 0 2.5 Mabsadd m
.17388 .27939 -0.56518 .7779 Mabsadd L
.17388 .27939 -2.37764 .77254 Mabsadd L
.17388 .27939 -0.91448 -0.29713 Mabsadd L
.17388 .27939 -1.46946 -2.02254 Mabsadd L
.17388 .27939 0 -0.96154 Mabsadd L
.17388 .27939 1.46946 -2.02254 Mabsadd L
.17388 .27939 .91448 -0.29713 Mabsadd L
.17388 .27939 2.37764 .77254 Mabsadd L
.17388 .27939 .56518 .7779 Mabsadd L
.17388 .27939 0 2.5 Mabsadd L closepath F
.17965 .28316 0 2.5 Mabsadd m
.17965 .28316 -0.56518 .7779 Mabsadd L
.17965 .28316 -2.37764 .77254 Mabsadd L
.17965 .28316 -0.91448 -0.29713 Mabsadd L
.17965 .28316 -1.46946 -2.02254 Mabsadd L
.17965 .28316 0 -0.96154 Mabsadd L
.17965 .28316 1.46946 -2.02254 Mabsadd L
.17965 .28316 .91448 -0.29713 Mabsadd L
.17965 .28316 2.37764 .77254 Mabsadd L
.17965 .28316 .56518 .7779 Mabsadd L
.17965 .28316 0 2.5 Mabsadd L closepath F
.18543 .28358 0 2.5 Mabsadd m
.18543 .28358 -0.56518 .7779 Mabsadd L
.18543 .28358 -2.37764 .77254 Mabsadd L
.18543 .28358 -0.91448 -0.29713 Mabsadd L
.18543 .28358 -1.46946 -2.02254 Mabsadd L
.18543 .28358 0 -0.96154 Mabsadd L
.18543 .28358 1.46946 -2.02254 Mabsadd L
.18543 .28358 .91448 -0.29713 Mabsadd L
.18543 .28358 2.37764 .77254 Mabsadd L
.18543 .28358 .56518 .7779 Mabsadd L
.18543 .28358 0 2.5 Mabsadd L closepath F
.1912 .28421 0 2.5 Mabsadd m
.1912 .28421 -0.56518 .7779 Mabsadd L
.1912 .28421 -2.37764 .77254 Mabsadd L
.1912 .28421 -0.91448 -0.29713 Mabsadd L
.1912 .28421 -1.46946 -2.02254 Mabsadd L
.1912 .28421 0 -0.96154 Mabsadd L
.1912 .28421 1.46946 -2.02254 Mabsadd L
.1912 .28421 .91448 -0.29713 Mabsadd L
.1912 .28421 2.37764 .77254 Mabsadd L
.1912 .28421 .56518 .7779 Mabsadd L
.1912 .28421 0 2.5 Mabsadd L closepath F
.19697 .27734 0 2.5 Mabsadd m
.19697 .27734 -0.56518 .7779 Mabsadd L
.19697 .27734 -2.37764 .77254 Mabsadd L
.19697 .27734 -0.91448 -0.29713 Mabsadd L
.19697 .27734 -1.46946 -2.02254 Mabsadd L
.19697 .27734 0 -0.96154 Mabsadd L
.19697 .27734 1.46946 -2.02254 Mabsadd L
.19697 .27734 .91448 -0.29713 Mabsadd L
.19697 .27734 2.37764 .77254 Mabsadd L
.19697 .27734 .56518 .7779 Mabsadd L
.19697 .27734 0 2.5 Mabsadd L closepath F
.20274 .283 0 2.5 Mabsadd m
.20274 .283 -0.56518 .7779 Mabsadd L
.20274 .283 -2.37764 .77254 Mabsadd L
.20274 .283 -0.91448 -0.29713 Mabsadd L
.20274 .283 -1.46946 -2.02254 Mabsadd L
.20274 .283 0 -0.96154 Mabsadd L
.20274 .283 1.46946 -2.02254 Mabsadd L
.20274 .283 .91448 -0.29713 Mabsadd L
.20274 .283 2.37764 .77254 Mabsadd L
.20274 .283 .56518 .7779 Mabsadd L
.20274 .283 0 2.5 Mabsadd L closepath F
.20851 .28117 0 2.5 Mabsadd m
.20851 .28117 -0.56518 .7779 Mabsadd L
.20851 .28117 -2.37764 .77254 Mabsadd L
.20851 .28117 -0.91448 -0.29713 Mabsadd L
.20851 .28117 -1.46946 -2.02254 Mabsadd L
.20851 .28117 0 -0.96154 Mabsadd L
.20851 .28117 1.46946 -2.02254 Mabsadd L
.20851 .28117 .91448 -0.29713 Mabsadd L
.20851 .28117 2.37764 .77254 Mabsadd L
.20851 .28117 .56518 .7779 Mabsadd L
.20851 .28117 0 2.5 Mabsadd L closepath F
.21429 .28357 0 2.5 Mabsadd m
.21429 .28357 -0.56518 .7779 Mabsadd L
.21429 .28357 -2.37764 .77254 Mabsadd L
.21429 .28357 -0.91448 -0.29713 Mabsadd L
.21429 .28357 -1.46946 -2.02254 Mabsadd L
.21429 .28357 0 -0.96154 Mabsadd L
.21429 .28357 1.46946 -2.02254 Mabsadd L
.21429 .28357 .91448 -0.29713 Mabsadd L
.21429 .28357 2.37764 .77254 Mabsadd L
.21429 .28357 .56518 .7779 Mabsadd L
.21429 .28357 0 2.5 Mabsadd L closepath F
.22006 .28111 0 2.5 Mabsadd m
.22006 .28111 -0.56518 .7779 Mabsadd L
.22006 .28111 -2.37764 .77254 Mabsadd L
.22006 .28111 -0.91448 -0.29713 Mabsadd L
.22006 .28111 -1.46946 -2.02254 Mabsadd L
.22006 .28111 0 -0.96154 Mabsadd L
.22006 .28111 1.46946 -2.02254 Mabsadd L
.22006 .28111 .91448 -0.29713 Mabsadd L
.22006 .28111 2.37764 .77254 Mabsadd L
.22006 .28111 .56518 .7779 Mabsadd L
.22006 .28111 0 2.5 Mabsadd L closepath F
.22583 .27354 0 2.5 Mabsadd m
.22583 .27354 -0.56518 .7779 Mabsadd L
.22583 .27354 -2.37764 .77254 Mabsadd L
.22583 .27354 -0.91448 -0.29713 Mabsadd L
.22583 .27354 -1.46946 -2.02254 Mabsadd L
.22583 .27354 0 -0.96154 Mabsadd L
.22583 .27354 1.46946 -2.02254 Mabsadd L
.22583 .27354 .91448 -0.29713 Mabsadd L
.22583 .27354 2.37764 .77254 Mabsadd L
.22583 .27354 .56518 .7779 Mabsadd L
.22583 .27354 0 2.5 Mabsadd L closepath F
.2316 .29272 0 2.5 Mabsadd m
.2316 .29272 -0.56518 .7779 Mabsadd L
.2316 .29272 -2.37764 .77254 Mabsadd L
.2316 .29272 -0.91448 -0.29713 Mabsadd L
.2316 .29272 -1.46946 -2.02254 Mabsadd L
.2316 .29272 0 -0.96154 Mabsadd L
.2316 .29272 1.46946 -2.02254 Mabsadd L
.2316 .29272 .91448 -0.29713 Mabsadd L
.2316 .29272 2.37764 .77254 Mabsadd L
.2316 .29272 .56518 .7779 Mabsadd L
.2316 .29272 0 2.5 Mabsadd L closepath F
.23737 .28216 0 2.5 Mabsadd m
.23737 .28216 -0.56518 .7779 Mabsadd L
.23737 .28216 -2.37764 .77254 Mabsadd L
.23737 .28216 -0.91448 -0.29713 Mabsadd L
.23737 .28216 -1.46946 -2.02254 Mabsadd L
.23737 .28216 0 -0.96154 Mabsadd L
.23737 .28216 1.46946 -2.02254 Mabsadd L
.23737 .28216 .91448 -0.29713 Mabsadd L
.23737 .28216 2.37764 .77254 Mabsadd L
.23737 .28216 .56518 .7779 Mabsadd L
.23737 .28216 0 2.5 Mabsadd L closepath F
.24315 .27683 0 2.5 Mabsadd m
.24315 .27683 -0.56518 .7779 Mabsadd L
.24315 .27683 -2.37764 .77254 Mabsadd L
.24315 .27683 -0.91448 -0.29713 Mabsadd L
.24315 .27683 -1.46946 -2.02254 Mabsadd L
.24315 .27683 0 -0.96154 Mabsadd L
.24315 .27683 1.46946 -2.02254 Mabsadd L
.24315 .27683 .91448 -0.29713 Mabsadd L
.24315 .27683 2.37764 .77254 Mabsadd L
.24315 .27683 .56518 .7779 Mabsadd L
.24315 .27683 0 2.5 Mabsadd L closepath F
.24892 .28281 0 2.5 Mabsadd m
.24892 .28281 -0.56518 .7779 Mabsadd L
.24892 .28281 -2.37764 .77254 Mabsadd L
.24892 .28281 -0.91448 -0.29713 Mabsadd L
.24892 .28281 -1.46946 -2.02254 Mabsadd L
.24892 .28281 0 -0.96154 Mabsadd L
.24892 .28281 1.46946 -2.02254 Mabsadd L
.24892 .28281 .91448 -0.29713 Mabsadd L
.24892 .28281 2.37764 .77254 Mabsadd L
.24892 .28281 .56518 .7779 Mabsadd L
.24892 .28281 0 2.5 Mabsadd L closepath F
.25469 .28788 0 2.5 Mabsadd m
.25469 .28788 -0.56518 .7779 Mabsadd L
.25469 .28788 -2.37764 .77254 Mabsadd L
.25469 .28788 -0.91448 -0.29713 Mabsadd L
.25469 .28788 -1.46946 -2.02254 Mabsadd L
.25469 .28788 0 -0.96154 Mabsadd L
.25469 .28788 1.46946 -2.02254 Mabsadd L
.25469 .28788 .91448 -0.29713 Mabsadd L
.25469 .28788 2.37764 .77254 Mabsadd L
.25469 .28788 .56518 .7779 Mabsadd L
.25469 .28788 0 2.5 Mabsadd L closepath F
.26046 .2706 0 2.5 Mabsadd m
.26046 .2706 -0.56518 .7779 Mabsadd L
.26046 .2706 -2.37764 .77254 Mabsadd L
.26046 .2706 -0.91448 -0.29713 Mabsadd L
.26046 .2706 -1.46946 -2.02254 Mabsadd L
.26046 .2706 0 -0.96154 Mabsadd L
.26046 .2706 1.46946 -2.02254 Mabsadd L
.26046 .2706 .91448 -0.29713 Mabsadd L
.26046 .2706 2.37764 .77254 Mabsadd L
.26046 .2706 .56518 .7779 Mabsadd L
.26046 .2706 0 2.5 Mabsadd L closepath F
.26623 .30086 0 2.5 Mabsadd m
.26623 .30086 -0.56518 .7779 Mabsadd L
.26623 .30086 -2.37764 .77254 Mabsadd L
.26623 .30086 -0.91448 -0.29713 Mabsadd L
.26623 .30086 -1.46946 -2.02254 Mabsadd L
.26623 .30086 0 -0.96154 Mabsadd L
.26623 .30086 1.46946 -2.02254 Mabsadd L
.26623 .30086 .91448 -0.29713 Mabsadd L
.26623 .30086 2.37764 .77254 Mabsadd L
.26623 .30086 .56518 .7779 Mabsadd L
.26623 .30086 0 2.5 Mabsadd L closepath F
.27201 .26098 0 2.5 Mabsadd m
.27201 .26098 -0.56518 .7779 Mabsadd L
.27201 .26098 -2.37764 .77254 Mabsadd L
.27201 .26098 -0.91448 -0.29713 Mabsadd L
.27201 .26098 -1.46946 -2.02254 Mabsadd L
.27201 .26098 0 -0.96154 Mabsadd L
.27201 .26098 1.46946 -2.02254 Mabsadd L
.27201 .26098 .91448 -0.29713 Mabsadd L
.27201 .26098 2.37764 .77254 Mabsadd L
.27201 .26098 .56518 .7779 Mabsadd L
.27201 .26098 0 2.5 Mabsadd L closepath F
.27778 .29756 0 2.5 Mabsadd m
.27778 .29756 -0.56518 .7779 Mabsadd L
.27778 .29756 -2.37764 .77254 Mabsadd L
.27778 .29756 -0.91448 -0.29713 Mabsadd L
.27778 .29756 -1.46946 -2.02254 Mabsadd L
.27778 .29756 0 -0.96154 Mabsadd L
.27778 .29756 1.46946 -2.02254 Mabsadd L
.27778 .29756 .91448 -0.29713 Mabsadd L
.27778 .29756 2.37764 .77254 Mabsadd L
.27778 .29756 .56518 .7779 Mabsadd L
.27778 .29756 0 2.5 Mabsadd L closepath F
.28355 .27708 0 2.5 Mabsadd m
.28355 .27708 -0.56518 .7779 Mabsadd L
.28355 .27708 -2.37764 .77254 Mabsadd L
.28355 .27708 -0.91448 -0.29713 Mabsadd L
.28355 .27708 -1.46946 -2.02254 Mabsadd L
.28355 .27708 0 -0.96154 Mabsadd L
.28355 .27708 1.46946 -2.02254 Mabsadd L
.28355 .27708 .91448 -0.29713 Mabsadd L
.28355 .27708 2.37764 .77254 Mabsadd L
.28355 .27708 .56518 .7779 Mabsadd L
.28355 .27708 0 2.5 Mabsadd L closepath F
.28932 .28128 0 2.5 Mabsadd m
.28932 .28128 -0.56518 .7779 Mabsadd L
.28932 .28128 -2.37764 .77254 Mabsadd L
.28932 .28128 -0.91448 -0.29713 Mabsadd L
.28932 .28128 -1.46946 -2.02254 Mabsadd L
.28932 .28128 0 -0.96154 Mabsadd L
.28932 .28128 1.46946 -2.02254 Mabsadd L
.28932 .28128 .91448 -0.29713 Mabsadd L
.28932 .28128 2.37764 .77254 Mabsadd L
.28932 .28128 .56518 .7779 Mabsadd L
.28932 .28128 0 2.5 Mabsadd L closepath F
.29509 .27469 0 2.5 Mabsadd m
.29509 .27469 -0.56518 .7779 Mabsadd L
.29509 .27469 -2.37764 .77254 Mabsadd L
.29509 .27469 -0.91448 -0.29713 Mabsadd L
.29509 .27469 -1.46946 -2.02254 Mabsadd L
.29509 .27469 0 -0.96154 Mabsadd L
.29509 .27469 1.46946 -2.02254 Mabsadd L
.29509 .27469 .91448 -0.29713 Mabsadd L
.29509 .27469 2.37764 .77254 Mabsadd L
.29509 .27469 .56518 .7779 Mabsadd L
.29509 .27469 0 2.5 Mabsadd L closepath F
.30087 .28613 0 2.5 Mabsadd m
.30087 .28613 -0.56518 .7779 Mabsadd L
.30087 .28613 -2.37764 .77254 Mabsadd L
.30087 .28613 -0.91448 -0.29713 Mabsadd L
.30087 .28613 -1.46946 -2.02254 Mabsadd L
.30087 .28613 0 -0.96154 Mabsadd L
.30087 .28613 1.46946 -2.02254 Mabsadd L
.30087 .28613 .91448 -0.29713 Mabsadd L
.30087 .28613 2.37764 .77254 Mabsadd L
.30087 .28613 .56518 .7779 Mabsadd L
.30087 .28613 0 2.5 Mabsadd L closepath F
.30664 .27932 0 2.5 Mabsadd m
.30664 .27932 -0.56518 .7779 Mabsadd L
.30664 .27932 -2.37764 .77254 Mabsadd L
.30664 .27932 -0.91448 -0.29713 Mabsadd L
.30664 .27932 -1.46946 -2.02254 Mabsadd L
.30664 .27932 0 -0.96154 Mabsadd L
.30664 .27932 1.46946 -2.02254 Mabsadd L
.30664 .27932 .91448 -0.29713 Mabsadd L
.30664 .27932 2.37764 .77254 Mabsadd L
.30664 .27932 .56518 .7779 Mabsadd L
.30664 .27932 0 2.5 Mabsadd L closepath F
.31241 .28137 0 2.5 Mabsadd m
.31241 .28137 -0.56518 .7779 Mabsadd L
.31241 .28137 -2.37764 .77254 Mabsadd L
.31241 .28137 -0.91448 -0.29713 Mabsadd L
.31241 .28137 -1.46946 -2.02254 Mabsadd L
.31241 .28137 0 -0.96154 Mabsadd L
.31241 .28137 1.46946 -2.02254 Mabsadd L
.31241 .28137 .91448 -0.29713 Mabsadd L
.31241 .28137 2.37764 .77254 Mabsadd L
.31241 .28137 .56518 .7779 Mabsadd L
.31241 .28137 0 2.5 Mabsadd L closepath F
.31818 .28616 0 2.5 Mabsadd m
.31818 .28616 -0.56518 .7779 Mabsadd L
.31818 .28616 -2.37764 .77254 Mabsadd L
.31818 .28616 -0.91448 -0.29713 Mabsadd L
.31818 .28616 -1.46946 -2.02254 Mabsadd L
.31818 .28616 0 -0.96154 Mabsadd L
.31818 .28616 1.46946 -2.02254 Mabsadd L
.31818 .28616 .91448 -0.29713 Mabsadd L
.31818 .28616 2.37764 .77254 Mabsadd L
.31818 .28616 .56518 .7779 Mabsadd L
.31818 .28616 0 2.5 Mabsadd L closepath F
.32395 .28087 0 2.5 Mabsadd m
.32395 .28087 -0.56518 .7779 Mabsadd L
.32395 .28087 -2.37764 .77254 Mabsadd L
.32395 .28087 -0.91448 -0.29713 Mabsadd L
.32395 .28087 -1.46946 -2.02254 Mabsadd L
.32395 .28087 0 -0.96154 Mabsadd L
.32395 .28087 1.46946 -2.02254 Mabsadd L
.32395 .28087 .91448 -0.29713 Mabsadd L
.32395 .28087 2.37764 .77254 Mabsadd L
.32395 .28087 .56518 .7779 Mabsadd L
.32395 .28087 0 2.5 Mabsadd L closepath F
.32973 .27756 0 2.5 Mabsadd m
.32973 .27756 -0.56518 .7779 Mabsadd L
.32973 .27756 -2.37764 .77254 Mabsadd L
.32973 .27756 -0.91448 -0.29713 Mabsadd L
.32973 .27756 -1.46946 -2.02254 Mabsadd L
.32973 .27756 0 -0.96154 Mabsadd L
.32973 .27756 1.46946 -2.02254 Mabsadd L
.32973 .27756 .91448 -0.29713 Mabsadd L
.32973 .27756 2.37764 .77254 Mabsadd L
.32973 .27756 .56518 .7779 Mabsadd L
.32973 .27756 0 2.5 Mabsadd L closepath F
.3355 .27678 0 2.5 Mabsadd m
.3355 .27678 -0.56518 .7779 Mabsadd L
.3355 .27678 -2.37764 .77254 Mabsadd L
.3355 .27678 -0.91448 -0.29713 Mabsadd L
.3355 .27678 -1.46946 -2.02254 Mabsadd L
.3355 .27678 0 -0.96154 Mabsadd L
.3355 .27678 1.46946 -2.02254 Mabsadd L
.3355 .27678 .91448 -0.29713 Mabsadd L
.3355 .27678 2.37764 .77254 Mabsadd L
.3355 .27678 .56518 .7779 Mabsadd L
.3355 .27678 0 2.5 Mabsadd L closepath F
.34127 .28562 0 2.5 Mabsadd m
.34127 .28562 -0.56518 .7779 Mabsadd L
.34127 .28562 -2.37764 .77254 Mabsadd L
.34127 .28562 -0.91448 -0.29713 Mabsadd L
.34127 .28562 -1.46946 -2.02254 Mabsadd L
.34127 .28562 0 -0.96154 Mabsadd L
.34127 .28562 1.46946 -2.02254 Mabsadd L
.34127 .28562 .91448 -0.29713 Mabsadd L
.34127 .28562 2.37764 .77254 Mabsadd L
.34127 .28562 .56518 .7779 Mabsadd L
.34127 .28562 0 2.5 Mabsadd L closepath F
.34704 .28189 0 2.5 Mabsadd m
.34704 .28189 -0.56518 .7779 Mabsadd L
.34704 .28189 -2.37764 .77254 Mabsadd L
.34704 .28189 -0.91448 -0.29713 Mabsadd L
.34704 .28189 -1.46946 -2.02254 Mabsadd L
.34704 .28189 0 -0.96154 Mabsadd L
.34704 .28189 1.46946 -2.02254 Mabsadd L
.34704 .28189 .91448 -0.29713 Mabsadd L
.34704 .28189 2.37764 .77254 Mabsadd L
.34704 .28189 .56518 .7779 Mabsadd L
.34704 .28189 0 2.5 Mabsadd L closepath F
.35281 .27658 0 2.5 Mabsadd m
.35281 .27658 -0.56518 .7779 Mabsadd L
.35281 .27658 -2.37764 .77254 Mabsadd L
.35281 .27658 -0.91448 -0.29713 Mabsadd L
.35281 .27658 -1.46946 -2.02254 Mabsadd L
.35281 .27658 0 -0.96154 Mabsadd L
.35281 .27658 1.46946 -2.02254 Mabsadd L
.35281 .27658 .91448 -0.29713 Mabsadd L
.35281 .27658 2.37764 .77254 Mabsadd L
.35281 .27658 .56518 .7779 Mabsadd L
.35281 .27658 0 2.5 Mabsadd L closepath F
.35859 .28756 0 2.5 Mabsadd m
.35859 .28756 -0.56518 .7779 Mabsadd L
.35859 .28756 -2.37764 .77254 Mabsadd L
.35859 .28756 -0.91448 -0.29713 Mabsadd L
.35859 .28756 -1.46946 -2.02254 Mabsadd L
.35859 .28756 0 -0.96154 Mabsadd L
.35859 .28756 1.46946 -2.02254 Mabsadd L
.35859 .28756 .91448 -0.29713 Mabsadd L
.35859 .28756 2.37764 .77254 Mabsadd L
.35859 .28756 .56518 .7779 Mabsadd L
.35859 .28756 0 2.5 Mabsadd L closepath F
.36436 .27631 0 2.5 Mabsadd m
.36436 .27631 -0.56518 .7779 Mabsadd L
.36436 .27631 -2.37764 .77254 Mabsadd L
.36436 .27631 -0.91448 -0.29713 Mabsadd L
.36436 .27631 -1.46946 -2.02254 Mabsadd L
.36436 .27631 0 -0.96154 Mabsadd L
.36436 .27631 1.46946 -2.02254 Mabsadd L
.36436 .27631 .91448 -0.29713 Mabsadd L
.36436 .27631 2.37764 .77254 Mabsadd L
.36436 .27631 .56518 .7779 Mabsadd L
.36436 .27631 0 2.5 Mabsadd L closepath F
.37013 .28936 0 2.5 Mabsadd m
.37013 .28936 -0.56518 .7779 Mabsadd L
.37013 .28936 -2.37764 .77254 Mabsadd L
.37013 .28936 -0.91448 -0.29713 Mabsadd L
.37013 .28936 -1.46946 -2.02254 Mabsadd L
.37013 .28936 0 -0.96154 Mabsadd L
.37013 .28936 1.46946 -2.02254 Mabsadd L
.37013 .28936 .91448 -0.29713 Mabsadd L
.37013 .28936 2.37764 .77254 Mabsadd L
.37013 .28936 .56518 .7779 Mabsadd L
.37013 .28936 0 2.5 Mabsadd L closepath F
.3759 .27996 0 2.5 Mabsadd m
.3759 .27996 -0.56518 .7779 Mabsadd L
.3759 .27996 -2.37764 .77254 Mabsadd L
.3759 .27996 -0.91448 -0.29713 Mabsadd L
.3759 .27996 -1.46946 -2.02254 Mabsadd L
.3759 .27996 0 -0.96154 Mabsadd L
.3759 .27996 1.46946 -2.02254 Mabsadd L
.3759 .27996 .91448 -0.29713 Mabsadd L
.3759 .27996 2.37764 .77254 Mabsadd L
.3759 .27996 .56518 .7779 Mabsadd L
.3759 .27996 0 2.5 Mabsadd L closepath F
.38167 .27972 0 2.5 Mabsadd m
.38167 .27972 -0.56518 .7779 Mabsadd L
.38167 .27972 -2.37764 .77254 Mabsadd L
.38167 .27972 -0.91448 -0.29713 Mabsadd L
.38167 .27972 -1.46946 -2.02254 Mabsadd L
.38167 .27972 0 -0.96154 Mabsadd L
.38167 .27972 1.46946 -2.02254 Mabsadd L
.38167 .27972 .91448 -0.29713 Mabsadd L
.38167 .27972 2.37764 .77254 Mabsadd L
.38167 .27972 .56518 .7779 Mabsadd L
.38167 .27972 0 2.5 Mabsadd L closepath F
.38745 .27819 0 2.5 Mabsadd m
.38745 .27819 -0.56518 .7779 Mabsadd L
.38745 .27819 -2.37764 .77254 Mabsadd L
.38745 .27819 -0.91448 -0.29713 Mabsadd L
.38745 .27819 -1.46946 -2.02254 Mabsadd L
.38745 .27819 0 -0.96154 Mabsadd L
.38745 .27819 1.46946 -2.02254 Mabsadd L
.38745 .27819 .91448 -0.29713 Mabsadd L
.38745 .27819 2.37764 .77254 Mabsadd L
.38745 .27819 .56518 .7779 Mabsadd L
.38745 .27819 0 2.5 Mabsadd L closepath F
.39322 .27991 0 2.5 Mabsadd m
.39322 .27991 -0.56518 .7779 Mabsadd L
.39322 .27991 -2.37764 .77254 Mabsadd L
.39322 .27991 -0.91448 -0.29713 Mabsadd L
.39322 .27991 -1.46946 -2.02254 Mabsadd L
.39322 .27991 0 -0.96154 Mabsadd L
.39322 .27991 1.46946 -2.02254 Mabsadd L
.39322 .27991 .91448 -0.29713 Mabsadd L
.39322 .27991 2.37764 .77254 Mabsadd L
.39322 .27991 .56518 .7779 Mabsadd L
.39322 .27991 0 2.5 Mabsadd L closepath F
.39899 .28094 0 2.5 Mabsadd m
.39899 .28094 -0.56518 .7779 Mabsadd L
.39899 .28094 -2.37764 .77254 Mabsadd L
.39899 .28094 -0.91448 -0.29713 Mabsadd L
.39899 .28094 -1.46946 -2.02254 Mabsadd L
.39899 .28094 0 -0.96154 Mabsadd L
.39899 .28094 1.46946 -2.02254 Mabsadd L
.39899 .28094 .91448 -0.29713 Mabsadd L
.39899 .28094 2.37764 .77254 Mabsadd L
.39899 .28094 .56518 .7779 Mabsadd L
.39899 .28094 0 2.5 Mabsadd L closepath F
.40476 .28166 0 2.5 Mabsadd m
.40476 .28166 -0.56518 .7779 Mabsadd L
.40476 .28166 -2.37764 .77254 Mabsadd L
.40476 .28166 -0.91448 -0.29713 Mabsadd L
.40476 .28166 -1.46946 -2.02254 Mabsadd L
.40476 .28166 0 -0.96154 Mabsadd L
.40476 .28166 1.46946 -2.02254 Mabsadd L
.40476 .28166 .91448 -0.29713 Mabsadd L
.40476 .28166 2.37764 .77254 Mabsadd L
.40476 .28166 .56518 .7779 Mabsadd L
.40476 .28166 0 2.5 Mabsadd L closepath F
.41053 .28335 0 2.5 Mabsadd m
.41053 .28335 -0.56518 .7779 Mabsadd L
.41053 .28335 -2.37764 .77254 Mabsadd L
.41053 .28335 -0.91448 -0.29713 Mabsadd L
.41053 .28335 -1.46946 -2.02254 Mabsadd L
.41053 .28335 0 -0.96154 Mabsadd L
.41053 .28335 1.46946 -2.02254 Mabsadd L
.41053 .28335 .91448 -0.29713 Mabsadd L
.41053 .28335 2.37764 .77254 Mabsadd L
.41053 .28335 .56518 .7779 Mabsadd L
.41053 .28335 0 2.5 Mabsadd L closepath F
.41631 .28058 0 2.5 Mabsadd m
.41631 .28058 -0.56518 .7779 Mabsadd L
.41631 .28058 -2.37764 .77254 Mabsadd L
.41631 .28058 -0.91448 -0.29713 Mabsadd L
.41631 .28058 -1.46946 -2.02254 Mabsadd L
.41631 .28058 0 -0.96154 Mabsadd L
.41631 .28058 1.46946 -2.02254 Mabsadd L
.41631 .28058 .91448 -0.29713 Mabsadd L
.41631 .28058 2.37764 .77254 Mabsadd L
.41631 .28058 .56518 .7779 Mabsadd L
.41631 .28058 0 2.5 Mabsadd L closepath F
.42208 .27737 0 2.5 Mabsadd m
.42208 .27737 -0.56518 .7779 Mabsadd L
.42208 .27737 -2.37764 .77254 Mabsadd L
.42208 .27737 -0.91448 -0.29713 Mabsadd L
.42208 .27737 -1.46946 -2.02254 Mabsadd L
.42208 .27737 0 -0.96154 Mabsadd L
.42208 .27737 1.46946 -2.02254 Mabsadd L
.42208 .27737 .91448 -0.29713 Mabsadd L
.42208 .27737 2.37764 .77254 Mabsadd L
.42208 .27737 .56518 .7779 Mabsadd L
.42208 .27737 0 2.5 Mabsadd L closepath F
.42785 .28514 0 2.5 Mabsadd m
.42785 .28514 -0.56518 .7779 Mabsadd L
.42785 .28514 -2.37764 .77254 Mabsadd L
.42785 .28514 -0.91448 -0.29713 Mabsadd L
.42785 .28514 -1.46946 -2.02254 Mabsadd L
.42785 .28514 0 -0.96154 Mabsadd L
.42785 .28514 1.46946 -2.02254 Mabsadd L
.42785 .28514 .91448 -0.29713 Mabsadd L
.42785 .28514 2.37764 .77254 Mabsadd L
.42785 .28514 .56518 .7779 Mabsadd L
.42785 .28514 0 2.5 Mabsadd L closepath F
.43362 .2808 0 2.5 Mabsadd m
.43362 .2808 -0.56518 .7779 Mabsadd L
.43362 .2808 -2.37764 .77254 Mabsadd L
.43362 .2808 -0.91448 -0.29713 Mabsadd L
.43362 .2808 -1.46946 -2.02254 Mabsadd L
.43362 .2808 0 -0.96154 Mabsadd L
.43362 .2808 1.46946 -2.02254 Mabsadd L
.43362 .2808 .91448 -0.29713 Mabsadd L
.43362 .2808 2.37764 .77254 Mabsadd L
.43362 .2808 .56518 .7779 Mabsadd L
.43362 .2808 0 2.5 Mabsadd L closepath F
.43939 .28206 0 2.5 Mabsadd m
.43939 .28206 -0.56518 .7779 Mabsadd L
.43939 .28206 -2.37764 .77254 Mabsadd L
.43939 .28206 -0.91448 -0.29713 Mabsadd L
.43939 .28206 -1.46946 -2.02254 Mabsadd L
.43939 .28206 0 -0.96154 Mabsadd L
.43939 .28206 1.46946 -2.02254 Mabsadd L
.43939 .28206 .91448 -0.29713 Mabsadd L
.43939 .28206 2.37764 .77254 Mabsadd L
.43939 .28206 .56518 .7779 Mabsadd L
.43939 .28206 0 2.5 Mabsadd L closepath F
.44517 .28208 0 2.5 Mabsadd m
.44517 .28208 -0.56518 .7779 Mabsadd L
.44517 .28208 -2.37764 .77254 Mabsadd L
.44517 .28208 -0.91448 -0.29713 Mabsadd L
.44517 .28208 -1.46946 -2.02254 Mabsadd L
.44517 .28208 0 -0.96154 Mabsadd L
.44517 .28208 1.46946 -2.02254 Mabsadd L
.44517 .28208 .91448 -0.29713 Mabsadd L
.44517 .28208 2.37764 .77254 Mabsadd L
.44517 .28208 .56518 .7779 Mabsadd L
.44517 .28208 0 2.5 Mabsadd L closepath F
.45094 .27958 0 2.5 Mabsadd m
.45094 .27958 -0.56518 .7779 Mabsadd L
.45094 .27958 -2.37764 .77254 Mabsadd L
.45094 .27958 -0.91448 -0.29713 Mabsadd L
.45094 .27958 -1.46946 -2.02254 Mabsadd L
.45094 .27958 0 -0.96154 Mabsadd L
.45094 .27958 1.46946 -2.02254 Mabsadd L
.45094 .27958 .91448 -0.29713 Mabsadd L
.45094 .27958 2.37764 .77254 Mabsadd L
.45094 .27958 .56518 .7779 Mabsadd L
.45094 .27958 0 2.5 Mabsadd L closepath F
.45671 .28262 0 2.5 Mabsadd m
.45671 .28262 -0.56518 .7779 Mabsadd L
.45671 .28262 -2.37764 .77254 Mabsadd L
.45671 .28262 -0.91448 -0.29713 Mabsadd L
.45671 .28262 -1.46946 -2.02254 Mabsadd L
.45671 .28262 0 -0.96154 Mabsadd L
.45671 .28262 1.46946 -2.02254 Mabsadd L
.45671 .28262 .91448 -0.29713 Mabsadd L
.45671 .28262 2.37764 .77254 Mabsadd L
.45671 .28262 .56518 .7779 Mabsadd L
.45671 .28262 0 2.5 Mabsadd L closepath F
.46248 .28016 0 2.5 Mabsadd m
.46248 .28016 -0.56518 .7779 Mabsadd L
.46248 .28016 -2.37764 .77254 Mabsadd L
.46248 .28016 -0.91448 -0.29713 Mabsadd L
.46248 .28016 -1.46946 -2.02254 Mabsadd L
.46248 .28016 0 -0.96154 Mabsadd L
.46248 .28016 1.46946 -2.02254 Mabsadd L
.46248 .28016 .91448 -0.29713 Mabsadd L
.46248 .28016 2.37764 .77254 Mabsadd L
.46248 .28016 .56518 .7779 Mabsadd L
.46248 .28016 0 2.5 Mabsadd L closepath F
.46825 .27808 0 2.5 Mabsadd m
.46825 .27808 -0.56518 .7779 Mabsadd L
.46825 .27808 -2.37764 .77254 Mabsadd L
.46825 .27808 -0.91448 -0.29713 Mabsadd L
.46825 .27808 -1.46946 -2.02254 Mabsadd L
.46825 .27808 0 -0.96154 Mabsadd L
.46825 .27808 1.46946 -2.02254 Mabsadd L
.46825 .27808 .91448 -0.29713 Mabsadd L
.46825 .27808 2.37764 .77254 Mabsadd L
.46825 .27808 .56518 .7779 Mabsadd L
.46825 .27808 0 2.5 Mabsadd L closepath F
.47403 .28339 0 2.5 Mabsadd m
.47403 .28339 -0.56518 .7779 Mabsadd L
.47403 .28339 -2.37764 .77254 Mabsadd L
.47403 .28339 -0.91448 -0.29713 Mabsadd L
.47403 .28339 -1.46946 -2.02254 Mabsadd L
.47403 .28339 0 -0.96154 Mabsadd L
.47403 .28339 1.46946 -2.02254 Mabsadd L
.47403 .28339 .91448 -0.29713 Mabsadd L
.47403 .28339 2.37764 .77254 Mabsadd L
.47403 .28339 .56518 .7779 Mabsadd L
.47403 .28339 0 2.5 Mabsadd L closepath F
.4798 .28106 0 2.5 Mabsadd m
.4798 .28106 -0.56518 .7779 Mabsadd L
.4798 .28106 -2.37764 .77254 Mabsadd L
.4798 .28106 -0.91448 -0.29713 Mabsadd L
.4798 .28106 -1.46946 -2.02254 Mabsadd L
.4798 .28106 0 -0.96154 Mabsadd L
.4798 .28106 1.46946 -2.02254 Mabsadd L
.4798 .28106 .91448 -0.29713 Mabsadd L
.4798 .28106 2.37764 .77254 Mabsadd L
.4798 .28106 .56518 .7779 Mabsadd L
.4798 .28106 0 2.5 Mabsadd L closepath F
.48557 .27952 0 2.5 Mabsadd m
.48557 .27952 -0.56518 .7779 Mabsadd L
.48557 .27952 -2.37764 .77254 Mabsadd L
.48557 .27952 -0.91448 -0.29713 Mabsadd L
.48557 .27952 -1.46946 -2.02254 Mabsadd L
.48557 .27952 0 -0.96154 Mabsadd L
.48557 .27952 1.46946 -2.02254 Mabsadd L
.48557 .27952 .91448 -0.29713 Mabsadd L
.48557 .27952 2.37764 .77254 Mabsadd L
.48557 .27952 .56518 .7779 Mabsadd L
.48557 .27952 0 2.5 Mabsadd L closepath F
.49134 .28201 0 2.5 Mabsadd m
.49134 .28201 -0.56518 .7779 Mabsadd L
.49134 .28201 -2.37764 .77254 Mabsadd L
.49134 .28201 -0.91448 -0.29713 Mabsadd L
.49134 .28201 -1.46946 -2.02254 Mabsadd L
.49134 .28201 0 -0.96154 Mabsadd L
.49134 .28201 1.46946 -2.02254 Mabsadd L
.49134 .28201 .91448 -0.29713 Mabsadd L
.49134 .28201 2.37764 .77254 Mabsadd L
.49134 .28201 .56518 .7779 Mabsadd L
.49134 .28201 0 2.5 Mabsadd L closepath F
.49711 .28153 0 2.5 Mabsadd m
.49711 .28153 -0.56518 .7779 Mabsadd L
.49711 .28153 -2.37764 .77254 Mabsadd L
.49711 .28153 -0.91448 -0.29713 Mabsadd L
.49711 .28153 -1.46946 -2.02254 Mabsadd L
.49711 .28153 0 -0.96154 Mabsadd L
.49711 .28153 1.46946 -2.02254 Mabsadd L
.49711 .28153 .91448 -0.29713 Mabsadd L
.49711 .28153 2.37764 .77254 Mabsadd L
.49711 .28153 .56518 .7779 Mabsadd L
.49711 .28153 0 2.5 Mabsadd L closepath F
.50289 .28156 0 2.5 Mabsadd m
.50289 .28156 -0.56518 .7779 Mabsadd L
.50289 .28156 -2.37764 .77254 Mabsadd L
.50289 .28156 -0.91448 -0.29713 Mabsadd L
.50289 .28156 -1.46946 -2.02254 Mabsadd L
.50289 .28156 0 -0.96154 Mabsadd L
.50289 .28156 1.46946 -2.02254 Mabsadd L
.50289 .28156 .91448 -0.29713 Mabsadd L
.50289 .28156 2.37764 .77254 Mabsadd L
.50289 .28156 .56518 .7779 Mabsadd L
.50289 .28156 0 2.5 Mabsadd L closepath F
.50866 .28001 0 2.5 Mabsadd m
.50866 .28001 -0.56518 .7779 Mabsadd L
.50866 .28001 -2.37764 .77254 Mabsadd L
.50866 .28001 -0.91448 -0.29713 Mabsadd L
.50866 .28001 -1.46946 -2.02254 Mabsadd L
.50866 .28001 0 -0.96154 Mabsadd L
.50866 .28001 1.46946 -2.02254 Mabsadd L
.50866 .28001 .91448 -0.29713 Mabsadd L
.50866 .28001 2.37764 .77254 Mabsadd L
.50866 .28001 .56518 .7779 Mabsadd L
.50866 .28001 0 2.5 Mabsadd L closepath F
.51443 .28105 0 2.5 Mabsadd m
.51443 .28105 -0.56518 .7779 Mabsadd L
.51443 .28105 -2.37764 .77254 Mabsadd L
.51443 .28105 -0.91448 -0.29713 Mabsadd L
.51443 .28105 -1.46946 -2.02254 Mabsadd L
.51443 .28105 0 -0.96154 Mabsadd L
.51443 .28105 1.46946 -2.02254 Mabsadd L
.51443 .28105 .91448 -0.29713 Mabsadd L
.51443 .28105 2.37764 .77254 Mabsadd L
.51443 .28105 .56518 .7779 Mabsadd L
.51443 .28105 0 2.5 Mabsadd L closepath F
.5202 .2813 0 2.5 Mabsadd m
.5202 .2813 -0.56518 .7779 Mabsadd L
.5202 .2813 -2.37764 .77254 Mabsadd L
.5202 .2813 -0.91448 -0.29713 Mabsadd L
.5202 .2813 -1.46946 -2.02254 Mabsadd L
.5202 .2813 0 -0.96154 Mabsadd L
.5202 .2813 1.46946 -2.02254 Mabsadd L
.5202 .2813 .91448 -0.29713 Mabsadd L
.5202 .2813 2.37764 .77254 Mabsadd L
.5202 .2813 .56518 .7779 Mabsadd L
.5202 .2813 0 2.5 Mabsadd L closepath F
.52597 .28105 0 2.5 Mabsadd m
.52597 .28105 -0.56518 .7779 Mabsadd L
.52597 .28105 -2.37764 .77254 Mabsadd L
.52597 .28105 -0.91448 -0.29713 Mabsadd L
.52597 .28105 -1.46946 -2.02254 Mabsadd L
.52597 .28105 0 -0.96154 Mabsadd L
.52597 .28105 1.46946 -2.02254 Mabsadd L
.52597 .28105 .91448 -0.29713 Mabsadd L
.52597 .28105 2.37764 .77254 Mabsadd L
.52597 .28105 .56518 .7779 Mabsadd L
.52597 .28105 0 2.5 Mabsadd L closepath F
.53175 .2826 0 2.5 Mabsadd m
.53175 .2826 -0.56518 .7779 Mabsadd L
.53175 .2826 -2.37764 .77254 Mabsadd L
.53175 .2826 -0.91448 -0.29713 Mabsadd L
.53175 .2826 -1.46946 -2.02254 Mabsadd L
.53175 .2826 0 -0.96154 Mabsadd L
.53175 .2826 1.46946 -2.02254 Mabsadd L
.53175 .2826 .91448 -0.29713 Mabsadd L
.53175 .2826 2.37764 .77254 Mabsadd L
.53175 .2826 .56518 .7779 Mabsadd L
.53175 .2826 0 2.5 Mabsadd L closepath F
.53752 .2792 0 2.5 Mabsadd m
.53752 .2792 -0.56518 .7779 Mabsadd L
.53752 .2792 -2.37764 .77254 Mabsadd L
.53752 .2792 -0.91448 -0.29713 Mabsadd L
.53752 .2792 -1.46946 -2.02254 Mabsadd L
.53752 .2792 0 -0.96154 Mabsadd L
.53752 .2792 1.46946 -2.02254 Mabsadd L
.53752 .2792 .91448 -0.29713 Mabsadd L
.53752 .2792 2.37764 .77254 Mabsadd L
.53752 .2792 .56518 .7779 Mabsadd L
.53752 .2792 0 2.5 Mabsadd L closepath F
.54329 .28046 0 2.5 Mabsadd m
.54329 .28046 -0.56518 .7779 Mabsadd L
.54329 .28046 -2.37764 .77254 Mabsadd L
.54329 .28046 -0.91448 -0.29713 Mabsadd L
.54329 .28046 -1.46946 -2.02254 Mabsadd L
.54329 .28046 0 -0.96154 Mabsadd L
.54329 .28046 1.46946 -2.02254 Mabsadd L
.54329 .28046 .91448 -0.29713 Mabsadd L
.54329 .28046 2.37764 .77254 Mabsadd L
.54329 .28046 .56518 .7779 Mabsadd L
.54329 .28046 0 2.5 Mabsadd L closepath F
.54906 .28221 0 2.5 Mabsadd m
.54906 .28221 -0.56518 .7779 Mabsadd L
.54906 .28221 -2.37764 .77254 Mabsadd L
.54906 .28221 -0.91448 -0.29713 Mabsadd L
.54906 .28221 -1.46946 -2.02254 Mabsadd L
.54906 .28221 0 -0.96154 Mabsadd L
.54906 .28221 1.46946 -2.02254 Mabsadd L
.54906 .28221 .91448 -0.29713 Mabsadd L
.54906 .28221 2.37764 .77254 Mabsadd L
.54906 .28221 .56518 .7779 Mabsadd L
.54906 .28221 0 2.5 Mabsadd L closepath F
.55483 .28004 0 2.5 Mabsadd m
.55483 .28004 -0.56518 .7779 Mabsadd L
.55483 .28004 -2.37764 .77254 Mabsadd L
.55483 .28004 -0.91448 -0.29713 Mabsadd L
.55483 .28004 -1.46946 -2.02254 Mabsadd L
.55483 .28004 0 -0.96154 Mabsadd L
.55483 .28004 1.46946 -2.02254 Mabsadd L
.55483 .28004 .91448 -0.29713 Mabsadd L
.55483 .28004 2.37764 .77254 Mabsadd L
.55483 .28004 .56518 .7779 Mabsadd L
.55483 .28004 0 2.5 Mabsadd L closepath F
.56061 .28131 0 2.5 Mabsadd m
.56061 .28131 -0.56518 .7779 Mabsadd L
.56061 .28131 -2.37764 .77254 Mabsadd L
.56061 .28131 -0.91448 -0.29713 Mabsadd L
.56061 .28131 -1.46946 -2.02254 Mabsadd L
.56061 .28131 0 -0.96154 Mabsadd L
.56061 .28131 1.46946 -2.02254 Mabsadd L
.56061 .28131 .91448 -0.29713 Mabsadd L
.56061 .28131 2.37764 .77254 Mabsadd L
.56061 .28131 .56518 .7779 Mabsadd L
.56061 .28131 0 2.5 Mabsadd L closepath F
.56638 .28047 0 2.5 Mabsadd m
.56638 .28047 -0.56518 .7779 Mabsadd L
.56638 .28047 -2.37764 .77254 Mabsadd L
.56638 .28047 -0.91448 -0.29713 Mabsadd L
.56638 .28047 -1.46946 -2.02254 Mabsadd L
.56638 .28047 0 -0.96154 Mabsadd L
.56638 .28047 1.46946 -2.02254 Mabsadd L
.56638 .28047 .91448 -0.29713 Mabsadd L
.56638 .28047 2.37764 .77254 Mabsadd L
.56638 .28047 .56518 .7779 Mabsadd L
.56638 .28047 0 2.5 Mabsadd L closepath F
.57215 .28162 0 2.5 Mabsadd m
.57215 .28162 -0.56518 .7779 Mabsadd L
.57215 .28162 -2.37764 .77254 Mabsadd L
.57215 .28162 -0.91448 -0.29713 Mabsadd L
.57215 .28162 -1.46946 -2.02254 Mabsadd L
.57215 .28162 0 -0.96154 Mabsadd L
.57215 .28162 1.46946 -2.02254 Mabsadd L
.57215 .28162 .91448 -0.29713 Mabsadd L
.57215 .28162 2.37764 .77254 Mabsadd L
.57215 .28162 .56518 .7779 Mabsadd L
.57215 .28162 0 2.5 Mabsadd L closepath F
.57792 .28082 0 2.5 Mabsadd m
.57792 .28082 -0.56518 .7779 Mabsadd L
.57792 .28082 -2.37764 .77254 Mabsadd L
.57792 .28082 -0.91448 -0.29713 Mabsadd L
.57792 .28082 -1.46946 -2.02254 Mabsadd L
.57792 .28082 0 -0.96154 Mabsadd L
.57792 .28082 1.46946 -2.02254 Mabsadd L
.57792 .28082 .91448 -0.29713 Mabsadd L
.57792 .28082 2.37764 .77254 Mabsadd L
.57792 .28082 .56518 .7779 Mabsadd L
.57792 .28082 0 2.5 Mabsadd L closepath F
.58369 .2814 0 2.5 Mabsadd m
.58369 .2814 -0.56518 .7779 Mabsadd L
.58369 .2814 -2.37764 .77254 Mabsadd L
.58369 .2814 -0.91448 -0.29713 Mabsadd L
.58369 .2814 -1.46946 -2.02254 Mabsadd L
.58369 .2814 0 -0.96154 Mabsadd L
.58369 .2814 1.46946 -2.02254 Mabsadd L
.58369 .2814 .91448 -0.29713 Mabsadd L
.58369 .2814 2.37764 .77254 Mabsadd L
.58369 .2814 .56518 .7779 Mabsadd L
.58369 .2814 0 2.5 Mabsadd L closepath F
.58947 .28084 0 2.5 Mabsadd m
.58947 .28084 -0.56518 .7779 Mabsadd L
.58947 .28084 -2.37764 .77254 Mabsadd L
.58947 .28084 -0.91448 -0.29713 Mabsadd L
.58947 .28084 -1.46946 -2.02254 Mabsadd L
.58947 .28084 0 -0.96154 Mabsadd L
.58947 .28084 1.46946 -2.02254 Mabsadd L
.58947 .28084 .91448 -0.29713 Mabsadd L
.58947 .28084 2.37764 .77254 Mabsadd L
.58947 .28084 .56518 .7779 Mabsadd L
.58947 .28084 0 2.5 Mabsadd L closepath F
.59524 .2804 0 2.5 Mabsadd m
.59524 .2804 -0.56518 .7779 Mabsadd L
.59524 .2804 -2.37764 .77254 Mabsadd L
.59524 .2804 -0.91448 -0.29713 Mabsadd L
.59524 .2804 -1.46946 -2.02254 Mabsadd L
.59524 .2804 0 -0.96154 Mabsadd L
.59524 .2804 1.46946 -2.02254 Mabsadd L
.59524 .2804 .91448 -0.29713 Mabsadd L
.59524 .2804 2.37764 .77254 Mabsadd L
.59524 .2804 .56518 .7779 Mabsadd L
.59524 .2804 0 2.5 Mabsadd L closepath F
.60101 .28026 0 2.5 Mabsadd m
.60101 .28026 -0.56518 .7779 Mabsadd L
.60101 .28026 -2.37764 .77254 Mabsadd L
.60101 .28026 -0.91448 -0.29713 Mabsadd L
.60101 .28026 -1.46946 -2.02254 Mabsadd L
.60101 .28026 0 -0.96154 Mabsadd L
.60101 .28026 1.46946 -2.02254 Mabsadd L
.60101 .28026 .91448 -0.29713 Mabsadd L
.60101 .28026 2.37764 .77254 Mabsadd L
.60101 .28026 .56518 .7779 Mabsadd L
.60101 .28026 0 2.5 Mabsadd L closepath F
.60678 .28226 0 2.5 Mabsadd m
.60678 .28226 -0.56518 .7779 Mabsadd L
.60678 .28226 -2.37764 .77254 Mabsadd L
.60678 .28226 -0.91448 -0.29713 Mabsadd L
.60678 .28226 -1.46946 -2.02254 Mabsadd L
.60678 .28226 0 -0.96154 Mabsadd L
.60678 .28226 1.46946 -2.02254 Mabsadd L
.60678 .28226 .91448 -0.29713 Mabsadd L
.60678 .28226 2.37764 .77254 Mabsadd L
.60678 .28226 .56518 .7779 Mabsadd L
.60678 .28226 0 2.5 Mabsadd L closepath F
.61255 .28095 0 2.5 Mabsadd m
.61255 .28095 -0.56518 .7779 Mabsadd L
.61255 .28095 -2.37764 .77254 Mabsadd L
.61255 .28095 -0.91448 -0.29713 Mabsadd L
.61255 .28095 -1.46946 -2.02254 Mabsadd L
.61255 .28095 0 -0.96154 Mabsadd L
.61255 .28095 1.46946 -2.02254 Mabsadd L
.61255 .28095 .91448 -0.29713 Mabsadd L
.61255 .28095 2.37764 .77254 Mabsadd L
.61255 .28095 .56518 .7779 Mabsadd L
.61255 .28095 0 2.5 Mabsadd L closepath F
.61833 .28137 0 2.5 Mabsadd m
.61833 .28137 -0.56518 .7779 Mabsadd L
.61833 .28137 -2.37764 .77254 Mabsadd L
.61833 .28137 -0.91448 -0.29713 Mabsadd L
.61833 .28137 -1.46946 -2.02254 Mabsadd L
.61833 .28137 0 -0.96154 Mabsadd L
.61833 .28137 1.46946 -2.02254 Mabsadd L
.61833 .28137 .91448 -0.29713 Mabsadd L
.61833 .28137 2.37764 .77254 Mabsadd L
.61833 .28137 .56518 .7779 Mabsadd L
.61833 .28137 0 2.5 Mabsadd L closepath F
.6241 .27973 0 2.5 Mabsadd m
.6241 .27973 -0.56518 .7779 Mabsadd L
.6241 .27973 -2.37764 .77254 Mabsadd L
.6241 .27973 -0.91448 -0.29713 Mabsadd L
.6241 .27973 -1.46946 -2.02254 Mabsadd L
.6241 .27973 0 -0.96154 Mabsadd L
.6241 .27973 1.46946 -2.02254 Mabsadd L
.6241 .27973 .91448 -0.29713 Mabsadd L
.6241 .27973 2.37764 .77254 Mabsadd L
.6241 .27973 .56518 .7779 Mabsadd L
.6241 .27973 0 2.5 Mabsadd L closepath F
.62987 .28029 0 2.5 Mabsadd m
.62987 .28029 -0.56518 .7779 Mabsadd L
.62987 .28029 -2.37764 .77254 Mabsadd L
.62987 .28029 -0.91448 -0.29713 Mabsadd L
.62987 .28029 -1.46946 -2.02254 Mabsadd L
.62987 .28029 0 -0.96154 Mabsadd L
.62987 .28029 1.46946 -2.02254 Mabsadd L
.62987 .28029 .91448 -0.29713 Mabsadd L
.62987 .28029 2.37764 .77254 Mabsadd L
.62987 .28029 .56518 .7779 Mabsadd L
.62987 .28029 0 2.5 Mabsadd L closepath F
.63564 .28106 0 2.5 Mabsadd m
.63564 .28106 -0.56518 .7779 Mabsadd L
.63564 .28106 -2.37764 .77254 Mabsadd L
.63564 .28106 -0.91448 -0.29713 Mabsadd L
.63564 .28106 -1.46946 -2.02254 Mabsadd L
.63564 .28106 0 -0.96154 Mabsadd L
.63564 .28106 1.46946 -2.02254 Mabsadd L
.63564 .28106 .91448 -0.29713 Mabsadd L
.63564 .28106 2.37764 .77254 Mabsadd L
.63564 .28106 .56518 .7779 Mabsadd L
.63564 .28106 0 2.5 Mabsadd L closepath F
.64141 .28028 0 2.5 Mabsadd m
.64141 .28028 -0.56518 .7779 Mabsadd L
.64141 .28028 -2.37764 .77254 Mabsadd L
.64141 .28028 -0.91448 -0.29713 Mabsadd L
.64141 .28028 -1.46946 -2.02254 Mabsadd L
.64141 .28028 0 -0.96154 Mabsadd L
.64141 .28028 1.46946 -2.02254 Mabsadd L
.64141 .28028 .91448 -0.29713 Mabsadd L
.64141 .28028 2.37764 .77254 Mabsadd L
.64141 .28028 .56518 .7779 Mabsadd L
.64141 .28028 0 2.5 Mabsadd L closepath F
.64719 .28055 0 2.5 Mabsadd m
.64719 .28055 -0.56518 .7779 Mabsadd L
.64719 .28055 -2.37764 .77254 Mabsadd L
.64719 .28055 -0.91448 -0.29713 Mabsadd L
.64719 .28055 -1.46946 -2.02254 Mabsadd L
.64719 .28055 0 -0.96154 Mabsadd L
.64719 .28055 1.46946 -2.02254 Mabsadd L
.64719 .28055 .91448 -0.29713 Mabsadd L
.64719 .28055 2.37764 .77254 Mabsadd L
.64719 .28055 .56518 .7779 Mabsadd L
.64719 .28055 0 2.5 Mabsadd L closepath F
.65296 .28059 0 2.5 Mabsadd m
.65296 .28059 -0.56518 .7779 Mabsadd L
.65296 .28059 -2.37764 .77254 Mabsadd L
.65296 .28059 -0.91448 -0.29713 Mabsadd L
.65296 .28059 -1.46946 -2.02254 Mabsadd L
.65296 .28059 0 -0.96154 Mabsadd L
.65296 .28059 1.46946 -2.02254 Mabsadd L
.65296 .28059 .91448 -0.29713 Mabsadd L
.65296 .28059 2.37764 .77254 Mabsadd L
.65296 .28059 .56518 .7779 Mabsadd L
.65296 .28059 0 2.5 Mabsadd L closepath F
.65873 .28089 0 2.5 Mabsadd m
.65873 .28089 -0.56518 .7779 Mabsadd L
.65873 .28089 -2.37764 .77254 Mabsadd L
.65873 .28089 -0.91448 -0.29713 Mabsadd L
.65873 .28089 -1.46946 -2.02254 Mabsadd L
.65873 .28089 0 -0.96154 Mabsadd L
.65873 .28089 1.46946 -2.02254 Mabsadd L
.65873 .28089 .91448 -0.29713 Mabsadd L
.65873 .28089 2.37764 .77254 Mabsadd L
.65873 .28089 .56518 .7779 Mabsadd L
.65873 .28089 0 2.5 Mabsadd L closepath F
.6645 .28038 0 2.5 Mabsadd m
.6645 .28038 -0.56518 .7779 Mabsadd L
.6645 .28038 -2.37764 .77254 Mabsadd L
.6645 .28038 -0.91448 -0.29713 Mabsadd L
.6645 .28038 -1.46946 -2.02254 Mabsadd L
.6645 .28038 0 -0.96154 Mabsadd L
.6645 .28038 1.46946 -2.02254 Mabsadd L
.6645 .28038 .91448 -0.29713 Mabsadd L
.6645 .28038 2.37764 .77254 Mabsadd L
.6645 .28038 .56518 .7779 Mabsadd L
.6645 .28038 0 2.5 Mabsadd L closepath F
.67027 .27995 0 2.5 Mabsadd m
.67027 .27995 -0.56518 .7779 Mabsadd L
.67027 .27995 -2.37764 .77254 Mabsadd L
.67027 .27995 -0.91448 -0.29713 Mabsadd L
.67027 .27995 -1.46946 -2.02254 Mabsadd L
.67027 .27995 0 -0.96154 Mabsadd L
.67027 .27995 1.46946 -2.02254 Mabsadd L
.67027 .27995 .91448 -0.29713 Mabsadd L
.67027 .27995 2.37764 .77254 Mabsadd L
.67027 .27995 .56518 .7779 Mabsadd L
.67027 .27995 0 2.5 Mabsadd L closepath F
.67605 .28261 0 2.5 Mabsadd m
.67605 .28261 -0.56518 .7779 Mabsadd L
.67605 .28261 -2.37764 .77254 Mabsadd L
.67605 .28261 -0.91448 -0.29713 Mabsadd L
.67605 .28261 -1.46946 -2.02254 Mabsadd L
.67605 .28261 0 -0.96154 Mabsadd L
.67605 .28261 1.46946 -2.02254 Mabsadd L
.67605 .28261 .91448 -0.29713 Mabsadd L
.67605 .28261 2.37764 .77254 Mabsadd L
.67605 .28261 .56518 .7779 Mabsadd L
.67605 .28261 0 2.5 Mabsadd L closepath F
.68182 .27881 0 2.5 Mabsadd m
.68182 .27881 -0.56518 .7779 Mabsadd L
.68182 .27881 -2.37764 .77254 Mabsadd L
.68182 .27881 -0.91448 -0.29713 Mabsadd L
.68182 .27881 -1.46946 -2.02254 Mabsadd L
.68182 .27881 0 -0.96154 Mabsadd L
.68182 .27881 1.46946 -2.02254 Mabsadd L
.68182 .27881 .91448 -0.29713 Mabsadd L
.68182 .27881 2.37764 .77254 Mabsadd L
.68182 .27881 .56518 .7779 Mabsadd L
.68182 .27881 0 2.5 Mabsadd L closepath F
.68759 .28185 0 2.5 Mabsadd m
.68759 .28185 -0.56518 .7779 Mabsadd L
.68759 .28185 -2.37764 .77254 Mabsadd L
.68759 .28185 -0.91448 -0.29713 Mabsadd L
.68759 .28185 -1.46946 -2.02254 Mabsadd L
.68759 .28185 0 -0.96154 Mabsadd L
.68759 .28185 1.46946 -2.02254 Mabsadd L
.68759 .28185 .91448 -0.29713 Mabsadd L
.68759 .28185 2.37764 .77254 Mabsadd L
.68759 .28185 .56518 .7779 Mabsadd L
.68759 .28185 0 2.5 Mabsadd L closepath F
.69336 .28042 0 2.5 Mabsadd m
.69336 .28042 -0.56518 .7779 Mabsadd L
.69336 .28042 -2.37764 .77254 Mabsadd L
.69336 .28042 -0.91448 -0.29713 Mabsadd L
.69336 .28042 -1.46946 -2.02254 Mabsadd L
.69336 .28042 0 -0.96154 Mabsadd L
.69336 .28042 1.46946 -2.02254 Mabsadd L
.69336 .28042 .91448 -0.29713 Mabsadd L
.69336 .28042 2.37764 .77254 Mabsadd L
.69336 .28042 .56518 .7779 Mabsadd L
.69336 .28042 0 2.5 Mabsadd L closepath F
.69913 .28133 0 2.5 Mabsadd m
.69913 .28133 -0.56518 .7779 Mabsadd L
.69913 .28133 -2.37764 .77254 Mabsadd L
.69913 .28133 -0.91448 -0.29713 Mabsadd L
.69913 .28133 -1.46946 -2.02254 Mabsadd L
.69913 .28133 0 -0.96154 Mabsadd L
.69913 .28133 1.46946 -2.02254 Mabsadd L
.69913 .28133 .91448 -0.29713 Mabsadd L
.69913 .28133 2.37764 .77254 Mabsadd L
.69913 .28133 .56518 .7779 Mabsadd L
.69913 .28133 0 2.5 Mabsadd L closepath F
.70491 .28021 0 2.5 Mabsadd m
.70491 .28021 -0.56518 .7779 Mabsadd L
.70491 .28021 -2.37764 .77254 Mabsadd L
.70491 .28021 -0.91448 -0.29713 Mabsadd L
.70491 .28021 -1.46946 -2.02254 Mabsadd L
.70491 .28021 0 -0.96154 Mabsadd L
.70491 .28021 1.46946 -2.02254 Mabsadd L
.70491 .28021 .91448 -0.29713 Mabsadd L
.70491 .28021 2.37764 .77254 Mabsadd L
.70491 .28021 .56518 .7779 Mabsadd L
.70491 .28021 0 2.5 Mabsadd L closepath F
.71068 .28075 0 2.5 Mabsadd m
.71068 .28075 -0.56518 .7779 Mabsadd L
.71068 .28075 -2.37764 .77254 Mabsadd L
.71068 .28075 -0.91448 -0.29713 Mabsadd L
.71068 .28075 -1.46946 -2.02254 Mabsadd L
.71068 .28075 0 -0.96154 Mabsadd L
.71068 .28075 1.46946 -2.02254 Mabsadd L
.71068 .28075 .91448 -0.29713 Mabsadd L
.71068 .28075 2.37764 .77254 Mabsadd L
.71068 .28075 .56518 .7779 Mabsadd L
.71068 .28075 0 2.5 Mabsadd L closepath F
.71645 .28099 0 2.5 Mabsadd m
.71645 .28099 -0.56518 .7779 Mabsadd L
.71645 .28099 -2.37764 .77254 Mabsadd L
.71645 .28099 -0.91448 -0.29713 Mabsadd L
.71645 .28099 -1.46946 -2.02254 Mabsadd L
.71645 .28099 0 -0.96154 Mabsadd L
.71645 .28099 1.46946 -2.02254 Mabsadd L
.71645 .28099 .91448 -0.29713 Mabsadd L
.71645 .28099 2.37764 .77254 Mabsadd L
.71645 .28099 .56518 .7779 Mabsadd L
.71645 .28099 0 2.5 Mabsadd L closepath F
.72222 .28064 0 2.5 Mabsadd m
.72222 .28064 -0.56518 .7779 Mabsadd L
.72222 .28064 -2.37764 .77254 Mabsadd L
.72222 .28064 -0.91448 -0.29713 Mabsadd L
.72222 .28064 -1.46946 -2.02254 Mabsadd L
.72222 .28064 0 -0.96154 Mabsadd L
.72222 .28064 1.46946 -2.02254 Mabsadd L
.72222 .28064 .91448 -0.29713 Mabsadd L
.72222 .28064 2.37764 .77254 Mabsadd L
.72222 .28064 .56518 .7779 Mabsadd L
.72222 .28064 0 2.5 Mabsadd L closepath F
.72799 .28058 0 2.5 Mabsadd m
.72799 .28058 -0.56518 .7779 Mabsadd L
.72799 .28058 -2.37764 .77254 Mabsadd L
.72799 .28058 -0.91448 -0.29713 Mabsadd L
.72799 .28058 -1.46946 -2.02254 Mabsadd L
.72799 .28058 0 -0.96154 Mabsadd L
.72799 .28058 1.46946 -2.02254 Mabsadd L
.72799 .28058 .91448 -0.29713 Mabsadd L
.72799 .28058 2.37764 .77254 Mabsadd L
.72799 .28058 .56518 .7779 Mabsadd L
.72799 .28058 0 2.5 Mabsadd L closepath F
.74531 .28055 0 2.5 Mabsadd m
.74531 .28055 -0.56518 .7779 Mabsadd L
.74531 .28055 -2.37764 .77254 Mabsadd L
.74531 .28055 -0.91448 -0.29713 Mabsadd L
.74531 .28055 -1.46946 -2.02254 Mabsadd L
.74531 .28055 0 -0.96154 Mabsadd L
.74531 .28055 1.46946 -2.02254 Mabsadd L
.74531 .28055 .91448 -0.29713 Mabsadd L
.74531 .28055 2.37764 .77254 Mabsadd L
.74531 .28055 .56518 .7779 Mabsadd L
.74531 .28055 0 2.5 Mabsadd L closepath F
.75108 .28083 0 2.5 Mabsadd m
.75108 .28083 -0.56518 .7779 Mabsadd L
.75108 .28083 -2.37764 .77254 Mabsadd L
.75108 .28083 -0.91448 -0.29713 Mabsadd L
.75108 .28083 -1.46946 -2.02254 Mabsadd L
.75108 .28083 0 -0.96154 Mabsadd L
.75108 .28083 1.46946 -2.02254 Mabsadd L
.75108 .28083 .91448 -0.29713 Mabsadd L
.75108 .28083 2.37764 .77254 Mabsadd L
.75108 .28083 .56518 .7779 Mabsadd L
.75108 .28083 0 2.5 Mabsadd L closepath F
.75685 .28032 0 2.5 Mabsadd m
.75685 .28032 -0.56518 .7779 Mabsadd L
.75685 .28032 -2.37764 .77254 Mabsadd L
.75685 .28032 -0.91448 -0.29713 Mabsadd L
.75685 .28032 -1.46946 -2.02254 Mabsadd L
.75685 .28032 0 -0.96154 Mabsadd L
.75685 .28032 1.46946 -2.02254 Mabsadd L
.75685 .28032 .91448 -0.29713 Mabsadd L
.75685 .28032 2.37764 .77254 Mabsadd L
.75685 .28032 .56518 .7779 Mabsadd L
.75685 .28032 0 2.5 Mabsadd L closepath F
.76263 .28059 0 2.5 Mabsadd m
.76263 .28059 -0.56518 .7779 Mabsadd L
.76263 .28059 -2.37764 .77254 Mabsadd L
.76263 .28059 -0.91448 -0.29713 Mabsadd L
.76263 .28059 -1.46946 -2.02254 Mabsadd L
.76263 .28059 0 -0.96154 Mabsadd L
.76263 .28059 1.46946 -2.02254 Mabsadd L
.76263 .28059 .91448 -0.29713 Mabsadd L
.76263 .28059 2.37764 .77254 Mabsadd L
.76263 .28059 .56518 .7779 Mabsadd L
.76263 .28059 0 2.5 Mabsadd L closepath F
.7684 .28011 0 2.5 Mabsadd m
.7684 .28011 -0.56518 .7779 Mabsadd L
.7684 .28011 -2.37764 .77254 Mabsadd L
.7684 .28011 -0.91448 -0.29713 Mabsadd L
.7684 .28011 -1.46946 -2.02254 Mabsadd L
.7684 .28011 0 -0.96154 Mabsadd L
.7684 .28011 1.46946 -2.02254 Mabsadd L
.7684 .28011 .91448 -0.29713 Mabsadd L
.7684 .28011 2.37764 .77254 Mabsadd L
.7684 .28011 .56518 .7779 Mabsadd L
.7684 .28011 0 2.5 Mabsadd L closepath F
.77417 .27661 0 2.5 Mabsadd m
.77417 .27661 -0.56518 .7779 Mabsadd L
.77417 .27661 -2.37764 .77254 Mabsadd L
.77417 .27661 -0.91448 -0.29713 Mabsadd L
.77417 .27661 -1.46946 -2.02254 Mabsadd L
.77417 .27661 0 -0.96154 Mabsadd L
.77417 .27661 1.46946 -2.02254 Mabsadd L
.77417 .27661 .91448 -0.29713 Mabsadd L
.77417 .27661 2.37764 .77254 Mabsadd L
.77417 .27661 .56518 .7779 Mabsadd L
.77417 .27661 0 2.5 Mabsadd L closepath F
.77994 .28394 0 2.5 Mabsadd m
.77994 .28394 -0.56518 .7779 Mabsadd L
.77994 .28394 -2.37764 .77254 Mabsadd L
.77994 .28394 -0.91448 -0.29713 Mabsadd L
.77994 .28394 -1.46946 -2.02254 Mabsadd L
.77994 .28394 0 -0.96154 Mabsadd L
.77994 .28394 1.46946 -2.02254 Mabsadd L
.77994 .28394 .91448 -0.29713 Mabsadd L
.77994 .28394 2.37764 .77254 Mabsadd L
.77994 .28394 .56518 .7779 Mabsadd L
.77994 .28394 0 2.5 Mabsadd L closepath F
.78571 .28094 0 2.5 Mabsadd m
.78571 .28094 -0.56518 .7779 Mabsadd L
.78571 .28094 -2.37764 .77254 Mabsadd L
.78571 .28094 -0.91448 -0.29713 Mabsadd L
.78571 .28094 -1.46946 -2.02254 Mabsadd L
.78571 .28094 0 -0.96154 Mabsadd L
.78571 .28094 1.46946 -2.02254 Mabsadd L
.78571 .28094 .91448 -0.29713 Mabsadd L
.78571 .28094 2.37764 .77254 Mabsadd L
.78571 .28094 .56518 .7779 Mabsadd L
.78571 .28094 0 2.5 Mabsadd L closepath F
.79149 .28104 0 2.5 Mabsadd m
.79149 .28104 -0.56518 .7779 Mabsadd L
.79149 .28104 -2.37764 .77254 Mabsadd L
.79149 .28104 -0.91448 -0.29713 Mabsadd L
.79149 .28104 -1.46946 -2.02254 Mabsadd L
.79149 .28104 0 -0.96154 Mabsadd L
.79149 .28104 1.46946 -2.02254 Mabsadd L
.79149 .28104 .91448 -0.29713 Mabsadd L
.79149 .28104 2.37764 .77254 Mabsadd L
.79149 .28104 .56518 .7779 Mabsadd L
.79149 .28104 0 2.5 Mabsadd L closepath F
.79726 .28057 0 2.5 Mabsadd m
.79726 .28057 -0.56518 .7779 Mabsadd L
.79726 .28057 -2.37764 .77254 Mabsadd L
.79726 .28057 -0.91448 -0.29713 Mabsadd L
.79726 .28057 -1.46946 -2.02254 Mabsadd L
.79726 .28057 0 -0.96154 Mabsadd L
.79726 .28057 1.46946 -2.02254 Mabsadd L
.79726 .28057 .91448 -0.29713 Mabsadd L
.79726 .28057 2.37764 .77254 Mabsadd L
.79726 .28057 .56518 .7779 Mabsadd L
.79726 .28057 0 2.5 Mabsadd L closepath F
.80303 .28171 0 2.5 Mabsadd m
.80303 .28171 -0.56518 .7779 Mabsadd L
.80303 .28171 -2.37764 .77254 Mabsadd L
.80303 .28171 -0.91448 -0.29713 Mabsadd L
.80303 .28171 -1.46946 -2.02254 Mabsadd L
.80303 .28171 0 -0.96154 Mabsadd L
.80303 .28171 1.46946 -2.02254 Mabsadd L
.80303 .28171 .91448 -0.29713 Mabsadd L
.80303 .28171 2.37764 .77254 Mabsadd L
.80303 .28171 .56518 .7779 Mabsadd L
.80303 .28171 0 2.5 Mabsadd L closepath F
.8088 .27984 0 2.5 Mabsadd m
.8088 .27984 -0.56518 .7779 Mabsadd L
.8088 .27984 -2.37764 .77254 Mabsadd L
.8088 .27984 -0.91448 -0.29713 Mabsadd L
.8088 .27984 -1.46946 -2.02254 Mabsadd L
.8088 .27984 0 -0.96154 Mabsadd L
.8088 .27984 1.46946 -2.02254 Mabsadd L
.8088 .27984 .91448 -0.29713 Mabsadd L
.8088 .27984 2.37764 .77254 Mabsadd L
.8088 .27984 .56518 .7779 Mabsadd L
.8088 .27984 0 2.5 Mabsadd L closepath F
.81457 .28134 0 2.5 Mabsadd m
.81457 .28134 -0.56518 .7779 Mabsadd L
.81457 .28134 -2.37764 .77254 Mabsadd L
.81457 .28134 -0.91448 -0.29713 Mabsadd L
.81457 .28134 -1.46946 -2.02254 Mabsadd L
.81457 .28134 0 -0.96154 Mabsadd L
.81457 .28134 1.46946 -2.02254 Mabsadd L
.81457 .28134 .91448 -0.29713 Mabsadd L
.81457 .28134 2.37764 .77254 Mabsadd L
.81457 .28134 .56518 .7779 Mabsadd L
.81457 .28134 0 2.5 Mabsadd L closepath F
.82035 .27982 0 2.5 Mabsadd m
.82035 .27982 -0.56518 .7779 Mabsadd L
.82035 .27982 -2.37764 .77254 Mabsadd L
.82035 .27982 -0.91448 -0.29713 Mabsadd L
.82035 .27982 -1.46946 -2.02254 Mabsadd L
.82035 .27982 0 -0.96154 Mabsadd L
.82035 .27982 1.46946 -2.02254 Mabsadd L
.82035 .27982 .91448 -0.29713 Mabsadd L
.82035 .27982 2.37764 .77254 Mabsadd L
.82035 .27982 .56518 .7779 Mabsadd L
.82035 .27982 0 2.5 Mabsadd L closepath F
.82612 .28079 0 2.5 Mabsadd m
.82612 .28079 -0.56518 .7779 Mabsadd L
.82612 .28079 -2.37764 .77254 Mabsadd L
.82612 .28079 -0.91448 -0.29713 Mabsadd L
.82612 .28079 -1.46946 -2.02254 Mabsadd L
.82612 .28079 0 -0.96154 Mabsadd L
.82612 .28079 1.46946 -2.02254 Mabsadd L
.82612 .28079 .91448 -0.29713 Mabsadd L
.82612 .28079 2.37764 .77254 Mabsadd L
.82612 .28079 .56518 .7779 Mabsadd L
.82612 .28079 0 2.5 Mabsadd L closepath F
.83189 .28082 0 2.5 Mabsadd m
.83189 .28082 -0.56518 .7779 Mabsadd L
.83189 .28082 -2.37764 .77254 Mabsadd L
.83189 .28082 -0.91448 -0.29713 Mabsadd L
.83189 .28082 -1.46946 -2.02254 Mabsadd L
.83189 .28082 0 -0.96154 Mabsadd L
.83189 .28082 1.46946 -2.02254 Mabsadd L
.83189 .28082 .91448 -0.29713 Mabsadd L
.83189 .28082 2.37764 .77254 Mabsadd L
.83189 .28082 .56518 .7779 Mabsadd L
.83189 .28082 0 2.5 Mabsadd L closepath F
.83766 .28085 0 2.5 Mabsadd m
.83766 .28085 -0.56518 .7779 Mabsadd L
.83766 .28085 -2.37764 .77254 Mabsadd L
.83766 .28085 -0.91448 -0.29713 Mabsadd L
.83766 .28085 -1.46946 -2.02254 Mabsadd L
.83766 .28085 0 -0.96154 Mabsadd L
.83766 .28085 1.46946 -2.02254 Mabsadd L
.83766 .28085 .91448 -0.29713 Mabsadd L
.83766 .28085 2.37764 .77254 Mabsadd L
.83766 .28085 .56518 .7779 Mabsadd L
.83766 .28085 0 2.5 Mabsadd L closepath F
.84343 .27985 0 2.5 Mabsadd m
.84343 .27985 -0.56518 .7779 Mabsadd L
.84343 .27985 -2.37764 .77254 Mabsadd L
.84343 .27985 -0.91448 -0.29713 Mabsadd L
.84343 .27985 -1.46946 -2.02254 Mabsadd L
.84343 .27985 0 -0.96154 Mabsadd L
.84343 .27985 1.46946 -2.02254 Mabsadd L
.84343 .27985 .91448 -0.29713 Mabsadd L
.84343 .27985 2.37764 .77254 Mabsadd L
.84343 .27985 .56518 .7779 Mabsadd L
.84343 .27985 0 2.5 Mabsadd L closepath F
.84921 .28088 0 2.5 Mabsadd m
.84921 .28088 -0.56518 .7779 Mabsadd L
.84921 .28088 -2.37764 .77254 Mabsadd L
.84921 .28088 -0.91448 -0.29713 Mabsadd L
.84921 .28088 -1.46946 -2.02254 Mabsadd L
.84921 .28088 0 -0.96154 Mabsadd L
.84921 .28088 1.46946 -2.02254 Mabsadd L
.84921 .28088 .91448 -0.29713 Mabsadd L
.84921 .28088 2.37764 .77254 Mabsadd L
.84921 .28088 .56518 .7779 Mabsadd L
.84921 .28088 0 2.5 Mabsadd L closepath F
.85498 .28081 0 2.5 Mabsadd m
.85498 .28081 -0.56518 .7779 Mabsadd L
.85498 .28081 -2.37764 .77254 Mabsadd L
.85498 .28081 -0.91448 -0.29713 Mabsadd L
.85498 .28081 -1.46946 -2.02254 Mabsadd L
.85498 .28081 0 -0.96154 Mabsadd L
.85498 .28081 1.46946 -2.02254 Mabsadd L
.85498 .28081 .91448 -0.29713 Mabsadd L
.85498 .28081 2.37764 .77254 Mabsadd L
.85498 .28081 .56518 .7779 Mabsadd L
.85498 .28081 0 2.5 Mabsadd L closepath F
.86075 .28026 0 2.5 Mabsadd m
.86075 .28026 -0.56518 .7779 Mabsadd L
.86075 .28026 -2.37764 .77254 Mabsadd L
.86075 .28026 -0.91448 -0.29713 Mabsadd L
.86075 .28026 -1.46946 -2.02254 Mabsadd L
.86075 .28026 0 -0.96154 Mabsadd L
.86075 .28026 1.46946 -2.02254 Mabsadd L
.86075 .28026 .91448 -0.29713 Mabsadd L
.86075 .28026 2.37764 .77254 Mabsadd L
.86075 .28026 .56518 .7779 Mabsadd L
.86075 .28026 0 2.5 Mabsadd L closepath F
.86652 .2808 0 2.5 Mabsadd m
.86652 .2808 -0.56518 .7779 Mabsadd L
.86652 .2808 -2.37764 .77254 Mabsadd L
.86652 .2808 -0.91448 -0.29713 Mabsadd L
.86652 .2808 -1.46946 -2.02254 Mabsadd L
.86652 .2808 0 -0.96154 Mabsadd L
.86652 .2808 1.46946 -2.02254 Mabsadd L
.86652 .2808 .91448 -0.29713 Mabsadd L
.86652 .2808 2.37764 .77254 Mabsadd L
.86652 .2808 .56518 .7779 Mabsadd L
.86652 .2808 0 2.5 Mabsadd L closepath F
.87229 .28013 0 2.5 Mabsadd m
.87229 .28013 -0.56518 .7779 Mabsadd L
.87229 .28013 -2.37764 .77254 Mabsadd L
.87229 .28013 -0.91448 -0.29713 Mabsadd L
.87229 .28013 -1.46946 -2.02254 Mabsadd L
.87229 .28013 0 -0.96154 Mabsadd L
.87229 .28013 1.46946 -2.02254 Mabsadd L
.87229 .28013 .91448 -0.29713 Mabsadd L
.87229 .28013 2.37764 .77254 Mabsadd L
.87229 .28013 .56518 .7779 Mabsadd L
.87229 .28013 0 2.5 Mabsadd L closepath F
.87807 .28081 0 2.5 Mabsadd m
.87807 .28081 -0.56518 .7779 Mabsadd L
.87807 .28081 -2.37764 .77254 Mabsadd L
.87807 .28081 -0.91448 -0.29713 Mabsadd L
.87807 .28081 -1.46946 -2.02254 Mabsadd L
.87807 .28081 0 -0.96154 Mabsadd L
.87807 .28081 1.46946 -2.02254 Mabsadd L
.87807 .28081 .91448 -0.29713 Mabsadd L
.87807 .28081 2.37764 .77254 Mabsadd L
.87807 .28081 .56518 .7779 Mabsadd L
.87807 .28081 0 2.5 Mabsadd L closepath F
.88384 .2808 0 2.5 Mabsadd m
.88384 .2808 -0.56518 .7779 Mabsadd L
.88384 .2808 -2.37764 .77254 Mabsadd L
.88384 .2808 -0.91448 -0.29713 Mabsadd L
.88384 .2808 -1.46946 -2.02254 Mabsadd L
.88384 .2808 0 -0.96154 Mabsadd L
.88384 .2808 1.46946 -2.02254 Mabsadd L
.88384 .2808 .91448 -0.29713 Mabsadd L
.88384 .2808 2.37764 .77254 Mabsadd L
.88384 .2808 .56518 .7779 Mabsadd L
.88384 .2808 0 2.5 Mabsadd L closepath F
.88961 .28044 0 2.5 Mabsadd m
.88961 .28044 -0.56518 .7779 Mabsadd L
.88961 .28044 -2.37764 .77254 Mabsadd L
.88961 .28044 -0.91448 -0.29713 Mabsadd L
.88961 .28044 -1.46946 -2.02254 Mabsadd L
.88961 .28044 0 -0.96154 Mabsadd L
.88961 .28044 1.46946 -2.02254 Mabsadd L
.88961 .28044 .91448 -0.29713 Mabsadd L
.88961 .28044 2.37764 .77254 Mabsadd L
.88961 .28044 .56518 .7779 Mabsadd L
.88961 .28044 0 2.5 Mabsadd L closepath F
.89538 .28027 0 2.5 Mabsadd m
.89538 .28027 -0.56518 .7779 Mabsadd L
.89538 .28027 -2.37764 .77254 Mabsadd L
.89538 .28027 -0.91448 -0.29713 Mabsadd L
.89538 .28027 -1.46946 -2.02254 Mabsadd L
.89538 .28027 0 -0.96154 Mabsadd L
.89538 .28027 1.46946 -2.02254 Mabsadd L
.89538 .28027 .91448 -0.29713 Mabsadd L
.89538 .28027 2.37764 .77254 Mabsadd L
.89538 .28027 .56518 .7779 Mabsadd L
.89538 .28027 0 2.5 Mabsadd L closepath F
.90115 .2809 0 2.5 Mabsadd m
.90115 .2809 -0.56518 .7779 Mabsadd L
.90115 .2809 -2.37764 .77254 Mabsadd L
.90115 .2809 -0.91448 -0.29713 Mabsadd L
.90115 .2809 -1.46946 -2.02254 Mabsadd L
.90115 .2809 0 -0.96154 Mabsadd L
.90115 .2809 1.46946 -2.02254 Mabsadd L
.90115 .2809 .91448 -0.29713 Mabsadd L
.90115 .2809 2.37764 .77254 Mabsadd L
.90115 .2809 .56518 .7779 Mabsadd L
.90115 .2809 0 2.5 Mabsadd L closepath F
.90693 .28007 0 2.5 Mabsadd m
.90693 .28007 -0.56518 .7779 Mabsadd L
.90693 .28007 -2.37764 .77254 Mabsadd L
.90693 .28007 -0.91448 -0.29713 Mabsadd L
.90693 .28007 -1.46946 -2.02254 Mabsadd L
.90693 .28007 0 -0.96154 Mabsadd L
.90693 .28007 1.46946 -2.02254 Mabsadd L
.90693 .28007 .91448 -0.29713 Mabsadd L
.90693 .28007 2.37764 .77254 Mabsadd L
.90693 .28007 .56518 .7779 Mabsadd L
.90693 .28007 0 2.5 Mabsadd L closepath F
.9127 .28092 0 2.5 Mabsadd m
.9127 .28092 -0.56518 .7779 Mabsadd L
.9127 .28092 -2.37764 .77254 Mabsadd L
.9127 .28092 -0.91448 -0.29713 Mabsadd L
.9127 .28092 -1.46946 -2.02254 Mabsadd L
.9127 .28092 0 -0.96154 Mabsadd L
.9127 .28092 1.46946 -2.02254 Mabsadd L
.9127 .28092 .91448 -0.29713 Mabsadd L
.9127 .28092 2.37764 .77254 Mabsadd L
.9127 .28092 .56518 .7779 Mabsadd L
.9127 .28092 0 2.5 Mabsadd L closepath F
.91847 .28006 0 2.5 Mabsadd m
.91847 .28006 -0.56518 .7779 Mabsadd L
.91847 .28006 -2.37764 .77254 Mabsadd L
.91847 .28006 -0.91448 -0.29713 Mabsadd L
.91847 .28006 -1.46946 -2.02254 Mabsadd L
.91847 .28006 0 -0.96154 Mabsadd L
.91847 .28006 1.46946 -2.02254 Mabsadd L
.91847 .28006 .91448 -0.29713 Mabsadd L
.91847 .28006 2.37764 .77254 Mabsadd L
.91847 .28006 .56518 .7779 Mabsadd L
.91847 .28006 0 2.5 Mabsadd L closepath F
.92424 .28026 0 2.5 Mabsadd m
.92424 .28026 -0.56518 .7779 Mabsadd L
.92424 .28026 -2.37764 .77254 Mabsadd L
.92424 .28026 -0.91448 -0.29713 Mabsadd L
.92424 .28026 -1.46946 -2.02254 Mabsadd L
.92424 .28026 0 -0.96154 Mabsadd L
.92424 .28026 1.46946 -2.02254 Mabsadd L
.92424 .28026 .91448 -0.29713 Mabsadd L
.92424 .28026 2.37764 .77254 Mabsadd L
.92424 .28026 .56518 .7779 Mabsadd L
.92424 .28026 0 2.5 Mabsadd L closepath F
.93001 .28098 0 2.5 Mabsadd m
.93001 .28098 -0.56518 .7779 Mabsadd L
.93001 .28098 -2.37764 .77254 Mabsadd L
.93001 .28098 -0.91448 -0.29713 Mabsadd L
.93001 .28098 -1.46946 -2.02254 Mabsadd L
.93001 .28098 0 -0.96154 Mabsadd L
.93001 .28098 1.46946 -2.02254 Mabsadd L
.93001 .28098 .91448 -0.29713 Mabsadd L
.93001 .28098 2.37764 .77254 Mabsadd L
.93001 .28098 .56518 .7779 Mabsadd L
.93001 .28098 0 2.5 Mabsadd L closepath F
.93579 .28087 0 2.5 Mabsadd m
.93579 .28087 -0.56518 .7779 Mabsadd L
.93579 .28087 -2.37764 .77254 Mabsadd L
.93579 .28087 -0.91448 -0.29713 Mabsadd L
.93579 .28087 -1.46946 -2.02254 Mabsadd L
.93579 .28087 0 -0.96154 Mabsadd L
.93579 .28087 1.46946 -2.02254 Mabsadd L
.93579 .28087 .91448 -0.29713 Mabsadd L
.93579 .28087 2.37764 .77254 Mabsadd L
.93579 .28087 .56518 .7779 Mabsadd L
.93579 .28087 0 2.5 Mabsadd L closepath F
.94156 .27996 0 2.5 Mabsadd m
.94156 .27996 -0.56518 .7779 Mabsadd L
.94156 .27996 -2.37764 .77254 Mabsadd L
.94156 .27996 -0.91448 -0.29713 Mabsadd L
.94156 .27996 -1.46946 -2.02254 Mabsadd L
.94156 .27996 0 -0.96154 Mabsadd L
.94156 .27996 1.46946 -2.02254 Mabsadd L
.94156 .27996 .91448 -0.29713 Mabsadd L
.94156 .27996 2.37764 .77254 Mabsadd L
.94156 .27996 .56518 .7779 Mabsadd L
.94156 .27996 0 2.5 Mabsadd L closepath F
.94733 .28099 0 2.5 Mabsadd m
.94733 .28099 -0.56518 .7779 Mabsadd L
.94733 .28099 -2.37764 .77254 Mabsadd L
.94733 .28099 -0.91448 -0.29713 Mabsadd L
.94733 .28099 -1.46946 -2.02254 Mabsadd L
.94733 .28099 0 -0.96154 Mabsadd L
.94733 .28099 1.46946 -2.02254 Mabsadd L
.94733 .28099 .91448 -0.29713 Mabsadd L
.94733 .28099 2.37764 .77254 Mabsadd L
.94733 .28099 .56518 .7779 Mabsadd L
.94733 .28099 0 2.5 Mabsadd L closepath F
.9531 .28095 0 2.5 Mabsadd m
.9531 .28095 -0.56518 .7779 Mabsadd L
.9531 .28095 -2.37764 .77254 Mabsadd L
.9531 .28095 -0.91448 -0.29713 Mabsadd L
.9531 .28095 -1.46946 -2.02254 Mabsadd L
.9531 .28095 0 -0.96154 Mabsadd L
.9531 .28095 1.46946 -2.02254 Mabsadd L
.9531 .28095 .91448 -0.29713 Mabsadd L
.9531 .28095 2.37764 .77254 Mabsadd L
.9531 .28095 .56518 .7779 Mabsadd L
.9531 .28095 0 2.5 Mabsadd L closepath F
.95887 .27933 0 2.5 Mabsadd m
.95887 .27933 -0.56518 .7779 Mabsadd L
.95887 .27933 -2.37764 .77254 Mabsadd L
.95887 .27933 -0.91448 -0.29713 Mabsadd L
.95887 .27933 -1.46946 -2.02254 Mabsadd L
.95887 .27933 0 -0.96154 Mabsadd L
.95887 .27933 1.46946 -2.02254 Mabsadd L
.95887 .27933 .91448 -0.29713 Mabsadd L
.95887 .27933 2.37764 .77254 Mabsadd L
.95887 .27933 .56518 .7779 Mabsadd L
.95887 .27933 0 2.5 Mabsadd L closepath F
.96465 .27985 0 2.5 Mabsadd m
.96465 .27985 -0.56518 .7779 Mabsadd L
.96465 .27985 -2.37764 .77254 Mabsadd L
.96465 .27985 -0.91448 -0.29713 Mabsadd L
.96465 .27985 -1.46946 -2.02254 Mabsadd L
.96465 .27985 0 -0.96154 Mabsadd L
.96465 .27985 1.46946 -2.02254 Mabsadd L
.96465 .27985 .91448 -0.29713 Mabsadd L
.96465 .27985 2.37764 .77254 Mabsadd L
.96465 .27985 .56518 .7779 Mabsadd L
.96465 .27985 0 2.5 Mabsadd L closepath F
.97042 .28171 0 2.5 Mabsadd m
.97042 .28171 -0.56518 .7779 Mabsadd L
.97042 .28171 -2.37764 .77254 Mabsadd L
.97042 .28171 -0.91448 -0.29713 Mabsadd L
.97042 .28171 -1.46946 -2.02254 Mabsadd L
.97042 .28171 0 -0.96154 Mabsadd L
.97042 .28171 1.46946 -2.02254 Mabsadd L
.97042 .28171 .91448 -0.29713 Mabsadd L
.97042 .28171 2.37764 .77254 Mabsadd L
.97042 .28171 .56518 .7779 Mabsadd L
.97042 .28171 0 2.5 Mabsadd L closepath F
.97619 .28077 0 2.5 Mabsadd m
.97619 .28077 -0.56518 .7779 Mabsadd L
.97619 .28077 -2.37764 .77254 Mabsadd L
.97619 .28077 -0.91448 -0.29713 Mabsadd L
.97619 .28077 -1.46946 -2.02254 Mabsadd L
.97619 .28077 0 -0.96154 Mabsadd L
.97619 .28077 1.46946 -2.02254 Mabsadd L
.97619 .28077 .91448 -0.29713 Mabsadd L
.97619 .28077 2.37764 .77254 Mabsadd L
.97619 .28077 .56518 .7779 Mabsadd L
.97619 .28077 0 2.5 Mabsadd L closepath F
0 0 1 r
.005 w
.02381 .24942 m
.02958 .25339 L
.03535 .25339 L
.04113 .16599 L
.0469 .25207 L
.05267 .25163 L
.05844 .25295 L
.06421 .25075 L
.06999 .24942 L
.07576 .25251 L
.08153 .25163 L
.0873 .25119 L
.09307 .25207 L
.09885 .25075 L
.10462 .24103 L
.11039 .25163 L
.11616 .2481 L
.12193 .25075 L
.12771 .2503 L
.13348 .25075 L
.13925 .25075 L
.14502 .25251 L
.15079 .24942 L
.15657 .12096 L
.16234 .20263 L
.16811 .22205 L
.17388 .19689 L
.17965 .23044 L
.18543 .19865 L
.1912 .22691 L
.19697 .21719 L
.20274 .23574 L
.20851 .22911 L
.21429 .21013 L
.22006 .2322 L
.22583 .15672 L
.2316 .21631 L
.23737 .20351 L
.24315 .19291 L
.24892 .21499 L
.25469 .11963 L
.26046 .16157 L
.26623 .17923 L
.27201 .09138 L
.27778 .13244 L
.28355 .05783 L
.28932 .07372 L
.29509 .14656 L
.30087 .16245 L
.30664 .16731 L
Mistroke
.31241 .17526 L
.31818 .19115 L
.32395 .12228 L
.32973 .08652 L
.3355 .07725 L
.34127 .03796 L
.34704 .14568 L
.35281 .11963 L
.35859 .15936 L
.36436 .15142 L
.37013 .12537 L
.3759 .19071 L
.38167 .18409 L
.38745 .15672 L
.39322 .20483 L
.39899 .17923 L
.40476 .18629 L
.41053 .17305 L
.41631 .2141 L
.42208 .16466 L
.42785 .18055 L
.43362 .18982 L
.43939 .22426 L
.44517 .19512 L
.45094 .19954 L
.45671 .19865 L
.46248 .2013 L
.46825 .21366 L
.47403 .21719 L
.4798 .22911 L
.48557 .22249 L
.49134 .21631 L
.49711 .22735 L
.50289 .22602 L
.50866 .21764 L
.51443 .23176 L
.5202 .21764 L
.52597 .23971 L
.53175 .20837 L
.53752 .23176 L
.54329 .22117 L
.54906 .23485 L
.55483 .23662 L
.56061 .22293 L
.56638 .2322 L
.57215 .23618 L
.57792 .22028 L
.58369 .23 L
.58947 .24059 L
.59524 .23883 L
Mistroke
.60101 .24456 L
.60678 .22779 L
.61255 .23838 L
.61833 .24412 L
.6241 .24059 L
.62987 .23927 L
.63564 .24236 L
.64141 .24015 L
.64719 .24677 L
.65296 .24633 L
.65873 .24412 L
.6645 .24677 L
.67027 .24324 L
.67605 .24766 L
.68182 .24059 L
.68759 .2428 L
.69336 .24721 L
.69913 .2375 L
.70491 .2481 L
.71068 .24898 L
.71645 .2503 L
.72222 .24324 L
.72799 .24942 L
.74531 .24986 L
.75108 .24677 L
.75685 .21984 L
.76263 .24192 L
.7684 .24501 L
.77417 .22338 L
.77994 .2503 L
.78571 .24986 L
.79149 .25251 L
.79726 .24854 L
.80303 .24545 L
.8088 .24103 L
.81457 .24942 L
.82035 .24766 L
.82612 .24192 L
.83189 .24854 L
.83766 .24412 L
.84343 .24456 L
.84921 .24986 L
.85498 .25163 L
.86075 .25163 L
.86652 .25163 L
.87229 .24986 L
.87807 .25163 L
.88384 .25075 L
.88961 .22514 L
.89538 .25207 L
Mistroke
.90115 .24677 L
.90693 .24324 L
.9127 .24545 L
.91847 .24368 L
.92424 .24721 L
.93001 .24721 L
.93579 .25163 L
.94156 .24854 L
.94733 .24898 L
.9531 .24986 L
.95887 .24854 L
.96465 .24854 L
.97042 .25472 L
.97619 .24898 L
Mfstroke
.02381 .3152 m
.02958 .31299 L
.03535 .31564 L
.04113 .41276 L
.0469 .31476 L
.05267 .31034 L
.05844 .31167 L
.06421 .31343 L
.06999 .31122 L
.07576 .31696 L
.08153 .31255 L
.0873 .31211 L
.09307 .3152 L
.09885 .3174 L
.10462 .31564 L
.11039 .31608 L
.11616 .31608 L
.12193 .31343 L
.12771 .31564 L
.13348 .3152 L
.13925 .31078 L
.14502 .31343 L
.15079 .31431 L
.15657 .54828 L
.16234 .38097 L
.16811 .34786 L
.17388 .33771 L
.17965 .34742 L
.18543 .34654 L
.1912 .36199 L
.19697 .3355 L
.20274 .34345 L
.20851 .33859 L
.21429 .34919 L
.22006 .33241 L
.22583 .33948 L
.2316 .40481 L
.23737 .39378 L
.24315 .377 L
.24892 .35846 L
.25469 .41276 L
.26046 .45205 L
.26623 .4569 L
.27201 .44454 L
.27778 .45911 L
.28355 .52621 L
.28932 .46088 L
.29509 .42777 L
.30087 .39466 L
.30664 .38539 L
Mistroke
.31241 .39554 L
.31818 .39598 L
.32395 .4569 L
.32973 .4207 L
.3355 .47721 L
.34127 .509 L
.34704 .41188 L
.35281 .42335 L
.35859 .4388 L
.36436 .41541 L
.37013 .45249 L
.3759 .36685 L
.38167 .38936 L
.38745 .41761 L
.39322 .35581 L
.39899 .37435 L
.40476 .37965 L
.41053 .39378 L
.41631 .34654 L
.42208 .39775 L
.42785 .377 L
.43362 .38583 L
.43939 .3408 L
.44517 .37612 L
.45094 .36243 L
.45671 .37435 L
.46248 .35934 L
.46825 .34786 L
.47403 .34786 L
.4798 .33506 L
.48557 .3333 L
.49134 .33992 L
.49711 .33374 L
.50289 .3355 L
.50866 .34301 L
.51443 .32932 L
.5202 .33771 L
.52597 .32314 L
.53175 .34919 L
.53752 .33727 L
.54329 .33904 L
.54906 .32579 L
.55483 .32138 L
.56061 .33065 L
.56638 .33109 L
.57215 .32623 L
.57792 .33109 L
.58369 .33021 L
.58947 .3174 L
.59524 .32138 L
Mistroke
.60101 .32226 L
.60678 .32977 L
.61255 .32403 L
.61833 .32491 L
.6241 .32358 L
.62987 .3227 L
.63564 .32226 L
.64141 .31961 L
.64719 .31476 L
.65296 .31387 L
.65873 .32049 L
.6645 .3152 L
.67027 .3174 L
.67605 .31917 L
.68182 .31829 L
.68759 .32182 L
.69336 .31343 L
.69913 .31917 L
.70491 .31078 L
.71068 .3152 L
.71645 .31167 L
.72222 .31785 L
.72799 .31167 L
.74531 .31167 L
.75108 .31829 L
.75685 .33418 L
.76263 .31652 L
.7684 .31785 L
.77417 .33462 L
.77994 .31961 L
.78571 .31078 L
.79149 .31476 L
.79726 .31431 L
.80303 .32049 L
.8088 .3174 L
.81457 .31343 L
.82035 .31476 L
.82612 .31476 L
.83189 .31034 L
.83766 .33065 L
.84343 .31652 L
.84921 .31211 L
.85498 .31343 L
.86075 .30902 L
.86652 .31564 L
.87229 .31255 L
.87807 .31255 L
.88384 .31696 L
.88961 .33948 L
.89538 .30902 L
Mistroke
.90115 .31122 L
.90693 .32623 L
.9127 .32226 L
.91847 .31211 L
.92424 .31255 L
.93001 .3152 L
.93579 .31211 L
.94156 .31387 L
.94733 .31431 L
.9531 .31785 L
.95887 .31078 L
.96465 .31122 L
.97042 .31564 L
.97619 .31078 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{364.625, 225.25},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40008R0001DA000`40O003h00Oogooogoo97oo003o
OoooOolTOol00?mooomoobAoo`00ogooogoo97oo003oOoooOolTOol00?mooomoobAoo`00ogooogoo
97oo003oOoooOolTOol00?mooomoobAoo`00ogooogoo97oo003oOoooOolTOol00?mooomoobAoo`00
ogooogoo97oo003oOoooOolTOol00?mooomoobAoo`00ogooogoo97oo003oOoooOolTOol00?mooomo
obAoo`00ogooogoo97oo001<Ool50010Ool90004Ool50006Ool50006Ool5000eOol20006Ool50006
Ool50006Ool5000bOol40006Ool50006Ool50006Ool5000bOol50006Ool50006Ool50006Ool5000^
Ool20007Ool50006Ool50006Ool50006Ool50005Ool004]oo`L003moo`T000=oo`L000Aoo`L000Ao
o`L003Aoo`8000Eoo`L000Aoo`L000Aoo`L002moo`L000Aoo`L000Aoo`L000Aoo`L0031oo`L000Ao
o`L000Aoo`L000Aoo`L002eoo`8000Ioo`L000Aoo`L000Aoo`L000Aoo`L000Aoo`00Bgoo0P000goo
0P00@7oo0P002Goo0P000goo0P0017oo0P000goo0P0017oo0P000goo0P00=7oo0P001Goo0P000goo
0P0017oo0P000goo0P0017oo0P000goo0P00;goo0P000goo0P0017oo0P000goo0P0017oo0P000goo
0P0017oo0P000goo0P00;goo0`000goo0`000goo0P000goo0P0017oo0P000goo0P0017oo0P000goo
0P00;Goo0P001Woo0P000goo0P0017oo0P000goo0P0017oo0P000goo0P0017oo0P000goo0P0017oo
001:Ool20005Ool20010Ool20007Ool20005Ool20002Ool20005Ool20002Ool20005Ool2000]Ool:
0002Ool20005Ool20002Ool20005Ool20002Ool20005Ool2000]Ool20005Ool20002Ool20005Ool2
0002Ool20005Ool20002Ool20005Ool2000^Ool20005Ool20002Ool20005Ool20002Ool20005Ool2
0002Ool20005Ool2000/Ool20005Ool20005Ool20002Ool20005Ool20002Ool20005Ool20002Ool2
0005Ool20003Ool004Yoo`8000Eoo`80045oo`8000Ioo`8000Eoo`80009oo`8000Eoo`80009oo`80
00Eoo`8002eoo`X0009oo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8002eoo`8000Eoo`80
009oo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8002ioo`8000Eoo`80009oo`8000Eoo`80
009oo`8000Eoo`80009oo`8000Eoo`8002aoo`8000Eoo`8000Eoo`80009oo`8000Eoo`80009oo`80
00Eoo`80009oo`8000Eoo`8000=oo`00BWoo0P001Goo0P00@Woo0P001Goo0P001Goo0P000Woo0P00
1Goo0P000Woo0P001Goo0P00;Goo0P0017oo0P0017oo0P001Goo0P000Woo0P001Goo0P000Woo0P00
1Goo0P00;Goo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P00;Woo0P00
1Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P00;7oo0P001Goo0P001Goo0P00
0Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000goo001:Ool20005Ool20013Ool20004
Ool20005Ool20002Ool20005Ool20002Ool20005Ool2000^Ool20003Ool20004Ool20005Ool20002
Ool20005Ool20002Ool20005Ool2000]Ool20005Ool20002Ool20005Ool20002Ool20005Ool20002
Ool20005Ool2000^Ool30003Ool30002Ool20005Ool20002Ool20005Ool20002Ool20005Ool2000/
Ool20005Ool20005Ool20002Ool20005Ool20002Ool20005Ool20002Ool20005Ool20003Ool004Yo
o`8000Eoo`8004Aoo`8000=oo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8002moo`80009o
o`8000Aoo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8002eoo`<000=oo`<0009oo`8000Eo
o`80009oo`8000Eoo`80009oo`8000Eoo`8002moo`L000=oo`8000Eoo`80009oo`8000Eoo`80009o
o`8000Eoo`8002aoo`8000Eoo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eo
o`8000=oo`00BWoo0P001Goo0P00AGoo00@007ooOomoo`8000Eoo`80009oo`8000Eoo`80009oo`80
00Eoo`8002moo`80009oo`8000Aoo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8002eoo`P0
00=oo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8002moo`L000=oo`8000Eoo`80009oo`80
00Eoo`80009oo`8000Eoo`8002aoo`8000Eoo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`80
009oo`8000Eoo`8000=oo`00BWoo0P001Goo0P00AGoo0P000Woo0P001Goo0P000Woo0P001Goo0P00
0Woo0P001Goo0P00<7oo0P0000=oo`00000017oo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo
0P00;Goo0P0000=oo`0000000P0017oo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P00;Woo
0`000goo0`000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P00;7oo0P001Goo0P001Goo
0P000Woo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P000goo001:Ool20005Ool20015Ool2
0002Ool20005Ool20002Ool20005Ool20002Ool20005Ool2000aOol40004Ool20005Ool20002Ool2
0005Ool20002Ool20005Ool2000]Ool20009Ool20005Ool20002Ool20005Ool20002Ool20005Ool2
000^Ool20005Ool20002Ool20005Ool20002Ool20005Ool20002Ool20005Ool2000YOol200000goo
00000005Ool20005Ool20002Ool20005Ool20002Ool20005Ool20002Ool20005Ool20003Ool004Yo
o`8000Eoo`8003ioo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`80039o
o`<000Aoo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8002ioo`8000Aoo`80009oo`8000Eo
o`80009oo`8000Eoo`80009oo`8000Eoo`8002ioo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eo
o`80009oo`8000Eoo`8002Uoo`D000Eoo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`80009o
o`8000Eoo`8000=oo`00Bgoo0P000goo0P00@7oo0P0017oo0P000goo0P000goo0P0017oo0P000goo
0P0017oo0P000goo0P00<goo0`001Goo0P000goo0P0017oo0P000goo0P0017oo0P000goo0P00;goo
0P000goo0`000goo0P000goo0P0017oo0P000goo0P0017oo0P000goo0P00;goo0`000goo0`000goo
0P000goo0P0017oo0P000goo0P0017oo0P000goo0P00;7oo0`001Woo0P000goo0P0017oo0P000goo
0P0017oo0P000goo0P0017oo0P000goo0P0017oo001;Ool70010Ool70004Ool70004Ool70004Ool7
000dOol20005Ool70004Ool70004Ool7000`Ool60004Ool70004Ool70004Ool7000`Ool70004Ool7
0004Ool70004Ool7000]Ool20006Ool70004Ool70004Ool70004Ool70004Ool004aoo`D0049oo`D0
00Ioo`D000Ioo`D000Ioo`D003Ioo`03001oogoo00Aoo`D000Ioo`D000Ioo`D0039oo`@000Ioo`D0
00Ioo`D000Ioo`D0039oo`D000Ioo`D000Ioo`D000Ioo`D002ioo`8000Moo`D000Ioo`D000Ioo`D0
00Ioo`D000Eoo`00ogooogoo97oo003oOoooOolTOol00?mooomoobAoo`00ogooogoo97oo003oOooo
OolTOol00?mooomoobAoo`00ogooogoo97oo003oOoooOolTOol00?mooomoobAoo`00A7ooo`00b000
5goo0014Ool00`00Oomoo`08Ool00`00Oomoo`0BOol00`00Oomoo`0COol00`00Oomoo`0COol00`00
Oomoo`0COol00`00Oomoo`0COol00`00Oomoo`0COol00`00Oomoo`0COol00`00Oomoo`0BOol00`00
Oomoo`0COol00`00Oomoo`0COol00`00Oomoo`0COol00`00Oomoo`0COol00`00Oomoo`0COol00`00
Oomoo`0BOol00`00Oomoo`0COol00`00Oomoo`0COol00`00Oomoo`0COol00`00Oomoo`0COol00`00
Oomoo`0COol00`00Oomoo`0BOol00`00Oomoo`04Ool00`00Oomoo`0EOol004Aoo`03001oogoo00Qo
o`03001oogoo019oo`03001oogoo01=oo`03001oogoo01=oo`03001oogoo01=oo`03001oogoo01=o
o`03001oogoo01=oo`03001oogoo01=oo`03001oogoo019oo`03001oogoo01=oo`03001oogoo01=o
o`03001oogoo01=oo`03001oogoo01=oo`03001oogoo01=oo`03001oogoo019oo`03001oogoo01=o
o`03001oogoo01=oo`03001oogoo01=oo`03001oogoo01=oo`03001oogoo01=oo`03001oogoo019o
o`03001oogoo00Aoo`03001oogoo01Eoo`00A7oo00<007ooOol027oo00<007ooOol0E7oo00<007oo
Ool0E7oo00<007ooOol0EGoo00<007ooOol0E7oo00<007ooOol0E7oo00<007ooOol017oo00<007oo
Ool05Goo0014Ool00`00Oomoo`1OOol00`0?Oomoo`1DOol00`0?Oomoo`1EOol00`0?Oomoo`1DOol0
0`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001oogoo05moo`0300moogoo05Aoo`0300moogoo
05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00A7oo00<007ooOol0Ggoo00<0
3gooOol0E7oo00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014
Ool00`00Oomoo`1OOol00`0?Oomoo`1DOol00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1K
Ool00`00Oomoo`0EOol004Aoo`03001oogoo05moo`0300moogoo05Aoo`0300moogoo05Eoo`0300mo
ogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00A7oo00<007ooOol0Ggoo00<03gooOol0E7oo
00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool3001OOol0
0`0?Oomoo`1DOol00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1IOol3000GOol004Aoo`03
001oogoo05moo`0300moogoo05Aoo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03
001oogoo01Eoo`00A7oo00<007ooOol0Ggoo00<03gooOol0E7oo00<03gooOol0EGoo00<03gooOol0
E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00Oomoo`1OOol00`0?Oomoo`1DOol00`0?
Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol001Eoo`8000Yoo`8000Io
o`D000Ioo`D000]oo`03001oogoo05moo`0300moogoo05Aoo`0300moogoo05Eoo`0300moogoo05Ao
o`0300moogoo05]oo`03001oogoo01Eoo`005Goo0P002Woo0P001Goo1`0017oo1`002Woo00<007oo
Ool0Ggoo00<03gooOol0E7oo00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0Fgoo00<007oo
Ool05Goo000EOol2000:Ool20005Ool20003Ool20004Ool20003Ool2000:Ool00`00Oomoo`1OOol0
0`0?Oomoo`1DOol00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol0
01Eoo`8000Aoo`X0009oo`8000Eoo`80009oo`8000Eoo`8000Uoo`03001oogoo05moo`0300moogoo
03Eoo`807aeoo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00
17oo2P001goo0P0017oo2P000Woo0P001Goo0P000Woo0P001Goo0P002Goo00<007ooOol0Ggoo00<0
3gooOol0=Goo0P0O7Goo00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol0
5Goo000EOol20004Ool20004Ool20004Ool20005Ool20002Ool20005Ool20009Ool00`00Oomoo`1O
Ool00`0?Oomoo`0eOol201lMOol00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00
Oomoo`0EOol001Eoo`8000Eoo`8000=oo`8000Aoo`8000Eoo`80009oo`8000Eoo`8000Uoo`@009D0
3`<07ol03bP03`@001Moo`005Goo0P001Woo0P000Woo0P0017oo0P001Goo0P000Woo0P001Goo0P00
2Goo00<007ooOol0Ggoo00<03gooOol0=7oo0`0O7Goo00<03gooOol0EGoo00<03gooOol0E7oo00<0
3gooOol0Fgoo00<007ooOol05Goo000EOol20006Ool20002Ool20004Ool20005Ool20002Ool20005
Ool20009Ool00`00Oomoo`1OOol00`0?Oomoo`0dOol301lMOol00`0?Oomoo`1EOol00`0?Oomoo`1D
Ool00`0?Oomoo`1KOol00`00Oomoo`0EOol001Eoo`8000Moo`800003Ool0000000Aoo`8000Eoo`80
009oo`8000Eoo`8000Uoo`03001oogoo05moo`0300moogoo03Aoo`<07aeoo`0300moogoo05Eoo`03
00moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`004Woo0P0000=oo`00000027oo100017oo
0P001Goo0P000Woo0P001Goo0P002Goo00<007ooOol0Ggoo00<03gooOol0=7oo0`0O7Goo00<03goo
Ool0EGoo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo000BOol50009Ool30004Ool2
0005Ool20002Ool20005Ool20009Ool00`00Oomoo`1OOol00`0?Oomoo`0dOol301lMOol00`0?Oomo
o`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol001Aoo`<000Uoo`<000Eoo`80
00=oo`8000Aoo`8000=oo`8000Yoo`03001oogoo05moo`0300moogoo01]oo`807aMoo`<07aeoo`03
00moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`005Goo0P002Woo0P00
1Goo1`0017oo1`002Woo00<007ooOol0Ggoo00<03gooOol06goo0`0O5Goo100O7Goo00<03gooOol0
EGoo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo000EOol2000;Ool00`00Oomoo`04
Ool50006Ool5000;Ool00`00Oomoo`1OOol00`0?Oomoo`0KOol301lEOol401lMOol00`0?Oomoo`1E
Ool00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001oogoo05moo`0300mo
ogoo01]oo`<07aEoo`807`03Ool07`0O01aoo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo
05]oo`03001oogoo01Eoo`00A7oo0`00Ggoo00<03gooOol06goo0`0O5Goo0P0O00=oo`0O01l077oo
00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0FGoo0`005goo0014Ool00`00Oomoo`1OOol0
0`0?Oomoo`0KOol401lDOol201l00goo01l07`0LOol00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?
Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001oogoo05moo`0300moogoo01Yoo`807`03Ool07`0O
01Aoo`807`03Ool07`0O01aoo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03001o
ogoo01Eoo`00A7oo00<007ooOol0Ggoo00<03gooOol06Woo0P0O00=oo`0O01l04goo0P0O0Woo0P0O
77oo00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00
Oomoo`1OOol00`0?Oomoo`0JOol201l00goo01l07`0COol201l2Ool201lLOol00`0?Oomoo`1EOol0
0`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001oogoo05moo`0300moogoo
01Yoo`807`03Ool07`0O01=oo`807`9oo`807aaoo`0300moogoo05Eoo`0300moogoo05Aoo`0300mo
ogoo05]oo`03001oogoo01Eoo`00A7oo00<007ooOol0Ggoo00<03gooOol06Woo0P0O00=oo`0O01l0
4goo0P0O0Woo0P0O77oo00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol0
5Goo0014Ool00`00Oomoo`1OOol00`0?Oomoo`0JOol201l00goo01l07`0BOol201l3Ool201lLOol0
0`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001oogoo
05moo`0300moogoo01Yoo`807`9oo`807a5oo`0301moogoo009oo`807aaoo`0300moogoo05Eoo`03
00moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00A7oo00<007ooOol0Ggoo00<03gooOol0
6Woo0P0O0Woo0P0O47oo0P0O17oo0P0O77oo00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0
Fgoo00<007ooOol05Goo0014Ool3001OOol00`0?Oomoo`0JOol201l2Ool201l@Ool201l4Ool201lL
Ool00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1IOol3000GOol004Aoo`03001oogoo05mo
o`0300moogoo01Eoo`807`=oo`807`9oo`807a1oo`807`Aoo`807aaoo`0300moogoo05Eoo`0300mo
ogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00A7oo00<007ooOol0Ggoo00<03gooOol05Goo
0P0O0goo0P0O0Woo0P0O3goo0P0O1Goo0P0O77oo00<03gooOol0EGoo00<03gooOol0E7oo00<03goo
Ool0Fgoo00<007ooOol05Goo0014Ool00`00Oomoo`1OOol00`0?Oomoo`0EOol201l3Ool201l2Ool2
01l?Ool201l5Ool201lLOol00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomo
o`0EOol004Aoo`03001oogoo05moo`0300moogoo01Eoo`<07`03Ool07`0O00=oo`807`moo`807`Eo
o`807aaoo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00A7oo
00<007ooOol0Ggoo00<03gooOol05Goo0`0O00=oo`0O01l00goo0P0O3goo0P0O1Goo0P0O77oo00<0
3gooOol0EGoo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00Oomoo`1O
Ool00`0?Oomoo`0EOol301l00goo01l07`03Ool201l?Ool201l5Ool201lLOol00`0?Oomoo`1EOol0
0`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001oogoo05moo`0300moogoo
01Eoo`<07`03Ool07`0O00=oo`807`moo`807`Eoo`807aaoo`0300moogoo05Eoo`0300moogoo05Ao
o`0300moogoo05]oo`03001oogoo01Eoo`00A7oo00<007ooOol0Ggoo00<03gooOol05Goo0`0O00=o
o`0O01l00goo0P0O3Woo0P0O1Woo0P0O77oo00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0
Fgoo00<007ooOol05Goo0014Ool00`00Oomoo`1OOol00`0?Oomoo`0EOol301l00goo01l07`03Ool2
01l>Ool201l6Ool201lLOol00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomo
o`0EOol004Aoo`<005moo`0300moogoo01Eoo`<07`03Ool07`0O00Aoo`807`eoo`807`Ioo`807aao
o`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo05Uoo`<001Moo`00A7oo00<007ooOol0Ggoo
00<03gooOol057oo0P0O00=oo`0O01l00P0O17oo0P0O3Goo0P0O1Woo0P0O77oo00<03gooOol0EGoo
00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00Oomoo`1OOol00`0?Oomo
o`0DOol201l00goo01l07`0201l4Ool201l=Ool201l6Ool201lLOol00`0?Oomoo`1EOol00`0?Oomo
o`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001oogoo05moo`0300moogoo01Aoo`80
7`03Ool07`0O00807`Aoo`807`eoo`807`Ioo`807`9oo`807aQoo`0300moogoo05Eoo`0300moogoo
05Aoo`0300moogoo05]oo`03001oogoo01Eoo`005Goo0P001Goo2@0017oo1@001Woo1@002goo00<0
07ooOol0Ggoo00<03gooOol03Goo0P0O1Goo0P0O00=oo`0O01l00P0O17oo0P0O37oo0P0O1goo0P0O
0Woo0P0O67oo00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo000E
Ool20005Ool90003Ool70004Ool7000:Ool00`00Oomoo`13Ool201lJOol00`0?Oomoo`0=Ool201l5
Ool201l00goo01l07`0201l4Ool201l<Ool201l8Ool501lHOol00`0?Oomoo`1EOol00`0?Oomoo`1D
Ool00`0?Oomoo`1KOol00`00Oomoo`0EOol001Eoo`8000Ioo`8000Uoo`8000=oo`8000Aoo`8000=o
o`8000Yoo`03001oogoo04=oo`807aYoo`0300moogoo00eoo`807`Eoo`807`04Ool07`0O01l5Ool2
01l<Ool201l8Ool501l6Ool201l@Ool00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol0
0`00Oomoo`0EOol001Eoo`8000Moo`8000Moo`8000Eoo`80009oo`8000Eoo`8000Uoo`03001oogoo
04=oo`807aYoo`0300moogoo00eoo`<07`Aoo`807`04Ool07`0O01l5Ool201l<Ool201l8Ool601l5
Ool201l@Ool00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol000Ao
o`X000Moo`8000Qoo`8000Ioo`8000Eoo`80009oo`8000Eoo`8000Uoo`03001oogoo04=oo`807aYo
o`0300moogoo00eoo`<07`Aoo`807`9oo`807`Eoo`807`aoo`807`Qoo`H07`Aoo`<07a1oo`0300mo
ogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`005Goo0P002Goo0P001Goo
0P001Goo0P000Woo0P001Goo0P002Goo00<007ooOol0@goo0P0O6Woo00<03gooOol03Goo0`0O17oo
0P0O0Woo0P0O1@000P0O37oo0P0O27oo1P0O17oo0`0O47oo00<03gooOol0EGoo00<03gooOol0E7oo
00<03gooOol0Fgoo00<007ooOol05Goo000EOol2000:Ool20004Ool20005Ool20002Ool20005Ool2
0009Ool4001200l201lZ00l301l400l201l200l201l200l00`0000l03`0201l;00l201l900l601l4
00l301oo00lK00l4000GOol001Eoo`8000]oo`8000=oo`8000Eoo`80009oo`8000Eoo`8000Uoo`03
001oogoo04=oo`<07aUoo`0300moogoo00eoo`<07`Aoo`807`9oo`807`9oo`03001oogoo00807`]o
o`807`Uoo`<07`03Ool07`0O00Aoo`<07a1oo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo
05]oo`03001oogoo01Eoo`005Goo0P0037oo00@007ooOomoo`8000Eoo`80009oo`8000Eoo`8000Uo
o`03001oogoo04=oo`<07aUoo`0300moogoo00eoo`<07`Aoo`807`Ioo`04001oogooOol201l:Ool2
01l9Ool301l00goo01l07`04Ool401l?Ool00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1K
Ool00`00Oomoo`0EOol001Eoo`8000aoo`80009oo`8000Eoo`80009oo`8000Eoo`8000Uoo`03001o
ogoo04=oo`<07aUoo`0300moogoo00eoo`<07`Aoo`807`Ioo`04001oogooOol201l:Ool201l9Ool3
01l2Ool201l2Ool501l?Ool00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomo
o`0EOol0019oo`800003Ool0000000aoo`80009oo`8000Eoo`80009oo`8000Eoo`8000Uoo`03001o
ogoo04=oo`<07aUoo`0300moogoo00eoo`@07`=oo`807`Ioo`04001oogooOol201l:Ool201l9Ool3
01l2Ool201l2Ool201l00goo01l07`0?Ool00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1K
Ool00`00Oomoo`0EOol0019oo`D000Eoo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8000Uo
o`03001oogoo049oo`@07aUoo`0300moogoo00aoo`807`03Ool07`0O00=oo`807`Ioo`04001oogoo
Ool201l:Ool201l9Ool201l3Ool201l2Ool201l00goo01l07`0?Ool00`0?Oomoo`1EOol00`0?Oomo
o`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol001Aoo`<000Ioo`8000Aoo`8000=oo`8000=oo`80
00Aoo`8000=oo`8000Yoo`03001oogoo049oo`@07aUoo`0300moogoo00aoo`807`03Ool07`0O00=o
o`807`Ioo`04001oogooOol201l:Ool201l9Ool201l3Ool201l2Ool201l00goo01l07`0?Ool00`0?
Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol001Eoo`8000Ioo`L000Ao
o`L000Aoo`L000Yoo`03001oogoo049oo`@07aUoo`0300moogoo00aoo`807`03Ool07`0O00=oo`80
7`Ioo`04001oogooOol301l9Ool201l9Ool201l3Ool201l01Goo01l07`0OOol00P0O3goo00<03goo
Ool0EGoo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo000EOol20007Ool50006Ool5
0006Ool5000;Ool00`00Oomoo`12Ool401lIOol00`0?Oomoo`0<Ool201l00goo01l07`03Ool201l6
Ool00`00Oomoo`02Ool201l9Ool201l>Ool201l00goo01l07`02Ool201l?Ool00`0?Oomoo`1EOol0
0`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001oogoo049oo`@07aUoo`03
00moogoo00aoo`807`03Ool07`0O00=oo`807`Ioo`03001oogoo009oo`807`Uoo`807`ioo`807`03
Ool07`0O009oo`807`moo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo
01Eoo`00A7oo0`00@Woo100O6Goo00<03gooOol037oo0P0O00=oo`0O01l00goo0P0O1Woo00<007oo
Ool00Woo0P0O2Goo0P0O3goo0`0O0goo0P0O1Goo0P0O27oo00<03gooOol0EGoo00<03gooOol0E7oo
00<03gooOol0FGoo0`005goo0014Ool00`00Oomoo`12Ool401lIOol00`0?Oomoo`0201l:Ool201l2
Ool201l2Ool201l6Ool00`00Oomoo`02Ool201l9Ool201l?Ool301l3Ool201l5Ool201l8Ool00`0?
Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001oogoo049o
o`@07aUoo`0300moogoo00807`Yoo`807`9oo`807`03Ool07`0O00Moo`03001oogoo009oo`<07`Mo
o`807a1oo`807`Eoo`807`=oo`<07`Qoo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]o
o`03001oogoo01Eoo`00A7oo00<007ooOol0@Woo100O6Goo00<03gooOol00P0O2Woo0P0O0Woo0P0O
00=oo`0O01l01goo00<007ooOol00goo0P0O1goo0P0O47oo0P0O1Goo0P0O0goo0`0O27oo00<03goo
Ool0EGoo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00Oomoo`12Ool2
01l00goo01l07`0HOol00`0?Oomoo`0201l:Ool201l2Ool501l7Ool00`00Oomoo`03Ool301l6Ool2
01l7Ool5000;Ool201l3Ool301l8Ool00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol0
0`00Oomoo`0EOol004Aoo`03001oogoo00ioo`807c9oo`807`03Ool07`0O01Qoo`0300moogoo00<0
7`Uoo`807`=oo`@07`Moo`03001oogoo00Aoo`@07`Aoo`807`Uoo`03001oogoo00]oo`807`=oo`<0
7`Qoo`0300moogoo009oo`807e5oo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00
A7oo00<007ooOol03Woo0P0O<Woo0P0O00=oo`0O01l067oo00<03gooOol00`0O2Goo0P0O0goo100O
1goo00<007ooOol01Goo100O0goo0P0O2Goo00<007ooOol02goo0P0O0goo100O1goo00<03gooOol0
0Woo0`0OD7oo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00Oomoo`0>
Ool201lbOol201l00goo01l07`0HOol00`0?Oomoo`0301l9Ool201l3Ool401l7Ool00`00Oomoo`06
Ool301l3Ool201l9Ool00`00Oomoo`0;Ool201l2Ool501l7Ool00`0?Oomoo`02Ool301m@Ool00`0?
Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001oogoo00ioo`807c9oo`807`03
Ool07`0O01Qoo`0300moogoo00<07`Uoo`807`Aoo`<07`Moo`03001oogoo00Qoo`<07`03Ool07`0O
00Uoo`03001oogoo00]oo`807`9oo`807`03Ool07`0O00Moo`807`=oo`@07dmoo`0300moogoo05Ao
o`0300moogoo05]oo`03001oogoo01Eoo`00A7oo00<007ooOol03Woo0P0O<Woo0P0O00=oo`0O01l0
67oo00<03gooOol00`0O2Goo0P0O17oo0`0O1goo00<007ooOol027oo0`0O00=oo`0O01l02Goo00<0
07ooOol02goo0P0O0Woo0P0O00=oo`0O01l01Woo0`0O0goo100OCgoo00<03gooOol0E7oo00<03goo
Ool0Fgoo00<007ooOol05Goo0014Ool3000>Ool201lbOol201l00goo01l07`0HOol00`0?Ool07`03
01l9Ool201l4Ool301l7Ool00`00Oomoo`09Ool501l4Ool6000=Ool201l2Ool201l00goo01l07`02
Ool201l2Ool301l3Ool401m?Ool00`0?Oomoo`1DOol00`0?Oomoo`1IOol3000GOol004Aoo`03001o
ogoo00ioo`<07c5oo`807`03Ool07`0O01Qoo`0300moo`0O00<07`Uoo`807`Eoo`807`Moo`03001o
ogoo00Yoo`@07`Ioo`04001oogoo000=Ool201l01Goo01l07`0OOol00P0O0Woo0`0O00=oo`0O01l0
0P0O00Aoo`0O01moo`<07dioo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00A7oo
00<007ooOol03Woo0`0O<Goo0P0O00=oo`0O01l067oo00<03goo01l00`0O2Goo0P0O1Goo0P0O1goo
00<007ooOol02Woo100O1Woo00@007ooOol000ioo`@07`9oo`807`03Ool07`0O00807`03Ool07`0O
00807`03Ool07`0O009oo`807dioo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00
A7oo00<007ooOol03Woo0`0O<Goo0P0O00=oo`0O01l067oo00<03goo01l00`0O2Goo0P0O3Woo00<0
07ooOol02Woo0`0O1goo00@007ooOol000ioo`@07`9oo`807`04Ool07`0OOol601l00goo01l07`03
Ool201m=Ool00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001oogoo00io
o`<07c5oo`807`03Ool07`0O01Qoo`0300moo`0O00<07`Uoo`807`Ioo`H0009oo`03001oogoo00Yo
o`<07`Moo`04001oogoo000>Ool301l3Ool201l017oo01l07goo0`0O00@03`0O01moo`807`Aoo`03
01moogoo04]oo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00A7oo00<007ooOol0
3Goo100O<7oo0P0O0Woo0P0O67oo00D03goo01l07goo00807`Qoo`807`Qoo`03001oogoo00=oo`03
001oogoo00Yoo`<07`Moo`04001oogoo000>Ool301l3Ool201l00goo01l07`02Ool201l0100?01l0
7goo0P0O17oo0P0OC7oo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00
Oomoo`0=Ool401l`Ool201l2Ool201lHOol01@0?Ool07`0OOol00P0O27oo0P0O27oo00<007ooOol0
0goo00<007ooOol02goo0P0O1goo00@007ooOol000ioo`807`Aoo`807`03Ool07`0O00Aoo`0400l0
7`0OOol201l5Ool201m;Ool00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03
001oogoo00eoo`@07c1oo`807`=oo`807aMoo`0500moo`0O01moo`0201l4Ool201l00goo01l07`09
Ool00`00Oomoo`03Ool50009Ool201l7Ool01000Oomoo`003Woo0P0O17oo1@0O17oo00@03`0O01mo
o`807`Eoo`807`Aoo`0301moogoo04Aoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eo
o`00A7oo00<007ooOol03Goo100O<7oo0P0O0goo0P0O0goo0P0O4Woo00D03goo01l07goo00807`=o
o`H07`Uoo`03001oogoo00=oo`03001oo`0000]oo`807`Moo`04001oogoo000EOol301l5Ool00`0?
Ool07`0301l5Ool201l3Ool301m5Ool00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol0
04Aoo`03001oogoo00eoo`@07c1oo`807`=oo`807`=oo`807`=oo`807`eoo`0500moo`0O01moo`02
01l3Ool601l9Ool00`00Oomoo`03Ool00`00Ool0000=Ool50002Ool01000Oomoo`005Goo0`0O1Goo
00<03goo01l00`0O1Goo0P0O0goo100OA7oo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol0
5Goo0014Ool3000=Ool401l`Ool201l3Ool201l2Ool301l3Ool201l=Ool01@0?Ool07`0OOol00P0O
0goo1P0O2Goo00<007ooOol00goo00<007oo00003goo00<007ooOol00Woo00@007ooOol0009oo`D0
00ioo`<07`Eoo`0300moo`0O00<07`Ioo`807`04Ool07`0OOol801lnOol00`0?Oomoo`1DOol00`0?
Oomoo`1IOol3000GOol004Aoo`03001oogoo00eoo`@07c1oo`807`<000807`9oo`<07`=oo`<07`ao
o`0500moo`0O01moo`0201l2Ool701l2Ool50002Ool00`00Oomoo`03Ool00`00Ool0000?Ool00`00
Oomoo`02Ool01000Oomoo`0017oo00<007ooOol03Woo0`0O1Goo00<03goo01l00`0O1Woo0P0O00=o
o`0O01l00Woo1`0O?Woo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00
Oomoo`0=Ool401l`Ool201l00goo001oo`0201l2Ool301l2Ool401l<Ool01@0?Ool07`0OOol00P0O
0Woo0P0O00=oo`0O01l00P0O17oo00<007ooOol00Woo00<007ooOol00goo00<007oo00003goo00<0
07ooOol00Woo00@007ooOol000Aoo`03001oogoo00ioo`807`Ioo`0300moo`0O00<07`Ioo`807`03
Ool07`0O00Moo`<07ceoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00A7oo00<0
07ooOol03Goo100O<7oo0P0O00=oo`00Ool00`0O00=oo`0O01l00P0O00=oo`0O01l00P0O37oo00D0
3goo01l07goo00807`9oo`0401moogooOol301l4Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00
Ool00005000:Ool00`00Oomoo`02Ool01000Oomoo`0017oo00<007ooOol03Woo0P0O1Woo00<03goo
01l00P0O1goo0P0O00=oo`0O01l027oo0P0O?Goo00<03gooOol0E7oo00<03gooOol0Fgoo00<007oo
Ool05Goo000EOol20007Ool50006Ool50006Ool5000;Ool00`00Oomoo`0=Ool401l`Ool201l017oo
001oogoo0P0O00=oo`0O01l00P0O00=oo`0O01l00P0O37oo00D03goo01l07goo00807`03Ool07`0O
00=oo`<07`Aoo`03001oogoo009oo`03001oogoo00=oo`03001oo`00009oo`03001oogoo00Yoo`H0
009oo`03001oogoo009oo`03001oogoo00ioo`807`Ioo`0300moo`0O00807`Moo`807`03Ool07`0O
00Qoo`807aaoo`807amoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`005Goo0P00
1Woo1`0017oo1`0017oo1`002Woo00<007ooOol03Goo0P0O00=oo`0O01l0;goo0P0O00Aoo`00Oomo
o`L07`03Ool07`0O00807`Yoo`807`0400moo`0O01l2Ool301l4Ool301l4Ool00`00Oomoo`02Ool0
0`00Oomoo`03Ool00`00Ool00002Ool00`00Oomoo`0:Ool01P00Oomoo`00Ool0009oo`03001oogoo
009oo`03001oogoo00H0011oo`0300moogoo00807`Moo`807`03Ool07`0O00Qoo`807aaoo`<07aio
o`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`005Goo0P001Woo0P000goo0P0017oo
0P000goo0P0017oo0P000goo0P002Woo00<007ooOol03Goo0P0O00=oo`0O01l0;goo0P0O00Aoo`00
Oomoo`@07`04Ool07`0OOol501l8Ool401l00goo01l07`02Ool301l4Ool301l4Ool00`00Oomoo`02
Ool00`00Oomoo`03Ool00`00Ool00002Ool00`00Oomoo`0:Ool01P00Oomoo`00Ool0009oo`03001o
ogoo009oo`03001oogoo009oo`03001oogoo015oo`0300moogoo00807`Moo`@07`Uoo`<07a]oo`<0
7aioo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`005Goo0P001Goo0P001Goo0P00
0Woo0P001Goo0P000Woo0P001Goo0P002Goo00<007ooOol03Goo0P0O00=oo`0O01l0;goo0P0O00Ao
o`00Oomoo`@07`04Ool07`0OOol201l00goo01l07`08Ool401l00goo01l07`02Ool201l6Ool201l4
Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Ool00002Ool00`00Oomoo`0:Ool01P00Oomoo`00
Ool0009oo`03001oogoo009oo`03001oogoo009oo`03001oogoo015oo`0300moogoo00807`Moo`@0
7`Yoo`D07`Ioo`807a1oo`<07aioo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00
17oo2P001goo0P001Goo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P002Goo00<007ooOol0
3Goo0P0O00=oo`0O01l0;goo0P0O00Aoo`00Oomoo`@07`03Ool07`0O00807`9oo`807`03Ool07`0O
00Eoo`H07`=oo`807`Ioo`807`Aoo`03001oogoo009oo`L00005Ool007ooOol0000<Ool01P00Oomo
o`00Ool0009oo`03001oogoo009oo`03001oogoo009oo`03001oogoo00Ioo`D000Ioo`0300moogoo
00807`Moo`@07`Yoo`D07`Eoo`<07`Ioo`807`Moo`@07aioo`0300moogoo05Aoo`0300moogoo05]o
o`03001oogoo01Eoo`005Goo0P001Goo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P002Goo
00<007ooOol03Goo0P0O00=oo`0O01l0;goo0P0O00Aoo`00Oomoo`@07`03Ool07`0O00807`9oo`H0
7`Aoo`H07`=oo`807`aoo`03001oogoo009oo`04001oogoo0002Ool00`00Ool00002Ool00`00Oomo
o`0:Ool01P00Oomoo`00Ool0009oo`03001oogoo009oo`03001oogoo009oo`H000Eoo`03001oogoo
00Ioo`0300moogoo00Yoo`<07`ioo`0301moogoo00=oo`<07`Eoo`<07`=oo`807`9oo`D07aeoo`03
00moogoo04=oo`807`moo`0300moogoo05]oo`03001oogoo01Eoo`005Goo0P001Goo0P001Goo0P00
0Woo0P001Goo0P000Woo0P001Goo0P002Goo1000300?0P0O00<03`0O01l0;`0?0P0O00<03`0000l0
0P0?0P0O0P0?100O0P0?1P0O0`0?1`0O300?1P00100?00@0000?00l000803`0300003`0000803`03
00003`0?00X03`0600003`0?00003`000P0?1P00100?00@0000?00l000L03`0300003`0?01<03`<0
7`h03`807`<03`807`0300l07`0O00@03`@07`803`<07`0400l07`0O00l201l200l201l=00l201m@
00l201l500l201mS00l4000GOol001Eoo`8000Eoo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eo
o`8000Uoo`03001oogoo00eoo`807`03Ool07`0O02moo`807`03Ool007oo009oo`807`=oo`<07`9o
o`H07`=oo`807`9oo`<07`ioo`04001oogoo0004Ool01000Oomoo`000Woo00<007oo00000Woo00<0
07ooOol02Woo00H007ooOol007oo0002Ool00`00Ool00002Ool00`00Oomoo`02Ool01000Oomoo`00
1goo00<007ooOol01Woo00<03gooOol02Woo0P0O47oo00<07gooOol00`0O0Woo0P0O0Woo0P0O00Ao
o`0O01moo`@07`04Ool07`0OOol201l2Ool301l4Ool201l6Ool301l9Ool00`0?Oomoo`12Ool301l5
Ool201l8Ool00`0?Oomoo`0XOol201laOol00`00Oomoo`0EOol001Eoo`8000Eoo`8000Eoo`80009o
o`8000Eoo`80009oo`8000Eoo`8000Uoo`03001oogoo00eoo`807`03Ool07`0O02moo`807`03Ool0
07oo00Moo`<07`=oo`H07`9oo`807`9oo`<07`ioo`04001oogoo0004Ool01000Oomoo`000Woo00<0
07oo00000Woo00<007ooOol02Woo00H007ooOol007oo0002Ool00`00Ool00002Ool00`00Oomoo`02
Ool01000Oomoo`001goo00<007ooOol01Woo00<03gooOol02Woo0P0O47oo0P0O00=oo`0O01l017oo
1@0O00Ioo`0O01moo`0OOol201l017oo01l07goo0P0O00=oo`0O01l00P0O0goo100O17oo100O2Goo
00<03`0O01l0@Woo100O17oo0`0O1goo00<03gooOol0:7oo0P0O<Goo00<007ooOol05Goo000EOol2
0005Ool20005Ool20002Ool20005Ool20002Ool20005Ool20009Ool00`00Oomoo`0=Ool201l00goo
01l07`0_Ool201l00goo001oo`07Ool201l4Ool201l2Ool201l017oo01l07`0O0Woo0`0O3Woo00<0
07ooOol01P000Woo00H007ooOol007oo0002Ool60007Ool01P00Oomoo`00Ool0009oo`03001oo`00
009oo`H0009oo`03001oogoo00Eoo`03001oogoo00Ioo`0300moogoo00Yoo`807a5oo`<07`Eoo`@0
7`9oo`@07`04Ool07`0OOol201l00goo01l07`0201l00goo01l07`03Ool201l00goo01moo`03Ool2
01l00goo01l07`08Ool00`0?01l07`12Ool401l4Ool301l7Ool00`0?Oomoo`0WOol301laOol00`00
Oomoo`0EOol0019oo`800003Ool0000000Eoo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`80
00Uoo`03001oogoo00eoo`807`03Ool07`0O02moo`807`03Ool007oo00Moo`807`Qoo`807`03Ool0
7`0O00=oo`<07`ioo`04001oogoo0002Ool00`00Ool00002Ool01P00Oomoo`00Ool0009oo`03001o
o`0000Yoo`06001oogoo001oo`000Woo00<007oo00000Woo00H007ooOol007oo0002Ool00`00Oomo
o`05Ool00`00Oomoo`06Ool00`0?Oomoo`0MOol201l<Ool301l2Ool501l2Ool201l2Ool201l3Ool2
01l017oo01l07`0O0Woo0P0O0Woo0P0O1goo0`0O@Woo100O0goo100O1goo00<03gooOol09goo0`0O
<Goo00<007ooOol05Goo000BOol50005Ool20005Ool20002Ool20005Ool20002Ool20005Ool20009
Ool00`00Oomoo`0=Ool201l00goo01l07`0^Ool201l2Ool00`00Oomoo`06Ool201l8Ool201l00goo
01moo`03Ool00`0?01l07`0>Ool01000Oomoo`000Woo00<007oo00000Woo00H007ooOol007oo0002
Ool00`00Ool000050005Ool01P00Oomoo`00Ool0009oo`03001oo`00009oo`06001oogoo001oo`00
0Woo00<007ooOol01Goo00<007ooOol01@0000=oo`0?Ool0;Goo0P0O0goo100O0Woo0P0O0Woo0`0O
0Woo0P0O0Woo0`0O00=oo`0O01l00Woo0`0O1Woo100O@Goo100O0goo100O1goo00<03gooOol09goo
100O<7oo00<007ooOol05Goo000DOol30006Ool20003Ool20004Ool20003Ool20004Ool20003Ool2
000:Ool00`00Oomoo`0<Ool201l2Ool201l^Ool201l2Ool00`00Oomoo`0@Ool401l4Ool00`0?01l0
7`0>Ool01000Oomoo`000Woo00<007oo00000Woo00H007ooOol007oo0002Ool00`00Ool00002Ool6
0002Ool01P00Oomoo`00Ool0009oo`03001oo`00009oo`06001oogoo001oo`000Woo00<007ooOol0
1Goo00<007ooOol00Woo00D007ooOomoo`0?00=oo`H002Yoo`<07`Qoo`807`04Ool07`0O01l3Ool5
01l3Ool201l6Ool201l00goo01l07`0oOol501l3Ool401l7Ool00`0?Oomoo`0WOol401l`Ool00`00
Oomoo`0EOol001Eoo`8000Ioo`L000Aoo`L000Aoo`L000Yoo`03001oogoo00aoo`807`9oo`807bio
o`807`9oo`03001oogoo015oo`807`Eoo`0300moogoo00ioo`04001oogoo0002Ool00`00Ool00002
Ool01P00Oomoo`00Ool0009oo`03001oo`00009oo`04001oogoo0004Ool01P00Oomoo`00Ool0009o
o`03001oo`00009oo`06001oogoo001oo`000Woo00<007ooOol01Goo00<007ooOol00Woo00D007oo
Oomoo`0?00Eoo`03001oogoo02]oo`<07`Qoo`D07`Eoo`<07`Aoo`807`=oo`0301moogoo00807`9o
o`807cioo`807`03Ool07`0O00=oo`D07`Ioo`0300moogoo02Moo`@07c1oo`03001oogoo01Eoo`00
5Goo0P001goo1@001Woo1@001Woo1@002goo00<007ooOol037oo0P0O0Woo0P0O;Woo0P0O0Woo00<0
07ooOol067oo00<03gooOol01goo1@000Woo00@007ooOol0009oo`03001oo`00009oo`06001oogoo
001oo`000Woo00<007oo00000Woo00<007ooOol01P000Woo00<007oo00000Woo00<007oo00000Woo
00H007ooOol007oo0002Ool00`00Oomoo`060004Ool01@00Oomoogoo00l01Goo00<007ooOol0:goo
0`0O27oo1@0O1Woo0P0O17oo1`0O00=oo`0O01l00Woo0`0O0Woo0P0O1goo00<07gooOol05Woo0P0O
5goo0P0O0Woo0P0O0Woo0P0O00=oo`0O01l01Woo00<03gooOol00goo0P0O8Goo1@0O<7oo00<007oo
Ool05Goo0014Ool00`00Oomoo`0<Ool201l3Ool201lHOol201lCOol201l2Ool00`00Oomoo`0HOol0
0`0?Ool000050004Ool00`00Oomoo`02Ool01000Oomoo`000Woo00<007oo00000Woo00H007ooOol0
07oo0002Ool00`00Ool00002Ool01P00Oomoo`00Ool0009oo`06001oogoo001oo`000Woo00<007oo
00000Woo00H007ooOol007oo0002Ool00`00Oomoo`02Ool01000Oomoo`0017oo00D007ooOomoo`0?
00Eoo`03001oogoo02aoo`807aYoo`T07`=oo`807`9oo`/07`Yoo`807`=oo`D07`=oo`807aMoo`80
7`9oo`807`9oo`807`03Ool07`0O00Ioo`0300moogoo009oo`<07`Ioo`807aUoo`807`03Ool07`0O
00Eoo`807bUoo`03001oogoo01Eoo`00A7oo0`0037oo0P0O0goo0P0O5goo0`0O4goo0P0O0Woo00<0
07ooOol067oo00D03gooOomoo`0000Moo`03001oogoo009oo`04001oogoo0002Ool00`00Ool00002
Ool01P00Oomoo`00Ool0009oo`03001oo`00009oo`06001oogoo001oo`000Woo00H007ooOol007oo
0002Ool00`00Ool00002Ool01P00Oomoo`00Ool0009oo`H0009oo`03001oogoo009oo`05001oogoo
Ool03`05Ool90005Ool5000LOol201lOOol301l5Ool>01l:Ool301l2Ool601l00goo01l07`0201l8
Ool201l;Ool301l2Ool501l2Ool201l6Ool00`0?Oomoo`0601l4Ool401l2Ool201l2Ool00`0OOomo
o`0?Ool201l00goo01l07`04Ool501lUOol3000GOol004Aoo`03001oogoo00aoo`807`=oo`807aMo
o`@07a9oo`807`9oo`03001oogoo01Qoo`0500moogooOol00002Ool60004Ool01000Oomoo`000Woo
00<007oo00000Woo00H007ooOol007oo0002Ool00`00Ool00002Ool01P00Oomoo`00Ool0009oo`06
001oogoo001oo`000Woo00<007oo00000Woo00H007ooOol007oo0002Ool01P00Oomoo`00Ool0009o
o`03001oogoo009oo`05001oogooOol03`05Ool01000Oomoo`000Woo00<007ooOol01@000Woo00<0
07ooOol0?Goo0`0O1Goo100O2Goo0P0O00=oo`0O01l0300O0goo100O0Woo0P0O1Woo100O2Woo0P0O
0goo1@0O0Woo0P0O1Woo00<03gooOol00`0O0Woo00@07gooOomoo`D07`9oo`H07a1oo`807`03Ool0
7`0O009oo`d07`03Ool07goo01ioo`03001oogoo01Eoo`00A7oo00<007ooOol027oo00@07gooOomo
o`807`=oo`807`Yoo`0301moogoo00Yoo`807`03Ool07`0O015oo`807`9oo`H001Eoo`0500moogoo
Ool00005Ool00`00Ool00004Ool01000Oomoo`000Woo00<007oo00000Woo00H007ooOol007oo0002
Ool00`00Ool00002Ool01P00Oomoo`00Ool0009oo`06001oogoo001oo`000Woo00<007oo00000Woo
00H007ooOol007oo0002Ool01P00Oomoo`00Ool0009oo`03001oogoo009oo`03001oogoo00D0009o
o`04001oogoo0002Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`0nOol201l6Ool201l:
Ool<01l017oo01l07`0O1Goo0`0O0goo00<07gooOol017oo00@07gooOol07`Moo`D07`Ioo`807`9o
o`<07`9oo`0401moogoo00l401l3Ool601l2Ool901l?Ool301l2Ool601l2Ool;01lBOol201l:Ool0
0`00Oomoo`0EOol004Aoo`03001oogoo00Moo`<07`9oo`807`=oo`807`Ioo`H07`Yoo`<07`9oo`L0
7`]oo`807`9oo`04001oogoo000GOol01@0?Oomoogoo00001Goo00<007oo000017oo00@007ooOol0
009oo`03001oo`00009oo`06001oogoo001oo`000Woo00<007oo00000Woo00H007ooOol007oo0002
Ool01P00Oomoo`00Ool0009oo`03001oo`00009oo`06001oogoo001oo`000Woo00H007ooOol007oo
0002Ool00`00Oomoo`02Ool01P00Oomoogoo00l000Aoo`04001oogoo0002Ool00`00Oomoo`02Ool0
0`00Oomoo`02Ool00`00Oomoo`0oOol00`0?Oomoo`0AOol401l5Ool0100OOomoogoo0P0O1Woo0P0O
0goo200O0Woo3@0O2goo1P0O00=oo`0O01l00P0O1Goo100O0goo100O17oo1@0O0goo2P0O0goo1@0O
2Goo500O0Woo0`0O2Woo00<007ooOol05Goo0014Ool00`00Oomoo`08Ool201l2Ool201l3OolJ01l3
OolD01l2Ool01000Oomoo`005goo00D03gooOomoo`0000Eoo`03001oo`0000Aoo`04001oogoo0002
Ool00`00Ool00002Ool01P00Oomoo`00Ool0009oo`03001oo`00009oo`06001oogoo001oo`000Woo
00H007ooOol007oo0002Ool00`00Ool00002Ool01P00Oomoo`00Ool0009oo`06001oogoo001oo`00
0Woo00<007ooOol00Woo1P0017oo00@007ooOol0009oo`03001oogoo009oo`/003aoo`0300moogoo
02]oo`L07`=oo`X07`ioo`X07`ioo`807`Eooa807`=oo`<07`mooa407`03Ool07`0O00]oo`03001o
ogoo01Eoo`00A7oo00<007ooOol02Goo1@0O0goo2P0O0goo3@0O27oo3P0O0goo00@007ooOol001Mo
o`0400moogooOol700000goo001oo`03Ool01000Oomoo`000Woo00<007oo00000Woo00H007ooOol0
07oo0002Ool00`00Ool00002Ool01P00Oomoo`00Ool0009oo`06001oogoo001oo`000Woo00<007oo
00000Woo00H007ooOol007oo0002Ool01P00Oomoo`00Ool0009oo`H0009oo`0300003`0000D0009o
o`04001oogoo0004Ool01P00Oomoo`00Ool0009oo`H003Uoo`0300moogoo059oo`<07aYoo`L07`Ao
o`@07`=oo`807a5oo`807`eoo`<07`aoo`03001oogoo01Eoo`00A7oo00<007ooOol02Goo1@0O47oo
0P0O7goo00<07gooOol00Woo00@007ooOol0009oo`D0009oo`T000Eoo`0500moogooOol00002Ool0
1@00Oomoo`00Ool01P000Woo00T007ooOoml07oo001oog`000000Woo00<007oo00000Woo00<007oo
00000Woo00H007ooOol007oo0002Ool01P00Oomoo`00Ool0009oo`03001oo`00009oo`06001oogoo
001oo`000Woo00H007ooOol007oo0002Ool00`00Ool00002Ool01P00Oomoo`0000l0009oo`03001o
o`00009oo`04001oogoo0004Ool01P00Oomoo`00Ool0009oo`03001oo`0000D001Eoo`H001aoo`03
00moogoo059oo`807`0300moogoo04aoo`<07`aoo`03001oogoo01Eoo`00A7oo00<007ooOol0A7oo
00@007ooOol000Aoo`03001oogoo00Eoo`03001oo`0000Qoo`0500moogooOol00002Ool01P00Oomo
o`00Ool0009oo`03001oo`00009oo`04001oogoo0003O0001goo001oogoo001oo`00009oo`03001o
o`00009oo`06001oogoo001oo`000Woo00H007ooOol007oo0002Ool00`00Ool00002Ool01P00Oomo
o`00Ool0009oo`06001oogoo001oo`000Woo00<007oo00000Woo00H007ooOol0000?0002Ool00`00
Ool00002Ool01000Oomoo`0017oo00H007ooOol007oo0002Ool00`00Ool00002Ool00`00Oomoo`0G
Ool00`00Oomoo`0MOol00`0?Oomoo`1DOol00`0?Oomoo`1=Ool00`0OOomoo`0;Ool00`00Oomoo`0E
Ool004Aoo`03001oogoo04Aoo`03001oogoo00H000Moo`03001oo`0000Eoo`D0009oo`04001oogoo
0002Ool00`00Ool00002Ool00`00Ool00002Ool01000Oomoo`000g`000Moo`00Oomoo`00Ool00002
Ool00`00Ool00002Ool01P00Oomoo`00Ool0009oo`06001oogoo001oo`000Woo00<007oo00000Woo
00H007ooOol007oo0002Ool01P00Oomoo`00Ool0009oo`03001oo`00009oo`06001oogoo00003`00
0Woo00<007oo00000Woo00@007ooOol000Aoo`06001oogoo001oo`000Woo00<007oo00000Woo00<0
07ooOol01P002Woo1@000Woo00<007ooOol07Goo00<03gooOol0E7oo00<03gooOol0Fgoo00<007oo
Ool05Goo0014Ool00`00Oomoo`14Ool01P00Oomoo`00Ool0009oo`03001oogoo00H00003Ool007oo
00Ioo`0300003goo009oo`04001oogoo0002Ool00`00Ool00002Ool00`00Ool00002Ool00`00Oomo
o`05O0001P00Oomoo`00Ool0009oo`03001oo`00009oo`06001oogoo001oo`000Woo00H007ooOol0
07oo0002Ool00`00Ool00002Ool01P00Oomoo`00Ool0009oo`06001oogoo001oo`000Woo00<007oo
00000Woo00H007ooOol0000?0002Ool00`00Ool00002Ool00`00Oomoo`060002Ool00`00Ool00002
Ool00`00Ool00002Ool;0003Ool50002Ool00`00Oomoo`02Ool00`00Oomoo`06000GOol00`0?Oomo
o`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`<004Aoo`06001oogoo001oo`000Woo1P00
0Woo00<007oo00000Woo2`000Woo00H007ooOol007oo0002Ool00`00Ool00002Ool00`00Ooml0006
O002Ool00`00Ool00002Ool00`00Ool00002Ool01P00Oomoo`00Ool0009oo`06001oogoo001oo`00
0Woo00<007oo00000Woo00H007ooOol007oo0002Ool01P00Oomoo`00Ool0009oo`03001oo`00009o
o`06001oogoo00003`000Woo00<007oo00000Woo00H007ooOol007oo0002Ool01P00Oomoo`00Ool0
009oo`03001oo`00009oo`06001oogoo001oo`000Woo3P0017oo00<007ooOol00goo00<007ooOol0
1Goo1@000goo1@001Goo00<03gooOol0BGoo1P001Goo00<03gooOol0FGoo0`005goo0014Ool00`00
Oomoo`14Ool01P00Oomoo`00Ool0009oo`03001oo`00009oo`05001oogoo001oo`060002Ool01P00
00moo`00Ool0009oo`06001oogoo001oo`000Woo00@007oo001oo`=l0003Ool007`0009l0007Ool0
07ooOol007oo00000Woo00<007oo00000Woo00H007ooOol007oo0002Ool01P00Oomoo`00Ool0009o
o`03001oo`00009oo`06001oogoo001oo`000Woo00H007ooOol007oo0002Ool00`00Ool00002Ool0
1P00Oomoo`0000l0009oo`03001oo`00009oo`06001oogoo001oo`000Woo00H007ooOol007oo0002
Ool00`00Ool00002Ool01P00Oomoo`00Ool0009oo`06001oogoo001oo`000Woo00<007oo00000Woo
00<007ooOol00Woo1`0017oo1P000Woo1P001goo00<03gooOol0Bgoo00<007ooOol01Woo00<03goo
Ool0Fgoo00<007ooOol05Goo0014Ool00`00Oomoo`0=Ool6000aOol01P00Oomoo`00Ool0009oo`03
001oo`00009oo`06001oogoo001oo`000Woo00<007oo00000Woo00T0000?Ooml07oo001oog`00000
0Woo00<007oo00000Woo00@007oo001oo`9l009oo`06001l0000O01oo`000Woo00<007oo00000Woo
00<007oo00000Woo00H007ooOol007oo0002Ool01P00Oomoo`00Ool0009oo`03001oo`00009oo`06
001oogoo001oo`000Woo00H007ooOol007oo0002Ool00`00Ool00002Ool01P00Oomoo`0000l0009o
o`03001oo`00009oo`06001oogoo001oo`000Woo00H007ooOol007oo0002Ool00`00Ool00002Ool0
1P00Oomoo`00Ool0009oo`06001oogoo001oo`000Woo00<007oo00000Woo1P000Woo00<007ooOol0
20000Woo00<007ooOol00Woo00<007ooOol07@0027oo2`009Woo00<007ooOol01Woo00<03gooOol0
4Woo1@0047oo1@000Woo1P009goo00<007ooOol05Goo0014Ool00`00Oomoo`0?Ool00`00Oomoo`0b
Ool01P00Oomoo`00Ool0009oo`03001oo`00009oo`06001oogoo001oo`000Woo00<007oo00000Woo
00@0000?Ool000=l0007Ool007ooOol007oo00000Woo00<007oo000017`000Moo`00O00007`0Ool0
0002Ool00`00Ool00004O0001000Oomoo`000Woo00<007oo00000Woo00H007ooOol007oo0002Ool0
3P00Ool007`0Ool007`0Ooml07oo001l07oo0002Ool00`00Ool00002Ool00`00Ool00002Ool01P00
Oomoo`0000l0009oo`03001oo`00009oo`06001oogoo001oo`000Woo00H007ooOol007oo0002Ool0
0`00Ool00002Ool01P00Oomoo`00Ool0009oo`06001oogoo001oo`000Woo00<007oo00000Woo00H0
07ooOol007oo0002Ool01P00Oomoo`00Ool0009oo`03001oo`00009oo`H0009oo`03001oo`00009o
o`06001oogoo001oo`000Woo00H007ooOol007oo0002Ool00`00Ool00002Ool>0004OolI0002Ool=
00001G`0000007ooO0001Woo60002Woo1@001Goo1P000Woo4`000Woo20003Goo00<007ooOol05Goo
0014Ool00`00Oomoo`05Oom00002Ool00`00Ool00002Ool00`00Ool00002Ool01000Oomoo`000g`0
00Aoo`00Ool0009oo`0400003goo0003O0001Goo001oogoo00000g`000=oo`00Ool027`000@007`0
Ool000Al0006001l07`0O01oo`000Woo00H007ooOol007oo0002Ool00`00O01l0004O0001Goo001o
o`00Ool00W`00Woo00H007`0O01l07oo0002Ool02000Ool007`0Ool007`00002Ool01P00Oomoo`00
00l0009oo`03001l07`0009l0006001oogoo001oo`000Woo00H007ooOol007oo0002Ool01W`0Ool0
07ooO000009oo`03001oo`00009oo`06001oogoo001oo`000Woo00<007oo00000Woo00H007ooOol0
07oo0002Ool01P00Oomoo`00Ool0009oo`03001oo`00009oo`06001oogoo001oo`000Woo00<007oo
00000Woo00H007ooOol007oo0002Ool01P00Oomoo`00Ool0009oo`03001oo`00009oo`06001oogoo
001oo`000Woo00D007ooOol007oo00<00006O0000000Ooml00000Woo00<007oo00000Woo00<007oo
Ool01P0000=oo`00Ool01Woo1`0000Eoo`00Oomoo`0000=l0003Ool000000140009oo`H0009ooaH0
0003Ool0000000D00005Ool007ooOol00002Ool00`00Ool00002Ool00`00Ool000080002Ool80008
Ool00`00Oomoo`0EOol004Aoo`03001oogoo00Qoo`03001oo`00009oo`03001oo`00009oo`05001o
ogoo001oo`0700001goo001oogoo001oo`00009oo`06001oogoo001oo`000Woo00H007ooOol007oo
0002Ool00`00Ool00002Ool01`00Oomoo`00Ool007oo00=l0004Ool007oo0004O0001000Oomoo`00
0Woo00P007`0O01l07oo001oo`000Woo00<007`0O0001W`00Woo00P007`0O01l07oo001oo`0017`0
00Moo`00O00007`0Ool00003O0000gooO01l0003O0000`00Ooml0002O0001goo001oo`00O01oo`00
00Il0004Ool007oo0003O002Ool01P00O01l07`0Ooml009oo`03001l07`000=l000=Ool007`0O000
07`0Ool0000?001l07oo00000g`000Moo`00O01oo`00O0000002Ool00`00O01l0002O0001000O01o
o`000g`000Eoog`0Oomoo`0000Al0008001oogoo001oo`00O01oo`9l0004001oog`0O002Ool01000
O00007oo0g`001Ioo`00O00007`0Ool007`0001l07oo001l07ooO01oo`00O01oog`0Ool0009l000<
001l07oo001l0000O01oo`00O01oo`000g`000Aoog`0O000009l0003001l07oo009l0003001l07`0
009l0007Ooml07`0001l07oo00000g`000Uoog`0Oomoo`00O00007`0Ool00W`000Ioog`0O00007`0
Ool2O0000`00Ooml0003Ool01G`0Ool007`0Ool00W`000Moog`0O00007`0Ool00003O0001000O000
00000W`000L007`0Ool007`0001oo`03O0000goo001l0005O00017ooO01l000017`000<007`0O000
0W`000Aoog`0O00000Al0006Ooml07`0001l07oo0W`000<007`0O0000W`000Ioog`0O00007`0Ool2
O0001WooO01l0000O01oo`9l0003001l07`0009l0003Ool007`000El0005001l07oo001l0009Ool0
0`00Oomoo`0EOol004Aoo`03001oogoo00Eoo`03O01oogoo009l0003001l07oo009l0007001l07`0
001l07oo00001W`000Aoog`0O00000Ml0003001l07`0009l0003001oog`0009l0003Ool007`000Ql
0003001l07`000Il009oo`03001l07`000=l0005Ool007ooO0000002Ool5O0000`00O01l0002O000
0`00O01l0004O00017ooO01oogoo1G`000<007`000000W`000P007`0Ool007`0001l07oo2g`000=o
og`0O00017`000<007ooO00027`000<007`0O00077`000<007`0O0001G`000<007`0O0001G`000H0
07`0O01l07oo0006O0000goo001l000GO0000goo001l0005O0000goo001l000GO0000goo001oo`0M
O0000goo001l000GO005OolAO0000`00O01l001GO00:Ool00`00Oomoo`0EOol004Aoo`03001oogoo
00Iooeal0005Ool007`0Ool00008O0002Woo001l07`0001l07oo001l00002G`000Aoog`0O00000Il
0003Ool007`001El0003Ool007`00?ml01=l00Yoo`03001oogoo01Eoo`00A7oo00<007ooOol01Woo
GG`000@007`0O00000Ql000:Ool007`0Ool007`0Ool007`00002O0000`00O01l0005O0000`00O01l
000=O0000`00Ooml000:O0000`00O000003oO00HO009Ool00`00Oomoo`0EOol004Aoo`03001oogoo
00Aooe9l0003001l07`001Yl0006001l07`0001l00000W`000<007`0O0005G`000<007oo000037`0
00@007`0O000009l0003001l07`0009oo`03001l07`000]l0003Ool007`0019l0003Ool007`001ml
0003Ooml07`001Ml0003Ooml07`000Yl0003Ooml07`000El0003Ooml07`0029l0003Ooml07oo00Eo
o`Ml0004001l07`0Ool2O0000goo001l0011O0000gooO01l000?O0000gooO01l0008O007Ool00`00
Oomoo`0EOol004Aoo`<000Moo`05O00007`0O01oo`02O0000gooO01oo`02O0000gooO01l0002O000
1WooO01l07ooO000009l009oo`04O01oog`0Ool2O002Ool3O00027ooO01l07ooO00007ooO002Ool0
17`0Ooml07oo0W`000=oog`0O0000W`000=oog`0O0001G`000=oog`0O0005G`000Eoo`00O01l0000
009l0004001oo`00Ool5O0001goo001l07`0001l0000009l0006001l07`0001oog`00Woo1G`000<0
07ooOol01W`00Woo00<007ooO0001G`00Woo00L007`0O01l07oo001oo`03O00017oo001oo`000Woo
00<007oo00000Woo00=l07ooO0001W`000=oog`0O0001g`000=oog`0O0000Woo00<007`0O0000Woo
00<007ooO0001W`000=oo`00Ool00W`00Woo00<007ooO0000Woo00<007ooO0000Woo00=l07ooOol0
0g`00Woo00@007ooOol0009l009oo`03001oog`0009oo`06001oogoo001oog`00Woo00=l07ooO000
0Woo00H007ooOol007ooO002Ool01W`0Oomoo`00Ool0009oo`03001oog`0009oo`06001oogoo001o
o`000Woo00Ml07ooOol007oo001oo`02O0000goo001oo`02O002Ool00`00Ooml0002Ool01P00Oomo
o`00Ooml009oo`03001oo`0000Moo`04001oogooO002Ool00`00Ool00002Ool00`00O01l0006O000
1gooO0003goo001oog`0009oo`06001oogooO01oo`000Woo00Il07ooOol007ooO002Ool00`00Ooml
0002Ool01W`0Oomoo`00Ooml009oo`06001oogoo001oog`00Woo00<007oo00000Woo00Il07ooOol0
07ooO002Ool01P00Oomoo`00Ooml009oo`03O01oo`00009oo`03O01oogoo00L00004O01oogooO008
Ool3000GOol004Aoo`03001oogoo00Qoo`03001oo`00009oo`03001oo`00009oo`03001oogoo00T0
0003Ool0000000@0009oo`H0009oo`03001oogoo00/0009oo`03001oo`00009oo`06001oog`0001o
o`000Woo00<007ooO0000Woo00Il07ooO00007oo0002Ool00g`0Ool00002Ool01W`000moo`00Ool0
00Al0004Ool007oo0002Ool5O0001P00O01l0000O000009l0007001l07`0001oo`00Ool01g`000=o
og`0O0001G`00Woo00<007`0O0001G`00Woo1G`000@007ooO01l009oo`03001oo`00009oo`03001o
o`00009oo`06001oogoo00003g`00Woo00@007oo001oo`9l009oo`03001oo`00009oo`06001oogoo
001oo`000Woo00<007oo00000Woo00Il07ooOol007oo0002Ool01P00Oomoo`00Ool0009oo`03001o
o`00009oo`06001oogoo001oo`000Woo00H007ooOol007oo0002Ool00`00Ool00002Ool01P00Oomo
o`00Ool0009oo`03001oo`00009oo`06001oogoo001oo`000Woo00H007ooOol007oo0002Ool00`00
Ool00002Ool01P00Oomoo`00Ool0009oo`H0009ooa/0009ooa40009l01X0009oocD0009oo`P000Qo
o`03001oogoo01Eoo`00A7oo00<007ooOol01Goo4`000Woo:`000Woo00<007oo00000Woo00<007oo
00000Woo00H007ooOol007oo0002Ool00`00Ool00002Ool01P0000moo`00Ool000=l009oo`04001o
o`00Ool9O0000`00O0000002O0001P00O01oo`00Ool0009oo`03001oog`0009oo`04001oogoo0002
O002Ool01P00Oomoo`00Ool0009oo`04O01oo`00Ool<O0000`00Ool00002Ool00`00Ool00002Ool0
1P00Oomoo`0000l0009oo`03001oo`00009oo`06O01oogoo001oo`000Woo00H007ooOol007oo0002
Ool00`00Ool00002Ool01P00Oomoo`00Ool0009oo`06001oogoo001oo`000Woo00<007oo00000Woo
00H007ooOol007oo0002Ool01@00Oomoo`00Ool01P000Woo00H007ooOol007oo0002Ool;0002Ool0
0`00Oomoo`0K0004Ool00`00Oomoo`060005Ool5000:Ool60007Ool00`00Oomoo`03Ool00`0?Ool0
000G000@Ool50002Ool60005Ool5000MOol00`00Oomoo`0EOol004Aoo`03001oogoo00moo`03001o
ogoo039oo`06001oogoo001oo`000Woo00<007oo00000Woo00H007ooOol007oo0002Ool00`00Ool0
0002Ool01P0000moo`00Ool000Al0004Ool007oo0002Ool00`00Ooml0002Ool01P00O01l0000Ool0
009l0006001l07oo001oo`000Woo00<007oo00000Woo00H007ooOol007oo0002Ool01P00Oomoo`00
Ool0009oo`03001oo`00009oo`07001oogoo001oo`00Ool00W`00Woo00<007oo00000Woo00<007oo
00000Woo00H007ooOol0000?0002Ool00`00Ool00002Ool01P00Oomoo`00Ool0009oo`06001oogoo
001oo`000Woo00<007oo00000Woo00H007ooOol007oo0002Ool01P00Oomoo`00Ool0009oo`03001o
o`00009oo``00003Ool007oo00=oo`03001oogoo00H0009oo`03001oogoo00D0000300moo`0000P0
009oo`H000eoo`X002Eoo`D000=oo`0300moogoo05]oo`03001oogoo01Eoo`00A7oo00<007ooOol0
3Goo1P00<Goo00H007ooOol007oo0002Ool00`00Ool00002Ool01P00Oomoo`00Ool0009oo`03001o
o`00009oo`0400003goo0008O0001goo001oogoo001oo`00009oo`07001l07`0001l0000Ool00g`0
00Aoo`00Ool0009oo`03001oo`00009oo`06001oogoo001oo`000Woo00H007ooOol007oo0002Ool0
0`00Ool00002Ool01P00Oomoo`00Ool0009oo`06001oogoo001oo`000Woo00<007oo00000Woo00H0
07ooOol0000?0002Ool00`00Ool00002Ool01P00Oomoo`00Ool0009oo`06001oogoo001oo`000Woo
00<007oo00000Woo00H007ooOol007oo0002Ool01P00Oomoo`00Ool0009oo`03001oo`00009oo`03
001oogoo009oo`03001oogoo00=oo`D0009oo`04001oogoo0002Ool60007Ool00`0?Ool0001DOol0
0`0?Oomoo`1KOol00`00Oomoo`0EOol001ioo`D000Ioo`D000Ioo`D000]oo`03001oogoo04Aoo`06
001oogoo001oo`000Woo00<007oo00000Woo00H007ooOol007oo0002Ool;00000gooO01l0002Ool0
0`00Ool00002Ool00`00Ool00002Ool01`00O01l07`0Ool007oo009l009oo`03001oo`00009oo`03
001oo`00009oo`06001oogoo001oo`000Woo00H007ooOol007oo0002Ool00`00Ool00002Ool01P00
Oomoo`00Ool0009oo`06001oogoo001oo`000Woo00<007oo00000Woo00H007ooOol0000?0002Ool0
0`00Ool00002Ool01P00Oomoo`00Ool0009oo`06001oogoo001oo`000Woo00<007oo00000Woo00H0
07ooOol007oo0002Ool01@00Oomoo`00Ool02@0017oo00<007ooOol01P000Woo20000goo1@001Goo
1P00DGoo00<03gooOol0Fgoo00<007ooOol05Goo000MOol70004Ool70004Ool7000:Ool00`00Oomo
o`14Ool01P00Oomoo`00Ool0009oo`03001oo`00009oo`03001oogoo00P0009oo`0300003goo009o
o`04001oogoo0002Ool00`00Ool00002Ool00`00Ool00002Ool00`00O01l0009O0001goo001oogoo
001oo`00009oo`06001oogoo001oo`000Woo00H007ooOol007oo0002Ool00`00Ool00002Ool01P00
Oomoo`00Ool0009oo`06001oogoo001oo`000Woo00<007oo00000Woo00H007ooOol0000?0002Ool0
0`00Ool00002Ool01P00Oomoo`00Ool0009oo`06001oogoo001oo`000Woo00<007oo00000Woo4000
1Goo00<007ooOol00Woo00<007ooOol07Goo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol0
5Goo000LOol30003Ool30003Ool20003Ool20004Ool20003Ool2000:Ool00`00Oomoo`14Ool00`00
Oomoo`0;0002Ool00`00Oomoo`07Ool00`0000moo`02Ool01000Oomoo`000Woo00<007oo00000Woo
00<007oo00000Woo1G`000@007ooO01l009oo`03001oo`00009oo`03001oo`00009oo`06001oogoo
001oo`000Woo00H007ooOol007oo0002Ool00`00Ool00002Ool01P00Oomoo`00Ool0009oo`06001o
ogoo001oo`000Woo00<007oo00000Woo00H007ooOol0000?0002Ool00`00Ool00002Ool00`00Oomo
o`060002Ool00`00Ool00002Ool60005Ool50008Ool50002Ool00`00Oomoo`0MOol00`0?Oomoo`1D
Ool00`0?Oomoo`1KOol00`00Oomoo`0EOol001aoo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eo
o`8000Uoo`03001oogoo04Aoo`04001oogoo0007Ool60009Ool00`0000moo`02Ool01000Oomoo`00
0Woo00<007oo00000Woo00@007oo001oo`Ml009oo`06001oogoo001oo`000Woo00<007oo00000Woo
00H007ooOol007oo0002Ool01P00Oomoo`00Ool0009oo`03001oo`00009oo`06001oogoo001oo`00
0Woo00H007ooOol007oo0002Ool00`00Ool00002Ool01P00Oomoo`0000l0009oo`03001oo`00009o
o`04001oogoo0004Ool01P00Oomoo`00Ool0009oo`03001oogoo009oo`03001oogoo01Moo`03001o
ogoo01eoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`003Woo2P0017oo0P001Goo
0P000Woo0P001Goo0P000Woo0P001Goo0P002Goo00<007ooOol0A7oo00@007ooOol000aoo`03001o
ogoo00Moo`0300003goo00H0009oo`03001oo`00009oo`03001oo`00009oo`06001oogooO01oo`00
0Woo00H007ooOol007oo0002Ool00`00Ool00002Ool01P00Oomoo`00Ool0009oo`06001oogoo001o
o`000Woo00<007oo00000Woo00H007ooOol007oo0002Ool01P00Oomoo`00Ool0009oo`H0009oo`03
00003`00009oo`03001oo`00009oo`04001oogoo0004Ool60002Ool00`00Oomoo`02Ool00`00Oomo
o`0GOol00`00Oomoo`0MOol00`0?Oomoo`1DOol00`0?Oomoo`0KOol201lnOol00`00Oomoo`0EOol0
01aoo`8000Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8000Uoo`@001<03`807bD03`807`L03`04
00003`0?000<00l00`0000l03`0500l5000500l01@0000l03`0000l01P000P0?00H0000?00l0000?
000200l01P0000l03`0000l000803`0300003`0000803`0600003`0?00003`000P0?00H0000?00l0
000?000200l00`0000l0000200l01P0000l03`0000l000803`0600003`0?00003`000P0?00<0000?
00l00P0?00D0000?00l0000?00H000803`0400003`0?000400l00`0000l03`0200l0100000l03`00
100?00<0000?00l05@0?1P00B`0?00@07`0?00l03`807``03`0301l03`0?01003`0301l03`0?0140
3`807`X03`<07`d03`D07a`03`0401l03`0?00l201l700l4000GOol001aoo`<000=oo`<0009oo`80
00Eoo`80009oo`8000Eoo`8000Uoo`03001oogoo01=oo`/07`=oo`807aMoo`H07`Aoo`04001oogoo
000<Ool00`00Oomoo`08Ool00`0?Oomoo`04Ool01P00Oomoo`00Ool000Aoo`04001oogoo0002Ool0
0`00Ool00002Ool01P00Oomoo`00Ool0009oo`03001oo`00009oo`06001oogoo001oo`000Woo00H0
07ooOol007oo0002Ool00`00Ool00002Ool01P00Oomoo`00Ool0009oo`06001oogoo001oo`000Woo
00<007ooOol00Woo1P0017oo00@007ooOol0009oo`03001oogoo009oo`03001oogoo009oo`/003Mo
o`0300moogoo01Aoo`0301moogoo01Aoo`<07`9oo`<07`9oo`/07a1oo`<07`03Ool03goo00Moo`80
7`Eoo`<07`Eoo`807`9oo`D07`Ioo`0301moogoo00=oo`D07aQoo`H07`04Ool07`0O01l:Ool00`00
Oomoo`0EOol001eoo`L000=oo`8000Eoo`80009oo`8000Eoo`8000Uoo`03001oogoo00Moo`L07`Ao
o`d07`03Ool07`0O00H07`Uoo`L07`03Ool07`0O00L07`9oo`04001oogoo0009Ool60008Ool00`0?
Oomoo`04Ool01P00Oomoo`00Ool000Aoo`04001oogoo0002Ool00`00Ool00002Ool01P00Oomoo`00
Ool0009oo`03001oo`00009oo`06001oogoo001oo`000Woo00H007ooOol007oo0002Ool00`00Ool0
0002Ool01P00Oomoo`00Ool0009oo`06001oogoo001oo`000Woo00<007ooOol00Woo00H007ooOomo
o`0?0004Ool01000Oomoo`000Woo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol0?goo00<0
3gooOol04Goo1@0O0goo0P0O2goo0P0O0Woo2@0O0Woo2`0O3goo100O00Aoo`0?Ool07`Eoo`D07`9o
o`D07`Aoo`H07`9oo`T07`Aoo`807`04Ool07`0O01l5Ool?01l3Ool901l;Ool00`00Oomoo`0EOol0
01eoo`L000=oo`8000Eoo`80009oo`8000Eoo`8000Uoo`03001oogoo00Qoo`H07`=oo`<07`Yooad0
7`=oo`@07`9oo`04001oogoo000GOol00`0?Oomoo`04Ool00`00Oomoo`050002Ool01000Oomoo`00
0Woo00<007oo00000Woo00H007ooOol007oo0002Ool00`00Ool00002Ool01P00Oomoo`00Ool0009o
o`06001oogoo001oo`000Woo00<007oo00000Woo00H007ooOol007oo0002Ool01P00Oomoo`00Ool0
009oo`03001oogoo009oo`06001oogooOol03`0017oo00@007ooOol0009oo`03001oogoo009oo`03
001oogoo009oo`03001oogoo03Uoo`0301moogoo00=oo`0300moogoo011oo`H07`9oo`D07`Aoo`04
01moogooOol401l017oo01moogoo100O00=oo`0O01l00P0O2Goo0P0O17oo0P0O00=oo`0O01l01Goo
00<07gooOol01P0O0goo2P0O00=oo`0O01l00goo1P0O0goo2@0O17oo0P0O0Woo0P0O17oo500O17oo
0`0O37oo00<007ooOol05Goo000LOol30003Ool30002Ool20005Ool20002Ool20005Ool20009Ool0
0`00Oomoo`0<Ool201l3Ool201l;Ool301l5Ool=01l3Ool401l6Ool201l2Ool01000Oomoo`005goo
00<03gooOol017oo00@007ooOol000Ioo`04001oogoo0002Ool00`00Ool00002Ool01P00Oomoo`00
Ool0009oo`03001oo`00009oo`06001oogoo001oo`000Woo00H007ooOol007oo0002Ool00`00Ool0
0002Ool01P00Oomoo`00Ool0009oo`H0009oo`03001oogoo009oo`06001oogooOol03`0017oo00@0
07ooOol0009oo`03001oogoo009oo`03001oogoo009oo`03001oogoo03Qoo`<07`Aoo`0300moogoo
00moo`<07`=oo`h07`9oo`0301moogoo00<07`=oo`807`9oo`<07`Yoo`<07`=oo`D07`Aoo`807`9o
o`/07`9oo`D07`9oo`807`9oo`<07`moo`807`9oo`807`=oo`<07`=oo`0301moogoo00]oo`D07`Eo
o`807`eoo`03001oogoo01Eoo`0077oo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P002Goo
00<007ooOol037oo0P0O0goo0P0O37oo00<07gooOol07Woo0P0O0Woo1P005Goo00<03gooOol017oo
1P0017oo00@007ooOol0009oo`03001oo`00009oo`06001oogoo001oo`000Woo00<007oo00000Woo
00H007ooOol007oo0002Ool01P00Oomoo`00Ool0009oo`03001oo`00009oo`06001oogoo001oo`00
0Woo00<007ooOol00Woo00@007ooOol000Aoo`03001oogoo00D0009oo`03001oogoo00/0009oo`03
001oogoo02Uoo`807`aoo`D07`=oo`0300moogoo00]oo`H07`Aoo`@07`9oo`/07`9oo`807`Uoo`80
7`]oo`<07`03Ool07`0O00D07`9oo`807`Eoo`0300moogoo00D07`Yoo`807`9oo`807a1oo`<07`03
Ool07`0O00Aoo`807`04Ool07`0O01lVOol00`00Oomoo`0EOol001aoo`8000Eoo`80009oo`8000Eo
o`80009oo`8000Eoo`8000Uoo`03001oogoo00aoo`807`=oo`807beoo`807`9oo`03001oogoo01Qo
o`0300moogoo00Aoo`03001oogoo00Moo`04001oogoo0002Ool00`00Ool00002Ool01P00Oomoo`00
Ool0009oo`03001oo`00009oo`06001oogoo001oo`000Woo00H007ooOol007oo0002Ool00`00Ool0
0002Ool01P00Oomoo`00Ool0009oo`03001oogoo00H000Aoo`05001oogooOol03`03Ool6000:Ool5
000XOol301l<Ool201l00goo01l07`0401l3Ool201l3Ool:01l6Ool201l:Ool301lLOol201l00goo
01l07`03Ool201l017oo01l07`0O1Goo00<03gooOol00P0O3Goo0P0O00=oo`0O01l04Woo0P0O00=o
o`0O01l017oo0P0O00=oo`0O01l09goo00<007ooOol05Goo000LOol30003Ool30003Ool20003Ool2
0004Ool20003Ool2000:Ool00`00Oomoo`0<Ool201l3Ool201l]Ool201l2Ool00`00Oomoo`0HOol0
0`0?Oomoo`04Ool00`00Oomoo`07Ool01000Oomoo`000Woo00<007oo00000Woo00H007ooOol007oo
0002Ool00`00Ool00002Ool01P00Oomoo`00Ool0009oo`06001oogoo001oo`000Woo00<007oo0000
0Woo00H007ooOol007oo0002Ool00`00Oomoo`05Ool00`00Oomoo`02Ool01@00Oomoogoo00l027oo
00<007ooOol0:Goo0P0O2Goo100O1Woo0P0O17oo0P0O0Woo1@0O0Woo3@0O5Woo0P0O77oo0P0O00=o
o`0O01l00goo0P0O00=oo`0O01l01Woo00<03gooOol03goo0P0O00=oo`0OOol04Woo0P0O00=oo`0O
01l017oo100O:7oo00<007ooOol05Goo000MOol70004Ool70004Ool7000:Ool00`00Oomoo`0<Ool2
01l3Ool201l]Ool201l2Ool00`00Oomoo`0HOol00`0?Oomoo`04Ool00`00Oomoo`07Ool01000Oomo
o`000Woo00<007oo00000Woo00H007ooOol007oo0002Ool00`00Ool0000600001Goo001oogoo0000
0Woo00<007oo00000Woo00<007oo00000Woo00H007ooOol007oo0002Ool00`00Oomoo`05Ool00`00
Oomoo`02Ool01@00Oomoogoo00l027oo00<007ooOol0:Goo0P0O27oo0`0O00=oo`0O01l017oo100O
0goo0P0O1Woo0P0O00=oo`0O01l01P0O>Goo1@0O0goo0P0O00=oo`0O01l01Woo00<03gooOol03goo
100O4goo0P0O00=oo`0O01l01Goo0`0O:7oo00<007ooOol05Goo000NOol50006Ool50006Ool5000;
Ool00`00Oomoo`0<Ool201l3Ool201l]Ool201l2Ool00`00Oomoo`0HOol00`0?Oomoo`04Ool00`00
Oomoo`07Ool01000Oomoo`000Woo00<007oo00000Woo00H007ooOol007oo0002Ool00`00Ool00002
Ool60002Ool01P00Oomoo`00Ool0009oo`03001oo`00009oo`06001oogoo001oo`000Woo00<007oo
Ool01Goo00<007ooOol00Woo00D007ooOomoo`0?00Ioo`D002=oo`807`=oo`<07`Moo`<07`=oo`04
01moogooOol501l2Ool301l6Ool0100?01l07`0O@Goo100O0goo1@0O1Woo00<03gooOol047oo0P0O
57oo1@0O1Woo00<07gooOol09goo00<007ooOol05Goo0014Ool3000<Ool201l2Ool201l^Ool201l2
Ool00`00Oomoo`0HOol00`0?Oomoo`04Ool00`00Oomoo`07Ool01000Oomoo`000Woo00<007oo0000
0Woo00H007ooOol007oo0002Ool00`00Ool00007Ool01000Oomoo`000Woo00<007oo00000Woo00<0
07oo00000Woo00H007ooOol007oo0002Ool00`00Oomoo`05Ool00`00Oomoo`0500000goo00moo`0Q
Ool00`0OOomoo`02Ool201l5Ool301l2Ool0100OOol07`0O1Woo0P0O17oo1P0O0Woo1@0O1goo00@0
3`0O01l07d5oo`@07`=oo`@07`Moo`0300moogoo011oo`807aEoo`@07bioo`<001Moo`00A7oo00<0
07ooOol037oo0P0O0Woo0P0O;Woo0P0O0Woo00<007ooOol067oo00<03`0O01l00Woo1@001goo00@0
07ooOol0009oo`03001oo`00009oo`06001oogoo001oo`000Woo00<007oo00001Goo1P000Woo00<0
07oo00000Woo00<007oo00000Woo00H007ooOol007oo0002Ool00`00Oomoo`05Ool00`00Oomoo`06
Ool00`0?Oomoo`0MOol501l2Ool501l2Ool401l017oo01l07goo0P0O1Woo0P0O17oo1@0O0goo1@0O
1goo00<03goo01l0@Woo0`0O1Goo0`0O1goo00<03gooOol09goo100O<7oo00<007ooOol05Goo0014
Ool00`00Oomoo`0<Ool201l2Ool201l^Ool201l2Ool00`00Oomoo`0>Ool201l8Ool401l=Ool01000
Oomoo`000Woo1P000Woo00<007oo00000Woo00<007oo00002Woo00H007ooOol007oo0002Ool00`00
Ool00002Ool01P00Oomoo`00Ool0009oo`03001oogoo00Eoo`03001oogoo00Ioo`0300moogoo01eo
o`D07`9oo`D07`9oo`807`04Ool07`0O01l2Ool201l6Ool201lHOol00`0?Oomoo`13Ool201l5Ool2
01l8Ool00`0?Oomoo`0WOol301laOol00`00Oomoo`0EOol004Aoo`03001oogoo00aoo`807`9oo`80
7bioo`807`9oo`03001oogoo00Aoo`807`Qoo`<07`9oo`807`=oo`807`03Ool07goo00aoo`04001o
ogoo0002Ool00`00Oomoo`02Ool01P00Oomoo`00Ool0009oo`03001oo`0000Yoo`06001oogoo001o
o`000Woo00<007oo00000Woo00H007ooOol007oo0002Ool00`00Oomoo`05Ool00`00Oomoo`06Ool0
0`0?Oomoo`0MOol0100OOomoogoo100O0goo0P0O00=oo`0O01l00Woo0P0O0Woo0P0O0Woo1P0O67oo
00<03gooOol0E7oo00<03gooOol09goo0`0O<Goo00<007ooOol05Goo0014Ool00`00Oomoo`0=Ool2
01l00goo01l07`0^Ool201l2Ool00`00Oomoo`04Ool301l7Ool301l2Ool201l3Ool201l00goo01l0
7`0<Ool70004Ool01P00Oomoo`00Ool0009oo`03001oo`0000Yoo`06001oogoo001oo`000Woo00<0
07oo00000Woo00H007ooOol007oo0002Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`0?Oomoo`0L
Ool201l3Ool401l4Ool0100OOol07`0O1Woo0P0O0Woo1@0O6Goo00<03gooOol0E7oo00<03gooOol0
:7oo0P0O<Goo00<007ooOol05Goo0014Ool00`00Oomoo`0=Ool201l00goo01l07`0^Ool201l2Ool0
0`00Oomoo`03Ool401l6Ool201l00goo01l07`0401l01Goo01l07`0OOol00P0O37oo00<007ooOol0
0goo00<007ooOol00Woo00H007ooOol007oo0002Ool00`00Ool0000:Ool01P00Oomoo`00Ool0009o
o`H0009oo`03001oo`00009oo`03001oogoo00=oo`D000Ioo`0300moogoo00Yoo`807a1oo`0301mo
ogoo00=oo`807`Eoo`@07`Ioo`807`03Ool07`0O01eoo`0300moogoo05Aoo`0300moogoo02Qoo`80
7c5oo`03001oogoo01Eoo`00A7oo00<007ooOol03Goo0P0O00=oo`0O01l0;Woo0P0O0Woo00<007oo
Ool00Woo0P0O00=oo`0O01l01Woo0P0O0Woo0`0O00=oo`0OOol00P0O0Woo0P0O37oo00<007ooOol0
0goo00<007ooOol00Woo00D007ooOol007oo00H000Yoo`06001oogoo001oo`000Woo00<007ooOol0
0Woo00<007ooOol01P0047oo00<03gooOol02Woo0P0O3goo0P0O3Goo0P0O27oo0`0O7Woo00<03goo
Ool0E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00Oomoo`0=Ool201l00goo01l07`0^
Ool201l2Ool00`00Oomoo`02Ool00`0OOomoo`0601l2Ool201l3Ool00`0OOomoo`0401l2Ool201l<
Ool00`00Oomoo`03Ool00`00Oomoo`02Ool01P00Oomoo`00Ool0009oo`H000Moo`06001oogoo001o
o`000Woo00<007ooOol00Woo00@007ooOol000Aoo`03001oogoo00ioo`0300moogoo00807`Qoo`<0
7`ioo`0301moogoo01Ioo`807amoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00
A7oo00<007ooOol03Goo0P0O00=oo`0O01l0;Woo0P0O0Woo00@007ooOomoo`807`=oo`H07`03Ool0
7`0O00Moo`807`0300moogoo00807`aoo`03001oogoo00=oo`03001oogoo009oo`06001oogoo001o
o`003goo1P000Woo00<007ooOol00Woo1P000Woo00<007ooOol03Woo00<03gooOol00P0O27oo0`0O
2Woo1@0O67oo0P0O7goo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00
Oomoo`0=Ool201l00goo01l07`0^Ool201l2Ool01000Oomoogoo0P0O1goo0P0O00=oo`0O01l02Goo
00@03gooOomoo`807`]oo`03001oogoo00=oo`03001oogoo009oo`06001oogoo001oo`003goo00<0
07ooOol00Woo00@007ooOol000Aoo`03001oogoo00Eoo`03001oogoo00ioo`0300moogoo00807`Qo
o`<07`Uoo`H07aQoo`807amoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00A7oo
0`003Goo0P0O00=oo`0O01l0;goo0P0O00=oo`00Ool00Woo0P0O1goo1@0O2Goo00@03gooOomoo`80
7`]oo`03001oogoo00=oo`03001oogoo009oo`06001oogoo001oo`003goo00<007ooOol00Woo00@0
07ooOol000Aoo`03001oogoo00Eoo`03001oogoo00ioo`0300moogoo00807`Moo`@07`Uoo`807ceo
o`0300moogoo05Aoo`0300moogoo05Uoo`<001Moo`00A7oo00<007ooOol03Goo0P0O00=oo`0O01l0
;goo0P0O00=oo`00Ool00Woo0P0O27oo100O2Goo00@03gooOomoo`807`]oo`03001oogoo00=oo`03
001oogoo009oo`06001oogoo001oo`003goo00<007ooOol00Woo00@007ooOol0009oo`D000Eoo`03
001oogoo00Ioo`807`Ioo`0300moo`0O00807`Moo`@07`Uoo`807ceoo`0300moogoo05Aoo`0300mo
ogoo05]oo`03001oogoo01Eoo`00A7oo00<007ooOol03Goo0P0O00=oo`0O01l0;goo0P0O00Aoo`00
Oomoo`807`Uoo`<07`Yoo`0400moogooOol201l9Ool60002Ool00`00Oomoo`02Ool01P00Oomoo`00
Ool000moo`03001oogoo009oo`04001oogoo000<Ool00`00Oomoo`06Ool301l5Ool00`0?Ool07`02
01l7Ool401l8Ool301lmOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03
001oogoo00eoo`807`03Ool07`0O02moo`807`@000807`Uoo`<07`Yoo`0400moogooOol201l8Ool2
01l7Ool00`00Oomoo`02Ool01P00Oomoo`00Ool000moo`03001oogoo009oo`04001oogoo000<Ool0
0`00Oomoo`06Ool301l5Ool00`0?Ool07`0301l6Ool201l00goo01l07`07Ool201lnOol00`0?Oomo
o`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001oogoo00eoo`807`03Ool07`0O02mo
o`807`Aoo`807`Yoo`807`Yoo`0400moogooOol201l7Ool301l7Ool00`00Oomoo`02Ool01P00Oomo
o`00Ool000moo`03001oogoo009oo`04001oogoo000:Ool50006Ool301l5Ool00`0?Ool07`0301l6
Ool201l00goo01l07`02Ool201l2Ool301lnOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomo
o`0EOol004Aoo`03001oogoo00eoo`807`03Ool07`0O02moo`807`Aoo`807`Yoo`807`Yoo`0400mo
ogooOol201l7Ool301l7Ool00`00Oomoo`02Ool01P00Oomoo`00Ool000eoo`D0009oo`04001oogoo
000EOol401l4Ool00`0?Ool07`0301l6Ool201l017oo01l07goo100O00=oo`0O01l0?goo00<03goo
Ool0E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00Oomoo`0=Ool401l`Ool201l4Ool2
01lFOol0100?Oomoogoo0P0O1goo0`0O1goo00<007ooOol00Woo00H007ooOol007oo000DOol01000
Oomoo`003Woo0P0O1Goo100O17oo00<03goo01l00`0O1Woo0P0O00Aoo`0O01moo`@07`03Ool07`0O
03moo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00A7oo00<007ooOol03Goo100O
<7oo0P0O17oo0P0O5Woo00@03gooOomoo`807`Ioo`@07`Moo`03001oogoo009oo`06001oogoo001o
o`0057oo00@007ooOol000ioo`807`Eoo`@07`Aoo`0300moo`0O00<07`Ioo`807`04Ool07`0OOol7
01loOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001oogoo00eoo`@0
7c1oo`807`Aoo`807aIoo`0400moogooOol201l6Ool401l7Ool00`00Oomoo`02Ool01P00Oomoo`00
Ool001Aoo`04001oogoo000>Ool201l4Ool201l00goo01l07`04Ool00`0?Ool07`0301l6Ool201l0
0goo01l07`0701m0Ool00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001o
ogoo00eoo`@07c1oo`807`Aoo`807aIoo`0300moogoo009oo`807`Eoo`807`03Ool07`0O00Ioo`03
001oogoo009oo`03001oogoo00D0019oo`04001oogoo000>Ool301l3Ool201l017oo01l07`0O0goo
00@03`0O01moo`807`Ioo`807`9oo`<07`03Ool07`0O00807d1oo`0300moogoo05Aoo`0300moogoo
05]oo`03001oogoo01Eoo`00A7oo0`003Goo100O<7oo0P0O0goo0P0O5goo00<03gooOol00Woo0P0O
17oo0`0O00=oo`0O01l00goo1P000Woo00@007ooOol001Ioo`04001oogoo000>Ool301l3Ool201l2
Ool301l2Ool0100?01l07goo0P0O1Woo0P0O0Woo0`0O0Woo0P0O@Goo00<03gooOol0E7oo00<03goo
Ool0FGoo0`005goo0014Ool00`00Oomoo`0=Ool401l`Ool201l3Ool201lGOol00`0?Oomoo`02Ool2
01l4Ool201l2Ool201l;Ool01000Oomoo`0057oo1P003Woo0`0O0goo0P0O0goo0`0O00Eoo`0?01l0
7goo00807`9oo`807`03Ool07`0O00=oo`807dIoo`0300moogoo05Aoo`0300moogoo05]oo`03001o
ogoo01Eoo`00A7oo00<007ooOol03Goo100O<7oo0P0O0goo0P0O5goo00<03gooOol00Woo0P0O0goo
0`0O0Woo0P0O2Goo1P006Goo00<007ooOol02goo100O0goo0P0O17oo0`0O00@03`0O01moo`807`03
Ool07`0O00@07`=oo`807dIoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00A7oo
00<007ooOol03Goo100O<7oo0P0O0goo0P0O5goo00<03gooOol00Woo0P0O0goo0P0O0goo0P0O3Woo
00<007ooOol05goo00<007ooOol02goo100O0goo0P0O1Goo1@0O00Aoo`0O01moo`807`04Ool07`0O
01m;Ool00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001oogoo00eoo`@0
7c1oo`807`=oo`807aMoo`0300moogoo009oo`807`=oo`807`=oo`807`ioo`03001oogoo01Moo`03
001oogoo00]oo`@07`=oo`807`Ioo`@07`04Ool07`0OOol201l017oo01l07`0OBgoo00<03gooOol0
E7oo00<03gooOol0Fgoo00<007ooOol05Goo000OOol40006Ool50006Ool5000;Ool00`00Oomoo`0=
Ool401l`Ool201l3Ool201lGOol00`0?Oomoo`02Ool201l3Ool201l3Ool201l>Ool00`00Oomoo`06
Ool201l?Ool00`00Oomoo`0;Ool201l00goo01l07`02Ool201l6Ool401l2Ool401l2Ool201m;Ool0
0`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol001eoo`L000Aoo`L000Aoo`L000Yoo`03
001oogoo00eoo`@07c1oo`807`=oo`807aMoo`0300moogoo009oo`807`=oo`807`=oo`807`ioo`03
001oogoo00Eoo`@07`ioo`03001oogoo00]oo`807`03Ool07`0O009oo`807`Ioo`@07`9oo`@07`9o
o`807d]oo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`007Goo0P000goo0P0017oo
0P000goo0P0017oo0P000goo0P002Woo00<007ooOol03Woo0`0O<7oo0P0O0goo0P0O5goo00<03goo
Ool00Woo0P0O0Woo0`0O0goo0P0O3Woo00<007ooOol01Goo00@07gooOol07`ioo`03001oogoo00]o
o`807`03Ool07`0O009oo`807`Ioo`<07`=oo`<07e1oo`0300moogoo05Aoo`0300moogoo05]oo`03
001oogoo01Eoo`0077oo0P001Goo0P000Woo0P001Goo0P000Woo0P001Goo0P002Goo00<007ooOol0
3Woo0`0O<7oo0P0O0goo0P0O5goo00<03gooOol00Woo0P0O0Woo0P0O17oo0P0O3Woo00<007ooOol0
17oo0P0O0Woo0P0O3Goo00<007ooOol02goo0P0O00Eoo`0O01l07goo00807`Moo`807`=oo`<07e1o
o`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`003Woo2P0017oo0P001Goo0P000Woo
0P001Goo0P000Woo0P001Goo0P002Goo00<007ooOol03Woo0`0O<7oo0P0O0goo0P0O5goo00<03goo
Ool00Woo0P0O0Woo0P0O17oo0P0O3Woo00<007ooOol00goo0P0O17oo100O2Woo00<007ooOol02goo
0P0O0Woo0P0O00=oo`0O01l01goo0P0O0goo0`0OD7oo00<03gooOol0E7oo00<03gooOol0Fgoo00<0
07ooOol05Goo000LOol20005Ool20002Ool20005Ool20002Ool20005Ool20009Ool4000=00l301l`
00l201l300l201lM00l201l00`0?01l03`0400l201l>00l00`0000l03`0300l201l400l401l:00l0
0`0000l03`0;00l201l200l201l00`0?01l07`0700l201l300l201oo00l400l4000GOol001aoo`80
00Eoo`80009oo`8000Eoo`80009oo`8000Eoo`8000Uoo`03001oogoo00ioo`807c5oo`807`=oo`80
7aMoo`0300moogoo00=oo`@07`Eoo`807`ioo`03001oogoo00=oo`807`Ioo`807`Yoo`03001oogoo
00]oo`807`9oo`807`03Ool07`0O00Moo`0300moogoo009oo`807e5oo`0300moogoo05Aoo`0300mo
ogoo05]oo`03001oogoo01Eoo`0077oo0`000goo0`000Woo0P001Goo0P000Woo0P001Goo0P002Goo
00<007ooOol03Woo0P0O<Goo0P0O0goo0P0O5goo00<03gooOol00goo0`0O1goo0P0O3Goo00<007oo
Ool00goo0P0O1Woo0P0O2Woo00<007ooOol02goo0P0O0Woo1@0O1goo00<03gooOol0EGoo00<03goo
Ool0E7oo00<03gooOol0Fgoo00<007ooOol05Goo000LOol80003Ool20005Ool20002Ool20005Ool2
0009Ool00`00Oomoo`0>Ool201laOol201l3Ool201lGOol00`0?Oomoo`03Ool201l8Ool201l=Ool0
0`00Oomoo`02Ool201l7Ool201l:Ool00`00Oomoo`0;Ool201l3Ool301l8Ool00`0?Oomoo`1EOol0
0`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol001aoo`800003Ool00000008000Aoo`80
00Eoo`80009oo`8000Eoo`8000Uoo`03001oogoo00ioo`807c5oo`807`=oo`807aMoo`0300moogoo
00=oo`807`Qoo`807`eoo`03001oogoo009oo`807`Qoo`807`Uoo`03001oogoo00]oo`807`=oo`<0
7`Qoo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`0077oo0P00
2Goo0P001Goo0P000Woo0P001Goo0P002Goo00<007ooOol03Woo0P0O<Goo0P0O0goo0P0O5goo00<0
3gooOol00goo0P0O27oo0P0O3Goo00<007ooOol00Woo0P0O27oo0P0O1goo1@002goo0P0O0goo0`0O
27oo00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo000MOol20004
Ool20002Ool20005Ool20002Ool20005Ool20009Ool00`00Oomoo`0>Ool201laOol201l2Ool201lH
Ool00`0?Oomoo`0=Ool201l=Ool00`00Oomoo`02Ool201l8Ool201lFOol201l4Ool301l8Ool00`0?
Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol001eoo`8000=oo`<000=o
o`8000=oo`8000Aoo`8000=oo`8000Yoo`03001oogoo00ioo`807c5oo`807`9oo`807aQoo`0300mo
ogoo00eoo`807`eoo`03001oogoo009oo`807`Qoo`807`Yoo`807`Yoo`807`Aoo`<07`Qoo`0300mo
ogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`007Woo1P0017oo1`0017oo
1`002Woo00<007ooOol03Woo0P0O<Goo0P0O0Woo0P0O67oo00<03gooOol03Goo0P0O3Goo00<007oo
Ool00Woo0P0O27oo0P0O2Woo0P0O1Woo0P0O0Woo0P0O1Goo0P0O27oo00<03gooOol0EGoo00<03goo
Ool0E7oo00<03gooOol0Fgoo00<007ooOol05Goo000OOol40006Ool50006Ool5000;Ool00`00Oomo
o`11Ool201l2Ool201lHOol00`0?Oomoo`0=Ool201l=Ool00`00Oomoo`02Ool201l8Ool201l:Ool3
01l5Ool201l2Ool201l5Ool201l8Ool00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol0
0`00Oomoo`0EOol004Aoo`<0045oo`807`9oo`807aQoo`0300moogoo00ioo`807`aoo`03001oogoo
009oo`807`Qoo`807`Yoo`@07`Aoo`807`9oo`807`Eoo`807`Qoo`0300moogoo05Eoo`0300moogoo
05Aoo`0300moogoo05Uoo`<001Moo`00A7oo00<007ooOol0@Goo0P0O0Woo0P0O67oo00<03gooOol0
3Woo0P0O37oo00@007ooOomoo`807`Uoo`807`=oo`807`Eoo`807`03Ool07goo009oo`@07`03Ool0
7`0O00moo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00A7oo
00<007ooOol0@Goo0P0O0Woo0P0O67oo00<03gooOol03Woo0P0O2Goo1P0000=oo`0O01l02Goo0P0O
0goo0P0O1Goo0P0O00=oo`0O01l00Woo100O00=oo`0O01l03goo00<03gooOol0EGoo00<03gooOol0
E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00Oomoo`11Ool201l2Ool201lHOol00`0?
Oomoo`0>Ool201l@Ool201l9Ool201l3Ool201l5Ool201l01Goo01l07`0OOol0100O00=oo`0O01l0
3goo00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00
Oomoo`11Ool201l2Ool201lHOol00`0?Oomoo`0>Ool201l@Ool201l:Ool201l017oo01l07`0O1Goo
0P0O0Woo0P0O00=oo`0O01l00P0O00=oo`0O01l03goo00<03gooOol0EGoo00<03gooOol0E7oo00<0
3gooOol0Fgoo00<007ooOol05Goo0014Ool00`00Oomoo`11Ool201l2Ool201lHOol00`0?Oomoo`0>
Ool201l@Ool201l:Ool201l017oo01l07`0O1Goo0P0O0Woo1`0O00=oo`0O01l03goo00<03gooOol0
EGoo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00Oomoo`12Ool201l0
0goo01l07`0HOol00`0?Oomoo`0?Ool201l>Ool201l;Ool201l017oo01l07`0O1Goo0P0O0goo1P0O
00=oo`0O01l03goo00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo
0014Ool00`00Oomoo`12Ool201l00goo01l07`0HOol00`0?Oomoo`0?Ool201l>Ool201l;Ool201l0
0goo01l07`0201l4Ool201l3Ool301l2Ool401l?Ool00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?
Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001oogoo049oo`807`03Ool07`0O01Qoo`0300moogoo
00moo`807`ioo`807`]oo`807`03Ool07`0O00807`=oo`807`Aoo`<07`9oo`@07`moo`0300moogoo
05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00A7oo00<007ooOol0@Woo0P0O
00=oo`0O01l067oo00<03gooOol03goo0P0O3Woo0P0O2goo0P0O00=oo`0O01l00P0O0goo0P0O1Goo
0P0O0Woo100O3goo00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo
0014Ool30012Ool201l00goo01l07`0HOol00`0?Oomoo`0?Ool201l>Ool201l;Ool401l00goo01l0
7`03Ool201l5Ool201l2Ool301l@Ool00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1IOol3
000GOol004Aoo`03001oogoo049oo`807`03Ool07`0O01Qoo`0300moogoo00moo`807`ioo`807`]o
o`@07`03Ool07`0O00=oo`807`Uoo`<07a1oo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo
05]oo`03001oogoo01Eoo`00A7oo00<007ooOol0@Woo0P0O00=oo`0O01l067oo00<03gooOol03goo
0P0O17oo0P0O1goo0P0O37oo100O00=oo`0O01l00goo0P0O2Goo0`0O47oo00<03gooOol0EGoo00<0
3gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00Oomoo`12Ool201l00goo01l0
7`0HOol00`0?Oomoo`0@Ool201l2Ool301l7Ool201l<Ool401l00goo01l07`03Ool201l:Ool201l@
Ool00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001o
ogoo049oo`807`03Ool07`0O01Qoo`0300moogoo011oo`807`9oo`@07`Ioo`807`eoo`<07`03Ool0
7`0O00=oo`807`Yoo`807a1oo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03001o
ogoo01Eoo`00A7oo00<007ooOol0@Woo0P0O00=oo`0O01l067oo00<03gooOol047oo0`0O00Eoo`0O
01moo`0O00Ioo`807`eoo`<07`03Ool07`0O00=oo`807`Yoo`807a1oo`0300moogoo05Eoo`0300mo
ogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00A7oo00<007ooOol0@Woo0P0O00=oo`0O01l0
67oo00<03gooOol047oo1P0O00=oo`0O01l01Goo0P0O3Goo0P0O0Woo0P0O0goo0P0O2Woo0P0O47oo
00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00Oomo
o`12Ool201l00goo01l07`0HOol00`0?Oomoo`0BOol301l3Ool00`0OOomoo`03Ool201l=Ool201l2
Ool201l3Ool201lLOol00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0E
Ool004Aoo`03001oogoo049oo`807`03Ool07`0O01Qoo`0300moogoo01=oo`807`=oo`807`=oo`80
7`ioo`807`9oo`807`=oo`807aaoo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03
001oogoo01Eoo`00A7oo00<007ooOol0@Woo0P0O00=oo`0O01l067oo00<03gooOol067oo0P0O0goo
0P0O4Woo0P0O0goo0P0O77oo00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0Fgoo00<007oo
Ool05Goo0014Ool30012Ool201l00goo01l07`0HOol00`0?Oomoo`0HOol201l3Ool201lBOol201l3
Ool201lLOol00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1IOol3000GOol004Aoo`03001o
ogoo049oo`807`03Ool07`0O01Qoo`0300moogoo01Qoo`807`=oo`807a=oo`807`9oo`807aaoo`03
00moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00A7oo00<007ooOol0
@Woo100O6Goo00<03gooOol067oo0P0O0goo0P0O4goo0P0O0Woo0P0O77oo00<03gooOol0EGoo00<0
3gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00Oomoo`12Ool401lIOol00`0?
Oomoo`0IOol201l2Ool201lCOol201l2Ool201lLOol00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?
Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001oogoo049oo`@07aUoo`0300moogoo01Uoo`807`9o
o`807a=oo`807`9oo`807aaoo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03001o
ogoo01Eoo`008Goo0P001Woo1@001Woo1@002goo00<007ooOol0@Woo100O6Goo00<03gooOol06Goo
0P0O0Woo0P0O4goo0P0O0Woo0P0O77oo00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0Fgoo
00<007ooOol05Goo000QOol20005Ool70004Ool7000:Ool00`00Oomoo`12Ool401lIOol00`0?Oomo
o`0IOol201l2Ool201lCOol201l2Ool201lLOol00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomo
o`1KOol00`00Oomoo`0EOol0025oo`8000Eoo`8000=oo`8000Aoo`8000=oo`8000Yoo`03001oogoo
049oo`@07aUoo`0300moogoo01Uoo`807`03Ool07`0O01Aoo`807`9oo`807aaoo`0300moogoo05Eo
o`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`006goo2P000Woo0P001Goo0P000Woo
0P001Goo0P002Goo00<007ooOol0@Woo100O6Goo00<03gooOol06Goo0P0O00=oo`0O01l057oo0P0O
0Woo0P0O77oo00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo000>
Ool:0003Ool:0002Ool20005Ool20002Ool20005Ool20009Ool00`00Oomoo`12Ool401lIOol00`0?
Oomoo`0IOol201l00goo01l07`0EOol201l00goo01l07`0LOol00`0?Oomoo`1EOol00`0?Oomoo`1D
Ool00`0?Oomoo`1KOol00`00Oomoo`0EOol001]oo`8000Aoo`8000Aoo`8000Eoo`80009oo`8000Eo
o`8000Uoo`@004403`@07cD03`807`0300l07`0O01D03`807`0300l07`0O0?l03bL03`@001Moo`00
77oo0P000goo0P0017oo0P001Goo0P000Woo0P001Goo0P002Goo00<007ooOol0@Woo100O6Goo00<0
3gooOol06Goo0P0O00=oo`0O01l05Goo100O7Goo00<03gooOol0EGoo00<03gooOol0E7oo00<03goo
Ool0Fgoo00<007ooOol05Goo000MOol20002Ool20004Ool20005Ool20002Ool20005Ool20009Ool0
0`00Oomoo`12Ool401lIOol00`0?Oomoo`0IOol201l00goo01l07`0EOol401lMOol00`0?Oomoo`1E
Ool00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol001eoo`80009oo`8000Aoo`8000Eo
o`80009oo`8000Eoo`8000Uoo`03001oogoo049oo`@07aUoo`0300moogoo01Uoo`807`03Ool07`0O
01Eoo`@07aeoo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00
7Woo0P0000=oo`00000017oo0P001Goo0P000Woo0P001Goo0P002Goo00<007ooOol0@Woo100O6Goo
00<03gooOol06Woo100O5Woo0`0O7Goo00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0Fgoo
00<007ooOol05Goo000OOol40004Ool20005Ool20002Ool20005Ool20009Ool00`00Oomoo`12Ool4
01lIOol00`0?Oomoo`0JOol401lFOol301lMOol00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomo
o`1KOol00`00Oomoo`0EOol0021oo`<000Aoo`8000Eoo`80009oo`8000Eoo`8000Uoo`03001oogoo
049oo`@07aUoo`0300moogoo01Yoo`@07aIoo`<07aeoo`0300moogoo05Eoo`0300moogoo05Aoo`03
00moogoo05]oo`03001oogoo01Eoo`0087oo0`001Goo0P000goo0P0017oo0P000goo0P002Woo00<0
07ooOol0@Woo100O6Goo00<03gooOol06Woo100O5Woo0`0O7Goo00<03gooOol0EGoo00<03gooOol0
E7oo00<03gooOol0Fgoo00<007ooOol05Goo000QOol20005Ool70004Ool7000:Ool00`00Oomoo`12
Ool401lIOol00`0?Oomoo`0JOol401lFOol301lMOol00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?
Oomoo`1KOol00`00Oomoo`0EOol0029oo`03001oogoo00Aoo`D000Ioo`D000]oo`03001oogoo049o
o`@07aUoo`0300moogoo01Yoo`@07aIoo`<07aeoo`0300moogoo05Eoo`0300moogoo05Aoo`0300mo
ogoo05]oo`03001oogoo01Eoo`00A7oo0`00@Woo100O6Goo00<03gooOol06Woo100O5goo0P0O7Goo
00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0FGoo0`005goo0014Ool00`00Oomoo`12Ool4
01lIOol00`0?Oomoo`0JOol401lGOol201lMOol00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomo
o`1KOol00`00Oomoo`0EOol004Aoo`03001oogoo04=oo`<07aUoo`0300moogoo01Yoo`@07aMoo`80
7aeoo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00A7oo00<0
07ooOol0@goo0`0O6Goo00<03gooOol06Woo0`0O=goo00<03gooOol0EGoo00<03gooOol0E7oo00<0
3gooOol0Fgoo00<007ooOol05Goo0014Ool00`00Oomoo`13Ool301lIOol00`0?Oomoo`0JOol301lg
Ool00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001o
ogoo04=oo`<07aUoo`0300moogoo01]oo`807cMoo`0300moogoo05Eoo`0300moogoo05Aoo`0300mo
ogoo05]oo`03001oogoo01Eoo`00A7oo00<007ooOol0@goo0`0O6Goo00<03gooOol06goo0P0O=goo
00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00Oomo
o`13Ool301lIOol00`0?Oomoo`0KOol201lgOol00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomo
o`1KOol00`00Oomoo`0EOol004Aoo`03001oogoo04=oo`807aYoo`0300moogoo01]oo`807cMoo`03
00moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00A7oo00<007ooOol0
@goo0P0O6Woo00<03gooOol06goo0P0O=goo00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0
Fgoo00<007ooOol05Goo0014Ool30013Ool201lJOol00`0?Oomoo`0KOol201lgOol00`0?Oomoo`1E
Ool00`0?Oomoo`1DOol00`0?Oomoo`1IOol3000GOol004Aoo`03001oogoo04=oo`807aYoo`0300mo
ogoo05Aoo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00A7oo
00<007ooOol0@goo0P0O6Woo00<03gooOol0E7oo00<03gooOol0EGoo00<03gooOol0E7oo00<03goo
Ool0Fgoo00<007ooOol05Goo0014Ool00`00Oomoo`13Ool201lJOol00`0?Oomoo`1DOol00`0?Oomo
o`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001oogoo04=oo`80
7aYoo`0300moogoo05Aoo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo
01Eoo`00A7oo00<007ooOol0@goo0P0O6Woo00<03gooOol0E7oo00<03gooOol0EGoo00<03gooOol0
E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00Oomoo`13Ool201lJOol00`0?Oomoo`1D
Ool00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001o
ogoo04=oo`807aYoo`0300moogoo05Aoo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]o
o`03001oogoo01Eoo`00A7oo00<007ooOol0@goo0P0O6Woo00<03gooOol0E7oo00<03gooOol0EGoo
00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00Oomoo`13Ool201lJOol0
0`0?Oomoo`1DOol00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol0
04Aoo`<004=oo`807aYoo`0300moogoo05Aoo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo
05Uoo`<001Moo`00A7oo00<007ooOol0Ggoo00<03gooOol0E7oo00<03gooOol0EGoo00<03gooOol0
E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00Oomoo`1OOol00`0?Oomoo`1DOol00`0?
Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001oogoo05mo
o`0300moogoo05Aoo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eo
o`00A7oo00<007ooOol0Ggoo00<03gooOol0E7oo00<03gooOol0EGoo00<03gooOol0E7oo00<03goo
Ool0Fgoo00<007ooOol05Goo000LOol90004Ool50006Ool5000;Ool00`00Oomoo`1OOol00`0?Oomo
o`1DOol00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol001aoo`T0
00=oo`L000Aoo`L000Yoo`03001oogoo05moo`0300moogoo05Aoo`0300moogoo05Eoo`0300moogoo
05Aoo`0300moogoo05]oo`03001oogoo01Eoo`007Goo0P002Goo0P000goo0P0017oo0P000goo0P00
2Woo00<007ooOol0Ggoo00<03gooOol0E7oo00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0
Fgoo00<007ooOol05Goo000NOol20007Ool20005Ool20002Ool20005Ool20009Ool00`00Oomoo`1O
Ool00`0?Oomoo`1DOol00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0E
Ool000ioo`X000Moo`8000Ioo`8000Eoo`80009oo`8000Eoo`8000Uoo`03001oogoo05moo`0300mo
ogoo05Aoo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`0087oo
0P001Goo0P001Goo0P000Woo0P001Goo0P002Goo1000o`0?`00?10005goo000QOol20004Ool20005
Ool20002Ool20005Ool20009Ool00`00Oomoo`1OOol00`0?Oomoo`1DOol00`0?Oomoo`1EOol00`0?
Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol0029oo`8000=oo`8000Eoo`80009oo`8000Eo
o`8000Uoo`03001oogoo05moo`0300moogoo05Aoo`0300moogoo05Eoo`0300moogoo05Aoo`0300mo
ogoo05]oo`03001oogoo01Eoo`008goo00@007ooOomoo`8000Eoo`80009oo`8000Eoo`8000Uoo`03
001oogoo05moo`0300moogoo05Aoo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03
001oogoo01Eoo`008goo0P000Woo0P001Goo0P000Woo0P001Goo0P002Goo00<007ooOol0Ggoo00<0
3gooOol0E7oo00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo000S
Ool20002Ool20005Ool20002Ool20005Ool20009Ool00`00Oomoo`1OOol00`0?Oomoo`1DOol00`0?
Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol001aoo`8000Eoo`80009o
o`8000Eoo`80009oo`8000Eoo`8000Uoo`03001oogoo05moo`0300moogoo05Aoo`0300moogoo05Eo
o`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`007Goo0P0017oo0P000goo0P000goo
0P0017oo0P000goo0P002Woo00<007ooOol0Ggoo00<03gooOol0E7oo00<03gooOol0EGoo00<03goo
Ool0E7oo00<03gooOol0Fgoo00<007ooOol05Goo000MOol70004Ool70004Ool7000:Ool00`00Oomo
o`1OOol00`0?Oomoo`1DOol00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomo
o`0EOol001ioo`D000Ioo`D000Ioo`D000]oo`03001oogoo05moo`0300moogoo05Aoo`0300moogoo
05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03001oogoo01Eoo`00A7oo0`00Ggoo00<03gooOol0
E7oo00<03gooOol0EGoo00<03gooOol0E7oo00<03gooOol0FGoo0`005goo0014Ool00`00Oomoo`1O
Ool00`0?Oomoo`1DOol00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0E
Ool004Aoo`03001oogoo05moo`0300moogoo05Aoo`0300moogoo05Eoo`0300moogoo05Aoo`0300mo
ogoo05]oo`03001oogoo01Eoo`00A7oo00<007ooOol0Ggoo00<03gooOol0E7oo00<03gooOol0EGoo
00<03gooOol0E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00Oomoo`1OOol00`0?Oomo
o`1DOol00`0?Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03
001oogoo05moo`0300moogoo05Aoo`0300moogoo05Eoo`0300moogoo05Aoo`0300moogoo05]oo`03
001oogoo01Eoo`00A7oo00<007ooOol0Ggoo00<03gooOol0E7oo00<03gooOol0EGoo00<03gooOol0
E7oo00<03gooOol0Fgoo00<007ooOol05Goo0014Ool00`00Oomoo`1OOol00`0?Oomoo`1DOol00`0?
Oomoo`1EOol00`0?Oomoo`1DOol00`0?Oomoo`1KOol00`00Oomoo`0EOol004Aoo`03001oogoo00Qo
o`03001oogoo05Aoo`03001oogoo05Aoo`03001oogoo05Eoo`03001oogoo05Aoo`03001oogoo05Ao
o`03001oogoo00Aoo`03001oogoo01Eoo`00A7oo00<007ooOol027oo00<007ooOol04Woo00<007oo
Ool04goo00<007ooOol04goo00<007ooOol04goo00<007ooOol04goo00<007ooOol04goo00<007oo
Ool04goo00<007ooOol04Woo00<007ooOol04goo00<007ooOol04goo00<007ooOol04goo00<007oo
Ool04goo00<007ooOol04goo00<007ooOol04Woo00<007ooOol04goo00<007ooOol04goo00<007oo
Ool04goo00<007ooOol04goo00<007ooOol04goo00<007ooOol04Woo00<007ooOol017oo00<007oo
Ool05Goo0014Ool00`00Oomoo`08Ool00`00Oomoo`0BOol00`00Oomoo`0COol00`00Oomoo`0COol0
0`00Oomoo`0COol00`00Oomoo`0COol00`00Oomoo`0COol00`00Oomoo`0COol00`00Oomoo`0BOol0
0`00Oomoo`0COol00`00Oomoo`0COol00`00Oomoo`0COol00`00Oomoo`0COol00`00Oomoo`0COol0
0`00Oomoo`0BOol00`00Oomoo`0COol00`00Oomoo`0COol00`00Oomoo`0COol00`00Oomoo`0COol0
0`00Oomoo`0COol00`00Oomoo`0BOol00`00Oomoo`04Ool00`00Oomoo`0EOol004Aoool00<P001Mo
o`00ogooogoo97oo003oOoooOolTOol00?mooomoobAoo`00ogooogoo97oo003oOoooOolTOol00?mo
oomoobAoo`00ogooogoo97oo003oOoooOolTOol00?mooomoobAoo`00ogooogoo97oo003oOoooOolT
Ool00?mooomoobAoo`00\
\>"],
  ImageRangeCache->{{{0, 545.938}, {336.875, 0}} -> {-1810.99, -1715.62, 
  34.3355, 7.48228}}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Example 3: Compute the total data rate", "Subsection"],

Cell["\<\
We define some functions to extract the bytes per second value\
\>", "Text"],

Cell[BoxData[{
    \(bytes[n_Integer] := 
      Switch[n, 1, 2, \ \ 2, 4, \ 3, 8, \ 4, 4, \ 5, 8, \ _, \ 0]\), 
    \(BPS[rate_Integer, dtype_Integer] := rate*bytes[dtype]\)}], "Input"],

Cell["Now we get the channel information", "Text"],

Cell[BoxData[
    \(\(info = NDSGetChannelInfo[Site \[Rule] LHO]; \)\)], "Input"],

Cell["and compute the data rate", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plus@@\((\(\((BPS[DataRate, NDSDataType] /. #)\)&\)/@info)\)/1024. \^2 
      "\<MB/s\>"\)], "Input"],

Cell[BoxData[
    \(17.8281669616699218`\ "MB/s"\)], "Output"]
}, Open  ]],

Cell["\<\
Oops! We mistakenly included all digital test points. Since digital test \
points belong to either Data Collection Units 13 to 16, or 25 to 28, we get a \
better estimate of the effective data rate of the data acqusition system by \
using\
\>", "Text"],

Cell[BoxData[{
    \(DCUYes[dcu_Integer] := 
      Which[13 \[LessEqual] dcu \[LessEqual] 16, 0, 
        25 \[LessEqual] dcu \[LessEqual] 28, 0, True, \ 1]\), 
    \(BPS[rate_Integer, dtype_Integer, \ dcu_Integer] := 
      rate*bytes[dtype]*DCUYes[dcu]\)}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plus@@\((\(\((BPS[DataRate, NDSDataType, DCU] /. #)\)&\)/@info)\)/
        1024^2. \ "\<MB/s\>"\)], "Input"],

Cell[BoxData[
    \(5.20316696166992187`\ "MB/s"\)], "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Reference", "Section"],

Cell[CellGroupData[{

Cell["ChannelName", "Subsubsection"],

Cell[TextData[{
  StyleBox["ChannelName",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is an options used by ",
  StyleBox["NDSGetChannelInfo",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " to return the channel name."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["ColumnFormat", "Subsubsection"],

Cell[TextData[{
  StyleBox["ColumnFormat",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is an option value specify the format of the returned data. See ",
  StyleBox["DataFormat",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["DataFormat", "Subsubsection"],

Cell[TextData[{
  "The option ",
  StyleBox["DataFormat",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " specifies the format of the returned data array. Possible values are ",
  StyleBox["RowFormat",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " or ",
  StyleBox["ColumnFormat",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ". In row format the data is returned as a list of n-tuples representing \
the time, the value of the first channel, the value of the second channel, \
etc. In column format the data is returned as a list of channels data arrays. \
The first data array always describes the time. "
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["DataRate", "Subsubsection"],

Cell[TextData[{
  StyleBox["DataRate",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is an option to specify the sampling rate of the returned data array. The \
value is specified in units of Hz."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["DCU", "Subsubsection"],

Cell[TextData[{
  StyleBox["DCU",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is an options used by ",
  StyleBox["NDSGetChannelInfo",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " to return the DCU information."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Heterodyne", "Subsubsection"],

Cell[TextData[{
  "The ",
  StyleBox["Heterodyne",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " option is used to down-convert the data by the specified rate before it \
is returned. The value is specified in units of Hz. The returned data values \
will be complex."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["InterferometerId", "Subsubsection"],

Cell[TextData[{
  StyleBox["InterferometerId",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is an options used by ",
  StyleBox["NDSGetChannelInfo",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " to return the interferometer identification number."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["LHO", "Subsubsection"],

Cell[TextData[{
  "The option value ",
  StyleBox["LHO",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is used to represente the site of the LIGO Hanford Observatory. See ",
  StyleBox["Site",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["LLO", "Subsubsection"],

Cell[TextData[{
  "The option value ",
  StyleBox["LHO",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is used to represente the site of the LIGO Hanford Observatory. See ",
  StyleBox["Site",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["NDSClose", "Subsubsection"],

Cell[TextData[{
  StyleBox["NDSClose",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " terminates the connection to the network data server."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["NDSDataType", "Subsubsection"],

Cell[TextData[{
  StyleBox["NDSDataType",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is an options used by ",
  StyleBox["NDSGetChannelInfo",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " to return the native data type."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["NDSGetChannel", "Subsubsection",
  ParagraphSpacing->{0, -3}],

Cell[TextData[{
  StyleBox["NDSGetChannel[]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " obtains a list of all currently available channel names. The following \
options can be given:\n",
  StyleBox["Site\t\t$DefaultSite",
    FontFamily->"Courier",
    FontSize->11,
    FontWeight->"Bold"],
  StyleBox["\t",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "specifies the location of the NDS server."
}], "Text",
  ParagraphSpacing->{0, 6}]
}, Open  ]],

Cell[CellGroupData[{

Cell["NDSGetChannelInfo", "Subsubsection",
  ParagraphSpacing->{0, -3}],

Cell[TextData[{
  StyleBox["NDSGetChannelInfo[]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " obtains a list of all currently available channel names and the \
information assciated with them. Each channel is represented by a list with \
the following elements:\n",
  StyleBox[
  "{ChannelName\[Rule]\"channel\", DataRate\[Rule]rate, InterferometerId\
\[Rule]ifo,\n DCU\[Rule]id, NDSDataType\[Rule]dtype}",
    FontFamily->"Courier",
    FontSize->11,
    FontWeight->"Bold"],
  ".\nThe data rate is specified in units of Hz; the interferometer number is \
either 0 for the 4k intereformeter or 1 for the 2k; the DCU id is the \
idendification number of data collection unit which collected the channel; \
and the data type is one of the following: 1 \[Dash] short integer (16 bit), \
2 \[Dash] integer (32 bit), 3 \[Dash] long integer (64 bit), 4 \[Dash] single \
precision floating point number (32 bit), and 5 \[Dash] double precision \
floating point number (64 bit).\nThe following options can be given:\n",
  StyleBox["Site\t\t$DefaultSite",
    FontFamily->"Courier",
    FontSize->11,
    FontWeight->"Bold"],
  StyleBox["\t",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "specifies the location of the NDS server."
}], "Text",
  ParagraphSpacing->{0, 6}]
}, Open  ]],

Cell[CellGroupData[{

Cell["NDSGetData", "Subsubsection",
  ParagraphSpacing->{0, -3}],

Cell[TextData[{
  StyleBox["NDSGetData[\"channel\", start, duration]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " obtains data for a single channel from the network data server.\n",
  StyleBox["NDSGetData[{\"chn1\", \"chn2\", ...}, start, duration]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " obtains data for multiple channels from the network data server.",
  StyleBox["\n",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "The following options can be given:\n",
  StyleBox["DataRate\t256",
    FontFamily->"Courier",
    FontSize->11,
    FontWeight->"Bold"],
  "\t\t\tspecifies the sampling rate of the returned data\n",
  StyleBox["DataFormat\tColumnForma",
    FontFamily->"Courier",
    FontSize->11,
    FontWeight->"Bold"],
  StyleBox["t",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "\tspecifies the format of the returned data\n",
  StyleBox["Heterodyne\t0",
    FontFamily->"Courier",
    FontSize->11,
    FontWeight->"Bold"],
  "\t\t\tspecifies the down-conversion ",
  Cell[BoxData[
      \(TraditionalForm\`frequency\)]],
  "\n",
  StyleBox["Site\t\t$DefaultSite",
    FontFamily->"Courier",
    FontSize->11,
    FontWeight->"Bold"],
  StyleBox["\t",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "specifies the location of the NDS server."
}], "Text",
  ParagraphSpacing->{0, 6}]
}, Open  ]],

Cell[CellGroupData[{

Cell["NDSGetTrend", "Subsubsection"],

Cell[TextData[{
  StyleBox["NDSGetTrend[\"channel\", start, duration]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " obtains trend data for a single channel from the network data server.\n",
  StyleBox["NDSGetTrend[{\"chn1\", \"chn2\", ...}, start, duration]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " obtains trend data for multiple channels from the network data server.\n\
Each returned data point constist of an n-tuple consisting of the time, the \
number of points, the mean value, the standard deviation, the minimum and the \
maximum. The returned data format is always in row format. The data rate has \
to be smaller or equal one Hertz. For rates faster than once a minute the \
second trend information is used, whereas for rates equal or slower than once \
the minute trend information is used. The following options can be given:\n",
  StyleBox["DataRate\t1",
    FontFamily->"Courier",
    FontSize->11,
    FontWeight->"Bold"],
  "\t\t\tspecifies the sampling rate of the returned data\n",
  StyleBox["Site\t\t$DefaultSite",
    FontFamily->"Courier",
    FontSize->11,
    FontWeight->"Bold"],
  StyleBox["\t",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "specifies the location of the NDS server."
}], "Text",
  ParagraphSpacing->{0, 6}]
}, Open  ]],

Cell[CellGroupData[{

Cell["NDSTrendEliminateInvalid", "Subsubsection"],

Cell[TextData[{
  StyleBox["NDSTrendEliminateInvalid[]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " eliminates invalid data points from a returned trend data array. Invalid \
data points are recognized by containing an number of points of zero."
}], "Text",
  ParagraphSpacing->{0, 6}]
}, Open  ]],

Cell[CellGroupData[{

Cell["RowFormat", "Subsubsection"],

Cell[TextData[{
  StyleBox["RowFormat",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is an option value specifying the format of the returned data. See ",
  StyleBox["DataFormat",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["SimpleFormat", "Subsubsection"],

Cell[TextData[{
  StyleBox["SimpleFormat",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is an option value specifying the format of the returned trend data. If \
",
  StyleBox["True",
    FontWeight->"Bold"],
  ", the returned data array is a list of duplets representing the time stamp \
and the mean trend value. If ",
  StyleBox["False",
    FontWeight->"Bold"],
  ", the returned data array is a list of sixtuplets which represent the time \
stamp, the number of data points used for the averaging, the mean trend \
value, the standard deviation, the minimum value and the maximum value."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Site", "Subsubsection"],

Cell[TextData[{
  "Site is an option which is used to specify the loaction of the data \
server. Supported values are ",
  StyleBox["LHO",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " or ",
  StyleBox["LLO",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " or a string describing the port number and server name in the format \
\"1234@daniel.ligo-wa.caltech.edu\"."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["$DefaultSite", "Subsubsection"],

Cell[TextData[{
  StyleBox["$DefaultSite",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is used to specify the default location of the NDS data server."
}], "Text"]
}, Open  ]]
}, Closed]]
},
FrontEndVersion->"5.2 for Microsoft Windows",
ScreenRectangle->{{0, 1920}, {0, 1113}},
WindowToolbars->{"RulerBar", "EditBar"},
WindowSize->{610, 891},
WindowMargins->{{380, Automatic}, {Automatic, 15}},
StyleDefinitions -> Notebook[{
  
  Cell[CellGroupData[{
  Cell["Style Definitions", "Subtitle"],
  
  Cell["\<\
Modify the definitions below to change the default appearance of all cells in \
a given style.  Make modifications to any definition using commands in the \
Format menu.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell["Style Environment Names", "Section"],
  
  Cell[StyleData[All, "Working"],
    PageWidth->WindowWidth,
    ScriptMinSize->9],
  
  Cell[StyleData[All, "Presentation"],
    PageWidth->WindowWidth,
    ScriptMinSize->12,
    FontSize->16],
  
  Cell[StyleData[All, "Condensed"],
    PageWidth->WindowWidth,
    CellBracketOptions->{"Margins"->{1, 1},
    "Widths"->{0, 5}},
    ScriptMinSize->8,
    FontSize->11],
  
  Cell[StyleData[All, "Printout"],
    PageWidth->PaperWidth,
    ScriptMinSize->7,
    FontSize->10,
    PrivateFontOptions->{"FontType"->"Outline"}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Notebook Options", "Section"],
  
  Cell["\<\
The options defined for the style below will be used at the Notebook level.\
\>", "Text"],
  
  Cell[StyleData["Notebook"],
    PageHeaders->{{Cell[ 
            TextData[ {
              CounterBox[ "Page"]}], "PageNumber"], None, Cell[ 
            TextData[ {
              ValueBox[ "FileName"]}], "Header"]}, {Cell[ 
            TextData[ {
              ValueBox[ "FileName"]}], "Header"], None, Cell[ 
            TextData[ {
              CounterBox[ "Page"]}], "PageNumber"]}},
    PageHeaderLines->{True, True},
    PrintingOptions->{"FirstPageHeader"->False,
    "FacingPages"->True},
    CellFrameLabelMargins->6,
    StyleMenuListing->None]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Headings", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Title"],
    ShowCellBracket->False,
    CellMargins->{{0, 0}, {0, 0}},
    PageBreakBelow->False,
    LineSpacing->{1, 0},
    CounterIncrements->"Title",
    CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
        "Subtitle", 0}, {"Subsubtitle", 0}},
    FontSize->34,
    FontColor->GrayLevel[1],
    Background->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["Title", "Presentation"],
    CellMargins->{{0, 0}, {0, 0}},
    LineSpacing->{1, 0},
    FontSize->44],
  
  Cell[StyleData["Title", "Condensed"],
    CellMargins->{{0, 0}, {0, 0}},
    FontSize->20],
  
  Cell[StyleData["Title", "Printout"],
    CellMargins->{{0, 0}, {0, 0}},
    FontSize->24,
    FontTracking->"Plain",
    Background->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subtitle"],
    ShowCellBracket->False,
    CellMargins->{{0, 0}, {0, 0}},
    PageBreakBelow->False,
    LineSpacing->{1, 3},
    ParagraphIndent->-96,
    CounterIncrements->"Subtitle",
    CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
        "Subsubtitle", 0}},
    FontFamily->"Helvetica",
    FontSize->18,
    FontColor->GrayLevel[1],
    Background->RGBColor[0.2, 0.700008, 0.700008]],
  
  Cell[StyleData["Subtitle", "Presentation"],
    CellMargins->{{0, 0}, {0, 0}},
    LineSpacing->{1, 6},
    ParagraphIndent->-157,
    FontSize->30],
  
  Cell[StyleData["Subtitle", "Condensed"],
    CellMargins->{{0, 0}, {0, 0}},
    ParagraphIndent->-78,
    FontSize->14],
  
  Cell[StyleData["Subtitle", "Printout"],
    CellMargins->{{0, 0}, {0, 0}},
    ParagraphIndent->-85,
    FontSize->16,
    Background->GrayLevel[0.6]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subsubtitle"],
    ShowCellBracket->False,
    CellMargins->{{10, 4}, {30, 10}},
    PageBreakBelow->False,
    CounterIncrements->"Subsubtitle",
    CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}},
    FontFamily->"Helvetica",
    FontSize->14,
    FontSlant->"Italic"],
  
  Cell[StyleData["Subsubtitle", "Presentation"],
    CellMargins->{{8, 10}, {40, 20}},
    LineSpacing->{1, 0},
    FontSize->24],
  
  Cell[StyleData["Subsubtitle", "Condensed"],
    CellMargins->{{8, 10}, {12, 8}},
    FontSize->12],
  
  Cell[StyleData["Subsubtitle", "Printout"],
    CellMargins->{{9, 10}, {50, 10}},
    FontSize->14]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Section"],
    CellFrame->{{6, 0}, {0, 1}},
    CellDingbat->None,
    CellMargins->{{12, Inherited}, {4, 24}},
    CellGroupingRules->{"SectionGrouping", 30},
    PageBreakBelow->False,
    CellFrameMargins->6,
    CounterIncrements->"Section",
    CounterAssignments->{{"Subsection", 0}, {"Subsubsection", 0}},
    FontFamily->"Helvetica",
    FontSize->16,
    FontWeight->"Bold",
    FontColor->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["Section", "Presentation"],
    CellMargins->{{10, 10}, {8, 32}},
    LineSpacing->{1, 2},
    FontSize->24,
    FontTracking->"Condensed"],
  
  Cell[StyleData["Section", "Condensed"],
    CellMargins->{{8, Inherited}, {2, 12}},
    FontSize->12],
  
  Cell[StyleData["Section", "Printout"],
    CellMargins->{{9, 0}, {2, 50}},
    FontSize->14,
    FontTracking->"Plain",
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subsection"],
    CellMargins->{{12, Inherited}, {8, 20}},
    CellGroupingRules->{"SectionGrouping", 40},
    PageBreakBelow->False,
    CounterIncrements->"Subsection",
    CounterAssignments->{{"Subsubsection", 0}},
    FontFamily->"Helvetica",
    FontSize->13,
    FontWeight->"Bold"],
  
  Cell[StyleData["Subsection", "Presentation"],
    CellMargins->{{11, 10}, {8, 32}},
    LineSpacing->{1, 0},
    FontSize->22],
  
  Cell[StyleData["Subsection", "Condensed"],
    CellMargins->{{8, Inherited}, {2, 12}},
    FontSize->12],
  
  Cell[StyleData["Subsection", "Printout"],
    CellMargins->{{9, 0}, {4, 40}},
    FontSize->12]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subsubsection"],
    CellDingbat->"\[FilledSquare]",
    CellMargins->{{25, Inherited}, {0, 16}},
    CellElementSpacings->{"ClosedGroupTopMargin"->4},
    CellGroupingRules->{"SectionGrouping", 50},
    PageBreakBelow->False,
    LineSpacing->{1, 0},
    ParagraphSpacing->{0, 0},
    CounterIncrements->"Subsubsection",
    FontFamily->"Courier",
    FontSize->13,
    FontWeight->"Bold"],
  
  Cell[StyleData["Subsubsection", "Presentation"],
    CellMargins->{{29, 10}, {0, 16}},
    CellElementSpacings->{"ClosedGroupTopMargin"->4},
    LineSpacing->{0.1, 0},
    ParagraphSpacing->{0, 0},
    FontSize->18],
  
  Cell[StyleData["Subsubsection", "Condensed"],
    CellMargins->{{22, Inherited}, {0, 16}},
    CellElementSpacings->{"ClosedGroupTopMargin"->4},
    LineSpacing->{1, 0},
    ParagraphSpacing->{0, 0},
    FontSize->10],
  
  Cell[StyleData["Subsubsection", "Printout"],
    CellMargins->{{21, 0}, {0, 16}},
    CellElementSpacings->{"ClosedGroupTopMargin"->4},
    LineSpacing->{1, 0},
    ParagraphSpacing->{0, 0},
    FontSize->11]
  }, Open  ]]
  }, Open  ]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Body Text", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Text"],
    CellMargins->{{12, 10}, {5, 5}},
    TextAlignment->Left,
    TextJustification->1,
    LineSpacing->{1, 3},
    ParagraphSpacing->{0, 12},
    CounterIncrements->"Text",
    FontFamily->"Times"],
  
  Cell[StyleData["Text", "Presentation"],
    CellMargins->{{13, 10}, {8, 8}},
    TextAlignment->Left,
    TextJustification->1,
    LineSpacing->{1, 5},
    ParagraphSpacing->{0, 12}],
  
  Cell[StyleData["Text", "Condensed"],
    CellMargins->{{8, 10}, {4, 4}},
    TextAlignment->Left,
    TextJustification->1,
    LineSpacing->{1, 1},
    ParagraphSpacing->{0, 4}],
  
  Cell[StyleData["Text", "Printout"],
    CellMargins->{{9, 0}, {4, 4}},
    TextAlignment->Left,
    TextJustification->1,
    ParagraphSpacing->{0, 6}]
  }, Open  ]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["SmallText"],
    CellMargins->{{12, 10}, {5, 5}},
    TextAlignment->Left,
    TextJustification->1,
    LineSpacing->{1, 3},
    ParagraphSpacing->{0, 6},
    CounterIncrements->"SmallText",
    FontFamily->"Times",
    FontSize->9],
  
  Cell[StyleData["SmallText", "Presentation"],
    CellMargins->{{13, 10}, {8, 8}},
    TextAlignment->Left,
    TextJustification->1,
    LineSpacing->{1, 5},
    FontSize->12],
  
  Cell[StyleData["SmallText", "Condensed"],
    CellMargins->{{8, 10}, {2, 2}},
    TextAlignment->Left,
    TextJustification->1,
    LineSpacing->{1, 2},
    FontSize->9],
  
  Cell[StyleData["SmallText", "Printout"],
    CellMargins->{{9, 0}, {4, 4}},
    TextAlignment->Left,
    TextJustification->1,
    FontSize->7]
  }, Open  ]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Input/Output", "Section"],
  
  Cell["\<\
The cells in this section define styles used for input and output to the \
kernel.  Be careful when modifying, renaming, or removing these styles, \
because the front end associates special meanings with these style names.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Input"],
    CellFrame->{{3, 0}, {0, 0}},
    CellMargins->{{52, 10}, {8, 8}},
    Evaluatable->True,
    CellGroupingRules->"InputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    CellLabelMargins->{{5, Inherited}, {Inherited, Inherited}},
    DefaultFormatType->DefaultInputFormatType,
    FormatType->InputForm,
    ShowStringCharacters->True,
    NumberMarks->True,
    CounterIncrements->"Input",
    FontWeight->"Bold",
    Background->RGBColor[1, 0.700008, 0.4]],
  
  Cell[StyleData["Input", "Presentation"],
    CellMargins->{{62, Inherited}, {10, 10}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Input", "Condensed"],
    CellMargins->{{40, 10}, {4, 4}}],
  
  Cell[StyleData["Input", "Printout"],
    CellMargins->{{44, 0}, {6, 6}},
    Background->GrayLevel[0.8]]
  }, Closed]],
  
  Cell[StyleData["InlineInput"],
    Evaluatable->True,
    CellGroupingRules->"InputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    DefaultFormatType->DefaultInputFormatType,
    AutoItalicWords->{},
    FormatType->InputForm,
    ShowStringCharacters->True,
    NumberMarks->True,
    CounterIncrements->"Input",
    FontWeight->"Bold"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Output"],
    CellFrame->{{3, 0}, {0, 0}},
    CellMargins->{{52, 10}, {8, 8}},
    CellEditDuplicate->True,
    CellGroupingRules->"OutputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    CellLabelMargins->{{3, Inherited}, {Inherited, Inherited}},
    DefaultFormatType->DefaultOutputFormatType,
    FormatType->InputForm,
    CounterIncrements->"Output",
    Background->RGBColor[1, 0.900008, 0.900008]],
  
  Cell[StyleData["Output", "Presentation"],
    CellMargins->{{62, Inherited}, {12, 5}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Output", "Condensed"],
    CellMargins->{{40, Inherited}, {4, 1}}],
  
  Cell[StyleData["Output", "Printout"],
    CellMargins->{{44, 0}, {6, 2}},
    Background->GrayLevel[0.900008]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Message"],
    CellMargins->{{62, Inherited}, {Inherited, Inherited}},
    CellGroupingRules->"OutputGrouping",
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    ShowCellLabel->False,
    DefaultFormatType->DefaultOutputFormatType,
    FormatType->InputForm,
    CounterIncrements->"Message",
    StyleMenuListing->None,
    FontColor->RGBColor[1, 0, 0]],
  
  Cell[StyleData["Message", "Presentation"],
    CellMargins->{{74, Inherited}, {Inherited, Inherited}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Message", "Condensed"],
    CellMargins->{{50, Inherited}, {Inherited, Inherited}}],
  
  Cell[StyleData["Message", "Printout"],
    CellMargins->{{54, Inherited}, {Inherited, Inherited}},
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Print"],
    CellMargins->{{62, Inherited}, {Inherited, Inherited}},
    CellGroupingRules->"OutputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    ShowCellLabel->False,
    DefaultFormatType->DefaultOutputFormatType,
    FormatType->InputForm,
    CounterIncrements->"Print",
    StyleMenuListing->None],
  
  Cell[StyleData["Print", "Presentation"],
    CellMargins->{{74, Inherited}, {Inherited, Inherited}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Print", "Condensed"],
    CellMargins->{{50, Inherited}, {Inherited, Inherited}}],
  
  Cell[StyleData["Print", "Printout"],
    CellMargins->{{54, Inherited}, {Inherited, Inherited}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Graphics"],
    CellMargins->{{62, Inherited}, {Inherited, Inherited}},
    CellGroupingRules->"GraphicsGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    ShowCellLabel->False,
    DefaultFormatType->DefaultOutputFormatType,
    FormatType->InputForm,
    CounterIncrements->"Graphics",
    StyleMenuListing->None],
  
  Cell[StyleData["Graphics", "Presentation"],
    CellMargins->{{74, Inherited}, {Inherited, Inherited}}],
  
  Cell[StyleData["Graphics", "Condensed"],
    CellMargins->{{52, Inherited}, {Inherited, Inherited}},
    ImageSize->{175, 175}],
  
  Cell[StyleData["Graphics", "Printout"],
    CellMargins->{{54, Inherited}, {Inherited, Inherited}},
    ImageSize->{250, 250}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["CellLabel"],
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->11,
    FontWeight->"Bold",
    FontColor->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["CellLabel", "Presentation"],
    FontSize->12],
  
  Cell[StyleData["CellLabel", "Condensed"],
    FontSize->8],
  
  Cell[StyleData["CellLabel", "Printout"],
    FontSize->8,
    FontColor->GrayLevel[0]]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Unique Styles", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Author"],
    ShowCellBracket->False,
    CellMargins->{{10, 4}, {2, 10}},
    FontSize->16,
    FontSlant->"Italic"],
  
  Cell[StyleData["Author", "Presentation"],
    CellMargins->{{12, 10}, {2, 12}},
    LineSpacing->{1, 5},
    ParagraphSpacing->{0, 12},
    FontSize->20],
  
  Cell[StyleData["Author", "Condensed"],
    CellMargins->{{8, 10}, {1, 4}},
    LineSpacing->{1, 1},
    ParagraphSpacing->{0, 4},
    FontSize->12],
  
  Cell[StyleData["Author", "Printout"],
    CellMargins->{{9, 0}, {4, 12}},
    ParagraphSpacing->{0, 6},
    FontSize->14]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Copyright"],
    ShowCellBracket->False,
    CellMargins->{{10, 10}, {40, 2}},
    FontFamily->"Helvetica",
    FontSize->9],
  
  Cell[StyleData["Copyright", "Presentation"],
    CellMargins->{{12, 10}, {50, 2}},
    LineSpacing->{1, 5},
    FontSize->12],
  
  Cell[StyleData["Copyright", "Condensed"],
    CellMargins->{{8, 10}, {12, 1}},
    LineSpacing->{1, 2},
    FontSize->9],
  
  Cell[StyleData["Copyright", "Printout"],
    CellMargins->{{9, 0}, {72, 4}},
    FontSize->7]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Example"],
    CellMargins->{{12, 10}, {5, 12}},
    LineSpacing->{1, 3},
    ParagraphSpacing->{0, 12},
    CounterIncrements->"Example",
    FontFamily->"Times",
    FontWeight->"Bold"],
  
  Cell[StyleData["Example", "Presentation"],
    CellMargins->{{18, 10}, {8, 20}},
    LineSpacing->{1, 5},
    ParagraphSpacing->{0, 12}],
  
  Cell[StyleData["Example", "Condensed"],
    CellMargins->{{8, 10}, {4, 8}},
    LineSpacing->{1, 1},
    ParagraphSpacing->{0, 4}],
  
  Cell[StyleData["Example", "Printout"],
    CellMargins->{{9, 0}, {4, 10}},
    ParagraphSpacing->{0, 6}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Definition"],
    CellFrame->{{3, 0}, {0, 0}},
    CellMargins->{{52, 10}, {8, 8}},
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    CellLabelMargins->{{23, Inherited}, {Inherited, Inherited}},
    TextAlignment->Left,
    ShowStringCharacters->True,
    CounterIncrements->"Definition",
    FontFamily->"Times",
    FontWeight->"Normal",
    FontColor->GrayLevel[0],
    Background->RGBColor[1, 0.570001, 0.570001],
    GridBoxOptions->{ColumnSpacings->1.2,
    ColumnAlignments->{Right, Left}}],
  
  Cell[StyleData["Definition", "Presentation"],
    CellMargins->{{62, Inherited}, {5, 12}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Definition", "Condensed"],
    CellMargins->{{40, 10}, {1, 4}}],
  
  Cell[StyleData["Definition", "Printout"],
    CellMargins->{{44, 0}, {2, 6}},
    Background->GrayLevel[0.6]]
  }, Open  ]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Theorem"],
    CellFrame->{{3, 0}, {0, 0}},
    CellMargins->{{52, 10}, {8, 8}},
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    CellLabelMargins->{{23, Inherited}, {Inherited, Inherited}},
    ShowStringCharacters->True,
    CounterIncrements->"Theorem",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontColor->GrayLevel[1],
    Background->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["Theorem", "Presentation"],
    CellMargins->{{62, Inherited}, {5, 12}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Theorem", "Condensed"],
    CellMargins->{{40, 10}, {1, 4}}],
  
  Cell[StyleData["Theorem", "Printout"],
    CellMargins->{{44, 0}, {2, 6}},
    Background->GrayLevel[0.4]]
  }, Open  ]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["ExerciseMain"],
    CellFrame->{{6, 0}, {0, 1}},
    CellMargins->{{12, Inherited}, {4, 20}},
    CellGroupingRules->{"SectionGrouping", 30},
    PageBreakBelow->False,
    CellFrameColor->RGBColor[0.571389, 0.19675, 0.570504],
    CounterIncrements->"Subsection",
    CounterAssignments->{{"Subsubsection", 0}},
    FontFamily->"Helvetica",
    FontSize->16,
    FontWeight->"Bold",
    FontColor->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["ExerciseMain", "Presentation"],
    CellMargins->{{18, 10}, {8, 32}},
    LineSpacing->{1, 2},
    FontSize->24,
    FontTracking->"Condensed"],
  
  Cell[StyleData["ExerciseMain", "Condensed"],
    CellMargins->{{8, Inherited}, {2, 12}},
    FontSize->12],
  
  Cell[StyleData["ExerciseMain", "Printout"],
    CellMargins->{{9, 0}, {2, 50}},
    CellFrameColor->GrayLevel[0.500008],
    FontSize->14,
    FontTracking->"Plain",
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Exercise"],
    CellDingbat->"\[FilledDownTriangle]",
    CellMargins->{{23, Inherited}, {4, 18}},
    CellGroupingRules->{"SectionGrouping", 50},
    PageBreakBelow->False,
    CounterIncrements->"Subsubsection",
    FontFamily->"Times",
    FontSize->13,
    FontWeight->"Bold",
    FontColor->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["Exercise", "Presentation"],
    CellMargins->{{33, 10}, {8, 26}},
    LineSpacing->{1, 0},
    FontSize->18],
  
  Cell[StyleData["Exercise", "Condensed"],
    CellMargins->{{17, Inherited}, {2, 12}},
    FontSize->10],
  
  Cell[StyleData["Exercise", "Printout"],
    CellFrame->{{0, 0}, {0.5, 0}},
    CellDingbat->None,
    CellMargins->{{9, 0}, {6, 20}},
    FontSize->11,
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["ExerciseText"],
    CellMargins->{{62, 10}, {5, 5}},
    LineSpacing->{1, 3},
    ParagraphSpacing->{0, 8},
    CounterIncrements->"ExcersiceText",
    FontFamily->"Times"],
  
  Cell[StyleData["ExerciseText", "Presentation"],
    CellMargins->{{74, 10}, {8, 8}},
    LineSpacing->{1, 5},
    ParagraphSpacing->{0, 12}],
  
  Cell[StyleData["ExerciseText", "Condensed"],
    CellMargins->{{52, 10}, {2, 2}},
    LineSpacing->{1, 1},
    ParagraphSpacing->{0, 4}],
  
  Cell[StyleData["ExerciseText", "Printout"],
    CellMargins->{{54, 0}, {4, 4}},
    ParagraphSpacing->{0, 6}]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Formulas and Programming", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["DisplayFormula"],
    CellMargins->{{62, 10}, {2, 10}},
    CellHorizontalScrolling->True,
    DefaultFormatType->DefaultInputFormatType,
    ScriptLevel->0,
    SingleLetterItalics->True,
    UnderoverscriptBoxOptions->{LimitsPositioning->True}],
  
  Cell[StyleData["DisplayFormula", "Presentation"],
    CellMargins->{{74, 10}, {2, 10}},
    FontSize->10],
  
  Cell[StyleData["DisplayFormula", "Condensed"],
    CellMargins->{{52, 10}, {2, 10}},
    FontSize->10],
  
  Cell[StyleData["DisplayFormula", "Printout"],
    CellMargins->{{54, 10}, {2, 10}},
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["ChemicalFormula"],
    CellMargins->{{62, 10}, {2, 10}},
    DefaultFormatType->DefaultInputFormatType,
    AutoSpacing->False,
    ScriptLevel->1,
    ScriptBaselineShifts->{0.6, Automatic},
    SingleLetterItalics->False,
    ZeroWidthTimes->True],
  
  Cell[StyleData["ChemicalFormula", "Presentation"],
    CellMargins->{{74, 10}, {2, 10}},
    FontSize->10],
  
  Cell[StyleData["ChemicalFormula", "Condensed"],
    CellMargins->{{52, 10}, {2, 10}},
    FontSize->10],
  
  Cell[StyleData["ChemicalFormula", "Printout"],
    CellMargins->{{54, 10}, {2, 10}},
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Program"],
    CellMargins->{{12, 10}, {Inherited, 6}},
    FontFamily->"Courier"],
  
  Cell[StyleData["Program", "Presentation"],
    CellMargins->{{13, 30}, {Inherited, 4}},
    FontSize->9.5],
  
  Cell[StyleData["Program", "Condensed"],
    CellMargins->{{8, 10}, {Inherited, 4}},
    FontSize->9.5],
  
  Cell[StyleData["Program", "Printout"],
    CellMargins->{{9, 0}, {Inherited, 4}},
    FontSize->9.5]
  }, Closed]]
  }, Open  ]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Automatic Numbering", "Section"],
  
  Cell["\<\
The following styles are useful for numbered equations, figures, etc.  They \
automatically give the cell a FrameLabel containing a reference to a \
particular counter, and also increment that counter.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["NumberedEquation"],
    CellMargins->{{62, 10}, {Inherited, Inherited}},
    CellFrameLabels->{{None, Cell[ 
            TextData[ {"(", 
              CounterBox[ "NumberedEquation"], ")"}]]}, {None, None}},
    DefaultFormatType->DefaultInputFormatType,
    CounterIncrements->"NumberedEquation",
    FormatTypeAutoConvert->False],
  
  Cell[StyleData["NumberedEquation", "Presentation"],
    CellMargins->{{74, 10}, {Inherited, Inherited}}],
  
  Cell[StyleData["NumberedEquation", "Condensed"],
    CellMargins->{{52, 10}, {Inherited, Inherited}}],
  
  Cell[StyleData["NumberedEquation", "Printout"],
    CellMargins->{{54, 0}, {Inherited, Inherited}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["NumberedFigure"],
    CellMargins->{{62, 145}, {Inherited, Inherited}},
    CellFrameLabels->{{None, None}, {Cell[ 
            TextData[ {"Figure ", 
              CounterBox[ "NumberedFigure"]}]], None}},
    CounterIncrements->"NumberedFigure",
    ImageMargins->{{43, Inherited}, {Inherited, 0}},
    FormatTypeAutoConvert->False],
  
  Cell[StyleData["NumberedFigure", "Presentation"]],
  
  Cell[StyleData["NumberedFigure", "Condensed"]],
  
  Cell[StyleData["NumberedFigure", "Printout"]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["NumberedTable"],
    CellMargins->{{62, 145}, {Inherited, Inherited}},
    CellFrameLabels->{{None, None}, {Cell[ 
            TextData[ {"Table ", 
              CounterBox[ "NumberedTable"]}]], None}},
    TextAlignment->Center,
    CounterIncrements->"NumberedTable",
    FormatTypeAutoConvert->False],
  
  Cell[StyleData["NumberedTable", "Presentation"]],
  
  Cell[StyleData["NumberedTable", "Condensed"]],
  
  Cell[StyleData["NumberedTable", "Printout"]]
  }, Closed]]
  }, Open  ]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Headers and Footers", "Section"],
  
  Cell[StyleData["Header"],
    CellMargins->{{0, 0}, {4, 1}},
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->9,
    FontSlant->"Italic"],
  
  Cell[StyleData["Footer"],
    CellMargins->{{0, 0}, {0, 4}},
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->6],
  
  Cell[StyleData["PageNumber"],
    CellMargins->{{0, 0}, {4, 1}},
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->9,
    FontWeight->"Bold"]
  }, Open  ]],
  
  Cell[CellGroupData[{
  
  Cell["Hyperlink Styles", "Section"],
  
  Cell["\<\
The cells below define styles useful for making hypertext ButtonBoxes.  The \
\"Hyperlink\" style is for links within the same Notebook, or between \
Notebooks.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Hyperlink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->GrayLevel[1],
    Background->RGBColor[1, 0.4, 0],
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`NotebookLocate[ #2]}]&),
    Active->True,
    ButtonFrame->"None",
    ButtonNote->ButtonData}],
  
  Cell[StyleData["Hyperlink", "Presentation"]],
  
  Cell[StyleData["Hyperlink", "Condensed"]],
  
  Cell[StyleData["Hyperlink", "Printout"],
    FontColor->GrayLevel[0],
    Background->GrayLevel[1]]
  }, Closed]],
  
  Cell["\<\
The following styles are for linking automatically to the on-line help \
system.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["MainBookLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->GrayLevel[1],
    Background->RGBColor[1, 0.4, 0],
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "MainBook", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["MainBookLink", "Presentation"]],
  
  Cell[StyleData["MainBookLink", "Condensed"]],
  
  Cell[StyleData["MainBookLink", "Printout"],
    FontColor->GrayLevel[0],
    Background->GrayLevel[1]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["AddOnsLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontFamily->"Courier",
    FontColor->GrayLevel[1],
    Background->RGBColor[1, 0.4, 0],
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "AddOns", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["AddOnsLink", "Presentation"]],
  
  Cell[StyleData["AddOnsLink", "Condensed"]],
  
  Cell[StyleData["AddOnLink", "Printout"],
    FontColor->GrayLevel[0],
    Background->GrayLevel[1]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["RefGuideLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontFamily->"Courier",
    FontColor->GrayLevel[1],
    Background->RGBColor[1, 0.4, 0],
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "RefGuideLink", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["RefGuideLink", "Presentation"]],
  
  Cell[StyleData["RefGuideLink", "Condensed"]],
  
  Cell[StyleData["RefGuideLink", "Printout"],
    FontColor->GrayLevel[0],
    Background->GrayLevel[1]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["GettingStartedLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->GrayLevel[1],
    Background->RGBColor[1, 0.4, 0],
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "GettingStarted", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["GettingStartedLink", "Presentation"]],
  
  Cell[StyleData["GettingStartedLink", "Condensed"]],
  
  Cell[StyleData["GettingStartedLink", "Printout"],
    FontColor->GrayLevel[0],
    Background->GrayLevel[1]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["OtherInformationLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->GrayLevel[1],
    Background->RGBColor[1, 0.4, 0],
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "OtherInformation", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["OtherInformationLink", "Presentation"]],
  
  Cell[StyleData["OtherInformationLink", "Condensed"]],
  
  Cell[StyleData["OtherInformationLink", "Printout"],
    FontColor->GrayLevel[0],
    Background->GrayLevel[1]]
  }, Closed]]
  }, Open  ]]
  }, Open  ]]
  }]
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 92, 1, 42, "Subtitle"],
Cell[1849, 54, 122, 4, 84, "Copyright"],
Cell[1974, 60, 95, 5, 96, "Author"],

Cell[CellGroupData[{
Cell[2094, 69, 34, 0, 56, "Section"],

Cell[CellGroupData[{
Cell[2153, 73, 30, 0, 46, "Subsection"],
Cell[2186, 75, 1418, 44, 162, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[3641, 124, 34, 0, 30, "Subsection"],
Cell[3678, 126, 597, 16, 86, "Text"],
Cell[4278, 144, 55, 1, 50, "Input"],
Cell[4336, 147, 91, 2, 29, "Text"],
Cell[4430, 151, 49, 1, 50, "Input"],
Cell[4482, 154, 42, 0, 29, "Text"],
Cell[4527, 156, 68, 0, 49, "Definition"]
}, Closed]],

Cell[CellGroupData[{
Cell[4632, 161, 33, 0, 30, "Subsection"],
Cell[4668, 163, 142, 3, 48, "Text"],
Cell[4813, 168, 102, 2, 70, "Input"],
Cell[4918, 172, 202, 4, 48, "Text"],
Cell[5123, 178, 58, 1, 50, "Input"],
Cell[5184, 181, 51, 0, 29, "Text"],
Cell[5238, 183, 345, 10, 65, "Definition"],
Cell[5586, 195, 61, 0, 29, "Text"],

Cell[CellGroupData[{
Cell[5672, 199, 351, 7, 130, "Input"],
Cell[6026, 208, 100338, 2569, 222, 29103, 1685, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]],
Cell[106379, 2780, 206, 4, 48, "Text"],
Cell[106588, 2786, 43, 1, 50, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[106668, 2792, 41, 0, 30, "Subsection"],
Cell[106712, 2794, 402, 11, 83, "Definition"],
Cell[107117, 2807, 303, 8, 48, "Text"],
Cell[107423, 2817, 747, 18, 119, "Definition"]
}, Closed]],

Cell[CellGroupData[{
Cell[108207, 2840, 42, 0, 30, "Subsection"],
Cell[108252, 2842, 2316, 63, 65, "Definition"],
Cell[110571, 2907, 673, 19, 67, "Text"],
Cell[111247, 2928, 480, 13, 65, "Definition"],
Cell[111730, 2943, 123, 4, 29, "Text"],
Cell[111856, 2949, 1386, 38, 119, "Definition"],
Cell[113245, 2989, 1039, 15, 181, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[114321, 3009, 43, 0, 30, "Subsection"],
Cell[114367, 3011, 2436, 67, 65, "Definition"],
Cell[116806, 3080, 859, 21, 105, "Text"],
Cell[117668, 3103, 2281, 60, 53, "Definition"],
Cell[119952, 3165, 66, 1, 29, "Text"],
Cell[120021, 3168, 477, 12, 53, "Definition"],
Cell[120501, 3182, 156, 4, 29, "Text"],
Cell[120660, 3188, 1240, 35, 101, "Definition"],
Cell[121903, 3225, 533, 12, 92, "Text"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[122485, 3243, 27, 0, 36, "Section"],

Cell[CellGroupData[{
Cell[122537, 3247, 62, 0, 46, "Subsection"],
Cell[122602, 3249, 97, 2, 29, "Text"],
Cell[122702, 3253, 365, 7, 150, "Input"],
Cell[123070, 3262, 255, 5, 67, "Text"],
Cell[123328, 3269, 233, 4, 90, "Input"],
Cell[123564, 3275, 54, 0, 29, "Text"],

Cell[CellGroupData[{
Cell[123643, 3279, 62, 1, 50, "Input"],
Cell[123708, 3282, 127451, 3932, 225, 48026, 2947, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]],
Cell[251174, 7217, 173, 3, 48, "Text"],
Cell[251350, 7222, 136, 2, 70, "Input"],
Cell[251489, 7226, 218, 5, 84, "Input"],
Cell[251710, 7233, 135, 2, 70, "Input"],
Cell[251848, 7237, 127, 2, 29, "Text"],

Cell[CellGroupData[{
Cell[252000, 7243, 128, 2, 70, "Input"],
Cell[252131, 7247, 158350, 4554, 224, 53926, 3260, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[410530, 11807, 60, 0, 30, "Subsection"],
Cell[410593, 11809, 209, 4, 48, "Text"],
Cell[410805, 11815, 481, 8, 170, "Input"],
Cell[411289, 11825, 103, 3, 29, "Text"],
Cell[411395, 11830, 78, 1, 50, "Input"],
Cell[411476, 11833, 505, 9, 170, "Input"],
Cell[411984, 11844, 291, 5, 67, "Text"],

Cell[CellGroupData[{
Cell[412300, 11853, 759, 13, 110, "Input"],
Cell[413062, 11868, 198841, 5347, 234, 120330, 4373, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[611952, 17221, 60, 0, 30, "Subsection"],
Cell[612015, 17223, 86, 2, 29, "Text"],
Cell[612104, 17227, 185, 3, 70, "Input"],
Cell[612292, 17232, 50, 0, 29, "Text"],
Cell[612345, 17234, 81, 1, 50, "Input"],
Cell[612429, 17237, 41, 0, 29, "Text"],

Cell[CellGroupData[{
Cell[612495, 17241, 121, 2, 50, "Input"],
Cell[612619, 17245, 62, 1, 49, "Output"]
}, Open  ]],
Cell[612696, 17249, 262, 5, 67, "Text"],
Cell[612961, 17256, 268, 5, 90, "Input"],

Cell[CellGroupData[{
Cell[613254, 17265, 128, 2, 70, "Input"],
Cell[613385, 17269, 62, 1, 49, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[613508, 17277, 28, 0, 36, "Section"],

Cell[CellGroupData[{
Cell[613561, 17281, 36, 0, 32, "Subsubsection"],
Cell[613600, 17283, 251, 9, 30, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[613888, 17297, 37, 0, 32, "Subsubsection"],
Cell[613928, 17299, 260, 9, 30, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[614225, 17313, 35, 0, 32, "Subsubsection"],
Cell[614263, 17315, 646, 17, 88, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[614946, 17337, 33, 0, 32, "Subsubsection"],
Cell[614982, 17339, 218, 6, 49, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[615237, 17350, 28, 0, 32, "Subsubsection"],
Cell[615268, 17352, 246, 9, 30, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[615551, 17366, 35, 0, 32, "Subsubsection"],
Cell[615589, 17368, 289, 8, 49, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[615915, 17381, 41, 0, 32, "Subsubsection"],
Cell[615959, 17383, 280, 9, 49, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[616276, 17397, 28, 0, 32, "Subsubsection"],
Cell[616307, 17399, 272, 10, 30, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[616616, 17414, 28, 0, 32, "Subsubsection"],
Cell[616647, 17416, 272, 10, 30, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[616956, 17431, 33, 0, 32, "Subsubsection"],
Cell[616992, 17433, 161, 5, 30, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[617190, 17443, 36, 0, 32, "Subsubsection"],
Cell[617229, 17445, 255, 9, 30, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[617521, 17459, 67, 1, 32, "Subsubsection"],
Cell[617591, 17462, 460, 15, 55, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[618088, 17482, 71, 1, 32, "Subsubsection"],
Cell[618162, 17485, 1284, 29, 229, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[619483, 17519, 64, 1, 32, "Subsubsection"],
Cell[619550, 17522, 1349, 43, 217, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[620936, 17570, 36, 0, 32, "Subsubsection"],
Cell[620975, 17572, 1290, 29, 244, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[622302, 17606, 49, 0, 32, "Subsubsection"],
Cell[622354, 17608, 298, 7, 49, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[622689, 17620, 34, 0, 32, "Subsubsection"],
Cell[622726, 17622, 260, 9, 30, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[623023, 17636, 37, 0, 32, "Subsubsection"],
Cell[623063, 17638, 615, 15, 87, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[623715, 17658, 29, 0, 32, "Subsubsection"],
Cell[623747, 17660, 395, 12, 49, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[624179, 17677, 37, 0, 32, "Subsubsection"],
Cell[624219, 17679, 175, 5, 30, "Text"]
}, Open  ]]
}, Closed]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

